*DECK ASM
      SUBROUTINE ASM(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
C
C-----------------------------------------------------------------------
C
C COLLISION PROBABILITY ASSEMBLY MODULE.
C
C INPUT/OUTPUT PARAMETERS:
C  NENTRY : NUMBER OF LINKED LISTS OR FILES USED BY THE MODULE.
C  HENTRY : CHARACTER*12 NAME OF EACH LINKED LIST OR FILE.
C  IENTRY : =1 LINKED LIST; =2 XSM FILE; =3 SEQUENTIAL BINARY FILE;
C           =4 SEQUENTIAL ASCII FILE.
C  JENTRY : =0 THE LINKED LIST OR FILE IS CREATED.
C           =1 THE LINKED LIST OR FILE IS OPEN FOR MODIFICATIONS;
C           =2 THE LINKED LIST OR FILE IS OPEN IN READ-ONLY MODE.
C  KENTRY : =FILE UNIT NUMBER; =LINKED LIST ADDRESS OTHERWISE.
C           DIMENSION HENTRY(NENTRY),IENTRY(NENTRY),JENTRY(NENTRY),
C           KENTRY(NENTRY)
C
C LINKED LIST / XSM FILE:
C  HENTRY(1) : CREATE OR MODIFICATION TYPE(L_PIJ).
C  HENTRY(2) : READ-ONLY TYPE(L_MACROLIB OR L_LIBRARY)
C  HENTRY(3) : READ-ONLY LINKED LIST TYPE(L_TRACK)
C  HENTRY(4) : OPTIONAL READ-ONLY SEQUENTIAL BINARY TRACKING FILE.
C
C-------------------------------------- AUTHOR: A. HEBERT ; 17/10/94 ---
C
      IMPLICIT NONE
C----
C PARAMETERS
C----
      INTEGER      IOUT,NSTATE
      CHARACTER    NAMSBR*6
      PARAMETER   (IOUT=6,NSTATE=40,NAMSBR='ASM   ')
C----
C INTERFACE VARIABLES
C----
      INTEGER      NENTRY
      INTEGER      IENTRY(NENTRY),JENTRY(NENTRY),
     1             KENTRY(NENTRY)
      CHARACTER    HENTRY(NENTRY)*12
C----
C MEMORY ALLOCATION
C----
      INTEGER      IBASE(1)
      REAL         RBASE(1)
      COMMON       RBASE
      EQUIVALENCE (RBASE(1),IBASE(1))
      INTEGER      IMAT,IVOL,IIDL,IXSSIT,IXSSI0
C----
C LOCAL VARIABLES
C----
      INTEGER          IPSYS,IPMACR,IPTRK,I,NREG,NUN,NBMIXT,
     1                 NANI,NPIS,NGEN,NPSS,IFTRAK,NICM,
     2                 LENGT,ITYLCM,NGRP,MAXMIX,ITRANC,IPHASE,ISCATR,
     3                 KNORM,INDIC,NITMA,IPRINT,JJ,NSBG
      REAL             FLOTT
      INTEGER          KCHAR(18),ISTATE(NSTATE)
      CHARACTER        TEXT4*4,HSIGN*12,TEXT12*12,HSMG*131,
     1                 CDOOR*12,TITRE*72,CNAME*12
      LOGICAL          LEAKSW,LNORM,LALBS,LR2
      DOUBLE PRECISION DFLOTT
      INTEGER          IFUITE
C
C PARAMETER VALIDATION.
C
      IF(NENTRY.LT.3) CALL XABORT(NAMSBR//
     >  ':AT LEAST THREE DATA STRUCTURES REQUIRED')
      IPSYS=KENTRY(1)
      IPMACR=KENTRY(2)
      IPTRK=KENTRY(3)
      IF(IENTRY(1).NE.1.AND.IENTRY(1).NE.2) CALL XABORT(NAMSBR//
     >  ': FIRST DATA STRUCTURE NOT A LINKED LIST OR XSM FILE')
      IF(JENTRY(1).NE.0.AND.JENTRY(1).NE.1) CALL XABORT(NAMSBR//
     >  ': FIRST DATA STRUCTURE NOT IN CREATE OR MODIFY MODE')
      IF(IENTRY(2).NE.1.AND.IENTRY(2).NE.2) CALL XABORT(NAMSBR//
     >  ': SECOND DATA STRUCTURE NOT A LINKED LIST OR XSM FILE')
      IF(JENTRY(2).NE.2) CALL XABORT(NAMSBR//
     >  ': SECOND DATA STRUCTURE NOT IN READ-ONLY MODE')
      IF(IENTRY(3).NE.1.AND.IENTRY(3).NE.2) CALL XABORT(NAMSBR//
     >  ': THIRD DATA STRUCTURE NOT A LINKED LIST OR XSM FILE')
      IF(JENTRY(3).NE.2) CALL XABORT(NAMSBR//
     >  ': THIRD DATA STRUCTURE NOT IN READ-ONLY MODE')
      CALL LCMGET(IPTRK,'SIGNATURE',KCHAR)
      WRITE(HSIGN,'(3A4)') (KCHAR(I),I=1,3)
      IF(HSIGN.NE.'L_TRACK') THEN
        TEXT12=HENTRY(3)
        CALL XABORT(NAMSBR//': SIGNATURE OF '//TEXT12//' IS '//
     >    HSIGN//'. L_TRACK EXPECTED.')
      ENDIF
      HSIGN='L_PIJ'
      CALL XDISET(ISTATE,NSTATE,0)
      CALL LCMGET(IPTRK,'STATE-VECTOR',ISTATE)
      CALL LCMGET(IPTRK,'TRACK-TYPE',KCHAR)
      WRITE(CDOOR,'(3A4)') (KCHAR(I),I=1,3)
      READ(HSIGN,'(3A4)') (KCHAR(I),I=1,3)
      CALL LCMPUT(IPSYS,'SIGNATURE',3,3,KCHAR)
*
* RECOVER TRACKING FILE INFORMATION.
      IF(NENTRY.LT.4)THEN
        IFTRAK=0
      ELSE
        TEXT12= HENTRY(4)
        IF( IENTRY(4).EQ.3 )THEN
          IF( JENTRY(4).NE.2 ) CALL XABORT(NAMSBR//
     >      ': BINARY TRACKING FILE NAMED '//TEXT12//
     >      ' IS NOT IN REAL-ONLY MODE.')
          IFTRAK= KENTRY(4)
        ENDIF
      ENDIF
*
* RECOVER GENERAL TRACKING INFORMATION.
      NREG=ISTATE(1)
      NUN=ISTATE(2)
      LEAKSW=ISTATE(3).EQ.0
      NBMIXT=ISTATE(4)
      NPIS=ISTATE(5)
      NANI=MAX(1,ISTATE(6))
      IF(CDOOR .EQ. 'JPM') THEN
        NGEN=ISTATE(12)
        NPSS=ISTATE(16)
        NICM=ISTATE(17)
      ELSE
        NGEN=1
        NPSS=0
        NICM=0
      ENDIF
      CALL SETARA(IBASE,NREG,IMAT)
      CALL SETARA(RBASE,NREG,IVOL)
      CALL SETARA(IBASE,NREG,IIDL)
      CALL LCMGET(IPTRK,'MATCOD',IBASE(IMAT))
      CALL LCMGET(IPTRK,'VOLUME',RBASE(IVOL))
      CALL LCMGET(IPTRK,'KEYFLX',IBASE(IIDL))
      CALL LCMLEN(IPTRK,'TITLE',LENGT,ITYLCM)
      IF(LENGT.GT.0) THEN
        CALL LCMGET(IPTRK,'TITLE',KCHAR)
        WRITE(TITRE,'(18A4)') (KCHAR(I),I=1,18)
      ELSE
        TITRE='*** NO TITLE PROVIDED ***'
      ENDIF
*
* RECOVER MACROLIB PARAMETERS.
      CALL LCMGET(IPMACR,'SIGNATURE',KCHAR)
      WRITE(HSIGN,'(3A4)') (KCHAR(I),I=1,3)
      IF(HSIGN.EQ.'L_LIBRARY') THEN
        CALL LCMSIX(IPMACR,'MACROLIB',1)
      ELSE IF(HSIGN.NE.'L_MACROLIB') THEN
        TEXT12=HENTRY(2)
        CALL XABORT(NAMSBR//': SIGNATURE OF '//TEXT12//' IS '//
     >    HSIGN//'. L_MACROLIB EXPECTED.')
      ENDIF
      CALL XDISET(ISTATE,NSTATE,0)
      CALL LCMGET(IPMACR,'STATE-VECTOR',ISTATE)
      NGRP=ISTATE(1)
      MAXMIX=ISTATE(2)
      ITRANC=ISTATE(6)
      IF(NBMIXT.GT.MAXMIX) THEN
         WRITE(HSMG,9000) NAMSBR,NBMIXT,MAXMIX
         CALL XABORT(HSMG)
      ENDIF
*
      LNORM=.FALSE.
      LALBS=.FALSE.
      LR2=.TRUE.
      IPHASE=0
      ISCATR=0
      IF( CDOOR .EQ. 'EXCELL' ) THEN
        KNORM=0
      ELSE
        KNORM=1
      ENDIF
      IF(JENTRY(1).EQ.1) THEN
        CALL LCMGET(IPSYS,'SIGNATURE',KCHAR)
        WRITE(HSIGN,'(3A4)') (KCHAR(I),I=1,3)
        IF(HSIGN.NE.'L_PIJ') THEN
           TEXT12=HENTRY(1)
           CALL XABORT(NAMSBR//': SIGNATURE OF '//TEXT12//' IS '//
     >       HSIGN//'. L_PIJ EXPECTED.')
        ENDIF
        CALL XDISET(ISTATE,NSTATE,0)
        CALL LCMGET(IPSYS,'STATE-VECTOR',ISTATE)
        IF((ISTATE(8).NE.NGRP).OR.(ISTATE(9).NE.NUN)) THEN
           WRITE(HSMG,9001) NAMSBR,ISTATE(8),NGRP,ISTATE(9),NUN
           CALL XABORT(HSMG)
        ENDIF
        IPHASE=ISTATE(1)
        ISCATR=ISTATE(2)
        IF(ISTATE(3) .EQ. 1) THEN
          LR2=.FALSE.
        ENDIF
        IF(ISTATE(4) .EQ. 1) THEN
          LNORM=.TRUE.
        ELSE IF(ISTATE(4) .EQ. 2) THEN
          LALBS=.TRUE.
        ENDIF
C----
C  CVS-UPDATE: 2000/10/05 
C  KNORM=ISTATE(7) IS THE DRAGON 3.03  OPTION
C  IN THE RESTRUCTURATION TO DRAGON 3.04
C  KNORM=ISTATE(5)
C----
        KNORM=ISTATE(5)
      ENDIF
      CNAME=' '
      IPRINT=1
      NSBG = 1
   15 CALL REDGET(INDIC,NITMA,FLOTT,TEXT4,DFLOTT)
      IF(INDIC.EQ.10) GO TO 40
   20 IF(INDIC.NE.3) CALL XABORT(NAMSBR//
     >  ': CHARACTER DATA EXPECTED(1).')
      IF(TEXT4.EQ.'EDIT') THEN
        CALL REDGET(INDIC,IPRINT,FLOTT,TEXT4,DFLOTT)
        IF(INDIC.NE.1) CALL XABORT(NAMSBR//
     >  ': INTEGER DATA EXPECTED(1).')
      ELSE IF(TEXT4.EQ.'PNOR') THEN
        CALL REDGET(INDIC,NITMA,FLOTT,TEXT4,DFLOTT)
        IF(INDIC.NE.3) CALL XABORT(NAMSBR//
     >  ': CHARACTER DATA EXPECTED(2).')
        IF(TEXT4.EQ.'NONE') THEN
          KNORM=-1
        ELSE IF(TEXT4.EQ.'HELI') THEN
          KNORM=0
        ELSE IF(TEXT4.EQ.'GELB') THEN
          KNORM=1
        ELSE IF(TEXT4.EQ.'DIAG') THEN
          KNORM=2
        ELSE IF(TEXT4.EQ.'NONL') THEN
          KNORM=3
         ELSE
          GO TO 20
        ENDIF
      ELSE IF(TEXT4.EQ.'NOR2') THEN
        LR2=.FALSE.
      ELSE IF(TEXT4(1:3).EQ.'ARM') THEN
        IPHASE=-1
      ELSE IF(TEXT4(1:3).EQ.'PIJ') THEN
        IPHASE=0
        IF(TEXT4(4:4).EQ.'K') THEN
          IF(CDOOR .EQ. 'EXCELL') THEN
            IPHASE=1
          ELSE
            WRITE(IOUT,6300) CDOOR
          ENDIF
        ENDIF
  30    CALL REDGET(INDIC,NITMA,FLOTT,TEXT4,DFLOTT)
        IF(INDIC.NE.3) CALL XABORT(NAMSBR//
     >  ': KEYWORD EXPECTED.')
        IF(TEXT4.EQ.'NAME') THEN
          CALL REDGET(INDIC,NITMA,FLOTT,CNAME,DFLOTT)
          IF(INDIC.NE.3) CALL XABORT(NAMSBR//
     >    ': RECORD NAME EXPECTED')
        ELSE IF(TEXT4.EQ.'NORM') THEN
          LNORM=.TRUE.
        ELSE IF(TEXT4.EQ.'ALBS') THEN
          LALBS=.TRUE.
          IF(.NOT.LEAKSW) CALL XABORT(NAMSBR//
     >    ': THE ALSB OPTION REQUIRES SOME BOUNDARY LEAKAGE.')
        ELSE
          GO TO 20
        ENDIF
        GO TO 30
      ELSE IF(TEXT4.EQ.'SKIP') THEN
        ISCATR=1
      ELSE IF(TEXT4.EQ.'ALLG') THEN
        NSBG=NGRP
      ELSE IF(TEXT4.EQ.';') THEN
        GO TO 40
      ELSE
        CALL XABORT(NAMSBR//
     >  ': '//TEXT4//' IS AN INVALID KEY WORD.')
      ENDIF
      GO TO 15
*
* STORE PIJ PARAMETERS.
   40 CONTINUE
      CALL XDISET(ISTATE,NSTATE,0)
      ISTATE(1)=IPHASE
      ISTATE(2)=ISCATR
      IF(.NOT. LR2) THEN
        ISTATE(3)=1
      ENDIF
      IF(LNORM) THEN
        ISTATE(4)=1
      ELSE IF (LALBS) THEN
        ISTATE(4)=2
      ELSE IF(LEAKSW) THEN
        ISTATE(4)=-1
      ENDIF
      ISTATE( 5)=KNORM
      ISTATE( 6)=NGRP
      ISTATE( 7)=NGRP
      ISTATE( 8)=NUN
      ISTATE( 9)=MAXMIX
      ISTATE(10)=NANI
      ISTATE(12)=NREG
      ISTATE(11)=NGEN
      ISTATE(13)=NPIS
      ISTATE(14)=NPSS
      ISTATE(15)=NICM
      CALL LCMPUT(IPSYS,'STATE-VECTOR',NSTATE,1,ISTATE)
      IF(IPRINT.GE.1) THEN
        WRITE(IOUT,6000) IPRINT,15,(ISTATE(JJ),JJ=1,15)
      ENDIF
*
* BUILD COLLISION PROBABILITIES.
      CALL SETARA(RBASE,(MAXMIX+1)*NSBG,IXSSIT)
      CALL SETARA(RBASE,(MAXMIX+1)*NANI*NSBG,IXSSI0)
      IFUITE=0
      CALL ASMDRV(IPSYS ,IPTRK ,IPMACR,IFTRAK,CDOOR ,IPRINT  ,
     >            NGRP  ,MAXMIX,NREG  ,NANI  ,LEAKSW,ITRANC,
     >            TITRE ,LNORM ,LALBS ,LR2   ,IPHASE,ISCATR,
     >            KNORM ,CNAME ,
     >            IBASE(IMAT)  ,RBASE(IVOL)  ,RBASE(IXSSIT),
     >            RBASE(IXSSI0),NSBG  ,IFUITE)
      ISTATE(16)=IFUITE
      CALL LCMPUT(IPSYS,'STATE-VECTOR',NSTATE,1,ISTATE)
      CALL RLSARA(RBASE(IXSSI0))
      CALL RLSARA(RBASE(IXSSIT))
*
      IF(IPRINT.GE.5) CALL LCMLIB(IPSYS)
*
* RELEASE GENERAL TRACKING INFORMATION.
      CALL RLSARA(IBASE(IIDL))
      CALL RLSARA(RBASE(IVOL))
      CALL RLSARA(IBASE(IMAT))
      CALL LCMSIX(IPMACR,' ',0)
      RETURN
C----
C  FORMATS
C----
 6000 FORMAT(' ASM PRINT LEVEL =',I8/
     > ' ASMPIJ STATE-VECTOR  (',I2,') ELEMENTS'/1X,17('-')/
     > ' IPHASE =',I8,' (TYPE OF SYSTEM MATRIX)'/
     > ' ISCATR =',I8,' (SCATTERING REDUCTION OPTION)'/
     > ' IRES   =',I8,' (RESIDUAL MATRIX OPTION)'/
     > ' ICLOS  =',I8,' (TYPE OF CP CLOSURE RELATION)'/
     > ' KNORM  =',I8,' (CP MATRIX NORMALIZATION OPTION)'/
     > ' NSUBG  =',I8,' (NUMBER OF GROUP IN SUBGROUP)'/
     > ' NGROUP =',I8,' (NUMBER OF GROUPS)'/
     > ' NUNKNO =',I8,' (NUMBER OF UNKNOWNS IN SYSTEM)'/
     > ' MAXMIX =',I8,' (NUMBER OF MIXTURES)'/
     > ' NANI   =',I8,' (FLUX ANISOTROPY ORDER)'/
     > ' NGEN   =',I8,' (NUMBER OF GENERATING CELL)'/
     > ' NREG   =',I8,' (NUMBER OF REGIONS)'/
     > ' NSUR   =',I8,' (NUMBER OF SURFACES)'/
     > ' NPSS   =',I8,' (DIMENSION OF PSS MATRIX)'/
     > ' NICM   =',I8,' (DIMENSION OF ICM MATRIX)'/1X,17('-'))
 6300 FORMAT(//' *** WARNING:  OPTION PIJK IS INVALID FOR DOOR = ',
     >          A12/'               OPTION PIJ USED INSTEAD')
 9000 FORMAT(A6,':  NUMBER OF MIXTURES IN THE TRACKING (',I5,
     > ') >  NUMBER OF MIXTURES IN THE MACROLIB (',I5,')')
 9001 FORMAT(A6,': INCONSISTENT NUMBER OF GROUPS (',I3,
     >      ' VS',I4,') OR UNKNOWNS (',I5,' VS',I6,')')
      END
