*DECK B1HXS1
      SUBROUTINE B1HXS1(MAXPTS,IPMACR,IPAS,NGRO,NBM,IAN,NFISSI,VOL,MAT,
     1 KEYFLX,FLUX,IJJ0,IJJ1,NJJ0,NJJ1,IDEL,IJJ,NJJ)
*
*-----------------------------------------------------------------------
*
* HOMOGENIZATION OF THE LATTICE CELL NUCLEAR PROPERTIES BEFORE A B-N
* CALCULATION.
*
* INPUT PARAMETERS:
*  MAXPTS  : FIRST DIMENSION OF MATRIX FLUX.
*  IPMACR  : POINTER TO THE MACROLIB (L_MACROLIB SIGNATURE).
*  IPAS    : NUMBER OF VOLUMES.
*  NGRO    : NUMBER OF GROUPS.
*  NBM     : NUMBER OF MIXTURES.
*  IAN     : TYPE OF HOMOGENIZATION.
*            =-1 : TRANSPORT CORRECTED P0 ;      =0 : P0 ;      =1 : P1
*  NFISSI  : MAXIMUM NUMBER OF FISSION SPECTRUM ASSIGNED TO A MIXTURE.
*  VOL     : VOLUMES. DIMENSION VOL(IPAS)
*  MAT     : MIXTURE NUMBER OF EACH VOLUME. DIMENSION MAT(IPAS)
*  KEYFLX  : POSITION OF EACH FLUX IN THE UNKNOWN VECTOR.
*            DIMENSION KEYFLX(IPAS)
*  FLUX    : DIRECT UNKNOWN VECTOR. DIMENSION FLUX(MAXPTS,NGRO)
*  INORM   : FLUX NORMALIZATION OPTION
*            =-1: FOR B1 HETEREGENEOUS; =2: FOR B1 HOMOGENEOUS.
*
* OUTPUT PARAMETERS (RELATED TO THE HOMOGENEOUS MEDIA):
*  IJJ0,IJJ1 : MOST THERMAL GROUP IN BAND FOR P0 AND P1 SCATTERING.
*              DIMENSION IJJ0(NGRO),IJJ1(NGRO)
*  NJJ0,NJJ1 : NUMBER OF GROUPS IN BAND FOR P0 AND P1 SCATTERING.
*              DIMENSION NJJ0(NGRO),NJJ1(NGRO)
*  IDEL    : DIMENSION OF MATRICES SCAT0 AND SCAT1. DIMENSION IDEL(2)
*  FLXIN   : INTEGRATED FLUXES. DIMENSION FLXIN(NGRO)
*  SA      : ABSORPTION MACROSCOPIC CROSS SECTIONS. DIMENSION SA(NGRO)
*  ST      : TOTAL MACROSCOPIC CROSS SECTIONS. DIMENSION ST(NGRO)
*  SFNU    : NU * MACROSCOPIC FISSION CROSS-SECTIONS.
*            DIMENSION SFNU(NGRO)
*  XHI     : FISSION SPECTRUM. DIMENSION XHI(NGRO)
*  SCAT0   : PACKED DIFFUSION P0 MACROSCOPIC CROSS SECTIONS.
*  SCAT1   : PACKED DIFFUSION P1 MACROSCOPIC CROSS SECTIONS.
*            DIMENSION SCAT0(IDEL(1)),SCAT1(IDEL(2))
*  SIGF0   : SPACE FOR FISSION VECTORS. SIGF0(NBM,NFISSI)
*  NGROIN  : NUMBER OF GROUPS WITHOUT UP-SCATTERING.
*  B2      : ORIGINAL DIRECTION DEPENDANT B2 DIMENSION B2(4)
*  B2HOM   : HOMOGENEOUS B2 DOUBLE PRECISION
*
* SCRATCH PARAMETERS:
*  XSCAT   : SCATTERING VECTOR                    R(0:NBM*NGRO)
*            XSCAT(0) = 0.0
*  NJJ     : NUMBER OF SCATTERING GROUP           I(0:NBM)
*            NJJ(0) = -NGROUP
*  IJJ     : LAST SCATTERING GROUP                I(0:NBM)
*            IJJ(0) = 0
*  IPOS    : POSITION SELF SCATTERING IN XSCAT    I(0:NBM)
*            IPOS(0)= NGROUP+1
*  GAR     : DIMENSION GAR(NBM+1)
*  GAF     : DIMENSION DOUBLE PRECISION GAF(NGRO)
*  A14     : DIMENSION A14(2*NFISSI*NBM)
*
*  AUTHOR  : A. HEBERT 88/02/08
*  MODIFIED: R. ROY    98/02/24
*            REPLACE 'SIGS 1' BY 'TRANC'
*            FOR TRANSPORT CORRECTION IN BOTR
*
*
      PARAMETER (NSTATE=40)
      LOGICAL    LOGIC
      CHARACTER  CGRPNM*12,CM*2
      DOUBLE PRECISION SUM,A11,A13,A14(NFISSI,0:NBM),FLXIN(*),CUR(*),
     1 GAF(*),B2T(3),B2HOM,DSIGS
      DIMENSION VOL(*),MAT(*),KEYFLX(*),FLUX(MAXPTS,*),IJJ0(*),IJJ1(*),
     1 NJJ0(*),NJJ1(*),IDEL(2),GAR(0:NBM),XSCAT(0:NBM*NGRO),
     2 SA(*),ST(*),SFNU(*),XHI(*),SCAT0(*),SCAT1(*),SIGF0(NBM,*),
     3 ISTATE(NSTATE),B2(4),IJJ(0:NBM),NJJ(0:NBM),IPOS(0:NBM)
*
      CALL XDRSET(ISTATE,NSTATE,0)
      CALL LCMGET(IPMACR,'STATE-VECTOR',ISTATE)
      LOGIC=(  NGRO.EQ.ISTATE(1)).AND.(NBM.EQ.ISTATE(2)).AND.
     1      (NFISSI.EQ.ISTATE(4)).AND.(ISTATE(3).GE.1)
      IF(.NOT.LOGIC) CALL XABORT('B1HXS1: INCONSISTENT LCM FILE.')
      IANN=IAN
      IF(IAN.LT.0) IANN=-(IAN+1)
      IDEL(1)=0
      IDEL(2)=0
      DO 30 LLL=1,NGRO
      WRITE (CGRPNM,'(5HGROUP,I3,1H/,I3)') LLL,NGRO
      CALL LCMSIX(IPMACR,CGRPNM,1)
      DO 20 M=0,IANN
      WRITE (CM,'(I2)') M
      CALL LCMGET(IPMACR,'NJJD'//CM,NJJ(1))
      CALL LCMGET(IPMACR,'IJJD'//CM,IJJ(1))
      IMAX=1
      IMIN=NGRO
      DO 10 I=1,NBM
      IMAX=MAX(IJJ(I),IMAX)
   10 IMIN=MIN(IJJ(I)-NJJ(I)+1,IMIN)
      IF(M.EQ.0) THEN
         IJJ0(LLL)=IMAX
         NJJ0(LLL)=IMAX-IMIN+1
      ELSE IF(M.EQ.1) THEN
         IJJ1(LLL)=IMAX
         NJJ1(LLL)=IMAX-IMIN+1
      ENDIF
      IDEL(M+1)=IDEL(M+1)+IMAX-IMIN+1
   20 CONTINUE
      CALL LCMSIX(IPMACR,' ',2)
   30 CONTINUE
      RETURN
*
      ENTRY B1HXS2(MAXPTS,IPMACR,IPAS,NGRO,NBM,IAN,NFISSI,VOL,MAT,
     1 KEYFLX,FLUX,IJJ0,IJJ1,NJJ0,NJJ1,IDEL,XSCAT,IJJ,NJJ,IPOS,GAR,GAF,
     2 A14,FLXIN,SA,ST,SFNU,XHI,SCAT0,SCAT1,SIGF0,CUR,NGROIN,INORM,
     3 B2,B2HOM)
*
      IANN=IAN
      IF(IAN.LT.0) IANN=-(IAN+1)
      NGROIN=0
      SUM=0.0D0
      DO 40 NF=1,NFISSI
      DO 40 IBM=1,NBM
   40 A14(NF,IBM)=0.0D0
      DO 60 LLL=1,NGRO
      WRITE (CGRPNM,'(5HGROUP,I3,1H/,I3)') LLL,NGRO
      CALL LCMSIX(IPMACR,CGRPNM,1)
      CALL LCMLEN(IPMACR,'NUSIGF',ILCMLN,ILCMTY)
      IF(ILCMLN.GT.0) THEN
        CALL LCMGET(IPMACR,'NUSIGF',SIGF0)
        DO 50 I=1,IPAS
          IBM=MAT(I)
          IF(IBM.GT.0) THEN
            IFLXL=KEYFLX(I)
            DO 51 NF=1,NFISSI
              A14(NF,IBM)=A14(NF,IBM)+FLUX(IFLXL,LLL)*VOL(I)*
     >                                SIGF0(IBM,NF)
 51         CONTINUE
          ENDIF
 50     CONTINUE
      ENDIF
      CALL LCMSIX(IPMACR,' ',2)
   60 CONTINUE
      A13=0.0D0
      DO 70 NF=1,NFISSI
      DO 70 IBM=1,NBM
   70 A13=A13+A14(NF,IBM)
*
      IF(INORM.GE.2) THEN
        B2HOM=B2(4)
        DO 85 LLL=1,NGRO
        A11=0.0D0
        DO 80 I=1,IPAS
   80   A11=A11+FLUX(KEYFLX(I),LLL)*VOL(I)
   85   FLXIN(LLL)=A11
        IDEL(1)=0
        IDEL(2)=0
      ENDIF
*
      DO 200 LLL=1,NGRO
      WRITE (CGRPNM,'(5HGROUP,I3,1H/,I3)') LLL,NGRO
      CALL LCMSIX(IPMACR,CGRPNM,1)
      A11=0.0D0
      CALL LCMLEN(IPMACR,'NUSIGF',ILCMLN,ITYLCM)
      IF(ILCMLN.GT.0) THEN
        CALL LCMGET(IPMACR,'NUSIGF',SIGF0)
        DO 90 I=1,IPAS
          IBM=MAT(I)
          IF(IBM.GT.0) THEN
            IFLXL=KEYFLX(I)
            DO 91 NF=1,NFISSI
              A11=A11+FLUX(IFLXL,LLL)*VOL(I)*SIGF0(IBM,NF)
   91       CONTINUE
          ENDIF
   90   CONTINUE
      ENDIF
      SFNU(LLL)=A11/FLXIN(LLL)
*
      IF(INORM.GE.2) THEN
        CALL LCMGET(IPMACR,'TOTAL',GAR(1))
        A11=0.0D0
        DO 100 I=1,IPAS
        IBM=MAT(I)
  100   A11=A11+FLUX(KEYFLX(I),LLL)*VOL(I)*GAR(IBM)
        ST(LLL)=A11/FLXIN(LLL)
      ELSE
        A11=ST(LLL)*FLXIN(LLL)
      ENDIF
*
      CALL LCMGET(IPMACR,'NJJA 0',NJJ(1))
      CALL LCMGET(IPMACR,'IJJA 0',IJJ(1))
      CALL LCMGET(IPMACR,'IPOA 0',IPOS(1))
      CALL LCMGET(IPMACR,'SCAA 0',XSCAT(1))
      DO 95 IMIX=1,NBM
        DSIGS=0.0D0
        DO 96 IG2=IJJ(IMIX)-NJJ(IMIX)+1,IJJ(IMIX)
          IXSS=IPOS(IMIX)+IJJ(IMIX)-IG2
          DSIGS=DSIGS+DBLE(XSCAT(IXSS))
 96     CONTINUE
        GAR(IMIX)=REAL(DSIGS)
 95   CONTINUE
      DO 110 I=1,IPAS
      IBM=MAT(I)
  110 A11=A11-FLUX(KEYFLX(I),LLL)*VOL(I)*GAR(IBM)
      SA(LLL)=A11/FLXIN(LLL)
*
      IF(NFISSI.GT.0) THEN
         A11=0.0D0
         CALL LCMLEN(IPMACR,'CHI',ILCMLN,ITYLCM)
         IF(ILCMLN.GT.0) THEN
           CALL LCMGET(IPMACR,'CHI',SIGF0)
           DO 120 NF=1,NFISSI
             DO 121 IBM=1,NBM
               A11=A11+A14(NF,IBM)*SIGF0(IBM,NF)
  121        CONTINUE
  120      CONTINUE
         ENDIF
         XHI(LLL)=A11/A13
         SUM=SUM+XHI(LLL)
      ELSE
         XHI(LLL)=0.0
      ENDIF
      IF(INORM.GE.2) THEN
*
* TRANSPORT CORRECTION
        A11=0.0D0
        IF(IAN.EQ.-1) THEN
           CALL LCMGET(IPMACR,'TRANC',GAR(1))
           DO 130 I=1,IPAS
           IBM=MAT(I)
  130      A11=A11+FLUX(KEYFLX(I),LLL)*VOL(I)*GAR(IBM)
           ST(LLL)=ST(LLL)-A11/FLXIN(LLL)
        ENDIF
*
        DO 190 M=0,IANN
        WRITE (CM,'(I2)') M
        CALL LCMGET(IPMACR,'NJJD'//CM,NJJ(1))
        CALL LCMGET(IPMACR,'IJJD'//CM,IJJ(1))
        CALL LCMGET(IPMACR,'IPOD'//CM,IPOS(1))
        CALL LCMGET(IPMACR,'SCAD'//CM,XSCAT(1))
        DO 140 IG=1,NGRO
  140   GAF(IG)=0.0D0
        DO 160 I=1,IPAS
        IBM=MAT(I)
        DO 150 IG=IJJ(IBM)-NJJ(IBM)+1,IJJ(IBM)
        IGAR=IPOS(IBM)+IJJ(IBM)-IG
  150   GAF(IG)=GAF(IG)+FLUX(KEYFLX(I),IG)*VOL(I)*XSCAT(IGAR)
        IF(IAN.EQ.-1) THEN
           IGAR=IPOS(IBM)+IJJ(IBM)-LLL
           GAF(LLL)=GAF(LLL)-FLUX(KEYFLX(I),LLL)*VOL(I)*GAR(IBM)
        ENDIF
  160   CONTINUE
        IF(M.EQ.0) THEN
           DO 170 IG=IJJ0(LLL)-NJJ0(LLL)+1,IJJ0(LLL)
           IGAR=IDEL(1)+1+IJJ0(LLL)-IG
  170      SCAT0(IGAR)=GAF(IG)/FLXIN(IG)
           IDEL(1)=IDEL(1)+NJJ0(LLL)
        ELSE IF(M.EQ.1) THEN
           DO 180 IG=IJJ1(LLL)-NJJ1(LLL)+1,IJJ1(LLL)
           IGAR=IDEL(2)+1+IJJ1(LLL)-IG
  180      SCAT1(IGAR)=GAF(IG)/FLXIN(IG)
           IDEL(2)=IDEL(2)+NJJ1(LLL)
        ENDIF
  190   CONTINUE
        LOGIC=(IJJ0(LLL).LE.LLL).AND.(NGROIN.EQ.LLL-1)
        IF(IANN.GE.1) THEN
           LOGIC=LOGIC.AND.(IJJ1(LLL).LE.LLL)
        ENDIF
        IF(LOGIC) NGROIN=LLL
      ENDIF
      CALL LCMSIX(IPMACR,' ',2)
  200 CONTINUE
      IF((ABS(1.0D0-SUM).GT.1.0D-3).AND.(NFISSI.GT.0)) THEN
         CALL XABORT('B1HXS2: INCONSISTENT FISSION SPECTRUM.')
      ENDIF
      RETURN
*
      ENTRY B1HXS3(MAXPTS,IPMACR,IPAS,NGRO,NBM,IAN,NFISSI,VOL,MAT,
     1 KEYFLX,FLUX,IJJ0,IJJ1,NJJ0,NJJ1,IDEL,XSCAT,IJJ,NJJ,IPOS,GAR,GAF,
     2 A14,FLXIN,SA,ST,SFNU,XHI,SCAT0,SCAT1,SIGF0,CUR,NGROIN,INORM,
     3 B2,B2HOM)
*
      IANN=IAN
      IF(IAN.LT.0) IANN=-(IAN+1)
      NGROIN=0
*
* FIND HOMOGENISED FLUX AND CURRENTS
      DO 300 LLL=1,NGRO
        FLXIN(LLL)=0.0D0
        DO 301 I=1,IPAS
          FLXIN(LLL)=FLXIN(LLL)+FLUX(KEYFLX(I),LLL)*VOL(I)
 301    CONTINUE
 300  CONTINUE
      IF(B2(4).EQ.0.0) THEN
        B2T(1)=1.0D0
        B2T(2)=1.0D0
        B2T(3)=1.0D0
      ELSE IF(INORM.EQ.-1) THEN
        B2T(1)=DBLE(B2(1))/DBLE(B2(4))
        B2T(2)=DBLE(B2(2))/DBLE(B2(4))
        B2T(3)=DBLE(B2(3))/DBLE(B2(4))
      ENDIF
      DO 302 LLL=1,NGRO
        CUR(LLL)=0.0D0
        DO 303 IDIR=1,3
          A13=0.0D0
          ILOCR=IDIR*IPAS
          DO 304 I=1,IPAS
            A13=A13+FLUX(ILOCR+KEYFLX(I),LLL)*VOL(I)
 304      CONTINUE
          CUR(LLL)=CUR(LLL)+A13*B2T(IDIR)
 303    CONTINUE
 302  CONTINUE
      B2HOM=DBLE(B2(4))
*
      IDEL(1)=0
      IDEL(2)=0
      DO 310 LLL=1,NGRO
      WRITE (CGRPNM,'(5HGROUP,I3,1H/,I3)') LLL,NGRO
      CALL LCMSIX(IPMACR,CGRPNM,1)
      CALL LCMGET(IPMACR,'TOTAL',GAR(1))
      A11=0.0D0
      DO 320 I=1,IPAS
        IBM=MAT(I)
        A11=A11+FLUX(KEYFLX(I),LLL)*VOL(I)*GAR(IBM)
 320  CONTINUE
      ST(LLL)=A11/FLXIN(LLL)
      A11=ST(LLL)*CUR(LLL)
      DO 321 IDIR=1,3
        ILOCR=IDIR*IPAS
        DO 322 I=1,IPAS
          IBM=MAT(I)
           A11=A11-B2T(IDIR)*VOL(I)*GAR(IBM)*FLUX(ILOCR+KEYFLX(I),LLL)
 322    CONTINUE
 321  CONTINUE
*
      DO 330 M=0,IANN
        WRITE (CM,'(I2)') M
        CALL LCMGET(IPMACR,'NJJD'//CM,NJJ(1))
        CALL LCMGET(IPMACR,'IJJD'//CM,IJJ(1))
        CALL LCMGET(IPMACR,'IPOD'//CM,IPOS(1))
        CALL LCMGET(IPMACR,'SCAD'//CM,XSCAT(1))
        DO 331 IG=1,NGRO
          GAF(IG)=0.0D0
 331    CONTINUE
        IF(M.EQ.0) THEN
          DO 332 I=1,IPAS
            IBM=MAT(I)
            DO 333 IG=IJJ(IBM)-NJJ(IBM)+1,IJJ(IBM)
              IGAR=IPOS(IBM)+IJJ(IBM)-IG
              GAF(IG)=GAF(IG)+FLUX(KEYFLX(I),IG)*VOL(I)*XSCAT(IGAR)
 333        CONTINUE
 332      CONTINUE
          DO 334 IG=IJJ0(LLL)-NJJ0(LLL)+1,IJJ0(LLL)
            IGAR=IDEL(1)+1+IJJ0(LLL)-IG
            SCAT0(IGAR)=GAF(IG)/FLXIN(IG)
 334      CONTINUE
          IDEL(1)=IDEL(1)+NJJ0(LLL)
        ELSE IF(M.EQ.1) THEN
          DO 335 IDIR=1,3
            ILOCR=IDIR*IPAS
            DO 336 I=1,IPAS
              IBM=MAT(I)
              DO 337 IG=IJJ(IBM)-NJJ(IBM)+1,IJJ(IBM)
                IGAR=IPOS(IBM)+IJJ(IBM)-IG
                GAF(IG)=GAF(IG)+B2T(IDIR)*VOL(I)*XSCAT(IGAR)
     >                         *FLUX(ILOCR+KEYFLX(I),IG)
 337          CONTINUE
 336        CONTINUE
 335      CONTINUE
          GAF(LLL)=GAF(LLL)+A11
          DO 338 IG=IJJ1(LLL)-NJJ1(LLL)+1,IJJ1(LLL)
            IGAR=IDEL(2)+1+IJJ1(LLL)-IG
            SCAT1(IGAR)=GAF(IG)/CUR(IG)
 338      CONTINUE
          IDEL(2)=IDEL(2)+NJJ1(LLL)
        ENDIF
 330  CONTINUE
      LOGIC=(IJJ0(LLL).LE.LLL).AND.(NGROIN.EQ.LLL-1)
      IF(IANN.GE.1) THEN
         LOGIC=LOGIC.AND.(IJJ1(LLL).LE.LLL)
      ENDIF
      IF(LOGIC) NGROIN=LLL
      CALL LCMSIX(IPMACR,' ',2)
 310  CONTINUE
      RETURN
      END
