*DECK BIVFSH
      SUBROUTINE BIVFSH (IEX,NGCOND,IGR,NMERGE,FLXMER,SUNMER,NREG,NUN,
     1 ISPLH,NELEM,SIDE,MAT,KN,QFR,VOL,MERG,COUR,FUNKNO,T,RH,RT,SOURCE,
     2 ZGAR1,ZGAR2)
C
C-----------------------------------------------------------------------
C
C  SOURCE TERM CALCULATION FOR FINITE ELEMENT OR MESH CORNER FINITE
C  DIFFERENCES IN HEXAGONAL GEOMETRY.
C
C  INPUT PARAMETERS:
C   IEX    : ITERATION NUMBER.
C   NGCOND : NUMBER OF GROUPS CONDENSED.
C   IGR    : ENERGY GROUP INDEX.
C   NMERGE : NUMBER OF MIXTURES.
C   FLXMER : FLUX ESTIMATE PER MIXTURE. DIMENSION FLXMER(NMERGE,NGCOND)
C   SUNMER : INCOMING SOURCE (SCATTERING+FISSION) X-SECTIONS.
C            DIMENSION SUNMER(NMERGE,NGCOND)
C   NREG   : NUMBER OF HEXAGONS IN BIVAC.
C   NUN    : NUMBER OF UNKNOWN PER GROUP IN BIVAC.
C   ISPLH  : =1 : HEXAGONAL ELEMENTS ; .GT.1 : TRIANGULAR ELEMENTS.
C   NELEM  : NUMBER OF FINITE ELEMENTS (HEXAGONS OR TRIANGLES) EXCLUDING
C            THE VIRTUAL ELEMENTS.
C   SIDE   : SIDE OF THE HEXAGONS.
C   MAT    : MIXTURE INDEX PER HEXAGON. DIMENSION MAT(NREG)
C   KN     : ELEMENT-ORDERED UNKNOWN LIST. DIMENSION KN((LH+1)C NELEM)
C            WHERE LH=6 (HEXAGONS) OR 3 (TRIANGLES).
C   QFR    : ELEMENT-ORDERED INFORMATION. DIMENSION QFR((LH+1)C NELEM)
C   VOL    : VOLUME OF HEXAGONS. DIMENSION VOL(NREG)
C   MERG   : INDEX OF MERGED HEXAGONS. DIMENSION MERG(NREG)
C   COUR   : FOUR TIMES THE INCOMING CURRENT PER UNIT SURFACE.
C   FUNKNO : PREVIOUSLY CALCULATED FLUXES. DIMENSION FUNKNO(NUN,NGCOND)
C   T,RH,RT : UNIT MATRICES. DIMENSION T(2),RH(6,6),RT(3,3)
C
C  OUTPUT PARAMETER:
C    SOURCE : FISSION AND DIFFUSION SOURCES. DIMENSION SOURCE(NUN)
C
C  SCRATCH STORAGE:
C    ZGAR1  : DIMENSION ZGAR1(NBMIX) WHERE NBMIX=MAX(MAT(I))
C    ZGAR2  : DIMENSION ZGAR2(NBMIX)
C
C------------------------------------- AUTHOR: A. HEBERT ; 18/07/94 ----
C
      IMPLICIT NONE
C----
C LOCAL VARIABLES
C----
      INTEGER  I,J,LH,ISPLH,NMERGE,NUN,NBMIX,NREG,NL,K,IND1,IND2,
     &         L,NUM1,NELEM,KHEX,IC,I1,IGR2,IEX,NGCOND,IGR
      INTEGER  MAT(*),KN(*),MERG(*),ISR(6,2),ISRH(6,2),ISRT(3,2)
      REAL     CONST,AA,SIDE,COUR,QFR1,VOL0,PV
      REAL     FLXMER(NMERGE,*),SUNMER(NMERGE,*),QFR(*),
     1         VOL(*),FUNKNO(NUN,*),SOURCE(*),ZGAR1(*),ZGAR2(*),
     2         RH2(6,6),TH(6),T(*),RH(6,*),RT(3,*)
C----
C DATA
C----
      DATA     ISRH
     &              /2,1,4,5,6,3,1,4,5,6,3,2/
      DATA     ISRT
     &              /1,2,3,2,3,1/
C
C RECOVER THE HEXAGONAL MASS (RH2) MATRICES.
C
      IF(ISPLH.EQ.1) THEN
C        HEXAGONAL BASIS.
         LH=6
         DO 10 I=1,6
         DO 10 J=1,2
   10    ISR(I,J)=ISRH(I,J)
         DO 20 I=1,6
         TH(I)=0.0
         DO 20 J=1,6
         RH2(I,J)=RH(I,J)
   20    TH(I)=TH(I)+RH(I,J)
         CONST=1.5*SQRT(3.0)
         AA=SIDE
      ELSE
*        TRIANGULAR BASIS.
         LH=3
         DO 30 I=1,3
         DO 30 J=1,2
   30    ISR(I,J)=ISRT(I,J)
         DO 40 I=1,3
         TH(I)=0.0
         DO 40 J=1,3
         RH2(I,J)=RT(I,J)
   40    TH(I)=TH(I)+RT(I,J)
         CONST=0.25*SQRT(3.0)
         AA=SIDE/REAL(ISPLH-1)
      ENDIF
*
      NBMIX=0
      DO 50 I=1,NREG
   50 NBMIX=MAX(NBMIX,MAT(I))
      DO 60 I=1,NUN
   60 SOURCE(I)=0.0
*
* INCOMING CURRENT SOURCE.
      IF(COUR.NE.0.0) THEN
         NUM1=0
         DO 90 K=1,NELEM
         KHEX=KN(NUM1+LH+1)
         IF(VOL(KHEX).EQ.0.0) GO TO 90
         DO 80 IC=1,LH
         QFR1=QFR(NUM1+IC)
         IF(QFR1.EQ.0.0) GO TO 80
         DO 70 I1=1,2
         IND1=KN(NUM1+ISR(IC,I1))
         IF(IND1.NE.0) SOURCE(IND1)=SOURCE(IND1)+COUR*QFR1*T(I1)
   70    CONTINUE
   80    CONTINUE
   90    NUM1=NUM1+LH+1
      ENDIF
*
* DIFFUSION AND FISSION SOURCE.
      IGR2=NGCOND
      IF(IEX.EQ.1) IGR2=IGR-1
      DO 150 NL=1,IGR2
      DO 100 L=1,NBMIX
      ZGAR1(L)=0.0
  100 ZGAR2(L)=0.0
      NUM1=0
      DO 120 K=1,NELEM
      KHEX=KN(NUM1+LH+1)
      IF(VOL(KHEX).EQ.0.0) GO TO 120
      L=MAT(KHEX)
      VOL0=QFR(NUM1+LH+1)
      ZGAR1(L)=ZGAR1(L)+VOL0
      DO 110 I=1,LH
      IND1=KN(NUM1+I)
      IF(IND1.EQ.0) GO TO 110
      ZGAR2(L)=ZGAR2(L)+TH(I)*VOL0*FUNKNO(IND1,NL)
  110 CONTINUE
  120 NUM1=NUM1+LH+1
      NUM1=0
      DO 150 K=1,NELEM
      KHEX=KN(NUM1+LH+1)
      IF(VOL(KHEX).EQ.0.0) GO TO 150
      L=MAT(KHEX)
      VOL0=QFR(NUM1+LH+1)
      PV=SUNMER(MERG(KHEX),NL)*FLXMER(MERG(KHEX),NL)*ZGAR1(L)/ZGAR2(L)
      DO 140 I=1,LH
      IND1=KN(NUM1+I)
      IF(IND1.EQ.0) GO TO 140
      DO 130 J=1,LH
      IND2=KN(NUM1+J)
      IF(IND2.EQ.0) GO TO 130
      SOURCE(IND1)=SOURCE(IND1)+RH2(I,J)*FUNKNO(IND2,NL)*VOL0*PV
  130 CONTINUE
  140 CONTINUE
  150 NUM1=NUM1+LH+1
      IF(IEX.GT.1) RETURN
      NUM1=0
      DO 190 K=1,NELEM
      KHEX=KN(NUM1+LH+1)
      IF(VOL(KHEX).EQ.0.0) GO TO 190
      VOL0=QFR(NUM1+LH+1)
      PV=0.0
      DO 170 NL=IGR,NGCOND
  170 PV=PV+SUNMER(MERG(KHEX),NL)*FLXMER(MERG(KHEX),NL)
      DO 180 I=1,LH
      IND1=KN(NUM1+I)
      IF(IND1.NE.0) SOURCE(IND1)=SOURCE(IND1)+TH(I)*VOL0*PV
  180 CONTINUE
  190 NUM1=NUM1+LH+1
      RETURN
      END
