*DECK BIVG2H
      SUBROUTINE BIVG2H (DIFF0,SIGMA,NREG,NUN,LL4,ISPLH,SIDE,MAT,KN,
     1 QFR,BFR,VOL,IDL,MU,SOURCE,FUNKNO,A11)
C
C-----------------------------------------------------------------------
C
C  MESH CENTERED FINITE DIFFERENCE FLUX CALCULATION IN A SINGLE ENERGY
C  GROUP. HEXAGONAL GEOMETRY.
C
C  INPUT PARAMETERS:
C   DIFF0  : DIFFUSION COEFFICIENTS. DIMENSION DIFF0(NBMIX)
C   SIGMA  : REMOVAL MACROSCOPIC X-SECTIONS. DIMENSION SIGMA(NBMIX)
C            WHERE NBMIX IS THE NUMBER OF MIXTURES.
C   NREG   : NUMBER OF HEXAGONS IN BIVAC.
C   NUN    : DIMENSION OF VECTOR FUNKNO.
C   LL4    : NUMBER OF UNKNOWNS PER GROUP IN BIVAC. EQUAL TO THE NUMBER
C            OF FINITE ELEMENTS (HEXAGONS OR TRIANGLES) EXCLUDING THE
C            VIRTUAL ELEMENTS.
C   ISPLH  : =1 : HEXAGONAL ELEMENTS ; .GT.1 : TRIANGULAR ELEMENTS.
C   SIDE   : SIDE OF THE HEXAGONS.
C   MAT    : MIXTURE INDEX PER HEXAGON. DIMENSION MAT(NREG)
C   KN     : ELEMENT-ORDERED UNKNOWN LIST. DIMENSION KN((LH+1)C LL4)
C            WHERE LH=6 (HEXAGONS) OR 3 (TRIANGLES).
C   QFR    : ELEMENT-ORDERED INFORMATION. DIMENSION QFR((LH+1)C LL4)
C   BFR    : ELEMENT-ORDERED SURFACE FRACTIONS.  BFR((LH+1)C LL4)
C   VOL    : VOLUME OF HEXAGONS. DIMENSION VOL(NREG)
C   IDL    : POSITION OF THE AVERAGE FLUX COMPONENT ASSOCIATED WITH
C            EACH VOLUME. DIMENSION IDL(NREG)
C   MU     : INDICES USED WITH COMPRESSED DIAGONAL STORAGE MODE MATRIX
C            A11. DIMENSION MU(LL4)
C   SOURCE : FISSION AND DIFFUSION SOURCES. DIMENSION SOURCE(LL4)
C
C  OUTPUT PARAMETER:
C   FUNKNO : NEUTRON FLUXES. SURFACE-AVERAGED FLUX ARE IN POSITION
C            FUNKNO(NUN). DIMENSION FUNKNO(NUN)
C
C  SCRATCH STORAGE:
C   A11    : DIMENSION A11(MU(NREG))
C
C------------------------------------- AUTHOR: A. HEBERT ; 15/07/94 ----
C
      IMPLICIT NONE
C----
C INTERFACE AND LOCAL VARIABLES
C----
      INTEGER          ISPLH,NSURF,I,LL4,NUM1,IND1,IND2,IC,NREG,
     &                 NUN,KHEX,L,KEY0,IX,K,KEY,LL
      INTEGER          IDL(*),MU(*),MAT(*),KN(*)
      REAL             DIF1,DIF2,X1,X2,DS,SIDE,FACT,VOL0,SIDEB
      REAL             DIFF0(*),SIGMA(*),QFR(*),BFR(*),VOL(*),
     1                 SOURCE(*),FUNKNO(*),A11(*)
      DOUBLE PRECISION A1,VAR1
C----
C INTERNAL FUNCTION
C----
      DOUBLE PRECISION DHARM
      DHARM(X1,X2,DIF1,DIF2)=2.0D0*DIF1*DIF2/(X1*DIF2+X2*DIF1)
C
C-----
C
      IF(ISPLH.EQ.1) THEN
         DS=SQRT(3.0)*SIDE
         FACT=2.0/(3.0*DS)
         NSURF=6
      ELSE
         DS=SIDE/(SQRT(3.0)*REAL(ISPLH-1))
         FACT=4.0/(3.0*DS)
         NSURF=3
      ENDIF
*
* ASSEMBLY OF THE MAIN COEFFICIENT MATRIX.
      DO 10 I=1,MU(LL4)
   10 A11(I)=0.0
      NUM1=0
      DO 30 IND1=1,LL4
      KHEX=KN(NUM1+NSURF+1)
      IF(VOL(KHEX).EQ.0.0) GO TO 30
      L=MAT(KHEX)
      VOL0=QFR(NUM1+NSURF+1)
      SIDEB=FACT*VOL0
      VAR1=0.0D0
      KEY0=MU(IND1)-IND1
      DO 20 IX=1,NSURF
      IND2=KN(NUM1+IX)
      IF(IND2.GT.0) THEN
         LL=MAT(KN(IND2*(NSURF+1)))
         A1=DHARM(DS,DS,DIFF0(L),DIFF0(LL))*SIDEB
      ELSE IF(IND2.EQ.-1) THEN
         A1=DHARM(DS,DS,DIFF0(L),DS*QFR(NUM1+IX)/2.0)*SIDEB
      ELSE IF(IND2.EQ.-2) THEN
         A1=0.0D0
      ELSE IF(IND2.EQ.-3) THEN
         A1=2.0D0*DHARM(DS,DS,DIFF0(L),DIFF0(L))*SIDEB
      ENDIF
      VAR1=VAR1+A1
      IF(IND2.GT.0) THEN
         IF(IND2.LT.IND1) THEN
            KEY=KEY0+IND2
            A11(KEY)=A11(KEY)-A1
         ENDIF
      ENDIF
   20 CONTINUE
      KEY=KEY0+IND1
      A11(KEY)=A11(KEY)+VAR1+SIGMA(L)*VOL0
   30 NUM1=NUM1+NSURF+1
*
* L-D-L(T) FACTORIZATION AND RESOLUTION.
      CALL ALLDLF (LL4,A11,MU)
      DO 90 I=1,LL4
   90 FUNKNO(I)=SOURCE(I)
      CALL ALLDLS (LL4,MU,A11,FUNKNO)
*
* CALCULATION OF ELEMENT-AVERAGED FLUXES.
      IF(ISPLH.GT.1) THEN
         NSURF=3
         DO 100 K=1,NREG
         IF(IDL(K).NE.0) FUNKNO(IDL(K))=0.0
  100    CONTINUE
         NUM1=0
         DO 110 IND1=1,LL4
         K=KN(NUM1+NSURF+1)
         IF(VOL(K).EQ.0.0) GO TO 110
         FUNKNO(IDL(K))=FUNKNO(IDL(K))+QFR(NUM1+NSURF+1)*FUNKNO(IND1)/
     1   VOL(K)
  110    NUM1=NUM1+NSURF+1
      ELSE
         NSURF=6
      ENDIF
*
* CALCULATION OF SURFACE-AVERAGED FLUX.
      FUNKNO(NUN)=0.0
      NUM1=0
      DO 130 IND1=1,LL4
      K=KN(NUM1+NSURF+1)
      IF(VOL(K).EQ.0.0) GO TO 130
      DO 120 IC=1,NSURF
      IF(BFR(NUM1+IC).GT.0.0) THEN
         FUNKNO(NUN)=FUNKNO(NUN)+FUNKNO(IND1)*BFR(NUM1+IC)
      ENDIF
  120 CONTINUE
  130 NUM1=NUM1+NSURF+1
      RETURN
      END
