*DECK CPOMAW
      SUBROUTINE CPOMAW(IPCPO ,NGROUP,NL    ,NPROC ,INDPRO,ITYPRO,
     >                  DXSMAC,DSCMAC,DXSREM,DSCREM,DISFC ,XSREC ,
     >                  XSCAT ,SCATC ,IJJ   ,NJJ   ,DMJMAC,
     >                  IFCDIS,DISFAC,DISTMP)

C
C---------------------------  CPOMAW  ---------------------------------
C
C  1- PROGRAMME STATISTICS:
C      NAME     : CPOMAW
C      USE      : PUT MACROSCOPIC CROSS SECTION ON CPO
C      MODIFIED : 97-07-18 (G.M.)
C      AUTHOR   : G.MARLEAU
C
C  2- ROUTINE PARAMETERS:
C    INPUT
C      IPCPO    : POINTER TO THE CPO                 I
C      NGROUP   : NUMBER OF GROUPS CONDENSED         I
C      NL       : NUMBER OF LEGENDRE ORDERS          I
C      NPROC    : NUMBER OF MIC XS TO PROCESS        I
C      INDPRO   : IDENTIFIER FOR XS PROCESSING       I(NPROC)
C      ITYPRO   : IDENTIFIER FOR XS PROCESSED        I(NPROC)
C      DXSMAC   : MAC AVERAGED REGION/GROUP X-S      D(NGROUP,NPROC)
C      DSCMAC   : MAC SCATTERING                     D(NGROUP,NGROUP,
C                                                          NL)
C      DXSREM   : REM AVERAGED REGION/GROUP X-S      D(NGROUP,NPROC)
C      DSCREM   : REMOVE SCATTERING RATES            D(NGROUP,NGROUP,
C                                                          NL)
C      DISFC    : DISADVANTAGE FACTOR                R(NGROUP)
C      IFCDIS   : DISCONTINUITY FACTOR PRESENT (1)   I
C                 OR ABSENT
C      DISFAC   : DISCONTINUITY FACTORS              D(2,NGROUP,3)
C    WORK
C      XSREC    : MICRO VECTOR XS                    R(NGROUP*NPROC)
C      XSCAT    : COMPRESS SCATTERING                R(NGROUP*NGROUP*NL)
C      SCATC    : COMPRESS SCATTERING                R(NGROUP*NGROUP)
C      IJJ      : POSITION OF FIRST DIFFUSION GROUP  I(NGROUP)
C      NJJ      : NUMBER OF DIFFUSION GROUP          I(NGROUP)
C      DMJMAC   : DOUBLE PRECISION ENERGY            D
C      DISTMP   : temporary storage for              R(2,NGROUP)
C                 DISCONTINUITY FACTORS              
C
C
C---------------------------   CPOMAW  --------------------------------
C
      IMPLICIT         NONE
      INTEGER          IPCPO,NGROUP,NL,NPROC,INDPRO(NPROC),
     >                 ITYPRO(NPROC),IJJ(NGROUP),NJJ(NGROUP)
      REAL             DISFC(NGROUP),XSREC(NGROUP,NPROC),
     >                 XSCAT(NGROUP,NGROUP,NL),SCATC(NGROUP*NGROUP)
      DOUBLE PRECISION DXSMAC(NGROUP,NPROC),
     >                 DSCMAC(NGROUP,NGROUP,NL),
     >                 DXSREM(NGROUP,NPROC),
     >                 DSCREM(NGROUP,NGROUP,NL),
     >                 DMJMAC
      INTEGER          IFCDIS
      DOUBLE PRECISION DISFAC(2,NGROUP,3)
      REAL             DISTMP(2,NGROUP)
C----
C  LOCAL PARAMETERS
C----
      INTEGER          NDPROC
      REAL             CUTOFF
      PARAMETER       (NDPROC=20,CUTOFF=1.0E-7)
      INTEGER          IXSR,JXSR,KXSR,IL,IGR,JGR
      REAL             CUTLIM
      DOUBLE PRECISION DNUFI,DNUFT
C----
C   SAVE AVERAGE XS
C----
      CALL XDRSET(ITYPRO,NPROC,1)
      CALL LCMSIX(IPCPO,'MACR',1)
      CALL XDRSET(XSREC,NGROUP*NPROC,0.0)
      CALL XDRSET(XSCAT,NGROUP*NGROUP*NL,0.0)
      DO 100 IXSR=1,4
        IF(INDPRO(IXSR).GT.0.AND.MOD(ITYPRO(IXSR),2).EQ.1) THEN
          DO 101 IGR=1,NGROUP
            XSREC(IGR,IXSR)=REAL(DXSMAC(IGR,IXSR)-DXSREM(IGR,IXSR))
            CUTLIM=ABS(REAL(DXSMAC(IGR,IXSR))*CUTOFF)
            IF(XSREC(IGR,IXSR).LT.CUTLIM)  XSREC(IGR,IXSR)=0.0
 101      CONTINUE
        ENDIF
 100  CONTINUE
      IXSR=5
      IF(INDPRO(IXSR).GT.0.AND.MOD(ITYPRO(IXSR),2).EQ.1) THEN
        JXSR=3
        KXSR=16
        DNUFI=0.0D0
        DNUFT=0.0D0
        DO 120 IGR=1,NGROUP
          DNUFT=DNUFT+DXSMAC(IGR,KXSR)*DXSMAC(IGR,JXSR)
          DNUFI=DNUFI+(DXSMAC(IGR,KXSR)-DXSREM(IGR,KXSR))
     >      *(DXSMAC(IGR,JXSR)-DXSREM(IGR,JXSR))
 120    CONTINUE
        CUTLIM=ABS(REAL(DNUFT)*CUTOFF)
        IF(REAL(DNUFI).GT.CUTLIM) THEN
          DNUFI=1.0D0/DNUFI
          DNUFT=1.0D0/DNUFT
          DO 130 IGR=1,NGROUP
            XSREC(IGR,IXSR)=REAL(DNUFI*(DXSMAC(IGR,IXSR)
     >        -DXSREM(IGR,IXSR)))
 130      CONTINUE
        ENDIF
      ENDIF
      IXSR=6
      IF(INDPRO(IXSR).GT.0.AND.MOD(ITYPRO(IXSR),2).EQ.1) THEN
        JXSR=4
        KXSR=3
        DO 140 IGR=1,NGROUP
          DNUFI=DXSMAC(IGR,JXSR)-DXSREM(IGR,JXSR)
          CUTLIM=ABS(REAL(DXSMAC(IGR,JXSR))*CUTOFF)
          IF(REAL(DNUFI).GT.CUTLIM) THEN
            XSREC(IGR,IXSR)=REAL((DXSMAC(IGR,KXSR)
     >        -DXSREM(IGR,KXSR))/DNUFI)
          ENDIF
 140    CONTINUE
      ENDIF
      DO 150 IXSR=7,NDPROC
        IF(INDPRO(IXSR).GT.0.AND.MOD(ITYPRO(IXSR),2).EQ.1) THEN
          DO 160 IGR=1,NGROUP
            XSREC(IGR,IXSR)=REAL(DXSMAC(IGR,IXSR)-DXSREM(IGR,IXSR))
            CUTLIM=ABS(REAL(DXSMAC(IGR,IXSR))*CUTOFF)
            IF(XSREC(IGR,IXSR).LT.CUTLIM)  XSREC(IGR,IXSR)=0.0
 160      CONTINUE
        ENDIF
 150  CONTINUE
      IL=0
      DO 170 IXSR=NDPROC+1,NDPROC+NL
        IL=IL+1
        IF(INDPRO(IXSR).GT.0.AND.MOD(ITYPRO(IXSR),2).EQ.1) THEN
          DO 180 IGR=1,NGROUP
            XSREC(IGR,IXSR)=REAL(DXSMAC(IGR,IXSR)-DXSREM(IGR,IXSR))
            CUTLIM=ABS(REAL(DXSMAC(IGR,IXSR))*CUTOFF)
            IF(ABS(XSREC(IGR,IXSR)).LT.CUTLIM)
     >         XSREC(IGR,IXSR)=0.0
            DO 190 JGR=1,NGROUP
              XSCAT(IGR,JGR,IL)=REAL(DSCMAC(IGR,JGR,IL)
     >          -DSCREM(IGR,JGR,IL))
              CUTLIM=ABS(REAL(DSCMAC(IGR,JGR,IL))*CUTOFF)
              IF(ABS(XSCAT(IGR,JGR,IL)).LT.CUTLIM)
     >          XSCAT(IGR,JGR,IL)=0.0
 190        CONTINUE
 180      CONTINUE
        ENDIF
 170  CONTINUE
C----
C  COMPUTE AVERAGED ENERGY PER FISSION
C----
      JXSR=4
      KXSR=16
      DNUFI=0.0D0
      DNUFT=0.0D0
      DO 200 IGR=1,NGROUP
        DNUFT=DNUFT+DXSMAC(IGR,KXSR)*DXSMAC(IGR,JXSR)
        DNUFI=DNUFI+(DXSMAC(IGR,KXSR)-DXSREM(IGR,KXSR))
     >    *(DXSMAC(IGR,JXSR)-DXSREM(IGR,JXSR))
 200  CONTINUE
      CUTLIM=ABS(REAL(DNUFT)*CUTOFF)
      IF(REAL(DNUFI).GT.CUTLIM) THEN
        DMJMAC=DMJMAC/DNUFI
      ELSE
        DMJMAC=0.0D0
      ENDIF
C----
C  SAVE CPO MICRO
C----
      CALL XDRLGS(IPCPO,1,NPROC,0,NL-1,1,NGROUP,INDPRO,
     >            ITYPRO,XSREC,XSCAT,SCATC,IJJ,NJJ)
      CALL LCMSIX(IPCPO,'MACR',2)
      IXSR=13
      DO 210 IGR=1,NGROUP
        XSREC(IGR,1)=REAL(DXSMAC(IGR,IXSR))
 210  CONTINUE
      CALL LCMPUT(IPCPO,'OVERV',NGROUP,2,XSREC)
      IXSR=16
      DO 220 IGR=1,NGROUP
        XSREC(IGR,1)=REAL(DXSMAC(IGR,IXSR))
 220  CONTINUE
      CALL LCMPUT(IPCPO,'FLUX-INTG',NGROUP,2,XSREC)
      CALL LCMPUT(IPCPO,'FLUXDISAFACT',NGROUP,2,DISFC)
      IF(IFCDIS .EQ. 1) THEN
        CALL XDRSDB(2*NGROUP,DISTMP,DISFAC(1,1,1),1)
        CALL LCMPUT(IPCPO,'DISFACX',2*NGROUP,2,DISTMP)
        CALL XDRSDB(2*NGROUP,DISTMP,DISFAC(1,1,2),1)
        CALL LCMPUT(IPCPO,'DISFACY',2*NGROUP,2,DISTMP)
        CALL XDRSDB(2*NGROUP,DISTMP,DISFAC(1,1,3),1)
        CALL LCMPUT(IPCPO,'DISFACZ',2*NGROUP,2,DISTMP)
      ENDIF
      RETURN
      END
