*DECK EDIMIC
      SUBROUTINE EDIMIC(IPEDIT,IPLIB ,IPRINT,NGROUP,NGCOND,NREGIO,
     >                  NMERGE,NL    ,NBISO ,NBMIX ,NED   ,ILEAKS,
     >                  ILUPS ,NSSTP ,NTAUXT,B2    ,TIMEF ,NBNMIC,
     >                  HVECT ,ISOCAR,ISONAM,IGCOND,IMERGE,
     >                  FLXINT,DENISO,VOLMER,RATECM,INDISO,INDMIC,
     >                  DNUM  ,DFLXDN,DFLDNC,DNUMMX,ISOCOD,DXSTRD,
     >                  SPH   )
*
*-----------------------------------------------------------------------
*
* HOMOGENIZATION AND CONDENSATION OF MICROSCOPIC CROSS SECTION.
*
* INPUT PARAMETERS:
*  IPEDIT  : POINTER TO THE EDITION
*  IPLIB   : POINTER TO THE INTERNAL MICROSCOPIC CROSS SECTION LIBRARY
*  IPRINT  : PRINT INDEX.
*  NGROUP  : NUMBER OF ENERGY GROUPS.
*  NGCOND  : NUMBER OF CONDENSED GROUPS.
*  NREGIO  : NUMBER OF VOLUMES.
*  NMERGE  : NUMBER OF MERGED REGIONS.
*  NL      : NUMBER OF LEGENDRE ORDERS REQUIRED IN THE CALCULATION.
*  NBISO   : NUMBER OF ISOTOPES.
*  NBMIX   : NUMBER OF MIXTURES.
*  NED     : NUMBER OF EXTRA VECTOR EDITS FROM MATXS.
*  ILEAKS  : LEAK CALCULATION
*            = 1 APPROXIMATE LEAKAGE COEFFICIENT
*            = 2 HOMOGENEOUS LEAKAGE COEFFICIENTS
*            = 3 DIRECTIONAL LEAKAGE COEFFICIENTS
*  ILUPS   : =1 TO REMOVE UP-SCATTERING FROM OUTPUT X-S.
*  NSSTP   : HOMOGENIZED X-S COMPUTATION+SAVE
*            = 0  NO COMPUTE NO SAVE
*            = 1  COMPUTE, NO SAVE
*            = 2  COMPUTE, SAVE
*            = 3  COMPUTE, SAVE MICRO PERTURBATION
*            = 4  ACTIVATION CROSS SECTION
*  NTAUXT  : NUMBER OF REACTION RATE EDITS      I
*  B2      : SQUARE BUCKLING                    R(4)
*            FOR ILEAKS = 2 B2(4) IS HOMOGENEOUS
*            FOR ILEAKS = 3 B2(1),B2(2),B2(3)
*            ARE DIRECTIONAL HETEROGENEOUS
*            AND B2(4) IS HOMOGENEOUS
*  TIMEF   : FINAL TIME (DAYS)/BURNUP/IRRADIATION TIMEF(3)
*  NBNMIC  : NUMBER OF MICROSCOPIC SAVED
*  HVECT   : NAMES OF EXTRA VECTOR EDITS.    HVECT(NED)*6
*  ISOCAR  : NAMES OF SAVED ISOTOPES.        ISOCAR(2,NBNMIC)
*  ISONAM  : NAMES OF ISOTOPES.              ISONAM(3,NBISO)
*  IGCOND  : LIMITS OF CONDENSED GROUPS.     IGCOND(NGCOND)
*  IMERGE  : INDEX OF MERGED REGIONS.        IMERGE(NREGIO)
*  FLXINT  : FLUXES.                         FLXINT(NREGIO,NGROUP,4)
*  DENISO  : NUMBER DENSITY OF EACH ISOTOPE. DENISO(NBISO)
*  VOLMER  : VOLUME OF REGION MERGED.        VOLMER(NMERGE)
*  RATECM  : AVERAGED REGION/GROUP X-S       RATECM(0:NMERGE,NGCOND,
*                                                   NTAUXT)
*  INDISO  : FLAG TO TREAT OR NOT ISOTOPE.   INDISO(NBISO,2)
*  INDMIC  : FLAG TO GENERATE MICROSCOPIC.   INDMIC(NMERGE+1,NBNMIC)
*  DNUM    : DENSITY OF ISOTOPE.             DNUM(NMERGE,NBNMIC)
*  DNUMMX  : NUMBER OF NUCLIDE IN MIX        DNUMMX(NMERGE)
*  ISOCOD  : NEW ISOTOPE IN OLD REGION.      ISOCOD(NREGIO,NBISO)
*  DXSTRD  : EQUIVALENT XS TRANSPORT         D(4,NGCOND,NMERGE)
*  SPH     : SPH HOMOGENIZATION FACTORS      R(NMERGE,NGCOND)
* WORK
*  DFLXDN  : FLUX*VOLUME*DENSITY.            DFLXDN(NGROUP,4)
*  DFLDNC  : FLUX*VOLUME*DENSITY.            DFLDNC(NGCOND,2)
*
*                                     MODIFIED: G. MARLEAU 97/04/02 ----
*-------------------------------------- AUTHOR: A. HEBERT  24/04/89 ----
*
      IMPLICIT         NONE
      INTEGER          IOUT,ILCMUP,ILCMDN,MAXED,NDPROC
      CHARACTER        NAMSBR*6
      PARAMETER       (IOUT=6,ILCMUP=1,ILCMDN=2,MAXED=50,NDPROC=20,
     >                 NAMSBR='EDIMIC')
C----
C  ROUTINE PARAMETERS
C----
      INTEGER          IPEDIT,IPLIB,IPRINT,NGROUP,NGCOND,NREGIO,
     >                 NMERGE,NL,NBISO,NBMIX,NED,ILEAKS,ILUPS,NSSTP,
     >                 NTAUXT,NBNMIC
      INTEGER          ISOCAR(2,NBNMIC),ISONAM(3,NBISO),
     >                 IGCOND(NGCOND),IMERGE(NREGIO),INDISO(NBISO,2),
     >                 INDMIC(NMERGE+1,NBNMIC),ISOCOD(NREGIO,NBISO)
      CHARACTER        HVECT(NED)*6
      REAL             B2(4),TIMEF(3),FLXINT(NREGIO,NGROUP,4),
     >                 DENISO(NBISO),SPH(NMERGE,NGCOND)
      DOUBLE PRECISION VOLMER(NMERGE),RATECM(0:NMERGE,NGCOND,NTAUXT),
     >                 DNUM(NMERGE,NBNMIC),DFLXDN(NGROUP,4),
     >                 DXSTRD(4,NGCOND,NMERGE),DFLDNC(NGCOND,3),
     >                 DNUMMX(NMERGE)
C----
C  MEMORY ALLOCATION
C----
      INTEGER          IBASE(1)
      REAL             RBASE
      COMMON           RBASE(1)
      EQUIVALENCE     (RBASE(1),IBASE(1))
      INTEGER          INDPRO,ITYPRO,IXSREC,IXSCAT,IDRATE,
     >                 IDRSCT,IJJ,NJJ,IXSCMP
C----
C  LOCAL PARAMETERS
C----
      CHARACTER        HNAMIS*12
      INTEGER          IVECT(MAXED),IORDL,IORDE,IMR,IREGIO,IRDJSO,
     >                 INDC,IGRC,IGR,ISOC,MICJSO,IMICR,ISO,ILCMTY,
     >                 NPROC,ILCMLN,MICISO,IOFSC,JSO,IOFRA,IRDISO,ITC
      REAL             TIMEP(2,3),TIMER(3),PERTL,PERTE
      DOUBLE PRECISION DTEMP
C----
C  ALLOCATE MEMORY FOR INDPRO AND ITYPRO
C----
      NPROC=NDPROC+NL
      CALL SETARA(IBASE,NPROC,INDPRO)
      CALL SETARA(IBASE,NPROC,ITYPRO)
C----
C  SET UP INDPRO FOR CROSS SECTION TYPE TO READ
C----
      CALL XDRNED(NPROC,0,NL-1,MAX(2,ILEAKS),NED,HVECT,
     >            IVECT,IBASE(INDPRO))
C----
C TEST IF PERTURBATIVE OR DIRECT MICROSCOPIC XS ON LIBRARY
C----
      IORDL=1
      PERTL=0.0
      CALL LCMLEN(IPLIB,'TIMESPER',ILCMLN,ILCMTY)
      IF(ILCMLN.GE.2.AND.ILCMLN.LE.6) THEN
        CALL LCMGET(IPLIB,'TIMESPER',TIMEP)
        PERTL=TIMEF(1)-TIMEP(1,1)
        IF(PERTL.NE.0.0) IORDL=2
      ENDIF
C----
C TEST IF PERTURBATIVE OR DIRECT MICROSCOPIC XS ON EDITION
C----
      IORDE=1
      PERTE=0.0
      IF(NSSTP.EQ.3) THEN
C----
C  FOR PERTURBATION XS FIND DELTAT
C----
        CALL LCMSIX(IPEDIT,'MACROLIB',ILCMUP)
        CALL LCMLEN(IPEDIT,'TIMESTAMP',ILCMLN,ILCMTY)
        IF(ILCMLN.GE.1.AND.ILCMLN.LE.3) THEN
          CALL LCMGET(IPEDIT,'TIMESTAMP',TIMER)
          TIMEP(1,1)=TIMER(1)
          TIMEP(1,2)=TIMER(2)
          TIMEP(1,3)=TIMER(3)
        ELSE
          TIMEP(1,1)=TIMEF(1)
          TIMEP(1,2)=TIMEF(2)
          TIMEP(1,2)=TIMEF(3)
        ENDIF
        TIMEP(2,1)=TIMEF(1)-TIMEP(1,1)
        TIMEP(2,2)=TIMEF(2)-TIMEP(1,2)
        TIMEP(2,3)=TIMEF(3)-TIMEP(1,3)
        CALL LCMSIX(IPEDIT,'MACROLIB',ILCMDN)
        IF(TIMEP(2,1).NE.0.0) THEN
          PERTE=1.0/DBLE(TIMEP(2,1))
          IORDE=2
          CALL LCMPUT(IPEDIT,'TIMESPER',6,2,TIMEP)
        ENDIF
      ENDIF
C----
C  ALLOCATE MEMORY FOR CROSS SECTION READ
C----
      CALL SETARA(RBASE,NGROUP*NPROC*IORDL,IXSREC)
      CALL SETARA(RBASE,NGROUP*NGROUP*NL*IORDL,IXSCAT)
      CALL SETARA(RBASE,2*NMERGE*NPROC*NGCOND,IDRATE)
      CALL SETARA(RBASE,2*NMERGE*NL*NGCOND*NGCOND,IDRSCT)
      CALL SETARA(IBASE,NGROUP,IJJ)
      CALL SETARA(IBASE,NGROUP,NJJ)
      CALL SETARA(RBASE,NGROUP*NGROUP,IXSCMP)
C----
C  READ INDEPENDENT ISOTOPES CROSS SECTION REQUIRED
C  NAMELY CASES WHERE INDISO(ISO)=-ISO
C----
      DO 130 IMICR=1,NBNMIC
        IF(INDMIC(NMERGE+1,IMICR).EQ.0) GO TO 135
C----
C  INITIALIZE REACTION RATES FOR ALL MERGED REGIONS
C----
        CALL XDRDBL(RBASE(IDRATE),NMERGE*NPROC*NGCOND,0.0D0)
        CALL XDRDBL(RBASE(IDRSCT),NMERGE*NL*NGCOND*NGCOND,0.0D0)
C----
C  MICROSCOPIC IS PRESENT
C----
        DO 140 ISO=1,NBISO
          MICISO=INDISO(ISO,2)
          IRDISO=INDISO(ISO,1)
          IF(MICISO.NE.IMICR.OR.IRDISO.LE.0) GO TO 145
C----
C  READ SET OF XS ASSOCIATED WITH MICROSCOPIC
C----
          WRITE(HNAMIS,'(3A4)') (ISONAM(ITC,ISO),ITC=1,3)
          CALL LCMSIX(IPLIB,HNAMIS,ILCMUP)
          CALL XDRLGS(IPLIB,-1,NPROC,0,NL-1,1,NGROUP,IBASE(INDPRO),
     >                IBASE(ITYPRO),RBASE(IXSREC),RBASE(IXSCAT),
     >                RBASE(IXSCMP),IBASE(IJJ),IBASE(NJJ))
          IF(IORDL.EQ.2) THEN
            IOFRA=IXSREC+NGROUP*NPROC
            IOFSC=IXSCAT+NGROUP*NGROUP*NL
            CALL XDRLGS(IPLIB,-1,NPROC,0,NL-1,2,NGROUP,IBASE(INDPRO),
     >                  IBASE(ITYPRO),RBASE(IOFRA),RBASE(IOFSC),
     >                  RBASE(IXSCMP),IBASE(IJJ),IBASE(NJJ))
            CALL EDIMPR(NGROUP,NPROC,0,NL-1,IBASE(INDPRO),PERTL,
     >                  RBASE(IXSREC),RBASE(IXSCAT))
          ENDIF
          CALL LCMSIX(IPLIB,HNAMIS,ILCMDN)
C----
C  SCAN ALL ISOTOPE USING THIS SET
C----
          DO 150 JSO=ISO,NBISO
            MICJSO=INDISO(JSO,2)
            IRDJSO=ABS(INDISO(JSO,1))
            IF(MICJSO.NE.IMICR.OR.IRDJSO.NE.IRDISO) GO TO 155
C----
C  SCAN ALL REGION ASSOCIATED WITH THIS SET
C----
            DO 160 IREGIO=1,NREGIO
              IMR=IMERGE(IREGIO)
              IF(IMR.EQ.0) GO TO 165
              INDC=INDMIC(IMR,IMICR)
              ISOC=ISOCOD(IREGIO,JSO)
              IF(INDC.EQ.0.OR.ISOC.EQ.0) GO TO 165
C----
C  THIS REGION CONTAINS THE REQUIRED SET OF ISOTOPE AND CONTRIBUTE
C  TO MERGE REGION. CONDENSE AND MERGE CROSS SECTION
C  NOTE - IF TOTAL DENSITY IS NEGATIVE
C         ASSUME UNIFORM DENSITY OF -1.0 (SEE ABOVE)
C----
              IF(DNUM(IMR,IMICR).LT.0.0D0) THEN
                DTEMP=-1.0
              ELSE
                DTEMP=DBLE(DENISO(JSO))
              ENDIF
              DO 161 IGR=1,NGROUP
                DFLXDN(IGR,1)=DBLE(FLXINT(IREGIO,IGR,1))*DTEMP
 161          CONTINUE
              IF(ILEAKS.EQ.3) THEN
                DO 162 IGR=1,NGROUP
                  DFLXDN(IGR,2)=DBLE(FLXINT(IREGIO,IGR,2))*DTEMP
                  DFLXDN(IGR,3)=DBLE(FLXINT(IREGIO,IGR,3))*DTEMP
                  DFLXDN(IGR,4)=DBLE(FLXINT(IREGIO,IGR,4))*DTEMP
 162            CONTINUE
              ENDIF
C----
C  ADD REACTION RATES TO THIS MERGED REGION FOR THIS MICROSCOPIC
C----
              CALL EDIMRR(NGROUP,NGCOND,NPROC,0,NL-1,NMERGE,IMR,
     >                    ILEAKS,IGCOND,IBASE(INDPRO),IBASE(ITYPRO),
     >                    RBASE(IXSREC),RBASE(IXSCAT),DFLXDN,
     >                    RBASE(IDRATE),RBASE(IDRSCT))
C------------------------
C ON A AJOUTE B2 COMME ARGUMENT DANS EDIMRR
C----------------------------
C----
C  EXIT REGION DOES NOT CONTAIN ISOTOPE
C----
 165          CONTINUE
 160        CONTINUE
C----
C  EXIT ISOTOPE IS NOT REQUIRED OR HAS BEEN PROCESSED
C----
 155        CONTINUE
 150      CONTINUE
C----
C  EXIT XS SET IS NOT REQUIRED OR HAS BEEN PROCESSED
C----
 145      CONTINUE
 140    CONTINUE
C----
C  UP-SCATTERING CORRECTIONS.
C----
        IF(ILUPS.EQ.1) THEN
          DO 170 IMR=1,NMERGE
            IF(INDMIC(IMR,IMICR).NE.0) THEN
              CALL EDIUPS(NGCOND,NL,NMERGE,IMR,RBASE(IDRSCT))
            ENDIF
 170      CONTINUE
        ENDIF
C----
C  GENERATE EQUIVALENT MICROSCOPIC CROSS SECTIONS FOR
C  ALL MERGED REGION
C----
        DO 180 IMR=1,NMERGE
          IF(INDMIC(IMR,IMICR).EQ.0) GO TO 185
          DO 181 IGRC=1,NGCOND
            DFLDNC(IGRC,1)=RATECM(IMR,IGRC,1)*DNUM(IMR,IMICR)
            IF(NL.GE.2.AND.ILEAKS.EQ.3) THEN
C----
C  FOR B1 HETEROGENOUS LEAKAGE MODEL
C  COMPUTE DFLDNC= SUM_{K}[ J_{K} B2_{K} ]
C-----
              DFLDNC(IGRC,2)=(RATECM(IMR,IGRC,11)*B2(1)
     >                       +RATECM(IMR,IGRC,12)*B2(2)
     >                       +RATECM(IMR,IGRC,13)*B2(3))
     >                       *DNUM(IMR,IMICR)*RATECM(IMR,IGRC,1)
              DFLDNC(IGRC,3)=0.0D0
              IF (DFLDNC(IGRC,2).NE.0.0D0) THEN
                DTEMP=DBLE(SPH(IMR,IGRC))
                DFLDNC(IGRC,3)=(DTEMP*DTEMP-1.0D0)*(B2(1)+B2(2)+B2(3))
     >                         *DFLDNC(IGRC,1)
     >                       /(3.0D0*DNUMMX(IMR)*DFLDNC(IGRC,2))
                DFLDNC(IGRC,2)=DTEMP/DFLDNC(IGRC,2)
              ENDIF
            ENDIF
            IF (DFLDNC(IGRC,1).NE.0.0D0) THEN
              DFLDNC(IGRC,1)=1.0D0/DFLDNC(IGRC,1)
            ENDIF
 181      CONTINUE
          WRITE(HNAMIS,'(2A4,I4)') (ISOCAR(ITC,IMICR),ITC=1,2),IMR
          CALL LCMSIX(IPEDIT,HNAMIS,ILCMUP)
          IF(IORDE.EQ.2) THEN
            CALL XDRLGS(IPEDIT,-1,NPROC,0,NL-1,1,NGCOND,IBASE(INDPRO),
     >                  IBASE(ITYPRO),RBASE(IXSREC),RBASE(IXSCAT),
     >                  RBASE(IXSCMP),IBASE(IJJ),IBASE(NJJ))
          ENDIF
          CALL EDIMXS(NGCOND,NPROC,0,NL-1,NMERGE,IMR,ILEAKS,
     >                IBASE(INDPRO),DFLDNC,RBASE(IDRATE),
     >                RBASE(IDRSCT),PERTE,RBASE(IXSREC),
     >                RBASE(IXSCAT),DXSTRD(1,1,IMR))
C----
C  SAVE CROSS SECTIONS
C----
          IF(NSSTP.GE.2) THEN
            CALL XDRLGS(IPEDIT,1,NPROC,0,NL-1,IORDE,NGCOND,
     >                  IBASE(INDPRO),IBASE(ITYPRO),RBASE(IXSREC),
     >                  RBASE(IXSCAT),RBASE(IXSCMP),
     >                  IBASE(IJJ),IBASE(NJJ))
          ENDIF
          CALL LCMSIX(IPEDIT,HNAMIS,ILCMDN)
          IF(IPRINT.GE.1) THEN
            WRITE(IOUT,6000) HNAMIS,MAX(DNUM(IMR,IMICR),0.0D0)
            IF(IPRINT.GE.10) THEN
              CALL XDRLPR(NPROC,0,NL-1,IORDE,NGCOND,IBASE(INDPRO),
     >                  IBASE(ITYPRO),RBASE(IXSREC),RBASE(IXSCAT))
            ENDIF
          ENDIF
C----
C  EXIT MERGE REGION DOES NOT CONTAIN ISOTOPE
C----
 185      CONTINUE
 180    CONTINUE
C----
C  EXIT MICROSCOPIC IS ABSENT
C----
 135    CONTINUE
 130  CONTINUE
C----
C  RELEASE MEMORY
C----
      CALL RLSARA(IBASE(NJJ))
      CALL RLSARA(IBASE(IJJ))
      CALL RLSARA(RBASE(IXSCMP))
      CALL RLSARA(RBASE(IDRSCT))
      CALL RLSARA(RBASE(IDRATE))
      CALL RLSARA(RBASE(IXSCAT))
      CALL RLSARA(RBASE(IXSREC))
      CALL RLSARA(IBASE(ITYPRO))
      CALL RLSARA(IBASE(INDPRO))
      RETURN
C----
C  FORMATS
C----
 6000 FORMAT(' NAME OF MERGED/CONDENSED ISOTOPE =',A12,5X,
     >       'DENSITY = ',1P,E15.7)
      END
