*DECK EVORK
      SUBROUTINE EVORK(Y,N,X,HTRY,EPS,YSCAL,HDID,HNEXT,MU1,IMA,MAXA,
     1 NSUPF,NFISS,KFISS,YSF,ADPL,BDPL,DYDX,TEMP,YSAV,DYSAV,TEMR,AK)
C
C-----------------------------------------------------------------------
C
C FIFTH-ORDER RUNGE-KUTTA STEP FOR INTEGRATING STIFF O.D.E.'S, WITH
C MONITORING OF LOCAL TRUNCATION ERROR TO ADJUST STEPSIZE.
C SPECIAL VERSION FOR ISOTOPIC DEPLETION CALCULATIONS.
C
C INPUT AND OUTPUT PARAMETERS:
C  Y       : DEPENDENT VARIABLE VECTOR. DIMENSION Y(N)
C  N       : DIMENSION OF THE DEPENDENT VARIABLE VECTOR.
C  X       : INDEPENDENT VARIABLE.
C  HTRY    : STEPSIZE TO BE ATTEMPTED.
C  EPS     : REQUIRED ACCURACY.
C  YSCAL   : VECTOR AGAINST WHICH THE ERROR IS SCALED.
C            DIMENSION YSCAL(N)
C  HDID    : STEPSIZE THAT WAS ACTUALLY ACCOMPLISHED.
C  HNEXT   : ESTIMATED NEXT STEPSIZE.
C
C INPUT DATA RELATED TO THE ISOTOPIC DEPLETION CALCULATION:
C  MU1     : POSITION OF EACH DIAGONAL ELEMENT IN VECTOR ADPL.
C  IMA     : POSITION OF THE FIRST NON-ZERO COLUMN ELEMENT IN VECTOR
C            ADPL. DIMENSION MU1(N),IMA(N)
C  MAXA    : FIRST DIMENSION OF ADPL.
C  NSUPF   : NUMBER OF DEPLETING FISSION PRODUCTS.
C  NFISS   : NUMBER OF FISSILE ISOTOPES PRODUCING FISSION PRODUCTS.
C  KFISS   : POSITION IN CHAIN OF THE FISSILE ISOTOPES.
C            DIMENSION KFISS(NFISS)
C  YSF     : COMPONENTS OF THE PRODUCT OF THE FISSION YIELDS AND FISSION
C            RATES. DIMENSION YSF(NFISS,NSUPF,2)
C  ADPL    : DEPLETION MATRIX COMPONENTS. DIMENSION ADPL(MAXA,2)
C  BDPL    : DEPLETION SOURCE COMPONENTS. DIMENSION BDPL(N,2)
C
C
C SCRATCH STORAGE:
C  DYDX    : DIMENSION DYDX(N)
C  TEMP    : DIMENSION TEMP(N)
C  YSAV    : DIMENSION YSAV(N)
C  DYSAV   : DIMENSION DYSAV(N)
C  TEMR    : DIMENSION TEMR(N)
C  AK      : DIMENSION AK(N,3)
C
C REFERENCE:
C  W.H. PRESS, B.P. FLANNERY, S.A. TEUKOLSKY AND W.T. VETTERLING,
C  'NUMERICAL RECIPIES (FORTRAN VERSION)', CAMBRIDGE UNIVERSITY PRESS,
C  CHAPTER 15, CAMBRIDGE (1990).
C
C-----------------------------------------------------------------------
C
      IMPLICIT NONE
C----
C PARAMETERS
C----
      REAL       FCOR,SAFETY,ERRCON
      PARAMETER (FCOR=.0666666667,SAFETY=0.9,ERRCON=6.E-4)
C----
C INTERFACE VARIABLES
C----
      INTEGER   NFISS,NSUPF,N,MAXA
      INTEGER   MU1(*),IMA(*),KFISS(*)
      REAL      Y(*),YSCAL(*),YSF(NFISS,NSUPF,*),
     1          ADPL(MAXA,*),BDPL(N,*),DYDX(*),TEMP(*),
     2          YSAV(*),DYSAV(*),TEMR(*),AK(N,*)
C----
C LOCAL VARIABLES
C----
      INTEGER   NSUPL,I,J
      REAL      PGROW,PSHRNK,XSAV,X,H,HTRY,HH,ERRMAX,EPS,HDID,
     1          HNEXT
C
C----
C
      NSUPL=N-NSUPF
      PGROW=-0.20
      PSHRNK=-0.25
      XSAV=X
      CALL ALLUM(N,ADPL(1,1),Y,DYDX,MU1,IMA)
      CALL ALLUM(N,ADPL(1,2),Y,TEMR,MU1,IMA)
      DO 10 I=1,NSUPF
      DO 10 J=1,NFISS
      DYDX(NSUPL+I)=DYDX(NSUPL+I)+YSF(J,I,1)*Y(KFISS(J))
   10 TEMR(NSUPL+I)=TEMR(NSUPL+I)+YSF(J,I,2)*Y(KFISS(J))
      DO 20 I=1,N
      DYDX(I)=DYDX(I)+X*TEMR(I)+BDPL(I,1)+X*BDPL(I,2)
      YSAV(I)=Y(I)
   20 DYSAV(I)=DYDX(I)
      H=HTRY
   30 HH=0.5*H
      CALL EVORK4(YSAV,DYSAV,N,XSAV,HH,TEMP,MU1,IMA,MAXA,NSUPF,NFISS,
     1 KFISS,YSF,ADPL,BDPL,AK,TEMR)
      X=XSAV+HH
      CALL ALLUM(N,ADPL(1,1),TEMP,DYDX,MU1,IMA)
      CALL ALLUM(N,ADPL(1,2),TEMP,TEMR,MU1,IMA)
      DO 40 I=1,NSUPF
      DO 40 J=1,NFISS
      DYDX(NSUPL+I)=DYDX(NSUPL+I)+YSF(J,I,1)*TEMP(KFISS(J))
   40 TEMR(NSUPL+I)=TEMR(NSUPL+I)+YSF(J,I,2)*TEMP(KFISS(J))
      DO 50 I=1,N
   50 DYDX(I)=DYDX(I)+X*TEMR(I)+BDPL(I,1)+X*BDPL(I,2)
      CALL EVORK4(TEMP,DYDX,N,X,HH,Y,MU1,IMA,MAXA,NSUPF,NFISS,KFISS,YSF,
     1 ADPL,BDPL,AK,TEMR)
      X=XSAV+H
      IF(X.EQ.XSAV) CALL XABORT('EVORK: STEPSIZE NOT SIGNIFICANT.')
      CALL EVORK4(YSAV,DYSAV,N,XSAV,H,TEMP,MU1,IMA,MAXA,NSUPF,NFISS,
     1 KFISS,YSF,ADPL,BDPL,AK,TEMR)
      ERRMAX=0.
      DO 60 I=1,N
      TEMP(I)=Y(I)-TEMP(I)
   60 ERRMAX=MAX(ERRMAX,ABS(TEMP(I)/YSCAL(I)))
      ERRMAX=ERRMAX/EPS
      IF(ERRMAX.GT.1.0) THEN
        H=SAFETY*H*(ERRMAX**PSHRNK)
        GOTO 30
      ELSE
        HDID=H
        IF(ERRMAX.GT.ERRCON)THEN
          HNEXT=SAFETY*H*(ERRMAX**PGROW)
        ELSE
          HNEXT=4.*H
        ENDIF
      ENDIF
      DO 70 I=1,N
   70 Y(I)=Y(I)+TEMP(I)*FCOR
      RETURN
      END
