*DECK EVOSAT
      SUBROUTINE EVOSAT(IMPX,LOGY,NSAT,NVAR,KSAT,YST1,YSAT,MU1,IMA,
     1 MAXA,NSUPF,NFISS,KFISS,YSF,ADPL,BDPL,KEV,YSTG,A22,A21,A12,MGAR,
     2 IGAR,AGAR,BGAR,YSFG,GAR)
C
C-----------------------------------------------------------------------
C
C LUMPING OF THE DEPLETION MATRIX, FISSION YIELDS, SOURCES AND INITIAL
C CONDITIONS TO TAKE INTO ACCOUNT THE SATURATION OF DEPLETING NUCLIDES.
C
C INPUT PARAMETERS:
C  IMPX    : PRINT PARAMETER.
C  LOGY    : =1: UPDATE YSAT AND YST1.
C            =2: DO NOT UPDATE YSAT AND YST1.
C  NSAT    : NUMBER OF SATURATING NUCLIDES.
C  NVAR    : NUMBER OF NUCLIDES IN THE COMPLETE DEPLETION CHAIN.
C  KSAT    : POSITION IN CHAIN OF THE SATURATING NUCLIDES.
C            DIMENSION KSAT(NSAT)
C  YST1    : NUMBER DENSITIES FOR ALL ISOTOPES. DIMENSION YST1(NVAR)
C  MAXA    : DIMENSION OF VECTORS ADPL AND AGAR.
C  NFISS   : NUMBER OF FISSILE ISOTOPES PRODUCING FISSION PRODUCTS.
C
C INPUT AND OUTPUT PARAMETERS:
C  MU1     : POSITION OF EACH DIAGONAL ELEMENT IN VECTOR ADPL.
C  IMA     : POSITION OF THE FIRST NON-ZERO COLUMN ELEMENT IN VECTOR
C            ADPL. DIMENSION MU1(NVAR),IMA(NVAR)
C  NSUPF   : NUMBER OF DEPLETING FISSION PRODUCTS.
C  KFISS   : POSITION IN CHAIN OF THE FISSILE ISOTOPES.
C            DIMENSION KFISS(NFISS)
C  YSF     : PRODUCT OF THE FISSION YIELDS AND FISSION RATES.
C            DIMENSION YSF(NFISS,NSUPF)
C  ADPL    : DEPLETION MATRIX. DIMENSION ADPL(MAXA)
C  BDPL    : DEPLETION SOURCE. DIMENSION BDPL(NVAR)
C
C OUTPUT PARAMETERS:
C  YST1    : NUMBER DENSITIES OF THE NON-SATURATED ISOTOPES.
C            DIMENSION YST1(NVAR-NSAT)
C  YSAT    : NUMBER DENSITIES OF THE SATURATING ISOTOPES.
C            DIMENSION YSAT(NSAT)
C
C SCRATCH STORAGE:
C  KEV     : DIMENSION KEV(NVAR)
C  YSTG    : DIMENSION YSTG(NVAR-NSAT)
C  A22     : DIMENSION A22(NSAT,NSAT)
C  A21     : DIMENSION A21(NSAT,NVAR-NSAT)
C  A12     : DIMENSION A12(NVAR-NSAT,NSAT)
C  MGAR    : DIMENSION MGAR(NVAR-NSAT)
C  IGAR    : DIMENSION IGAR(NVAR-NSAT)
C  AGAR    : DIMENSION AGAR(MAXA)
C  BGAR    : DIMENSION BGAR(NVAR-NSAT)
C  YSFG    : DIMENSION YSFG(NFISS,NSUPF)
C  GAR     : DIMENSION GAR(NSAT)
C
C--------------------------------------- AUTHOR: A. HEBERT 21/06/90 ----
C
      IMPLICIT NONE
C----
C PARAMETERS
C----
      INTEGER   IOUT
      REAL      EPS
      PARAMETER(IOUT=6,EPS=1.0E-5)
C----
C INTERFACE VARIABLES
C----
      INTEGER   IMPX,LOGY,NSAT,NVAR,NFISS,NSUPF
      INTEGER   KSAT(*),MU1(*),IMA(*),KFISS(*),KEV(*),MGAR(*),IGAR(*)
      REAL      YST1(*),YSAT(*),YSTG(*),YSF(NFISS,*),ADPL(*),
     1          BDPL(*),A22(NSAT,*),A21(NSAT,*),A12(NVAR-NSAT,*),
     2          AGAR(*),BGAR(*),YSFG(NFISS,*),GAR(*)
C----
C LOCAL VARIABLES
C----
      INTEGER   NSUPL,NSUPLG,JMN0,IMN0,I0,IMAM1,I,II,JMN,J,J0,IROW,ICOL,
     1          JJ,IMN,MAXA,JMAM1,K,IER,ITER
      REAL      ERR1,ERR2,ZCOMP,TEST
      CHARACTER HSMG*131
*----
*  DETERMINE THE PROFILE PATTERN OF THE LUMPED DEPLETION MATRIX.
*----
      NSUPL=NVAR-NSUPF
      I0=0
      DO 40 I=1,NVAR
      DO 10 II=1,NSAT
      IF(I.EQ.KSAT(II)) GO TO 20
   10 CONTINUE
      I0=I0+1
      KEV(I)=I0
      GO TO 40
   20 KEV(I)=0
      DO 30 II=1,NFISS
      IF(I.EQ.KFISS(II)) CALL XABORT('EVOSAT: A FISSILE ISOTOPE IS SAT'
     1 //'URATING.')
   30 CONTINUE
   40 CONTINUE
      DO 50 I=1,NFISS
   50 KFISS(I)=KEV(KFISS(I))
*----
*  COMPUTE MATRICES A22**-1, A21, AND A12.
*----
      DO 180 II=1,NSAT
      I=KSAT(II)
      IMAM1=0
      IF(I.GT.1) IMAM1=IMA(I-1)
      DO 150 JJ=1,NSAT
      J=KSAT(JJ)
      IF((J.LE.I).AND.(J.GT.I+IMAM1-MU1(I))) THEN
         A22(II,JJ)=ADPL(MU1(I)-I+J)
      ELSE IF((I.LE.J).AND.(I.GE.J-IMA(J)+MU1(J))) THEN
         A22(II,JJ)=ADPL(MU1(J)+J-I)
      ELSE
         A22(II,JJ)=0.0
      ENDIF
  150 CONTINUE
      JMAM1=0
      DO 160 J=1,NVAR
      J0=KEV(J)
      IF(J0.EQ.0) GO TO 160
      IF((J.LE.I).AND.(J.GT.I+IMAM1-MU1(I))) THEN
         A21(II,J0)=ADPL(MU1(I)-I+J)
      ELSE IF((I.LE.J).AND.(I.GE.J-IMA(J)+MU1(J))) THEN
         A21(II,J0)=ADPL(MU1(J)+J-I)
      ELSE
         A21(II,J0)=0.0
      ENDIF
      IF((I.LE.J).AND.(I.GT.J+JMAM1-MU1(J))) THEN
         A12(J0,II)=ADPL(MU1(J)-J+I)
      ELSE IF((J.LE.I).AND.(J.GE.I-IMA(I)+MU1(I))) THEN
         A12(J0,II)=ADPL(MU1(I)+I-J)
      ELSE
         A12(J0,II)=0.0
      ENDIF
  160 JMAM1=IMA(J)
      IF(I.GT.NSUPL) THEN
         DO 170 K=1,NFISS
  170    A21(II,KFISS(K))=A21(II,KFISS(K))+YSF(K,I-NSUPL)
      ENDIF
  180 CONTINUE
      CALL ALINV(NSAT,A22,NSAT,IER,GAR)
      IF(IER.NE.0) CALL XABORT('EVOSAT: SINGULAR MATRIX.')
*----
*  COMPUTE VECTOR YSTG ANT YSAT.
*----
      IF(LOGY.EQ.1) THEN
*        BEGINNING-OF-STAGE DIRAC DELTA CONTRIBUTIONS:
         DO 190 I=1,NSAT
  190    YSAT(I)=YST1(KSAT(I))
         DO 200 I=1,NVAR
         IF(KEV(I).GT.0) YSTG(KEV(I))=YST1(I)
  200    CONTINUE
         ITER=0
  230    ITER=ITER+1
         IF(ITER.GT.50) CALL XABORT('EVOSAT: CONVERGENCE FAILURE.')
         DO 240 I=1,NSAT
         GAR(I)=BDPL(KSAT(I))
         DO 240 J=1,NVAR-NSAT
  240    GAR(I)=GAR(I)+A21(I,J)*YSTG(J)
         ERR1=0.0
         ERR2=0.0
         DO 260 I=1,NSAT
         ZCOMP=YSAT(I)
         YSAT(I)=0.0
         DO 250 J=1,NSAT
  250    YSAT(I)=YSAT(I)-A22(I,J)*GAR(J)
         ERR1=MAX(ERR1,ABS(ZCOMP-YSAT(I)))
  260    ERR2=MAX(ERR2,ABS(YSAT(I)))
         DO 270 I=1,NSAT
         GAR(I)=0.0
         DO 270 J=1,NSAT
  270    GAR(I)=GAR(I)-A22(I,J)*(YST1(KSAT(J))-YSAT(J))
         DO 280 I=1,NVAR
         IF(KEV(I).GT.0) YSTG(KEV(I))=YST1(I)
  280    CONTINUE
         DO 300 I=1,NVAR-NSAT
         DO 290 J=1,NSAT
  290    YSTG(I)=YSTG(I)+A12(I,J)*GAR(J)
  300    ERR2=MAX(ERR2,ABS(YSTG(I)))
         IF(ERR1.LE.EPS*ERR2) GO TO 350
         GO TO 230
      ENDIF
*----
*  COMPUTE MATRICES AGAR AND BGAR.
*----
  350 DO 360 I=1,NSAT
      GAR(I)=0.0
      DO 360 J=1,NSAT
  360 GAR(I)=GAR(I)-A22(I,J)*BDPL(KSAT(J))
      DO 370 I=1,NVAR
      IF(KEV(I).GT.0) BGAR(KEV(I))=BDPL(I)
  370 CONTINUE
      DO 380 I=1,NVAR-NSAT
      DO 380 J=1,NSAT
  380 BGAR(I)=BGAR(I)+A12(I,J)*GAR(J)
      DO 400 J=1,NVAR-NSAT
      DO 390 K=1,NSAT
  390 GAR(K)=A21(K,J)
      DO 400 I=1,NSAT
      A21(I,J)=0.0
      DO 400 K=1,NSAT
  400 A21(I,J)=A21(I,J)+A22(I,K)*GAR(K)
*----
*  DETERMINE THE PROFILE PATTERN OF THE LUMPED DEPLETION MATRIX.
*----
      NSUPLG=NSUPL
      JMN0=NVAR
      IMN0=NVAR
      IMAM1=0
      DO 500 I=1,NVAR
        IF(KEV(I).EQ.0) THEN
          IF(I.LE.NSUPL) NSUPLG=NSUPLG-1
          JMN0=MIN(JMN0,I-MU1(I)+IMAM1+1)
          IMN0=MIN(IMN0,I-IMA(I)+MU1(I))
        ENDIF
        IMAM1=IMA(I)
  500 CONTINUE
      NSUPF=NVAR-NSAT-NSUPLG
      IMAM1=0
      DO 510 I=1,NVAR
        I0=KEV(I)
        JMN=I-MU1(I)+IMAM1+1
        IMN=I-IMA(I)+MU1(I)
        IF(I0.EQ.0) THEN
          DO 511 J=JMN,I-1
          J0=KEV(J)
          IF(J0.GT.NVAR-NSAT) 
     >    CALL XABORT('EVOSAT: ALGORITHM FAILURE 2.')
          IF(J0.GT.0) MGAR(J0)=MIN(MGAR(J0),JMN0)
  511     CONTINUE
          DO 512 J=IMN,I-1
            J0=KEV(J)
            IF(J0.GT.0) IGAR(J0)=MIN(IGAR(J0),IMN0)
  512     CONTINUE
        ELSE
          IF(I0.GT.NVAR-NSAT) 
     >    CALL XABORT('EVOSAT: ALGORITHM FAILURE 1.')
          MGAR(I0)=JMN
          IGAR(I0)=IMN
          DO 513 J=JMN,I-1
            IF(KEV(J).EQ.0) MGAR(I0)=MIN(MGAR(I0),JMN0)
  513     CONTINUE
          DO 514 J=IMN,I-1
            IF(KEV(J).EQ.0) IGAR(I0)=MIN(IGAR(I0),IMN0)
  514     CONTINUE
        ENDIF
        IMAM1=IMA(I)
  510 CONTINUE
      II=0
      DO 520 I=1,NVAR-NSAT
  525   CONTINUE 
        IF(KEV(MGAR(I)).EQ.0) THEN
           MGAR(I)=MGAR(I)+1
           IF(MGAR(I).GT.NVAR) 
     >     CALL XABORT('EVOSAT: ALGORITHM FAILURE 3.')
           GO TO 525
        ENDIF
  526   CONTINUE
        IF(KEV(IGAR(I)).EQ.0) THEN
          IGAR(I)=IGAR(I)+1
          IF(IGAR(I).GT.NVAR) 
     >    CALL XABORT('EVOSAT: ALGORITHM FAILURE 4.')
          GO TO 526
        ENDIF
  520 CONTINUE
      DO 530 I=1,NVAR-NSAT
      DO 530 J=1,NVAR-NSAT
        DO 531 K=1,NFISS
          IF((I.GT.NSUPL).AND.(J.EQ.KFISS(K))) GO TO 530
  531   CONTINUE
        TEST=0.0
        DO 532 K=1,NSAT
          TEST=TEST+A12(I,K)*A21(K,J)
  532   CONTINUE
        IF((J.LT.I).AND.(TEST.NE.0.0)) THEN
           MGAR(I)=MIN(MGAR(I),J)
        ELSE IF((I.LT.J).AND.(TEST.NE.0.0)) THEN
           IGAR(J)=MIN(IGAR(J),I)
        ENDIF
  530 CONTINUE
      II=0
      DO 540 I=1,NVAR-NSAT
        IROW=I-KEV(MGAR(I))+1
        ICOL=I-KEV(IGAR(I))+1
        II=II+IROW
        MGAR(I)=II
        II=II+ICOL-1
        IGAR(I)=II
  540 CONTINUE
      IF(IMPX.GT.5) WRITE(IOUT,6000) IGAR(NVAR-NSAT),MAXA
      IF(IGAR(NVAR-NSAT).GT.MAXA) THEN
         WRITE(HSMG,'(24HEVOSAT: IGAR(NVAR-NSAT)=,I6,6H MAXA=,I6)')
     1   IGAR(NVAR-NSAT),MAXA
         CALL XABORT(HSMG)
      ENDIF
*----
*  COMPUTE MATRIX AGAR AND YIELDS YSFG.
*----
      DO 410 I=1,IGAR(NVAR-NSAT)
  410 AGAR(I)=0.0
      IMAM1=0
      DO 440 I=1,NVAR
      IF(KEV(I).EQ.0) GO TO 440
      DO 420 J=1,NVAR
      IF(KEV(J).EQ.0) GO TO 420
      IF((J.LE.I).AND.(J.GT.I+IMAM1-MU1(I))) THEN
         AGAR(MGAR(KEV(I))-KEV(I)+KEV(J))=ADPL(MU1(I)-I+J)
      ELSE IF((I.LE.J).AND.(I.GE.J-IMA(J)+MU1(J))) THEN
         AGAR(MGAR(KEV(J))+KEV(J)-KEV(I))=ADPL(MU1(J)+J-I)
      ENDIF
  420 CONTINUE
      IF(I.GT.NSUPL) THEN
         DO 430 K=1,NFISS
  430    YSFG(K,KEV(I)-NSUPLG)=YSF(K,I-NSUPL)
      ENDIF
  440 IMAM1=IMA(I)
      IMAM1=0
      DO 490 I=1,NVAR-NSAT
      DO 470 J=1,NVAR-NSAT
      IF((J.LE.I).AND.(J.GT.I+IMAM1-MGAR(I))) THEN
         DO 450 K=1,NSAT
  450    AGAR(MGAR(I)-I+J)=AGAR(MGAR(I)-I+J)-A12(I,K)*A21(K,J)
      ELSE IF((I.LE.J).AND.(I.GE.J-IGAR(J)+MGAR(J))) THEN
         DO 460 K=1,NSAT
  460    AGAR(MGAR(J)+J-I)=AGAR(MGAR(J)+J-I)-A12(I,K)*A21(K,J)
      ENDIF
  470 CONTINUE
      IF(I.GT.NSUPLG) THEN
         DO 480 J=1,NFISS
         DO 480 K=1,NSAT
  480    YSFG(J,I-NSUPLG)=YSFG(J,I-NSUPLG)-A12(I,K)*A21(K,KFISS(J))
      ENDIF
  490 IMAM1=IGAR(I)
*----
*  REPLACE THE ORIGINAL INFORMATION WITH THE LUMPED ONE.
*----
      DO 600 I=1,NVAR-NSAT
        IF(LOGY.NE.2) YST1(I)=YSTG(I)
        IMA(I)=IGAR(I)
        MU1(I)=MGAR(I)
        BDPL(I)=BGAR(I)
  600 CONTINUE
      DO 610 I=1,IMA(NVAR-NSAT)
        ADPL(I)=AGAR(I)
  610 CONTINUE
      DO 620 I=1,NFISS
        DO 621 J=1,NSUPF
          YSF(I,J)=YSFG(I,J)
  621   CONTINUE
  620 CONTINUE
      RETURN
 6000 FORMAT(/' EVOSAT: REAL SIZE OF ADPL=',I9,'  ALLOCATED SIZE=',I9)
      END
