*DECK FLUACV
      SUBROUTINE FLUACV(SMODUL,IPSYS ,IPMACR,IPTRK ,IPBTF ,IPRINT,
     >                  ITPIJ ,KDIR  ,NGROUP,NBMIX ,NREGIO,NUNKNO,
     >                  ILEAK ,MATCOD,VOLUME,KEYFLX,FUNKNO,SUNKNO,
     >                  ISADJ ,IGDEB ,ILASTG,DB2   ,SIGT0 ,SIGS0 ,
     >                  XSCAT ,IJJ   ,NJJ   ,IPOS  ,WOVER ,DELFL ,
     >                  GAMMA ,RESTE )
C
C---------------------------  FLUACV  ---------------------------------
C
C   1- PROGRAMME STATISTICS:
C       NAME     : FLUACV
C       USE      : COMPUTE OVER-RELAXATION FACTOR USING VARIATIONAL
C                  PRINCIPLE AND RETURN NEW FLUX APPROXIMATION
C       MODIFIED : DEAL WITH ADJOINT FLUX
C                  2000-01-20 (T. COURAU)
C                  91-03-15 (G. MARLEAU)
C       AUTHOR   : G.MARLEAU
C
C   2- ROUTINE PARAMETERS:
C    INPUT:
C      SMODUL  : RESIDUAL EVALUATION MODULE.          EXT ROUTINE
C      IPSYS   : POINTER TO THE PIJ                   I
C      IPMACR  : POINTER TO THE MACROLIB              I
C      IPTRK   : POINTER TO THE TRACKING              I
C      IPBTF   : POINTER TO THE BINARY TRACKING FILE  I
C      IPRINT  : PRINT SELECTION FOR FLUX MODULES:    I
C               = 0/1/2/3 NO PRINT/SHORT PRINT/LONG
C      ITPIJ   : TYPE OF CP AVAILABLE                 I
C                = 1 SCATT MOD PIJ (WIJ)
C                = 2 STAND. PIJ
C                = 3 SCATT MOD PIJ+PIJK (WIJ,WIJK)
C                = 4 STAND. PIJ+PIJK
C      KDIR    : DIRECTION FOR PC                     I
C                = 0 FOR PIJ OR WIJ
C                = K FOR PIJK OR WIJK K=1,2,3 B1
C                = K+3 FOR PIJK OR WIJK K=4,5,6 B0
C      NGROUP  : NUMBER OF GROUPS.                    I
C      NBMIX   : NUMBER OF MIXTURES.                  I
C      NREGIO  : NUMBER OF REGIONS.                   I
C      NUNKNO  : NUMBER OF UNKNOWN IN THE SYSTEM.     I
C      ILEAK   : METHOD USED TO INCLUDE DB2 EFFECT    I
C                = 0 NO DB2 EFFECT
C                = 1 THE SCATTERING MODIFIED CP MATRIX
C                    IS MULTIPLIED BY A PNL FACTOR
C                = 2 THE REDUCED CP MATRIX IS MULTIPLIED BY
C                    PNL FACTOR
C                = 3 SIGS0-DB2 APPROXIMATION
C                = 4 ALBEDO APPROXIMATION
C                = 5,6,7,8,9,10 HETEROGENEOUS WITH PIJK
C                  AND FIXED B->5 OR SEARCH
C                  X->6 Y->7 Z->8C R->9 ALL->10
C      MATCOD  : MATERIAL CODE IN REGIONS.            I(NREGIO)
C      VOLUME  : VOLUME OF REGIONS. DIMENSION         R(NREGIO)
C      KEYFLX  : FLUX ELEMENTS IN UNKNOWN SYSTEM.     I(NREGIO)
C      FUNKNO  : UNKNOWN VECTOR SOLVED FOR.           R(NUNKNO,NGROUP)
C      SUNKNO  : SOURCE FOR SYSTEM OF UNKNOWN.        R(NUNKNO,NGROUP)
C      ISADJ   : ADJOINT CALCULATION AND SAVE OPTION  I
C                = 0 ONLY DIRECT FLUX
C                = 1 DIRECT AND PSEUDO ADJOINT FLUX
C                = 2 DIRECT, PSEUDO-ADJOINT AND ADJOINT
C                = 3 DIRECT, PSEUDO ADJOINT AND
C                    GENERALIZED PSEUDO-ADJOINT
C                = 4 DIRECT, PSEUDO ADJOINT, ADJOINT,
C                    GENERALIZED PSEUDO-ADJOINT AND
C                    GENERALIZED ADJOINT
C      IGDEB   : FIRST GROUP TO PROCESS               I
C      ILASTG  : LAST GROUP TO PROCESS                I
C      DB2     : LEAKAGE PARAMETERS.                  R(NGROUP)
C                FOR ITPIJ=1,2 -> DB2(G)=D(G)*B**2
C                FOR ITPIJ=3,4 -> DB2(G)=LAMBDA(B/SIGMA(G))
C      SIGT0   : TOTAL CROSS SECTION VECTOR           R(0:NBMIX)
C                SIGT0(0)=0.0 IS FOR VOID
C      SIGS0   : WITHIN-GROUP SCATTERING              R(0:NBMIX)
C                SIGS0(0)=0.0 IS FOR VOID
C      XSCAT   : SCATTERING VECTOR                    R(0:NBMIX*NGROUP)
C                XSCAT(0) = 0.0
C      IJJ     : LAST SCATTERING GROUP                I(0:NBMIX)
C                IJJ(0) = 0
C      NJJ     : NUMBER OF SCATTERING GROUP           I(0:NBMIX)
C                NJJ(0) = -NGROUP
C      IPOS    : POSITION SELF SCATTERING IN XSCAT    I(0:NBMIX)
C                IPOS(0)= NGROUP+1
C      WOVER   : VARIATIONAL ACCELERATION FACTOR      R
C                VARIATIONAL ACCELERATION
C      DELFL   : INITIAL FLUXES FOR ITERATION         I(NUNKNO,NGROUP)
C      GAMMA   : ONE GROUP UNKNOWN VECTOR             R(NUNKNO)
C      RESTE   : RESIDUAL VECTORS                     R(NUNKNO,2)
C
C---------------------------   FLUACV  --------------------------------
C
      IMPLICIT    NONE
      INTEGER     IOUT,ILCMUP,ILCMDN
      CHARACTER   NAMSBR*6
      PARAMETER  (IOUT=6,ILCMUP=1,ILCMDN=2,NAMSBR='FLUACV')
C----
C  ROUTINE PARAMETERS
C----
      EXTERNAL    SMODUL
      INTEGER     IPSYS,IPMACR,IPTRK,IPBTF,IPRINT,ITPIJ,KDIR,
     >            NGROUP,NBMIX,NREGIO,NUNKNO,ILEAK,ISADJ,
     >            IGDEB,ILASTG
      INTEGER     MATCOD(NREGIO),KEYFLX(NREGIO),
     >            IJJ(0:NBMIX),NJJ(0:NBMIX),IPOS(0:NBMIX)
      REAL        VOLUME(NREGIO),FUNKNO(NUNKNO,NGROUP),
     >            SUNKNO(NUNKNO,NGROUP),DB2(NGROUP),
     >            SIGT0(0:NBMIX),SIGS0(0:NBMIX),XSCAT(0:NBMIX*NGROUP),
     >            WOVER,DELFL(NUNKNO,NGROUP),GAMMA(NUNKNO),
     >            RESTE(NUNKNO,2)
C----
C  LOCAL PARAMETERS
C----
      CHARACTER   CGRPNM*12,CAN*3,CNSIGW*12,TITLE*72
      INTEGER     LONS0,IDIR,KRSK,IUNKNO,ISTART,ISTOP,IGR,JGR,
     >            ILCMLN,ILCMTY,IREG,IR,IFPOS,IBM,IPRNTT,
     >            NGSCAT,IFSCAT,ILSCAT,ISCATP
      DOUBLE PRECISION WNUMER,WDENOM,ZNUM,ZDEN,ALP1,ALP2
C----
C  FOR DIRECTIONS KDIR=0 USE ISOTROPIC SCATTERING
C  FOR DIRECTIONS KDIR=1,2,3 USE ANISOTROPIC SCATTERING B1
C  FOR DIRECTIONS KDIR=4,5,6 USE ANISOTROPIC SCATTERING B0
C  KRSK = LOCATION OF FIRST ELEMENT FOR VARIOUS DIRECTION
C----
      TITLE=' '
      IPRNTT=0
      IF(ISADJ .EQ. 0) THEN
        CAN='D 0'
      ELSE
        CAN='A 0'
      ENDIF
      IF(KDIR .EQ. 0) THEN
        CNSIGW='DRAGON-S0XSC'
        LONS0=NBMIX+1
        IDIR=0
      ELSE IF(KDIR .GT. 3) THEN
        CAN(3:3)='1'
        CNSIGW='SIGW 1'
        LONS0=NBMIX
        IDIR=KDIR-3
      ELSE
        CAN(3:3)='1'
        CNSIGW='SIGW 1'
        LONS0=NBMIX
        IDIR=KDIR
      ENDIF
      KRSK=IDIR*NREGIO
      DO 300 IUNKNO=1,NUNKNO
        RESTE(IUNKNO,1)=0.0
        RESTE(IUNKNO,2)=0.0
 300  CONTINUE
C----
C  STORE DELTA(N) IN RESTE(NUNKNO,1)
C  COMPUTE SOURCES PROPORTIONAL TO DELTA(N)
C  CALL SMODUL TO RETURN DELTA(N)-SOLUTION WITH SOURCES IN DELTA(N)
C----
      WDENOM=0.0D0
      WNUMER=0.0D0
      ISTART=MIN(IGDEB,ILASTG)
      ISTOP=MAX(IGDEB,ILASTG)
      DO 100 IGR=ISTART,ISTOP
C----
C  READ SCATT X-SECTIONS.
C----
        WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGROUP
        CALL LCMSIX(IPMACR,CGRPNM,1)
        CALL LCMSIX(IPSYS,CGRPNM,1)
        IF(KDIR .LE. 3) THEN
          CALL LCMLEN(IPMACR,'NJJ'//CAN,ILCMLN,ILCMTY)
          IF(ILCMLN .NE. NBMIX) THEN
            CALL LCMLIB(IPMACR)
            CALL XABORT(NAMSBR//
     >      ': READ ERROR ON LCM RECORD = NJJ'//CAN)
          ELSE
            CALL LCMGET(IPMACR,'NJJ'//CAN,NJJ(1))
            CALL LCMGET(IPMACR,'IJJ'//CAN,IJJ(1))
            CALL LCMGET(IPMACR,'IPO'//CAN,IPOS(1))
            CALL LCMGET(IPMACR,'SCA'//CAN,XSCAT(1))
          ENDIF
        ENDIF
C----
C  READ TOTAL AND WITHIN GROUP XS IF REQUIRED FOR LEAKAGE CALCULATIONS
C----
        IF(CNSIGW .EQ. 'DRAGON-S0XSC') THEN
          CALL LCMLEN(IPSYS,CNSIGW,ILCMLN,ILCMTY)
          IF(ILCMLN .EQ. LONS0) THEN
            CALL LCMGET(IPSYS,CNSIGW,SIGS0(0))
          ELSE
            CALL LCMLIB(IPSYS)
            CALL XABORT(NAMSBR//
     >      ': READ ERROR ON LCM RECORD = '//CNSIGW)
          ENDIF
        ELSE
          CALL LCMLEN(IPMACR,CNSIGW,ILCMLN,ILCMTY)
          IF(ILCMLN .EQ. LONS0) THEN
            CALL LCMGET(IPMACR,CNSIGW,SIGS0(1))
          ELSE
            CALL LCMLIB(IPMACR)
            CALL XABORT(NAMSBR//
     >      ': READ ERROR ON LCM RECORD = '//CNSIGW)
          ENDIF
          SIGS0(0)=0.0
        ENDIF
        IF(ILEAK .LT. 5 .AND. ILEAK .GT. 0 .AND.
     >     DB2(IGR) .NE. 0.0) THEN
          IF(ILEAK .EQ. 1 .OR. ILEAK .EQ. 2 )THEN
            CALL LCMLEN(IPSYS,'DRAGON-TXSC',ILCMLN,ILCMTY)
            IF(ILCMLN .EQ. NBMIX+1) THEN
              CALL LCMGET(IPSYS,'DRAGON-TXSC',SIGT0)
            ELSE
              CALL LCMLIB(IPSYS)
              CALL XABORT(NAMSBR//
     >        ': READ ERROR ON LCM RECORD=DRAGON-TXSC')
            ENDIF
            IF(ILEAK .EQ. 1) THEN
              ZNUM=0.0D0
              ZDEN=0.0D0
              DO 10 IREG=1,NREGIO
                IFPOS=KRSK+KEYFLX(IREG)
                IR=MATCOD(IREG)
                ZNUM=ZNUM+DBLE((SIGT0(IR)-SIGS0(IR))
     >                   *FUNKNO(IFPOS,IGR)*VOLUME(IREG))
                ZDEN=ZDEN+DBLE(FUNKNO(IFPOS,IGR)*VOLUME(IREG))
 10           CONTINUE
              ALP1=ZNUM/(ZNUM+DBLE(DB2(IGR))*ZDEN)
              DO 15 IREG=1,NREGIO
                IFPOS=KRSK+KEYFLX(IREG)
                RESTE(IFPOS,1)=REAL(ALP1)*SUNKNO(IFPOS,IGR)
                RESTE(IFPOS,2)=0.0
 15           CONTINUE
            ELSE
              ZNUM=0.0D0
              ZDEN=0.0D0
              DO 20 IREG=1,NREGIO
                IFPOS=KRSK+KEYFLX(IREG)
                IR=MATCOD(IREG)
                ZNUM=ZNUM+DBLE(SIGT0(IR)*FUNKNO(IFPOS,IGR)*VOLUME(IREG))
                ZDEN=ZDEN+DBLE(FUNKNO(IFPOS,IGR)*VOLUME(IREG))
 20           CONTINUE
              ALP1=ZNUM/(ZNUM+DBLE(DB2(IGR))*ZDEN)
              ALP2=DBLE(DB2(IGR))*ZDEN/(ZNUM+DBLE(DB2(IGR))*ZDEN)
              DO 25 IREG=1,NREGIO
                IFPOS=KRSK+KEYFLX(IREG)
                IR=MATCOD(IREG)
                RESTE(IFPOS,1)=REAL(ALP1)*SUNKNO(IFPOS,IGR)
     >            +REAL(ALP2)*SIGS0(IR)*FUNKNO(IFPOS,IGR)
                RESTE(IFPOS,2)=REAL(ALP2)*SIGS0(IR)
     >            *(FUNKNO(IFPOS,IGR)-DELFL(IFPOS,IGR))
 25           CONTINUE
            ENDIF
          ELSE IF(ILEAK .EQ. 3)THEN
            ALP1=1.0D0
            DO 35 IREG=1,NREGIO
              IFPOS=KRSK+KEYFLX(IREG)
              RESTE(IFPOS,1)=DB2(IGR)*FUNKNO(IFPOS,IGR)
     >                      +SUNKNO(IFPOS,IGR)
              RESTE(IFPOS,2)=DB2(IGR)*(FUNKNO(IFPOS,IGR)
     >                      -DELFL(IFPOS,IGR))
 35         CONTINUE
          ELSE IF(ILEAK .EQ. 4)THEN
            WOVER=0.0
            WRITE(IOUT,6000) NAMSBR
            CALL LCMSIX(IPMACR,' ',2)
            CALL LCMSIX(IPSYS,' ',2)
            RETURN
          ENDIF
        ELSE
          IF(ILEAK .GE. 5 .AND. IDIR .GT. 0) THEN
            ALP1=DBLE(DB2(IGR))
          ELSE
            ALP1=1.0D0
          ENDIF
          DO 45 IREG=1,NREGIO
            IFPOS=KRSK+KEYFLX(IREG)
            RESTE(IFPOS,1)=REAL(ALP1)*SUNKNO(IFPOS,IGR)
            RESTE(IFPOS,2)=0.0
 45       CONTINUE
        ENDIF
C----
C  TEMPORARY SOURCES WITH FIXED SOURCES AND NEW FLUX
C----
        DO 110 IUNKNO=1,NUNKNO
          GAMMA(IUNKNO)=FUNKNO(IUNKNO,IGR)
 110    CONTINUE
        DO 120 IREG=1,NREGIO
          IBM=MATCOD(IREG)
          IFPOS=KRSK+KEYFLX(IREG)
          IF(KDIR .LE. 3) THEN
C----
C  INCLUDE SCATTERING SOURCES
C----
            NGSCAT=NJJ(IBM)
            ILSCAT=IJJ(IBM)
            IFSCAT=ILSCAT-NGSCAT+1
            ISCATP=IPOS(IBM)-1+NGSCAT
C----
C  DOWN-SCATTERING NEUTRONS FROM ALL GROUPS
C----
            DO 130 JGR=IFSCAT,IGR-1
              RESTE(IFPOS,1)=RESTE(IFPOS,1)
     >          +REAL(ALP1)*XSCAT(ISCATP)*FUNKNO(IFPOS,JGR)
              ISCATP=ISCATP-1
 130        CONTINUE
C----
C  WITHIN GROUP SCATTERING IF MOD(ITPIJ,2)=0
C----
            IF(MOD(ITPIJ,2) .EQ. 0) THEN
              RESTE(IFPOS,1)=RESTE(IFPOS,1)
     >          +REAL(ALP1)*SIGS0(IBM)*FUNKNO(IFPOS,IGR)
            ENDIF
            ISCATP=ISCATP-1
C----
C  UP-SCATTERING NEUTRONS
C----
            DO 140 JGR=IGR+1,ILSCAT
              RESTE(IFPOS,1)=RESTE(IFPOS,1)
     >          +REAL(ALP1)*XSCAT(ISCATP)*FUNKNO(IFPOS,JGR)
              ISCATP=ISCATP-1
 140        CONTINUE
          ELSE
C----
C  REMOVE WITHIN GROUP SCATTERING
C----
            RESTE(IFPOS,1)=RESTE(IFPOS,1)
     >        -REAL(ALP1)*SIGS0(IBM)*FUNKNO(IFPOS,IGR)
          ENDIF
 120    CONTINUE
C----
C  CALL SMODUL TO RETURN R(GAMMA)
C----
        IF(ILEAK .GE. 5) THEN
          CALL SMODUL(IPSYS ,IPTRK ,IPBTF ,IPRNTT,IDIR  ,NREGIO,
     >                NREGIO,MATCOD,VOLUME,KEYFLX,
     >                GAMMA(KRSK+1),RESTE(KRSK+1,1)     ,TITLE )
        ELSE
          CALL SMODUL(IPSYS ,IPTRK ,IPBTF ,IPRNTT,IDIR  ,NREGIO,
     >                NUNKNO,MATCOD,VOLUME,KEYFLX,
     >                GAMMA        ,RESTE(1,1)          ,TITLE )
        ENDIF
C----
C  STORE DELTA(N) IN GAMMA
C  INITIALIZE SOURCES IN DELTA(N) TO 0.0
C----
        DO 150 IUNKNO=1,NUNKNO
          GAMMA(IUNKNO)=FUNKNO(IUNKNO,IGR)-DELFL(IUNKNO,IGR)
 150    CONTINUE
C----
C  COMPUTE SCATTERING SOURCES IN DELTA(N)
C----
        DO 160 IREG=1,NREGIO
          IBM=MATCOD(IREG)
          IFPOS=KRSK+KEYFLX(IREG)
          IF(KDIR .LE. 3) THEN
C----
C  INCLUDE SCATTERING SOURCES
C----
            NGSCAT=NJJ(IBM)
            ILSCAT=IJJ(IBM)
            IFSCAT=ILSCAT-NGSCAT+1
            IF(IFSCAT .LT. ISTART) THEN
              NGSCAT=NGSCAT+IFSCAT-ISTART
              IFSCAT=ISTART
            ENDIF
            IF(ILSCAT .GT. ISTOP) THEN
              ILSCAT=ISTOP
            ENDIF
            ISCATP=IPOS(IBM)-1+NGSCAT
C----
C  DOWN-SCATTERING NEUTRONS FROM NON CONVERGED GROUPS
C----
            DO 170 JGR=IFSCAT,IGR-1
              RESTE(IFPOS,2)=RESTE(IFPOS,2)+REAL(ALP1)*XSCAT(ISCATP)*
     >          (FUNKNO(IFPOS,JGR)-DELFL(IFPOS,JGR))
              ISCATP=ISCATP-1
 170        CONTINUE
C----
C  WITHIN GROUP SCATTERING IF MOD(ITPIJ,2)=0
C----
            IF(MOD(ITPIJ,2) .EQ. 0) THEN
              RESTE(IFPOS,2)=RESTE(IFPOS,2)+REAL(ALP1)*SIGS0(IBM)*
     >          (FUNKNO(IFPOS,JGR)-DELFL(IFPOS,JGR))
            ENDIF
            ISCATP=ISCATP-1
C----
C  UP-SCATTERING NEUTRONS
C----
            DO 180 JGR=IGR+1,ILSCAT
              RESTE(IFPOS,2)=RESTE(IFPOS,2)+REAL(ALP1)*XSCAT(ISCATP)*
     >          (FUNKNO(IFPOS,JGR)-DELFL(IFPOS,JGR))
              ISCATP=ISCATP-1
 180        CONTINUE
          ELSE
C----
C  REMOVE WITHIN GROUP SCATTERING
C----
            RESTE(IFPOS,2)=RESTE(IFPOS,2)-REAL(ALP1)*SIGS0(IBM)*
     >        (FUNKNO(IFPOS,JGR)-DELFL(IFPOS,JGR))
          ENDIF
 160    CONTINUE
C----
C  CALL SMODUL TO RETURN R(DELTA)
C----
        IF(ILEAK .GE. 5) THEN
          CALL SMODUL(IPSYS ,IPTRK ,IPBTF ,IPRNTT,IDIR  ,NREGIO,
     >                NREGIO,MATCOD,VOLUME,KEYFLX,
     >                GAMMA(KRSK+1),RESTE(KRSK+1,2)     ,TITLE )
        ELSE
          CALL SMODUL(IPSYS ,IPTRK ,IPBTF ,IPRNTT,IDIR  ,NREGIO,
     >                NUNKNO,MATCOD,VOLUME,KEYFLX,
     >                GAMMA        ,RESTE(1,2)          ,TITLE )
        ENDIF
C----
C  ADD TO NUMERATOR AND DENOMINATOR
C----
        DO 190 IUNKNO=1,NUNKNO
          WNUMER=WNUMER-DBLE(RESTE(IUNKNO,2)*RESTE(IUNKNO,1))
          WDENOM=WDENOM+DBLE(RESTE(IUNKNO,2)*RESTE(IUNKNO,2))
 190    CONTINUE
        CALL LCMSIX(IPMACR,' ',2)
        CALL LCMSIX(IPSYS,' ',2)
 100  CONTINUE
C----
C  ADD TO CURRENT FLUXES WOVER*DELTA(N) WITH WOVER=WNUMER/WDENOM
C----
      IF(WDENOM .EQ. 0.0D0) THEN
        WOVER=-1.0
      ELSE
        WOVER=REAL(WNUMER/WDENOM)
      ENDIF
      IF(WOVER .GT. 0.0) THEN
        IF(ILEAK .LT. 5) THEN
          DO 200 IGR=ISTART,ISTOP
            DO 210 IUNKNO=1,NUNKNO
              FUNKNO(IUNKNO,IGR)=FUNKNO(IUNKNO,IGR)+
     >                WOVER*(FUNKNO(IUNKNO,IGR)-DELFL(IUNKNO,IGR))
 210        CONTINUE
 200      CONTINUE
        ELSE
          DO 220 IGR=ISTART,ISTOP
            DO 230 IREG=1,NREGIO
              IFPOS=KRSK+KEYFLX(IREG)
              FUNKNO(IFPOS,IGR)=FUNKNO(IFPOS,IGR)+
     >             WOVER*(FUNKNO(IFPOS,IGR)-DELFL(IFPOS,IGR))
 230        CONTINUE
 220      CONTINUE
        ENDIF
      ENDIF
      RETURN
C----
C  FORMAT
C----
 6000 FORMAT(' * * * WARNING * * * ',A6/
     >       ' NO ACCELERATION FOR LEAK TYPE = 4')
      END
