*DECK FLUBLN
      SUBROUTINE FLUBLN(IPMACR,IPRINT,NGROUP,NBMIX ,NREGIO,NUNKNO,
     >                  MATCOD,VOLUME,B2    ,FUNKNO,SIGT0 ,SIGS0 ,
     >                  IJJ   ,NJJ   ,IPOS  ,XSCAT ,IHETL ,REFKEF,
     >                  BIL1  )
C
C---------------------------  FLUBLN  ---------------------------------
C
C   1- PROGRAMME STATISTICS:
C       NAME     : FLUBLN
C       USE      : CALCULATION OF A DIRECTIONAL BUCKLING FROM THE
C                  CRITICAL NEUTRON BALANCE
C       MODIFIED : 94-07-21  (I. PETROVIC, G. MARLEAU)
C       MODIFIED : 95-07-24  (R.ROY)
C       AUTHOR   : I. PETROVIC
C
C   2- ROUTINE PARAMETERS:
C    INPUT:
C      IPMACR  : POINTER TO THE MACROLIB              I
C      IPRINT  : PRINT SELECTION FOR FLUX MODULES:    I
C      NGROUP  : NUMBER OF GROUPS.                    I
C      NBMIX   : NUMBER OF MIXTURES.                  I
C      NREGIO  : NUMBER OF REGIONS.                   I
C      NUNKNO  : NUMBER OF UNKNOWN IN THE SYSTEM.     I
C      MATCOD  : MATERIAL CODE IN REGIONS.            I(NREGIO)
C      IHETL   : TYPE OF BUCKLING CALCULATION         I
C                = 1 -> X-DIRECTION SEARCH
C                = 2 -> Y-DIRECTION SEARCH
C                = 3 -> Z-DIRECTION SEARCH
C                = 4 -> R-DIRECTION SEARCH  (X=Y)
C                = 5 -> GLOBAL-DIRECTION SEARCH (X=Y=Z)
C      VOLUME  : VOLUME OF REGIONS.                   R(NREGIO)
C      FUNKNO  : FLUX AND DIRECTIONAL CURRENTS.       R(NUNKNO,NGROUP)
C      REFKEF  : TARGET K-EFFECTIVE FOR TYPE B OR L   D
C      BIL1    : TOTAL NEUTRON PRODUCTION             D
C    OUTPUT
C      B2      : DIRECTIONAL BUCKLING (X,Y,Z,HOM)     R(4)
C    SCRATCH STORAGE:
C      SIGT0   : NU*FISSION CROSS SECTION OR          R(0:NBMIX)
C                TOTAL CROSS SECTION VECTOR
C                SIGT0(0)=0.0 IS FOR VOID
C      SIGS0   : WITHIN-GROUP SCATTERING              R(0:NBMIX)
C                SIGS0(0)=0.0 IS FOR VOID
C      NJJ     : NUMBER OF SCATTERING GROUP           I(0:NBMIX)
C                NJJ(0) = -NGROUP
C      IJJ     : LAST SCATTERING GROUP                I(0:NBMIX)
C                IJJ(0) = 0
C      IPOS    : POSITION SELF SCATTERING IN XSCAT    I(0:NBMIX)
C                IPOS(0)= NGROUP+1
C      XSCAT   : SCATTERING VECTOR                    R(0:NBMIX*NGROUP)
C                XSCAT(0) = 0.0
C
C   3- INTERNAL PARAMETERS:
C      CGRPNM  : CHARACTER GROUP NAME                 C*12
C
C---------------------------   FLUBLN  --------------------------------
C
      IMPLICIT   NONE
      INTEGER    IOUT,ILCMUP,ILCMDN
      CHARACTER  NAMSBR*6
      PARAMETER (IOUT=6,NAMSBR='FLUBLN',ILCMUP=1,ILCMDN=2)
C----
C  ROUTINE PARAMETERS
C----
      INTEGER    IPMACR,IPRINT,NGROUP,NBMIX,NREGIO,NUNKNO,
     >           IHETL
      INTEGER    MATCOD(NREGIO),IJJ(0:NBMIX),NJJ(0:NBMIX),
     >           IPOS(0:NBMIX)
      REAL       VOLUME(NREGIO),B2(4),FUNKNO(NUNKNO,NGROUP),
     >           SIGT0(0:NBMIX),SIGS0(0:NBMIX),
     >           XSCAT(0:NBMIX*NGROUP)
      DOUBLE PRECISION REFKEF,BIL1
C----
C  LOCAL PARAMETERS
C----
      INTEGER    IGR,IMIX,IG2,IXSS,IREG,IBM,IDIR
      CHARACTER  CGRPNM*12
      DOUBLE PRECISION SUM(0:3),RATIO,DSIGS
C----
C  COMPUTE FISSION SOURCE AND EVALUATE NEUTRON BALANCE
C----
      SUM(0)=BIL1/REFKEF
      SUM(1)=0.0D0
      SUM(2)=0.0D0
      SUM(3)=0.0D0
      DO 20 IGR=1,NGROUP
        WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGROUP
        CALL LCMSIX(IPMACR,CGRPNM,ILCMUP)
        CALL LCMGET(IPMACR,'TOTAL',SIGT0(1))
        CALL LCMGET(IPMACR,'NJJA 0',NJJ(1))
        CALL LCMGET(IPMACR,'IJJA 0',IJJ(1))
        CALL LCMGET(IPMACR,'IPOA 0',IPOS(1))
        CALL LCMGET(IPMACR,'SCAA 0',XSCAT(1))
        DO 190 IMIX=1,NBMIX
          DSIGS=0.0D0
          DO 191 IG2=IJJ(IMIX)-NJJ(IMIX)+1,IJJ(IMIX)
            IXSS=IPOS(IMIX)+IJJ(IMIX)-IG2
            DSIGS=DSIGS+DBLE(XSCAT(IXSS))
 191      CONTINUE
          SIGS0(IMIX)=REAL(DSIGS)-SIGT0(IMIX)
 190    CONTINUE
        DO 23 IREG=1,NREGIO
          IBM=MATCOD(IREG)
          SUM(0)=SUM(0)+SIGS0(IBM)*VOLUME(IREG)*FUNKNO(IREG,IGR)
          SUM(1)=SUM(1)+VOLUME(IREG)*FUNKNO(NREGIO+IREG,IGR)
          SUM(2)=SUM(2)+VOLUME(IREG)*FUNKNO(2*NREGIO+IREG,IGR)
          SUM(3)=SUM(3)+VOLUME(IREG)*FUNKNO(3*NREGIO+IREG,IGR)
 23     CONTINUE
        CALL LCMSIX(IPMACR,CGRPNM,ILCMDN)
 20   CONTINUE
      IF(IHETL .EQ. 1)THEN
        B2(1)=(SUM(0)-B2(2)*SUM(2)-B2(3)*SUM(3))/SUM(1)
      ELSEIF(IHETL .EQ. 2)THEN
        B2(2)=(SUM(0)-B2(1)*SUM(1)-B2(3)*SUM(3))/SUM(2)
      ELSEIF(IHETL .EQ. 3)THEN
        B2(3)=(SUM(0)-B2(1)*SUM(1)-B2(2)*SUM(2))/SUM(3)
      ELSEIF(IHETL .EQ. 4)THEN
        B2(1)=(SUM(0)-B2(3)*SUM(3))/(SUM(1)+SUM(2))
        B2(2)=B2(1)
      ELSEIF(IHETL .EQ. 5)THEN
        B2(1)=SUM(0)/(SUM(1)+SUM(2)+SUM(3))
        B2(2)=B2(1)
        B2(3)=B2(1)
      ELSEIF(IHETL .EQ. 6)THEN
        RATIO= B2(1)*SUM(1)+B2(2)*SUM(2)+B2(3)*SUM(3)
        IF(RATIO .NE. 0.0D0) THEN
          RATIO= SUM(0)/RATIO
        ENDIF
        B2(1)= RATIO*B2(1)
        B2(2)= RATIO*B2(2)
        B2(3)= RATIO*B2(3)
      ELSE
        CALL XABORT(NAMSBR//
     >  ': WHICH DIRECTIONAL BUCKLING WOULD YOU LIKE TO CALCULATE ? ')
      ENDIF
      IF(IPRINT .GE. 10) THEN
        WRITE(IOUT,6000) NAMSBR,(B2(IDIR),IDIR=1,3)
      ENDIF
C----
C  FORMATS
C----
 6000 FORMAT(1X,A6,' OUTPUT'/1X,'HETEROGENEOUS B2 = ',1P,3E15.7)
      RETURN
      END
