*DECK FLUDRV
      SUBROUTINE FLUDRV(IPFLUX,IPSYS ,IPMACR,IPTRK ,IPBTF ,ITPIJ ,
     >                  CXDOOR,COPTIO,IPRINT,LEAKSW,ITYPEC,ISADJ ,
     >                  ILEAK ,ICOPAR,EPSPAR,NGROUP,NBMIX ,NREGIO,
     >                  NUNKNO,NIFISS,MXNL  ,INITFL,B2    ,REFKEF,
     >                  TITLE ,NAMGPA,NAMGAS,IFSUR ,
     >                  FUNKNO,SUNKNO,MATCOD,VOLUME,KEYFLX)
C
C---------------------------  FLUDRV  ---------------------------------
C
C   1- PROGRAMME STATISTICS:
C       NAME     : FLUDRV
C       USE      : SOLVE THE TRANSPORT EQUATION USING THE POWER
C                  ITERATION METHOD WITH THERMAL GROUP REBALANCING
C                  AND VARIATIONAL ACCELERATION
C       MODIFIED : MODIFY DEFINITION OF ITYPEC
C                  2006/01/31  (P. DUFOUR)
C                  2000/08/01  (T. COURAU)
C                  2000/01/26  (G. MARLEAU)
C                  2000/01/20  (T. COURAU, G. MARLEAU)
C                  1994/10/26  (A. HEBERT)
C                  1994/07/21  (I. PETROVIC, G. MARLEAU)
C                  1993/07/12  (G. MARLEAU)
C                  1991/04/02  (R. ROY)
C                  1991/03/15  (G. MARLEAU)
C                  1991/01/28  (A. HEBERT)
C                  1986/05/14  (G.MARLEAU)
C
C   2- ROUTINE PARAMETERS:
C    INPUT:
C      IPFLUX  : POINTER TO THE SOLUTION                   I
C      IPSYS   : POINTER TO THE PIJ                        I
C      IPMACR  : POINTER TO THE MACROLIB                   I
C      IPTRK   : POINTER TO THE TRACKING                   I
C      IPBTF   : POINTER TO THE BINARY TRACKING FILE       I
C      ITPIJ   : TYPE OF CP AVAILABLE                      I
C                = 1 SCATT MOD PIJ (WIJ)
C                = 2 STAND. PIJ
C                = 3 SCATT MOD PIJ+PIJK (WIJ,WIJK)
C                = 4 STAND. PIJ+PIJK
C      CXDOOR  : NAME OF DOOR FOR FLUX SOLUTION            C*12
C      COPTIO  : CHARACTER FOR OPTION TYPE                 C*4
C      IPRINT  : PRINT SELECTION FOR FLUX MODULES          I
C                = 0/1/2/3 NO PRINT/SHORT PRINT/LONG
C      LEAKSW  : LEAKAGE SWITCH.                           L
C      ITYPEC  : TYPE OF FLUX EVALUATION                   I
C                = 0  SKIP THE FLUX CALCULATION
C                = 1  DIRECT PROBLEM FISSION SOURCES/
C                     K EFFECTIVE CONVERGENCE
C                = 2  DIRECT PROBLEM FISSION SOURCES/
C                     K EFFECTIVE CONVERGENCE/
C                     FIXED DB2 BUCKLING
C                = 3  DIRECT PROBLEM FISSION SOURCES/
C                     K EFFECTIVE CONVERGENCE/
C                     DB2 BUCKLING EVALUATION
C                = 4  DIRECT PROBLEM FISSION SOURCES/
C                     DB2 BUCKLING CONVERGENCE
C                = 5  DIRECT PROBLEM DB2 SOURCES
C                = 6  DIRECT PROBLEM FIXED SOURCES
C      ISADJ   : ADJOINT CALCULATION AND SAVE OPTION       I
C                = 0 ONLY DIRECT FLUX
C                = 1 DIRECT AND PSEUDO ADJOINT FLUX
C                = 2 DIRECT, PSEUDO-ADJOINT AND ADJOINT
C                = 3 DIRECT, PSEUDO ADJOINT AND
C                    GENERALIZED PSEUDO-ADJOINT
C                = 4 DIRECT, PSEUDO ADJOINT, ADJOINT,
C                    GENERALIZED PSEUDO-ADJOINT AND
C                    GENERALIZED ADJOINT
C      ILEAK   : METHOD USED TO INCLUDE LEAKAGE EFFECT     I
C                = 0 NO DB2 EFFECT
C                = 1 HOMOGENEOUS WITH SCATTERING MODIFIED
C                    CP MATRIX MULTIPLIED BY A PNL FACTOR
C                = 2 HOMOGENEOUS REDUCED CP MATRIX
C                    MULTIPLIED BY PNL FACTOR
C                    (DEFAULT FOR TYPE B AND L)
C                = 3 HOMOGENEOUS SIGS0-DB2 APPROXIMATION
C                = 4 HOMOGENEOUS ALBEDO APPROXIMATION
C                = 5,6,7,8,9,10 HETEROGENEOUS WITH PIJK
C                  AND FIXED B->5 OR
C                  SEARCH X->6 Y->7 Z->8 R->9 ALL->10
C      ICOPAR  : ITERATION PARAMETERS                      I(6)
C                IFRITR=ICOPAR(1)
C                IACITR=ICOPAR(2)
C                IREBAL=ICOPAR(3)
C                MAXINR=ICOPAR(4)
C                MAXOUT=ICOPAR(5)
C                ICONTM=ICOPAR(6)
C      EPSPAR  : ITERATION PRECISION                       R(NBEPS)
C                EPSINR=EPSPAR(1)
C                EPSOUT=EPSPAR(2)
C                EPSUNK=EPSPAR(3)
C                EPSGPA=EPSPAR(4)
C                CONGPA=EPSPAR(5)
C      NGROUP  : NUMBER OF GROUPS.                         I
C      NBMIX   : NUMBER OF MIXTURES.                       I
C      NREGIO  : NUMBER OF REGIONS.                        I
C      NUNKNO  : NUMBER OF UNKNOWN IN THE SYSTEM.          I
C      NIFISS  : NUMBER OF FISSILE ISOTOPES.               I
C      MXNL    : MAXIMUM ANISOTROPY
C      INITFL  : FLUX INITIALIZATION OPTION                I
C                = 0 DEFAULT INITIALIZATION
C                = 1 TAKEN FROM FLXUNK
C                = 2 TAKEN FROM DATA STRUCTURE
C      B2      : INITIAL ESTIMATE OF THE BUCKLING FOR      R(4)
C                THE B2 SEARCH OR IMPOSED BUCKLING.
C      REFKEF  : TARGET K-EFFECTIVE FOR TYPE B OR L        D
C      TITLE   : TITLE OF THE DRAGON CASE.                 C*72
C      NAMGPA  : NAME OF GENERALIZED PSEUDO-ADJOINT        C(2)*12
C      NAMGAS  : NAME OF GENERALIZED ADJOINT SOURCE        C*12  
C      IFSUR   : FLAG FOR SURFACE FLUX                     I
C      FUNKNO  : UNKNOWN VECTOR SOLVED FOR.                R(NUNKNO,
C                                                            NGROUP)
C      SUNKNO  : SOURCE FOR SYSTEM OF UNKNOWN.             R(NUNKNO,
C                                                            NGROUP)
C      MATCOD  : MATERIAL CODE IN REGIONS.                 I(NREGIO)
C      VOLUME  : VOLUME OF REGIONS. DIMENSION              R(NREGIO)
C      KEYFLX  : FLUX ELEMENTS IN UNKNOWN SYSTEM.          I(NREGIO)
C
C   3- INTERNAL PARAMETERS:
C      JPMF    : JPM FLUX SOLUTION DOOR                    EXT ROUTINE
C      JPMS    : JPM RESIDUAL CALCULATION DOOR             EXT ROUTINE
C      TRFICF  : DEFAULT FLUX SOLUTION DOOR                EXT ROUTINE
C      TRFICS  : DEFAULT RESIDUAL CALCULATION DOOR         EXT ROUTINE
C
C---------------------------   FLUDRV  --------------------------------
C
      IMPLICIT    NONE
      INTEGER     IOUT,NSTATE,NBEPS,ILCMUP,ILCMDN
      REAL        CONVF
      CHARACTER   NAMSBR*6
      PARAMETER  (IOUT=6,NSTATE=40,NBEPS=5,ILCMUP=1,ILCMDN=2,
     >            CONVF=1.0E-7,NAMSBR='FLUDRV')
      INTEGER     MAXIXS 
      PARAMETER  (MAXIXS=3)
*----
*  ROUTINE PARAMETERS
*----
      INTEGER     IPFLUX,IPSYS,IPMACR,IPTRK,IPBTF,ITPIJ,IPRINT,ITYPEC,
     >            ISADJ,ILEAK,NGROUP,NBMIX,NREGIO,NUNKNO,NIFISS,MXNL,
     >            INITFL
      CHARACTER   CXDOOR*12,COPTIO*4,TITLE*72,NAMGPA(2)*12,NAMGAS*12,
     >            CGRPNM*12
      INTEGER     IFSUR
      INTEGER     ICOPAR(6),MATCOD(NREGIO),KEYFLX(NREGIO),IBM
      LOGICAL     LEAKSW
      REAL        EPSPAR(NBEPS),B2(4),FUNKNO(NUNKNO,NGROUP),
     >            SUNKNO(NUNKNO,NGROUP),VOLUME(NREGIO)
      DOUBLE PRECISION REFKEF
C----
C  MEMORY ALLOCATION
C----
      INTEGER     IBASE(1)
      REAL        RBASE
      COMMON      RBASE(1)
      EQUIVALENCE(RBASE(1),IBASE(1))
      INTEGER     IQTOTL,IIGCON,IDELFL,ISOURC,IDB2,IPNL,IIJJ,INJJ,
     >            IIPOS,ISIGT0,ISIGF0,ISIGTH,ISIGS0,
     >            IXSCAT,IFFDIR,IFGUPS,IRGUPS,IQFDIR,IFLUA
C----
C  LOCAL PARAMETERS
C----
      EXTERNAL    JPMF,JPMS,TRFICF,TRFICS
      CHARACTER   CDIRFL*12,CFIXSR*12,CREBAL*3
      LOGICAL     LCONV,LOSCIL
      INTEGER     IOUTER,NBEPSF,IFRITR,IACITR,IREBAL,MAXINR,MAXOUT,
     >            IINNER,IGR,IDR,NDR,IDIR,IRLOC,KDIR,ILCMLN,
     >            ILCMTY,ILRG,ILFG,INL,IREG,IMIX,ISADJO,ICONTM
      REAL        EPSOFL,EPSINR,EPSOUT,EPSUNK,EPSEIG,
     >            EPSFLX,EPSTHR,DELFLX,FLXSAV,EIGENK(2),RELFOL
      DOUBLE PRECISION DSCAL,DNORMX,DNORMS,DNORMF,DNORMA
C----
C  Variables for surface flux
C----
      INTEGER     ISTATE(NSTATE),KCHAR(3),ITC
      INTEGER     NUNSUR,NSUR,NVOL
      INTEGER     BCRFTR,COND,IDFSUR,IDFPOS,IDPFUI,
     >            KEYMRG,MATALB,ALBEDO
      CHARACTER   TRKTYP*12
      REAL        ALBD(6)
C----
C  INITIALIZE CONTROL PARAMETERS
C----      
      LOSCIL=.FALSE.
      RELFOL=-1.0
      IOUTER=0
      NBEPSF=0
      ISADJO=0
      IFRITR=ICOPAR(1)
      IACITR=ICOPAR(2)
      IREBAL=ICOPAR(3)
      MAXINR=ICOPAR(4)
      MAXOUT=ICOPAR(5)
      ICONTM=ICOPAR(6)
      EPSOFL=0.0
      IF (ISADJ .GE. 3) THEN
        EPSINR=EPSPAR(4)*EPSPAR(1)
        EPSOUT=EPSPAR(4)*EPSPAR(2)
        EPSUNK=EPSPAR(4)*EPSPAR(3)
      ELSE  
        EPSINR=EPSPAR(1)
        EPSOUT=EPSPAR(2)
        EPSUNK=EPSPAR(3)
      ENDIF
C----
C  ALLOCATE MEMORY BLOCK USED THROUGHOUT CALCULATION
C----
      CALL SETARA(RBASE,NGROUP,IDB2)
      CALL SETARA(RBASE,NGROUP,IPNL)
      CALL SETARA(IBASE,NBMIX+1,IIJJ)
      CALL SETARA(IBASE,NBMIX+1,INJJ)
      CALL SETARA(IBASE,NBMIX+1,IIPOS)
      CALL SETARA(RBASE,NBMIX+1,ISIGT0)
      CALL SETARA(RBASE,NBMIX*MAX(NIFISS,1),ISIGF0)
      CALL SETARA(RBASE,NGROUP,ISIGTH)
      CALL SETARA(RBASE,NBMIX+1,ISIGS0)
      CALL SETARA(RBASE,NBMIX*NGROUP+1,IXSCAT)
      CALL SETARA(IBASE,MXNL,IFGUPS)
      IF (ISADJ .GE. 3 .AND. NIFISS .GE. 1) THEN
        CALL SETARA(RBASE,NUNKNO*NGROUP,IFFDIR)
        CALL SETARA(RBASE,2*NREGIO*MAX(NIFISS,1),IQFDIR)
      ENDIF
      CALL SETARA(IBASE,MXNL*NBMIX,IRGUPS)
      CALL LCMGET(IPMACR,'FGWITHUPSCAT',IBASE(IRGUPS))
      ILRG=IRGUPS-1
      ILFG=IFGUPS
      DO 110 INL=1,MXNL
        IBASE(ILFG)=NGROUP+1
        DO 111 IREG=1,NREGIO
          IMIX=MATCOD(IREG)
          IF(IMIX .NE. 0) THEN
            IBASE(ILFG)=MIN(IBASE(ILFG),IBASE(ILRG+IMIX))
          ENDIF
 111    CONTINUE
        ILRG=ILRG+NBMIX
        ILFG=ILFG+1
 110  CONTINUE
      CALL RLSARA(IBASE(IRGUPS))
C----
C  INITIALIZE SIGT0(0), SIGS0(0) AND XSCAT(0) TO 0.0
C  INITIALIZE NJJ(0)=-NGROUP, IJJ(0)=0
C  AND IPOS(0)=NGROUP+1
C----
      IBASE(IIJJ)=0
      IBASE(INJJ)=-NGROUP
      IBASE(IIPOS)=NGROUP+1
      RBASE(ISIGT0)=0.0
      RBASE(ISIGS0)=0.0
      RBASE(IXSCAT)=0.0
      CDIRFL='FLUXDIRECT  '
      CALL XDRSET(RBASE(IDB2),NGROUP,0.0)
      CALL XDRSET(RBASE(IPNL),NGROUP,0.0)
C----
C  READ DIFFUSION COEFFICIENTS FROM FLUX DIRECTORY
C  FOR FIXED LEAKAGE
C----
      IF(ITYPEC .EQ. 2) THEN
        CALL LCMSIX(IPFLUX,CDIRFL,ILCMUP)
        CALL LCMLEN(IPFLUX,'DIFFB1HOM   ',ILCMLN,ILCMTY)
        IF(ILCMLN .EQ. NGROUP) THEN
          CALL LCMGET(IPFLUX,'DIFFB1HOM   ',RBASE(IDB2))
          CALL LCMGET(IPFLUX,'B2  B1HOM   ',B2(4))
        ELSE
          CALL XDRSET(RBASE(IDB2),NGROUP,0.0)
          CALL LCMPUT(IPFLUX,'DIFFB1HOM   ',NGROUP,2,RBASE(IDB2))
          CALL LCMPUT(IPFLUX,'B2  B1HOM   ',1,2,B2(4))
        ENDIF
        CALL LCMLEN(IPFLUX,'PNL         ',ILCMLN,ILCMTY)
        IF(ILCMLN .EQ. NGROUP) THEN
          CALL LCMGET(IPFLUX,'PNL         ',RBASE(IPNL))
        ENDIF
        CALL LCMSIX(IPFLUX,CDIRFL,ILCMDN)
      ENDIF
      CALL LCMLEN(IPFLUX,'K-EFFECTIVE ',ILCMLN,ILCMTY)
      IF(ILCMLN .EQ. 1) THEN
        CALL LCMGET(IPFLUX,'K-EFFECTIVE ',EIGENK)
      ENDIF
C----
C  INITIALIZE EIGENVALUE IF ADJOINT OR GENERALIZED
C----
      EIGENK(1)=1.0
      EIGENK(2)=1.0
      IF(ISADJ .GE. 3) CDIRFL='PSEUDOADJ   '
      CALL LCMSIX(IPFLUX,CDIRFL,ILCMUP)
      CALL LCMLEN(IPFLUX,'1/EIGENVALUE',ILCMLN,ILCMTY)
      IF(ILCMLN .EQ. 1) THEN
        CALL LCMGET(IPFLUX,'1/EIGENVALUE',EIGENK)
      ENDIF
      CALL LCMSIX(IPFLUX,CDIRFL,ILCMDN)
      IF(ISADJ .GE. 3) THEN
        CDIRFL=NAMGPA(1)
        CFIXSR=NAMGAS
      ELSE IF(ISADJ .GE. 1) THEN
        CDIRFL='PSEUDOADJ   '
        CFIXSR='FIXA        '
      ELSE
        CFIXSR='FIXE        '
      ENDIF
C----
C FOR GENERALIZED ADJOINT CHECK THE FIXED
C SOURCE ORTHOGONALITY & RENORMALIZE IT
C---
      IF(ISADJ.GE.3 .AND. ITYPEC .NE. 0) THEN
C---
C IF WE CONSIDER SYSTEMATIC DECONTAMINATION 
C OR NO FLUX INITIALIZATION WE DO NOT ITERATE
C FOR A CONTAMINATED SOLUTION (ISADJO->1) 
C--
        IF(INITFL .EQ. 0 .OR. ICONTM .EQ. 1) THEN
          ISADJO=1
          IREBAL=0
        ENDIF   
        CALL XDRSET(SUNKNO,NUNKNO*NGROUP,0.0)
        CALL XDRSET(FUNKNO,NUNKNO*NGROUP,0.0)
        CALL LCMSIX(IPFLUX,'FLUXDIRECT  ',ILCMUP)
        DSCAL=0.D0
        DNORMS=0.D0
        DNORMF=0.D0
        DO 10 IGR=1,NGROUP
          WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGROUP
          CALL LCMSIX(IPMACR,CGRPNM,ILCMUP) 
          CALL LCMGET(IPMACR,CFIXSR,SUNKNO(1,IGR))
          CALL LCMSIX(IPMACR,CGRPNM,ILCMDN)
          WRITE(CGRPNM,'(4HFLUX,I3)') IGR
          CALL LCMGET(IPFLUX,CGRPNM,FUNKNO(1,IGR))
          DO 15 IREG=1,NREGIO
            IBM=MATCOD(IREG)
            IF(IBM .GT. 0) THEN 
              DSCAL=DSCAL+DBLE(SUNKNO(IBM,IGR)
     >                   *FUNKNO(KEYFLX(IREG),IGR)*VOLUME(IREG))
              DNORMF=DNORMF+DBLE(FUNKNO(KEYFLX(IREG),IGR)
     >                           *VOLUME(IREG))**2
              DNORMS=DNORMS+DBLE(SUNKNO(IBM,IGR))**2
            ENDIF 
 15       CONTINUE   
 10     CONTINUE
        DSCAL=DSCAL/SQRT(DNORMS*DNORMF)
        CALL LCMSIX(IPFLUX,'FLUXDIRECT  ',ILCMDN)
        CALL LCMSIX(IPFLUX,'PSEUDOADJ   ',ILCMUP)
        DNORMA=0.D0
        DO 11 IGR=1,NGROUP
          WRITE(CGRPNM,'(4HFLUX,I3)') IGR
          CALL LCMGET(IPFLUX,CGRPNM,FUNKNO(1,IGR))
          DO 16 IREG=1,NREGIO
           DNORMA=DNORMA+DBLE(FUNKNO(KEYFLX(IREG),IGR))**2
 16       CONTINUE
 11     CONTINUE
        CALL LCMSIX(IPFLUX,'PSEUDOADJ   ',ILCMDN)
C---
C CHECK THE SOURCE ORTHOGONALITY
C---   
        IF(IPRINT .GE. 5) WRITE(IOUT,5500) DSCAL
        IF(ABS(DSCAL) .GT. CONVF .OR. SQRT(DNORMS) .LT. 1.0E-10 ) THEN
          WRITE(IOUT,5600) DSCAL,SQRT(DNORMS)
          GOTO 500
        ENDIF 
C----
C  FOR GENERALIZED ADJOINT
C  SAVE REFERENCE EIGENVALUE
C----
        CALL LCMSIX(IPFLUX,CDIRFL,ILCMUP)
        REFKEF=EIGENK(1)
        CALL LCMPUT(IPFLUX,'1/EIGENVALUE',1,2,EIGENK)
      ELSE
        CALL LCMSIX(IPFLUX,CDIRFL,ILCMUP)
      ENDIF
C----
C  SAVE INITIALIZATION FLUX IF REQUIRED
C----
      FLXSAV=1.0 
      IF (INITFL .EQ. 0 .AND. ISADJ .GE. 3) FLXSAV=0.0
      IF(INITFL .NE. 1) THEN
        IF (ITYPEC .EQ. 0 .OR. ITYPEC .GE. 6) FLXSAV=0.0
        CALL XDRSET(FUNKNO,NUNKNO*NGROUP,FLXSAV)
C----
C  IF IPFLUX EXISTS, TRY TO INITIALIZE FLUX
C----
        IF(ISADJ  .GE. 3 .AND. INITFL .EQ. 2 .AND. ITYPEC .NE. 0) THEN
C----
C RENORMALIZE THE  ADJOINT FLUX
C----
          DNORMX=EPSPAR(5)*SQRT(DNORMS/DNORMA)
          DO 20 IGR=1,NGROUP
            DO 25 IREG=1,NREGIO
               FUNKNO(KEYFLX(IREG),IGR)=FUNKNO(KEYFLX(IREG),IGR)*DNORMX
 25         CONTINUE
 20       CONTINUE
        ELSE IF(INITFL .EQ. 2) THEN
          CALL FLUGFL(IPFLUX,NGROUP,NUNKNO,FUNKNO)
        ENDIF             
      ENDIF
C----
C  SAVE FLUX
C----
      CALL FLUSFL(IPFLUX,NGROUP,NUNKNO,FUNKNO)
C----
C  SAVE DIFFUSION COEFFICIENTS IF REQUIRED
C----
      IF(ITYPEC .EQ. 2 ) THEN
        IF( ISADJ .GE. 1) THEN
          CALL LCMPUT(IPFLUX,'DIFFB1HOM   ',NGROUP,2,RBASE(IDB2))
          CALL LCMPUT(IPFLUX,'B2  B1HOM   ',1,2,B2(4)) 
        ENDIF
C----
C  COMPUTE LEAKAGE TERM FROM DIFFUSION COEFFICIENTS
C  AND BUCKLING
C----
        DO 120 IGR=1,NGROUP
          RBASE(IDB2+IGR-1)=RBASE(IDB2+IGR-1)*B2(4)
 120    CONTINUE
      ENDIF
C----
C  IF ITYPEC = 0 NO FLUX CALCULATION/
C  READ EIGENVALUE AND RETURN
C----
      IF(ITYPEC .EQ. 0) THEN
        EIGENK(1)=0.0
        EIGENK(2)=0.0
        CALL LCMPUT(IPFLUX,'1/EIGENVALUE',1,2,EIGENK)
        CALL LCMSIX(IPFLUX,CDIRFL,ILCMDN)
        GOTO 500
      ENDIF
C----
C  IOUTER = O -> INITIALIZE SOURCE
C----
      CALL SETARA(RBASE,2*NREGIO*MAX(NIFISS,1),IQTOTL)
      CALL FLUQFE(IPFLUX,IPSYS ,IPMACR,IPRINT,LEAKSW,ITYPEC,
     >            ISADJ ,ILEAK ,IOUTER,NGROUP,NBMIX ,NREGIO,
     >            NUNKNO,NIFISS,MATCOD,VOLUME,KEYFLX,EPSEIG,
     >            EPSFLX,COPTIO,REFKEF,B2    ,EIGENK,LCONV ,
     >            CFIXSR,FUNKNO,SUNKNO,
     >            RBASE(IDB2)  ,RBASE(ISIGT0),RBASE(ISIGS0),
     >            RBASE(IXSCAT),IBASE(IIJJ)  ,IBASE(INJJ)  ,
     >            IBASE(IIPOS) ,RBASE(ISIGTH),RBASE(ISIGF0),
     >            RBASE(IQTOTL),RELFOL,LOSCIL)
      CALL RLSARA(RBASE(IQTOTL))
C----
C  PRINT REQUIRED INFORMATION
C----
      IF(IPRINT.GE.1) THEN
        CREBAL='ON '
        IF(IREBAL.EQ.0) THEN
          CREBAL='OFF '
        ENDIF
        WRITE(IOUT,6000) CXDOOR,NGROUP,NREGIO,MAXOUT,MAXINR,IFRITR,
     >                   IACITR,CREBAL,EPSOUT,EPSUNK,EPSINR
      ENDIF
C----
C  START OUTER ITERATION
C----
 1000 IOUTER=IOUTER+1
      EPSEIG=EPSOUT
      EPSFLX=EPSUNK
      EPSTHR=EPSINR
      IINNER=MAXINR
      IF(IPRINT.GE.1) WRITE(IOUT,6004) IOUTER
C----
C  INNER ITERATION FOR FLUX SOLUTION
C----
      CALL SETARA(IBASE,NGROUP,IIGCON)
      CALL SETARA(RBASE,NUNKNO*NGROUP,IDELFL)
      CALL SETARA(RBASE,NUNKNO,ISOURC)
      IDIR=0
      IF(CXDOOR.EQ.'TRAFIC') THEN
        CALL FLUINR(TRFICF,TRFICS,IPSYS ,IPMACR,IPTRK ,IPBTF ,
     >              IPRINT,ITPIJ ,IDIR  ,NGROUP,NBMIX ,NREGIO,
     >              NUNKNO,IINNER,EPSTHR,IFRITR,IACITR,IREBAL,
     >              LEAKSW,ILEAK ,TITLE ,MATCOD,VOLUME,KEYFLX,
     >              FUNKNO,SUNKNO,MXNL  ,ISADJ ,
     >              RBASE(IDB2)  ,RBASE(ISIGT0),RBASE(ISIGS0),
     >              RBASE(IXSCAT),IBASE(IIJJ)  ,IBASE(INJJ)  ,
     >              IBASE(IIPOS) ,IBASE(IFGUPS),
     >              IBASE(IIGCON),RBASE(IDELFL),RBASE(ISOURC),
     >              RBASE(IPNL)  )
      ELSE IF(CXDOOR.EQ.'JPM') THEN
        CALL FLUINR(JPMF  ,JPMS  ,IPSYS ,IPMACR,IPTRK ,IPBTF ,
     >              IPRINT,ITPIJ ,IDIR  ,NGROUP,NBMIX ,NREGIO,
     >              NUNKNO,IINNER,EPSTHR,IFRITR,IACITR,IREBAL,
     >              LEAKSW,ILEAK ,TITLE ,MATCOD,VOLUME,KEYFLX,
     >              FUNKNO,SUNKNO,MXNL  ,ISADJ ,
     >              RBASE(IDB2)  ,RBASE(ISIGT0),RBASE(ISIGS0),
     >              RBASE(IXSCAT),IBASE(IIJJ)  ,IBASE(INJJ)  ,
     >              IBASE(IIPOS) ,IBASE(IFGUPS),
     >              IBASE(IIGCON),RBASE(IDELFL),RBASE(ISOURC),
     >              RBASE(IPNL)  )
      ELSE
        CALL XABORT(NAMSBR//': INVALID FLUX SOLUTION MODULE '//CXDOOR)
      ENDIF
      IF(ITYPEC .GT. 1 .AND.
     >   ITYPEC .LT. 6 .AND.
     >   ILEAK .GE. 5) THEN
C----
C  PIJK CALCULATION ALLOWED ONLY WITH CXDOOR=TRAFIC
C----
        IF(IOUTER.EQ.1) THEN
          IDR=NREGIO+1
          NDR=3*NREGIO
          DO 200 IGR=1,NGROUP
            CALL XDRSET(FUNKNO(IDR,IGR),NDR,0.0)
 200      CONTINUE
        ENDIF
        CALL FLUQFC(IPSYS ,IPMACR,ITPIJ ,COPTIO,IPRINT,NGROUP,
     >              NBMIX ,NREGIO,NUNKNO,MATCOD,FUNKNO,SUNKNO,
     >              RBASE(IDB2)  ,RBASE(ISIGT0),RBASE(ISIGS0),
     >              RBASE(ISIGTH))
        DO 210 IDIR=1,3
          IDR=NREGIO*IDIR
          EPSTHR=EPSINR
          IINNER=MAXINR
          IF(COPTIO .EQ. 'B0TR' .OR.
     >       COPTIO .EQ. 'B0  ' .OR.
     >       COPTIO .EQ. 'P0  ') THEN
            KDIR=IDIR+3
          ELSE
            KDIR=IDIR
          ENDIF
          CALL FLUINR(TRFICF,TRFICS,IPSYS ,IPMACR,IPTRK ,IPBTF ,
     >                IPRINT,ITPIJ ,KDIR  ,NGROUP,NBMIX ,NREGIO,
     >                NUNKNO,IINNER,EPSTHR,IFRITR,IACITR,IREBAL,
     >                LEAKSW,ILEAK ,TITLE ,MATCOD,VOLUME,KEYFLX,
     >                FUNKNO,SUNKNO,MXNL  ,ISADJ ,
     >                RBASE(IDB2)  ,RBASE(ISIGT0),RBASE(ISIGS0),
     >                RBASE(IXSCAT),IBASE(IIJJ)  ,IBASE(INJJ)  ,
     >                IBASE(IIPOS) ,IBASE(IFGUPS),
     >                IBASE(IIGCON),RBASE(IDELFL),RBASE(ISOURC),
     >                RBASE(IPNL)  )
 210    CONTINUE
      ENDIF
      CALL RLSARA(RBASE(ISOURC))
      CALL RLSARA(RBASE(IDELFL))
      CALL RLSARA(IBASE(IIGCON))
C----
C  RENORMALIZE FLUXES,SOURCES/EIGENVALUE CONVERGENCE
C----
      CALL SETARA(RBASE,2*NREGIO*MAX(NIFISS,1),IQTOTL)
      CALL FLUQFE(IPFLUX,IPSYS ,IPMACR,IPRINT,LEAKSW,ITYPEC,
     >            ISADJ ,ILEAK ,IOUTER,NGROUP,NBMIX ,NREGIO,
     >            NUNKNO,NIFISS,MATCOD,VOLUME,KEYFLX,EPSEIG,
     >            EPSFLX,COPTIO,REFKEF,B2    ,EIGENK,LCONV ,
     >            CFIXSR,FUNKNO,SUNKNO,
     >            RBASE(IDB2)  ,RBASE(ISIGT0),RBASE(ISIGS0),
     >            RBASE(IXSCAT),IBASE(IIJJ)  ,IBASE(INJJ)  ,
     >            IBASE(IIPOS) ,RBASE(ISIGTH),RBASE(ISIGF0),
     >            RBASE(IQTOTL),RELFOL,LOSCIL)
      CALL RLSARA(RBASE(IQTOTL))
C----
C  FOR GENERALIZED ADJOINT CALCULATION:
C  CONTAMINATED SOLUTION HAS CONVERGED, DECONTAMINATE IT
C  AND CONTINUE ITERATION FOR DECONTAMINATED SOLUTION
C----
      IF(ISADJ .GE. 3 .AND. LCONV .AND. ISADJO .EQ. 0) THEN
        CALL LCMSIX(IPFLUX,CDIRFL,ILCMDN)
        CALL SETARA(RBASE,NUNKNO*NGROUP,IFLUA)
        CALL FLUADN(IPFLUX,IPMACR,IPRINT,NGROUP,NBMIX ,NREGIO,
     >              NUNKNO,NIFISS,FUNKNO,MATCOD,VOLUME,KEYFLX,
     >              1,
     >              RBASE(ISIGF0),RBASE(IFFDIR),RBASE(IQFDIR),
     >              RBASE(IFLUA) )
        CALL RLSARA(RBASE(IFLUA))
        IF(IPRINT.GE.1) WRITE(IOUT,5700) 
        CALL LCMSIX(IPFLUX,CDIRFL,ILCMUP)
        CALL SETARA(RBASE,2*NREGIO*MAX(NIFISS,1),IQTOTL)
        LOSCIL=.FALSE.
        RELFOL=-1.0 
        CALL FLUQFE(IPFLUX,IPSYS ,IPMACR,IPRINT,LEAKSW,ITYPEC,
     >              ISADJ ,ILEAK ,IOUTER,NGROUP,NBMIX ,NREGIO,
     >              NUNKNO,NIFISS,MATCOD,VOLUME,KEYFLX,EPSEIG,
     >              EPSFLX,COPTIO,REFKEF,B2    ,EIGENK,LCONV ,
     >              CFIXSR,FUNKNO,SUNKNO,
     >              RBASE(IDB2)  ,RBASE(ISIGT0),RBASE(ISIGS0),
     >              RBASE(IXSCAT),IBASE(IIJJ)  ,IBASE(INJJ)  ,
     >              IBASE(IIPOS) ,RBASE(ISIGTH),RBASE(ISIGF0),
     >              RBASE(IQTOTL),RELFOL,LOSCIL)
        CALL RLSARA(RBASE(IQTOTL))
        ISADJO=1
        IREBAL=0
      ENDIF
C----
C  CHECK CONVERGENCE PARAMETERS
C----
      IF( IOUTER .LT. MAXOUT .AND. .NOT. LCONV) THEN
        IF(EPSOFL .GT. 0.0) THEN 
          DELFLX=ABS(EPSOFL-EPSFLX)/MAX(EPSOFL,EPSFLX)
          EPSOFL=EPSFLX
          IF(IOUTER .GT. 3) THEN
            IF(DELFLX .LT. EPSUNK) THEN
              NBEPSF=NBEPSF+1  
            ELSE
              NBEPSF=0
            ENDIF
          ENDIF
        ELSE IF(EPSFLX .GT. 0.0) THEN
          EPSOFL=EPSFLX
          NBEPSF=0
        ENDIF
        IF(NBEPSF .LE. MAXIXS) GOTO 1000
      ENDIF
C----
C  Compute surface flux only for direct flux calculation
C  in the case where CP matrices are available
C----
      IF(IFSUR .EQ. 1) THEN
        IF(CXDOOR .EQ. 'TRAFIC' .AND. ISADJ .EQ. 0) THEN
          CALL LCMGET(IPTRK,'TRACK-TYPE',KCHAR)
          WRITE(TRKTYP,'(3A4)') (KCHAR(ITC),ITC=1,3)
          IF(TRKTYP .EQ. 'EXCELL') THEN
            CALL LCMGET(IPTRK,'STATE-VECTOR',ISTATE)
            NUNSUR=ISTATE(5)
            CALL SETARA(IBASE,NUNSUR,BCRFTR)
            CALL SETARA(IBASE,NUNSUR,COND)
            CALL SETARA(RBASE,NUNSUR*NGROUP,IDFSUR)
            CALL SETARA(RBASE,NUNSUR*NGROUP,IDFPOS)
            CALL SETARA(RBASE,2*NUNSUR*NREGIO,IDPFUI)
            CALL LCMGET(IPTRK,'ALBEDO      ',ALBD  )      
            CALL LCMGET(IPTRK,'BC-REFL+TRAN',IBASE(BCRFTR))
            IF(ISTATE(7) .LT. 4) THEN
C----
C  2) compute surface flux from sources
C----
              CALL LCMSIX(IPTRK,'EXCELL      ',ILCMUP)
              CALL LCMGET(IPTRK,'STATE-VECTOR',ISTATE)
              NSUR=ISTATE(2)
              NVOL=ISTATE(3)
              CALL SETARA(IBASE,NSUR+NVOL+1,KEYMRG)
              CALL SETARA(IBASE,NSUR+NVOL+1,MATALB)
              CALL LCMGET(IPTRK,'MATALB      ',IBASE(MATALB))
              CALL LCMGET(IPTRK,'KEYMRG      ',IBASE(KEYMRG))
              CALL LCMSIX(IPTRK,'EXCELL      ',ILCMDN)
            ELSE
              NSUR=ISTATE(5)
              NVOL=ISTATE(1)
              CALL SETARA(IBASE,NSUR+NVOL+1,KEYMRG)
              CALL SETARA(IBASE,NSUR+NVOL+1,MATALB)
              CALL LCMSIX(IPTRK,'NXTRecords  ',ILCMUP)
              CALL LCMGET(IPTRK,'MATALB      ',IBASE(MATALB))
              CALL LCMGET(IPTRK,'KEYMRG      ',IBASE(KEYMRG))
              CALL LCMSIX(IPTRK,'NXTRecords  ',ILCMDN)
            ENDIF
            CALL SETARA(RBASE,NSUR,ALBEDO)
            CALL SETARA(RBASE,2*NREGIO*MAX(NIFISS,1),IQTOTL)
            CALL FLUSUR(IPFLUX,IPSYS ,IPMACR,IPRINT,NGROUP,NBMIX ,
     >                  NREGIO,NIFISS,ITYPEC,ILEAK ,NUNKNO,NUNSUR,NSUR,
     >                  NVOL  ,MATCOD,KEYFLX,FUNKNO,EIGENK,B2    ,
     >                  SUNKNO,       RBASE(ISIGF0),RBASE(ISIGT0),
     >                  IBASE(MATALB),RBASE(ALBEDO),IBASE(BCRFTR),
     >                  RBASE(IQTOTL),RBASE(IDB2) ,
     >                  IBASE(IIJJ)  ,IBASE(INJJ)  ,IBASE(IIPOS),
     >                  IBASE(KEYMRG),IBASE(COND),
     >                  RBASE(IXSCAT),RBASE(IDFPOS),RBASE(IDFSUR),
     >                  RBASE(IDPFUI),ALBD)
            CALL RLSARA(RBASE(IQTOTL))
            CALL RLSARA(RBASE(ALBEDO))
            CALL RLSARA(IBASE(MATALB))
            CALL RLSARA(IBASE(KEYMRG))
            CALL RLSARA(RBASE(IDPFUI))
            CALL RLSARA(RBASE(IDFPOS))
            CALL RLSARA(RBASE(IDFSUR))
            CALL RLSARA(IBASE(COND  ))
            CALL RLSARA(IBASE(BCRFTR))
          ENDIF
        ELSE
          IFSUR=0
        ENDIF
      ENDIF
      IF(IPRINT .GE. 1) THEN
        IF(MAXOUT .EQ. 1) THEN
          EPSEIG=0.0
          EPSFLX=0.0
        ENDIF
        IF(LCONV .OR. MAXOUT .EQ. 1) THEN
          WRITE(IOUT,7000) IOUTER
        ELSE
          WRITE(IOUT,7001) IOUTER
        ENDIF
        WRITE(IOUT,6006) EPSFLX
        IF( ITYPEC .GT. 0 .AND.
     >      ITYPEC .LT. 6 ) THEN
          WRITE(IOUT,6007) EPSEIG,EIGENK(1),EIGENK(2)
          IF(ITYPEC .GT. 1) THEN
            WRITE(IOUT,6008) B2(4)
            IF(ILEAK .GE. 5) THEN
              WRITE(IOUT,6009) B2(1),B2(2),B2(3)
            ENDIF
          ENDIF
        ENDIF
      ENDIF
C-----
C GENERALIZED ADJOINT FLUX HAS CONVERGED, 
C A LAST DECONTAMINATION IS PERFORMED 
C-----
      IF(ISADJ .GE. 3 .AND. NIFISS .GE. 1) THEN
        CALL LCMSIX(IPFLUX,CDIRFL,ILCMDN)
        CALL SETARA(RBASE,NUNKNO*NGROUP,IFLUA)
        CALL FLUADN(IPFLUX,IPMACR,IPRINT,NGROUP,NBMIX ,NREGIO,
     >              NUNKNO,NIFISS,FUNKNO,MATCOD,VOLUME,KEYFLX,
     >              1,
     >              RBASE(ISIGF0),RBASE(IFFDIR),RBASE(IQFDIR),
     >              RBASE(IFLUA) )
        CALL RLSARA(RBASE(IFLUA))
        CALL LCMSIX(IPFLUX,CDIRFL,ILCMUP)
C------
C REMORMALIZE THE FLUX
C-----
      ENDIF
C----
C  RETURN TO MAIN FLUX DIRECTORY
C----
      IF(ITYPEC .GE. 3 .AND. ITYPEC .LT. 6 .AND. ILEAK .GE. 5) THEN
        CALL LCMPUT(IPFLUX,'B2  HETE',3,2,B2)
      ENDIF
      IF(ITYPEC .LT. 6) THEN
        CALL LCMPUT(IPFLUX,'1/EIGENVALUE',1,2,EIGENK)
      ENDIF
      IF(ISADJ .EQ. 0) THEN
        IF(ILEAK .EQ. 1 .OR. ILEAK .EQ. 2 ) THEN
          CALL LCMPUT(IPFLUX,'PNL         ',NGROUP,2,RBASE(IPNL))
        ENDIF
      ENDIF
      CALL LCMSIX(IPFLUX,CDIRFL,ILCMDN)
      IF(ISADJ .EQ. 0) THEN
        IF(ITYPEC .LT. 6) THEN
          CALL LCMPUT(IPFLUX,'K-EFFECTIVE ',1,2,EIGENK)
        ENDIF
      ENDIF
      IF(ISADJ .EQ. 2 .OR. ISADJ .EQ. 4) THEN
C----
C  ADJOINT IS PSEUDO ADJOINT SOURCE
C  GENERALIZED ADJOINT IS GENERALIZED PSEUDO ADJOINT SOURCE
C----
        IDIR=0
        CALL FLUASR(IPSYS ,IPMACR,IPRINT,IDIR  ,NGROUP,NBMIX ,
     >              NREGIO,NUNKNO,ILEAK ,MATCOD,KEYFLX,
     >              FUNKNO,SUNKNO,
     >              RBASE(IDB2)  ,RBASE(ISIGS0),RBASE(IXSCAT),
     >              IBASE(IIJJ)  ,IBASE(INJJ)  ,IBASE(IIPOS) )
        IF(ISADJ .EQ. 2) THEN
          CDIRFL='FLUXADJ     '
        ELSE
          CDIRFL=NAMGPA(2)
        ENDIF
        CALL LCMSIX(IPFLUX,CDIRFL,ILCMUP)
        CALL FLUSFL(IPFLUX,NGROUP,NUNKNO,SUNKNO)
        CALL LCMSIX(IPFLUX,CDIRFL,ILCMDN)
      ENDIF
C----
C IF ILEAK=1,2 SAVE THE STANDARD CP PSEUDO ADJOINT FLUX:
C TAKE THE MODIFIED CP (CPM= PIJ*PNL) PSEUDO ADJOINT  
C FLUX AND DIVIDE IT BY PNL. 
C---
      IF(ISADJ .GT. 0 .AND. (ILEAK .EQ. 1 .OR. ILEAK .EQ. 2)) THEN
        DO 300 IGR=1,NGROUP
          DO 350 IREG=1,NREGIO
            IRLOC=KEYFLX(IREG)
            FUNKNO(IRLOC,IGR)=FUNKNO(IRLOC,IGR)/RBASE(IPNL+IGR-1)
 350      CONTINUE
 300    CONTINUE
        IF(ISADJ .EQ. 1 .OR. ISADJ .EQ. 2 ) THEN
          CDIRFL='PSEUDOADJ   '
        ELSE
          CDIRFL=NAMGPA(1)
        ENDIF 
        CALL LCMSIX(IPFLUX,CDIRFL,ILCMUP)
        CALL FLUSFL(IPFLUX,NGROUP,NUNKNO,FUNKNO)
        CALL LCMSIX(IPFLUX,CDIRFL,ILCMDN)
      ENDIF 
C----
C  RELEASE MEMORY
C----
 500  CONTINUE
      IF (ISADJ .GE. 3 .AND. NIFISS .GE. 1) THEN
        CALL RLSARA(RBASE(IQFDIR))
        CALL RLSARA(RBASE(IFFDIR))
      ENDIF
      CALL RLSARA(IBASE(IFGUPS))
      CALL RLSARA(RBASE(IXSCAT))
      CALL RLSARA(RBASE(ISIGS0))
      CALL RLSARA(RBASE(ISIGTH))
      CALL RLSARA(RBASE(ISIGF0))
      CALL RLSARA(RBASE(ISIGT0))
      CALL RLSARA(IBASE(IIPOS))
      CALL RLSARA(IBASE(INJJ))
      CALL RLSARA(IBASE(IIJJ))
      CALL RLSARA(RBASE(IPNL))
      CALL RLSARA(RBASE(IDB2))
      CALL XDRSET(FUNKNO,NUNKNO*NGROUP,0.0)
      CALL LCMSIX(IPFLUX,'FLUXDIRECT  ',ILCMUP)
      CALL FLUGFL(IPFLUX,NGROUP,NUNKNO,FUNKNO)
      CALL LCMSIX(IPFLUX,'FLUXDIRECT  ',ILCMDN)
      RETURN
C----
C  FORMATS
C---
 5500 FORMAT(1X,'SCALAR PRODUCT <SOURCE,FLUX>=',1P,E11.3)
 5600 FORMAT(1X,'*** WARNING *** THE SCALAR PRODUCT <SOURCE,FLUX>',
     >       ' IS NOT ZERO',1P,E11.3,
     >       ' OR THE SOURCE NORM IS CLOSE TO ZERO',1P,E11.3)
 5700 FORMAT(/1X,'--- CONTAMINATED FLUX HAS CONVERGED ---'/)
 6000 FORMAT(
     > ' FLUX SOLUTION DOOR          =',4X,A6/
     > ' NUMBER OF GROUPS            =',I10/
     > ' NUMBER OF REGIONS           =',I10/
     > ' MAX. OUTER ITERATIONS       =',I10/
     > ' MAX. THERMAL ITERATIONS     =',I10/
     > ' ACCELERATION SCHEME         =(',I2,' FREE,',I2,' ACCELERATED)'/
     > ' REBALANCING OPTION          = ',A3/
     > ' EIGENVALUE TOLERANCE        = ',1P,E10.3/
     > ' UNKNOWN OUTER TOLERANCE     = ',E10.3/
     > ' UNKNOWN INNER TOLERANCE     = ',E10.3)
 6001 FORMAT(/' --- F I X E D   S O U R C E   P R O B L E M ---')
 6002 FORMAT(/' --- F I S S I O N    S O U R C E   ',
     > 'P R O B L E M ---'//1X,A11,' COMPUTATION  ')
 6003 FORMAT(/' --- F I S S I O N   S O U R C E   P R O B L E M ---'//
     > 1X,A11,' FIXED TO 1.0'/' INITIAL BUCKLING',10X,'=',1P,E13.5)
 6004 FORMAT(//
     >       1X,'OUTER ITERATION NUMBER                 = ',I10/
     >       1X,51('-'))
 6006 FORMAT(1X,'MAXIMUM FLUX ERROR          =',1P,E11.3)
 6007 FORMAT(1X,'MAXIMUM EIGENVALUE ERROR    =',1P,E11.3/
     >       1X,'K-EFFECTIVE                 =',0P,F10.6/
     >       1X,'K-INFINITE                  =',0P,F10.6)
 6008 FORMAT(1X,'TOTAL BUCKLING              =',1P,E13.5)
 6009 FORMAT(1X,'BUCKLING - X                =',1P,E13.5/
     >       1X,'BUCKLING - Y                =',E13.5/
     >       1X,'BUCKLING - Z                =',E13.5)
 7000 FORMAT(//1X,51('=')/
     >       1X,'CONVERGENCE REACHED AT OUTER ITERATION = ',I10)
 7001 FORMAT(//1X,75(1H=)/' *** WARNING *** CONVERGENCE NOT REACHED ',
     > ' AFTER OUTER ITERATION = ',I10)
      END
