*DECK FLUQFC
      SUBROUTINE FLUQFC(IPSYS ,IPMACR,ITPIJ ,COPTIO,IPRINT,NGROUP,
     >                  NBMIX ,NREGIO,NUNKNO,MATCOD,FUNKNO,SUNKNO,
     >                  CLBD  ,XSA   ,XSB   ,SIGTH )
C
C---------------------------  FLUQFC  ---------------------------------
C
C   1- PROGRAMME STATISTICS:
C       NAME     : FLUQFC
C       USE      : ADD CURRENT SOURCES TO SUNKNO
C       MODIFIED : 2000/02/08  (G. MARLEAU)
C                  1994/07/21  (I. PETROVIC, G. MARLEAU)
C
C   2- ROUTINE PARAMETERS:
C    INPUT:
C      IPSYS   : POINTER TO THE PIJ                   I
C      IPMACR  : POINTER TO THE MACROLIB              I
C      ITPIJ   : TYPE OF CP AVAILABLE                 I
C                = 1 SCATT MOD PIJ (WIJ)
C                = 2 STAND. PIJ
C                = 3 SCATT MOD PIJ+PIJK (WIJ,WIJK)
C                = 4 STAND. PIJ+PIJK
C      COPTIO  : CAN BE 'LKRD' (IMPOSED LEAKAGE),     C*4
C                'B0' (B-0), 'P0' (P-0), 'B1' (B-1),
C                'P1' (P-1) OR
C                'B0TR' (B-0  TRANSPORT CORRECTION).
C      IPRINT  : PRINT SELECTION FOR FLUX MODULES:    I
C                >= 100 PRINT FISSION SOURCES
C      NGROUP  : NUMBER OF GROUPS.                    I
C      NBMIX   : NUMBER OF MIXTURES.                  I
C      NREGIO  : NUMBER OF REGIONS.                   I
C      NUNKNO  : NUMBER OF UNKNOWN IN THE SYSTEM.     I
C      MATCOD  : MATERIAL CODE IN REGIONS.            I(NREGIO)
C      FUNKNO  : UNKNOWNS                             R(NUNKNO,NGROUP)
C      SUNKNO  : SOURCE FOR SYSTEM OF UNKNOWN.        R(NUNKNO,NGROUP)
C      CLBD    : LAMBDA                               R(NGROUP)
C      XSA     : ARBITRARY XS VECTOR (XSA(0)= 0.0)    R(0:NBMIX)
C      XSB     : ARBITRARY XS VECTOR (XSA(0)= 0.0)    R(0:NBMIX)
C      SIGTH   : TOTAL HOMO CROSS SECTION VECTOR      R(NGROUP)
C
C   3- INTERNAL PARAMETERS:
C      CGRPNM  : CHARACTER GROUP NAME                 C*12
C      IUNOUT  : OUTPUT UNIT                          P
C
C---------------------------   FLUQFC  --------------------------------
C
      IMPLICIT    NONE
      INTEGER     IOUT,ILCMUP,ILCMDN
      CHARACTER   NAMSBR*6
      PARAMETER  (IOUT=6,ILCMUP=1,ILCMDN=2,NAMSBR='FLUQFC')
C----
C  ROUTINE PARAMETERS
C----
      INTEGER     IPSYS,IPMACR,ITPIJ,IPRINT,NGROUP,NBMIX,
     >            NREGIO,NUNKNO
      CHARACTER   COPTIO*4
      INTEGER     MATCOD(NREGIO)
      REAL        FUNKNO(NUNKNO,NGROUP),SUNKNO(NUNKNO,NGROUP),
     >            CLBD(NGROUP),XSA(0:NBMIX),XSB(0:NBMIX),
     >            SIGTH(NGROUP)
C----
C  LOCAL PARAMETERS
C----
      CHARACTER   CGRPNM*12
      INTEGER     IGR,IREG,IBM,IFPOS1,IFPOS2,IFPOS3,ILCMLN,ILCMTY
      REAL        CCLBT,CCLBD
C-----
C     ADD CURRENT SOURCES TO SUNKNO
C-----
      DO 100 IGR=1,NGROUP
        WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGROUP
        CALL LCMSIX(IPMACR,CGRPNM,ILCMUP)
        CALL LCMGET(IPMACR,'TOTAL',XSB(1))
        CCLBT=1.0-1.0/CLBD(IGR)
        CALL LCMLEN(IPMACR,'SIGW 1',ILCMLN,ILCMTY)
        IF(ILCMLN .EQ. NBMIX) THEN
          CALL LCMGET(IPMACR,'SIGW 1',XSA(1))
          CCLBD=0.0
          IF(ITPIJ .EQ. 3 .AND. COPTIO .EQ. 'B1') THEN
            CCLBD=CCLBT
          ENDIF
C-----
C     CURRENT SOURCES
C-----
          DO 110 IREG=1,NREGIO
            IBM=MATCOD(IREG)
            IFPOS1=IREG+NREGIO
            IFPOS2=IFPOS1+NREGIO
            IFPOS3=IFPOS2+NREGIO
            SUNKNO(IFPOS1,IGR)=SUNKNO(IFPOS1,IGR)
     >        +FUNKNO(IREG,IGR)/3.0
     >        +CCLBD*XSA(IBM)*FUNKNO(IFPOS1,IGR)
     >        +CCLBT*(SIGTH(IGR)-XSB(IBM))*FUNKNO(IFPOS1,IGR)
            SUNKNO(IFPOS2,IGR)=SUNKNO(IFPOS2,IGR)
     >        +FUNKNO(IREG,IGR)/3.0
     >        +CCLBD*XSA(IBM)*FUNKNO(IFPOS2,IGR)
     >        +CCLBT*(SIGTH(IGR)-XSB(IBM))*FUNKNO(IFPOS2,IGR)
            SUNKNO(IFPOS3,IGR)=SUNKNO(IFPOS3,IGR)
     >        +FUNKNO(IREG,IGR)/3.0
     >        +CCLBD*XSA(IBM)*FUNKNO(IFPOS3,IGR)
     >        +CCLBT*(SIGTH(IGR)-XSB(IBM))*FUNKNO(IFPOS3,IGR)
 110      CONTINUE
          IF(IPRINT .GE. 100) THEN
            WRITE(IOUT,6002) IGR
            WRITE(IOUT,6001)
     >           (IREG, SUNKNO(IREG+NREGIO,IGR),IREG=1,NREGIO)
            WRITE(IOUT,6003) IGR
            WRITE(IOUT,6001)
     >           (IREG, SUNKNO(IREG+2*NREGIO,IGR),IREG=1,NREGIO)
            WRITE(IOUT,6004) IGR
            WRITE(IOUT,6001)
     >           (IREG, SUNKNO(IREG+3*NREGIO,IGR),IREG=1,NREGIO)
          ENDIF
        ELSE
          CALL LCMLIB(IPMACR)
          CALL XABORT(NAMSBR//
     >    ': SIGW 1 REQUIRED FOR TYPE -- K OR B -- CALCULATION')
        ENDIF
        CALL LCMSIX(IPMACR,CGRPNM,ILCMDN)
 100  CONTINUE
      RETURN
C-----
C     FORMATS
C-----
 6002 FORMAT(' REGIONAL X-CURRENT SOURCES IN GROUP = ',I5)
 6003 FORMAT(' REGIONAL Y-CURRENT SOURCES IN GROUP = ',I5)
 6004 FORMAT(' REGIONAL Z-CURRENT SOURCES IN GROUP = ',I5)
 6001 FORMAT(1P,5(3X,I5,2X,E15.7))
      END
