*DECK JPM2
      SUBROUTINE JPM2(ITYPE,IAPP,IQUAD,XX,YY,ZZ,NGEN,ISURF,SIGT,MASK,
     1 PII,PIS,PSS)
*
*-----------------------------------------------------------------------
*
* THIS SUBROUTINE COMPUTES THE COLLISION PROBABILITIES FOR THE
* INTERFACE CURRENT ANALYSIS OF A 1-D, 2-D OR 3-D CARTESIAN,
* CYLINDRICAL OR SPHERICAL DOMAIN.
*
* INPUT PARAMETERS:
*   ITYPE  : TYPE OF GEOMETRY.
*   IAPP   : TYPE OF ANGULAR FLUX EXPANSION BETWEEN THE BLOCKS.
*            =1: DP-0  =2: DP-1  =3: DP-1 INSIDE AND DP-0 OUTSIDE
*   IQUAD  : QUADRATURE PARAMETER.
*   XX,YY, : X-, Y- AND Z-THICKNESS OF THE GENERATING BLOCKS.
*   ZZ       DIMENSION XX(NGEN),YY(NGEN),ZZ(NGEN)
*   NGEN   : TOTAL NUMBER OF GENERATING BLOCKS.
*   ISURF  : NUMBER OF SURFACES ASSOCIATED WITH EACH GENERATING BLOCK.
*            DIMENSION ISURF(NGEN)
*   SIGT   : MACROSCOPIC CROSS SECTIONS. DIMENSION SIGT(NGEN)
*   MASK   : COLLISION PROBABILITIES ARE COMPUTED ONLY IF MASK=.TRUE.
*            DIMENSION MASK(NGEN)
*
* OUTPUT PARAMETERS:
*   PII    : VOLUME TO VOLUME REDUCED COLLISION PROBABILITIES.
*            DIMENSION PII(NGEN)
*   PIS    : VOLUME TO SURFACE COLLISION PROBABILITIES.
*            DIMENSION PIS(NPIS) WHERE NPIS=SUM(ISURF(I))
*   PSS    : SURFACE TO SURFACE COLLISION PROBABILITIES.
*            DIMENSION PSS(NPSS) WHERE NPSS=SUM(ISURF(I)**2)
*
*----------------------------------------------- AUTHOR: A. HEBERT -----
*
      PARAMETER (SMALL=1.0E-2,SQ=1.414213562)
      LOGICAL MASK(*)
      DIMENSION XX(*),YY(*),ZZ(*),ISURF(*),SIGT(*),PII(*),PIS(*),
     1 PSS(*),XGAUSS(64),WGAUSS(64),DELTA(3),PART(36),RAYRE(2),
     2 GPIS(4),GPSS(16)
*
      NPIS=0
      NPSS=0
      DO 70 IKG=1,NGEN
      NCOUR=ISURF(IKG)
      IF (MASK(IKG)) THEN
         A=XX(IKG)
         B=YY(IKG)
         C=ZZ(IKG)
         SECTI=SIGT(IKG)
         DO 10 I=1,NCOUR
   10    PIS(NPIS+I)=0.0
         DO 20 I=1,NCOUR*NCOUR
   20    PSS(NPSS+I)=0.0
         IF ((ITYPE.EQ.2).AND.(IAPP.EQ.1)) THEN
*           DP-0 APPROXIMATION.
            CALL SLAB(1,A,SECTI,SMALL,PII(IKG),PIS(NPIS+1),PSS(NPSS+1))
         ELSE IF ((ITYPE.EQ.2).AND.(IAPP.EQ.2)) THEN
*           DP-1 APPROXIMATION.
            CALL SLAB(3,A,SECTI,SMALL,PII(IKG),PIS(NPIS+1),PSS(NPSS+1))
         ELSE IF ((ITYPE.EQ.3).AND.(IAPP.EQ.1)) THEN
*           DP-0 APPROXIMATION.
            RAYRE(1)=A
            RAYRE(2)=B
            CALL TUBE1(IQUAD,RAYRE,SECTI,SMALL,PII(IKG),PIS(NPIS+1),
     1      PSS(NPSS+1))
         ELSE IF ((ITYPE.EQ.3).AND.(IAPP.EQ.3).AND.(IKG.EQ.NGEN)) THEN
*           DP-1 APPROXIMATION INSIDE / DP-0 APPROXIMATION OUTSIDE.
            RAYRE(1)=A
            RAYRE(2)=B
            IF (NGEN.EQ.1) THEN
               CALL TUBE1(IQUAD,RAYRE,SECTI,SMALL,PII(IKG),PIS(NPIS+1),
     1         PSS(NPSS+1))
            ELSE
               CALL TUBE2(IQUAD,RAYRE,SECTI,SMALL,PII(IKG),GPIS,GPSS)
               PIS(NPIS+1)=GPIS(1)
               PIS(NPIS+2)=GPIS(2)
               PIS(NPIS+3)=GPIS(3)
               PSS(NPSS+1)=GPSS(1)
               PSS(NPSS+2)=GPSS(2)
               PSS(NPSS+3)=GPSS(3)
               PSS(NPSS+4)=GPSS(5)
               PSS(NPSS+5)=GPSS(6)
               PSS(NPSS+6)=GPSS(7)
               PSS(NPSS+7)=GPSS(9)
               PSS(NPSS+8)=GPSS(10)
               PSS(NPSS+9)=GPSS(11)
            ENDIF
         ELSE IF (ITYPE.EQ.3) THEN
*           DP-1 APPROXIMATION.
            RAYRE(1)=A
            RAYRE(2)=B
            CALL TUBE2(IQUAD,RAYRE,SECTI,SMALL,PII(IKG),PIS(NPIS+1),
     1      PSS(NPSS+1))
         ELSE IF ((ITYPE.EQ.4).AND.(IAPP.EQ.1)) THEN
*           DP-0 APPROXIMATION.
            RAYRE(1)=A
            RAYRE(2)=B
            CALL SHELL1(IQUAD,RAYRE,SECTI,SMALL,PII(IKG),PIS(NPIS+1),
     1      PSS(NPSS+1))
         ELSE IF ((ITYPE.EQ.4).AND.(IAPP.EQ.3).AND.(IKG.EQ.NGEN)) THEN
*           DP-1 APPROXIMATION INSIDE / DP-0 APPROXIMATION OUTSIDE.
            RAYRE(1)=A
            RAYRE(2)=B
            IF (NGEN.EQ.1) THEN
               CALL SHELL1(IQUAD,RAYRE,SECTI,SMALL,PII(IKG),PIS(NPIS+1),
     1         PSS(NPSS+1))
            ELSE
               CALL SHELL2(IQUAD,RAYRE,SECTI,SMALL,PII(IKG),GPIS,GPSS)
               PIS(NPIS+1)=GPIS(1)
               PIS(NPIS+2)=GPIS(2)
               PIS(NPIS+3)=GPIS(3)
               PSS(NPSS+1)=GPSS(1)
               PSS(NPSS+2)=GPSS(2)
               PSS(NPSS+3)=GPSS(3)
               PSS(NPSS+4)=GPSS(5)
               PSS(NPSS+5)=GPSS(6)
               PSS(NPSS+6)=GPSS(7)
               PSS(NPSS+7)=GPSS(9)
               PSS(NPSS+8)=GPSS(10)
               PSS(NPSS+9)=GPSS(11)
            ENDIF
         ELSE IF (ITYPE.EQ.4) THEN
*           DP-1 APPROXIMATION.
            RAYRE(1)=A
            RAYRE(2)=B
            CALL SHELL2(IQUAD,RAYRE,SECTI,SMALL,PII(IKG),PIS(NPIS+1),
     1      PSS(NPSS+1))
         ELSE IF ((ITYPE.EQ.5).AND.(IAPP.EQ.1)) THEN
*           DP-0 APPROXIMATION.
            CALL ALGPT(IQUAD,-1.0,1.0,XGAUSS,WGAUSS)
            CALL RECT1 (IQUAD,A,B,SECTI,SMALL,PII(IKG),PIS(NPIS+1),
     1      PSS(NPSS+1),XGAUSS,WGAUSS)
         ELSE IF ((ITYPE.EQ.5).AND.(IAPP.EQ.2)) THEN
*           DP-1 APPROXIMATION.
            CALL ALGPT(IQUAD,-1.0,1.0,XGAUSS,WGAUSS)
            CALL RECT2 (IQUAD,A,B,SECTI,SMALL,PII(IKG),PIS(NPIS+1),
     1      PSS(NPSS+1),XGAUSS,WGAUSS)
         ELSE IF ((ITYPE.EQ.6).AND.(IAPP.EQ.1)) THEN
*           DP-0 APPROXIMATION.
            RAYRE(1)=A
            RAYRE(2)=B
            CALL TUBE3D(IQUAD,RAYRE,C,SECTI,SMALL,PII(IKG),PIS(NPIS+1),
     1      PSS(NPSS+1))
         ELSE IF ((ITYPE.EQ.7).AND.(IAPP.EQ.1)) THEN
*           DP-0 APPROXIMATION.
            DELTA(1)=A
            DELTA(2)=B
            DELTA(3)=C
            CALL ALGPT(IQUAD,0.0,1.0,XGAUSS,WGAUSS)
            CALL XELPA3(IQUAD,XGAUSS,WGAUSS,SECTI,DELTA,PART)
            PII(IKG)=PART(36)/PART(35)
            DO 50 IP=1,NCOUR
            PIS(NPIS+NCOUR-IP+1)=PART(28+IP)/PART(35)
            IOF=IP*(IP+1)/2-IP
            DO 50 JP=1,IP
            GAR=PART(IOF+JP)
            PSS(NPSS+(1+NCOUR-JP)*NCOUR+1-IP)=GAR/PART(21+IP)
   50       PSS(NPSS+(1+NCOUR-IP)*NCOUR+1-JP)=GAR/PART(21+JP)
         ELSE IF (IAPP.EQ.2) THEN
            CALL XABORT('JPM2: OPTION NOT AVAILABLE.')
         ENDIF
      ENDIF
      NPIS=NPIS+NCOUR
   70 NPSS=NPSS+NCOUR*NCOUR
      RETURN
      END
