*DECK JPM3
      SUBROUTINE JPM3(NSUPCE,IAPP,IQUAD,NMC,RAYRE,PROCEL,NGEN,ISURF,
     1 SIGT,MASK,PII,PIS,PSS)
*
*-----------------------------------------------------------------------
*
* THIS SUBROUTINE COMPUTES THE COLLISION PROBABILITIES FOR THE
* J+- ANALYSIS OF THE 'DO-IT-YOURSELF' MULTICELL OPTION.
*
* INPUT PARAMETERS:
*   NSUPCE : TOTAL NUMBER OF CELLS.
*   IAPP   : TYPE OF MULTICELL APPROXIMATION.
*   IQUAD  : QUADRATURE PARAMETER.
*   NMC    : OFFSET OF THE FIRST BLOCK IN EACH CELL.
*            DIMENSION NMC(NSUPCE+1)
*   RAYRE  : RADIUS OF THE TUBES IN EACH CELL.
*            DIMENSION RAYRE(NSUPCE+NGEN)
*   NGEN   : TOTAL NUMBER OF BLOCKS.
*   ISURF  : NUMBER OF SURFACES ASSOCIATED WITH EACH GENERATING BLOCK.
*            DIMENSION ISURF(NGEN)
*   PROCEL : USER SUPPLIED GEOMETRICAL MATRIX.
*            DIMENSION PROCEL(NSUPCE**2)
*   SIGT   : MACROSCOPIC CROSS SECTIONS. DIMENSION SIGT(NGEN)
*   MASK   : COLLISION PROBABILITIES ARE COMPUTED ONLY IF MASK=.TRUE.
*            DIMENSION MASK(NGEN)
*
* OUTPUT PARAMETERS:
*   PII    : VOLUME TO VOLUME REDUCED COLLISION PROBABILITIES.
*            DIMENSION PII(NGEN)
*   PIS    : VOLUME TO SURFACE COLLISION PROBABILITIES.
*            DIMENSION PIS(NPIS) WHERE NPIS=SUM(ISURF(I))
*   PSS    : SURFACE TO SURFACE COLLISION PROBABILITIES.
*            DIMENSION PSS(NPSS) WHERE NPSS=SUM(ISURF(I)**2)
*
*-------------------------------------- AUTHOR : A. HEBERT 13/02/87 ----
*
      PARAMETER (SMALL=1.0E-2)
      LOGICAL MASK(*)
      DIMENSION IQUAD(*),NMC(*),RAYRE(*),PROCEL(NSUPCE,*),ISURF(*),
     1 SIGT(*),PII(*),PIS(*),PSS(*),PVBZ(4),PBBZ(16)
*
      NPIS=0
      NPSS=0
      I1=0
      DO 120 IKK=1,NSUPCE
      I2=NMC(IKK+1)-NMC(IKK)
      DO 110 IP=1,I2
      IBLK=I1+IP
      NCOUR=ISURF(IBLK)
      IF (MASK(IBLK)) THEN
         SECTI=SIGT(IBLK)
         IF (IAPP.EQ.1) THEN
*           DP-0 REPRESENTATION.
            CALL TUBE1(IQUAD,RAYRE(IKK+IBLK-1),SECTI,SMALL,PVV,PVBZ,
     1      PBBZ)
            PII(IBLK)=PVV
            IF (IP.LT.I2) THEN
               DO 10 I=1,NCOUR
               PIS(NPIS+I)=PVBZ(I)
               DO 10 J=1,NCOUR
   10          PSS(NPSS+(J-1)*NCOUR+I)=PBBZ((J-1)*NCOUR+I)
            ELSE IF (I2.EQ.1) THEN
               DO 30 I=1,NCOUR
               PIS(NPIS+I)=PVBZ(1)*PROCEL(IKK,I)
               DO 30 J=1,NCOUR
   30          PSS(NPSS+(J-1)*NCOUR+I)=PBBZ(1)*PROCEL(IKK,J)
            ELSE IF (IP.EQ.I2) THEN
               PIS(NPIS+1)=PVBZ(1)
               PSS(NPSS+1)=PBBZ(1)
               DO 50 I=2,NCOUR
               PIS(NPIS+I)=PVBZ(2)*PROCEL(IKK,I-1)
               PSS(NPSS+I)=PBBZ(2)
               PSS(NPSS+(I-1)*NCOUR+1)=PBBZ(3)*PROCEL(IKK,I-1)
               DO 50 J=2,NCOUR
   50          PSS(NPSS+(J-1)*NCOUR+I)=PBBZ(4)*PROCEL(IKK,J-1)
            ENDIF
         ELSE IF (IAPP.EQ.2) THEN
*           DP-1 REPRESENTATION.
            CALL TUBE2(IQUAD,RAYRE(IKK+IBLK-1),SECTI,SMALL,PVV,PVBZ,
     1      PBBZ)
            PII(IBLK)=PVV
            IF (IP.LT.I2) THEN
               DO 60 I=1,NCOUR
               PIS(NPIS+I)=PVBZ(I)
               DO 60 J=1,NCOUR
   60          PSS(NPSS+(J-1)*NCOUR+I)=PBBZ((J-1)*NCOUR+I)
            ELSE IF (I2.EQ.1) THEN
               DO 80 I=1,NCOUR
               PIS(NPIS+I)=PVBZ(1)*PROCEL(IKK,I)
               DO 80 J=1,NCOUR
   80          PSS(NPSS+(J-1)*NCOUR+I)=PBBZ(1)*PROCEL(IKK,J)
            ELSE IF (IP.EQ.I2) THEN
               PIS(NPIS+1)=PVBZ(1)
               PIS(NPIS+2)=PVBZ(2)
               PSS(NPSS+1)=PBBZ(1)
               PSS(NPSS+2)=PBBZ(2)
               PSS(NPSS+NCOUR+1)=PBBZ(5)
               PSS(NPSS+NCOUR+2)=PBBZ(6)
               DO 100 I=3,NCOUR
               PIS(NPIS+I)=PVBZ(3)*PROCEL(IKK,I-2)
               PSS(NPSS+I)=PBBZ(3)
               PSS(NPSS+NCOUR+I)=PBBZ(7)
               PSS(NPSS+(I-1)*NCOUR+1)=PBBZ(9)*PROCEL(IKK,I-2)
               PSS(NPSS+(I-1)*NCOUR+2)=PBBZ(10)*PROCEL(IKK,I-2)
               DO 100 J=3,NCOUR
  100          PSS(NPSS+(J-1)*NCOUR+I)=PBBZ(11)*PROCEL(IKK,J-2)
            ENDIF
         ENDIF
      ENDIF
      NPIS=NPIS+NCOUR
  110 NPSS=NPSS+NCOUR*NCOUR
  120 I1=I1+I2
      RETURN
      END
