*DECK JPM4
      SUBROUTINE JPM4(IWIGN,LMAIL,IQUAD,ITGEN,COORD,MAIL,ZMAILI,ZMAILR,
     1 NGEN,ISURF,SIGT,MASK,PII,PIS,PSS)
*
*-----------------------------------------------------------------------
*
* THIS SUBROUTINE COMPUTES THE COLLISION PROBABILITIES FOR THE
* INTERFACE CURRENT ANALYSIS OF AN ASSEMBLY OF HOMOGENEOUS OR
* HETEROGENEOUS CELLS.
*
* INPUT PARAMETERS:
*   IWIGN  : TYPE OF CYLINDERIZATION.
*   LMAIL  : SPACE REQUIRED TO STORE THE TRACKING INFORMATION.
*   IQUAD  : QUADRATURE PARAMETERS. DIMENSION IQUAD(4)
*   ITGEN  : TYPE OF EACH GENERATING BLOCK:
*            =1 : TUBE/DP-0 INSIDE/DP-0 OUTSIDE/
*            =2 : TUBE/DP-1 INSIDE/DP-1 OUTSIDE/
*            =3 : RECTANGLE/DP-0 OUTSIDE/
*            =4 : RECTANGLE/DP-1 OUTSIDE/
*            =5 : RECTANGULAR CROWN/DP-0 INSIDE/ROTH OUTSIDE/
*            =6 : RECTANGULAR CROWN/DP-0 INSIDE/ROTH X 4 OUTSIDE/
*            =7 : RECTANGULAR CROWN/DP-1 INSIDE/ROTH OUTSIDE/
*            =8 : RECTANGULAR CROWN/DP-1 INSIDE/ROTH X 4 OUTSIDE/
*            =9 : RECTANGULAR CROWN/DP-0 INSIDE/DP-0 OUTSIDE/
*            =10: RECTANGULAR CROWN/DP-1 INSIDE/DP-0 OUTSIDE/
*            =11: RECTANGULAR CROWN/DP-0 INSIDE/DP-1 OUTSIDE/
*            =12: RECTANGULAR CROWN/DP-1 INSIDE/DP-1 OUTSIDE/
*            =13: HEXAGON/DP-0 OUTSIDE/
*            =14: HEXAGON/DP-1 OUTSIDE/
*            =15: HEXAGONAL CROWN/DP-0 INSIDE/ROTH OUTSIDE/
*            =16: HEXAGONAL CROWN/DP-0 INSIDE/ROTH X 6 OUTSIDE/
*            =17: HEXAGONAL CROWN/DP-1 INSIDE/ROTH OUTSIDE/
*            =18: HEXAGONAL CROWN/DP-1 INSIDE/ROTH X 6 OUTSIDE/
*            =19: HEXAGONAL CROWN/DP-0 INSIDE/DP-0 OUTSIDE/
*            =20: HEXAGONAL CROWN/DP-1 INSIDE/DP-0 OUTSIDE/
*            =21: HEXAGONAL CROWN/DP-0 INSIDE/DP-1 OUTSIDE/
*            =22: HEXAGONAL CROWN/DP-1 INSIDE/DP-1 OUTSIDE/
*            DIMENSION ITGEN(NGEN)
*   COORD  : DIMENSIONS OF EACH GENERATING BLOCK.
*            DIMENSION COORD(3,NGEN)
*   MAIL   : OFFSET OF THE FIRST TRACKING INFORMATION IN EACH GENERATING
*            BLOCK. DIMENSION MAIL(NGEN)
*   ZMAILI : INTEGER TRACKING INFORMATION. DIMENSION ZMAILI(LMAIL)
*   ZMAILR : REAL TRACKING INFORMATION. DIMENSION ZMAILR(LMAIL)
*   NGEN   : TOTAL NUMBER OF GENERATING BLOCKS.
*   ISURF  : NUMBER OF SURFACES ASSOCIATED WITH EACH GENERATING BLOCK.
*            DIMENSION ISURF(NGEN)
*   SIGT   : MACROSCOPIC CROSS SECTIONS. DIMENSION SIGT(NGEN)
*   MASK   : COLLISION PROBABILITIES ARE COMPUTED ONLY IF MASK=.TRUE.
*            DIMENSION MASK(NGEN)
*
* OUTPUT PARAMETERS:
*   PII    : VOLUME TO VOLUME REDUCED COLLISION PROBABILITIES.
*            DIMENSION PII(NGEN)
*   PIS    : VOLUME TO SURFACE COLLISION PROBABILITIES.
*            DIMENSION PIS(NCOUR*NGEN) WHERE NCOUR=4 IF IAPP1.LE.3
*                                            NCOUR=12 IF IAPP1=4
*   PSS    : SURFACE TO SURFACE COLLISION PROBABILITIES.
*            DIMENSION PSS(NCOUR*NCOUR*NGEN)
*
*----------------------------------------------- AUTHOR: A. HEBERT -----
*
      PARAMETER (PI=3.141592654,SMALL=1.0E-2)
      LOGICAL MASK(*)
      DIMENSION IQUAD(*),ITGEN(*),COORD(3,*),MAIL(*),ZMAILI(*),
     1 ZMAILR(*),ISURF(*),SIGT(*),PII(*),PIS(*),PSS(*),RAD(2),GPIS(4),
     2 GPSS(16),SIDE(4),PBB(6),ISLR(4,4),ISLH(6,6),XGAUSS(64),WGAUSS(64)
      SAVE ISLR,ISLH
      DATA ISLR/6,4,1,1,4,6,1,1,3,3,5,2,3,3,2,5/
      DATA ISLH/4,1,2,3,2,1,1,4,1,2,3,2,2,1,4,1,2,3,
     1          3,2,1,4,1,2,2,3,2,1,4,1,1,2,3,2,1,4/
*
      NPIS=0
      NPSS=0
      DO 170 IKG=1,NGEN
      NCOUR=ISURF(IKG)
      IF (MASK(IKG)) THEN
         MAIKG=MAIL(IKG)
         SECTI=SIGT(IKG)
         A=COORD(2,IKG)
         B=COORD(3,IKG)
         IF ((ITGEN(IKG).GE.5).AND.(ITGEN(IKG).LE.8)) THEN
            RAD(1)=COORD(1,IKG)
            IF (IWIGN.EQ.1) THEN
*              ASKEW CYLINDERIZATION.
               RAD(2)=(A+B)/PI
               SCALE1=(A*B-PI*RAD(1)**2)/(PI*RAD(2)**2-PI*RAD(1)**2)
               SCALE2=1.0
               SECTI=SECTI*SCALE1
            ELSE IF (IWIGN.EQ.2) THEN
*              WIGNER CYLINDERIZATION.
               RAD(2)=SQRT(A*B/PI)
               SCALE1=1.0
               SCALE2=1.0
            ELSE IF (IWIGN.EQ.3) THEN
*              SANCHEZ CYLINDERIZATION.
               RAD(2)=SQRT(A*B/PI)
               SCALE1=1.0
               SCALE2=SQRT(PI*A*B)/(A+B)
            ENDIF
            IF (RAD(2).LE.RAD(1)) CALL XABORT('JPM4: CYLINDERIZATION '
     1      //'ERROR(1).')
            SIDE(1)=B
            SIDE(2)=B
            SIDE(3)=A
            SIDE(4)=A
         ELSE IF ((ITGEN(IKG).GE.15).AND.(ITGEN(IKG).LE.18)) THEN
            RAD(1)=COORD(1,IKG)
            VVV=1.5*SQRT(3.0)*A*A
            IF (IWIGN.EQ.1) THEN
*              ASKEW CYLINDERIZATION.
               RAD(2)=3.0*A/PI
               SCALE1=(VVV-PI*RAD(1)**2)/(PI*RAD(2)**2-PI*RAD(1)**2)
               SCALE2=1.0
               SECTI=SECTI*SCALE1
            ELSE IF (IWIGN.EQ.2) THEN
*              WIGNER CYLINDERIZATION.
               RAD(2)=SQRT(VVV/PI)
               SCALE1=1.0
               SCALE2=1.0
            ELSE IF (IWIGN.EQ.3) THEN
*              SANCHEZ CYLINDERIZATION.
               RAD(2)=SQRT(VVV/PI)
               SCALE1=1.0
               SCALE2=SQRT(PI*SQRT(3.0)/6.0)
            ENDIF
            IF (RAD(2).LE.RAD(1)) CALL XABORT('JPM4: CYLINDERIZATION '
     1      //'ERROR(2).')
         ENDIF
         IF (ITGEN(IKG).EQ.1) THEN
*           TUBE/DP-0 INSIDE/DP-0 OUTSIDE/
            CALL TUBE1(IQUAD(4),COORD(1,IKG),SECTI,SMALL,PII(IKG),
     1      PIS(NPIS+1),PSS(NPSS+1))
         ELSE IF (ITGEN(IKG).EQ.2) THEN
*           TUBE/DP-1 INSIDE/DP-1 OUTSIDE/
            CALL TUBE2(IQUAD(4),COORD(1,IKG),SECTI,SMALL,PII(IKG),
     1      PIS(NPIS+1),PSS(NPSS+1))
         ELSE IF (ITGEN(IKG).EQ.3) THEN
*           RECTANGLE/DP-0 OUTSIDE/
            CALL ALGPT(IQUAD(3),-1.0,1.0,XGAUSS,WGAUSS)
            CALL RECT1 (IQUAD(3),A,B,SECTI,SMALL,PII(IKG),PIS(NPIS+1),
     1      PSS(NPSS+1),XGAUSS,WGAUSS)
         ELSE IF (ITGEN(IKG).EQ.4) THEN
*           RECTANGLE/DP-1 OUTSIDE/
            CALL ALGPT(IQUAD(3),-1.0,1.0,XGAUSS,WGAUSS)
            CALL RECT2 (IQUAD(3),A,B,SECTI,SMALL,PII(IKG),PIS(NPIS+1),
     1      PSS(NPSS+1),XGAUSS,WGAUSS)
         ELSE IF (ITGEN(IKG).EQ.5) THEN
*           RECTANGULAR CELL/DP-0 INSIDE/ROTH OUTSIDE/
            CALL TUBE1(IQUAD(4),RAD,SECTI,SMALL,PVV,GPIS,GPSS)
            PII(IKG)=PVV*SCALE1
            DEN1=2.0*(A+B)
            PBB(1)=B/DEN1
            PBB(2)=A/DEN1
            PBB(3)=A/DEN1
            PBB(4)=B/DEN1
            PBB(5)=A/DEN1
            PBB(6)=B/DEN1
            IF (NCOUR.EQ.4) THEN
               GPSS(1)=1.0-SCALE2*(1.0-GPSS(1))
               DO 10 IC=1,NCOUR
               PIS(NPIS+IC)=GPIS(1)*SIDE(IC)/DEN1
               DO 10 JC=1,NCOUR
               IB=ISLR(IC,JC)
   10          PSS(NPSS+(JC-1)*NCOUR+IC)=GPSS(1)*PBB(IB)
            ELSE IF (NCOUR.EQ.5) THEN
               GPSS(2)=SCALE2*GPSS(2)
               GPSS(4)=1.0-SCALE2*(1.0-GPSS(4))
               PIS(NPIS+1)=GPIS(1)
               PSS(NPSS+1)=GPSS(1)
               DO 20 IC=1,4
               PIS(NPIS+1+IC)=GPIS(2)*SIDE(IC)/DEN1
               PSS(NPSS+1+IC)=GPSS(2)
               PSS(NPSS+IC*NCOUR+1)=GPSS(3)*SIDE(IC)/DEN1
               DO 20 JC=1,4
               IB=ISLR(IC,JC)
   20          PSS(NPSS+JC*NCOUR+1+IC)=GPSS(4)*PBB(IB)
            ENDIF
         ELSE IF (ITGEN(IKG).EQ.6) THEN
*           RECTANGULAR CELL/DP-0 INSIDE/ROTH X 4 OUTSIDE/
            CALL TUBE1(IQUAD(4),RAD,SECTI,SMALL,PVV,GPIS,GPSS)
            PII(IKG)=PVV*SCALE1
            DEN1=MAX(2.0*B+A,2.0*A+B)
            PBB(1)=B/DEN1
            PBB(2)=MAX(A,2.0*A-B)/DEN1
            PBB(3)=A/DEN1
            PBB(4)=MAX(B,2.0*B-A)/DEN1
            PBB(5)=0.0
            PBB(6)=0.0
            DEN1=2.0*(A+B)
            IF (NCOUR.EQ.4) THEN
               GPSS(1)=1.0-SCALE2*(1.0-GPSS(1))
               DO 30 IC=1,NCOUR
               PIS(NPIS+IC)=GPIS(1)*SIDE(IC)/DEN1
               DO 30 JC=1,NCOUR
               IB=ISLR(IC,JC)
   30          PSS(NPSS+(JC-1)*NCOUR+IC)=GPSS(1)*PBB(IB)
            ELSE IF (NCOUR.EQ.5) THEN
               GPSS(2)=SCALE2*GPSS(2)
               GPSS(4)=1.0-SCALE2*(1.0-GPSS(4))
               PIS(NPIS+1)=GPIS(1)
               PSS(NPSS+1)=GPSS(1)
               DO 40 IC=1,4
               PIS(NPIS+1+IC)=GPIS(2)*SIDE(IC)/DEN1
               PSS(NPSS+1+IC)=GPSS(2)
               PSS(NPSS+IC*NCOUR+1)=GPSS(3)*SIDE(IC)/DEN1
               DO 40 JC=1,4
               IB=ISLR(IC,JC)
   40          PSS(NPSS+JC*NCOUR+1+IC)=GPSS(4)*PBB(IB)
            ENDIF
         ELSE IF (ITGEN(IKG).EQ.7) THEN
*           RECTANGULAR CELL/DP-1 INSIDE/ROTH OUTSIDE/
            CALL TUBE2(IQUAD(4),RAD,SECTI,SMALL,PVV,GPIS,GPSS)
            PII(IKG)=PVV*SCALE1
            DEN1=2.0*(A+B)
            PBB(1)=B/DEN1
            PBB(2)=A/DEN1
            PBB(3)=A/DEN1
            PBB(4)=B/DEN1
            PBB(5)=A/DEN1
            PBB(6)=B/DEN1
            IF (NCOUR.EQ.4) THEN
               GPSS(1)=1.0-SCALE2*(1.0-GPSS(1))
               DO 50 IC=1,NCOUR
               PIS(NPIS+IC)=GPIS(1)*SIDE(IC)/DEN1
               DO 50 JC=1,NCOUR
               IB=ISLR(IC,JC)
   50          PSS(NPSS+(JC-1)*NCOUR+IC)=GPSS(1)*PBB(IB)
            ELSE IF (NCOUR.EQ.6) THEN
               GPSS(3)=SCALE2*GPSS(3)
               GPSS(7)=SCALE2*GPSS(7)
               GPSS(11)=1.0-SCALE2*(1.0-GPSS(11))
               PIS(NPIS+1)=GPIS(1)
               PIS(NPIS+2)=GPIS(2)
               PSS(NPSS+1)=GPSS(1)
               PSS(NPSS+2)=GPSS(2)
               PSS(NPSS+NCOUR+1)=GPSS(5)
               PSS(NPSS+NCOUR+2)=GPSS(6)
               DO 60 IC=1,4
               PIS(NPIS+2+IC)=GPIS(3)*SIDE(IC)/DEN1
               PSS(NPSS+2+IC)=GPSS(3)
               PSS(NPSS+NCOUR+2+IC)=GPSS(7)
               PSS(NPSS+(IC+1)*NCOUR+1)=GPSS(9)*SIDE(IC)/DEN1
               PSS(NPSS+(IC+1)*NCOUR+2)=GPSS(10)*SIDE(IC)/DEN1
               DO 60 JC=1,4
               IB=ISLR(IC,JC)
   60          PSS(NPSS+(JC+1)*NCOUR+2+IC)=GPSS(11)*PBB(IB)
            ENDIF
         ELSE IF (ITGEN(IKG).EQ.8) THEN
*           RECTANGULAR CELL/DP-1 INSIDE/ROTH X 4 OUTSIDE/
            CALL TUBE2(IQUAD(4),RAD,SECTI,SMALL,PVV,GPIS,GPSS)
            PII(IKG)=PVV*SCALE1
            DEN1=MAX(2.0*B+A,2.0*A+B)
            PBB(1)=B/DEN1
            PBB(2)=MAX(A,2.0*A-B)/DEN1
            PBB(3)=A/DEN1
            PBB(4)=MAX(B,2.0*B-A)/DEN1
            PBB(5)=0.0
            PBB(6)=0.0
            DEN1=2.0*(A+B)
            IF (NCOUR.EQ.4) THEN
               GPSS(1)=1.0-SCALE2*(1.0-GPSS(1))
               DO 70 IC=1,NCOUR
               PIS(NPIS+IC)=GPIS(1)*SIDE(IC)/DEN1
               DO 70 JC=1,NCOUR
               IB=ISLR(IC,JC)
   70          PSS(NPSS+(JC-1)*NCOUR+IC)=GPSS(1)*PBB(IB)
            ELSE IF (NCOUR.EQ.6) THEN
               GPSS(3)=SCALE2*GPSS(3)
               GPSS(7)=SCALE2*GPSS(7)
               GPSS(11)=1.0-SCALE2*(1.0-GPSS(11))
               PIS(NPIS+1)=GPIS(1)
               PIS(NPIS+2)=GPIS(2)
               PSS(NPSS+1)=GPSS(1)
               PSS(NPSS+2)=GPSS(2)
               PSS(NPSS+NCOUR+1)=GPSS(5)
               PSS(NPSS+NCOUR+2)=GPSS(6)
               DO 80 IC=1,4
               PIS(NPIS+2+IC)=GPIS(3)*SIDE(IC)/DEN1
               PSS(NPSS+2+IC)=GPSS(3)
               PSS(NPSS+NCOUR+2+IC)=GPSS(7)
               PSS(NPSS+(IC+1)*NCOUR+1)=GPSS(9)*SIDE(IC)/DEN1
               PSS(NPSS+(IC+1)*NCOUR+2)=GPSS(10)*SIDE(IC)/DEN1
               DO 80 JC=1,4
               IB=ISLR(IC,JC)
   80          PSS(NPSS+(JC+1)*NCOUR+2+IC)=GPSS(11)*PBB(IB)
            ENDIF
         ELSE IF (ITGEN(IKG).EQ.9) THEN
*          RECTANGULAR CROWN/DP-0 INSIDE/DP-0 OUTSIDE/
           CALL CROWR1(1,COORD(1,IKG),ZMAILR(MAIKG),ZMAILI(MAIKG),
     1     SECTI,SMALL,PII(IKG),PIS(NPIS+1),PSS(NPSS+1))
         ELSE IF (ITGEN(IKG).EQ.10) THEN
*          RECTANGULAR CROWN/DP-1 INSIDE/DP-0 OUTSIDE/
           CALL CROWR1(2,COORD(1,IKG),ZMAILR(MAIKG),ZMAILI(MAIKG),
     1     SECTI,SMALL,PII(IKG),PIS(NPIS+1),PSS(NPSS+1))
         ELSE IF (ITGEN(IKG).EQ.11) THEN
*          RECTANGULAR CROWN/DP-0 INSIDE/DP-1 OUTSIDE/
           CALL CROWR2(1,COORD(1,IKG),ZMAILR(MAIKG),ZMAILI(MAIKG),
     1     SECTI,SMALL,PII(IKG),PIS(NPIS+1),PSS(NPSS+1))
         ELSE IF (ITGEN(IKG).EQ.12) THEN
*          RECTANGULAR CROWN/DP-1 INSIDE/DP-1 OUTSIDE/
           CALL CROWR2(2,COORD(1,IKG),ZMAILR(MAIKG),ZMAILI(MAIKG),
     1     SECTI,SMALL,PII(IKG),PIS(NPIS+1),PSS(NPSS+1))
         ELSE IF (ITGEN(IKG).EQ.13) THEN
*           HEXAGON/DP-0 OUTSIDE/
            CALL ALGPT(IQUAD(3),-1.0,1.0,XGAUSS,WGAUSS)
            CALL XHX2D0(IQUAD(3),XGAUSS,WGAUSS,A,SECTI,SMALL,PII(IKG),
     1      PIS(NPIS+1),PSS(NPSS+1))
         ELSE IF (ITGEN(IKG).EQ.14) THEN
*           HEXAGON/DP-1 OUTSIDE/
            CALL ALGPT(IQUAD(3),-1.0,1.0,XGAUSS,WGAUSS)
            CALL XHX2D1(IQUAD(3),XGAUSS,WGAUSS,A,SECTI,SMALL,PII(IKG),
     1      PIS(NPIS+1),PSS(NPSS+1))
         ELSE IF (ITGEN(IKG).EQ.15) THEN
*           HEXAGONAL CELL/DP-0 INSIDE/ROTH OUTSIDE/
            CALL TUBE1(IQUAD(4),RAD,SECTI,SMALL,PVV,GPIS,GPSS)
            PII(IKG)=PVV*SCALE1
            IF (NCOUR.EQ.6) THEN
               GPSS(1)=1.0-SCALE2*(1.0-GPSS(1))
               DO 90 IC=1,NCOUR
               PIS(NPIS+IC)=GPIS(1)/6.0
               DO 90 JC=1,NCOUR
   90          PSS(NPSS+(JC-1)*NCOUR+IC)=GPSS(1)/6.0
            ELSE IF (NCOUR.EQ.7) THEN
               GPSS(2)=SCALE2*GPSS(2)
               GPSS(4)=1.0-SCALE2*(1.0-GPSS(4))
               PIS(NPIS+1)=GPIS(1)
               PSS(NPSS+1)=GPSS(1)
               DO 100 IC=1,6
               PIS(NPIS+1+IC)=GPIS(2)/6.0
               PSS(NPSS+1+IC)=GPSS(2)
               PSS(NPSS+IC*NCOUR+1)=GPSS(3)/6.0
               DO 100 JC=1,6
  100          PSS(NPSS+JC*NCOUR+1+IC)=GPSS(4)/6.0
            ENDIF
         ELSE IF (ITGEN(IKG).EQ.16) THEN
*           HEXAGONAL CELL/DP-0 INSIDE/ROTH X 6 OUTSIDE/
            CALL TUBE1(IQUAD(4),RAD,SECTI,SMALL,PVV,GPIS,GPSS)
            PII(IKG)=PVV*SCALE1
            PBB(1)=1.0/5.0
            PBB(2)=1.0/5.0
            PBB(3)=1.0/5.0
            PBB(4)=0.0
            IF (NCOUR.EQ.6) THEN
               GPSS(1)=1.0-SCALE2*(1.0-GPSS(1))
               DO 110 IC=1,NCOUR
               PIS(NPIS+IC)=GPIS(1)/6.0
               DO 110 JC=1,NCOUR
               IB=ISLH(IC,JC)
  110          PSS(NPSS+(JC-1)*NCOUR+IC)=GPSS(1)*PBB(IB)
            ELSE IF (NCOUR.EQ.7) THEN
               GPSS(2)=SCALE2*GPSS(2)
               GPSS(4)=1.0-SCALE2*(1.0-GPSS(4))
               PIS(NPIS+1)=GPIS(1)
               PSS(NPSS+1)=GPSS(1)
               DO 120 IC=1,6
               PIS(NPIS+1+IC)=GPIS(2)/6.0
               PSS(NPSS+1+IC)=GPSS(2)
               PSS(NPSS+IC*NCOUR+1)=GPSS(3)/6.0
               DO 120 JC=1,6
               IB=ISLH(IC,JC)
  120          PSS(NPSS+JC*NCOUR+1+IC)=GPSS(4)*PBB(IB)
            ENDIF
         ELSE IF (ITGEN(IKG).EQ.17) THEN
*           HEXAGONAL CELL/DP-1 INSIDE/ROTH OUTSIDE/
            CALL TUBE2(IQUAD(4),RAD,SECTI,SMALL,PVV,GPIS,GPSS)
            PII(IKG)=PVV*SCALE1
            IF (NCOUR.EQ.6) THEN
               GPSS(1)=1.0-SCALE2*(1.0-GPSS(1))
               DO 130 IC=1,NCOUR
               PIS(NPIS+IC)=GPIS(1)/6.0
               DO 130 JC=1,NCOUR
  130          PSS(NPSS+(JC-1)*NCOUR+IC)=GPSS(1)/6.0
            ELSE IF (NCOUR.EQ.8) THEN
               GPSS(3)=SCALE2*GPSS(3)
               GPSS(7)=SCALE2*GPSS(7)
               GPSS(11)=1.0-SCALE2*(1.0-GPSS(11))
               PIS(NPIS+1)=GPIS(1)
               PIS(NPIS+2)=GPIS(2)
               PSS(NPSS+1)=GPSS(1)
               PSS(NPSS+2)=GPSS(2)
               PSS(NPSS+NCOUR+1)=GPSS(5)
               PSS(NPSS+NCOUR+2)=GPSS(6)
               DO 140 IC=1,6
               PIS(NPIS+2+IC)=GPIS(3)/6.0
               PSS(NPSS+2+IC)=GPSS(3)
               PSS(NPSS+NCOUR+2+IC)=GPSS(7)
               PSS(NPSS+(IC+1)*NCOUR+1)=GPSS(9)/6.0
               PSS(NPSS+(IC+1)*NCOUR+2)=GPSS(10)/6.0
               DO 140 JC=1,6
  140          PSS(NPSS+(JC+1)*NCOUR+2+IC)=GPSS(11)/6.0
            ENDIF
         ELSE IF (ITGEN(IKG).EQ.18) THEN
*           HEXAGONAL CELL/DP-1 INSIDE/ROTH X 6 OUTSIDE/
            CALL TUBE2(IQUAD(4),RAD,SECTI,SMALL,PVV,GPIS,GPSS)
            PII(IKG)=PVV*SCALE1
            PBB(1)=1.0/5.0
            PBB(2)=1.0/5.0
            PBB(3)=1.0/5.0
            PBB(4)=0.0
            IF (NCOUR.EQ.6) THEN
               GPSS(1)=1.0-SCALE2*(1.0-GPSS(1))
               DO 150 IC=1,NCOUR
               PIS(NPIS+IC)=GPIS(1)/6.0
               DO 150 JC=1,NCOUR
               IB=ISLH(IC,JC)
  150          PSS(NPSS+(JC-1)*NCOUR+IC)=GPSS(1)*PBB(IB)
            ELSE IF (NCOUR.EQ.8) THEN
               GPSS(3)=SCALE2*GPSS(3)
               GPSS(7)=SCALE2*GPSS(7)
               GPSS(11)=1.0-SCALE2*(1.0-GPSS(11))
               PIS(NPIS+1)=GPIS(1)
               PIS(NPIS+2)=GPIS(2)
               PSS(NPSS+1)=GPSS(1)
               PSS(NPSS+2)=GPSS(2)
               PSS(NPSS+NCOUR+1)=GPSS(5)
               PSS(NPSS+NCOUR+2)=GPSS(6)
               DO 160 IC=1,6
               PIS(NPIS+2+IC)=GPIS(3)/6.0
               PSS(NPSS+2+IC)=GPSS(3)
               PSS(NPSS+NCOUR+2+IC)=GPSS(7)
               PSS(NPSS+(IC+1)*NCOUR+1)=GPSS(9)/6.0
               PSS(NPSS+(IC+1)*NCOUR+2)=GPSS(10)/6.0
               DO 160 JC=1,6
               IB=ISLH(IC,JC)
  160          PSS(NPSS+(JC+1)*NCOUR+2+IC)=GPSS(11)*PBB(IB)
            ENDIF
         ELSE IF (ITGEN(IKG).EQ.19) THEN
*          HEXAGONAL CROWN/DP-0 INSIDE/DP-0 OUTSIDE/
           LZI=3*IQUAD(1)+2
           CALL CROWH1(ZMAILR(MAIKG+LZI),ZMAILI(MAIKG),
     1     COORD(2,IKG),SECTI,COORD(1,IKG),1,SMALL,PIS(NPIS+1),
     2     PSS(NPSS+1),PII(IKG))
         ELSE IF (ITGEN(IKG).EQ.20) THEN
*          HEXAGONAL CROWN/DP-1 INSIDE/DP-0 OUTSIDE/
           LZI=3*IQUAD(1)+2
           CALL CROWH1(ZMAILR(MAIKG+LZI),ZMAILI(MAIKG),
     1     COORD(2,IKG),SECTI,COORD(1,IKG),2,SMALL,PIS(NPIS+1),
     2     PSS(NPSS+1),PII(IKG))
         ELSE IF (ITGEN(IKG).EQ.21) THEN
*          HEXAGONAL CROWN/DP-0 INSIDE/DP-1 OUTSIDE/
           LZI=3*IQUAD(1)+2
           CALL CROWH2(ZMAILR(MAIKG+LZI),ZMAILI(MAIKG),
     1     COORD(2,IKG),SECTI,COORD(1,IKG),1,SMALL,PIS(NPIS+1),
     2     PSS(NPSS+1),PII(IKG))
         ELSE IF (ITGEN(IKG).EQ.22) THEN
*          HEXAGONAL CROWN/DP-1 INSIDE/DP-1 OUTSIDE/
           LZI=3*IQUAD(1)+2
           CALL CROWH2(ZMAILR(MAIKG+LZI),ZMAILI(MAIKG),
     1     COORD(2,IKG),SECTI,COORD(1,IKG),2,SMALL,PIS(NPIS+1),
     2     PSS(NPSS+1),PII(IKG))
         ELSE
           CALL XABORT('JPM4: OPTION NOT AVAILABLE.')
         ENDIF
      ENDIF
      NPIS=NPIS+NCOUR
  170 NPSS=NPSS+NCOUR*NCOUR
      RETURN
      END
