*DECK JPMA
      SUBROUTINE JPMA (IPSYS ,IPTRK ,IFTRAK,IMPX  ,LR2   ,NMERGE,
     >                 NBMIX ,NANI  ,MAT   ,VOL   ,KNORM ,SIGT0 ,
     1                 SIGS0 ,ILK   ,TITR  )
*
*-----------------------------------------------------------------------
*
* CALCULATION OF BLOCKWISE COLLISION PROBABILITIES, CALCULATION AND
* LU FACTORIZATION OF MATRIX (I-WSS*A) FOR JPM.
*
* GENERAL PARAMETERS:
*   IPSYS  : POINTER TO THE PIJ (L_PIJ SIGNATURE).
*   IPTRK  : POINTER TO THE TRACKING (L_TRACK SIGNATURE).
*   IFTRAK : FILE UNIT NUMBER USED TO STORE CLUSTER TRACKS.
*   IMPX   : PRINT FLAG. EQUAL TO ZERO FOR NO PRINT.
*   LR2    : =.TRUE. TO STORE INFORMATION REQUIRED TO COMPUTE THE L2
*            NORM OF THE SOLUTION.
*   NMERGE : TOTAL NUMBER OF MERGED BLOCKS FOR WHICH SPECIFIC VALUES
*            OF THE NEUTRON FLUX AND REACTIONS RATES ARE REQUIRED.
*   NMBIX  : NUMBER OF MIXTURES
*   NANI   : NUMBER OF LEGENDRE ORDER. ONLY NANI=1 ALLOWED
*   MAT    : INDEX-NUMBER OF THE MIXTURE TYPE ASSIGNED TO EACH VOLUME.
*            DIMENSION MAT(NMERGE)
*   VOL    : VOLUMES. DIMENSION VOL(NMERGE)
*   KNORM  : NOT USED.
*   SIGT0  : TOTAL MACROSCOPIC CROSS SECTIONS ORDERED BY MIXTURE.
*            DIMENSION SIGT0(0:NBMIX)
*   SIGS0  : WITHIN GROUP SCATTERING MACROSCOPIC CROSS SECTION ORDERED
*            BY MIXTURE. DIMENSION SIGS0(0:NBMIX,NANI)
*   ILK    : ILK=.TRUE. IF NEUTRON LEAKAGE THROUGH EXTERNAL BOUNDARY
*            IS PRESENT.
*   TITR   : TITLE.
*
*---------------------------------------------- A. HEBERT 27/09/88 -----
*
      PARAMETER (NSTATE=40)
      LOGICAL ILK,MASKK,LR2,USEOLD
      CHARACTER*72 TITR
      DIMENSION    VOL(NMERGE),MAT(NMERGE),
     >             SIGT0(0:NBMIX),SIGS0(0:NBMIX,NANI),
     >             ISTATE(NSTATE)
      INTEGER      IBASE(1)
      REAL         BASE
      COMMON       BASE(1)
      LOGICAL      LBASE(1)
      EQUIVALENCE (BASE(1),IBASE(1),LBASE(1))
*
* RECOVER JPM SPECIFIC PARAMETERS.
      CALL XDRSET(ISTATE,NSTATE,0)
      CALL LCMGET(IPTRK,'STATE-VECTOR',ISTATE)
      NPIS=ISTATE(5)
      ITG=ISTATE(7)
      USEOLD=ISTATE(10).EQ.1
      NMBLK=ISTATE(11)
      NGEN=ISTATE(12)
      IJAT=ISTATE(13)
      NPSS=ISTATE(16)
      NICM=ISTATE(17)
      IIFR=LCMIOF(IPTRK,'IFR')
      IALB=LCMIOF(IPTRK,'ALB')
      IINU=LCMIOF(IPTRK,'INUM')
      IMIX=LCMIOF(IPTRK,'MIX')
      IDVX=LCMIOF(IPTRK,'DVX')
      IIGE=LCMIOF(IPTRK,'IGEN')
      IISU=LCMIOF(IPTRK,'ISURF')
      ICHO=LCMIOF(IPTRK,'CHORD')
      IMU1=LCMIOF(IPTRK,'MU1')
      IIMA=LCMIOF(IPTRK,'IMA')
*
* COMPUTE THE INDIVIDUAL GENERATING BLOCK COLLISION PROBABILITIES.
      CALL SETARA(LBASE,NGEN,IMASK)
      DO 10 IKK=1,NMERGE
   10 LBASE(IMASK+IBASE(IIGE+IKK-1)-1)=.TRUE.
      CALL SETARA(BASE,NGEN,ISIGT)
      CALL SETARA(BASE,NGEN,ISIGS)
      CALL SETARA(BASE,NGEN,IPII)
      CALL SETARA(BASE,NPIS,IPIS)
      CALL SETARA(BASE,NPSS,IPSS)
      CALL LCMLEN(IPSYS,'DRAGONJPMTXS',LENGT1,ITYLCM)
      CALL LCMLEN(IPSYS,'DRAGONJPMPIS',LENGT2,ITYLCM)
      IF (USEOLD.AND.(LENGT1.EQ.NGEN).AND.(LENGT2.EQ.NPIS)) THEN
         CALL LCMGET(IPSYS,'DRAGONJPMTXS',BASE(ISIGT))
         MASKK=.TRUE.
         DO 30 IKK=1,NMERGE
         IKG=IBASE(IIGE+IKK-1)
         LBASE(IMASK+IKG-1)=BASE(ISIGT+IKG-1).NE.SIGT0(MAT(IKK))
   30    MASKK=MASKK.AND.LBASE(IMASK+IKG-1)
         IF (.NOT.MASKK) THEN
            CALL LCMGET(IPSYS,'DRAGONJPMPII',BASE(IPII))
            CALL LCMGET(IPSYS,'DRAGONJPMPIS',BASE(IPIS))
            CALL LCMGET(IPSYS,'DRAGONJPMPSS',BASE(IPSS))
         ENDIF
      ENDIF
      DO 40 IKK=1,NMERGE
      IKG=IBASE(IIGE+IKK-1)
      BASE(ISIGT+IKG-1)=SIGT0(MAT(IKK))
   40 BASE(ISIGS+IKG-1)=SIGS0(MAT(IKK),1)
      CALL JPMDIS(IPTRK,ITG,IMPX,NGEN,IFTRAK,IBASE(IISU),BASE(ICHO),
     1 BASE(ISIGT),LBASE(IMASK),BASE(IPII),BASE(IPIS),BASE(IPSS))
      CALL LCMPUT(IPSYS,'DRAGONJPMPII',NGEN,2,BASE(IPII))
      CALL LCMPUT(IPSYS,'DRAGONJPMPIS',NPIS,2,BASE(IPIS))
      CALL LCMPUT(IPSYS,'DRAGONJPMPSS',NPSS,2,BASE(IPSS))
      CALL LCMPUT(IPSYS,'DRAGONJPMTXS',NGEN,2,BASE(ISIGT))
      CALL LCMPUT(IPSYS,'DRAGONJPMSXS',NGEN,2,BASE(ISIGS))
*
* CALCULATION AND LU FACTORIZATION OF MATRIX (I-WSS*A).
      CALL SETARA(BASE,NICM,IWSSB)
      CALL SETARA(BASE,NMERGE,IIGAT)
      CALL SETARA(BASE,NGEN,IIGAU)
      CALL SETARA(BASE,NGEN,IIGAU2)
      CALL JPMASB(IPSYS,LR2,NMBLK,IBASE(IIFR),BASE(IALB),NMERGE,IJAT,
     1 IBASE(IINU),IBASE(IMIX),BASE(IDVX),IBASE(IMU1),IBASE(IIMA),NGEN,
     2 IBASE(IIGE),IBASE(IISU),BASE(ICHO),BASE(ISIGS),BASE(IPII),
     3 BASE(IPIS),BASE(IPSS),BASE(IWSSB),BASE(IIGAT),BASE(IIGAU),
     4 BASE(IIGAU2))
      CALL RLSARA(BASE(IIGAU2))
      CALL RLSARA(BASE(IIGAU))
      CALL RLSARA(BASE(IIGAT))
      CALL RLSARA(BASE(IWSSB))
*
      CALL RLSARA(BASE(IPSS))
      CALL RLSARA(BASE(IPIS))
      CALL RLSARA(BASE(IPII))
      CALL RLSARA(BASE(ISIGS))
      CALL RLSARA(BASE(ISIGT))
      CALL RLSARA(LBASE(IMASK))
      RETURN
      END
