*DECK JPMASB
      SUBROUTINE JPMASB (IPSYS,LR2,NMBLK,IFR,ALB,NMERGE,IJAT,INUM,MIX,
     1 DVX,MU1,IMA,NGEN,IGEN,ISURF,CHORD,SIGS,PII,PIS,PSS,WSSB,IGAT,
     2 IGAU,IGAU2)
*
*-----------------------------------------------------------------------
*
* THIS SUBROUTINE COMPUTES THE (I-WSS*A) AND L*U=(I-WSS*A) MATRICES IN
* COMPRESSED DIAGONAL STORAGE MODE AND STORE THEM ON LCM.
*
* INPUT PARAMETERS:
*   IPSYS  : POINTER TO THE PIJ (L_PIJ SIGNATURE).
*   LR2    : IF LR2=.TRUE. THE (I-WSS*A) MATRIX IS SAVED ON LCM BEFORE
*            FACTORIZATION.
*
* INPUT PARAMETERS RELATED TO BLOCKS DEFINITION:
*   NMBLK  : TOTAL NUMBER OF BLOCKS IN THE DOMAIN.
*   IFR    : INDEX-NUMBER OF IN-CURRENTS. DIMENSION IFR(IJAS) WHERE
*            IJAS IS THE TOTAL NUMBER OF IN-CURRENTS SURROUNDING ALL
*            THE BLOCKS.
*   ALB    : TRANSMISSION/ALBEDO ASSOCIATED WITH EACH IN-CURRENT.
*            DIMENSION ALB(IJAS)
*
* INPUT PARAMETERS RELATED TO THE MERGED BLOCKS DEFINITION:
*   NMERGE : TOTAL NUMBER OF MERGED BLOCKS FOR WHICH SPECIFIC VALUES
*            OF THE NEUTRON FLUX AND REACTIONS RATES ARE REQUIRED.
*            MANY BLOCKS WITH DIFFERENT POSITION IN THE DOMAIN CAN
*            BE MERGED BEFORE THE NEUTRON FLUX CALCULATION IF THEY
*            OWN THE SAME GENERATING BLOCK. THIS ALLOWS SOME REDUCTION
*            IN CPU TIME AND MEMORY. NMERGE.LE.NMBLK
*   IJAT   : TOTAL NUMBER OF DISTINCT OUT-CURRENTS.
*   INUM   : INDEX-NUMBER OF THE MERGED BLOCK ASSOCIATED TO EACH
*            BLOCK. DIMENSION INUM(NMBLK)
*   MIX    : INDEX-NUMBER OF OUT-CURRENTS. DIMENSION MIX(IJAR) WHERE
*            IJAR IS THE TOTAL NUMBER OF OUT-CURRENTS BELONGING TO THE
*            MERGED BLOCKS.
*   DVX    : WEIGHT ASSOCIATED WITH EACH OUT-CURRENT.
*            DIMENSION DVX(IJAR)
*   MU1    : POSITION OF EACH DIAGONAL ELEMENT IN MATRIX (I-WSS*A).
*   IMA    : POSITION OF THE FIRST NON-ZERO COLUMN ELEMENT IN MATRIX
*            (I-WSS*A). DIMENSION MU1(IJAT),IMA(IJAT)
*
*            REMARK:
*            IFR, ALB, MIX AND DVX CONTAINS INFORMATION TO REBUILD THE
*            GEOMETRICAL "A" MATRIX.
*
* INPUT PARAMETERS RELATED TO THE GENERATING BLOCKS DEFINITION:
*   NGEN   : TOTAL NUMBER OF GENERATING BLOCKS. A GENERATING BLOCK IS
*            DEFINED BY ITS MIXTURE AND DIMENSIONS IRRESPECTIVE OF
*            ITS POSITION IN THE DOMAIN. NGEN.LE.NMERGE
*   IGEN   : INDEX-NUMBER OF THE GENERATING BLOCK ASSOCIATED WITH EACH
*            MERGED BLOCK. DIMENSION IGEN(NMERGE)
*   ISURF  : NUMBER OF SURFACES ASSOCIATED WITH EACH GENERATING BLOCK.
*            DIMENSION ISURF(NGEN)
*   CHORD  : MEAN CHORD LENGTH ASSOCIATED WITH EACH SURFACE.
*            DIMENSION CHORD(NPIS) WHERE NPIS=SUM(ISURF(I)) IS THE
*            TOTAL NUMBER OF SURFACES SURROUNDING ALL THE GENERATING
*            BLOCKS.
*   SIGS   : WITHIN-GROUP SCATTERING CROSS SECTIONS OF THE GENERATING
*            BLOCKS. DIMENSION SIGS(NGEN)
*   PII    : VOLUME TO VOLUME REDUCED COLLISION PROBABILITIES.
*            DIMENSION PII(NGEN)
*   PIS    : VOLUME TO SURFACE COLLISION PROBABILITIES.
*            DIMENSION PIS(NPIS) WHERE NPIS=SUM(ISURF(I))
*   PSS    : SURFACE TO SURFACE COLLISION PROBABILITIES.
*            DIMENSION PSS(NPSS) WHERE NPSS=SUM(ISURF(I)**2)
*
* SCRATCH STORAGE:
*   WSSB   : DIMENSION WSSB(IMA(IJAT))
*   IGAT   : DIMENSION IGAT(NMERGE)
*   IGAU   : DIMENSION IGAU(NGEN)
*   IGAU2  : DIMENSION IGAU2(NGEN)
*
* ------------------------------------- AUTHOR: A. HEBERT ; 20/09/89 ---
*
      LOGICAL LR2
      DIMENSION IFR(*),ALB(*),INUM(*),MIX(*),DVX(*),MU1(*),IMA(*),
     1 IGEN(*),ISURF(*),CHORD(*),SIGS(*),PII(*),PIS(*),PSS(*),WSSB(*),
     2 IGAT(*),IGAU(*),IGAU2(*)
*
      IJAR=0
      DO 10 I=1,NMERGE
      IGAT(I)=IJAR
   10 IJAR=IJAR+ISURF(IGEN(I))
      NPIS=0
      NPSS=0
      DO 20 I=1,NGEN
      IGAU(I)=NPIS
      IGAU2(I)=NPSS
      NPIS=NPIS+ISURF(I)
   20 NPSS=NPSS+ISURF(I)**2
*
      DO 30 I=1,IMA(IJAT)
   30 WSSB(I)=0.0
      DO 40 I=1,IJAT
   40 WSSB(MU1(I))=1.0
      IS=0
      DO 60 ICEL=1,NMBLK
      IKK=INUM(ICEL)
      IKG=IGEN(IKK)
      FACT=SIGS(IKG)/(1.0-SIGS(IKG)*PII(IKG))
      NCOUR=ISURF(IKG)
      IT=IGAT(IKK)
      DO 50 IP=1,NCOUR
      J1=MIX(IT+IP)
      IDLIMI=MU1(J1)-J1
      PBJ=PIS(IGAU(IKG)+IP)*CHORD(IGAU(IKG)+IP)
      DO 50 JP=1,NCOUR
      J2=IFR(IS+JP)
      IOFSET=IGAU2(IKG)+(JP-1)*NCOUR+IP
      WSS=PSS(IOFSET)+PBJ*PIS(IGAU(IKG)+JP)*FACT
      IF(J1.GE.J2) THEN
         WSSB(IDLIMI+J2)=WSSB(IDLIMI+J2)-WSS*ALB(IS+JP)*DVX(IT+IP)
      ELSE
         IDLJPJ=MU1(J2)+J2
         WSSB(IDLJPJ-J1)=WSSB(IDLJPJ-J1)-WSS*ALB(IS+JP)*DVX(IT+IP)
      ENDIF
   50 CONTINUE
   60 IS=IS+NCOUR
*
      IF(LR2) CALL LCMPUT(IPSYS,'DRAGONJPMRMM',IMA(IJAT),2,WSSB)
      CALL ALLUF(IJAT,WSSB,MU1,IMA)
      CALL LCMPUT(IPSYS,'DRAGONJPMICM',IMA(IJAT),2,WSSB)
      RETURN
      END
