*DECK JPMEUR
      SUBROUTINE JPMEUR (MAXPTS,MAXJ,MAXZ,IPGEOM,IR,MAT,VOL,ILK,LMAIL,
     1 ITGEN,COORD,MAIL,IZMAIL,RZMAIL,IMPX,IAPP,MULTC,IWIGN,IHALT,ILIGN,
     2 INORM,IRECT,IQW,IQUAD,NMBLK,IFR,ALB,NMERGE,IJAT,INUM,MIX,DVX,
     3 NGEN,IGEN,ISURF,CHORD,XX,YY,RAYRE,NMC,IORI)
*
*-----------------------------------------------------------------------
*
* READING AND ANALYSIS OF THE GEOMETRY FOR THE DESCRIPTION OF A 2-D
* ASSEMBLY (EURYDICE-2) FOR THE J+- METHOD.
*
* GENERAL PARAMETERS:
*   MAXPTS : ALLOCATED STORAGE FOR ARRAYS OF DIMENSION NMBLK, NMERGE OR
*            NGEN.
*   MAXJ   : ALLOCATED STORAGE FOR ARRAYS OF DIMENSION IJAT OR IJAS
*            WHERE IJAT IS THE TOTAL NUMBER OF DISTINCT OUT-CURRENTS
*            AND IJAS IS THE TOTAL NUMBER OF OUT-CURRENTS LEAVING ALL
*            THE BLOCKS (MERGED OR NOT).
*   MAXZ   : ALLOCATED STORAGE FOR ARRAYS OF DIMENSION LMAIL.
*   IPGEOM : POINTER TO THE GEOMETRY (L_GEOM SIGNATURE).
*   IR     : NUMBER OF MIXTURES.
*   MAT    : INDEX-NUMBER OF THE MIXTURE TYPE ASSIGNED TO EACH VOLUME.
*            DIMENSION MAT(NMERGE)
*   VOL    : VOLUMES. DIMENSION VOL(NMERGE)
*   ILK    : ILK=.TRUE. IF NEUTRON LEAKAGE THROUGH EXTERNAL BOUNDARY
*            IS PRESENT.
*   LMAIL  : SPACE REQUIRED TO STORE THE TRACKING INFORMATION.
*   ITGEN  : TYPE OF EACH GENERATING BLOCK:
*            =1 : TUBE/DP-0 INSIDE/DP-0 OUTSIDE/
*            =2 : TUBE/DP-1 INSIDE/DP-1 OUTSIDE/
*            =3 : RECTANGLE/DP-0 OUTSIDE/
*            =4 : RECTANGLE/DP-1 OUTSIDE/
*            =5 : RECTANGULAR CROWN/DP-0 INSIDE/ROTH OUTSIDE/
*            =6 : RECTANGULAR CROWN/DP-0 INSIDE/ROTH X 4 OUTSIDE/
*            =7 : RECTANGULAR CROWN/DP-1 INSIDE/ROTH OUTSIDE/
*            =8 : RECTANGULAR CROWN/DP-1 INSIDE/ROTH X 4 OUTSIDE/
*            =9 : RECTANGULAR CROWN/DP-0 INSIDE/DP-0 OUTSIDE/
*            =10: RECTANGULAR CROWN/DP-1 INSIDE/DP-0 OUTSIDE/
*            =11: RECTANGULAR CROWN/DP-0 INSIDE/DP-1 OUTSIDE/
*            =12: RECTANGULAR CROWN/DP-1 INSIDE/DP-1 OUTSIDE/
*            =13: HEXAGON/DP-0 OUTSIDE/
*            =14: HEXAGON/DP-1 OUTSIDE/
*            =15: HEXAGONAL CROWN/DP-0 INSIDE/ROTH OUTSIDE/
*            =16: HEXAGONAL CROWN/DP-0 INSIDE/ROTH X 6 OUTSIDE/
*            =17: HEXAGONAL CROWN/DP-1 INSIDE/ROTH OUTSIDE/
*            =18: HEXAGONAL CROWN/DP-1 INSIDE/ROTH X 6 OUTSIDE/
*            =19: HEXAGONAL CROWN/DP-0 INSIDE/DP-0 OUTSIDE/
*            =20: HEXAGONAL CROWN/DP-1 INSIDE/DP-0 OUTSIDE/
*            =21: HEXAGONAL CROWN/DP-0 INSIDE/DP-1 OUTSIDE/
*            =22: HEXAGONAL CROWN/DP-1 INSIDE/DP-1 OUTSIDE/
*            DIMENSION ITGEN(NGEN)
*   COORD  : DIMENSIONS OF EACH GENERATING BLOCK.
*            DIMENSION COORD(3,NGEN)
*   MAIL   : OFFSET OF THE FIRST TRACKING INFORMATION IN EACH GENERATING
*            BLOCK. DIMENSION MAIL(NGEN)
*   IZMAIL  : TRACKING INFORMATION. DIMENSION IZMAIL(LMAIL)
*   IMPX   : PRINT FLAG. EQUAL TO 0 FOR NO PRINT.
*
* TRACKING OPTIONS:
*   IAPP   : =1 FOR DP-0  ;  =2 FOR DP-1
*   MULTC  : TYPE OF MULTICELL APPROXIMATION.
*   IWIGN  : =1: ASKEW CYLINDERIZATION. =2: SANCHEZ CYLINDERIZATION.
*   IHALT  : =1 TO STOP AFTER CALCULATION OF THE MAX TRACKING STORAGE.
*   ILIGN  : =1 TO PRINT THE TRACKING.
*   INORM  : =1 TO AVOID TRACK NORMALIZATION.
*   IRECT  : =1 TO AVOID CONSIDERING SYMMETRIES OF SQUARE CELLS.
*   IQW    : =1 TO USE EQUAL WEIGHT QUADRATURES IN ANGLE AND SPACE.
*   IQUAD  : QUADRATURE PARAMETERS. DIMENSION IQUAD(4)
*
* PARAMETERS RELATED TO BLOCKS DEFINITION:
*   NMBLK  : TOTAL NUMBER OF BLOCKS IN THE DOMAIN.
*   IFR    : INDEX-NUMBER OF IN-CURRENTS. DIMENSION IFR(IJAS) WHERE
*            IJAS IS THE TOTAL NUMBER OF IN-CURRENTS SURROUNDING ALL
*            THE BLOCKS (MERGED OR NOT).
*   ALB    : TRANSMISSION/ALBEDO ASSOCIATED WITH EACH IN-CURRENT.
*            DIMENSION ALB(IJAS)
*
* PARAMETERS RELATED TO THE MERGED BLOCKS DEFINITION:
*   NMERGE : TOTAL NUMBER OF MERGED BLOCKS FOR WHICH SPECIFIC VALUES
*            OF THE NEUTRON FLUX AND REACTIONS RATES ARE REQUIRED.
*            MANY BLOCKS WITH DIFFERENT POSITION IN THE DOMAIN CAN
*            BE MERGED BEFORE THE NEUTRON FLUX CALCULATION IF THEY
*            OWN THE SAME GENERATING BLOCK. THIS ALLOWS SOME REDUCTION
*            IN CPU TIME AND MEMORY. NMERGE.LE.NMBLK
*   IJAT   : TOTAL NUMBER OF DISTINCT OUT-CURRENTS.
*   INUM   : INDEX-NUMBER OF THE MERGED BLOCK ASSOCIATED TO EACH BLOCK.
*            DIMENSION INUM(NMBLK)
*   MIX    : INDEX-NUMBER OF OUT-CURRENTS. DIMENSION MIX(IJAR) WHERE
*            IJAR IS THE TOTAL NUMBER OF OUT-CURRENTS LEAVING ALL THE
*            MERGED BLOCKS.
*   DVX    : WEIGHT ASSOCIATED WITH EACH OUT-CURRENT.
*            DIMENSION DVX(IJAR)
*
*            REMARK:
*            IFR, ALB, MIX AND DVX CONTAINS INFORMATION TO REBUILD THE
*            GEOMETRICAL "A" MATRIX.
*
* PARAMETERS RELATED TO THE GENERATING BLOCKS DEFINITION:
*   NGEN   : TOTAL NUMBER OF GENERATING BLOCKS. A GENERATING BLOCK IS
*            DEFINED BY ITS MIXTURE AND DIMENSIONS IRRESPECTIVE OF
*            ITS POSITION IN THE DOMAIN. NGEN.LE.NMERGE
*   IGEN   : INDEX-NUMBER OF THE GENERATING BLOCK ASSOCIATED WITH EACH
*            MERGED BLOCK. DIMENSION IGEN(NMERGE)
*   ISURF  : NUMBER OF SURFACES ASSOCIATED WITH EACH GENERATING BLOCK.
*            DIMENSION ISURF(NGEN)
*   CHORD  : MEAN CHORD LENGTH ASSOCIATED WITH EACH SURFACE.
*            DIMENSION CHORD(NPIS) WHERE NPIS=SUM(ISURF(I)) IS THE
*            TOTAL NUMBER OF SURFACES SURROUNDING ALL THE GENERATING
*            BLOCKS.
*            DIMENSION ISURF(NGEN)
*
* SCRATCH STORAGE:
*   XX,YY  : DIMENSION XX(MAXPTS),YY(MAXPTS)
*   NMC    : DIMENSION NMC(MAXPTS+1)
*   RAYRE  : DIMENSION RAYRE(MAXPTS)
*   IORI   : DIMENSION IORI(MAXPTS)
*
*---------------------------------------- AUTHOR: A. HEBERT 09/04/87 ---
*
      LOGICAL ILK,LHEX
      CHARACTER HDOM*8
      DIMENSION MAT(*),VOL(*),IQUAD(*),ITGEN(*),COORD(3,*),IZMAIL(*),
     1 RZMAIL(*),MAIL(*),IFR(*),ALB(*),INUM(*),MIX(*),DVX(*),IGEN(*),
     2 ISURF(*),CHORD(*),XX(*),YY(*),RAYRE(*),NMC(*),IORI(*),NCODE(6),
     3 ZCODE(6),ISMR(6,8),ISMH(8,12),GAR(2),VAP(9),R2(2),IIT(15)
      COMMON BASE(1)
      SAVE ISMR,ISMH
      DATA ISMR/1,2,3,4,0,0,3,4,2,1,0,0,2,1,4,3,0,0,4,3,1,2,0,0,
     1          2,1,3,4,0,0,3,4,1,2,0,0,1,2,4,3,0,0,4,3,2,1,0,0/
      DATA ISMH/1,2,3,4,5,6,0,0,6,1,2,3,4,5,0,0,5,6,1,2,3,4,0,0,
     1          4,5,6,1,2,3,0,0,3,4,5,6,1,2,0,0,2,3,4,5,6,1,0,0,
     2          1,6,5,4,3,2,0,0,2,1,6,5,4,3,0,0,3,2,1,6,5,4,0,0,
     3          4,3,2,1,6,5,0,0,5,4,3,2,1,6,0,0,6,5,4,3,2,1,0,0/
      DATA IIT/1,2,3,4,5,6,7,8,9,10,12,16,20,32,64/
*
      CALL SETARA(BASE,MAXPTS,IMIL)
      CALL SETARA(BASE,MAXPTS,INBREG)
      CALL SETARA(BASE,MAXPTS,IMILIE)
      CALL SETARA(BASE,MAXPTS,IRAYON)
      CALL SETARA(BASE,3*MAXPTS,ICELL)
      CALL SETARA(BASE,MAXPTS+1,IXXX)
      CALL SETARA(BASE,MAXPTS+1,IYYY)
      CALL READEU(MAXPTS,IPGEOM,IR,MAT,ILK,NMBLK,NMERGE,NGEN,INUM,IGEN,
     1 NREG,LX,LY,XX,YY,RAYRE,NMC,IORI,NCODE,ZCODE,IHEX,IMPX,BASE(IMIL),
     2 BASE(INBREG),BASE(IMILIE),BASE(IRAYON),BASE(ICELL),BASE(IXXX),
     3 BASE(IYYY))
      LHEX=IHEX.NE.0
      CALL RLSARA(BASE(IYYY))
      CALL RLSARA(BASE(IXXX))
      CALL RLSARA(BASE(ICELL))
      CALL RLSARA(BASE(IRAYON))
      CALL RLSARA(BASE(IMILIE))
      CALL RLSARA(BASE(INBREG))
      CALL RLSARA(BASE(IMIL))
      IF (.NOT.LHEX) THEN
         A=0.0
         B=0.0
         DO 50 IKG=1,NGEN
         IF ((A.NE.0.0).AND.(A.NE.XX(IKG)).AND.(IWIGN.EQ.2).AND.(MULTC
     1   .LE.2)) CALL XABORT('JPMEUR: THE OPTION ASKE IS REQUIRED.')
         A=XX(IKG)
         IF ((B.NE.0.0).AND.(B.NE.YY(IKG)).AND.(IWIGN.EQ.2).AND.(MULTC
     1   .LE.2)) CALL XABORT('JPMEUR: THE OPTION ASKE IS REQUIRED.')
50       B=YY(IKG)
      ENDIF
*
      DO 100 IQQ=1,4
      DO 60 I=1,15
      IF (IQUAD(IQQ).EQ.IIT(I)) GO TO 100
60    CONTINUE
      CALL XABORT('JPMEUR: INVALID QUADRATURE PARAMETER.')
100   CONTINUE
      IF ((IAPP.NE.1).AND.(IAPP.NE.2)) THEN
         CALL XABORT('JPMEUR: IAPP SHOULD BE EQUAL TO 1 OR 2.')
      ENDIF
      IF (IMPX.GE.1) THEN
         IF (MULTC.EQ.1) THEN
            WRITE (6,'(/20H ROTH APPROXIMATION.)')
         ELSE IF (LHEX.AND.(MULTC.EQ.2)) THEN
            WRITE (6,'(/24H ROTH X 6 APPROXIMATION.)')
         ELSE IF (MULTC.EQ.2) THEN
            WRITE (6,'(/24H ROTH X 4 APPROXIMATION.)')
         ELSE IF (LHEX.AND.(MULTC.EQ.3)) THEN
            WRITE (6,'(/40H DP-0 APPROXIMATION AROUND EACH HEXAGON.)')
         ELSE IF (MULTC.EQ.3) THEN
            WRITE (6,'(/43H DP-0 APPROXIMATION AROUND EACH CARTESIAN C,
     1      4HELL.)')
         ELSE IF (LHEX.AND.(MULTC.EQ.4)) THEN
            WRITE (6,'(/40H DP-1 APPROXIMATION AROUND EACH HEXAGON.)')
         ELSE IF (MULTC.EQ.4) THEN
            WRITE (6,'(/43H DP-1 APPROXIMATION AROUND EACH CARTESIAN C,
     1      4HELL.)')
         ENDIF
         IF (IAPP.EQ.1) THEN
            WRITE (6,'(/38H DP-0 APPROXIMATION BETWEEN THE TUBES.)')
         ELSE IF (IAPP.EQ.2) THEN
            WRITE (6,'(/38H DP-1 APPROXIMATION BETWEEN THE TUBES.)')
         ENDIF
         IF ((MULTC.LE.2).AND.(IWIGN.EQ.1)) THEN
            WRITE (6,'(23H ASKEW CYLINDERIZATION./)')
         ELSE IF ((MULTC.LE.2).AND.(IWIGN.EQ.2)) THEN
            WRITE (6,'(24H WIGNER CYLINDERIZATION./)')
         ELSE IF ((MULTC.LE.2).AND.(IWIGN.EQ.3)) THEN
            WRITE (6,'(25H SANCHEZ CYLINDERIZATION./)')
         ENDIF
         IF (INORM.EQ.1) WRITE (6,620)
         IF (IRECT.EQ.1) WRITE (6,'(/30H SYMMETRIES OF SQUARE CELLS AR,
     1   17HE NOT CONSIDERED.)')
         IF (IQW.EQ.1) WRITE (6,'(/32H USE EQUAL WEIGHT QUADRATURES IN,
     1   17H ANGLE AND SPACE.)')
         HDOM='(0,PI/4)'
         IF (LHEX) HDOM='(0,PI/6)'
         WRITE (6,690) IQUAD(1),HDOM,IQUAD(2),IQUAD(3),IQUAD(4)
      ENDIF
*
      CALL SETARA(BASE,NMBLK,IPOURC)
      CALL SETARA(BASE,NMBLK,IIGEN2)
      CALL SETARA(BASE,NMBLK,IINUM2)
      IF (LHEX) THEN
         NCOUR=6
         CALL SETARA(BASE,8*MAXPTS,IJF2)
         CALL NUMER4 (MAXPTS,MAXJ,NCOUR,MULTC,IAPP,IWIGN,NCODE,ZCODE,
     1   IHEX,LX,LY,VOL,MAT,IORI,ISMH,IMPX,NMBLK,IFR,ALB,NMERGE,INUM,
     2   MIX,DVX,NGEN,IGEN,ISURF,CHORD,ITGEN,COORD,NSUPCE,XX,YY,RAYRE,
     3   NMC,BASE(IPOURC),BASE(IJF2),BASE(IIGEN2),BASE(IINUM2))
      ELSE
         NCOUR=4
         CALL SETARA(BASE,6*MAXPTS,IJF2)
         CALL NUMER4 (MAXPTS,MAXJ,NCOUR,MULTC,IAPP,IWIGN,NCODE,ZCODE,
     1   IHEX,LX,LY,VOL,MAT,IORI,ISMR,IMPX,NMBLK,IFR,ALB,NMERGE,INUM,
     2   MIX,DVX,NGEN,IGEN,ISURF,CHORD,ITGEN,COORD,NSUPCE,XX,YY,RAYRE,
     3   NMC,BASE(IPOURC),BASE(IJF2),BASE(IIGEN2),BASE(IINUM2))
      ENDIF
      CALL RLSARA(BASE(IINUM2))
      CALL RLSARA(BASE(IIGEN2))
      CALL RLSARA(BASE(IJF2))
      CALL RLSARA(BASE(IPOURC))
      IF (NMERGE.NE.NREG) CALL XABORT('JPMEUR: INCONSISTENT RESULT FRO'
     1 //'M NUMER4.')
*
      IJAR=0
      DO 250 I=1,NMERGE
250   IJAR=IJAR+ISURF(IGEN(I))
      IJAT=0
      DO 260 NSURF=1,IJAR
260   IJAT=MAX0(IJAT,MIX(NSURF))
*
* ESTIMATION OF THE MEMORY REQUIRED TO STORE THE TRACKING INFORMATION.
      NA=IQUAD(1)
      IF (.NOT.LHEX) NA=2*NA
      NX=IQUAD(2)
      LMAIL=0
      IF (MULTC.GE.3) THEN
         DO 280 IKG=1,NGEN
         IF ((ITGEN(IKG).GE.9).AND.(ITGEN(IKG).LE.12)) THEN
            A=COORD(2,IKG)
            B=COORD(3,IKG)
            IF ((A.EQ.B).AND.(IRECT.NE.1)) THEN
               LMAIL=LMAIL+6+NA*(5+15*NX)
            ELSE
               LMAIL=LMAIL+6+NA*(7+25*NX)
            ENDIF
         ELSE IF ((ITGEN(IKG).GE.19).AND.(ITGEN(IKG).LE.22)) THEN
            LMAIL=LMAIL+6+NA*(11+NX*27)
         ENDIF
280      CONTINUE
         IF ((IMPX.GE.1).OR.(LMAIL.GT.MAXZ)) WRITE (6,680) LMAIL,MAXZ
         IF (LMAIL.GT.MAXZ) WRITE (6,'(/27H JPMEUR: *** WARNING *** MA,
     1   46HXZ IS LESS THAN THE ESTIMATED TRACKING LENGTH.)')
         DO 290 I=1,LMAIL
         IZMAIL(I)=0
290      RZMAIL(I)=0.0
      ENDIF
      IF (IHALT.EQ.1) CALL XABORT('JPMEUR: STOP REQUESTED BY USER.')
*
* COMPUTE THE TRACKING INFORMATION.
      IF (MULTC.GE.3) THEN
         LMAIL=0
         PRECC=0.0
         DO 350 IKG=1,NGEN
         MAIL(IKG)=0
         IF ((ITGEN(IKG).GE.19).AND.(ITGEN(IKG).LE.22)) THEN
            LZI=3*NA+2
            GAR(1)=COORD(1,IKG)
            GAR(2)=0.0
            CALL SYBHTK(NA,NX,2,GAR,RZMAIL(LMAIL+LZI+1),IZMAIL(LMAIL+1),
     1      INORM,PREC,ILIGN,IQW,COORD(2,IKG),LL,R2,VAP,RACOS,RSQRT)
            L=LL+LZI
            PRECC=AMAX1(PRECC,PREC)
            MAIL(IKG)=LMAIL+1
            LMAIL=LMAIL+L
         ELSE IF ((ITGEN(IKG).GE.9).AND.(ITGEN(IKG).LE.12)) THEN
            GAR(1)=0.0
            GAR(2)=COORD(1,IKG)
            CALL SYBRTK(NA,NX,2,COORD(2,IKG),COORD(3,IKG),GAR,ILIGN,
     1      INORM,IRECT,IQW,L,RZMAIL(LMAIL+1),IZMAIL(LMAIL+1),PREC,VAP)
            PRECC=AMAX1(PRECC,PREC)
            MAIL(IKG)=LMAIL+1
            LMAIL=LMAIL+L
         ENDIF
350      CONTINUE
         IF (IMPX.GE.1) WRITE (6,710) LMAIL,PRECC
         IF (LMAIL.GT.MAXZ) CALL XABORT('JPMEUR: INSUFFICIENT MAXZ.')
      ENDIF
      RETURN
*
620   FORMAT (52H THE TRACKS ARE NOT NORMALIZED AND THE GEOMETRIC ACC,
     1 43HURACY OF THE TRACKING MESH IS NOT COMPUTED./)
680   FORMAT (/41H MAXIMUM STORAGE REQUIRED BY THE TRACKS =,I7,8X,
     1 24HAVAILABLE STORAGE MAXZ =,I7/)
690   FORMAT (/23H QUADRATURE PARAMETERS:/24H HETEROGENEOUS CELLS   -,
     1 17H    ANGLES   NA =,I3,4H IN ,A8/24X,17H  SEGMENTS   NX =,
     2 I3//41H HOMOGENEOUS CELLS     -    ANGLES   NH =,I3//
     3 41H CYLINDRICAL CELLS     -  SEGMENTS   MR =,I3//)
710   FORMAT (/39H EXACT STORAGE REQUIRED BY THE TRACKS =,I7,8X,
     1 10HACCURACY =,1P,E11.3/)
      END
