*DECK JPMP
      SUBROUTINE JPMP  (IPSYS ,IPTRK ,IFTRAK,IMPX  ,NMERGE,NBMIX ,
     >                  MAT   ,VOL   ,KNORM ,NELPIJ,IPIJK ,ILK   ,
     >                  TITR  ,SIGT0 ,PIJ   )
*
*-----------------------------------------------------------------------
*
* CALCULATION OF THE COLLISION PROBABILITIES FOR JPM.
*
* GENERAL PARAMETERS:
*   IPSYS  : POINTER TO THE PIJ (L_PIJ SIGNATURE).
*   IPTRK  : POINTER TO THE TRACKING (L_TRACK SIGNATURE).
*   IFTRAK : FILE UNIT NUMBER USED TO STORE CLUSTER TRACKS.
*   IMPX   : PRINT FLAG. EQUAL TO ZERO FOR NO PRINT.
*   NMERGE : TOTAL NUMBER OF MERGED BLOCKS FOR WHICH SPECIFIC VALUES
*            OF THE NEUTRON FLUX AND REACTIONS RATES ARE REQUIRED.
*            MANY BLOCKS WITH DIFFERENT POSITION IN THE DOMAIN CAN
*            BE MERGED BEFORE THE NEUTRON FLUX CALCULATION IF THEY
*            OWN THE SAME GENERATING BLOCK. THIS ALLOWS SOME
*            REDUCTION IN CPU TIME AND MEMORY. NMERGE.LE.NMBLK
*   NBMIX  : NUMBER OF MIXTURES.
*   MAT    : INDEX-NUMBER OF THE MIXTURE TYPE ASSIGNED TO EACH VOLUME.
*            DIMENSION MAT(NMERGE)
*   VOL    : VOLUMES. DIMENSION VOL(NMERGE)
*   KNORM  : NOT USED.
*   NELPIJ : NUMBER OF ELEMENTS IN PIJ MATRIX
*   IPIJK  : PIJ OPTION   =1 PIJ, =4 PIJK
*   ILK    : ILK=.TRUE. IF NEUTRON LEAKAGE THROUGH EXTERNAL BOUNDARY
*            IS PRESENT.
*   TITR   : TITLE.
*   SIGT0  : TOTAL MACROSCOPIC CROSS SECTIONS ORDERED BY MIXTURE.
*            DIMENSION SIGT0(IR) WHERE IR=MAX(MAT(I))
*   PIJ    : REDUCED AND SYMMETRICALIZED COLLISION PROBABILITIES.
*            DIMENSION PIJ(NELPIJ)
*
*---------------------------------------------- A. HEBERT 26/02/87 -----
*
      PARAMETER   (NSTATE=40)
      LOGICAL      ILK,MASKK,USEOLD
      CHARACTER*72 TITR
      INTEGER      NBMIX
      DIMENSION    VOL(NMERGE),MAT(NMERGE),SIGT0(0:NBMIX),PIJ(NELPIJ)
      INTEGER      ISTATE(NSTATE),IBASE(1)
      COMMON       BASE(1)
      LOGICAL      LBASE(1)
      EQUIVALENCE (BASE(1),IBASE(1),LBASE(1))
*
* RECOVER JPM SPECIFIC PARAMETERS.
      CALL LCMGET(IPTRK,'STATE-VECTOR',ISTATE)
      NPIS=ISTATE(5)
      ITG=ISTATE(7)
      USEOLD=ISTATE(10).EQ.1
      NMBLK=ISTATE(11)
      NGEN=ISTATE(12)
      IJAT=ISTATE(13)
      NPSS=ISTATE(16)
      IIFR=LCMIOF(IPTRK,'IFR')
      IALB=LCMIOF(IPTRK,'ALB')
      IINU=LCMIOF(IPTRK,'INUM')
      IMIX=LCMIOF(IPTRK,'MIX')
      IDVX=LCMIOF(IPTRK,'DVX')
      IIGE=LCMIOF(IPTRK,'IGEN')
      IISU=LCMIOF(IPTRK,'ISURF')
      ICHO=LCMIOF(IPTRK,'CHORD')
*
* COMPUTE THE INDIVIDUAL GENERATING BLOCK COLLISION PROBABILITIES.
      CALL SETARA(LBASE,NGEN,IMASK)
      DO 10 IKK=1,NMERGE
   10 LBASE(IMASK+IBASE(IIGE+IKK-1)-1)=.TRUE.
      CALL SETARA(BASE,NGEN,ISIG)
      CALL SETARA(BASE,NGEN,IPII)
      CALL SETARA(BASE,NPIS,IPIS)
      CALL SETARA(BASE,NPSS,IPSS)
      CALL LCMLEN(IPSYS,'DRAGONJPMTXS',LENGT1,ITYLCM)
      CALL LCMLEN(IPSYS,'DRAGONJPMPIS',LENGT2,ITYLCM)
      IF (USEOLD.AND.(LENGT1.EQ.NGEN).AND.(LENGT2.EQ.NPIS)) THEN
         CALL LCMGET(IPSYS,'DRAGONJPMTXS',BASE(ISIG))
         MASKK=.TRUE.
         DO 30 IKK=1,NMERGE
         IKG=IBASE(IIGE+IKK-1)
         LBASE(IMASK+IKG-1)=BASE(ISIG+IKG-1).NE.SIGT0(MAT(IKK))
   30    MASKK=MASKK.AND.LBASE(IMASK+IKG-1)
         IF (.NOT.MASKK) THEN
            CALL LCMGET(IPSYS,'DRAGONJPMPII',BASE(IPII))
            CALL LCMGET(IPSYS,'DRAGONJPMPIS',BASE(IPIS))
            CALL LCMGET(IPSYS,'DRAGONJPMPSS',BASE(IPSS))
         ENDIF
      ENDIF
      DO 40 IKK=1,NMERGE
   40 BASE(ISIG+IBASE(IIGE+IKK-1)-1)=SIGT0(MAT(IKK))
      CALL JPMDIS(IPTRK,ITG,IMPX,NGEN,IFTRAK,IBASE(IISU),BASE(ICHO),
     1 BASE(ISIG),LBASE(IMASK),BASE(IPII),BASE(IPIS),BASE(IPSS))
      IF ((.NOT.USEOLD).OR.(LENGT1.NE.NGEN).OR.(LENGT2.NE.NPIS)) THEN
         CALL LCMPUT(IPSYS,'DRAGONJPMTXS',NGEN,2,BASE(ISIG))
         CALL LCMPUT(IPSYS,'DRAGONJPMPII',NGEN,2,BASE(IPII))
         CALL LCMPUT(IPSYS,'DRAGONJPMPIS',NPIS,2,BASE(IPIS))
         CALL LCMPUT(IPSYS,'DRAGONJPMPSS',NPSS,2,BASE(IPSS))
      ENDIF
*
* BUILD THE REDUCED COLLISION PROBABILITY MATRIX FOR THE DOMAIN.
      CALL SETARA(BASE,NMERGE*NMERGE,IP)
      CALL SETARA(BASE,2*IJAT*IJAT,IPSSB)
      CALL SETARA(BASE,NMERGE,IIGAT)
      CALL SETARA(BASE,NGEN,IIGAU)
      CALL SETARA(BASE,NGEN,IIGAU2)
      CALL JPMCP(ILK,IMPX,NMBLK,IBASE(IIFR),BASE(IALB),NMERGE,IJAT,
     1 IBASE(IINU),VOL,IBASE(IMIX),BASE(IDVX),NGEN,IBASE(IIGE),
     2 IBASE(IISU),BASE(ICHO),BASE(ISIG),BASE(IPII),BASE(IPIS),
     3 BASE(IPSS),BASE(IP),BASE(IPSSB),BASE(IIGAT),BASE(IIGAU),
     4 BASE(IIGAU2))
      CALL RLSARA(BASE(IIGAU2))
      CALL RLSARA(BASE(IIGAU))
      CALL RLSARA(BASE(IIGAT))
      CALL RLSARA(BASE(IPSSB))
*
      CALL RLSARA(BASE(IPSS))
      CALL RLSARA(BASE(IPIS))
      CALL RLSARA(BASE(IPII))
      CALL RLSARA(BASE(ISIG))
      CALL RLSARA(LBASE(IMASK))
*
      IC=0
      DO 50 IKK=1,NMERGE
      IOF=(IKK-1)*NMERGE
      DO 50 JKK=1,IKK
      IC=IC+1
   50 PIJ(IC)=BASE(IP+IOF+JKK-1)*VOL(JKK)
      CALL RLSARA(BASE(IP))
      RETURN
      END
