*DECK LIBAPL
      SUBROUTINE LIBAPL(IPLIB,IMPX,CFILNA,NGRO,NBISO,NL,ISONAM,ISONRF,
     >                  ISHINA,TN,AW,SN,SB,ILIB,ITISOM,DELTA,NGFR,
     >                  ITRANC,IPR,VECT,SIGS,SIG1,SIGA,SIGN,PRI,VTHER,
     >                  PHI,PP,PP1,SSS,SSS1,SS1,SS11,UUU,SCAT,GAR,
     >                  NJJ,IJJ,SEFF,NISB,NISBEF,NPROC,ITYPRO)
C
C-----------------------------------------------------------------------
C
C TRANSCRIPTION OF THE USEFULL INTERPOLATED MICROSCOPIC CROSS SECTION
C DATA FROM APOLIB-1 TO LCM.
C
C INPUT PARAMETERS:
C  IPLIB   : POINTER TO THE LATTICE MICROSCOPIC CROSS SECTION LIBRARY
C            (L_LIBRARY SIGNATURE).
C  IMPX    : PRINT FLAG.
C  CFILNA  : CHARACTER*8 NAME OF THE APOLIB FILE.
C  NGRO    : NUMBER OF ENERGY GROUPS.
C  NBISO   : NUMBER OF ISOTOPES PRESENT IN THE CALCULATION DOMAIN.
C  NL      : NUMBER OF LEGENDRE ORDERS REQUIRED IN THE CALCULATION.
C            NL=1 OR HIGHER.
C  ISONAM  : ALIAS NAME OF ISOTOPES. ISONAM(3,NBISO)
C  ISONRF  : REFERENCE NAME OF ISOTOPES. ISONRF(3,NBISO)
C  ISHINA  : SELF SHIELDING NAME. ISHINA(2,NBISO)
C  TN      : TEMPERATURE OF EACH ISOTOPE. DIMENSION TN(NBISO)
C  AW      : ATOMIC WEIGHT. AW(NBISO)
C  SN      : DILUTION CROSS SECTION IN EACH ENERGY GROUP OF EACH
C            ISOTOPE. SN(NGRO,NBISO)
C  SB      : DILUTION CROSS SECTION AS USED BY LIVOLANT AND JEANPIERRE
C            NORMALIZATION. SB(NGRO,NBISO)
C  ILIB    : LIBRARY TO PROCESS
C  ITISOM  : ISOTOPE WITH INDEX ILIB IS PROCESSED. ITISOM(NBISO)
C  DELTA   : LETHARGY. DELTA(NGRO)
C  NGFR    : NG FAST+RESONANCE GROUPS
C  ITRANC  : TRANSPORT CORRECTION
C            0 -> NONE ; 1 -> APOL; 2 -> WIMS
C  NPROC   : NUMBER OF CROSS SECTION TYPE PROCESSED
C  ITYPRO  : FLAG FOR CROSS SECTION TYPE PROCESSED ITYPRO(NPROC)
C SCRATCH STORAGE:
C  IPR     : DIMENSION IPR(2,NBISO)
C  VECT    : DIMENSION VECT(NGRO+1)
C  SIGS    : DIMENSION SIGS(NGRO)
C  SIG1    : DIMENSION SIG1(NGRO)
C  SIGA    : DIMENSION SIGA(NGRO)
C  SIGN    : DIMENSION SIGN(NGRO)
C  PRI     : DIMENSION PRI(MAXTRA)
C  VTHER   : DIMENSION VTHER(NGRO)
C  PHI     : DIMENSION PHI(NGRO,MAXDIL) WHERE MAXDIL=25
C  PP      : DIMENSION PP(NGRO,NGRO+1)
C  PP1     : DIMENSION PP1(NGRO,NGRO+1)
C  SSS     : DIMENSION SSS(NGRO)
C  SSS1    : DIMENSION SSS1(NGRO)
C  SS1     : DIMENSION SS1(NGRO)
C  SS11    : DIMENSION SS11(NGRO)
C  UUU     : DIMENSION UUU(NGRO)
C  DELTA   : DIMENSION DELTA(NGRO)
C  SCAT    : DIMENSION SCAT(NGRO,NGRO)
C  GAR     : DIMENSION GAR(NGRO**2)
C  NJJ     : DIMENSION NJJ(NGRO)
C  IJJ     : DIMENSION IJJ(NGRO)
C  SEFF    : DIMENSION SEFF(MAXDIL,NGRO,3) WHERE MAXDIL=25
C  NISB    : DIMENSION NISB(NBISO)
C  NISBEF  : DIMENSION NISBEF(NBISO)
*
C SUBROUTINES CALLED BY LIBAPL:
C  LIBBAS, LIBLAG, LIBSEC, LCM---.
*
C REFERENCE:
C  A. HOFFMANN, F. JEANPIERRE, A. KAVENOKY, M. LIVOLANT AND H. LORAIN,
C  'APOLLO - CODE MULTIGROUPE DE RESOLUTION DE L'EQUATION DU TRANSPORT
C  POUR LES NEUTRONS THERMIQUES ET RAPIDES', RAPPORT SERMA 'T' NO.
C  1 193, COMMISSARIAT A L'ENERGIE ATOMIQUE, SACLAY (1973).
C
C--------------------------------------- AUTHOR: A. HEBERT 03/05/88 ----
C
      IMPLICIT NONE
C----
C SUBROUTINE PARAMETERS
C----
      INTEGER       IOUT,MAXIT,MAXVEC,MAXDIL,MAXTMP,NDPROC
      PARAMETER    (IOUT=6,MAXIT=1000,MAXVEC=11,MAXDIL=25,MAXTMP=5,
     1              NDPROC=20)
C----
C DIMENSIONS
C----
      INTEGER       NBISO,NGRO,NPROC
C----
C EXTERNALS
C----
      INTEGER       KDROPN,KDRCLS
C----
C LOCAL VARIABLES
C----
      INTEGER       MAXTRA,NL,IMX,ILIB,ITC,I,IMPX,NIN,INDLOR,NR,NIT,
     1              J,K,NR1,IG4EV,NGFR,NED,NTITLE,IMAIL,ITRANC,IX,IMT,
     2              NRST,IPLIB,NTYPE,NS1,IRST,IQUAN,ISEC,IMMOND,ITH,
     3              IS,ITEST,I2,KI,KJ,IE,IR,L,IG1,IG2,NLET,K1,I1,
     4              INGRO,IL,IGAR,IGMIN,IGMAX,NS,ISS,IAS,IFS,IFL,JTYSEC,
     5              IK,NSEI,N2,N6,ITE,ITT,LE,ITET,KC,NISOT,ICT,IER,
     6              IMAT,IC,NRSTR,ICC,NKDEB,IIIC,IMAT1,ITROUV,IMAT0,NN
C
      INTEGER       ISONAM(3,NBISO),ISONRF(3,NBISO),ISHINA(2,NBISO),
     1              ITISOM(NBISO),ITYPRO(NPROC),IPR(2,*),NJJ(*),IJJ(*),
     2              NISB(*),NISBEF(*),IANIS(80),ITY(80),NEXT(80),
     3              NEXU(80),NEXV(80),NEXW(80),III(80),IT(MAXIT),
     4              ITYPE(MAXVEC),ITYSEC(MAXVEC),NTETA(3),NSE(3)
C
      REAL          E0,DEL,AT,TEMPI,TEMPA,XX,T,AMT,RENORM,AUX,ZL,
     1              TEMP,DT,SEIM,SEIT,TMP1,AA
C
      REAL          TN(*),AW(*),SN(NGRO,*),SB(NGRO,*),
     1              VECT(*),SIGS(*),SIG1(*),SIGA(*),SIGN(*),
     2              PRI(*),VTHER(*),PHI(NGRO,*),PP(NGRO,*),
     3              PP1(NGRO,*),SSS(*),SSS1(*),SS1(*),SS11(*),
     4              UUU(*),DELTA(*),SCAT(NGRO,*),GAR(*),
     5              SEFF(MAXDIL,NGRO,*),TIT(18),TETAB(MAXTMP),
     6              SIGE(MAXDIL,3),SEAUX(MAXDIL,150),
     7              XE(MAXDIL),GE(MAXDIL)
C
      CHARACTER     CFILNA*8,FORM*4,CM*2,HVEC(5)*6,HSMG*131,
     1              HNAMIS*12,HNISOR*12,HSHI*8
C
      LOGICAL       NOTG,LEXC
C
      DOUBLE PRECISION X1,X2,DDE,ENER,TMP
C
      EQUIVALENCE     (AA,NN)
C
      MAXTRA=NL*NGRO*NGRO
      DO 10 IMX=1,NBISO
        IPR(1,IMX)=0
        IPR(2,IMX)=0
        NISB(IMX)=0
        NISBEF(IMX)=0
        IF(ITISOM(IMX).EQ.ILIB) THEN
          WRITE(HSHI,'(2A4)') (ISHINA(ITC,IMX),ITC=1,2)
          WRITE(HNISOR,'(3A4)') (ISONRF(ITC,IMX),ITC=1,3)
          I=INDEX(HNISOR,' ')
          IF(I.EQ.0) THEN
            READ(HNISOR,'(I8)') NISB(IMX)
          ELSE
            WRITE(FORM,'(2H(I,I1,1H))') I-1
            READ(HNISOR,FORM) NISB(IMX)
          ENDIF
          IF(HSHI.EQ.' ') THEN
            IPR(2,IMX)=1
          ELSE
            IPR(2,IMX)=0
            I=INDEX(HSHI,' ')
            IF(I.EQ.0) THEN
              READ(HSHI,'(I8)') NISBEF(IMX)
            ELSE
              WRITE(FORM,'(2H(I,I1,1H))') I-1
              READ(HSHI,FORM) NISBEF(IMX)
            ENDIF
          ENDIF
        ENDIF
 10   CONTINUE
      IF(IMPX.GT.0) WRITE(IOUT,6000) CFILNA
      NIN=KDROPN(CFILNA,2,2,0,0)
      IF(NIN.LE.0) THEN
        WRITE(HSMG,'(36HLIBAPL: UNABLE TO OPEN LIBRARY FILE ,A8,1H.)')
     1             CFILNA
        CALL XABORT(HSMG)
      ENDIF
*
*     RECOVER THE GROUP STRUCTURE
*
      REWIND(NIN)
 20   CONTINUE
      READ(NIN) INDLOR,NR,NIT,(IT(I),I=1,NIT)
      IF(INDLOR.EQ.9999) THEN
        WRITE(IOUT,8005)
      ELSE IF(IT(3).EQ.0) THEN
        DO 30 K=1,NR
          READ(NIN)
 30     CONTINUE
        GO TO 20
      ELSE
        READ(NIN) E0,DEL,(UUU(I),I=1,NGRO),(DELTA(I),I=1,NGRO)
        NR1=NR-1
      ENDIF
      IG4EV=0
      IF(ITRANC.GE.2) THEN
        IG4EV=NGFR
      ENDIF
*
*     ***MATERIAL/ISOTOPE LOOP***
*
      NED=0
 45   CONTINUE
      NOTG=.TRUE.
      REWIND(NIN)
      NTITLE=18
 50   CONTINUE
      READ(NIN) INDLOR,NR,NIT,(IT(I),I=1,NIT),(TIT(I),I=1,NTITLE)
      IF(NIT.GT.MAXIT) THEN
        WRITE(HSMG,9000) 'MAXIT'
        CALL XABORT(HSMG)
      ENDIF
      IF(INDLOR.EQ.9999) GO TO 700
      IMAIL=IT(1)
      IF(IMAIL.EQ.99) THEN
        IX=47
      ELSE IF(IMAIL.EQ.142) THEN
        IX=60
      ELSE IF(IMAIL.EQ.172) THEN
        IX=80
      ELSE
        WRITE(HSMG,'(45HLIBAPL: INCONSISTENT GROUP STRUCTURES. IT(1)=,
     1  I5)') IMAIL
        CALL XABORT(HSMG)
      ENDIF
      DO 70 IMX=1,NBISO
        IF(ITISOM(IMX).EQ.ILIB) THEN
          IMT=IMX
          IF((INDLOR.EQ.NISB(IMX)).AND.(IPR(1,IMX).EQ.0)) GO TO 90
          IF((INDLOR.EQ.NISBEF(IMX)).AND.(IPR(1,IMX).EQ.1).AND.
     >       (IPR(2,IMX).EQ.0)) GO TO 500
        ENDIF
 70   CONTINUE
      DO 80 K=1,NR
        READ(NIN)
 80   CONTINUE
      GO TO 50
*
*     MATERIAL CONTROL
*
   90 IPR(1,IMT)=1
      NOTG=.FALSE.
      NR1=NR
      IF(IT(2).NE.NGRO) CALL XABORT('LIBAPL: INCONSISTENT GROUP STRUC'
     1 //'TURES.')
      IF(IT(3).NE.0) THEN
        READ(NIN)
        NR1=NR-1
      ENDIF
      NRST=IT(4)
      WRITE(HNAMIS,'(3A4)') (ISONAM(ITC,IMT),ITC=1,3)
      IF(IMPX.GT.0) THEN
         WRITE(IOUT,6001) HNAMIS,NISBEF(IMT),(TIT(J),J=1,18)
      ENDIF
      CALL LCMSIX(IPLIB,HNAMIS,1)
      CALL XDRSET(ITYPRO,NPROC,0)
      DO 105 J=1,NGRO
        SIGS(J)=0.0
        SIG1(J)=0.0
        SIGA(J)=0.0
        SIGN(J)=0.0
 105  CONTINUE
      NTYPE=0
      NS1=0
      DO 200 IRST=1,NRST
        IF(IRST.GT.1) THEN
          IQUAN=1
          ITYSEC(1)=IT(4+NS1+IRST)
        ELSE IF(IT(5).GT.0) THEN
          IQUAN=1
          ITYSEC(1)=IT(5)
        ELSE IF(IT(5).EQ.0) THEN
          IQUAN=4
          ITYSEC(1)=1
          ITYSEC(2)=2
          ITYSEC(3)=3
          ITYSEC(4)=4
        ELSE IF(IT(5).LT.0) THEN
          NS1=-IT(5)
          IQUAN=NS1
          DO 110 I=1,IQUAN
            ITYSEC(I)=IT(5+I)
 110      CONTINUE
        ENDIF
        READ(NIN)((PHI(J,ISEC),J=1,NGRO),ISEC=1,IQUAN)
        NR1=NR1-1
        DO 201 ISEC=1,IQUAN
          IMMOND=ITYSEC(ISEC)
          DO 120 I=1,NTYPE
            IF(IMMOND.EQ.ITYPE(I)) GO TO 205
  120     CONTINUE
          IF(IMPX.GT.7) THEN
            WRITE(IOUT,6020) NISB(IMT),IRST,IMMOND
            WRITE(IOUT,6021) (PHI(J,ISEC),J=1,NGRO)
          ENDIF
          NTYPE=NTYPE+1
          ITYPE(NTYPE)=IMMOND
          IF(IMMOND.EQ.1) THEN
            DO 140 J=1,NGRO
              SIGS(J)=PHI(J,ISEC)
 140        CONTINUE
          ELSE IF(IMMOND.EQ.2) THEN
            DO 150 J=1,NGRO
              SIGA(J)=PHI(J,ISEC)
              SIGN(J)=SIGN(J)+PHI(J,ISEC)
 150        CONTINUE
          ELSE IF(IMMOND.EQ.3) THEN
            CALL LCMPUT(IPLIB,'NUSIGF',NGRO,2,PHI(1,ISEC))
            ITYPRO(3)=1
          ELSE IF(IMMOND.EQ.4) THEN
            DO 155 J=1,NGRO
              PHI(J,ISEC)=PHI(J,ISEC)*DELTA(J)
 155        CONTINUE
            CALL LCMPUT(IPLIB,'CHI',NGRO,2,PHI(1,ISEC))
            ITYPRO(5)=1
          ELSE IF(IMMOND.EQ.5) THEN
            CALL LCMPUT(IPLIB,'NG',NGRO,2,PHI(1,ISEC))
            ITYPRO(7)=1
            DO 160 I=1,NED
              IF(HVEC(I).EQ.'NG') GO TO 205
 160        CONTINUE
            NED=NED+1
            HVEC(NED)='NG'
          ELSE IF(IMMOND.EQ.6) THEN
            DO 165 J=1,NGRO
              SIGN(J)=SIGN(J)-PHI(J,ISEC)
 165        CONTINUE
            CALL LCMPUT(IPLIB,'NFTOT',NGRO,2,PHI(1,ISEC))
            ITYPRO(4)=1
          ELSE IF(IMMOND.EQ.10) THEN
            DO 180 J=1,NGRO
              SIG1(J)=PHI(J,ISEC)
 180        CONTINUE
          ELSE IF(IMMOND.EQ.11) THEN
            DO 185 J=1,NGRO
              IF(PHI(J,ISEC).NE.0.0) THEN
                PHI(J,ISEC)=1.0/(3.0*PHI(J,ISEC))
              ENDIF
 185        CONTINUE
            CALL LCMPUT(IPLIB,'STRD',NGRO,2,PHI(1,ISEC))
            ITYPRO(17)=1
            DO 190 I=1,NED
              IF(HVEC(I).EQ.'STRD') GO TO 205
  190       CONTINUE
            NED=NED+1
            HVEC(NED)='STRD'
          ELSE
            WRITE(IOUT,8004) HNAMIS,NISB(IMT),IRST,IMMOND
          ENDIF
  205   CONTINUE
  201   CONTINUE
  200 CONTINUE
*
*     SCATTERING MATRIX CONTROL
*
      ITH=0
      IMAT=0
      IC=5+NS1+NRST
      NRSTR=IT(IC)
      ICC=IC+6*NRSTR+1
      NN=IT(ICC)
      AW(IMT)=AA
      NN=IT(ICC+1)
      AT=AA
      NKDEB=1
      IIIC=1
*
      IF(NRSTR.EQ.0)GO TO 380
      IC=IC-5
      ITH=0
      IMAT=0
      IMAT1=0
      ITROUV=0
      IMAT0=0
      DO 290 IS=1,NRSTR
      IC=IC+6
      IF(IT(IC).GT.NL-1) GO TO 280
      IF(IT(IC+1).EQ.7) THEN
*        DO TEMPERATURE INTERPOLATION FOR THE THERMAL TRANSFER MATRICES.
         IF(IX.NE.IT(IC+3)) THEN
            WRITE(IOUT,8002) IX,IT(IC+3)
            IX=IT(IC+3)
         ENDIF
         IF(IT(IC).EQ.1) IMAT1=IMAT1+1
         IF(IMAT1.EQ.1) THEN
            IMAT=0
            ITROUV=0
         ENDIF
         ITH=1
         TEMPI=REAL(IT(IC+5))+0.6
         TEMPA=TN(IMT)
         IMAT=IMAT+1
         IF(ITROUV.NE.0) GO TO 280
         ITEST=(IMAT/2)*2-IMAT
         IF(ITEST.NE.0) THEN
            READ(NIN)((PP1(K,J),K=1,IX),J=1,IX),(SSS1(K),K=1,IX),
     1      (SS11(K),K=1,IX)
            NR1=NR1-1
            X1=TEMPI
         ELSE
            IF(IT(IC).EQ.0) THEN
               READ(NIN)((PP(K,J),K=1,IX),J=1,IX),(SSS(K),K=1,IX),
     1         (SS1(K),K=1,IX)
               NR1=NR1-1
               X2=TEMPI
            ELSE
               READ(NIN)((PP(K,J),K=1,IX),J=1,IX),(SSS1(K),K=1,IX),
     1         (SS11(K),K=1,IX)
               NR1=NR1-1
               X2=TEMPI
            ENDIF
         ENDIF
         IF(IMAT.EQ.1)GO TO 290
         XX=(TEMPA-X1)*(TEMPA-X2)
         IF(XX.LE.0.)ITROUV=1
         IF((TEMPA.LE.TEMPI).AND.(IMAT.EQ.2))ITROUV=1
         IF(IT(IC+6).EQ.1.AND.IT(IC).NE.1)IMAT0=IMAT
         IF(IMAT.EQ.IMAT0)ITROUV=1
         IF(ITROUV.NE.1)GO TO 290
         XX=(TEMPA-X1)/(X1-X2)
         IF(IMAT.EQ.1)XX=0.
         I2=IIIC+IX*IX-1
         IF(I2.GT.MAXTRA) THEN
            WRITE(HSMG,9000) 'MAXTRA'
            CALL XABORT(HSMG)
         ENDIF
         IF(IT(IC).EQ.0) THEN
            DO 210 K=1,IX
            SSS(K)=SSS1(K)+(SSS1(K)-SSS(K))*XX
            KI=NGRO-K+1
            SS1(K)=SS11(K)+(SS11(K)-SS1(K))*XX
            DO 210 J=1,IX
            KJ=NGRO-J+1
  210       PP(J,K)=PP1(J,K)+(PP1(J,K)-PP(J,K))*XX
            DO 220 J=1,IX
            DO 220 K=1,IX
  220       PRI(IIIC+(J-1)*IX+K-1)=PP(J,K)
         ELSE
*           ANISOTROPES
            DO 240 K=1,IX
            KI=NGRO-K+1
            DO 240 J=1,IX
            KJ=NGRO-J+1
  240       PP1(J,K)=PP1(J,K)+(PP1(J,K)-PP(J,K))*XX
            DO 250 J=1,IX
            DO 250 K=1,IX
  250       PRI(IIIC+(J-1)*IX+K-1)=PP1(J,K)
         ENDIF
         IANIS(NKDEB)=IT(IC)
         ITY(NKDEB)=7
         NEXT(NKDEB)=IX*IX
         NEXU(NKDEB)=IX
         NEXV(NKDEB)=IX
         NEXW(NKDEB)=TEMPA
      ELSE
         IANIS(NKDEB)=IT(IC)
         ITY(NKDEB)=IT(IC+1)
         NEXT(NKDEB)=IT(IC+2)
         NEXU(NKDEB)=IT(IC+3)
         NEXV(NKDEB)=IT(IC+4)
         NEXW(NKDEB)=IT(IC+5)
         I2=IIIC+NEXT(NKDEB)-1
         IF(I2.GE.IIIC) THEN
            IF(I2.GT.MAXTRA) THEN
               WRITE(HSMG,9000) 'MAXTRA'
               CALL XABORT(HSMG)
            ENDIF
            READ(NIN)(PRI(J),J=IIIC,I2)
            NR1=NR1-1
         ENDIF
      ENDIF
      III(NKDEB)=IIIC
      IIIC=I2+1
      NKDEB=NKDEB+1
      GOTO 290
  280 READ(NIN)
      NR1=NR1-1
  290 CONTINUE
*
* FREE GAS THERMAL DIFFUSION MATRICES.
*
      IF(IX.EQ.0)GO TO 380
      IF(ITH.NE.0)GO TO 360
      T=TN(IMT)/293.6
      AMT=AW(IMT)
      IF(AMT.LT.1.0)AMT=1.0
      DO 300 K=NGRO-IX+1,NGRO
  300 SIG1(K)=SIG1(K)/SIGS(K)
      X1=0.0253D-06
      DDE=-UUU(NGRO-IX)
      ENER=E0*EXP(DDE)
      X2=SQRT(ENER/X1)
      DO 305 J=1,IX
      K=IX+1-J
      IE=NGRO-IX+J
      DDE=-UUU(IE)
      ENER=E0*DEXP(DDE)
      DDE=SQRT(ENER/X1)
      VECT(K)=X2-DDE
      X2=DDE
      VTHER(K)=2.0*VECT(K)/DELTA(IE)
 305  CONTINUE
      CALL LIBBAS(1,AT,0.0,AMT,T,IX,VTHER,VECT,NGRO,PP,SSS,SSS1,SS11)
      IF(AMT.GT.100.) THEN
         DO 310 J=1,IX
         K=NGRO-J+1
  310    SSS(J)=SIGS(K)
      ENDIF
      DO 330 I=1,IX
      K=NGRO-I+1
      SIG1(K)=SIG1(K)*SSS(I)
      SS1(I)=SIG1(K)
      RENORM=0.0
      DO 320 J=1,IX
  320 RENORM=RENORM+VTHER(J)*VECT(J)*PP(J,I)
      RENORM=RENORM/(VTHER(I)*VECT(I))
      RENORM=1.0/RENORM
      DO 330 J=1,IX
  330 PP(J,I)=PP(J,I)*SSS(I)*RENORM
      DO 340 J=1,IX
      AUX=VTHER(J)*VTHER(J)
      DO 340 I=1,IX
  340 PP(I,J)=PP(I,J)/AUX*VTHER(I)*VTHER(I)
      I2=IIIC+IX*IX
      IF(I2.GT.MAXTRA) THEN
         WRITE(HSMG,9000) 'MAXTRA'
         CALL XABORT(HSMG)
      ENDIF
      DO 350 J=1,IX
      DO 350 K=1,IX
  350 PRI(IIIC+(J-1)*IX+K-1)=PP(J,K)
      IANIS(NKDEB)=0
      ITY(NKDEB)=7
      NEXT(NKDEB)=IX*IX
      NEXU(NKDEB)=IX
      NEXV(NKDEB)=IX
      NEXW(NKDEB)=TN(IMT)
      III(NKDEB)=IIIC
      IIIC=I2+1
      NKDEB=NKDEB+1
*
  360 DO 370 J=1,IX
      K=NGRO-J+1
      SIG1(K)=SS1(J)
  370 SIGS(K)=SSS(J)
*
  380 IF(NR1.GT.0) THEN
         DO 390 IR=1,NR1
  390    READ(NIN)
      ENDIF
      NKDEB=NKDEB-1
      IIIC=IIIC-1
      IF(IMPX.GT.0.AND.IMPX.LT.100) THEN
        WRITE(IOUT,6010) IIIC,(ITYPE(L),L=1,NTYPE)
        WRITE(IOUT,6011) (ITY(L),L=1,NKDEB)
      ELSE IF(IMPX.GE.100) THEN
        DO 395 K1=1,NKDEB
          I1=III(K1)
          I2=I1+NEXT(K1)-1
          WRITE(IOUT,6022) NISB(IMT),K1,ITY(K1),
     >      NEXU(K1),NEXV(K1),NEXW(K1),IANIS(K1),(PRI(K),K=I1,I2)
  395   CONTINUE
      ENDIF
*
*     SAVE SCATTERING MATRICES ON LCM
*
      INGRO=0
      DO 396 IG1=1,NGRO
      IF(SIG1(IG1).NE.0.0) INGRO=1
  396 CONTINUE
      DO 480 IL=0,MIN(INGRO,NL-1)
      ZL=2.0*REAL(IL)+1.0
      DO 420 IG2=1,NGRO
      CALL LIBSEC(IG2,IL,NGRO,IX,UUU,DELTA,SIGS,SIG1,PRI,NLET,VECT,
     1 DEL,NKDEB,IANIS,ITY,NEXT,NEXU,NEXV,NEXW,III)
      DO 400 IG1=1,IG2
  400 SCAT(IG1,IG2)=VECT(IG2-IG1+1)*DELTA(IG2)/(ZL*DELTA(IG1))
      DO 410 IG1=IG2+1,NGRO
  410 SCAT(IG1,IG2)=VECT(IG2+NGRO-IG1+1)*DELTA(IG2)/(ZL*DELTA(IG1))
  420 CONTINUE
*
      IF(IL.EQ.0) THEN
*        PROCESS NEXCESS INFORMATION.
         LEXC=.FALSE.
         DO 430 IG1=1,NGRO-IX
         VECT(IG1)=-SIGS(IG1)
         DO 425 IG2=1,NGRO
  425    VECT(IG1)=VECT(IG1)+SCAT(IG1,IG2)
         IF(VECT(IG1)/SIGS(IG1).GT.1.0E-5) THEN
            LEXC=.TRUE.
            SIGS(IG1)=SIGS(IG1)+VECT(IG1)
         ELSE
            VECT(IG1)=0.0
         ENDIF
  430    CONTINUE
         DO 431 IG1=NGRO-IX+1,NGRO
  431    VECT(IG1)=0.0
         IF(LEXC) THEN
           ITYPRO(9)=1
           CALL LCMPUT(IPLIB,'N2N',NGRO,2,VECT)
         ENDIF
      ENDIF
*
      WRITE(CM,'(I2)') IL
      IGAR=0
*     IG2 IS THE SECONDARY GROUP.
      DO 470 IG2=1,NGRO
      IGMIN=IG2
      IGMAX=IG2
      DO 435 IG1=NGRO,1,-1
      IF(SCAT(IG1,IG2).NE.0.0) THEN
         IGMIN=MIN(IGMIN,IG1)
         IGMAX=MAX(IGMAX,IG1)
      ENDIF
  435 CONTINUE
      IJJ(IG2)=IGMAX
      NJJ(IG2)=IGMAX-IGMIN+1
      DO 440 IG1=IGMAX,IGMIN,-1
      IGAR=IGAR+1
  440 GAR(IGAR)=SCAT(IG1,IG2)
  470 CONTINUE
      IF(IL.LT.NL) THEN
        CALL LCMPUT(IPLIB,'NJJ '//CM,NGRO,1,NJJ)
        CALL LCMPUT(IPLIB,'IJJ '//CM,NGRO,1,IJJ)
        CALL LCMPUT(IPLIB,'SCAT'//CM,IGAR,2,GAR)
        ITYPRO(NDPROC+1+IL)=1
        IF(IL.EQ.0) THEN
          CALL LCMPUT(IPLIB,'SIGS'//CM,NGRO,2,SIGS)
          CALL XDRSET(VECT,NGRO,0.0)
        ELSE IF(IL.GT.0) THEN
          DO 475 IG1=1,NGRO
          VECT(IG1)=0.0
          DO 475 IG2=1,NGRO
  475     VECT(IG1)=VECT(IG1)+SCAT(IG1,IG2)
          CALL LCMPUT(IPLIB,'SIGS'//CM,NGRO,2,VECT)
          IF(IL.EQ.1.AND.ITRANC.GE.1) THEN
            IF(ITRANC.GE.2) THEN
              DO 476 IG1=1,IG4EV
                VECT(IG1)=0.0
                DO 477 IG2=1,NGRO
                  VECT(IG1)=VECT(IG1)
     >                     +SCAT(IG1,IG2)*DELTA(IG2)/DELTA(IG1)
 477            CONTINUE
 476          CONTINUE
            ENDIF
            ITYPRO(2)=1
            CALL LCMPUT(IPLIB,'TRANC',NGRO,2,VECT)
          ENDIF
        ENDIF
      ENDIF
  480 CONTINUE
      IF(ITYPRO(7).EQ.0) THEN
        ITYPRO(7)=1
        CALL LCMPUT(IPLIB,'NG',NGRO,2,SIGN)
      ENDIF
      DO 490 IG1=1,NGRO
  490 VECT(IG1)=SIGA(IG1)+SIGS(IG1)
      ITYPRO(1)=1
      CALL LCMPUT(IPLIB,'TOTAL',NGRO,2,VECT)
      CALL LCMPUT(IPLIB,'XS-SAVED',NPROC,1,ITYPRO)
      CALL LCMPUT(IPLIB,'README',18,3,TIT)
      CALL LCMSIX(IPLIB,' ',2)
      GO TO 50
*
*     SELF SHIELDING CONTROL.
*
  500 CONTINUE
      IPR(2,IMT)=1
      IF((IT(1).NE.IMAIL).OR.(IT(2).NE.NGRO).OR.(IT(3).NE.0))
     1 CALL XABORT('LIBAPL: SELF SHIELDING FAILURE (1).')
      NS=IT(4)
*
      WRITE(HNAMIS,'(3A4)') (ISONAM(ITC,IMT),ITC=1,3)
      CALL LCMSIX(IPLIB,HNAMIS,1)
      CALL LCMGET(IPLIB,'XS-SAVED',ITYPRO)
      IF(ITYPRO(NDPROC+1).NE.0) THEN
        CALL LCMGET(IPLIB,'SCAT 0',GAR)
        CALL LCMGET(IPLIB,'NJJ  0',NJJ)
        CALL LCMGET(IPLIB,'IJJ  0',IJJ)
        CALL LCMGET(IPLIB,'SIGS 0',SIGS)
      ELSE
        CALL XABORT('LIBAPL: ISOTROPIC SCATTERING XS MISSING FOR SHI')
      ENDIF
      IF(ITYPRO(1).NE.0) THEN
        CALL LCMGET(IPLIB,'TOTAL',SIGA)
      ELSE
        CALL XABORT('LIBAPL: TOTAL XS MISSING FOR SHI')
      ENDIF
      IGAR=0
      DO 540 IG2=1,NGRO
      SIGA(IG2)=SIGA(IG2)-SIGS(IG2)
      DO 540 IG1=IJJ(IG2),IJJ(IG2)-NJJ(IG2)+1,-1
      IGAR=IGAR+1
  540 SCAT(IG1,IG2)=GAR(IGAR)/SIGS(IG1)
*
      ISS=0
      IAS=0
      IFS=0
      IFL=0
      JTYSEC=0
      NTYPE=0
      DO 550 IK=1,NS
      IF(IT(IK+4).NE.JTYSEC) THEN
         NTYPE=NTYPE+1
         NTETA(NTYPE)=1
         IF(IT(IK+4).EQ.101) ISS=NTYPE
         IF(IT(IK+4).EQ.102) IAS=NTYPE
         IF(IT(IK+4).EQ.103) IFS=NTYPE
         IF(IT(IK+4).EQ.104) IFL=NTYPE
         JTYSEC=IT(IK+4)
         ITYPE(NTYPE)=IT(IK+4)
      ELSE
         NTETA(NTYPE)=NTETA(NTYPE)+1
      ENDIF
  550 CONTINUE
      IF(IFS.GT.0) CALL LCMGET(IPLIB,'NUSIGF',SSS)
      IF((ISS.EQ.0).OR.(IAS.EQ.0))
     1 CALL XABORT('LIBAPL: SELF SHIELDING FAILURE (2).')
      IF(IMPX.GT.0) THEN
        WRITE(IOUT,6002) HNAMIS,NISBEF(IMT),(TIT(I),I=1,18)
        WRITE(IOUT,6012) (ITYPE(I),I=1,NTYPE)
      ENDIF
*
*     TEMPERATURE INTERPOLATION OF EFFECTIVE REACTION RATES.
*
      DO 620 I=1,NTYPE
      IF(NTETA(I).EQ.1) THEN
         READ (NIN) TEMP,NSEI,(SIGE(K,I),K=1,NSEI),N2,N6,((SEFF(K,J
     1   ,I),J=1,N6),K=1,NSEI)
      ELSE
         IF(NTETA(I).GT.MAXTMP) THEN
            WRITE(HSMG,9000) 'MAXTMP'
            CALL XABORT(HSMG)
         ENDIF
         DO 560 ITET=1,NTETA(I)
         READ(NIN) TETAB(ITET),NSEI,(SIGE(K,I),K=1,NSEI),N2,N6,
     1   ((SEAUX(K,J),J=1,N6),K=1,NSEI)
         IF(ITET.NE.1) THEN
            IF(TN(IMT).LT.TETAB(ITET)) GO TO 570
            IF(ITET.EQ.NTETA(I)) GO TO 590
         ENDIF
         DO 560 K=1,NSEI
         DO 560 J=1,N6
  560    SEFF(K,J,I)=SEAUX(K,J)
  570    ITE=ITET+1
         DO 580 ITT=ITE,NTETA(I)
         READ(NIN)
  580    CONTINUE
  590    DT=SQRT(TN(IMT))-SQRT(TETAB(ITET))
         DT=DT/(SQRT(TETAB(ITET))-SQRT(TETAB(ITET-1)))
         DO 600 K=1,NSEI
         DO 600 J=1,N6
  600    SEFF(K,J,I)=(SEAUX(K,J)-SEFF(K,J,I))*DT+SEAUX(K,J)
      ENDIF
      NSE(I)=NSEI
      IF(I.EQ.IFL) THEN
        DO 605 J=1,N6
          DO 606 K=1,NSEI
            IF(SIGE(K,I).LT.1.0E10) THEN
              SEFF(K,J,I)=(1.0-SEFF(K,J,I))*SIGE(K,I)
            ELSE
              SEFF(K,J,I)=SEFF(K,J,IAS)
            ENDIF
  606     CONTINUE
  605   CONTINUE
      ENDIF
      DO 610 K=1,NSEI
  610 SIGE(K,I)=SQRT(SIGE(K,I))
  620 CONTINUE
*
*     DILUTION INTERPOLATION OF EFFECTIVE REACTION RATES.
*
      DO 625 L=1,NGRO
      VECT(L)=SIGS(L)
      IF(ISS.NE.0) PHI(L,ISS)=SIGS(L)
      IF(IAS.NE.0) PHI(L,IAS)=SIGA(L)
      IF(IFL.NE.0) PHI(L,IFL)=SIGA(L)
  625 CONTINUE
      DO 660 LE=1,N6
      L=LE+N2-1
      SEIM=SN(L,IMT)
      SEIT=SQRT(MAX(0.0,SEIM))
      DO 660 I=1,NTYPE
      NSEI=NSE(I)
      IF(NSEI.GT.MAXDIL) THEN
         WRITE(HSMG,'(37HLIBAPL: MAXDIL SHOULD BE INCREASED TO,I4)')
     1   NSEI
         CALL XABORT(HSMG)
      ENDIF
      PHI(L,I)=SEFF(1,LE,I)
      IF(NSEI.NE.1) THEN
         IF(SIGE(NSEI,I).LT.999.) GOTO 630
         NSEI=NSEI-1
         IF(SEIT.GT.SIGE(NSEI,I)) GOTO 650
  630    DO 640 K=1,NSEI
         KC=K
         IF(SIGE(1,I).GT.SIGE(2,I)) KC=NSEI-K+1
         XE(KC)=SIGE(K,I)
  640    GE(KC)=SEFF(K,LE,I)
         CALL LIBLAG(NSEI,XE,GE,SEIT,PHI(L,I))
         GO TO 660
  650    AUX=(SIGE(NSEI,I)/SEIT)**2
         PHI(L,I)=AUX*SEFF(NSEI,LE,I)+(1.0-AUX)*SEFF(NSEI+1,LE,I)
      ENDIF
  660 CONTINUE
*
*     COMPUTE THE EFFECTIVE FLUX INTEGRAL.
*     OR RECOVER THE EFFECTIVE FLUX.
*
      IF(IMPX.GE.10) WRITE(IOUT,6030)
      DO 663 L=1,NGRO
  663 SS1(L)=1.0
      DO 670 L=N2,N2+N6-1
      SEIM=SN(L,IMT)
      IF(SEIM.EQ.0.) CALL XABORT('LIBAPL: SELF SHIELDING FAILURE (3).')
      IF(SB(L,IMT).GE.1.0E10) THEN
*        USE AN INFINITE DILUTION VALUE.
         AUX=0.0
      ELSE IF(IFL.NE.0) THEN
*        USE THE EFFECTIVE FLUX RECOVERED IN THE APOLIB.
         AUX=PHI(L,IFL)
      ELSE IF((IAS.NE.0).AND.(ISS.NE.0)) THEN
*        COMPUTE THE EFFECTIVE FLUX.
         TMP1=0.0D0
         DO 664 IG2=1,N2-1
  664    TMP1=TMP1+SCAT(IG2,L)*PHI(IG2,ISS)*DELTA(IG2)/DELTA(L)
         IF(TMP1.GT.5.0E-3*PHI(L,ISS)) THEN
*           USE A SIMPLIFIED MODEL.
            AUX=PHI(L,IAS)
         ELSE
*           USE A SLOWING-DOWN BALANCE EQUATION.
            TMP=TMP1
            DO 665 IG2=N2,N2+N6-1
  665       TMP=TMP+SCAT(IG2,L)*PHI(IG2,ISS)*DELTA(IG2)/DELTA(L)
            AUX=PHI(L,IAS)+PHI(L,ISS)-TMP
         ENDIF
      ELSE IF(IAS.NE.0) THEN
*        COMPUTE THE EFFECTIVE FLUX USING A SIMPLIFIED MODEL.
         AUX=PHI(L,IAS)
      ELSE
         AUX=0.0
      ENDIF
      SS1(L)=1.0-AUX/SB(L,IMT)
      IF((SS1(L).LE.0.0).OR.(SS1(L).GT.1.1)) THEN
        WRITE(IOUT,8003) L,SS1(L),AUX,SEIM
      ENDIF
      SSS(L)=0.0
      IF(IFS.GT.0) SSS(L)=PHI(L,IFS)/SS1(L)
      IF(IAS.GT.0) SIGA(L)=PHI(L,IAS)/SS1(L)
      IF(ISS.GT.0) SIGS(L)=PHI(L,ISS)/SS1(L)
      IF(IMPX.GE.10)
     >  WRITE(IOUT,6031) L,SS1(L),SSS(L),SIGA(L),SIGS(L),
     >                   SEIM,SB(L,IMT),AUX
  670 CONTINUE
*
      ITYPRO(16)=1
      CALL LCMPUT(IPLIB,'NWT0',NGRO,2,SS1)
      IF(ITYPRO(7).NE.0) THEN
        DO 541 IG1=1,NGRO
          SIGN(IG1)=SIGA(IG1)
 541    CONTINUE
        CALL LCMPUT(IPLIB,'NG',NGRO,2,SIGN)
      ENDIF
      DO 671 I=1,NGRO
  671 SIGA(I)=SIGA(I)+SIGS(I)
      CALL LCMPUT(IPLIB,'TOTAL',NGRO,2,SIGA)
      IF(ISS.NE.0) THEN
         CALL LCMPUT(IPLIB,'SIGS 0',NGRO,2,SIGS)
         IGAR=0
         DO 675 IG2=1,NGRO
         DO 675 IG1=IJJ(IG2),IJJ(IG2)-NJJ(IG2)+1,-1
         IGAR=IGAR+1
  675    GAR(IGAR)=SCAT(IG1,IG2)*SIGS(IG1)
         CALL LCMPUT(IPLIB,'SCAT 0',IGAR,2,GAR)
         IF(ITYPRO(NDPROC+2).GT.0) THEN
            CALL LCMGET(IPLIB,'SIGS 1',SIG1)
            DO 676 I=1,NGRO
  676       SIG1(I)=SIG1(I)*SIGS(I)/VECT(I)
            CALL LCMPUT(IPLIB,'SIGS 1',NGRO,2,SIG1)
            CALL LCMGET(IPLIB,'SCAT 1',GAR)
            CALL LCMGET(IPLIB,'NJJ  1',NJJ)
            CALL LCMGET(IPLIB,'IJJ  1',IJJ)
            IGAR=0
            DO 680 IG2=1,NGRO
            DO 680 IG1=IJJ(IG2),IJJ(IG2)-NJJ(IG2)+1,-1
            IGAR=IGAR+1
  680       GAR(IGAR)=GAR(IGAR)*SIGS(IG1)/VECT(IG1)
            CALL LCMPUT(IPLIB,'SCAT 1',IGAR,2,GAR)
         ENDIF
         IF(ITYPRO(2).GT.0) THEN
            CALL LCMGET(IPLIB,'TRANC',SIG1)
            DO 677 I=1,NGRO
  677       SIG1(I)=SIG1(I)*SIGS(I)/VECT(I)
            CALL LCMPUT(IPLIB,'TRANC',NGRO,2,SIG1)
         ENDIF
      ENDIF
      IF(IFS.NE.0) THEN
         IF(ITYPRO(3).GT.0) THEN
            CALL LCMGET(IPLIB,'NUSIGF',SIGS)
            CALL LCMGET(IPLIB,'NFTOT',VECT)
            DO 690 I=1,NGRO
            IF(SIGS(I).EQ.0.0) GO TO 690
            VECT(I)=VECT(I)*SSS(I)/SIGS(I)
  690       CONTINUE
            IF(ITYPRO(7).NE.0) THEN
              DO 542 IG1=1,NGRO
                SIGN(IG1)=SIGN(IG1)-VECT(IG1)
 542          CONTINUE
            ENDIF
            CALL LCMPUT(IPLIB,'NFTOT',NGRO,2,VECT)
         ENDIF
         CALL LCMPUT(IPLIB,'NUSIGF',NGRO,2,SSS)
      ELSE
         IF(ITYPRO(3).GT.0) THEN
            CALL LCMGET(IPLIB,'NFTOT',VECT)
            IF(ITYPRO(7).NE.0) THEN
              DO 543 IG1=1,NGRO
                SIGN(IG1)=SIGN(IG1)-VECT(IG1)
 543          CONTINUE
            ENDIF
         ENDIF
      ENDIF
      IF(ITYPRO(7).GT.0) THEN
        CALL LCMPUT(IPLIB,'NG',NGRO,2,SIGN)
      ENDIF
      CALL LCMPUT(IPLIB,'XS-SAVED',NPROC,1,ITYPRO)
      CALL LCMSIX(IPLIB,' ',2)
      GO TO 50
*
*     CHECK IF ALL NBISO ISOTOPES HAVE BEEN PROCESSED.
*
  700 CONTINUE
      NISOT=0
      DO 710 I=1,NBISO
      IF(ITISOM(I).EQ.ILIB) THEN
         IF((IPR(1,I).EQ.0).AND.(.NOT.NOTG)) THEN
            GO TO 45
         ELSE IF((IPR(1,I).EQ.0).AND.NOTG) THEN
            WRITE(IOUT,8000) (ISONAM(ITC,I),ITC=1,3),CFILNA
            NISOT=NISOT+1
         ELSE IF((IPR(2,I).EQ.0).AND.(.NOT.NOTG)) THEN
            GO TO 45
         ELSE IF((IPR(2,I).EQ.0).AND.NOTG) THEN
            WRITE(IOUT,8001) (ISHINA(ITC,I),ICT=1,2),CFILNA
            NISOT=NISOT+1
         ENDIF
      ENDIF
  710 CONTINUE
*
*     CLOSE APOLIB FILE.
*
      IER=KDRCLS(NIN,1)
      IF(IER.LT.0) THEN
        WRITE(HSMG,'(37HLIBAPL: UNABLE TO CLOSE LIBRARY FILE ,A8)')
     >    CFILNA
        CALL XABORT(HSMG)
      ENDIF
      IF((IMPX.GT.0).AND.(NED.GT.0))
     >  WRITE(IOUT,6040) (HVEC(I),I=1,NED)
      IF(NISOT.GT.0) CALL XABORT('LIBAPL: MISSING ISOTOPES')
      RETURN
C----
C  FORMAT
C----
 6000 FORMAT(/35H PROCESSING APOLLO-1 LIBRARY NAMED ,A8)
 6001 FORMAT(1X,'TREATING ISOTOPE    ',A12,' = ',I10/
     >       20X,18A4)
 6002 FORMAT(1X,'SELF-SHIELDING      ',A12,' = ',I10/
     >       20X,18A4)
 6010 FORMAT(20X,' VECTOR CROSS SECTION DATA = ',
     >       I12,2X,8I3)
 6011 FORMAT(20X,' SCATTERING CROSS SECTION DATA = ',20I3)
 6012 FORMAT(20X,' SELF-SHIELDING CROSS SECTION DATA = ',
     >       5I5)
 6020 FORMAT(8H ISOTOPE,I12,5X,8HRECORD =,I3,5X,10HREACTION =,I3)
 6021 FORMAT(1X,1P,10E13.5)
 6022 FORMAT(8H ISOTOPE,I12,5X,20HDIFFUSION MATRIX NB.,I3,5X,
     > 6HTYPE =,I3,5X,6HNEXU =,I3,5X,6HNEXV =,I3,5X,6HNEXW =,I4,5X,
     > 12HANISOTROPY =,I3/(1P,10E13.5))
 6030 FORMAT(/5X,'GROUP',11X,'PHI0',10X,'SIGF0',10X,'SIGA0',10X,
     1 'SIGS0',10X,'DILUT',13X,'SB',12X,'AUX')
 6031 FORMAT(5X,I5,1P,8E15.5)
 6040 FORMAT(/39H EXTRA REACTION EDITS FOUND ON APOLIB: ,5A7)
C----
C  WARNINGS
C----
 8000 FORMAT(/' LIBAPL: *** WARNING ***'/
     >        ' ISOTOPE ',3A4,
     >        ' MISSING ON APOLIB FILE ',A8/
     >        ' ***********************')
 8001 FORMAT(/' LIBAPL: *** WARNING ***'/
     >        ' SELF-SHIELDING ISOTOPE ',3A4,
     >        ' MISSING ON APOLIB FILE ',A8/
     >        ' ***********************')
 8002 FORMAT(/' LIBAPL: *** WARNING ***'/
     >        ' NUMBER OF THERMAL GROUPS CHANGED FROM ',I4,' TO ',I4/
     >        ' ***********************')
 8003 FORMAT(/' LIBAPL: *** WARNING ***'/
     >        ' INVALID VALUE OF PHI0 IN GROUP ' ,I4/1P,
     >        ' PHI0=',E12.3,3X,'AUX=',E12.3,3X,'SEIM=',E12.3/
     >        ' ***********************')
 8004 FORMAT(/' LIBAPL: *** WARNING ***'/
     >        ' ISOTOPE ',A12,/
     >        ' UNKNOWN REACTION = ',I3,5X,'RECORD = ',I3/
     >        ' ***********************')
 8005 FORMAT(/' LIBAPL: *** WARNING ***'/
     >        ' UNABLE TO RECOVER THE GROUP STRUCTURE.'/
     >        ' ***********************')
C----
C  ERRORS
C----
 9000 FORMAT('LIBAPL: INSUFFICIENT VALUE OF ',A6)
      END
