*DECK LIBCON
      SUBROUTINE LIBCON(IMX,NBISO,ISOMIX,DENISO,AWRISO,
     >                  DENMIX,IN)
C
C-----------------------------------------------------------------------
C
C CONVERT WEIGHT PERCENT TO ATOMIC DENSITY.
C
C INPUT PARAMETERS:
C  IMX     : MIXTURE INDEX TO PROCESS.
C  NBISO   : NUMBER OF ISOTOPES PRESENT IN THE CALCULATION DOMAIN.
C  ISOMIX  : MIX NUMBER OF EACH ISOTOPE. ISOMIX(NBISO)
C  DENISO  : DENSITY OF EACH ISOTOPE. DENISO(NBISO)
C  AWRISO  : ATOMIC WEIGHT OF EACH ISOTOPE. AWRISO(NBISO)
C  DENMIX  : MIXTURE DENSITY IN G*CM**(-3)
C  IN      : =1 CONVERSION OF WGT% TO NB ATOMS WITH DENMIX
C            =2 CONVERSION OF NB ATOMS TO WGT% AND DENMIX
C
C OUTPUT PARAMETERS:
C FOR IN=1,
C  DENISO  : NUMBER DENSITY FOR ISOTOPES PRESENT IN MIXTURE IMX.
C            DIMENSION DEN(NBISO)
C FOR IN=2,
C  DENISO  : WEIGHT PERCENT FOR ISOTOPES PRESENT IN MIXTURE IMX.
C  DENMIX  : MIXTURE DENSITY.
C
C PARAMETERS:
C  IOUT    : OUTPUT FILE = 6
C
C--------------------------------------- AUTHOR: G. MARLEAU 93/25/01 ---
C--------------------------------------- MODIFY: R. ROY     95/10/03 ---
C--------------------------------------- MODIFY: G. MARLEAU 97/01/22 ---
C
      IMPLICIT NONE
C----
C PARAMETERS
C----
      INTEGER          IOUT
      PARAMETER       (IOUT=6)
C----
C FUNCTIONS
C----
      DOUBLE PRECISION XDRCST
C----
C LOCAL VARIABLES
C----
      INTEGER          IMX,NBISO,IN,ISO,ISOMIX(NBISO)
      REAL             DENMIX,TWPC,WMIX,DENISO(NBISO),AWRISO(NBISO)
      DOUBLE PRECISION AVCON
C------
C  COMPUTE NUMBER DENSITIES FOR ISOTOPES
C------
      AVCON=1.0D-24*XDRCST('Avogadro','N/moles')
     >             /XDRCST('Neutron mass','amu')
      IF(IN.EQ.1) THEN
        IF(DENMIX.EQ.-1.0) CALL XABORT('LIBCON: DENMIX NOT DEFINED')
         TWPC=0.0
         DO 120 ISO=1,NBISO
           IF(ISOMIX(ISO).EQ.IMX) TWPC=DENISO(ISO)+TWPC
 120     CONTINUE
         IF(TWPC.EQ.0.0) THEN
           IF(DENMIX.EQ.0.0) THEN
             RETURN
           ELSE
             CALL XABORT('LIBCON: A MIXTURE OF DENSITY > 0.0 '//
     >         'HAS ALL ITS ISOTOPIC WEIGHT PERCENT = 0.0')
           ENDIF
         ENDIF
         WMIX=DENMIX*AVCON/TWPC
         DO 130 ISO=1,NBISO
           IF(ISOMIX(ISO).EQ.IMX) THEN
             IF(AWRISO(ISO).GT.0.0) THEN
               DENISO(ISO)=DENISO(ISO)*WMIX/AWRISO(ISO)
             ELSE
               DENISO(ISO)=0.0
             ENDIF
           ENDIF
 130     CONTINUE
      ELSE IF(IN.EQ.2) THEN
C------
C  COMPUTE MIXTURE DENSITIES AND ISOTOPIC WEIGHT PERCENTS
C  (NORMALIZED TO 100.)
C------
         DENMIX=0.0
         DO 220 ISO=1,NBISO
           IF(ISOMIX(ISO).EQ.IMX) THEN
             DENISO(ISO)=DENISO(ISO)*AWRISO(ISO)/AVCON
             DENMIX=DENMIX+DENISO(ISO)
           ENDIF
 220     CONTINUE
         IF(DENMIX.NE.0.0) THEN
           DO 230 ISO=1,NBISO
             IF(ISOMIX(ISO).EQ.IMX)
     >         DENISO(ISO)=100.*DENISO(ISO)/DENMIX
 230       CONTINUE
         ENDIF
      ELSE
         CALL XABORT('LIBCON: INVALID *IN* VALUE')
      ENDIF
      RETURN
      END
