*DECK LIBDRA
      SUBROUTINE LIBDRA(IPLIB,IMPX,CFILNA,NGRO,NBISO,NL,ISONAM,ISONRF,
     >                  TN,AW,SN,SB,ILIB,ITISOM,DELTA,NGFR,ITRANC,
     >                  NPROC,INDPRO,ITYPRO,XSREC,SCAT,XSCMP,NJJ,IJJ)
C
C-----------------------------------------------------------------------
C
C TRANSCRIPTION OF THE USEFULL INTERPOLATED MICROSCOPIC CROSS SECTION
C DATA FROM A MICROSCOPIC X-SECTION LIBRARY (DRAGLIB FORMAT) TO LCM.
C
C INPUT PARAMETERS:
C  IPLIB   : POINTER TO THE LATTICE MICROSCOPIC CROSS SECTION LIBRARY
C            (L_LIBRARY SIGNATURE).
C  IMPX    : PRINT FLAG.
C  CFILNA  : NAME OF THE DRAGON LIBRARY FILE CHARACTER CFILNA*8
C  NGRO    : NUMBER OF ENERGY GROUPS.
C  NBISO   : NUMBER OF ISOTOPES PRESENT IN THE CALCULATION DOMAIN.
C  NL      : NUMBER OF LEGENDRE ORDERS REQUIRED IN THE CALCULATION.
C            NL=1 OR HIGHER.
C  ISONAM  : LOCAL NAMES OF ISOTOPES. ISONAM(3,NBISO)
C  ISONRF  : LIBRARY NAMES OF ISOTOPES. ISONRF(3,NBISO)
C  TN      : TEMPERATURE OF ISOTOPESS. DIMENSION TN(NBISO)
C  AW      : ATOMIC WEIGHT OF ISOTOPE. DIMENSION AW(NBISO)
C  SN      : DILUTION CROSS SECTION IN EACH ENERGY GROUP OF EACH
C            ISOTOPE. SN(NGRO,NBISO)
C  SB      : DILUTION CROSS SECTION AS USED BY LIVOLANT AND JEANPIERRE
C            NORMALIZATION. SB(NGRO,NBISO)
C  ILIB    : LIBRARY TO PROCESS
C  ITISOM  : ISOTOPE WITH INDEX ILIB IS PROCESSED. ITISOM(NBISO)
C  DELTA   : LETHARGY WIDTH OF GROUPS. DELTA(NGRO)
C  NGFR    : NUMBER OF FAST+RESONANCE GROUPS.
C  ITRANC  : TRANSPORT CORRECTION
C            0 -> NONE ; 1 -> APOL; 2 -> WIMS
C  NPROC   : NUMBER OF CROSS SECTION TYPE PROCESSED
C  INDPRO  : FLAG FOR CROSS SECTION TYPE TO PROCESS INDPRO(NPROC)
C SCRATCH STORAGE:
C  ITYPRO  : FLAG FOR CROSS SECTION TYPE PROCESSED ITYPRO(NPROC)
C  XSREC   : DIMENSION XSREC(NGRO,NPROC)
C  SCAT    : DIMENSION SCAT(NGRO,NGRO,NL)
C  XSCMP   : DIMENSION XSCMP(NGRO*(NGRO+2))
C  NJJ     : DIMENSION NJJ(NGRO)
C  IJJ     : DIMENSION IJJ(NGRO)
C
C----------------------------------- AUTHOR:   A. HEBERT  29/01/90 ----
C----------------------------------- MODIFIED: G. MARLEAU 97/07/02 ----
C
      IMPLICIT NONE
C----
C PARAMETERS
C----
      INTEGER       IOUT,NDPROC,MAXTMP,MAXDIL,NOTX
      PARAMETER    (IOUT=6,NDPROC=20,MAXTMP=50,MAXDIL=50,NOTX=3)
C----
C MEMORY ALLOCATION PARAMETERS
C----
      INTEGER       IBASE(1)
      REAL          RBASE(1)
      COMMON        RBASE
      EQUIVALENCE  (RBASE(1),IBASE(1))
      INTEGER       ITITLE,INFS,IBIN,IEBIN,ILSDIL,ITERP,IXSRE2,ISCAT2,
     1              IBIN2
C----
C DIMENSIONS
C----
      INTEGER       NGRO,NBISO,NL
C----
C LOCAL VARIABLES
C----
      INTEGER       IPLIB,IMPX,NGFR,ITRANC,IPDRL,LENGT,ITYLCM,I2,J,I1,
     1              IG4EV,IMX,ITC,LTITLE,LBIN,ITYXSM,IG1,I,NTMP,ITM,IG2
C
      INTEGER       ISONRF(3,NBISO),ILIB,ITISOM(NBISO),ISONAM(3,NBISO),
     1              NPROC,INDPRO(NPROC),ITYPRO(NPROC),
     2              NJJ(NGRO),IJJ(NGRO)
C
      REAL          TN(NBISO),AW(NBISO),SN(NGRO,NBISO),
     1              SB(NGRO,NBISO),DELTA(NGRO),TEMP(MAXTMP),
     2              XSREC(NGRO,NPROC),SCAT(NGRO,NGRO,NL),
     3              XSCMP(NGRO*NGRO)
C
      CHARACTER     CFILNA*8,CD*4,HSMG*131,HTITLE*80,HNISOR*12,
     1              HNAMIS*12
      DOUBLE PRECISION FACTOR,TERP(MAXTMP),TERPM
C----
C  OPEN FILE AND READ INFORMATION DATA RECORDS
C----
      CALL LCMOP(IPDRL,CFILNA,2,2,0)
      IF(IMPX.GT.0) WRITE(IOUT,6000) CFILNA
      CALL LCMLEN(IPDRL,'README',LENGT,ITYLCM)
      IF((IMPX.GT.0).AND.(LENGT.GT.0)) THEN
        CALL SETARA(IBASE,LENGT,ITITLE)
        CALL LCMGET(IPDRL,'README',IBASE(ITITLE))
        WRITE(IOUT,6001)
        I2=0
        DO 100 J=0,LENGT/20
          I1=I2+1
          I2=MIN(I1+19,LENGT)
          WRITE(HTITLE,'(20A4)') (IBASE(ITITLE+I-1),I=I1,I2)
          WRITE(IOUT,'(1X,A80)') HTITLE
 100    CONTINUE
        CALL RLSARA(IBASE(ITITLE))
      ENDIF
      IG4EV=0
      IF(ITRANC.EQ.2) THEN
        IG4EV=NGFR
      ENDIF
*----
*  READ THROUGH DRAGON FILE AND ACCUMULATE CROSS SECTIONS
*  FOR THIS RANGE OF MATS, LEGENDRE ORDERS, AND GROUPS.
*----
      DO 110 IMX=1,NBISO
        IF(ITISOM(IMX).EQ.ILIB) THEN
          WRITE(HNAMIS,'(3A4)') (ISONAM(ITC,IMX),ITC=1,3)
          WRITE(HNISOR,'(3A4)') (ISONRF(ITC,IMX),ITC=1,3)
          CALL LCMLEN(IPDRL,HNISOR,LENGT,ITYLCM)
          IF(LENGT.EQ.0) THEN
            CALL LCMLIB(IPDRL)
            WRITE(HSMG,9000) HNAMIS,HNISOR,CFILNA
            CALL XABORT(HSMG)
          ENDIF
          CALL LCMSIX(IPDRL,HNISOR,1)
          IF(IMPX.GT.0) WRITE(IOUT,6002) HNAMIS
          CALL LCMGET(IPDRL,'AWR',AW(IMX))
          CALL LCMLEN(IPDRL,'README',LTITLE,ITYLCM)
          IF(LTITLE.GT.0) THEN
            CALL SETARA(IBASE,LTITLE,ITITLE)
            CALL LCMGET(IPDRL,'README',IBASE(ITITLE))
            IF(IMPX.GT.0) THEN
              WRITE(IOUT,6003)
              I2=0
              DO 120 J=0,LTITLE/20
                I1=I2+1
                I2=MIN(I1+19,LTITLE)
                WRITE(HTITLE,'(20A4)') (IBASE(ITITLE+I-1),I=I1,I2)
                WRITE(IOUT,'(1X,A80)') HTITLE
 120          CONTINUE
            ENDIF
          ENDIF
*----
*  RECOVER BIN TYPE INFORMATION (IF AVAILABLE).
*----
          LBIN=0
          CALL SETARA(IBASE,NGRO,INFS)
          CALL LCMLEN(IPDRL,'BIN-NFS',LENGT,ITYXSM)
          IF(LENGT.GT.0) THEN
            CALL LCMGET(IPDRL,'BIN-NFS',IBASE(INFS))
            DO 130 IG1=0,NGRO-1
              LBIN=LBIN+IBASE(INFS+IG1)
 130        CONTINUE
            CALL SETARA(RBASE,3*LBIN,IBIN)
            CALL SETARA(RBASE,LBIN+1,IEBIN)
            CALL LCMGET(IPDRL,'BIN-ENERGY',RBASE(IEBIN))
          ENDIF
          CALL SETARA(IBASE,2*NL,ILSDIL)
          CALL SETARA(RBASE,MAXDIL*NGRO,ITERP)
          CALL LCMLEN(IPDRL,'TEMPERATURE',NTMP,ITYLCM)
          IF(NTMP.GT.MAXTMP) CALL XABORT
     >      ('LIBDRA: INVALID MAXTMP.')
          IF(NTMP.EQ.0) THEN
            CALL LIBDRB(IPDRL,IMPX,NGRO,LBIN,NL,SN(1,IMX),SB(1,IMX),
     >                  DELTA,NPROC,INDPRO,ITYPRO,RBASE(IBIN),XSREC,
     >                  SCAT,XSCMP,NJJ,IJJ,RBASE(ITERP),IBASE(ILSDIL))
          ELSE
C----
C  PERFORM TEMPERATURE LAGRANGIAN INTERPOLATION (ORDER ABS(NOTX)).
C----
            CALL LCMGET(IPDRL,'TEMPERATURE',TEMP)
            CALL LIBLEX(NTMP,TN(IMX),TEMP,NOTX,TERP)
            CALL XDRSET(XSREC,NGRO*NPROC,0.0)
            CALL XDRSET(SCAT,NGRO*NGRO*NL,0.0)
            IF(LBIN.GT.0) THEN
              CALL XDRSET(RBASE(IBIN),3*LBIN,0.0)
            ENDIF
            CALL SETARA(RBASE,NGRO*NPROC,IXSRE2)
            CALL SETARA(RBASE,NGRO*NGRO*NL,ISCAT2)
            CALL SETARA(RBASE,3*LBIN,IBIN2)
            FACTOR=1.0D0
            DO 140 ITM=1,NTMP
              TERPM=TERP(ITM)
              FACTOR=FACTOR-TERPM
              IF(TERPM.NE.0.0D0) THEN
                IF(IMPX.GT.10) WRITE(IOUT,6004) TEMP(ITM),TERPM
                WRITE(CD,'(I4)') ITM
                CALL LCMSIX (IPDRL,'SUBTMP'//CD,1)
                CALL LIBDRB(IPDRL,IMPX,NGRO,LBIN,NL,SN(1,IMX),
     >                      SB(1,IMX),DELTA,NPROC,INDPRO,ITYPRO,
     >                      RBASE(IBIN2),RBASE(IXSRE2),RBASE(ISCAT2),
     >                      XSCMP,NJJ,IJJ,RBASE(ITERP),IBASE(ILSDIL))
                CALL LCMSIX (IPDRL,' ',2)
                CALL LIBDRC(NGRO,LBIN,NL,NPROC,INDPRO,ITYPRO,TERPM,
     >                      XSREC,SCAT,RBASE(IBIN),RBASE(IXSRE2),
     >                      RBASE(ISCAT2),RBASE(IBIN2))
              ENDIF
 140        CONTINUE
            CALL RLSARA(RBASE(IBIN2))
            CALL RLSARA(RBASE(ISCAT2))
            CALL RLSARA(RBASE(IXSRE2))
            IF(ABS(FACTOR).GT.1.0D-4) CALL XABORT
     >        ('LIBDRA: TERP ERROR')
          ENDIF
          CALL RLSARA(RBASE(ITERP))
          CALL RLSARA(IBASE(ILSDIL))
          CALL LCMSIX(IPDRL,' ',2)
*----
*  COMPUTE TRANSPORT CORRECTION
*----
          IF(NL.GE.2.AND.ITRANC.GE.1) THEN
            DO 150 IG1=1,IG4EV
              XSREC(IG1,2)=0.0
              DO 151 IG2=1,NGRO
                XSREC(IG1,2)=XSREC(IG1,2)
     >            +SCAT(IG1,IG2,2)*DELTA(IG2)/DELTA(IG1)
 151          CONTINUE
 150        CONTINUE
            DO 160 IG1=IG4EV+1,NGRO
              XSREC(IG1,2)=XSREC(IG1,NDPROC+2)
 160        CONTINUE
          ENDIF
*----
*  SAVE CROSS SECTION DATA ON LCM
*----
          CALL LCMSIX(IPLIB,HNAMIS,1)
          IF(LTITLE.GT.0) THEN
            CALL LCMPUT(IPLIB,'README',LTITLE,3,IBASE(ITITLE))
            CALL RLSARA(IBASE(ITITLE))
          ENDIF
          CALL XDRLGS(IPLIB,1,NPROC,0,NL-1,1,NGRO,INDPRO,ITYPRO,
     >                XSREC,SCAT,XSCMP,NJJ,IJJ)
          IF(LBIN.GT.0) THEN
            CALL LCMPUT(IPLIB,'BIN-NFS',NGRO,1,IBASE(INFS))
            CALL LCMPUT(IPLIB,'BIN-ENERGY',LBIN+1,2,RBASE(IEBIN))
            CALL LCMPUT(IPLIB,'BIN-TOTAL',LBIN,2,RBASE(IBIN))
            CALL LCMPUT(IPLIB,'BIN-SIGS 0',LBIN,2,RBASE(IBIN+LBIN))
            CALL LCMPUT(IPLIB,'BIN-NUSIGF',LBIN,2,RBASE(IBIN+2*LBIN))
            CALL RLSARA(RBASE(IEBIN))
            CALL RLSARA(RBASE(IBIN))
          ENDIF
          CALL RLSARA(IBASE(INFS))
          IF(IMPX.GE.100) CALL LCMLIB(IPLIB)
          CALL LCMSIX(IPLIB,' ',2)
        ENDIF
 110  CONTINUE
*----
*  CLOSE LIBRARY FILE
*----
      CALL LCMCL(IPDRL,1)
      RETURN
*----
*  PRINT FORMATS
*----
 6000 FORMAT(' PROCESSING DRAGON LIBRARY NAME ',A8)
 6001 FORMAT(' X-SECTION LIBRARY INFO:')
 6002 FORMAT(' PROCESSING ISOTOPE/MATERIAL ',A12)
 6003 FORMAT(' ISOTOPE/MATERIAL INFO:')
 6004 FORMAT(10X,' DRAGLIB ACCESS AT TEMPERATURE =',E12.4,
     >       ' KELVIN -> INTERPOLATION FACTOR = ',E12.4)
*----
*  ABORT FORMATS
*----
 9000 FORMAT('LIBDRA: MATERIAL/ISOTOPE ',A12,' = ',A12,
     >       ' IS MISING ON DRAGON LIBRARY FILE ',A8)
      END
