*DECK LIBDRB
      SUBROUTINE LIBDRB(IPDRL,IMPX,NGRO,LBIN,NL,SN,SB,DELTA,NPROC,
     >                  INDPRO,ITYPRO,BIN,XSREC,SCAT,XSCMP,NJJ,IJJ,
     >                  TERP,LSDIL)
C
C-----------------------------------------------------------------------
C
C READ AND INTERPOLATE IN DILUTION ONE ISOTOPE IN DRAGLIB FORMAT AT A
C SELECTED TEMPERATURE.
C
C INPUT PARAMETERS:
C  IPDRL   : POINTER TO THE DRAGLIB (L_DRAGLIB SIGNATURE).
C  IMPX    : PRINT FLAG.
C  NGRO    : NUMBER OF ENERGY GROUPS.
C  LBIN    : NUMBER OF FINE GROUPS.
C  NL      : NUMBER OF LEGENDRE ORDERS REQUIRED IN THE CALCULATION.
C            NL=1 OR HIGHER.
C  SN      : DILUTION CROSS SECTION IN EACH ENERGY GROUP. A VALUE OF
C            1.0E10 IS USED FOR INFINITE DILUTION. DIMENSION SN(NGRO)
C  SB      : DILUTION CROSS SECTION AS USED IN LIVOLANT AND JEANPIERRE
C            NORMALIZATION. DIMENSION SB(NGRO)
C  DELTA   : LETHARGY WIDTHS. DIMENSION DELTA(NGRO)
C  NPROC   : NUMBER OF CROSS SECTION TYPE PROCESSED
C  INDPRO  : FLAG FOR CROSS SECTION TYPE TO PROCESS INDPRO(NPROC)
C  ITYPRO  : FLAG FOR CROSS SECTION TYPE PROCESSED ITYPRO(NPROC)
C OUTPUT PARAMETERS:
C  BIN     : BIN(IGR,1) TOTAL FINE GROUP CROSS SECTIONS;
C            BIN(IGR,2) ISOTROPIC SCATTERING FINE GROUP CROSS SECTIONS;
C            BIN(IGR,3) NU*FISSION FINE GROUP CROSS SECTIONS.
C            DIMENSION BIN(LBIN,3)
C  XSREC   : DIMENSION XSREC(NGRO,NPROC)
C  SCAT    : DIMENSION SCAT(NGRO,NGRO,NL)
C
C SCRATCH STORAGE:
C  XSCMP   : DIMENSION XSCMP(NGRO*(NGRO+2))
C  NJJ     : DIMENSION NJJ(NGRO)
C  IJJ     : DIMENSION IJJ(NGRO)
C  TERP    : DIMENSION TERP(50,NGRO)
C  LSDIL   : DIMENSION LSDIL(2,NL)
C
C----------------------------------- AUTHOR:   A. HEBERT  29/01/90 ----
C----------------------------------- MODIFIED: G. MARLEAU 97/07/03 ----
C
      IMPLICIT NONE
C----
C PARAMETERS
C----
      INTEGER       MAXDIL,IOUT,NDPROC
      PARAMETER    (IOUT=6,NDPROC=20,MAXDIL=50)
C----
C DIMENSIONS
C----
      INTEGER       NGRO,NL,NPROC,LBIN
C----
C LOCAL VARIABLES
C----
      INTEGER       IPDRL,IMPX,IL,LENGTS,ITYLCM,IG1,IXSS,IGMAX,IG2,
     1              LENGTX,LENGTP,IXSR,LENGT,NDIL,IGMIN,IFIRST,I,NGRODP,
     2              NGROIN,NGRRE,IDIL,NCORF,NCORD,IXSF
C
      INTEGER       INDPRO(NPROC),ITYPRO(NPROC),KMLPRO(NDPROC),
     1              NJJ(NGRO),IJJ(NGRO),LSDIL(2,NL)
C
      REAL          DILX
C
      REAL          SN(NGRO),SB(NGRO),DELTA(NGRO),BIN(LBIN,3),
     1              XSREC(NGRO,NPROC),SCAT(NGRO,NGRO,NL),
     2              XSCMP(NGRO*(NGRO+2)),TERP(MAXDIL,NGRO),DILUT(MAXDIL)
      CHARACTER     CM*2,CD*4,HSMG*131,NAMDXS(NDPROC)*6
      DOUBLE PRECISION TMP,SQD,SQ0,SQ1,SQ2,SQ3
C----
C DATA
C----
      DATA NAMDXS  /'TOTAL ','TRANC ','NUSIGF','NFTOT ','CHI   ',
     1              'NU    ','NG    ','NHEAT ','N2N   ','N3N   ',
     2              'N4N   ','NP    ','NA    ','GOLD  ','ABS   ',
     3              'NWT0  ','STRD  ','STRD X','STRD Y','STRD Z'/
C
      SAVE          NAMDXS
C----
C   READ CROSS SECTION RECORDS
C----
      CALL XDRSET(XSREC,NGRO*NPROC,0.0)
      CALL XDRSET(SCAT,NGRO*NGRO*NL,0.0)
      CALL XDRSET(ITYPRO,NPROC,0)
      DO 100 IL=1,NL
        WRITE (CM,'(I2)') IL-1
        CALL LCMLEN(IPDRL,'SCAT'//CM,LENGTS,ITYLCM)
        CALL LCMLEN(IPDRL,'SIGS'//CM,LENGTX,ITYLCM)
        CALL LCMLEN(IPDRL,'PCAT'//CM,LENGTP,ITYLCM)
        IXSR=NDPROC+IL
        IF(LENGTX.GT.0) THEN
          ITYPRO(IXSR)=1
          CALL LCMGET(IPDRL,'SIGS'//CM,XSREC(1,IXSR))
        ENDIF
        IF(LENGTS.GT.0.OR.LENGTP.GT.0) THEN
          CALL LCMGET(IPDRL,'NJJ'//CM,NJJ)
          CALL LCMGET(IPDRL,'IJJ'//CM,IJJ)
          LENGT=0
          DO 110 IG1=1,NGRO
            LENGT=LENGT+NJJ(IG1)
 110      CONTINUE
          CALL XDRSET(XSCMP,LENGT,0.0)
          IF(LENGTS.GT.0) THEN
            CALL LCMGET(IPDRL,'SCAT'//CM,XSCMP)
            IXSS=0
            DO 120 IG1=1,NGRO
              IGMAX=IJJ(IG1)
              IGMIN=IGMAX-NJJ(IG1)+1
              DO 121 IG2=IGMAX,IGMIN,-1
                IXSS=IXSS+1
                SCAT(IG1,IG2,IL)=XSCMP(IXSS)
 121          CONTINUE
 120        CONTINUE
            IF(LENGTX.EQ.0) THEN
              ITYPRO(IXSR)=1
              DO 122 IG1=1,NGRO
                TMP=0.0D0
                DO 123 IG2=1,NGRO
                  TMP=TMP+SCAT(IG2,IG1,IL)
 123            CONTINUE
                XSREC(IG1,IXSR)=TMP
 122          CONTINUE
            ENDIF
          ELSE IF(LENGTX.GT.0) THEN
            CALL LCMGET(IPDRL,'PCAT'//CM,XSCMP)
            IXSS=0
            DO 130 IG1=1,NGRO
              IGMAX=IJJ(IG1)
              IGMIN=IGMAX-NJJ(IG1)+1
              DO 131 IG2=IGMAX,IGMIN,-1
                IXSS=IXSS+1
                SCAT(IG1,IG2,IL)=XSCMP(IXSS)*XSREC(IG2,IXSR)
 131          CONTINUE
 130        CONTINUE
          ENDIF
        ELSE IF(LENGTX.GT.0) THEN
          DO 140 IG1=1,NGRO
            SCAT(IG1,IG1,IL)=XSREC(IG1,IXSR)
 140      CONTINUE
        ENDIF
 100  CONTINUE
      DO 150 IXSR=1,NDPROC
        IF(INDPRO(IXSR).EQ.1) THEN
          CALL LCMLEN(IPDRL,NAMDXS(IXSR),LENGT,ITYLCM)
          IF(LENGT.GT.0) THEN
            ITYPRO(IXSR)=1
            CALL LCMGET(IPDRL,NAMDXS(IXSR),XSREC(1,IXSR))
          ENDIF
        ENDIF
 150  CONTINUE
      IXSR=14
      IF(ITYPRO(IXSR).EQ.0) THEN
        CALL XDRSET(XSREC(1,IXSR),NGRO,1.0)
        ITYPRO(IXSR)=1
      ENDIF
      IF(LBIN.GT.0) THEN
        CALL LCMGET(IPDRL,'BIN-TOTAL',BIN(1,1))
        CALL LCMGET(IPDRL,'BIN-SIGS 0',BIN(1,2))
        CALL LCMLEN(IPDRL,'BIN-NUSIGF',LENGT,ITYLCM)
        IF(LENGT.GT.0) THEN
          CALL LCMGET(IPDRL,'BIN-NUSIGF',BIN(1,3))
        ELSE
          CALL XDRSET(BIN(1,3),LBIN,0.0)
        ENDIF
      ENDIF
*----
*  PERFORM DILUTION INTERPOLATION
*----
      CALL LCMLEN(IPDRL,'DILUTION',NDIL,ITYLCM)
      IF(NDIL.GT.0) THEN
        IF(NDIL+1.GT.MAXDIL) CALL XABORT
     >    ('LIBDRB: INVALID MAXDIL')
        CALL LCMGET(IPDRL,'DILUTION',DILUT)
        IF(DILUT(NDIL).GE.1.0E10) CALL XABORT
     >    ('LIBDRB: DILUTION >= 1.0E10 IS INVALID ON DRAGLIB')
        DILUT(NDIL+1)=1.0E10
        DO 200 IG1=1,NGRO
          DILX=MIN(SN(IG1),1.0E10)
          IF(DILX.LE.0.0) THEN
            WRITE(HSMG,930) IG1
            CALL XABORT(HSMG)
          ENDIF
          CALL XDRSET(TERP(1,IG1),NDIL,0.0)
          IFIRST=0
          DO 210 I=1,NDIL+1
            IF(ABS(DILX-DILUT(I)).LE.1.0E-5*ABS(DILX)) THEN
              TERP(I,IG1)=1.0
              GO TO 205
            ELSE IF(DILX.LT.DILUT(I)) THEN
              IFIRST=I-1
              GO TO 215
            ENDIF
  210     CONTINUE
  215     CONTINUE
          SQD=SQRT(DILX)
          IF((IFIRST-1.GE.1).AND.(IFIRST+2.LE.NDIL)) THEN
            SQ0=SQRT(DILUT(IFIRST-1))
            SQ1=SQRT(DILUT(IFIRST))
            SQ2=SQRT(DILUT(IFIRST+1))
            SQ3=SQRT(DILUT(IFIRST+2))
            TERP(IFIRST-1,IG1)=(SQ1-SQD)*(SQ2-SQD)*(SQ3-SQD)
     >                        /(SQ1-SQ0)/(SQ2-SQ0)/(SQ3-SQ0)
            TERP(IFIRST,IG1)=(SQ0-SQD)*(SQ2-SQD)*(SQ3-SQD)
     >                      /(SQ0-SQ1)/(SQ2-SQ1)/(SQ3-SQ1)
            TERP(IFIRST+1,IG1)=(SQ0-SQD)*(SQ1-SQD)*(SQ3-SQD)
     >                        /(SQ0-SQ2)/(SQ1-SQ2)/(SQ3-SQ2)
            TERP(IFIRST+2,IG1)=(SQ0-SQD)*(SQ1-SQD)*(SQ2-SQD)
     >                        /(SQ0-SQ3)/(SQ1-SQ3)/(SQ2-SQ3)
          ELSE IF((IFIRST.EQ.1).AND.(IFIRST+2.LE.NDIL)) THEN
            SQ1=SQRT(DILUT(1))
            SQ2=SQRT(DILUT(2))
            SQ3=SQRT(DILUT(3))
            TERP(1,IG1)=(SQ2-SQD)*(SQ3-SQD)/(SQ2-SQ1)/(SQ3-SQ1)
            TERP(2,IG1)=(SQ1-SQD)*(SQ3-SQD)/(SQ1-SQ2)/(SQ3-SQ2)
            TERP(3,IG1)=(SQ1-SQD)*(SQ2-SQD)/(SQ1-SQ3)/(SQ2-SQ3)
          ELSE IF((IFIRST-1.GE.1).AND.(IFIRST+1.EQ.NDIL)) THEN
            SQ0=SQRT(DILUT(NDIL-2))
            SQ1=SQRT(DILUT(NDIL-1))
            SQ2=SQRT(DILUT(NDIL))
            TERP(NDIL-2,IG1)=(SQ1-SQD)*(SQ2-SQD)/(SQ1-SQ0)/(SQ2-SQ0)
            TERP(NDIL-1,IG1)=(SQ0-SQD)*(SQ2-SQD)/(SQ0-SQ1)/(SQ2-SQ1)
            TERP(NDIL,IG1)=(SQ0-SQD)*(SQ1-SQD)/(SQ0-SQ2)/(SQ1-SQ2)
          ELSE IF((IFIRST.EQ.0).OR.((IFIRST.EQ.1).AND.(NDIL.EQ.2)))THEN
            SQ0=SQRT(DILUT(1))
            SQ1=SQRT(DILUT(2))
            TERP(1,IG1)=(SQ1-SQD)/(SQ1-SQ0)
            TERP(2,IG1)=(SQ0-SQD)/(SQ0-SQ1)
          ELSE IF(IFIRST.EQ.NDIL) THEN
            TERP(NDIL,IG1)=DILUT(NDIL)/DILX
          ELSE
             CALL XABORT('LIBDRB: FAILURE OF DILUTION INTERPOLATION.')
          ENDIF
 205      CONTINUE
 200    CONTINUE
*----
*  FIND MAX LENGTH OF VECTORS ON SUBMAT
*----
        CALL XDRSET(KMLPRO,NDPROC,0)
        NGRODP=NGRO+1
        NGROIN=0
        NGRRE=0
        CALL XDRSET(LSDIL,2*NL,0)
        DO 220 IDIL=1,NDIL
          WRITE (CD,'(I4)') IDIL
          CALL LCMSIX(IPDRL,'SUBMAT'//CD,1)
          DO 230 IL=1,NL
            IXSR=NDPROC+IL
            IF(ITYPRO(IXSR).NE.0) THEN
              WRITE (CM,'(I2)') IL
              IF(LSDIL(1,IL).EQ.0) THEN
                CALL LCMLEN(IPDRL,'SCAT'//CM,LENGT,ITYLCM)
                IF(LENGT.GT.0) LSDIL(1,IL)=1
              ENDIF
              IF(LSDIL(2,IL).EQ.0) THEN
                CALL LCMLEN(IPDRL,'SIGS'//CM,LENGT,ITYLCM)
                IF(LENGT.GT.0) LSDIL(2,IL)=1
              ENDIF
            ENDIF
 230      CONTINUE
          DO 240 IXSR=1,NDPROC
            IF(ITYPRO(IXSR).NE.0) THEN
              CALL LCMLEN(IPDRL,NAMDXS(IXSR),LENGT,ITYLCM)
              KMLPRO(IXSR)=MAX(KMLPRO(IXSR),LENGT)
              NGRRE=MAX(NGRRE,KMLPRO(IXSR))
            ENDIF
 240      CONTINUE
          IXSR=16
          CALL LCMLEN(IPDRL,'NPHI',LENGT,ITYLCM)
          KMLPRO(IXSR)=MAX(KMLPRO(IXSR),LENGT)
          CALL LCMSIX(IPDRL,' ',2)
          NCORF=0
          DO 250 IG1=NGRO,1,-1
            IF(TERP(IDIL,IG1).NE.0.0) THEN
              NCORF=IG1
              GO TO 255
            ENDIF
 250      CONTINUE
 255      CONTINUE
          NGROIN=MAX(NCORF,NGROIN)
          NCORD=NGRO+1
          DO 260 IG1=1,NGROIN
            IF(TERP(IDIL,IG1).NE.0.0) THEN
              NCORD=IG1
              GO TO 265
            ENDIF
 260      CONTINUE
 265      CONTINUE
          NGRODP=MIN(NCORD,NGRODP)
 220    CONTINUE
        IF(NGRRE.GT.NGRO) CALL XABORT('LIBDRB: TOO MANY GROUPS.')
        IF(NGROIN.EQ.0.OR.NGRRE.EQ.0) THEN
          IXSR=16
          ITYPRO(IXSR)=1
          CALL XDRSET(XSREC(1,IXSR),NGRO,1.0)
          RETURN
        ENDIF
        DO 270 IXSR=1,NDPROC
          IF(ITYPRO(IXSR).NE.0) THEN
            KMLPRO(IXSR)=MIN(KMLPRO(IXSR),NGROIN)
          ENDIF
 270    CONTINUE
*----
*  VARIOUS DIMENSION OF VECTORS ARE SET
*  LOOP OVER DILUTION AND SELF-SHIELD XS
*  FROM NGRODP TO NGROIN (THESE CORRESPOND
*  TO CASES WHERE DIL<1.0E10 FOR AT LEAST ONE GROUP
*----
        DO 300 IDIL=1,NDIL
          DO 310 IG1=1,NGRO
            IF(TERP(IDIL,IG1).NE.0.0) GO TO 315
 310      CONTINUE
          GO TO 305
 315      CONTINUE
          WRITE (CD,'(I4)') IDIL
          CALL LCMSIX(IPDRL,'SUBMAT'//CD,1)
          DO 320 IXSR=1,NDPROC
            IF(ITYPRO(IXSR).NE.0.AND.KMLPRO(IXSR).NE.0) THEN
              CALL XDRSET(XSCMP,NGROIN,0.0)
              CALL LCMGET(IPDRL,NAMDXS(IXSR),XSCMP)
              DO 321 IG1=NGRODP,NGROIN
                XSREC(IG1,IXSR)=XSREC(IG1,IXSR)
     >                         +TERP(IDIL,IG1)*XSCMP(IG1)
 321          CONTINUE
            ENDIF
 320      CONTINUE
          DO 330 IL=1,NL
            WRITE (CM,'(I2)') IL-1
            IF(LSDIL(1,IL).EQ.1) THEN
              CALL LCMGET(IPDRL,'NJJ'//CM,NJJ)
              CALL LCMGET(IPDRL,'IJJ'//CM,IJJ)
              LENGT=0
              DO 340 I=1,NGRO
                LENGT=LENGT+NJJ(I)
 340          CONTINUE
              CALL XDRSET(XSCMP,LENGT,0.0)
              CALL LCMGET(IPDRL,'SCAT'//CM,XSCMP)
              IXSS=0
              DO 350 IG1=1,NGRO
                IGMAX=IJJ(IG1)
                IGMIN=IGMAX-NJJ(IG1)+1
                DO 351 IG2=IGMAX,IGMIN,-1
                  IXSS=IXSS+1
                  SCAT(IG1,IG2,IL)=SCAT(IG1,IG2,IL+1)
     >                            +XSCMP(IXSS)*TERP(IDIL,IG2)
 351            CONTINUE
 350          CONTINUE
            ENDIF
            IF(LSDIL(2,IL).EQ.1) THEN
              CALL XDRSET(XSCMP,NGRO,0.0)
              IXSR=NDPROC+IL
              CALL LCMGET(IPDRL,'SIGS'//CM,XSCMP)
              DO 360 IG1=NGRODP,NGROIN
                XSREC(IG1,IXSR)=XSREC(IG1,IXSR)
     >                         +TERP(IDIL,IG1)*XSCMP(IG1)
 360          CONTINUE
            ENDIF
 330      CONTINUE
          IXSR=16
          IF(KMLPRO(IXSR).GT.0) THEN
            CALL XDRSET(XSCMP,NGRO,0.0)
            CALL LCMGET(IPDRL,'NPHI',XSCMP)
            DO 370 IG1=NGRODP,KMLPRO(IXSR)
              XSREC(IG1,IXSR)=XSREC(IG1,IXSR)
     >                       +TERP(IDIL,IG1)*XSCMP(IG1)*DILUT(IDIL)
 370        CONTINUE
          ENDIF
          CALL LCMSIX(IPDRL,' ',2)
 305      CONTINUE
 300    CONTINUE
*----
*  COMPUTE MISSING SCATTERING INFORMATION.
*----
        DO 380 IL=1,NL
          IXSR=NDPROC+IL
          IF(LSDIL(2,IL).EQ.1.AND.LSDIL(1,IL).EQ.0) THEN
            WRITE (CM,'(I2)') IL-1
            CALL LCMGET(IPDRL,'NJJ'//CM,NJJ)
            CALL LCMGET(IPDRL,'IJJ'//CM,IJJ)
            LENGT=0
            DO 390 I=1,NGRO
              LENGT=LENGT+NJJ(I)
 390        CONTINUE
            CALL XDRSET(XSCMP,LENGT,0.0)
            CALL LCMGET(IPDRL,'PCAT'//CM,XSCMP)
            IXSS=0
            CALL XDRSET(SCAT(1,1,IL),NGRO*NGRO,0.0)
            DO 400 IG1=1,NGRO
              IGMAX=IJJ(IG1)
              IGMIN=IGMAX-NJJ(IG1)+1
              DO 401 IG2=IGMAX,IGMIN,-1
                IXSS=IXSS+1
                SCAT(IG1,IG2,IL)=XSCMP(IXSS)*XSREC(IG2,IXSR)
 401          CONTINUE
 400        CONTINUE
          ELSE IF(LSDIL(2,IL).EQ.0.AND.LSDIL(1,IL).EQ.1) THEN
            DO 410 IG1=1,NGRO
              TMP=0.0D0
              DO 411 IG2=1,NGRO
                TMP=TMP+SCAT(IG2,IG1,IL)
 411          CONTINUE
              XSREC(IG1,IXSR)=TMP
 410        CONTINUE
          ENDIF
 380    CONTINUE
*----
*  COMPUTE CONDENSED FINE STRUCTURE FUNCTION.
*----
        IXSF=16
        DO 420 IG1=1,NGROIN
          IF((KMLPRO(IXSF).EQ.0).AND.
     >       (LSDIL(2,1).EQ.0).AND.(LSDIL(1,1).EQ.0)) THEN
*----
*  SCATTERING CROSS SECTIONS ARE NOT SELF-SHIELDED.
*----
            TMP=0.0D0
            DO 430 IG2=1,IG1-1
              TMP=TMP+SCAT(IG1,IG2,1)*XSREC(IG2,IXSF)
     >           *DELTA(IG2)/DELTA(IG1)
 430        CONTINUE
            XSREC(IG1,IXSF)=(1.0+(TMP-XSREC(IG1,1))/SB(IG1))
     >                     /(1.0-SCAT(IG1,IG1,1)/SB(IG1))
          ELSE IF(KMLPRO(IXSF).EQ.0) THEN
*----
*           SCATTERING CROSS SECTIONS ARE SELF-SHIELDED.
*----
            TMP=0.0D0
            DO 440 IG2=1,IG1
              TMP=TMP+SCAT(IG1,IG2,1)*DELTA(IG2)/DELTA(IG1)
 440        CONTINUE
            XSREC(IG1,IXSF)=1.0+(TMP-XSREC(IG1,1))/SB(IG1)
          ELSE
            XSREC(IG1,IXSF)=1.0+XSREC(IG1,IXSF)/SB(IG1)
          ENDIF
          IF((XSREC(IG1,IXSF).LE.0.0).OR.
     >       (XSREC(IG1,IXSF).GT.1.1)) THEN
            WRITE (HSMG,960) XSREC(IG1,IXSF),IG1
            CALL XABORT(HSMG)
          ENDIF
 420    CONTINUE
        CALL XDRSET(XSREC(NGROIN+1,IXSF),NGRO-NGROIN,1.0)
*----
*  DIVIDE EFFECTIVE REACTION RATES BY ZNPHI.
*  FOR SELF-SHIELDED REACTION RATES
*----
        DO 450 IL=1,NL
          IXSR=NDPROC+IL
          IF(ITYPRO(IXSR).EQ.1.AND.
     >      (LSDIL(1,IL).EQ.1.OR.LSDIL(2,IL).EQ.1)) THEN
            DO 460 IG1=NGRODP,NGROIN
              XSREC(IG1,IXSR)=XSREC(IG1,IXSR)/XSREC(IG1,IXSF)
              DO 461 IG2=1,NGRO
                SCAT(IG2,IG1,IL)=SCAT(IG2,IG1,IL)/XSREC(IG1,IXSF)
 461          CONTINUE
 460        CONTINUE
          ENDIF
 450    CONTINUE
        DO 470 IXSR=1,NDPROC
          IF(IXSR.NE.5.AND.IXSR.NE.16) THEN
            IF(ITYPRO(IXSR).EQ.1) THEN
              DO 480 IG1=NGRODP,NGROIN
                XSREC(IG1,IXSR)=XSREC(IG1,IXSR)/XSREC(IG1,IXSF)
 480          CONTINUE
            ENDIF
          ENDIF
 470    CONTINUE
        IF(IMPX.GE.10) THEN
          WRITE(IOUT,940)
          DO 490 IG1=1,NGRO
            WRITE(IOUT,950) IG1,SN(IG1),SB(IG1),XSREC(IG1,16),
     >                      XSREC(IG1,1),XSREC(IG1,NDPROC+1),
     >                      XSREC(IG1,3),XSREC(IG1,14)
  490     CONTINUE
          WRITE(IOUT,'(/)')
        ENDIF
      ELSE
        IXSF=16
        ITYPRO(IXSF)=1
        CALL XDRSET(XSREC(1,IXSF),NGRO,1.0)
      ENDIF
      RETURN
*
  930 FORMAT(42HLIBDRB: NEGATIVE OR ZERO DILUTION IN GROUP,I4,1H.)
  940 FORMAT(/5X,'GROUP',10X,'DILUT',13X,'SB',11X,'NPHI',10X,'TOTAL',
     1 11X,'SIGS',9X,'NUSIGF',11X,'GOLD')
  950 FORMAT(5X,I5,1P,8E15.5)
  960 FORMAT(32HLIBDRB: INVALID VALUE OF ZNPHI (,1P,E12.3,0P,
     1 11H ) IN GROUP,I4,1H.)
      END
