*DECK LIBEIR
      SUBROUTINE LIBEIR(MAXR,NEL,ITNAM,ITZEA,KPAX,BPAX)
C
C-------------------------- LIBEIR   ----------------------------------
C
C  PROGRAMME STATISTICS:
C     NAME     : LIBEIR
C     ENTRY    : LIBEIR
C     USE      : READ DEPLETION DATA ON INPUT FILE
C     MODIFIED : 97-01-03
C     AUTHOR   : A.HEBERT, G. MARLEAU
C
C  ROUTINE PARAMETERS:
C   INPUT
C     MAXR   : NUMBER OF REACTION TYPES             I
C     NEL    : NUMBER OF ISOTOPES ON LIBRARY        I
C   OUTPUT
C     ITNAM  : REACTIVE ISOTOPE NAMES IN  CHAIN     I(2,NEL)
C              -> READ(NAME,'(2A4)')
C                 HNAME(1,I),HNAME(2,I)
C     ITZEA  : 6-DIGIT NUCLIDE IDENTIFIER           I(NEL)
C              ATOMIC NUMBER Z*10000 (DIGITS) +
C              MASS NUMBER A*10 +
C              ENERGY STATE (0 = GROUND STATE,
C              1 = FOR FIRST STATE, ETC.)
C     KPAX   : COMPLETE REACTION TYPE MATRIX        I(NEL+MAXR,NEL)
C     BPAX   : COMPLETE BRANCHING RATIO MATRIX      R(NEL+MAXR,NEL)
C   INTERNAL PARAMETERS
C     KCAPTU : DRAGON CAPTURE FLAG = 3              I
C     KDECAY : DRAGON DECAY FLAG = 1                I
C     KFISSP : DRAGON FISSION PRODUCT FLAG = 2      I
C     KFISSI : DRAGON FISSILE ISOTOPE FLAG = 6      I
C     KN2N   : DRAGON N2N FLAG = 4                  I
C     KN3N   : DRAGON N3N FLAG = 5                  I
C   INPUT FORMAT
C     CHAIN
C     [[ hnamson izea
C        [ { STABLE | [[ { DECAY constant |
C                          NG             |
C                          N2N            |
C                          N3N            |
C                          N4N            |
C                          FISSION energy } ]] } ]
C        [ FROM
C          [[ { DECAY | NG | N2N | N3N | N4N | FISSION }
C             [[ yield hnampar ]] ]]  ]
C     ]]
C     ENDCHAIN
C
C--------------------------  LIBEIR  ----------------------------------
C
      IMPLICIT NONE
C----
C PARAMETERS
C----
      INTEGER      KCAPTU,KDECAY,KFISSP,KFISSI,KN2N,KN3N,MXREAC
      PARAMETER   (KCAPTU=3,KDECAY=1,KFISSP=2,KFISSI=6,KN2N=4,KN3N=5,
     1             MXREAC=6)
C----
C INTERFACE VARIABLES
C----
      INTEGER      MAXR,NEL
      INTEGER      ITNAM(2,NEL),ITZEA(NEL),KPAX(NEL+MAXR,NEL)
      REAL         BPAX(NEL+MAXR,NEL)
C----
C INPUT FILE PARAMETERS
C----
      INTEGER          INDIC,NITMA,KNADPL(2)
      REAL             FLOTT
      CHARACTER        TEXT8*8
      DOUBLE PRECISION DBLINP
C----
C LOCAL VARIABLES
C----
      INTEGER      NDEPL,IEL,JEL,ISOT,IDEPL,INTG,IREAC,JREL,JDEPL,
     1             ILOCDR,KREAC,KR,IDFI
      INTEGER      IDREAC(MXREAC)
      REAL         RRAT,SUMFI
      CHARACTER    NMDEPL(MXREAC)*8,NMPROD(MXREAC)*8
C----
C DATA
C----
      DATA         IDREAC/4,5,1,2,3,5/
      DATA         NMDEPL/'DECAY   ','        ','NG      ',
     1                    'N2N     ','N3N     ','FISSION '/
      DATA         NMPROD/'DECAY   ','FISSION ','NG      ',
     1                    'N2N     ','N3N     ','        '/
      SAVE         NMDEPL,NMPROD,IDREAC
C----
C  READ LIST OF ISOTOPES AND PROPERTIES
C----
      TEXT8=' '
      CALL REDGET(INDIC,NITMA,FLOTT,TEXT8,DBLINP)
      IF(INDIC.NE.3.OR.TEXT8.NE.'CHAIN')
     >  CALL XABORT('LIBEIR: KEYWORD CHAIN MISSING')
      NDEPL=0
      CALL REDGET(INDIC,NITMA,FLOTT,TEXT8,DBLINP)
      DO 100 IEL=1,NEL
C----
C  EXIT IF ENDCHAIN READ
C----
        IF(TEXT8.EQ.'ENDCHAIN') GO TO 105
C----
C  ISOTOPE NAME READ
C  IF NAME ALREADY EXISTS SELECT ISOTOPE NUMBER
C  IF NAME NOT DEFINED ADD TO ISOTOPE LIST
C----
        IF(INDIC.NE.3)
     >    CALL XABORT('LIBEIR: ISOTOPE NAME hnamson MISSING')
        READ(TEXT8,'(2A4)') KNADPL(1),KNADPL(2)
        DO 110 JEL=1,NDEPL
          IF(KNADPL(1).EQ.ITNAM(1,JEL).AND.
     >       KNADPL(2).EQ.ITNAM(2,JEL)     ) THEN
            IDEPL=JEL
            GO TO 115
          ENDIF
 110    CONTINUE
        NDEPL=NDEPL+1
        IF(NDEPL.GT.NEL)
     >    CALL XABORT('LIBEIR: TO MANY ISOTOPES')
        IDEPL=NDEPL
        ITNAM(1,IDEPL)=KNADPL(1)
        ITNAM(2,IDEPL)=KNADPL(2)
 115    CONTINUE
C----
C  READ IZEA
C----
        CALL REDGET(INDIC,ITZEA(IDEPL),FLOTT,TEXT8,DBLINP)
        IF(INDIC.NE.1) CALL XABORT('LIBEIR: izae MISSING')
C----
C  LOOP OVER ALL PARAMETERS ASSOCIATED WITH SON ISOTOPES
C----
        CALL REDGET(INDIC,INTG,FLOTT,TEXT8,DBLINP)
 120    CONTINUE
        IF(INDIC.NE.3)
     >    CALL XABORT('LIBEIR: REACTION TYPE EXPECTED')
C----
C  IF KEYWORD IS 'FROM' READ LIST OF PARENT NUCLIDES
C----
        IF(TEXT8.EQ.'FROM') THEN
C----
C  LOOP OVER ALL PARAMETERS ASSOCIATED WITH PARENT ISOTOPES
C----
          CALL REDGET(INDIC,NITMA,FLOTT,TEXT8,DBLINP)
 130      CONTINUE
          IF(INDIC.NE.3)
     >      CALL XABORT('LIBEIR: REACTION TYPE EXPECTED')
          DO 140 IREAC=1,MXREAC
            RRAT=1.0
C----
C  TEST IF KEYVORD IS A REACTION
C----
            IF(TEXT8.EQ.NMPROD(IREAC)) THEN
C----
C  READ LIST OF YIELD AND PARENT ISOTOPES
C----
              DO 150 JEL=1,NEL
C----
C  IF YIELD ABSENT GO TO TEST FOR NEW REACTION TYPE
C----
                CALL REDGET(INDIC,ISOT,RRAT,TEXT8,DBLINP)
                IF(INDIC.NE.2) GO TO 130
                CALL REDGET(INDIC,ISOT,FLOTT,TEXT8,DBLINP)
                IF(INDIC.NE.3)
     >            CALL XABORT('LIBEIR: ISOTOPE NAME hnampar MISSING')
C----
C  ISOTOPE NAME READ
C  IF NAME ALREADY EXISTS SELECT ISOTOPE NUMBER
C  IF NAME NOT DEFINED ADD TO ISOTOPE LIST
C----
                READ(TEXT8,'(2A4)') KNADPL(1),KNADPL(2)
                DO 160 JREL=1,NDEPL
                  IF(KNADPL(1).EQ.ITNAM(1,JREL).AND.
     >               KNADPL(2).EQ.ITNAM(2,JREL)     ) THEN
                    JDEPL=JREL
                    GO TO 165
                  ENDIF
 160            CONTINUE
                NDEPL=NDEPL+1
                IF(NDEPL.GT.NEL)
     >            CALL XABORT('LIBEIR: TO MANY ISOTOPES')
                JDEPL=NDEPL
                ITNAM(1,JDEPL)=KNADPL(1)
                ITNAM(2,JDEPL)=KNADPL(2)
 165            CONTINUE
                KPAX(IDEPL,JDEPL)=IREAC
                BPAX(IDEPL,JDEPL)=RRAT
 150          CONTINUE
              CALL XABORT('LIBEIR: TO MANY PARENT ISOTOPES')
            ENDIF
 140      CONTINUE
        ELSE IF(TEXT8.EQ.'STABLE') THEN
          CALL REDGET(INDIC,INTG,FLOTT,TEXT8,DBLINP)
C----
C  READ NEXT KEYWORD FOR THIS ISOTOPE
C----
          GO TO 120
        ELSE
          DO 170 IREAC=1,MXREAC
            RRAT=0.0
            IF(TEXT8.EQ.NMDEPL(IREAC)) THEN
              CALL REDGET(INDIC,ISOT,RRAT,TEXT8,DBLINP)
              IF(INDIC.EQ.1) THEN
                CALL XABORT('LIBEIR: INVALID INTEGER')
              ELSE IF(INDIC.EQ.2) THEN
                CALL REDGET(INDIC,INTG,FLOTT,TEXT8,DBLINP)
              ENDIF
              ILOCDR=IDREAC(IREAC)
              KPAX(NEL+ILOCDR,IDEPL)=IREAC
              BPAX(NEL+ILOCDR,IDEPL)=RRAT
C----
C  READ NEXT KEYWORD FOR THIS ISOTOPE
C----
              GO TO 120
            ENDIF
 170      CONTINUE
        ENDIF
 100  CONTINUE
      IF(INDIC.NE.3.OR.TEXT8.NE.'ENDCHAIN')
     >  CALL XABORT('LIBEIR: KEYWORD ENDCHAIN MISSING')
 105  CONTINUE
C----
C  DEPLETION CHAIN COMPLETED
C  CHECK IF IT IS COHERENT
C  FIND FISSILE ISOTOPE AND FISSION PRODUCT
C  RENUMBER FISSILE ISOTOPE AND FISSION PRODUCT
C  DELETE ISOTOPES WITH NO REACTION
C----
      DO 200 IEL=1,NDEPL
        DO 210 JEL=1,NDEPL
          KREAC=KPAX(JEL,IEL)
          IF(KREAC.EQ.KFISSP) THEN
            IF(KPAX(NEL+5,JEL).EQ.0) THEN
              KPAX(NEL+5,JEL)=KFISSP
            ELSE IF(KPAX(NEL+5,JEL).NE.KFISSP) THEN
              CALL XABORT('LIBEIR: SON NOT A FISSION PRODUCT')
            ENDIF
            IF(KPAX(NEL+5,IEL).NE.KFISSI)
     >        CALL XABORT('LIBEIR: PARENT NOT FISSILE')
          ELSE IF(KREAC.NE.0) THEN
            KR=IDREAC(KREAC)
            IF(KPAX(NEL+KR,IEL).NE.KREAC)
     >        CALL XABORT('LIBEIR: PARENT DOES NOT DEPLETE '//
     >                    'VIA REACTION '//NMDEPL(KREAC))
          ENDIF
 210    CONTINUE
 200  CONTINUE
C----
C  FIND FISSILE ISOTOPE NON ZERO YIELD
C----
      IDFI=0
      DO 220 IEL=1,NDEPL
        IF(KPAX(NEL+5,IEL).EQ.KFISSI) THEN
          SUMFI=0.0
          DO 221 JEL=1,NDEPL
            IF(KPAX(JEL,IEL).EQ.KFISSP) THEN
              SUMFI=SUMFI+BPAX(JEL,IEL)
            ENDIF
 221      CONTINUE
          IF(SUMFI.GT.0.0) THEN
            IDFI=IDFI+10
            KPAX(NEL+5,IEL)=KPAX(NEL+5,IEL)+IDFI
          ENDIF
        ENDIF
 220  CONTINUE
C----
C  RETURN FROM LIBEIR
C----
      RETURN
      END
