*DECK LIBENR
      SUBROUTINE LIBENR(CFILNA,MAXR,NEL,ITNAM,KPAX,BPAX)
C
C-------------------------- LIBENR   ----------------------------------
C
C  PROGRAMME STATISTICS:
C     NAME     : LIBENR
C     ENTRY    : LIBENR
C     USE      : READ DEPLETION DATA ON WIMS-D4 FORMAT LIBRARY
C     MODIFIED : 97-01-08
C     AUTHOR   : G. MARLEAU
C
C  ROUTINE PARAMETERS:
C   INPUT
C     CFILNA : WIMS FILE NAME                       C*8
C     MAXR   : NUMBER OF REACTION TYPES             I
C     NEL    : NUMBER OF ISOTOPES ON LIBRARY        I
C   OUTPUT
C     ITNAM  : REACTIVE ISOTOPE NAMES IN  CHAIN     I(2,NEL)
C              -> READ(NAME,'(2A4)')
C                 HNAME(1,I),HNAME(2,I)
C     KPAX   : COMPLETE REACTION TYPE MATRIX        I(NEL+MAXR,NEL)
C     BPAX   : COMPLETE BRANCHING RATIO MATRIX      R(NEL+MAXR,NEL)
C   INTERNAL PARAMETERS
C     KCAPTU : DRAGON CAPTURE FLAG = 3              I
C     KDECAY : DRAGON DECAY FLAG = 1                I
C     KFISSP : DRAGON FISSION PRODUCT FLAG = 2      I
C     KFISSI : DRAGON FISSILE ISOTOPE FLAG = 6      I
C     KN2N   : DRAGON N2N FLAG = 4                  I
C     KN3N   : DRAGON N3N FLAG = 5                  I
C     CONVE  : ENERGY CONVERSION FACTOR             R
C              FROM JOULES/(MOLES*10**-24) TO
C              MEV/NUCLIDE = 1.03643526E+13
C     CONVD  : DECAY CONSTANT CONVERSION FACTOR     R
C              FROM S**(-1)) TO 10**(-8)*S**(-1))
C              = 1.0+8
C   WIMS-D4 LIBRARY PARAMETERS
C     IUTYPE : TYPE OF FILE = 2 (BINARY)            I
C     LRIND  : LENGHT RECORD ON DA FILE = 0         I
C     IACTO  : OPEN ACTION = 2 (READ ONLY)          I
C     IACTC  : CLOSE ACTION = 2 (KEEP)              I
C     MAXISO : MAX. NB. ISOTOPE = 246               I
C     MLDEP  : MAXIMUM NUMBER OF REACTION PER       I
C              ISOTOPE IN WIMS-D4 = MAXISO+4
C     LPZ    : LENGTH OF WIMS PARAMETER ARRAY = 8   I
C     NPZ    : LIST OF MAIN PARAMETERS              I(LPZ)
C     IWISO  : ID OF ISOTOPE                        I(MAXISO)
C     IBURN  : INTEGER BURNUP DATA                  I(MLDEP)
C     RBURN  : REAL BURNUP DATA                     R(MLDEP)
C
C--------------------------  LIBENR  ----------------------------------
C
      IMPLICIT NONE
C----
C PARAMETERS
C----
      INTEGER      KCAPTU,KDECAY,KFISSP,KFISSI,KN2N,KN3N
      REAL         CONVE,CONVD
      PARAMETER   (KCAPTU=3,KDECAY=1,KFISSP=2,KFISSI=6,KN2N=4,
     1             KN3N=5,CONVE=1.03643526E+13,CONVD=1.0E+8)
C----
C  WIMS-D4 LIBRARY PARAMETERS
C----
      INTEGER      IUTYPE,LRIND,IACTO,IACTC,MAXISO,MLDEP,LPZ
      PARAMETER   (IUTYPE=2,LRIND=0,IACTO=2,IACTC=1,MAXISO=246,
     1             MLDEP=MAXISO+4,LPZ=8)
C----
C EXTERNAL FUNCTIONS
C----
      INTEGER      KDROPN,KDRCLS,LIBWID
C----
C LOCAL VARIABLES
C----
      INTEGER      MAXR,NEL,IUNIT,II,J,ISO,JC,IHANDL,JB,KTYPE,JSO,IT,
     1             IERR,NFP
      INTEGER      NPZ(LPZ),IWISO(MAXISO),IBURN(MLDEP),ITNAM(2,NEL),
     1             KPAX(NEL+MAXR,NEL)
C
      REAL         RTEMP
      REAL         RBURN(MLDEP),BPAX(NEL+MAXR,NEL)
C
      CHARACTER    TEXT8*8,CFILNA*8
C----
C  OPEN WIMS-D4 LIBRARY
C  READ GENERAL DIMENSIONING
C  READ ISOTOPE ID NUMBER AND CREATE EQUIVALENT ISOTOPE NAME
C----
      IUNIT=KDROPN(CFILNA,IACTO,IUTYPE,LRIND,IHANDL)
      IF(IUNIT.LE.0) CALL XABORT('LIBENR: WIMS-D4 LIBRARY '//
     >    CFILNA//' CANNOT BE OPENED FOR DEPLETION')
      READ(IUNIT) (NPZ(II),II=1,LPZ)
      IF(NPZ(1).NE.NEL) CALL XABORT('LIBENR: TOO MANY ISOTOPES '//
     >    'ON WIMS-D4 LIBRARY'//CFILNA)
      READ(IUNIT) (IWISO(J),J=1,NEL)
      DO 10 ISO=1,NEL
        TEXT8='        '
        IF     (IWISO(ISO).LT.10) THEN
          WRITE(TEXT8,'(I1)') IWISO(ISO)
        ELSE IF(IWISO(ISO).LT.100) THEN
          WRITE(TEXT8,'(I2)') IWISO(ISO)
        ELSE IF(IWISO(ISO).LT.1000) THEN
          WRITE(TEXT8,'(I3)') IWISO(ISO)
        ELSE IF(IWISO(ISO).LT.10000) THEN
          WRITE(TEXT8,'(I4)') IWISO(ISO)
        ELSE IF(IWISO(ISO).LT.100000) THEN
          WRITE(TEXT8,'(I5)') IWISO(ISO)
        ELSE IF(IWISO(ISO).LT.1000000) THEN
          WRITE(TEXT8,'(I6)') IWISO(ISO)
        ELSE IF(IWISO(ISO).LT.10000000) THEN
          WRITE(TEXT8,'(I7)') IWISO(ISO)
        ELSE IF(IWISO(ISO).LT.100000000) THEN
          WRITE(TEXT8,'(I8)') IWISO(ISO)
        ENDIF
        READ(TEXT8,'(2A4)')  ITNAM(1,ISO),ITNAM(2,ISO)
 10   CONTINUE
C---
C  READ TWO ADDITIONAL RECORDS BEFORE DEPLETION DATA
C----
      READ(IUNIT) (RTEMP,J=1,NPZ(2)+1)
      READ(IUNIT) (RTEMP,J=1,NPZ(3))
C----
C  READ DEPLETION CHAIN FOR EACH ISOTOPES
C----
      DO 100 ISO=1,NEL
        RBURN(1)=0.0
        READ(IUNIT) JC,IBURN(1),
     >    (RBURN(JB),IBURN(JB),JB=2,JC/2)
C----
C DEPLETING ISOTOPE WITHOUT FISSION PRODUCT JC=8
C DEPLETING ISOTOPE WITH FISSION PRODUCT JC>8
C CAPTURE -> ALWAYS PRESENT
C   IF ISOTOPE RESULTING FROM CAPTURE IS KNOWN STORE IN ADEQUATE
C   POSITION ELSE STORE IN NEL+1
C DECAY   -> RBURN(2) > 0.0
C   IF ISOTOPE RESULTING FROM DECAY IS KNOWN STORE IN ADEQUATE
C   POSITION ELSE STORE IN NEL+2
C FISSILE -> IBURN(IT) > 1
C   JC=8 -> ISOTOPE RESULTING FROM FISSION NOT KNOWN STORE IN NEL+3
C   JC>8 -> ISOTOPE RESULTING FROM FISSION KNOWN STORE IN ADEQUATE
C   POSITION
C----
        IF(JC.GE.8) THEN
          KTYPE=KCAPTU
          JSO=LIBWID(NEL,IWISO,IBURN(2))
          IF(JSO.GT.0) THEN
            KPAX(JSO,ISO)=KTYPE
            BPAX(JSO,ISO)=RBURN(2)
            KTYPE=10*JSO+KTYPE
           ENDIF
          KPAX(NEL+1,ISO)=KTYPE
          BPAX(NEL+1,ISO)=1.0
          KTYPE=KDECAY
          IF(RBURN(3).GT.0.0) THEN
            JSO=LIBWID(NEL,IWISO,IBURN(3))
            IF(JSO.GT.0) THEN
              KPAX(JSO,ISO)=KTYPE
              BPAX(JSO,ISO)=1.0
              KTYPE=10*JSO+KTYPE
            ENDIF
            KPAX(NEL+4,ISO)=KTYPE
            BPAX(NEL+4,ISO)=RBURN(3)*CONVD
          ENDIF
          KTYPE=KFISSI
          IF(IBURN(4).GT.1) THEN
            IF(JC.GT.8) THEN
              NFP=0
              DO 102 IT=5,JC/2
                IF(RBURN(IT).GT.0.0) THEN
                  NFP=NFP+1
                  JSO=LIBWID(NEL,IWISO,IBURN(IT))
                  IF(JSO.GT.0) THEN
                    KPAX(JSO,ISO)=KFISSP
                    BPAX(JSO,ISO)=RBURN(IT)
                  ENDIF
                ENDIF
 102          CONTINUE
              KTYPE=10*NFP+KTYPE
            ENDIF
            KPAX(NEL+5,ISO)=KTYPE
            BPAX(NEL+5,ISO)=RBURN(4)*CONVE
          ENDIF
        ENDIF
 100  CONTINUE
C----
C  CLOSE WIMS-D4 LIBRARY
C----
      IERR=KDRCLS(IUNIT,IACTC)
      IF(IERR.LT.0)
     >  CALL XABORT('LIBENR: WIMS-D4 LIBRARY '//
     >    CFILNA//' CANNOT BE CLOSED')
C----
C  RETURN
C----
      RETURN
      END
