*DECK LIBLEX
      SUBROUTINE LIBLEX(NELE,PNTE,ELMT,NOTX,TERP)
C
C-------------------------- LIBLEX   ----------------------------------
C
C  PROGRAMME STATISTICS:
C     NAME     : LIBLEX
C     ENTRY    : LIBLEX
C     USE      : COMPUTE FACTORS FOR LAGRANGIAN INTERPOLATION
C     MODIFIED : 93-08-17
C     AUTHOR   : A. HEBERT (MODIFIED BY G. MARLEAU)
C
C  ROUTINE PARAMETERS:
C   INPUT
C     NELE   : NUMBER OF ELEMENTS IN TABLE          I
C     PNTE   : EXTRAPOLATION POINT                  R
C     ELMT   : VALUES OF ELEMENTS IN TABLE          R(NELE)
C     NOTX   : ORDER OF EXTRAPOLATION               I
C              > 0 : ORDER NOTX IN SQRT(PNTE)
C              < 0 : ORDER -NOTX IN PNTE
C   OUTPUT
C     TERP   : EXTRAPOLATION FACTOR                 D(NELE)
C
C-------------------------- LIBLEX   ----------------------------------
C
      IMPLICIT NONE
C----
C INTERFACE VARIABLES
C----
      INTEGER    NELE,NOTX
      REAL       PNTE,ELMT(NELE)
      DOUBLE PRECISION TERP(NELE)
C----
C LOCAL VARIABLES
C----
      INTEGER          I,I1,I2,J,NORD
      DOUBLE PRECISION FACTOR,TERM,SQPNTE
C
      I1=1
      I2=NELE
      NORD=ABS(NOTX)
      DO 100 I=1,NELE
        IF(PNTE.GT.ELMT(I)) I1=MAX(I-(NORD-1)/2,I1)
        IF(PNTE.LT.ELMT(I)) I2=MIN(I+(NORD-1)/2,I2)
        TERP(I)=0.0D0
 100  CONTINUE
      IF(NOTX.LT.0) THEN
        IF(I1.EQ.1) THEN
          I2=MIN(NELE,NORD+1)
        ELSE IF(I2.EQ.NELE) THEN
          I1=MAX(1,NELE-NORD)
        ENDIF
        FACTOR=1.0D0
        SQPNTE=DBLE(PNTE)
        DO 110 I=I1,I2
          IF(ABS(PNTE-ELMT(I)).LE.1.0E-5*ABS(PNTE)) THEN
            TERP(I)=1.0D0
            GO TO 111
          ENDIF
          FACTOR=FACTOR*( SQPNTE-DBLE(ELMT(I)) )
 110    CONTINUE
        DO 120 I=I1,I2
          TERM=FACTOR/( SQPNTE-DBLE(ELMT(I)) )
          DO 130 J=I1,I2
            IF(I.NE.J) TERM=TERM/
     >               ( DBLE(ELMT(I))-DBLE(ELMT(J)) )
 130      CONTINUE
          TERP(I)=TERM
 120    CONTINUE
 111    CONTINUE
      ELSE
        FACTOR=1.0D0
        SQPNTE=DBLE(SQRT(PNTE))
        DO 160 I=I1,I2
          IF(ABS(PNTE-ELMT(I)).LE.1.0E-5*ABS(PNTE)) THEN
            TERP(I)=1.0D0
            GO TO 161
          ENDIF
          FACTOR=FACTOR*( SQPNTE-DBLE(SQRT(ELMT(I))) )
 160    CONTINUE
        DO 170 I=I1,I2
          TERM=FACTOR/( SQPNTE-DBLE(SQRT(ELMT(I))) )
          DO 180 J=I1,I2
            IF(I.NE.J) TERM=TERM/
     >               ( DBLE(SQRT(ELMT(I)))-DBLE(SQRT(ELMT(J))) )
 180      CONTINUE
          TERP(I)=TERM
 170    CONTINUE
 161    CONTINUE
      ENDIF
C----
C  RETURN LIBLEX
C----
      RETURN
      END
