*DECK LIBLIB
      SUBROUTINE LIBLIB(IPLIB,IMPX,MAXISO,NBISO,NGROUP,ISONAM,
     >                  ISONRF,IHLIB,NTFG,TMPISO,AWRISO,SNISO,SBISO,
     >                  MASKI,NL,ITIME,NEDMAC,HVECT,ITRANC,ITISOM,
     >                  DELTA,NGFR,ISOADD)
C
C-----------------------------------------------------------------------
C
C TRANSCRIPTION OF THE USEFULL INTERPOLATED MICROSCOPIC CROSS SECTION
C DATA FROM VARIOUS FORMAT OF LIBRARIES TO LCM. A TWO DIMENSIONAL
C INTERPOLATION IN TEMPERATURE AND DILUTION IS PERFORMED.
C
C INPUT PARAMETERS:
C  IPLIB   : POINTER TO THE LATTICE MICROSCOPIC CROSS SECTION LIBRARY
C            (L_LIBRARY SIGNATURE).
C  IMPX    : PRINT FLAG. EQUAL TO ZERO FOR NO PRINT.
C  MAXISO  : MAXIMUM NUMBER OF ISOTOPES PERMITTED
C  NBISO   : NUMBER OF ISOTOPES PRESENT IN THE CALCULATION DOMAIN.
C  NGROUP  : NUMBER OF ENERGY GROUPS.
C  ISONAM  : NAME OF EACH ISOTOPE. ISONAM(3,MAXISO)
C  ISONRF  : REFERENCE NAME OF EACH ISOTOPE. ISONRF(3,MAXISO)
C  IHLIB   : ISOTOPE OPTIONS. IHLIB(2,MAXISO,5)
C  NTFG    : NUMBER OF THERMAL GROUPS WHERE THE THERMAL INELASTIC
C            CORRECTION IS APPLIED. NTFG(MAXISO)
C  TMPISO  : TEMPERATURE OF EACH ISOTOPE. TMPISO(MAXISO)
C  AWRISO  : ATOMIC WEIGHT OF EACH ISOTOPE. AWRISO(MAXISO)
C  SNISO   : DILUTION CROSS SECTION OF EACH GROUP AND EACH ISOTOPE.
C            1.0E10 -> INFINITE DILUTION. SNISO(NGROUP,MAXISO)
C  SBISO   : DILUTION CROSS SECTION OF EACH GROUP AND EACH ISOTOPE
C            USED WITH LIVOLANT-JEANPIERRE NORMALIZATION.
C            1.0E10 -> INFINITE DILUTION. SBISO(NGROUP,MAXISO)
C  MASKI   : ISOTOPE WITH INDEX I IS PROCESS IF MASKI(I)=.TRUE.
C            LOGICAL MASKI(MAXISO)
C  NL      : NUMBER OF LEGENDRE ORDERS REQUIRED IN THE CALCULATION.
C            NL=1 (FOR ISOTROPIC SCATTERING) OR HIGHER.
C  ITIME   : =1 : STEADY-STATE / =2 : PROMPT.
C  NEDMAC  : NUMBER OF EXTRA VECTOR EDITS FROM MATXS.
C  HVECT   : CHARACTER*6 MATXS NAMES OF THE EXTRA VECTOR EDITS.
C            HVEC(NEDMAC)
C  ITRANC  : TRANSPORT CORRECTION
C  ITISOM  : TEMPORARY MASK FOR ISOTOPE. ITISOM(NBISO)
C  DELTA   : LETHARGY LIMITS OF GROUPS. DELTA(NGROUP)
C  NGFR    : NUMBER OF FAST+RESONANCE GROUPS.
C  ISOADD  : FLAG TO COMPLETE MIXTURE DEFINITION
C            IF DEPLETION CHAIN PRESENT
C            =-1 Partial completion (warning if missing information)
C            = 0 COMPLETE MIXTURES
C            = 1 NO COMPLETION
C
C------------------------------------- AUTHOR: A. HEBERT  07/04/88 ----
C----------------------------------- MODIFIED: G. MARLEAU 97/01/23 ----
C
      IMPLICIT NONE
C----
C PARAMETERS
C----
      INTEGER          IOUT,MAXDIL,MAXED,NDPROC
      CHARACTER        NAMSBR*6
      PARAMETER       (IOUT=6,MAXDIL=25,MAXED=50,NDPROC=20,
     >                 NAMSBR='LIBLIB')
C----
C MEMORY ALLOCATION
C----
      INTEGER          IBASE(1)
      REAL             RBASE(1)
      COMMON           RBASE
      EQUIVALENCE     (RBASE(1),IBASE(1))
      INTEGER          INDPRO,ITYPRO,IVECT,IXSREC,ISCAT,IXSCMP,INJJ,
     >                 IIJJ,IXSOUT,ISORD,IIPR,ICNORM,ILCMAT,JVECT,
     >                 ISIGS,ISIGA,ISIGN,IPRI,IVTHER,IPHI,IPP,IPP1,
     >                 ISSS,ISSS1,ISS1,ISS11,IUUU,IDELT,IGAR,ISEFF,
     >                 ISIG1,NISB,NISBF
C----
C LOCAL AND INTERFACE VARIABLES
C----
      INTEGER          IPLIB,IMPX,MAXISO,NBISO,NGROUP,NL,ITIME,NEDMAC,
     >                 NPROC,NMLIB,ISO,JSO,ITC,NGRD,ITYLCM,ILIB,NGFR,
     >                 ISOADD,ITRANC
      INTEGER          ISONAM(3,MAXISO),ISONRF(3,MAXISO),
     >                 IHLIB(2,MAXISO,5),NTFG(MAXISO),ITISOM(NBISO)
C
      REAL             TMPISO(MAXISO),AWRISO(MAXISO),
     >                 SNISO(NGROUP,MAXISO),SBISO(NGROUP,MAXISO),
     >                 DELTA(NGROUP+1)
C
      CHARACTER        HVECT(NEDMAC)*6,HSMG*131
      LOGICAL          MASKI(MAXISO)
C
      CHARACTER        NAMFIL*8,NAMLBT*8
      INTEGER          IFACIS,JFACIS
C----
C  SCAN FOR ADDITIONAL AND STANDARD CROSS SECTIONS TO BE SAVED
C----
      NPROC=NDPROC+NL
      CALL SETARA(IBASE,NPROC,INDPRO)
      CALL SETARA(IBASE,NPROC,ITYPRO)
      CALL SETARA(IBASE,NEDMAC,IVECT)
      CALL XDRNED(NPROC,0,NL-1,0,NEDMAC,HVECT,
     >            IBASE(IVECT),IBASE(INDPRO))
C----
C  SET INITIAL TIME AND CLASSIFY ISOTOPES BY LIBRARY
C----
      NMLIB=1
      DO 100 ISO=1,NBISO
        IFACIS=0
        IF(ITISOM(ISO) .EQ. 0) THEN
          IFACIS=1
        ELSE IF(ITISOM(ISO) .EQ. -1) THEN
          IFACIS=-1
        ENDIF
        IF(MASKI(ISO) .AND. (IFACIS .NE. 0)) THEN
          NMLIB=NMLIB+1
          ITISOM(ISO)=NMLIB*IFACIS
          DO 101 JSO=ISO+1,NBISO
            IF(MASKI(JSO)) THEN
              JFACIS=0
              IF(ITISOM(JSO) .EQ. 0) THEN
                JFACIS=1
              ELSE IF(ITISOM(JSO) .EQ. -1) THEN
                JFACIS=-1
              ENDIF
C----
C  CHECK FOR DUPLICATE ISOTOPE NAME
C----
              IF( (ISONAM(1,JSO).EQ.ISONAM(1,ISO)) .AND.
     >            (ISONAM(2,JSO).EQ.ISONAM(2,ISO)) .AND.
     >            (ISONAM(3,JSO).EQ.ISONAM(3,ISO)) ) THEN
                WRITE(HSMG,9000) NAMSBR,(ISONAM(ITC,JSO),ITC=1,3)
                CALL XABORT(HSMG)
              ENDIF
C----
C  LOOK FOR OTHER ISOTOPES ON SAME LIBRARY
C----
              IF( (IHLIB(1,JSO,1).EQ.IHLIB(1,ISO,1)).AND.
     >            (IHLIB(2,JSO,1).EQ.IHLIB(2,ISO,1)).AND.
     >            (IHLIB(1,JSO,2).EQ.IHLIB(1,ISO,2)).AND.
     >            (IHLIB(2,JSO,2).EQ.IHLIB(2,ISO,2)) )
     >            ITISOM(JSO)=NMLIB*JFACIS
            ENDIF
 101      CONTINUE
        ENDIF
 100  CONTINUE
C----
C  READ LETHARGY VECTOR IF POSSIBLE ON CURRENT LIBRARY
C----
      CALL LCMLEN(IPLIB,'DELTAU',NGRD,ITYLCM)
      IF(NGRD.EQ.NGROUP) THEN
        CALL LCMGET(IPLIB,'DELTAU',DELTA)
      ELSE
        CALL XABORT(NAMSBR//': INVALID NUMBER OF GROUP.')
      ENDIF
      CALL SETARA(RBASE,NGROUP*NPROC,IXSREC)
      CALL SETARA(RBASE,NGROUP*NGROUP*NL,ISCAT)
      CALL SETARA(RBASE,NGROUP*(NGROUP+2),IXSCMP)
      CALL SETARA(IBASE,NGROUP,INJJ)
      CALL SETARA(IBASE,NGROUP,IIJJ)
      DO 110 ILIB=2,NMLIB
C----
C  FIND LIBRARY NAME OF FIRST ISOTOPE IN THIS SET
C----
        DO 111 JSO=1,NBISO
          IF(ABS(ITISOM(JSO)) .EQ. ILIB) THEN
            WRITE(NAMLBT,'(2A4)') (IHLIB(ITC,JSO,1),ITC=1,2)
            WRITE(NAMFIL,'(2A4)') (IHLIB(ITC,JSO,2),ITC=1,2)
            GO TO 115
          ENDIF
 111    CONTINUE
 115    CONTINUE
        IF(IMPX.GT.1) THEN
          WRITE(IOUT,6001) NAMFIL,NAMLBT
        ENDIF
        IF(NAMLBT.EQ.'DRAGON  ') THEN
C----
C  TRANSFER INFORMATION FROM DRAGON LIBRARY FORMAT
C  TO CURRENT LIBRARY.
C----
          IBASE(INDPRO+13)=1
          CALL LIBDRA(IPLIB,IMPX,NAMFIL,NGROUP,NBISO,NL,ISONAM,
     >                ISONRF,TMPISO,AWRISO,SNISO,SBISO,ILIB,ITISOM,
     >                DELTA,NGFR,ITRANC,NPROC,IBASE(INDPRO),
     >                IBASE(ITYPRO),RBASE(IXSREC),RBASE(ISCAT),
     >                RBASE(IXSCMP),IBASE(INJJ),IBASE(IIJJ))
         ELSE IF(NAMLBT(1:4).EQ.'WIMS') THEN
C----
C  TRANSFER INFORMATION FROM WIMS LIBRARY FILE
C  TO CURRENT LIBRARY.
C----
          CALL SETARA(RBASE,7*NGROUP,IXSOUT)
          IF(NAMLBT.EQ.'WIMSD4') THEN
C----
C  WIMS-D4 FORMAT
C----
            CALL SETARA(IBASE,NBISO,ISORD)
            IBASE(INDPRO+13)=1
            CALL LIBWD4(IPLIB,IMPX,NAMFIL,NGROUP,NBISO,NL,ISONAM,
     >                  ISONRF,IHLIB(1,1,5),TMPISO,AWRISO,SNISO,
     >                  SBISO,ILIB,ITISOM,DELTA,ITRANC,NPROC,
     >                  IBASE(INDPRO),IBASE(ITYPRO),RBASE(IXSREC),
     >                  RBASE(ISCAT),RBASE(IXSCMP),IBASE(INJJ),
     >                  IBASE(IIJJ),RBASE(IXSOUT),IBASE(ISORD))
            CALL RLSARA(IBASE(ISORD))
          ELSE
C----
C  WIMS-AECL FORMAT
C----
            IBASE(INDPRO+13)=1
            CALL LIBWIM(IPLIB,IMPX,NAMFIL,NGROUP,NBISO,NL,ISONAM,
     >                  ISONRF,IHLIB(1,1,5),TMPISO,AWRISO,SNISO,
     >                  SBISO,ILIB,ITISOM,DELTA,ITRANC,NPROC,
     >                  IBASE(INDPRO),IBASE(ITYPRO),RBASE(IXSREC),
     >                  RBASE(ISCAT),RBASE(IXSCMP),IBASE(INJJ),
     >                  IBASE(IIJJ),RBASE(IXSOUT))
          ENDIF
          CALL RLSARA(RBASE(IXSOUT))
        ELSE IF(NAMLBT(1:5).EQ.'MATXS   ') THEN
          CALL SETARA(IBASE,2*NBISO,IIPR)
          IF(NAMLBT.EQ.'MATXS   ') THEN
C----
C  TRANSFER INFORMATION FROM MATXS LIBRARY FORMAT
C  TO CURRENT LIBRARY.
C----
            CALL SETARA(RBASE,3*NBISO,ICNORM)
            CALL LIBTR1(IPLIB,IMPX,NAMFIL,NGROUP,NBISO,NL,ISONAM,
     >                  ISONRF,IHLIB(1,1,3),IHLIB(1,1,4),NTFG,
     >                  TMPISO,AWRISO,SNISO,SBISO,ILIB,ITISOM,
     >                  DELTA,NGFR,ITRANC,NEDMAC,HVECT,IBASE(IVECT),
     >                  ITIME,NPROC,IBASE(INDPRO),IBASE(ITYPRO),
     >                  RBASE(IXSREC),RBASE(ISCAT),IBASE(IIPR),
     >                  RBASE(IXSCMP),IBASE(INJJ),IBASE(IIJJ),
     >                  RBASE(ICNORM))
            CALL RLSARA(RBASE(ICNORM))
          ELSE IF(NAMLBT.EQ.'MATXS2  ') THEN
C----
C  TRANSFER INFORMATION FROM MATXS2 LIBRARY FORMAT
C  TO CURRENT LIBRARY.
C----
            CALL SETARA(RBASE,NGROUP*NGROUP*NL,ILCMAT)
            CALL LIBTR2(IPLIB,IMPX,NAMFIL,NGROUP,NBISO,NL,ISONAM,
     >                  ISONRF,IHLIB(1,1,3),IHLIB(1,1,4),NTFG,
     >                  TMPISO,AWRISO,SNISO,SBISO,ILIB,ITISOM,
     >                  DELTA,NGFR,ITRANC,NEDMAC,HVECT,IBASE(IVECT),
     >                  ITIME,NPROC,IBASE(INDPRO),IBASE(ITYPRO),
     >                  RBASE(IXSREC),RBASE(ISCAT),IBASE(IIPR),
     >                  RBASE(IXSCMP),IBASE(INJJ),IBASE(IIJJ),
     >                  RBASE(ILCMAT))
            CALL RLSARA(RBASE(ILCMAT))
          ENDIF
          CALL RLSARA(IBASE(IIPR))
        ELSE IF(NAMLBT.EQ.'APLIB1  ') THEN
C----
C  TRANSFER INFORMATION FROM APLIB LIBRARY FORMAT
C  TO CURRENT LIBRARY.
C----
         CALL SETARA(IBASE,2*NBISO,IIPR)
         CALL SETARA(RBASE,NGROUP+1,JVECT)
         CALL SETARA(RBASE,NGROUP,ISIGS)
         CALL SETARA(RBASE,NGROUP,ISIG1)
         CALL SETARA(RBASE,NGROUP,ISIGA)
         CALL SETARA(RBASE,NGROUP,ISIGN)
         CALL SETARA(RBASE,NGROUP,IVTHER)
         CALL SETARA(RBASE,MAXDIL*NGROUP,IPHI)
         CALL SETARA(RBASE,NGROUP*(NGROUP+1),IPP)
         CALL SETARA(RBASE,NGROUP*(NGROUP+1),IPP1)
         CALL SETARA(RBASE,NGROUP,ISSS)
         CALL SETARA(RBASE,NGROUP,ISSS1)
         CALL SETARA(RBASE,NGROUP,ISS1)
         CALL SETARA(RBASE,NGROUP,ISS11)
         CALL SETARA(RBASE,NGROUP,IUUU)
         CALL SETARA(RBASE,NGROUP,IDELT)
         CALL SETARA(RBASE,NGROUP**2,IGAR)
         CALL SETARA(RBASE,3*MAXDIL*NGROUP,ISEFF)
         CALL SETARA(IBASE,NBISO,NISB)
         CALL SETARA(IBASE,NBISO,NISBF)
         CALL LIBAPL(IPLIB,IMPX,NAMFIL,NGROUP,NBISO,NL,ISONAM,
     >               ISONRF,IHLIB(1,1,5),TMPISO,AWRISO,SNISO,SBISO,
     >               ILIB,ITISOM,RBASE(IDELT),NGFR,ITRANC,IBASE(IIPR),
     >               RBASE(JVECT),RBASE(ISIGS),RBASE(ISIG1),
     >               RBASE(ISIGA),RBASE(ISIGN),RBASE(ISCAT),
     >               RBASE(IVTHER),RBASE(IPHI),RBASE(IPP),
     >               RBASE(IPP1),RBASE(ISSS),RBASE(ISSS1),
     >               RBASE(ISS1),RBASE(ISS11),RBASE(IUUU),
     >               RBASE(IXSCMP),RBASE(IGAR),IBASE(INJJ),
     >               IBASE(IIJJ),RBASE(ISEFF),IBASE(NISB),
     >               IBASE(NISBF),NPROC,IBASE(ITYPRO))
         CALL RLSARA(IBASE(NISBF))
         CALL RLSARA(IBASE(NISB))
         CALL RLSARA(RBASE(ISEFF))
         CALL RLSARA(RBASE(IGAR))
         CALL RLSARA(RBASE(IDELT))
         CALL RLSARA(RBASE(IUUU))
         CALL RLSARA(RBASE(ISS11))
         CALL RLSARA(RBASE(ISS1))
         CALL RLSARA(RBASE(ISSS1))
         CALL RLSARA(RBASE(ISSS))
         CALL RLSARA(RBASE(IPP1))
         CALL RLSARA(RBASE(IPP))
         CALL RLSARA(RBASE(IPHI))
         CALL RLSARA(RBASE(IVTHER))
         CALL RLSARA(RBASE(ISIGN))
         CALL RLSARA(RBASE(ISIGA))
         CALL RLSARA(RBASE(ISIG1))
         CALL RLSARA(RBASE(ISIGS))
         CALL RLSARA(RBASE(JVECT))
         CALL RLSARA(IBASE(IIPR))
        ENDIF
 110  CONTINUE
C----
C  Isotopes with ITISOM < 0 were not located on a
C  library but are required for burnup
C  Set cross-sections to 0.0 
C----
      CALL LIBNXS(IPLIB,IMPX,NGROUP,NBISO,NL,ISONAM,
     >            ISONRF,ITISOM,NPROC,ISOADD,
     >            IBASE(INDPRO),IBASE(ITYPRO),RBASE(IXSREC),
     >            RBASE(ISCAT),RBASE(IXSCMP),IBASE(INJJ),
     >            IBASE(IIJJ))
      CALL RLSARA(IBASE(IIJJ))
      CALL RLSARA(IBASE(INJJ))
      CALL RLSARA(RBASE(IXSCMP))
      CALL RLSARA(RBASE(ISCAT))
      CALL RLSARA(RBASE(IXSREC))
      CALL RLSARA(IBASE(IVECT))
      CALL RLSARA(IBASE(ITYPRO))
      CALL RLSARA(IBASE(INDPRO))
      RETURN
C----
C  FORMAT
C----
 9000 FORMAT(A6,': ',3A4,' IS A DUPLICATE ISOTOPE NAME.')
 6000 FORMAT(/' TIME SPENT TO READ THE XS LIBRARY = ',F5.1,
     >        ' SECONDS.'/)
 6001 FORMAT(/' PROCESSING LIBRARY = ',A8,3X,'OF TYPE = ',A8)
      END
