*DECK LIBNXS
      SUBROUTINE LIBNXS(IPLIB,IPRINT,NGROUP,NBISO,NL,ISONAM,
     >                  ISONRF,ITISOM,NPROC,ISOADD,INDPRO,ITYPRO,
     >                  XSREC,SCAT,XSSCMP,NJJ,IJJ)
C
C------------------------------  LIBNXS  ------------------------------
C
C  PROGRAMME STATISTICS:
C     NAME     : LIBNXS
C     ENTRY    : LIBNXS
C     USE      : Generation of directory for the isotopes without
C                cross sections.
C     MODIFIED : 2006-04-04
C     AUTHOR   : G. MARLEAU
C
C  ROUTINE PARAMETERS:
C   INPUT
C     IPLIB  : POINTER TO THE INTERNAL LIBRARY      I
C     IPRINT : PRINT FLAG                           I
C     NGROUP : NUMBER OF GROUPS                     I
C     NBISO  : NUMBER OF ISOTOPES                   I
C     NL     : NUMBER OF LEGENDRE SCATTERING ORDER  I
C              =1 -> ISOTROPIC
C              =2 -> LINEARLY ANISOTROPIC
C     ISONAM : LOCAL ISOTOPE NAMES                  I(3,NBISO)
C     ISONRF : REFERENCE ISOTOPE NAMES              I(3,NBISO)
C     ITISOM : FLAG ISOTOPE PROCESSING              I(NBISO)
C     NPROC  : NUMBER OF POSSIBLE XS TO PROCESS     I
C     ISOADD : FLAG TO COMPLETE MIXTURE DEFINITION
C              IF DEPLETION CHAIN PRESENT
C              =-1 Partial completion (warning if missing information)
C              = 0 COMPLETE MIXTURES
C              = 1 NO COMPLETION
C     INDPRO : CROSS SECTION TO PROCESS             I(NPROC)
C     ITYPRO : CROSS SECTION PROCESSED              I(NPROC)
C   WORK
C     XSREC  : GENERAL XS VECTOR                    R(NGROUP,NPROC)
C     SCAT   : COMPLETE SCATTERING MATRIX           R(NGROUP,
C              SCAT(JG,IG) IS FROM IG TO JG         NGROUP,NL)
C     XSSCMP : COMPRESS SCATTERING FOR TRANSFER     R(NGROUP*
C                                                    (NGROUP+2))
C     NJJ    : NB SCATTERING GROUP                  I(NGROUP)
C     IJJ    : MAXIMUM SCATTERING GROUP             I(NGROUP)
C   INTERNAL PARAMETERS
C     IOUT   : OUTPUT UNIT = 6                      I

C
C------------------------------  LIBNXS  ------------------------------
C
      IMPLICIT    NONE
      INTEGER     IOUT
      CHARACTER*6 NAMSBR
      PARAMETER  (IOUT=6,NAMSBR='LIBNXS')
      INTEGER     IPLIB,IPRINT,NGROUP,NBISO,NL,NPROC,ISOADD
      INTEGER     ISONAM(3,NBISO),ISONRF(3,NBISO),
     >            ITISOM(NBISO),INDPRO(NPROC),ITYPRO(NPROC),
     >            NJJ(NGROUP),IJJ(NGROUP)
      REAL        XSREC(NGROUP,NPROC),
     >            SCAT(NGROUP,NGROUP,NL),XSSCMP(NGROUP*(NGROUP+2))
C----
C  INTERNAL PARAMETERS
C----
      INTEGER     JSO,ITC,NBMIS
      CHARACTER   HNAMIS*12,HNISOR*12,README*96
      INTEGER     IHGAR(24)
C----
C READ THROUGH DRAGON FILE AND ACCUMULATE CROSS SECTIONS FOR
C CROSS SECTION ARE SAVED ONLY IF ISOTOPE IS USED
C----
      CALL XDRSET(XSREC(1,1),NGROUP*NPROC,0.0)
      CALL XDRSET(SCAT,NGROUP*NGROUP*NL,0.0)
      IF(ISOADD .EQ. -1) THEN
        DO JSO=1,NBISO
          IF(ITISOM(JSO) .LT. 0) THEN
C----
C  LOCATE ISOTOPE
C----
            WRITE(HNAMIS,'(3A4)') (ISONAM(ITC,JSO),ITC=1,3)
            WRITE(HNISOR,'(3A4)') (ISONRF(ITC,JSO),ITC=1,3)
            IF(IPRINT.GE.5)  WRITE(IOUT,6000) NAMSBR,HNAMIS
C----
C  SAVE MAIN CROSS SECTIONS ON LCM
C----
            CALL LCMSIX(IPLIB,HNAMIS,1)
            INDPRO(5)=0
            CALL XDRLGS(IPLIB,1,NPROC,0,NL-1,1,NGROUP,INDPRO,
     >                ITYPRO,XSREC,SCAT,XSSCMP,NJJ,IJJ)
            INDPRO(5)=1
            WRITE(README,'(A8,A18,70X)') HNAMIS(1:8),
     >      ' has vanishing XS '
            READ(README,'(24A4)') (IHGAR(ITC),ITC=1,24)
            CALL LCMPUT(IPLIB,'README',24,3,IHGAR)
            IF(IPRINT.GE.100) CALL LCMLIB(IPLIB)
            CALL LCMSIX(IPLIB,' ',2)
            ITISOM(JSO)=-ITISOM(JSO)
          ENDIF      
        ENDDO
      ELSE
        NBMIS=0
        DO JSO=1,NBISO
          IF(ITISOM(JSO) .LT. 0) THEN
            NBMIS=NBMIS+1
            WRITE(HNAMIS,'(3A4)') (ISONAM(ITC,JSO),ITC=1,3)
            WRITE(HNISOR,'(3A4)') (ISONRF(ITC,JSO),ITC=1,3)
            WRITE(IOUT,6001) NAMSBR,HNAMIS
          ENDIF      
        ENDDO
        IF(NBMIS .GT. 0) CALL XABORT(NAMSBR//
     >  ': Isotopes not found') 
      ENDIF
C----
C  RETURN
C----
      RETURN
C----
C  FORMAT
C----
 6000 FORMAT('Warning from ',A6,' while processing isotope = ',A12,
     >       ' -- All the cross section vanishes')
 6001 FORMAT('Error from ',A6,' while processing isotope = ',A12,
     >       ' -- No cross section available')
      END
