*DECK LIBSEC
      SUBROUTINE LIBSEC(LLL,IANNN,NGRO,IX,UUU,DELTA,SIGS,SIG1,PRI,NLET,
     > STR,DEL,NRSTR,IANIS,ITY,NEXT,NEXU,NEXV,NEXW,III)
C
C-----------------------------------------------------------------------
C
C COMPUTE THE VALUES OF THE TRANSFER MACROSCOPIC CROSS SECTION FOR
C SECONDARY NEUTRONS IN GROUP LLL.
C
C INPUT PARAMETERS:
C  LLL     : GROUP NUMBER FOR SECONDARY NEUTRONS.
C  IANNN   : =-1 : TRANSPORT CORRECTED P0 ;      =0 : P0 ;      =1 : P1
C  NGRO    : NUMBER OF GROUPS.
C  IX      : NUMBER OF GROUPS WITH UP-SCATTERING.
C  UUU     : GROUPS LIMITS IN LETHARGY UNITS. DIMENSION UUU(MAXGRO)
C  DELTA   : GROUPS WIDTH IN LETHARGY UNITS. DIMENSION DELTA(MAXGRO)
C  SIGS    : DIFFUSION P0 MICROSCOPIC CROSS SECTIONS.
C  SIG1    : DIFFUSION P1 MICROSCOPIC CROSS SECTIONS.
C            DIMENSION SIGS(MAXISO,MAXGRO),SIG1(MAXISO,MAXGRO)
C  PRI     : TRANSFER MICROSCOPIC CROSS SECTIONS. DIMENSION PRI(MAXTRA)
C  DEL     : ELEMENTARY MESH ELEMENT IN LETHARGY.
C  NRSTR   : NUMBER OF X. SECTION STRUCTURES OWN BY THE ISOTOPE.
C  IANIS   : LEGENDRE ORDER CORRESPONDING TO EACH X. SECTION STRUCTURE.
C  ITY     : TYPE OF EACH X. SECTION STRUCTURE.
C  NEXT    : LENGTH OF EACH X. SECTION STRUCURE.
C  NEXU,NEXV,NEXW : INFORMATION RELATED TO EACH X. SECTION STRUCTURE.
C  III     : OFFSET IN VECTOR PRI OF EACH X. SECTION STRUCTURE.
C            DIMENSION IANIS(80),ITY(80),NEXT(80),NEXU(80),NEXV(80),
C            NEXW(80),III(80)
C
C OUTPUT PARAMETERS:
C  NLET    : NUMBER OF DOWN-SCATTERING GROUPS (INCLUDING GROUP LLL)
C  STR     : VALUES OF THE TRANSFER MACROSCOPIC CROSS SECTION.
C            STR(1)     : FROM GROUP LLL
C            STR(2)     : FROM GROUP LLL-1
C            STR(LLL)   : FROM GROUP 1
C            STR(LLL+1) : FROM GROUP NGRO
C            STR(LLL+2) : FROM GROUP NGRO-1
C            STR(NGRO)  : FROM GROUP LLL+1
C            DIMENSION STR(NGRO)
C
C---------------------------------------- AUTHOR: A. HEBERT 29/10/87 ---
C
      IMPLICIT NONE
C----
C LOCAL & INTERFACE VARIABLES
C----
      INTEGER          NGROIN,NGRO,IX,IAN,IANNN,MM,NLET,K,LLL,
     1                 LDELH,LARGRL,LDELB,NP2,IHM,LTES,ISOTOP,INDICE,
     2                 MM1,MDELH,LARGRM,MDELB,MAX0,LARG,NRSTR,J,IBAS,
     3                 LARGLI,I,IHAUT,INTER1,INTER2,MDEL,MML,NEX1,NEX2,
     4                 IPR,LN3,LN1,NEX3,NEX4
      INTEGER          IANIS(*),ITY(*),NEXT(*),NEXU(*),NEXV(*),
     1                 NEXW(*),III(*)
C
      REAL             DEL
      REAL             UUU(*),DELTA(*),PRI(*),SIGS(*),SIG1(*),STR(*)
      DOUBLE PRECISION DAUX
      LOGICAL          SELF
C-----
      NGROIN=NGRO-IX
      IAN=IANNN
      IF(IANNN.EQ.-1)IAN=0
      DO 71 MM=1,NGRO
   71 STR(MM)=0.0
      NLET=1
      SELF=.FALSE.
      IF (NRSTR.GT.0) THEN
         DO 250 K=1,NRSTR
         IF (IANIS(K).NE.IAN) GOTO 250
         IF (ITY(K).EQ.1) THEN
*           ELASTIC SLOWING-DOWN MATRIX.
            IF (LLL.GT.NGROIN) GO TO 250
            SELF=.TRUE.
            LDELH=UUU(LLL)/DEL+0.1
            LARGRL=DELTA(LLL)/DEL+0.1
            LDELB=LDELH-LARGRL+1
            NP2=NEXT(K)
            IHM=III(K)+NP2-1
            LTES=LDELB-NP2
            ISOTOP=K
            INDICE=1
            DO 210 MM1=1,LLL
            MM=LLL-MM1+1
            MDELH=UUU(MM)/DEL+0.1
            IF(MDELH.LE.LTES)GOTO 211
            LARGRM=DELTA(MM)/DEL+0.1
            MDELB=MDELH-LARGRM+1
            MDELB=MAX0(MDELB,LTES+1)
            DAUX=0.0
            LARG=MIN0(LARGRM,LARGRL)
            IF(LARG.GT.4) THEN
               IHAUT=LDELH-MDELB+III(K)
               IHAUT=MIN0(IHAUT,IHM)
               IF (INDICE.EQ.2) THEN
                  J=0
                  IBAS=LDELB-MDELH+III(K)
                  IBAS=MAX0(IBAS,III(K))
                  LARGLI=IABS(LARGRM-LARGRL)
                  INTER1=IBAS+LARG-2
                  INTER1=MIN0(INTER1,IHAUT)
                  DO 182 I=IBAS,INTER1
                  J=J+1
  182             DAUX=DAUX+PRI(I)*FLOAT(J)
                  INTER1=INTER1+1
                  INTER2=INTER1+LARGLI
                  INTER2=MIN0(IHAUT,INTER2)
                  IF(INTER1.GT.INTER2) GO TO 1004
                  J=LARG
                  DO 183 I=INTER1,INTER2
  183             DAUX=DAUX+PRI(I)*FLOAT(LARG)
               ELSE IF (INDICE.EQ.1) THEN
                  INDICE=2
                  INTER2=III(K)-1
                  J=LARG+1
               ENDIF
               INTER2=INTER2+1
               DO 184 I=INTER2,IHAUT
               J=J-1
  184          DAUX=DAUX+PRI(I)*FLOAT(J)
            ELSE
               DO 82 MDEL=MDELB,MDELH
               IBAS=LDELB-MDEL+III(K)
               IHAUT=LDELH-MDEL+III(K)
               IBAS=MAX0(IBAS,III(K))
               IHAUT=MIN0(IHAUT,IHM)
               DO 82 I=IBAS,IHAUT
   82          DAUX=DAUX+PRI(I)
            ENDIF
 1004       STR(MM1)=STR(MM1)+DAUX*SIGS(MM)*DEL/DELTA(LLL)
  210       CONTINUE
            MM=MM-1
  211       MML=LLL-MM
         ELSE IF (ITY(K).EQ.4) THEN
*           STANDARD GALOCHE.
            IF (LLL.GT.NGROIN) GO TO 250
            SELF=.TRUE.
            NEX1=NEXU(K)
            NEX2=NEXV(K)
            NEX3=NEXW(K)
            IF(LLL.GT.(NEX2+NEX3)) GO TO 801
            IPR=III(K)-1+(LLL*(LLL-1))/2
            DO 802 I=1,LLL
            IPR=IPR+1
  802       STR(I)=STR(I)+PRI(IPR)
            MML=LLL
            GO TO 240
  801       IF(LLL.GT.NEX1) GO TO 803
            IPR=III(K)-1+LLL*(NEX2+NEX3)-((NEX2+NEX3)*(NEX2+NEX3+1))/2
            DO 804 I=1,NEX2
            IPR=IPR+1
  804       STR(I)=STR(I)+PRI(IPR)
            LN3=LLL-NEX3+1
            DO 807 I=LN3,LLL
            IPR=IPR+1
  807       STR(I)=STR(I)+PRI(IPR)
            MML=LLL
            GO TO 240
  803       IF(NEX2.EQ.0) GO TO 250
            IPR=III(K)-1+NEX1*NEX3-((NEX2+NEX3)*(NEX2+NEX3-1))/2+
     1      (LLL-1)*NEX2
            DO 813 I=1,NEX2
            IPR=IPR+1
  813       STR(I)=STR(I)+PRI(IPR)
            MML=NEX2
         ELSE IF (ITY(K).EQ.7) THEN
*           THERMAL TRANSFER MATRIX.
            IF (LLL.LE.NGROIN) GO TO 250
            SELF=.TRUE.
            IPR=III(K)-1+(NGRO-LLL)*IX
            DO 5003 MM=1,LLL-NGROIN
 5003       STR(MM)=STR(MM)+PRI(IPR+MM+NGRO-LLL)
            DO 5004 MM=1,NGRO-LLL
 5004       STR(MM+LLL)=STR(MM+LLL)+PRI(IPR+MM)
            MML=LLL-NGROIN
         ELSE IF (ITY(K).EQ.8) THEN
*           RECTANGLE SLOWING-DOWN MATRIX.
            IF(LLL.LT.NEXU(K))GO TO 250
            IF(LLL.GT.NEXV(K))GO TO 250
            IPR=III(K)-1+(LLL-NEXU(K))*NEXW(K)
            LN1=LLL-NEXU(K)+2
            DO 355 I=LN1,LN1+NEXW(K)-1
            IPR=IPR+1
  355       STR(I)=STR(I)+PRI(IPR)
            MML=NEXW(K)+LLL-NEXU(K)+1
         ELSE IF (ITY(K).EQ.9) THEN
*           GREULING-GOERTZEL SLOWING DOWN MATRIX.
            NEX1=NEXU(K)
            NEX3=NEXW(K)
            IF ((LLL.LT.NEX1).OR.(LLL.GT.NEX3)) GO TO 250
            SELF=.TRUE.
            NEX2=NEXV(K)
            NEX4=NEXT(K)/3
            MML=LLL-NEX1+1
            IPR=NEX3-LLL+III(K)-1
            J=IPR+1+NEX4
            DAUX=PRI(J)
            J=IPR+1+2*NEX4
            IF(LLL.GT.NEX2) THEN
               LN1=LLL-NEX2+1
            ELSE
               LN1=1
               STR(1)=STR(1)+PRI(J)
            ENDIF
            J=IPR+LN1-1
            DO 360 MM=LN1,MML
            J=J+1
  360       STR(MM)=STR(MM)+PRI(J)*DAUX
         ENDIF
  240    NLET=MAX0(NLET,MML)
  250    CONTINUE
      ENDIF
      IF ((.NOT.SELF).AND.(IAN.EQ.0)) THEN
         STR(1)=STR(1)+SIGS(LLL)
      ELSE IF ((.NOT.SELF).AND.(IAN.EQ.1)) THEN
         STR(1)=STR(1)+3.0*SIG1(LLL)
      ENDIF
      IF (IANNN.EQ.-1) THEN
         STR(1)=STR(1)-SIG1(LLL)
      ENDIF
      RETURN
      END
