*DECK LIBTER
      SUBROUTINE LIBTER (NGRO,NSUBM,TMIX,SMIX,TN,SN,TERP)
C
C-----------------------------------------------------------------------
C
C COMPUTE THE LAGRANGE INTERPOLATION FACTORS (TERP) FOR TEMPERATURE AND
C DILUTION INTERPOLATION OF CROSS SECTIONS.
C
C INPUT PARAMETERS:
C  NGRO    : NUMBER OF ENERGY GROUPS.
C  NSUBM   : NUMBER OF SUBMATERIALS (NUMBER OF TEMPERATURE/DILUTION
C            COLLOCATION POINTS).
C  TMIX    : TEMPERATURE OF EACH SUBMATERIAL IN THE LIBRARY.
C  SMIX    : DILUTION OF EACH SUBMATERIAL IN THE LIBRARY.
C            DIMENSION TMIX(NSUBM),SMIX(NSUBM)
C            THE SUBMATERIALS ARE ORDERED BY DECREASING DILUTION AND
C            THEN BY INCREASING TEMPERATURE.
C  TN      : TEMPERATURE OF THE ISOTOPE.
C  SN      : DILUTION CROSS SECTION IN EACH ENERGY GROUP OF THE ISOTOPE.
C            A VALUE OF 1.0E10 IS USED FOR INFINITE DILUTION.
C            DIMENSION SN(NGRO)
C
C OUTPUT PARAMETERS:
C  TERP    : LAGRANGE INTERPOLATION FACTORS. DIMENSION TERP(NGRO,NSUBM)
C
C--------------------------------------- AUTHOR: A. HEBERT 05/04/88 ----
C
      IMPLICIT NONE
C----
C PARAMETERS
C----
      INTEGER    IOUT
      PARAMETER (IOUT=6)
C----
C INTERFACE VARIABLES
C----
      INTEGER           NGRO,NSUBM
      REAL              TN,TMIX(*),SMIX(*),SN(*),TERP(NGRO,*)
C----
C LOCAL VARABLES
C----
      INTEGER           I,ISUBM,JJ,ISM
      REAL              BREAK,SMIN,ST,TMAT,SMAT,TERPT,TT,TERPS,TLAST
      DOUBLE PRECISION  CHECK
C----
C TEMPERATURE AND BACKGROUND LOOP
C-----
      IF (NSUBM.EQ.1) THEN
         DO 10 I=1,NGRO
   10    TERP(I,1)=1.0
         RETURN
      ENDIF
      BREAK=0.0
      SMIN=1.0E10
      DO 20 ISUBM=1,NSUBM
      ST=SMIX(ISUBM)
      IF ((ST.LT.1.0E8).AND.(ST.GT.BREAK)) BREAK=ST
      IF (ST.LT.SMIN) SMIN=ST
   20 CONTINUE
      DO 70 ISUBM=1,NSUBM
      DO 30 JJ=1,NGRO
   30 TERP(JJ,ISUBM)=0.0
      TMAT=TMIX(ISUBM)
      SMAT=SMIX(ISUBM)
*
*     COMPUTE TERP FACTORS
*
      TERPT=1.0
      DO 40 ISM=1,NSUBM
      TT=TMIX(ISM)
      ST=SMIX(ISM)
      IF (ST.LE.0.99E10) GO TO 40
      IF (ABS(TMAT-TT).LT.1.0E-5*TMAT+1.0E-5) GO TO 40
      TERPT=TERPT*(TN-TT)/(TMAT-TT)
      IF (ABS(TERPT).LT.1.0E-5) GO TO 70
   40 CONTINUE
      DO 60 JJ=1,NGRO
      TERPS=TERPT
      IF ((SN(JJ).LT.SMIN).AND.(SMAT.GT.1.01*SMIN)) GO TO 60
      IF ((SN(JJ).LT.BREAK).AND.(SMAT.GT.1.01*BREAK)) GO TO 60
      IF ((SN(JJ).GE.BREAK).AND.(SMAT.LT.0.99*BREAK)) GO TO 60
      TLAST=-1.0
      DO 50 ISM=1,NSUBM
      TT=TMIX(ISM)
      ST=SMIX(ISM)
      IF (TLAST.LT.0.) TLAST=TT
      IF (TT.NE.TLAST) GO TO 50
      IF (ABS(SMAT-ST).LT.1.0E-5*SMAT) GO TO 50
      IF ((SN(JJ).GE.SMIN).AND.(SN(JJ).LT.BREAK)) THEN
         IF (ST.GT.1.01*BREAK) GO TO 50
         TERPS=TERPS*LOG(SN(JJ)/ST)/LOG(SMAT/ST)
      ELSE IF ((SN(JJ).GE.SMIN).AND.(SN(JJ).GE.BREAK)) THEN
         IF (ST.LT.0.99*BREAK) GO TO 50
         TERPS=TERPS*((ST/SN(JJ))-1.)/((ST/SMAT)-1.)
      ELSE
         IF (ST.GT.1.01*SMIN) GO TO 50
         TERPS=TERPS*(SN(JJ)**2-ST**2)/(SMAT**2-ST**2)
      ENDIF
      IF (ABS(TERPS).LE.1.0E-5) GO TO 60
   50 CONTINUE
      TERP(JJ,ISUBM)=TERPS
   60 CONTINUE
   70 CONTINUE
*
*     CHECK FOR CONSISTENCY OF THE TERP FACTORS.
*
      DO 90 JJ=1,NGRO
      CHECK=0.0D0
      DO 80 ISUBM=1,NSUBM
   80 CHECK=CHECK+TERP(JJ,ISUBM)
      IF (ABS(CHECK-1.0D0).GT.5.0D-3) THEN
         WRITE (IOUT,100) JJ,CHECK,(TERP(JJ,ISUBM),ISUBM=1,NSUBM)
         CALL XABORT('LIBTER: INTERPOLATION FAILURE.')
      ENDIF
   90 CONTINUE
      RETURN
  100 FORMAT (/51H LIBTER: INCONSISTENT LAGRANGE INTERPOLATION FACTOR,
     1 9H IN GROUP,I4,8H. CHECK=,1P,E13.3/9H FACTORS=,1P,9E13.3/
     2 (9X,1P,9E13.3))
      END
