*DECK LIBWIM
      SUBROUTINE LIBWIM(IPLIB,IPRINT,CFILNA,NGROUP,NBISO,NL,ISONAM,
     >                  ISONRF,ISHINA,TN,AW,SN,SB,ILIB,ITISOM,DELTA,
     >                  ITRANC,NPROC,INDPRO,ITYPRO,XSREC,SCAT,XSSCMP,
     >                  NJJ,IJJ,XSOUT)
C
C------------------------------  LIBWIM  ------------------------------
C
C  PROGRAMME STATISTICS:
C     NAME     : LIBWIM
C     ENTRY    : LIBWIM
C     USE      : TRANSCRIPTION OF THE INTERPOLATED MICROSCOPIC XS READ
C                FROM A MICROSCOPIC XS LIBRARY IN WIMS-AECL FORMAT
C                TO LCM DRAGON FORMAT
C     MODIFIED : 93-08-17
C     AUTHOR   : G. MARLEAU
C
C  ROUTINE PARAMETERS:
C   INPUT
C     IPLIB  : POINTER TO THE INTERNAL LIBRARY      I
C     IPRINT : PRINT FLAG                           I
C     CFILNA : WIMS FILE NAME                       C*8
C     NGROUP : NUMBER OF GROUPS                     I
C     NBISO  : NUMBER OF ISOTOPES                   I
C     NL     : NUMBER OF LEGENDRE SCATTERING ORDER  I
C              =1 -> ISOTROPIC
C              =2 -> LINEARLY ANISOTROPIC
C     ISONAM : LOCAL ISOTOPE NAMES                  I(3,NBISO)
C     ISONRF : REFERENCE ISOTOPE NAMES              I(3,NBISO)
C     ISHINA : SHIELDING ISOTOPE NAMES              I(2,NBISO)
C     TN     : ISOTOPE TEMPTERATURE                 R(NBISO)
C     AW     : ISOTOPE ATOMIC WEIGHT                R(NBISO)
C     SN     : DILUTION XS                          R(NGROUP,NBISO)
C     SB     : L-J DILUTION XS                      R(NGROUP,NBISO)
C     ILIB   : LIBRARY NUMBER TO PROCESS            I
C     ITISOM : FLAG ISOTOPE PROCESSING              I(NBISO)
C     DELTA  : LETHERGY                             R(NGROUP)
C     ITRANC : TRANSPORT CORRECTION                 I
C              0 -> NONE ; 1 -> APOL; 2 -> WIMS
C              3 -> OLD-WIMS
C     NPROC  : NUMBER OF POSSIBLE XS TO PROCESS     I
C     INDPRO : CROSS SECTION TO PROCESS             I(NPROC)
C     ITYPRO : CROSS SECTION PROCESSED              I(NPROC)
C   WORK
C     XSREC  : GENERAL XS VECTOR                    R(NGROUP,NPROC)
C     SCAT   : COMPLETE SCATTERING MATRIX           R(NGROUP,
C              SCAT(JG,IG) IS FROM IG TO JG         NGROUP,NL)
C     XSSCMP : COMPRESS SCATTERING FOR TRANSFER     R(NGROUP*
C                                                    (NGROUP+2))
C     NJJ    : NB SCATTERING GROUP                  I(NGROUP)
C     IJJ    : MAXIMUM SCATTERING GROUP             I(NGROUP)
C     XSOUT  : SELF SHIELDING PARAMETER             R(NGROUP,7)
C   INTERNAL PARAMETERS
C     IOUT   : OUTPUT UNIT = 6                      I
C     MAXTEM : MAX. NB. OF TMPT = 20                I
C     MAXRES : MAX. NB. OF DIL = 50                 I
C     NOTX   : INTERPOLATION METHOD = -1            I
C   WIMS-AECL LIBRARY PARAMETERS
C     IUTYPE : TYPE OF FILE = 4 (DA)                I
C     LRIND  : LENGHT RECORD ON DA FILE = 256       I
C     IACTO  : OPEN ACTION = 2 (READ ONLY)          I
C     IACTC  : CLOSE ACTION = 2 (KEEP)              I
C     MAXISO : MAX. NB. OF ISO = 246                I
C     NCT    : NUMBER OF C*8 IN TITLE = 10          I
C     LPZ    : LENGTH OF WIMS PARAMETER ARRAY = 9   I
C     LMASTB : LENGTH OF MST TAB = MAXISO+9         I
C     LMASIN : LENGTH OF MST IDX = LMASTB-4         I
C     LGENTB : LENGTH OF GEN TAB = 6                I
C     LGENIN : LENGTH OF GEN IDX = LGENTB           I
C     LSUBTB : LENGTH OF SUB TAB = 6*MAXTEM+21-5+12 I
C     LSUBIN : LENGTH OF SUB IDX = LSUBTB-12        I
C     LRESTB : LENGTH OF RES TAB = 5*MAXRES         I
C     LRESIN : LENGTH OF RES IDX = LRESIN           I
C     MASTER : MASTER INDEX ARRAY                   I(LMASTB)
C     GENINX : GENERAL INDEX ARRAY                  I(LGENTB)
C     SUBINX : SUB INDEX ARRAY GENERAL              I(LSUBTB)
C     SUBINR : SUB INDEX ARRAY RESONANCE            I(LSUBTB)
C     RESINX : RESONANCE INDEX ARRAY                I(LRESTB)
C     IWISO  : ID OF ISOTOPE                        I(2*MAXISO)
C     CWISO  : ISOTOPE NAMES                        C(MAXISO)*8
C     MASTER : MASTER INDEX ARRAY                   I(LMASTB)
C     GENINX : GENERAL INDEX ARRAY                  I(LGENTB)
C     SUBINX : SUB INDEX ARRAY                      I(LSUBTB)
C
C------------------------------  LIBWIM  ------------------------------
C
      IMPLICIT    NONE
      INTEGER     IOUT,NDPROC,ITLIB,MAXRES,MAXTEM,MAXDIL,NOTX
      PARAMETER  (IOUT=6,NDPROC=20,ITLIB=1,MAXRES=50,MAXTEM=20,
     >            MAXDIL=20,NOTX=-1)
      INTEGER     IPLIB,IPRINT,NGROUP,NBISO,NL,ILIB,ITRANC,NPROC
      CHARACTER  CFILNA*8
      INTEGER     ISONAM(3,NBISO),ISONRF(3,NBISO),ISHINA(2,NBISO),
     >            ITISOM(NBISO),INDPRO(NPROC),ITYPRO(NPROC),
     >            NJJ(NGROUP),IJJ(NGROUP)
      REAL        TN(NBISO),AW(NBISO),SN(NGROUP,NBISO),
     >            SB(NGROUP,NBISO),DELTA(NGROUP),XSREC(NGROUP,NPROC),
     >            SCAT(NGROUP,NGROUP,NL),XSSCMP(NGROUP*(NGROUP+2)),
     >            XSOUT(NGROUP,7)
      REAL        CONVM
C----
C FUNCTIONS
C----
      DOUBLE PRECISION XDRCST
C----
C  INTERNAL PARAMETERS
C----
      CHARACTER   HNAMIS*12,HNISOR*12,HSHIR*8,README*96,FMT*6
      INTEGER     IHGAR(24)
      REAL        TMPT(MAXTEM),DILT(MAXTEM),RS1(3,MAXRES),XSCOR(4)
      DOUBLE PRECISION TERP(MAXTEM)
      INTEGER     NRES,IGR,IGRF,JRES,IGF,LRESND,ITXS,ITMP,IACT,
     >            LSUBZ,LSUBTR,NTYP,IGRL,ILL,I,IG2,IG1,NDILR,
     >            NTMPR,KRES,IRRICS,ITT,NTD,NGTHER,NGFR,NGF,NGR,
     >            ILOCX,ILOCY,MXSCT,IP1,IUNIT,IP0,II,
     >            NEL,KDROPN,IHANDL,IG,NF,IENDF,IREC,JJJ,NSCT,NTMP,
     >            JSO,ITC,ILOCS,NRDT,IEL,IRISO,IDRES
      REAL        DSIGPL,ASIGPL,XRS1,RIND
C----
C  MEMORY ALLOCATION PARAMETERS
C----
      INTEGER     IBASE(1)
      REAL        RBASE
      COMMON      RBASE(1)
      EQUIVALENCE(RBASE(1),IBASE(1))
      INTEGER     ITMPXS,ITMPSC,JRRI,JRIT
C----
C  WIMS-AECL LIBRARY PARAMETERS
C----
      INTEGER     IUTYPE,LRIND,IACTO,IACTC,MAXISO,NCT,LPZ,LMASTB,
     >            LMASIN,LGENTB,LGENIN,LSUBTB,LSUBIN,LRESTB,LRESIN
      PARAMETER  (IUTYPE=4,LRIND=256,IACTO=2,IACTC=1,
     >            MAXISO=246,NCT=10,LPZ=9,LMASTB=MAXISO+9,
     >            LMASIN=LMASTB-4,LGENTB=6,LGENIN=LGENTB,
     >            LSUBTB=6*MAXTEM+28,LSUBIN=LSUBTB-12,
     >            LRESTB=MAXRES*5,LRESIN=LRESTB)
      CHARACTER   CWISO(MAXISO)*8,CTITLE(NCT)*8
      INTEGER     MASTER(LMASTB),GENINX(LGENTB),SUBINX(LSUBTB),
     >            SUBINR(LSUBTB),RESINX(LRESTB),NXS(MAXTEM),
     >            ITITLE(2*NCT),NPZ(LPZ),
     >            IWISO(2*MAXISO)
      REAL        AWR
      EQUIVALENCE(SUBINX(LSUBIN+3),AWR)
C----
C  OPEN WIMSLIB AND READ TITLE
C  READ GENERAL DIMENSIONING
C----
      CONVM=XDRCST('Neutron mass','amu')
      IP0=NDPROC+1
      IP1=NDPROC+2
      IUNIT=KDROPN(CFILNA,IACTO,IUTYPE,LRIND,IHANDL)
      IF(IUNIT.LE.0) CALL XABORT('LIBWIM: WIMS-AECL LIBRARY '//
     >    CFILNA//' CANNOT BE OPENED FOR MIXS')
      CALL OPNIND(IUNIT,MASTER,LMASTB)
      CALL REDIND(IUNIT,MASTER,LMASIN,GENINX,LGENTB,1)
      CALL REDIND(IUNIT,MASTER,LMASIN,ITITLE,2*NCT,2)
      CALL UPCKIC(ITITLE,CTITLE,NCT)
      WRITE(README(9:96),'(6H FROM ,10A8,A2)')
     >  (CTITLE(II),II=1,NCT),'  '
      IF(IPRINT.GE.5) THEN
        WRITE(IOUT,6000) CFILNA
        WRITE(IOUT,'(1X,10A8)') (CTITLE(II),II=1,NCT)
      ENDIF
      CALL REDIND(IUNIT,GENINX,LGENIN,NPZ,LPZ,1)
      IF(NPZ(2).NE.NGROUP) THEN
        WRITE(IOUT,9001) NGROUP,NPZ(2)
        CALL XABORT('LIBWIM: INVALID NUMBER OF GROUPS')
      ENDIF
      NEL=NPZ(1)
      NGF=NPZ(4)
      NGR=NPZ(5)
      NGTHER=NPZ(6)
      NGFR=NGF+NGR
      MXSCT=NGROUP*(NGROUP+2)
      IF(NGFR+NGTHER.NE.NGROUP) THEN
        WRITE(IOUT,9001) NGROUP,NGFR+NGTHER
        CALL XABORT('LIBWIM: INVALID NUMBER OF GROUPS')
      ENDIF
      IF(NEL.GT.MAXISO) THEN
        WRITE(IOUT,9003) MAXISO,NEL
        CALL XABORT('LIBWIM: INVALID NUMBER OF ISOTOPES')
      ENDIF
C----
C  READ ISOTOPES NAMES
C----
      CALL REDIND(IUNIT,GENINX,LGENIN,IWISO,2*NEL,3)
      CALL UPCKIC(IWISO,CWISO,NEL)
      CALL REDIND(IUNIT,GENINX,LGENIN,IWISO,NEL,2)
C----
C  INITIALIZE ALL XSREC
C  READ FISSION SPECTRUM
C----
      CALL XDRSET(XSREC(1,5),NGROUP,0.0)
      CALL REDIND(IUNIT,GENINX,LGENIN,XSREC(1,5),NPZ(3),5)
C----
C  ALLOCATE MEMORY FOR TEMPERATURE DEPENDENT XS
C  AND FOR RESONANCE CALCULATION
C----
      CALL SETARA(RBASE,5*NGROUP,ITMPXS)
      CALL SETARA(RBASE,NGROUP*NGROUP,ITMPSC)
      CALL SETARA(RBASE,MAXDIL*MAXTEM*2,JRRI)
      CALL SETARA(RBASE,MAXDIL,JRIT)
      ILOCX=ITMPXS
      ILOCY=ILOCX+NGFR
      ILOCS=ITMPSC
      NRDT=NGTHER-1
C----
C READ THROUGH DRAGON FILE AND ACCUMULATE CROSS SECTIONS FOR
C CROSS SECTION ARE SAVED ONLY IF ISOTOPE IS USED
C----
      DO 110 JSO=1,NBISO
        IF(ABS(ITISOM(JSO)).NE.ILIB) GO TO 115
C----
C  LOCATE ISOTOPE
C----
        WRITE(HNAMIS,'(3A4)') (ISONAM(ITC,JSO),ITC=1,3)
        WRITE(HNISOR,'(3A4)') (ISONRF(ITC,JSO),ITC=1,3)
        WRITE(HSHIR,'(2A4)') (ISHINA(ITC,JSO),ITC=1,2)
        IDRES=INDEX(HSHIR,'.')
        IF(IDRES.GT.0) THEN
          WRITE(FMT,'(2H(F,I1,3H.1))') IDRES+1
          READ(HSHIR,FMT) RIND
        ENDIF
        DO 120 IEL=1,NEL
          IF(CWISO(IEL).EQ.HNISOR(1:8)) THEN
            IRISO=IEL
            IF(IDRES.EQ.0) THEN
              RIND=FLOAT(IWISO(IRISO))
            ENDIF
            ITISOM(JSO)=ABS(ITISOM(JSO))
            GO TO 125
          ENDIF
 120    CONTINUE
        IF(ITISOM(JSO) .LT. 0) GO TO 115
        WRITE(IOUT,9002) HNISOR,CFILNA
        CALL XABORT('LIBWIM: ISOTOPE NOT FOUND ON LIBRARY')
 125    CONTINUE
        IF(IPRINT.GE.5) THEN
          WRITE(IOUT,6001) HNAMIS
        ENDIF
        CALL XDRSET(XSREC(1,1),NGROUP*4,0.0)
        CALL XDRSET(XSREC(1,6),NGROUP*(NPROC-5),0.0)
        CALL XDRSET(SCAT,NGROUP*NGROUP*NL,0.0)
C----
C  READ SUB INDEX ASSOCIATED WITH ISOTOPE
C----
        CALL REDIND(IUNIT,MASTER,LMASIN,SUBINX,LSUBTB,IRISO+4)
C----
C  FOR ENDF/B-VI LIBRARY : IENDF = 2
C  FOR ENDF/B-V  LIBRARY : IENDF = 1
C  FOR WINFRITH  LIBRARY : IENDF = 0
C----
        IENDF=SUBINX(LSUBIN+12)
        AW(JSO)=AWR/CONVM
C----
C  FAST AND/OR RESONANCE XS
C----
        CALL REDIND(IUNIT,SUBINX,LSUBIN,XSREC(NGF+1,14),NGR,9)
        IF(IENDF.EQ.0) THEN
          CALL REDIND(IUNIT,SUBINX,LSUBIN,XSREC(NGF+1,15),NGR,2)
          DSIGPL=0.0
          DO 130 IG=NGF+1,NGFR
            DSIGPL=DSIGPL+XSREC(IG,15)*XSREC(IG,14)
 130      CONTINUE
          DSIGPL=DSIGPL/FLOAT(NGR)
        ELSE
          DSIGPL=0.0
        ENDIF
        CALL REDIND(IUNIT,SUBINX,LSUBIN,XSREC(1,9),NGF,13)
        NF=SUBINX(LSUBIN+5)
        IF(NF.GT.1) THEN
          CALL REDIND(IUNIT,SUBINX,LSUBIN,XSREC(1,3),NGFR,10)
          CALL REDIND(IUNIT,SUBINX,LSUBIN,XSREC(1,4),NGFR,12)
        ENDIF
        NSCT=SUBINX(LSUBIN+8)
        IF(NSCT.GT.MXSCT) THEN
          WRITE(IOUT,9004) NSCT,MXSCT
          CALL XABORT('LIBWIM: INVALID SCATTERING MATRIX -E/0')
        ENDIF
C----
C  READ AND DECOMPRESS P0 SCATTERING CROSS SECTIONS
C  COMPUTE TOTAL P0 SCATTERING OUT OF GROUP
C----
        CALL REDIND(IUNIT,SUBINX,LSUBIN,XSSCMP,NSCT,14)
        CALL LIBWSC(NGROUP,1,NGFR,NSCT,XSSCMP,
     >              SCAT,XSREC(1,IP0))
C----
C  FOR IENDF=2   READ XS FOR NG AND TOTAL
C  FOR IENDF=0,1 READ XS FOR NG AND TRANSPORT
C----
        IF(IENDF.GE.2) THEN
C----
C  READ TOTAL XS FOR IENDF=2
C----
          CALL REDIND(IUNIT,SUBINX,LSUBIN,XSREC(1,2),NGFR,5)
          CALL REDIND(IUNIT,SUBINX,LSUBIN,XSREC(1,7),NGFR,5)
        ELSE
C----
C  COMPUTE TOTAL XS FOR IENDF=0,1
C----
          CALL REDIND(IUNIT,SUBINX,LSUBIN,XSREC(1,2),NGFR,4)
          CALL REDIND(IUNIT,SUBINX,LSUBIN,XSREC(1,7),NGFR,6)
        ENDIF
C----
C  THERMAL XS
C----
        NTMP=SUBINX(LSUBIN+6)
        IF(NTMP.GT.MAXTEM) THEN
          CALL XABORT('LIBWIM: INVALID MAXTEM FOR P0.')
        ENDIF
        CALL REDIND(IUNIT,SUBINX,LSUBIN,NXS,NTMP,3)
        CALL REDIND(IUNIT,SUBINX,LSUBIN,TMPT,NTMP,15)
        IREC=16
        IF(NTMP.EQ.1) THEN
          IF(IPRINT.GE.100) THEN
            WRITE(IOUT,6200) TN(JSO)
          ENDIF
          IREC=IREC+2
          IF(NF.GT.1) THEN
            CALL REDIND(IUNIT,SUBINX,LSUBIN,XSREC(NGFR+1,3),
     >        NGTHER,IREC)
            CALL REDIND(IUNIT,SUBINX,LSUBIN,XSREC(NGFR+1,4),
     >        NGTHER,IREC+1)
          ENDIF
          IREC=IREC+2
          IF(NXS(1).GT.MXSCT) THEN
            WRITE(IOUT,9004) NXS(1),MXSCT
            CALL XABORT('LIBWIM: INVALID SCATTERING MATRIX -T/0')
          ENDIF
          CALL REDIND(IUNIT,SUBINX,LSUBIN,XSSCMP,NXS(1),IREC)
          IREC=IREC+1
C----
C  DECOMPRESS P0 SCATTERING CROSS SECTIONS AND COMPUTE
C  P0 SCATTERING OUT OF GROUP
C----
          CALL LIBWSC(NGROUP,NGFR+1,NGROUP,NXS(1),XSSCMP,
     >                SCAT,XSREC(1,IP0))
          IF(IENDF.GE.2) THEN
            CALL REDIND(IUNIT,SUBINX,LSUBIN,XSREC(NGFR+1,2),
     >                  NGTHER,IREC-4)
          ELSE
            CALL REDIND(IUNIT,SUBINX,LSUBIN,XSREC(NGFR+1,2),
     >                  NGTHER,IREC-5)
          ENDIF
          CALL REDIND(IUNIT,SUBINX,LSUBIN,XSREC(NGFR+1,7),
     >                NGTHER,IREC-4)
        ELSE IF(NTMP.GT.1) THEN
C----
C  AVALUATE LAGRANGIAN INTERPOLATION FACTOR FOR
C  AVAILABLE TEMPERATURES (ORDER NOTX) AND INTERPOLATE.
C----
          CALL LIBLEX(NTMP,TN(JSO),TMPT,NOTX,TERP)
          IF(IPRINT.GE.100) THEN
            WRITE(IOUT,6201) TN(JSO)
            WRITE(IOUT,6202) (TMPT(JJJ),JJJ=1,NTMP)
            WRITE(IOUT,6203) (TERP(JJJ),JJJ=1,NTMP)
          ENDIF
          NRDT=NGTHER-1
          IACT=1
          DO 140 ITMP=1,NTMP
            IF(TERP(ITMP).EQ.0.0D0) THEN
              IREC=IREC+5
            ELSE
              IREC=IREC+2
              IF(NF.GT.1) THEN
                CALL REDIND(IUNIT,SUBINX,LSUBIN,RBASE(ILOCY+2*NGROUP),
     >                      NGTHER,IREC)
                CALL REDIND(IUNIT,SUBINX,LSUBIN,RBASE(ILOCY+3*NGROUP),
     >                      NGTHER,IREC+1)
              ELSE
                CALL XDRSET(RBASE(ILOCY+2*NGROUP),NGTHER,0.0)
                CALL XDRSET(RBASE(ILOCY+3*NGROUP),NGTHER,0.0)
              ENDIF
              IREC=IREC+2
              IF(NXS(ITMP).GT.MXSCT) THEN
                WRITE(IOUT,9004) NXS(ITMP),MXSCT
                CALL XABORT('LIBWIM: INVALID SCATTERING MATRIX -T/0')
              ENDIF
              CALL REDIND(IUNIT,SUBINX,LSUBIN,XSSCMP,NXS(ITMP),IREC)
              IREC=IREC+1
C----
C  DECOMPRESS P0 SCATTERING CROSS SECTIONS AND COMPUTE
C  P0 SCATTERING OUT OF GROUP
C----
              CALL LIBWSC(NGROUP,NGFR+1,NGROUP,NXS(ITMP),XSSCMP,
     >                    RBASE(ILOCS),RBASE(ILOCX+4*NGROUP))
              IF(IENDF.GE.2) THEN
                CALL REDIND(IUNIT,SUBINX,LSUBIN,RBASE(ILOCY),
     >                      NGTHER,IREC-4)
              ELSE
                CALL REDIND(IUNIT,SUBINX,LSUBIN,RBASE(ILOCY),
     >                      NGTHER,IREC-5)
              ENDIF
              CALL REDIND(IUNIT,SUBINX,LSUBIN,RBASE(ILOCY+NGROUP),
     >                    NGTHER,IREC-4)
              ITXS=1
              CALL LIBWTE(IACT,ITXS,NGROUP,NGTHER,1,NF,TERP(ITMP),
     >                    SCAT,XSREC(1,IP0),XSREC(1,7),XSREC(1,3),
     >                    XSREC(1,4),XSREC(1,2),
     >                    RBASE(ITMPXS),RBASE(ITMPSC))
              IACT=2
            ENDIF
 140      CONTINUE
        ENDIF
C----
C  BUILT CROSS SECTION FROM INFORMATION IN NG WHICH IS
C  CURRENTLY ABSORPTION AND SCATTERING OUT OF GROUP
C  COMPUTE REAL NG CROSS SECTION WHICH IS
C  CURRENT NG (ABSORPTION)-FISSION+N2N
C  SINCE ABSORPTION IS DEFINED AS
C  TOTAL-SIGS WHERE SIGS CONTAINE 2*N2N SINCE A N2N CONTRIBUTION
C  PRODUCES AN EQUIVALENT OF 2 NEUTRON BY DIFFUSION
C----
        DO 150 IG=1,NGROUP
          XSREC(IG,1)=XSREC(IG,7)+XSREC(IG,IP0)
          IF(NF.GT.1) THEN
            XSREC(IG,7)=XSREC(IG,7)+XSREC(IG,9)-XSREC(IG,4)
          ELSE
            XSREC(IG,7)=XSREC(IG,7)+XSREC(IG,9)
          ENDIF
          IF(XSREC(IG,4).NE.0) THEN
            XSREC(IG,6)=XSREC(IG,3)/XSREC(IG,4)
          ELSE
            XSREC(IG,6)=0
          ENDIF
 150    CONTINUE
        IF(IENDF.LT.2) THEN
C----
C  COMPUTE TRANSPORT CORRECTION AND STORE IN TRAN
C----
          DO 151 IG=1,NGROUP
            XSREC(IG,2)=XSREC(IG,1)-XSREC(IG,2)
 151      CONTINUE
        ENDIF
C----
C  SELF SHIELDING DATA
C----
        NTYP=1
        XSCOR(1)=0.0
        IF(SUBINX(LSUBIN+5).EQ.3) THEN
          NTYP=2
          XSCOR(2)=0.0
        ENDIF
C----
C  MODIFIED SUB IDX LENGTH FOR RESONANCE
C----
        LSUBTR=NGR+7
        LSUBZ=NGR+1
        CALL REDIND(IUNIT,MASTER,LMASIN,SUBINR,LSUBTR,NEL+5)
C----
C  MODIFIED RES IDX LENGTH FOR RESONANCE
C----
        LRESND=SUBINR(NGR+6)
        IF(NTYP.EQ.2.AND.SUBINR(NGR+7).EQ.1) THEN
          NTYP=3
          XSCOR(3)=0.0
        ENDIF
        XSCOR(4)=0.0
        IGRF=NGF
        DO 300 IGR=1,NGR
          IGRF=IGRF+1
          CALL REDIND(IUNIT,SUBINR,LSUBZ,RESINX,LRESND+1,IGR)
          NRES=RESINX(LRESND+1)
          IF(NRES.GT.MAXRES) THEN
            WRITE(IOUT,9005) NRES,MAXRES
            CALL XABORT('LIBWIM: INVALID NUMBER OF RESONANCE')
          ENDIF
          IF(IGR.EQ.1) THEN
            CALL REDIND(IUNIT,RESINX,LRESND,RS1,3*NRES,1)
C----
C  INITIALIZE XSOUT TO INFINITE DILUTION VALUES
C----
            DO 314 IGF=1,NGFR
              XSOUT(IGF,1)=XSREC(IGF,1)-XSREC(IGF,IP0)
              XSOUT(IGF,2)=XSREC(IGF,3)
              XSOUT(IGF,3)=XSREC(IGF,IP0)
              XSOUT(IGF,4)=1.0
              XSOUT(IGF,5)=1.0
 314        CONTINUE
C----
C  IDENTIFY SELF SHIELDING RESONNANT ISOTOPE
C----
            DO 310 JRES=1,NRES
              IF(IDRES.EQ.0) THEN
                XRS1=FLOAT(INT((RS1(1,JRES)+0.01)*10.)
     >              -INT(RS1(1,JRES)+0.01)*10)/10.+0.02
                XRS1=ABS(RS1(1,JRES)-XRS1-RIND)
              ELSE
                XRS1=ABS(RS1(1,JRES)-RIND)
              ENDIF
              IF(XRS1.LE.0.01) THEN
                KRES=JRES
                NTMPR=INT(RS1(2,KRES)+0.1)
                NDILR=INT(RS1(3,KRES)+0.1)
                IF(NTMPR.GT.MAXTEM) THEN
                  WRITE(IOUT,9006) NTMPR,MAXTEM
                  CALL XABORT('LIBWIM: INVALID NUMBER OF RES TEMP')
                ELSE IF(NDILR.GT.MAXTEM) THEN
                  WRITE(IOUT,9007) NDILR,MAXTEM
                  CALL XABORT('LIBWIM: INVALID NUMBER OF RES DIL')
                ENDIF
                NTD=NDILR*NTMPR
                IF(IPRINT.GE.5) THEN
                  WRITE(IOUT,6002) RS1(1,JRES)
                ENDIF
                CALL REDIND(IUNIT,RESINX,LRESND,TMPT,NTMPR,
     >                      2+5*(KRES-1))
                CALL REDIND(IUNIT,RESINX,LRESND,DILT,NDILR,
     >                      3+5*(KRES-1))
                IF(IPRINT.GE.100) THEN
                  WRITE(IOUT,6003) TN(JSO)
                  WRITE(IOUT,6008) (TMPT(ITT),ITT=1,NTMPR)
                  WRITE(IOUT,6004) SN(IGRF,JSO),DSIGPL
                  WRITE(IOUT,6008) (DILT(ITT),ITT=1,NDILR)
                ENDIF
                DO 312 II=1,NTMPR
                  TMPT(II)=SQRT(TMPT(II))
 312            CONTINUE
                DO 313 II=1,NDILR
                  IF(DILT(II)-DSIGPL.GT.0.0) THEN
                    DILT(II)=SQRT(DILT(II)-DSIGPL)
                  ELSE
                    DILT(II)=0.0
                  ENDIF
 313            CONTINUE
                GO TO 311
              ENDIF
 310        CONTINUE
C----
C  NO SELF SHIELDING DATA FOR THIS ISOTOPE EXIT TO 301
C----
            CALL XDRSET(XSREC(1,16),NGROUP,0.0)
            GO TO 301
          ENDIF
C----
C  READ SELF SHIELDING DATA FOR THIS ISOTOPE
C----
 311      CONTINUE
C----
C  READ FLUX FOR THIS RESONANCE INTEGRAL
C----
          IF(IENDF.GE.2) THEN
C----
C  READ TOTAL RR AND FLUX
C----
            CALL REDIND(IUNIT,RESINX,LRESND,RBASE(JRRI),2*NTD,
     >                  4+5*(KRES-1))
            CALL LIBWRI(NGR,NTMPR,NDILR,IGRF,TN(JSO),SN(IGRF,JSO),
     >                  TMPT,DILT,RBASE(JRRI),RBASE(JRIT),
     >                  XSOUT(IGRF,1),XSCOR(1))
            CALL LIBWRI(NGR,NTMPR,NDILR,IGRF,TN(JSO),SN(IGRF,JSO),
     >                  TMPT,DILT,RBASE(JRRI+NTD),RBASE(JRIT),
     >                  XSOUT(IGRF,4),XSCOR(4))
          ELSE
C----
C  READ TOTAL RR
C----
            CALL REDIND(IUNIT,RESINX,LRESND,RBASE(JRRI),NTD,
     >                  4+5*(KRES-1))
            CALL LIBWRI(NGR,NTMPR,NDILR,IGRF,TN(JSO),SN(IGRF,JSO),
     >                  TMPT,DILT,RBASE(JRRI),RBASE(JRIT),
     >                  XSOUT(IGRF,1),XSCOR(1))
            IF(IPRINT.GE.100) THEN
              WRITE(IOUT,6005) XSOUT(IGRF,1)
              WRITE(IOUT,6008) (RBASE(ITT),ITT=JRRI,JRRI+NTD-1)
            ENDIF
          ENDIF
          IF(NTYP.GE.2) THEN
C----
C  READ FISSION RR
C----
            CALL REDIND(IUNIT,RESINX,LRESND,RBASE(JRRI),NTD,
     >                  5+5*(KRES-1))
            CALL LIBWRI(NGR,NTMPR,NDILR,IGRF,TN(JSO),SN(IGRF,JSO),
     >                  TMPT,DILT,RBASE(JRRI),RBASE(JRIT),
     >                  XSOUT(IGRF,2),XSCOR(2))
            IF(IPRINT.GE.100) THEN
              WRITE(IOUT,6006) XSOUT(IGRF,2)
              WRITE(IOUT,6008) (RBASE(ITT),ITT=JRRI,JRRI+NTD-1)
            ENDIF
            IF(NTYP.GE.3) THEN
C----
C  READ SCATTERING RR
C----
              CALL REDIND(IUNIT,RESINX,LRESND,RBASE(JRRI),NTD,
     >                    6+5*(KRES-1))
              IRRICS=JRRI
              ASIGPL=0.0
              DO 340 ILL=1,NTD
                ASIGPL=ASIGPL+RBASE(IRRICS)
                IRRICS=IRRICS+1
 340          CONTINUE
              IF(ASIGPL.GT.0.0) THEN
                CALL LIBWRI(NGR,NTMPR,NDILR,IGRF,TN(JSO),SN(IGRF,JSO),
     >                      TMPT,DILT,RBASE(JRRI),RBASE(JRIT),
     >                      XSOUT(IGRF,3),XSCOR(3))
                IF(IPRINT.GE.100) THEN
                  WRITE(IOUT,6007) XSOUT(IGRF,3),XSCOR(3)
                  WRITE(IOUT,6008) (RBASE(ITT),ITT=JRRI,JRRI+NTD-1)
                ENDIF
              ENDIF
             ENDIF
          ENDIF
 300    CONTINUE
C----
C  CORRECT CROSS SECTIONS FOR CURRENT GROUP
C----
        IGRL=IGRF
        IGRF=NGF+1
        CALL LIBWRE(NTYP,ITLIB,NGROUP,IGRF,IGRL,SCAT,XSREC(1,IP0),
     >              XSREC(1,1),XSREC(1,7),XSREC(1,3),XSREC(1,4),
     >              XSREC(1,6),DELTA,SN(1,JSO),SB(1,JSO),
     >              XSOUT,XSCOR)
C----
C  PRINT CROSS SECTIONS IF REQUIRED
C----
        IF(IPRINT.GE.5) THEN
          WRITE(IOUT,6100)
          DO 400 IG1=NGF+1,NGFR
            WRITE(IOUT,6101) IG1,SN(IG1,JSO),SB(IG1,JSO),
     >                       XSOUT(IG1,4),XSREC(IG1,1),
     >                       XSREC(IG1,IP0),XSREC(IG1,3),
     >                       XSREC(IG1,14),XSREC(IG1,7)
 400      CONTINUE
        ENDIF
C----
C  SET NWT0 THE RESONANCE FLUX WEIGHTING
C----
        CALL XDRSET(XSREC(1,16),NGROUP,1.0)
        DO 401 IG1=NGF+1,NGFR
          XSREC(IG1,16)=XSOUT(IG1,4)
 401    CONTINUE
 301    CONTINUE
C----
C  P1 SCATTERING
C----
        IF(NL.EQ.2) THEN
          IREC=16+NTMP*5
          NTMP=SUBINX(LSUBIN+10)
          IF(NTMP+1.GT.MAXTEM) THEN
            CALL XABORT('LIBWIM: INVALID MAXTEM FOR P1.')
          ELSE IF(NTMP.GT.0) THEN
            CALL REDIND(IUNIT,SUBINX,LSUBIN,NXS,NTMP+1,7)
            CALL REDIND(IUNIT,SUBINX,LSUBIN,TMPT,NTMP,5)
            NSCT=NXS(NTMP+1)
            IF(NSCT.GT.MXSCT) THEN
              WRITE(IOUT,9004) NSCT,MXSCT
              CALL XABORT('LIBWIM: INVALID SCATTERING MATRIX -E/1')
            ENDIF
            CALL REDIND(IUNIT,SUBINX,LSUBIN,XSSCMP,NSCT,IREC)
            IREC=IREC+1
C----
C  DECOMPRESS P1 SCATTERING CROSS SECTIONS AND COMPUTE
C  P1 SCATTERING OUT OF GROUP
C----
            CALL LIBWSC(NGROUP,1,NGFR,NSCT,XSSCMP,
     >                  SCAT(1,1,2),XSREC(1,IP1))
          ENDIF
          IF(NTMP.EQ.1) THEN
            IF(NXS(1).GT.MXSCT) THEN
              WRITE(IOUT,9004) NXS(1),MXSCT
              CALL XABORT('LIBWIM: INVALID SCATTERING MATRIX -T/1')
            ENDIF
            CALL REDIND(IUNIT,SUBINX,LSUBIN,XSSCMP,NXS(1),IREC)
            IREC=IREC+1
C----
C  DECOMPRESS P1 SCATTERING CROSS SECTIONS AND COMPUTE
C  P1 SCATTERING OUT OF GROUP
C----
            CALL LIBWSC(NGROUP,NGFR+1,NGROUP,NXS(1),XSSCMP,
     >                  SCAT(1,1,2),XSREC(1,IP1))
          ELSE IF(NTMP.GT.1) THEN
C----
C  AVALUATE LAGRANGIAN INTERPOLATION FACTOR FOR
C  AVAILABLE TMPTERATURES (ORDER NOTX) AND INTERPOLATE.
C----
            CALL LIBLEX(NTMP,TN(JSO),TMPT,NOTX,TERP)
            NRDT=NGTHER-1
            IACT=1
            DO 170 ITMP=1,NTMP
              IF(TERP(ITMP).EQ.0.0D0) THEN
                IREC=IREC+1
              ELSE
                IF(NXS(ITMP).GT.MXSCT) THEN
                  WRITE(IOUT,9004) NXS(ITMP),MXSCT
                  CALL XABORT('LIBWIM: INVALID SCATTERING MATRIX -T/1')
                ENDIF
                CALL REDIND(IUNIT,SUBINX,LSUBIN,XSSCMP,NXS(ITMP),IREC)
                IREC=IREC+1
C----
C  DECOMPRESS P1 SCATTERING CROSS SECTIONS AND COMPUTE
C  P1 SCATTERING OUT OF GROUP
C----
                CALL LIBWSC(NGROUP,NGFR+1,NGROUP,NXS(ITMP),XSSCMP,
     >                      RBASE(ILOCS),RBASE(ILOCX+4*NGROUP))
                ITXS=2
                CALL LIBWTE(IACT,ITXS,NGROUP,NGTHER,1,NF,TERP(ITMP),
     >                      SCAT(1,1,2),XSREC(1,IP1),XSREC(1,7),
     >                      XSREC(1,3),XSREC(1,4),XSREC(1,2),
     >                      RBASE(ITMPXS),RBASE(ITMPSC))
                IACT=2
              ENDIF
 170        CONTINUE
          ENDIF
          IF(ITRANC.EQ.1) THEN
            IF(NTMP.GT.0) THEN
               DO 180 IG1=1,NGROUP
                XSREC(IG1,2)=XSREC(IG1,IP1)
 180          CONTINUE
            ELSE
              CALL XDRSET(SCAT(1,1,2),NGROUP*NGROUP,0.0)
              DO 181 IG1=1,NGROUP
                XSREC(IG1,IP1)=XSREC(IG1,2)
                SCAT(IG1,IG1,2)=XSREC(IG1,2)
 181          CONTINUE
            ENDIF
          ELSE IF(ITRANC.EQ.3) THEN
            IF(NTMP.GT.0) THEN
              DO 190 IG1=1,NGFR
                XSREC(IG1,2)=0.0
                DO 191 IG2=1,NGROUP
                  XSREC(IG1,2)=XSREC(IG1,2)+SCAT(IG1,IG2,2)*
     >                      DELTA(IG2)/DELTA(IG1)
 191            CONTINUE
 190          CONTINUE
              DO 192 IG1=NGFR+1,NGROUP
                XSREC(IG1,2)=XSREC(IG1,IP1)
 192          CONTINUE
            ELSE
              CALL XDRSET(SCAT(1,1,2),NGROUP*NGROUP,0.0)
              DO 193 IG1=1,NGROUP
                XSREC(IG1,IP1)=XSREC(IG1,2)
                SCAT(IG1,IG1,2)=XSREC(IG1,2)
 193          CONTINUE
            ENDIF
          ENDIF
        ENDIF
C----
C  SAVE MAIN CROSS SECTIONS ON LCM
C----
        CALL LCMSIX(IPLIB,HNAMIS,1)
        IF(NF.LE.1) THEN
          INDPRO(5)=0
          CALL XDRLGS(IPLIB,1,NPROC,0,NL-1,1,NGROUP,INDPRO,
     >                ITYPRO,XSREC,SCAT,XSSCMP,NJJ,IJJ)
          INDPRO(5)=1
        ELSE
          CALL XDRLGS(IPLIB,1,NPROC,0,NL-1,1,NGROUP,INDPRO,
     >                ITYPRO,XSREC,SCAT,XSSCMP,NJJ,IJJ)
        ENDIF
        IF(IPRINT.GE.100) THEN
          CALL XDRLPR(NPROC,0,NL-1,1,NGROUP,INDPRO,ITYPRO,
     >                XSREC,SCAT)
        ENDIF
        WRITE(README(:8),'(A8)') HNAMIS(1:8)
        READ(README,'(24A4)') (IHGAR(I),I=1,24)
        CALL LCMPUT(IPLIB,'README',24,3,IHGAR)
        IF(IPRINT.GE.100) CALL LCMLIB(IPLIB)
        CALL LCMSIX(IPLIB,' ',2)
 115  CONTINUE
 110  CONTINUE
      CALL RLSARA(RBASE(JRIT))
      CALL RLSARA(RBASE(JRRI))
      CALL RLSARA(RBASE(ITMPSC))
      CALL RLSARA(RBASE(ITMPXS))
      CALL CLSIND(IUNIT)
C----
C  RETURN
C----
      RETURN
C----
C  FORMAT
C----
 9000 FORMAT(/' ERROR IN PROCESSING WIMS LIBRARY:',A8)
 9001 FORMAT(/'  NUMBER OF GROUPS SPECIFIED :',I10/
     >        ' NUMBER OF GROUPS IN LIBRARY :',I10)
 9002 FORMAT(/' LIBWIM: MATERIAL/ISOTOPE ',A12,' IS MISSING ON WIMS',
     >        ' FILE NAME ',A8)
 9003 FORMAT(/' MAXIMUM NUMBER OF ISOTOPE SPECIFIED :',I10/
     >        '        NUMBER OF ISOTOPE IN LIBRARY :',I10)
 9004 FORMAT(/'        DIMENSION OF SCATTERING MATRIX  :',I10/
     >        ' MAXIMUM DIMENSION OF SCATTERING MATRIX :',I10)
 9005 FORMAT(/'         NUMBER OF RESONANT ISOTOPES :',I10/
     >        ' MAXIMUM NUMBER OF RESONANT ISOTOPES :',I10)
 9006 FORMAT(/'         NUMBER OF RESONANT TEMPERATURE :',I10/
     >        ' MAXIMUM NUMBER OF RESONANT TEMPERATURE :',I10)
 9007 FORMAT(/'         NUMBER OF RESONANT DILUTION :',I10/
     >        ' MAXIMUM NUMBER OF RESONANT DILUTION :',I10)
 6000 FORMAT(' READING WIMS-AECL LIBRARY NAME ',A8)
 6001 FORMAT('   PROCESSING ISOTOPE/MATERIAL = ',A12)
 6002 FORMAT('   SELF SHIELDING ISOTOPE      = ',F9.3)
 6003 FORMAT('   RESONANCE TEMPERATURE       = ',1P,E15.7)
 6004 FORMAT('   RESONANCE DILUTIONS         = ',1P,2E15.7)
 6005 FORMAT('   ABSORPTION RATE             = ',1P,E15.7)
 6006 FORMAT('   FISSION RATE                = ',1P,E15.7)
 6007 FORMAT('   SCATTERING RATE             = ',1P,E15.7)
 6008 FORMAT(1P,5E15.7)
 6100 FORMAT(/5X,'GROUP',10X,'DILUT',13X,'SB',11X,'NPHI',10X,'TOTAL',
     > 11X,'SIGS',9X,'NUSIGF',11X,'GOLD',13X,'NG')
 6101 FORMAT(5X,I5,1P,8E15.5)
 6200 FORMAT(' TEMPERATURE = ',F10.5,10X,
     >  ' CROSS SECTION TABULATED AT A SINGLE TEMPERATURE')
 6201 FORMAT(' TEMPERATURE = ',F10.5,10X,
     >  ' CROSS SECTION TABULATED AT MULTIPLE TEMPERATURES')
 6202 FORMAT(' TABULATION TEMPERATURES= ',/(5F15.5))
 6203 FORMAT(' INTERPOLATION FACTORS  = ',1P,/(5E15.5))
      END
