*DECK MOCDRV
      SUBROUTINE MOCDRV(IPFLUX,IPTRK,IPMACR,IFTRAK,NGRP,NMAT,NIFIS,
     > NANIS,NL,NBN,NUN,FLUXO,DIFFB1,
     > NINTMX,AIEPS,NEXTMX,AEEPS,IFRITR,IACITR,ITYPEC,ILEAK,
     > NREGIO,MATCOD,VOL,VOLINV,
     > XSOLD,XSSIGT,XTINV,XSSIGW,XSCHI,XSNUF,XSTK,
     > NJJ,IJJ,IPOS,XSCAT,VSCAT,NALB,ALBEDO,
     > LEAKSW,ILK,ITRANC,ITRB,TITRE,LCACA,NGSS,LEXIN,REFKEF,
     > IPRT,CDIRFL)
C
C---------------------------  MOCDRV  ---------------------------------
C
C 1- PROGRAMME STATISTICS:
C
C     NAME     :  MOCDRV
C     USE      :  DRIVER FOR CHARACTERISTICS SOLVER
C     AUTHOR   :  R. ROY (22/01/97)
C
C 2- ROUTINE PARAMETERS:
C   INPUT
C     IPFLUX   : POINTER TO THE FLUX                     I
C     IPTRK    : POINTER TO THE TRACKING                 I
C     IPMACR   : POINTER TO THE MACROLIB                 I
C     IFTRAK   : FILE UNIT NUMBER FOR TRACKS.            I
C     NGRP     : NUMBER OF GROUPS TREATED                I
C     NMAT     : NUMBER OF MIXTURES CONSIDERED           I
C     NREGIO   : NUMBER OF REGIONS CONSIDERED            I
C     MATCOD   : MIXTURE CODE IN REGION                  I(NREGIO)
C     VOL      : VOLUME OF REGION                        R(NREGIO)
C     VOLINV   : INVERSE OF VOLUMES                      R(NREGIO)
C     LEAKSW   : LEAKAGE SWITCH                          L
C     ITRANC   : TYPE OF TRANSPORT CORRECTIONS           I
C     TITRE    : EXECUTION TITLE                         C*72
C     REFKEF   : KEFF VALUE FOR B2 SEARCH                D
C     IPRT     : PRINT OPTION FOR CHARACTERISTIC SOLVER  I
C     CDIRFL   : FLUX DIRECTORY NAME                     C*12
C   WORK VECTOR
C     XSSIGT   : GROUP TOTAL CROSS SECTION               R(NMAT+1)
C     XSSIGW   : WITHIN GROUP SCATTERING                 R(NMAT+1)
C
C 3- INTERNAL PARAMETERS:
C     IOUT     : OUTPUT UNIT = 6                         I
C     ILCMUP   : GO UP LCM INDEX = 1                     I
C     ILCMDN   : GO DOWN LCM INDEX = 2                   I
C
C---------------------------   MOCDRV  --------------------------------
C
      IMPLICIT    NONE
      INTEGER     IOUT,ILCMUP,ILCMDN,NDIMO,MXGAUS
      INTEGER     IPFLUX,IPTRK,IPMACR,IFTRAK,NGRP,NMAT,NIFIS,
     >            NANIS,NL,NBN,NUN,NINTMX,NEXTMX,IFRITR,IACITR,
     >            ITYPEC,ILEAK,NREGIO,ITRANC,ITRB,LCACA,NGSS,IPRT
      REAL        AIEPS,AEEPS
      REAL        ZERO,ONE,EPSR
      PARAMETER ( IOUT=6,ILCMUP=1,ILCMDN=2 )
      PARAMETER ( NDIMO=2, MXGAUS=64, EPSR=1.E-6 )
      PARAMETER ( ZERO=0., ONE=1. )
      LOGICAL     LEAKSW,ILK,LEXIN,NOVOID
      CHARACTER   TITRE*72,CDIRFL*12,CGRPNM*12
      INTEGER     IGR,IS,ILNLCM,NS,NR,NFUNL,NRNL,NSCRP,JUN,NALB,
     >            IL,NMOD,MATMAX,MATMIN,JGSS,NPHI,IANI,IFIS,JGR,
     >            NCOMNT,IXSS,ITYLCM,IREGIO,IR,IMAT,NTRK,NCOR,
     >            NALBG,NANGL,NUNKNO,NDIM,IC,ISPEC,NSOUT,NREG,MXSEG
      REAL        ASCRP,RKEFF,RB2
      INTEGER     MATCOD(*),NJJ(*),IJJ(*),IPOS(*)
      REAL        VOL(*),VOLINV(*),
     >            XSSIGT(0:NMAT,NGRP),XSSIGW(0:NMAT,NGRP),
     >            XSOLD(0:NMAT,NGRP),XTINV(0:NMAT,NGRP),
     >            XSCAT(NGRP,0:NMAT,0:NANIS,NGRP),VSCAT(*),ALBEDO(NALB),
     >            XSNUF(0:NMAT,NIFIS,NGRP),XSCHI(0:NMAT,NIFIS,NGRP),
     >            XSTK(NMAT,NIFIS),DIFFB1(NGRP)
      REAL        XGSS(MXGAUS), WGSS(MXGAUS), YGSS(MXGAUS)
      REAL        FLUXO(NUN,NGRP)
      INTEGER     VOLSUR, MATALB, PHIANG, PHIDUM, SIGANG,
     >            FLUX  , NUMERO, LENGHT,
     >            EXPT  , EXP2  , ICYCLE, FLM   , FLP   ,
     >            ICYM  , ICYP  , IALPHA, DFLUX , DWEIG ,
     >            FMEAN , ISGNR , PNL   ,
     >            RHARM , KEYANI, TRHAR , XCDIA , DIFF  ,
     >            REBAL , FLUC  , XCSOU,  XCTOT , XCNUF , XCTRC , FLUN,
     >            XCCHI , XCABS , XCCAT , DB2   , IBETA
      CHARACTER*2 CAN(0:9)
      CHARACTER   CINI*4, COMENT*80
      DOUBLE PRECISION REFKEF
      REAL        A
      COMMON      A(1)
      INTEGER     I(1)
      EQUIVALENCE (A(1),I(1))
      DATA CAN /' 0',' 1',' 2',' 3',' 4',' 5',' 6',' 7',' 8',' 9'/
      DATA CINI/'$TRK'/
C
      CALL LCMSIX(IPFLUX,CDIRFL,ILCMUP)
      IF(IPRT.GE.1) THEN
        WRITE(IOUT,6200)
        IF(ITRANC.GT.0) THEN
          WRITE(IOUT,6100)
        ENDIF
      ENDIF
      IF(IPRT.GE.2) THEN
        WRITE(IOUT,6000)
        WRITE(IOUT,6001) (IREGIO,VOL(IREGIO),MATCOD(IREGIO),
     >                    IREGIO=1,NREGIO)
      ENDIF
      DO 150 IGR=1,NGRP
        WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGRP
        CALL LCMSIX(IPMACR,CGRPNM,ILCMUP)
        CALL LCMLEN(IPMACR,'TOTAL',ILNLCM,ITYLCM)
        IF(ILNLCM.GT.0) THEN
          CALL LCMGET(IPMACR,'TOTAL',XSSIGT(1,IGR))
          XSSIGT(0,IGR)=ZERO
          DO 111 IR= 0, NMAT
 111      XSOLD(IR,IGR)=XSSIGT(IR,IGR)
        ELSE
          CALL LCMLIB(IPMACR)
          CALL XABORT('MOCDRV: READ ERROR ON LCM RECORD= TOTAL')
        ENDIF
        CALL LCMLEN(IPMACR,'SIGW 0',ILNLCM,ITYLCM)
        IF(ILNLCM.GT.0) THEN
          CALL LCMGET(IPMACR,'SIGW 0',XSSIGW(1,IGR))
          XSSIGW(0,IGR)=ZERO
        ELSE
          DO 110 IR=0,NMAT
 110      XSSIGW(IR,IGR)=ZERO
        ENDIF
        IF(ITRANC.NE.0) THEN
*         TRANSPORT CORRECTION.
          CALL LCMLEN(IPMACR,'TRANC',ILNLCM,ITYLCM)
          IF(ILNLCM.GT.0) THEN
            CALL LCMGET(IPMACR,'TRANC',VSCAT)
            DO 120 IMAT=1,NMAT
              XSSIGT(IMAT,IGR)=XSSIGT(IMAT,IGR)-VSCAT(IMAT)
              XSSIGW(IMAT,IGR)=XSSIGW(IMAT,IGR)-VSCAT(IMAT)
 120        CONTINUE
          ENDIF
        ENDIF
        XTINV(0,IGR)= ONE
        DO 121 IMAT=1,NMAT
           IF( XSSIGT(IMAT,IGR).NE.ZERO )THEN
              XTINV(IMAT,IGR)= ONE/XSSIGT(IMAT,IGR)
           ELSE
              XTINV(IMAT,IGR)= ONE
           ENDIF
 121    CONTINUE
        IF(IPRT.GE.3) THEN
          WRITE(IOUT,6002)  IGR
          WRITE(IOUT,6003) (IMAT,XSSIGT(IMAT,IGR),
     >                             XSSIGW(IMAT,IGR),IMAT=1,NMAT)
        ENDIF
        IF( ITYPEC .EQ. 6 ) THEN
           IF( NIFIS.NE.1 )THEN
              CALL XABORT('MOCDRV: FOR FIXED SOURCE PROBLEM, NIFIS = 1')
           ENDIF
           XSNUF(0,1,IGR)= ZERO
           CALL LCMGET(IPMACR,'FIXE',XSNUF(1,1,IGR))
        ELSE
           CALL LCMGET(IPMACR,'CHI',XSTK)
           DO 132 IFIS= 1, NIFIS
              XSCHI(0,IFIS,IGR)= ZERO
              DO 131 IMAT= 1, NMAT
                 XSCHI(IMAT,IFIS,IGR)= XSTK(IMAT,IFIS)
  131         CONTINUE
  132      CONTINUE
           CALL LCMGET(IPMACR,'NUSIGF',XSTK)
           DO 134 IFIS= 1, NIFIS
              XSNUF(0,IFIS,IGR)= ZERO
              DO 133 IMAT= 1, NMAT
                 XSNUF(IMAT,IFIS,IGR)= XSTK(IMAT,IFIS)
  133         CONTINUE
  134      CONTINUE
           IF( IPRT.GT.3 )THEN
              WRITE(IOUT,6004) (IMAT,XSNUF(IMAT,1,IGR),
     >                                 XSCHI(IMAT,1,IGR),IMAT=1,NMAT)
           ENDIF
        ENDIF
        DO 50 IANI=0,NANIS
          CALL LCMLEN(IPMACR,'NJJD'//CAN(IANI),ILNLCM,ITYLCM)
          DO 39 IMAT= 0, NMAT
            DO 38 JGR= 1, NGRP
              XSCAT(JGR,IMAT,IANI,IGR)= ZERO
   38       CONTINUE
   39     CONTINUE
          IF(ILNLCM.GT.0) THEN
            CALL LCMGET(IPMACR,'NJJD'//CAN(IANI),NJJ)
            CALL LCMGET(IPMACR,'IJJD'//CAN(IANI),IJJ)
            CALL LCMGET(IPMACR,'IPOD'//CAN(IANI),IPOS)
            CALL LCMGET(IPMACR,'SCAD'//CAN(IANI),VSCAT)
            DO 45 IMAT= 1, NMAT
              DO 40 JGR= IJJ(IMAT)-NJJ(IMAT)+1, IJJ(IMAT)
                IXSS= IPOS(IMAT)+IJJ(IMAT)-JGR
                XSCAT(JGR,IMAT,IANI,IGR)= VSCAT(IXSS)
   40         CONTINUE
   45       CONTINUE
          ENDIF
          IF( IPRT.GT.10 )THEN
             DO 49 IMAT= 1, NMAT
                WRITE(IOUT,7000) IMAT,IANI,IGR,
     >                        (XSCAT(JGR,IMAT,IANI,IGR),JGR=1,NGRP)
   49        CONTINUE
          ENDIF
   50   CONTINUE
        CALL LCMSIX(IPMACR,' ',ILCMDN)
 150  CONTINUE
C
C     READ HEADERS OF TRACKING FILE
      REWIND IFTRAK
      READ(IFTRAK) CINI,NCOMNT,NTRK
      IF( CINI   .NE. '$TRK' .OR.
     >    NCOMNT .LT. 0      .OR.
     >    NTRK   .EQ. 0          ) THEN
        WRITE(IOUT,9000) 'MOCDRV'
        CALL XABORT('MOCDRV: PROBLEM WITH THE TRACKING FILE')
      ENDIF
      DO 30 IC= 1,NCOMNT
         READ(IFTRAK) COMENT
   30 CONTINUE
      READ(IFTRAK) NDIM,ISPEC,NREG,NSOUT,NALBG,NCOR,NANGL,MXSEG
      IF( NREG.NE.NREGIO )THEN
         CALL XABORT('MOCDRV: INVALID NUMBER OF REGIONS')
      ENDIF
      IF( NALB.NE.NALBG )THEN
         CALL XABORT('MOCDRV: INVALID NUMBER OF ALBEDOS')
      ENDIF
      IF( NDIM.NE.NDIMO )THEN
         CALL XABORT('MOCDRV: NON 2D TRACKING FILE')
      ENDIF
      IF( ISPEC.NE.1 )THEN
         CALL XABORT('MOCDRV: NON SPECULAR TRACKING FILE')
      ENDIF
      NUNKNO= NREG+NSOUT+1
      NPHI  = NANGL
      CALL SETARA( A, NUNKNO, VOLSUR )
      CALL SETARA( I, NUNKNO, MATALB )
      CALL SETARA( A, NPHI  , PHIANG )
      CALL SETARA( A, NPHI  , PHIDUM )
      READ(IFTRAK) (A(VOLSUR+JUN),JUN=0,NUNKNO-1)
      READ(IFTRAK) (I(MATALB+JUN),JUN=0,NUNKNO-1)
      READ(IFTRAK) ( NSCRP,JUN=1,NALBG)
      READ(IFTRAK) ( ASCRP,JUN=1,NALBG)
      READ(IFTRAK) ( A(PHIANG+IL-1),A(PHIDUM+IL-1),IL=1,NANGL)
      READ(IFTRAK) ( ASCRP,JUN=1,NANGL)
      CALL RLSARA( A(PHIDUM) )
      NR = NREG
      NS = NSOUT
      NRNL=   (NL+1)**2
      NFUNL= ((NL+1)*(NL+2))/2
C      IF( NFUNL*NREG.NE.NUN )THEN
      IF( NFUNL*NREG.GT.NUN )THEN
         WRITE(IOUT,7001) NL,NFUNL,NREG,NUN
         CALL XABORT(' MOCDRV: INCOMPATIBLE DIMENSION FOR FLUX')
      ENDIF
C
C     CHECK COMPATIBLITY OF MATCOD
      MATMIN= 0
      MATMAX= 0
      NOVOID= .TRUE.
      DO 20 IS=-NS,NR
         IF( IS.LT.0 )THEN
            IF( I(MATALB+NSOUT+IS).LT.0 )THEN
               MATMIN= MAX(MATMIN,I(MATALB+NSOUT+IS))
               ILK = ILK.AND.ALBEDO(-I(MATALB+NSOUT+IS)).EQ.ONE
            ELSE
               CALL XABORT('MOCDRV: IS THIS A PROBLEM?')
            ENDIF
         ELSEIF( IS.GT.0 )THEN
            MATMAX= MAX(MATMAX,I(MATALB+NSOUT+IS))
            IF( I(MATALB+NSOUT+IS).NE.MATCOD(IS) )THEN
               WRITE(IOUT,7002) I(MATALB+NSOUT+IS),MATCOD(IS)
               I(MATALB+NSOUT+IS)= MATCOD(IS)
            ENDIF
            IF( ABS(VOL(IS)-A(VOLSUR+NSOUT+IS))*VOLINV(IS).GT.EPSR )THEN
               CALL XABORT('MOCDRV: INCOMPATIBLE VOLS')
            ENDIF
            DO 25 IGR=1,NGRP
               NOVOID= NOVOID.AND.(XSSIGT(MATCOD(IS),IGR).NE.ZERO)
   25       CONTINUE
         ELSE
            IF(  I(MATALB+NSOUT+IS).NE.0 )
     >         CALL XABORT('MOCDRV: MATALB(0).NE.0 ?')
         ENDIF
   20 CONTINUE
      IF( IPRT.GT.0 )THEN
         IF( NL.EQ.0 )THEN
            IF( NOVOID )THEN
               WRITE(IOUT,7010) 'MOCC  '
            ELSE
               WRITE(IOUT,7011) 'MOCC  '
            ENDIF
         ELSE
            IF( NOVOID )THEN
               WRITE(IOUT,7012) 'MOCC  '
            ELSE
               WRITE(IOUT,7013) 'MOCC  '
            ENDIF
         ENDIF
         IF( ILK )THEN
            WRITE(IOUT,7020) 'MOCC  '
            IF( LEAKSW )THEN
               WRITE(IOUT,7021) 'MOCC  '
               CALL XABORT('MOCDRV: ERROR ON LEAKAGE SWITCH')
            ENDIF
         ELSE
            WRITE(IOUT,7022) 'MOCC  '
            IF( .NOT.LEAKSW )THEN
               WRITE(IOUT,7021) 'MOCC  '
            ENDIF
         ENDIF
      ENDIF
      IF( MATMIN.LT.-NALB )THEN
         CALL XABORT('MOCDRV: NOT ENOUGH ALBEDOS   ARE DEFINED')
      ENDIF
      IF( MATMAX.GT.NMAT )THEN
         CALL XABORT('MOCDRV: NOT ENOUGH MATERIALS ARE DEFINED')
      ENDIF
      IF( LCACA.EQ.0 )THEN
         CALL ALGPT ( NGSS, ZERO, ONE, XGSS, WGSS)
      ELSE
         CALL ALCACT( LCACA, NGSS, XGSS, WGSS)
      ENDIF
      IF( IPRT.GT.0 )THEN
        WRITE(IOUT,7030) 'MOCDRV',LCACA,NGSS
      ENDIF      
      DO 109 JGSS= 1, NGSS
         YGSS(JGSS)= ONE/SQRT(ONE - XGSS(JGSS)*XGSS(JGSS))
         WGSS(JGSS)= -WGSS(JGSS)*SQRT(ONE - XGSS(JGSS)*XGSS(JGSS))
  109 CONTINUE
*
      CALL SETARA( A, 8*NGRP*NR*NFUNL, FLUX )
      CALL SETARA( I, MXSEG , NUMERO )
      CALL SETARA( A, MXSEG , LENGHT )
      CALL SETARA( A, MXSEG*NGSS , EXPT )
      CALL SETARA( A, MXSEG*NGSS , EXP2 )
      CALL SETARA( A,       NGSS , ICYCLE )
      CALL SETARA( A, MXSEG*NGSS , FLM )
      CALL SETARA( A, MXSEG*NGSS , FLP )
      CALL SETARA( A, MXSEG*NGSS , ICYM )
      CALL SETARA( A, MXSEG*NGSS , ICYP )
      CALL SETARA( A,       NGSS , IALPHA )
      CALL SETARA( A,       NGSS , DFLUX )
      CALL SETARA( A,       NGSS , DWEIG )
      CALL SETARA( A,       NGSS , FMEAN )
      CALL SETARA( I,         8*NRNL , ISGNR )
      CALL SETARA( A,      NGSS*NRNL , RHARM )
      CALL SETARA( I,           NRNL , KEYANI )
*
      CALL MOCIK3(NDIM,NL,I(ISGNR),I(KEYANI))
      IF(NDIM.EQ.2) THEN
          NMOD = 4
      ENDIF
      CALL SETARA( A, NFUNL*NMOD*NGSS , TRHAR )
      CALL SETARA( A, 2*NGRP*(NGRP+1) , REBAL )
      CALL SETARA( A, 2*NGRP, FLUC)
      CALL SETARA( A, 2*NGRP, FLUN)
      CALL SETARA( A, 2*NGRP, PNL)
      CALL SETARA( A, 2*NGRP, XCSOU)
      CALL SETARA( A, 2*NGRP, XCTOT)
      CALL SETARA( A, 2*NGRP, XCTRC)
      CALL SETARA( A, 2*NGRP, XCDIA)
      CALL SETARA( A, 2*NGRP, XCNUF)
      CALL SETARA( A, 2*NGRP, XCCHI)
      CALL SETARA( A, 2*NGRP, XCABS)
      CALL SETARA( A, 2*NGRP*NGRP*(NANIS+1) , XCCAT)
      CALL SETARA( A, 2*NGRP, DIFF)
      CALL SETARA( A, 2*NGRP, DB2)
      CALL SETARA( A, 2*NGRP, IBETA)
      CALL SETARA( A, NGSS*NGRP*(NMAT+NALB+1), SIGANG )
*
      CALL MOC2DR ( IPFLUX,
     >              IFTRAK, NGRP,NREG,NSOUT,NANIS,NMAT,NGSS,MXSEG,NTRK,
     >              NPHI, NFUNL, NMOD, I(KEYANI),
     >              I(MATALB),VOL,VOLINV,WGSS,XGSS,YGSS,A(PHIANG),
     >              A(LENGHT),I(NUMERO),A(EXPT),A(EXP2),
     >              A(ICYCLE),A(FLM),A(FLP),A(ICYM),A(ICYP),
     >              A(IALPHA),A(DFLUX),A(DWEIG),A(FMEAN),
     >              A(SIGANG),NALB,ALBEDO,
     >              XSOLD,XSSIGT,XTINV,XSSIGW,XSCAT,
     >              NIFIS,XSNUF,XSCHI,ITRB,
     >              A(FLUX),FLUXO,DIFFB1,
     >              I(ISGNR),A(RHARM),LEAKSW,ILK,
     >              NINTMX,AIEPS,NEXTMX,AEEPS,
     >              IPRT,ITYPEC,ILEAK,NL,NBN,RKEFF,RB2,
     >              A(TRHAR),
     >              A(REBAL),IFRITR,IACITR,
     >              A(FLUC),A(FLUN),A(PNL),A(XCSOU),
     >              A(XCTOT),A(XCTRC),A(XCDIA),A(XCCAT),
     >              A(XCNUF),A(XCCHI),A(XCABS),
     >              A(DIFF),A(DB2),A(IBETA),LEXIN,NOVOID,REFKEF)
*
      CALL MOC2WR ( IPFLUX,ITYPEC,RKEFF,RB2,
     >              NFUNL,NREG,NGRP,FLUXO,DIFFB1)
      CALL LCMSIX(IPFLUX,CDIRFL,ILCMDN)
      CALL LCMPUT(IPFLUX,'K-EFFECTIVE',1,2,RKEFF)
*
*
      CALL RLSARA( A(SIGANG ) )
      CALL RLSARA( A(IBETA  ) )
      CALL RLSARA( A(DB2    ) )
      CALL RLSARA( A(DIFF   ) )
      CALL RLSARA( A(XCCAT  ) )
      CALL RLSARA( A(XCABS  ) )
      CALL RLSARA( A(XCCHI  ) )
      CALL RLSARA( A(XCNUF  ) )
      CALL RLSARA( A(XCDIA  ) )
      CALL RLSARA( A(XCTRC  ) )
      CALL RLSARA( A(XCTOT  ) )
      CALL RLSARA( A(XCSOU  ) )
      CALL RLSARA( A(PNL    ) )
      CALL RLSARA( A(FLUN   ) )
      CALL RLSARA( A(FLUC   ) )
      CALL RLSARA( A(REBAL  ) )
      CALL RLSARA( A(TRHAR  ) )
*
      CALL RLSARA( I(KEYANI ) )
      CALL RLSARA( A(RHARM  ) )
      CALL RLSARA( I(ISGNR  ) )
      CALL RLSARA( A(FMEAN  ) )
      CALL RLSARA( A(DWEIG  ) )
      CALL RLSARA( A(DFLUX  ) )
      CALL RLSARA( A(IALPHA ) )
      CALL RLSARA( A(ICYP   ) )
      CALL RLSARA( A(ICYM   ) )
      CALL RLSARA( A(FLP    ) )
      CALL RLSARA( A(FLM    ) )
      CALL RLSARA( A(ICYCLE ) )
      CALL RLSARA( A(EXP2   ) )
      CALL RLSARA( A(EXPT   ) )
      CALL RLSARA( A(LENGHT ) )
      CALL RLSARA( I(NUMERO ) )
      CALL RLSARA( A(FLUX   ) )
      CALL RLSARA( A(PHIANG ) )
      CALL RLSARA( I(MATALB ) )
      CALL RLSARA( A(VOLSUR ) )
*
      RETURN
C
C------ FORMATS
C
 6000 FORMAT(//30X,' EDITION REGION/VOL/MIXTURE '//
     >3(5X,'REGION',5X,'VOL  ',5X,'MIXTURE')/)
 6001 FORMAT(1P,3(5X,I4,4X,E12.5,4X,I4,4X))
 6002 FORMAT(//30X,' G R O U P : ',I5//
     >30X,' TOTAL MACROSCOPIC CROSS SECTIONS PER MIXTURE '/)
 6003 FORMAT(3(1X,'MIXTURE',4X,'TOTAL',11X,'SIGW',3X)/
     >1P,3(1X,I4,3X,E12.5,3X,E12.5))
 6004 FORMAT(3(1X,'MIXTURE',4X,'NUSIGF',11X,'CHI ',3X)/
     >1P,3(1X,I4,3X,E12.5,3X,E12.5))
 6100 FORMAT(//' USE TRANSPORT CORRECTED CROSS-SECTIONS')
 6200 FORMAT(//' COMPUTATION WITH THE CHARACTERISTIC METHOD       ')
 7000 FORMAT(1X,'MAT(',I10,') SCAT(',I10,') TO GROUP:',I10,1P/
     >       5E20.7)
 7001 FORMAT(1X,'MOCDRV: NL   =',I10/
     >       1X,'MOCDRV: NFUNL=',I10/
     >       1X,'MOCDRV: NREG =',I10/
     >       1X,'MOCDRV: NUN  =',I10)
 7002 FORMAT(1X,'MATCOD HAS BEEN CHANGED ',I10,'->',I10)
 7010 FORMAT(1X,A6,':   ISOTROPIC OPTION *WITHOUT VOID*')
 7011 FORMAT(1X,A6,':   ISOTROPIC OPTION *WITH    VOID*')
 7012 FORMAT(1X,A6,': ANISOTROPIC OPTION *WITHOUT VOID*')
 7013 FORMAT(1X,A6,': ANISOTROPIC OPTION *WITH    VOID*')
 7020 FORMAT(1X,A6,': NO LEAKAGE-> REBALANCING   * ON *')
 7021 FORMAT(1X,A6,':  *** INCOMPATIBILITY ON LEAKAGE SWITCH ***')
 7022 FORMAT(1X,A6,':    LEAKAGE-> REBALANCING   * OFF*')
 7030 FORMAT(1X,A6,': Polar quadrature '/
     >       1X,'LCACT =',I8/
     >       1X,'NMU   =',I8)
 9000 FORMAT(1X,A6,': ECHO = FIRST RECORD DESTROYED      ')
      END
