*DECK NUMER4
      SUBROUTINE NUMER4(MAXPTS,MAXJ,NCOUR,MULTC,IAPP,IWIGN,NCODE,ZCODE,
     1 IHEX,LX,LY,VOL,ISS,IORI,ISM,IMPX,NMBLK,IFR,ALB,NMERGE,INUM,MIX,
     2 DVX,NGEN,IGEN,ISURF,CHORD,ITGEN,COORD,NSUPCE,XX,YY,RAYRE,NMC,
     3 POURCE,JF2,IGEN2,INUM2)
*
*-----------------------------------------------------------------------
*
* SURFACE RENUMBERING ALGORITHM FOR AN ASSEMBLY OF RECTANGULAR OR
* HEXAGONAL CELLS WITH TUBES IN SOME CELLS.
*
*   GENERAL PARAMETERS:
*   MAXPTS : ALLOCATED STORAGE FOR ARRAYS OF DIMENSION NMBLK.
*   MAXJ   : ALLOCATED STORAGE FOR ARRAYS OF DIMENSION IJAT, IJAR OR
*            IJAS.
*   NCOUR  : NUMBER OF SURROUNDING SURFACES PER CELL (INPUT);
*            NUMBER OF SURROUNDING OUT-CURRENTS PER CELL (OUTPUT).
*   MULTC  : TYPE OF MULTICELL APPROXIMATION.
*            =1 : ROTH    =2 : ROTH X NCOUR    =3 : DP-0    =4 : DP-1
*   IAPP   : TYPE OF ANGULAR FLUX EXPANSION BETWEEN THE TUBES.
*   IWIGN  : =1/2/3: ASKEW/WIGNER/SANCHEZ CYLINDERIZATION.
*   NCODE  : TYPE OF BOUNDARY CONDITION ON EACH SIDE OF THE DOMAIN:
*            =0 : NOT USED   =1 : VIDE     =2 : REFL
*            =3 : DIAG       =4 : TRAN     =5 : SYME
*   ZCODE  : VALUE OF THE ALBEDO ON EACH SIDE OF THE DOMAIN.
*            DIMENSION NCODE(6),ZCODE(6)
*   IHEX   : =0 : CARTESIAN LATTICE ; .NE.0 : HEXAGONAL LATTICE.
*   LX,LY  : TOTAL NUMBER OF CELLS ALONG EACH DIRECTION.
*   VOL    : VOLUMES OF THE MERGED BLOCKS. DIMENSION VOL(NMERGE)
*   ISS    : MIXTURE NUMBER ASSOCIATED WITH EACH MERGED BLOCK.
*            DIMENSION ISS(NMERGE)
*   IORI   : ORIENTATION OF EACH CELL. DIMENSION IORI(NMBLK)
*   ISM    : PERMUTATION INDEX CORRESPONDING TO EACH ORIENTATION.
*            ISM(I,N)=I IS THE NATURAL ORIENTATION.
*            DIMENSION ISM(NCOUR+2,NN) WHERE NN IS THE TOTAL NUMBER OF
*            DIFFERENT ORIENTATIONS PERMITTED.
*   IMPX   : PRINT FLAG. EQUAL TO 0 FOR NO PRINT.
*
*   PARAMETERS RELATED TO BLOCKS DEFINITION:
*   NMBLK  : TOTAL NUMBER OF BLOCKS IN THE DOMAIN.
*   IFR    : INDEX-NUMBER OF IN-CURRENTS. DIMENSION IFR(IJAS) WHERE
*            IJAS IS THE TOTAL NUMBER OF IN-CURRENTS SURROUNDING ALL
*            THE BLOCKS (MERGED OR NOT).
*   ALB    : TRANSMISSION/ALBEDO ASSOCIATED WITH EACH IN-CURRENT.
*            DIMENSION ALB(IJAS)
*
*   PARAMETERS RELATED TO THE MERGED BLOCKS DEFINITION:
*   NMERGE : TOTAL NUMBER OF MERGED BLOCKS FOR WHICH SPECIFIC VALUES
*            OF THE NEUTRON FLUX AND REACTIONS RATES ARE REQUIRED.
*            MANY BLOCKS WITH DIFFERENT POSITION IN THE DOMAIN CAN
*            BE MERGED BEFORE THE NEUTRON FLUX CALCULATION IF THEY
*            OWN THE SAME GENERATING BLOCK. THIS ALLOWS SOME REDUCTION
*            IN CPU TIME AND MEMORY. NMERGE.LE.NMBLK
*   INUM   : INDEX-NUMBER OF THE MERGED BLOCK ASSOCIATED TO EACH BLOCK.
*            INUM(I)=0 MEANS THAT BLOCK I IS NOT PART OF THE DOMAIN.
*            DIMENSION INUM(NMBLK)
*   MIX    : INDEX-NUMBER OF OUT-CURRENTS. DIMENSION MIX(IJAR) WHERE
*            IJAR IS THE TOTAL NUMBER OF OUT-CURRENTS LEAVING ALL THE
*            MERGED BLOCKS.
*   DVX    : WEIGHT ASSOCIATED WITH EACH OUT-CURRENT.
*            DIMENSION DVX(IJAR)
*
*            REMARK:
*            IFR, ALB, MIX AND DVX CONTAINS INFORMATION TO REBUILD THE
*            GEOMETRICAL "A" MATRIX.
*
*   PARAMETERS RELATED TO THE GENERATING BLOCKS DEFINITION:
*   NGEN   : TOTAL NUMBER OF GENERATING BLOCKS.
*   IGEN   : INDEX-NUMBER OF THE GENERATING BLOCK ASSOCIATED WITH EACH
*            MERGED BLOCK. DIMENSION IGEN(NMERGE)
*            TOTAL NUMBER OF GENERATING BLOCKS IN THE CARTESIAN DOMAIN.
*   ISURF  : NUMBER OF SURFACES SURROUNDING EACH GENERATING BLOCK.
*            DIMENSION ISURF(NGEN)
*   CHORD  : MEAN CHORD LENGTH ASSOCIATED WITH SURFACE OF THE GENERATING
*            BLOCKS. DIMENSION CHORD(NPIS) WHERE NPIS=SUM(ISURF(I))
*   ITGEN  : TYPE OF EACH GENERATING BLOCK:
*            =1 : TUBE/DP-0 INSIDE/DP-0 OUTSIDE/
*            =2 : TUBE/DP-1 INSIDE/DP-1 OUTSIDE/
*            =3 : RECTANGLE/DP-0 OUTSIDE/
*            =4 : RECTANGLE/DP-1 OUTSIDE/
*            =5 : RECTANGULAR CROWN/DP-0 INSIDE/ROTH OUTSIDE/
*            =6 : RECTANGULAR CROWN/DP-0 INSIDE/ROTH X 4 OUTSIDE/
*            =7 : RECTANGULAR CROWN/DP-1 INSIDE/ROTH OUTSIDE/
*            =8 : RECTANGULAR CROWN/DP-1 INSIDE/ROTH X 4 OUTSIDE/
*            =9 : RECTANGULAR CROWN/DP-0 INSIDE/DP-0 OUTSIDE/
*            =10: RECTANGULAR CROWN/DP-1 INSIDE/DP-0 OUTSIDE/
*            =11: RECTANGULAR CROWN/DP-0 INSIDE/DP-1 OUTSIDE/
*            =12: RECTANGULAR CROWN/DP-1 INSIDE/DP-1 OUTSIDE/
*            =13: HEXAGON/DP-0 OUTSIDE/
*            =14: HEXAGON/DP-1 OUTSIDE/
*            =15: HEXAGONAL CROWN/DP-0 INSIDE/ROTH OUTSIDE/
*            =16: HEXAGONAL CROWN/DP-0 INSIDE/ROTH X 4 OUTSIDE/
*            =17: HEXAGONAL CROWN/DP-1 INSIDE/ROTH OUTSIDE/
*            =18: HEXAGONAL CROWN/DP-1 INSIDE/ROTH X 4 OUTSIDE/
*            =19: HEXAGONAL CROWN/DP-0 INSIDE/DP-0 OUTSIDE/
*            =20: HEXAGONAL CROWN/DP-1 INSIDE/DP-0 OUTSIDE/
*            =21: HEXAGONAL CROWN/DP-0 INSIDE/DP-1 OUTSIDE/
*            =22: HEXAGONAL CROWN/DP-1 INSIDE/DP-1 OUTSIDE/
*            DIMENSION ITGEN(NGEN)
*   COORD  : DIMENSIONS OF EACH GENERATING BLOCK.
*            DIMENSION COORD(3,NGEN)
*   NSUPCE : NUMBER OF GENERATING CARTESIAN CELLS. NSUPCE.LE.NGEN
*   XX,YY  : X AND Y THICKNESS OF THE GENERATING CELLS.
*            DIMENSION XX(NSUPCE),YY(NSUPCE)
*   RAYRE  : RADIUS OF THE TUBES IN EACH GENERATING CELL.
*            DIMENSION RAYRE(NSUPCE)
*   NMC    : OFFSET OF THE FIRST VOLUME IN EACH GENERATING CARTESIAN
*            CELL. DIMENSION NMC(NSUPCE)
*
*   SCRATCH STORAGE:
*   POURCE : DIMENSION POURCE(NMBLK)
*   JF2    : DIMENSION JF2((NCOUR+2)*MAXPTS)
*   IGEN2  : DIMENSION IGEN2(NMBLK)
*   INUM2  : DIMENSION INUM2(NMBLK)
*
*------------------------------------------------ A. HEBERT 26/03/87 ---
*
      LOGICAL LOG
      CHARACTER DIRR(6)*2,DIRH(8)*2,DIRZ(18)*2,HSMG*131
      PARAMETER (PI=3.141592654)
      DIMENSION NCODE(*),ZCODE(*),VOL(*),ISS(*),IORI(*),ISM(NCOUR+2,*),
     1 IFR(*),ALB(*),INUM(*),MIX(*),DVX(*),IGEN(*),ISURF(*),CHORD(*),
     2 ITGEN(*),COORD(3,*),XX(*),YY(*),RAYRE(*),NMC(*),POURCE(*),JF2(*),
     3 IGEN2(*),INUM2(*),GAR(2),ISMZ(18)
      COMMON BASE(1)
      SAVE DIRR,DIRH
      DATA DIRR/'X-','X+','Y-','Y+','Z-','Z+'/
      DATA DIRH/'H1','H2','H3','H4','H5','H6','Z-','Z+'/
*
* VERIFICATION OF THE ADEQUACY OF MAXJ.
      IF(MULTC.EQ.1) THEN
         IJAS=NMBLK
      ELSE IF(MULTC.EQ.2) THEN
         IJAS=NCOUR*NMBLK
      ELSE IF(MULTC.EQ.3) THEN
         IJAS=NCOUR*NMBLK
      ELSE IF(MULTC.EQ.4) THEN
         IJAS=3*NCOUR*NMBLK
      ENDIF
      DO 5 ICEL=1,NMBLK
      IKG=IGEN(INUM(ICEL))
    5 IJAS=IJAS+2*IAPP*(NMC(IKG+1)-NMC(IKG)-1)
      IF(IJAS.GT.MAXJ) THEN
         WRITE (HSMG,360) MAXJ,IJAS
         CALL XABORT(HSMG)
      ENDIF
*
      NCODE(5)=0
      NCODE(6)=0
      CALL SETARA(BASE,NGEN,IZZ)
      DO 10 IKG=1,NGEN
   10 BASE(IZZ+IKG-1)=1.0
      CALL SETARA(BASE,(NCOUR+2)*NMBLK,IJF2)
      CALL SETARA(BASE,NMBLK,IGG3)
      IF(IHEX.GT.0) THEN
         CALL NUMERH(NCOUR,MULTC,NCODE,ZCODE,IHEX,LX,1,IORI,ISM,POURCE,
     1   IMPX-10,NMBLK,IFR,ALB,NMERGE,INUM,MIX,DVX,IGEN,XX,BASE(IZZ),
     2   BASE(IJF2),BASE(IGG3))
      ELSE
         CALL NUMER3(NCOUR,MULTC,NCODE,ZCODE,LX,LY,1,IORI,ISM,POURCE,
     1   IMPX-10,NMBLK,IFR,ALB,NMERGE,INUM,MIX,DVX,IGEN,XX,YY,BASE(IZZ),
     2   BASE(IJF2),BASE(IGG3))
      ENDIF
      CALL RLSARA(BASE(IGG3))
      CALL RLSARA(BASE(IJF2))
      CALL RLSARA(BASE(IZZ))
*
* COMPUTE THE NUMBER OF DISTINCT OUT CURRENTS BETWEEN THE CELLS.
      IJAT=0
      DO 15 I=1,NCOUR*NMERGE
   15 IJAT=MAX(IJAT,MIX(I))
*
      NSUPCE=NGEN
      NMERG2=NMERGE
      NMBLK2=NMBLK
      DO 20 IKK=1,NMERG2
   20 IGEN2(IKK)=IGEN(IKK)
      DO 30 IBLK=1,NMBLK2
   30 INUM2(IBLK)=INUM(IBLK)
*
* SET-UP THE NEW INTERFACE CURRENTS.
*     PART 1.
      IJAS=NCOUR*NMBLK2
      NMBLK=0
      IJ=0
      DO 50 IBLK=1,NMBLK2
      IKG=IGEN2(INUM2(IBLK))
      IND=NMC(IKG+1)-NMC(IKG)
      NSURF=2*IAPP*(IND-1)
      IF(NSURF.GT.0) THEN
         IF(IJAS+NSURF.GT.MAXJ) CALL XABORT('NUMER4: ABORT NB 1.')
         DO 40 I=IJAS,IJ+1,-1
         IFR(I+NSURF)=IFR(I)
   40    ALB(I+NSURF)=ALB(I)
         IJAS=IJAS+NSURF
      ENDIF
      NMBLK=NMBLK+IND
   50 IJ=IJ+NCOUR+NSURF
*
*     PART 2.
      IJAR=NCOUR*NMERG2
      NMERGE=0
      IJ=0
      DO 160 IKK=1,NMERG2
      IKG=IGEN2(IKK)
      IND=NMC(IKG+1)-NMC(IKG)
      IB=0
      DO 80 IBLK=1,NMBLK2
      IKG2=IGEN2(INUM2(IBLK))
      IND2=NMC(IKG2+1)-NMC(IKG2)
      IF(INUM2(IBLK).EQ.IKK) THEN
         DO 70 K=1,IND
   70    INUM(IB+K)=NMERGE+K
      ENDIF
   80 IB=IB+IND2
      NMERGE=NMERGE+IND
*
      NSURF=2*IAPP*(IND-1)
      IF(NSURF.GT.0) THEN
         DO 90 I=IJAR,IJ+1,-1
         MIX(I+NSURF)=MIX(I)
   90    DVX(I+NSURF)=DVX(I)
         DO 100 I=1,NSURF
         MIX(IJ+I)=IJAT+I
  100    DVX(IJ+I)=1.0
         IJAR=IJAR+NSURF
         IJAT=IJAT+NSURF
         IFRACT=0
         DO 110 IBLK=1,NMBLK2
         IF(INUM2(IBLK).EQ.IKK) IFRACT=IFRACT+1
  110    CONTINUE
         IK=0
         DO 150 IBLK=1,NMBLK2
         IKG2=IGEN2(INUM2(IBLK))
         IND2=NMC(IKG2+1)-NMC(IKG2)
         IF(INUM2(IBLK).EQ.IKK) THEN
            DO 120 I=1,NSURF
  120       ALB(IK+I)=1.0/REAL(IFRACT)
            IF(IAPP.EQ.1) THEN
               DO 130 I=1,IND-1
               IFR(IK+2*I-1)=MIX(IJ+2*I-1)+1
  130          IFR(IK+2*I)=MIX(IJ+2*I)-1
            ELSE IF(IAPP.EQ.2) THEN
               DO 140 I=1,IND-1
               IFR(IK+4*I-3)=MIX(IJ+4*I-3)+2
               IFR(IK+4*I-2)=MIX(IJ+4*I-2)+2
               IFR(IK+4*I-1)=MIX(IJ+4*I-1)-2
  140          IFR(IK+4*I)=MIX(IJ+4*I)-2
            ENDIF
         ENDIF
  150    IK=IK+NCOUR+2*IAPP*(IND2-1)
      ENDIF
  160 IJ=IJ+NCOUR+NSURF
*
* REMOVE THE UNUSED SURFACE NUMBERS.
      IF(IJAS.GT.(NCOUR+2)*MAXPTS) CALL XABORT('NUMER4: ABORT NB 2.')
      DO 170 I=1,IJAS
  170 JF2(I)=0
      IJAT=0
      DO 180 I=1,IJAR
      J=MIX(I)
      IF(J.GT.IJAS) CALL XABORT('NUMER4: FAILURE OF THE BLOCK RENUMBE'
     1 //'RING ALGORITHM.')
      IF(JF2(J).EQ.0) THEN
         IJAT=IJAT+1
         JF2(J)=IJAT
      ENDIF
  180 CONTINUE
      IF(IMPX.GE.1) WRITE (6,400) IJAT,IJAR,IJAS,MAXJ
      IF(IJAS.GT.MAXJ) CALL XABORT('NUMER4: INSUFFICIENT MAXJ.')
      DO 190 I=1,IJAR
  190 MIX(I)=JF2(MIX(I))
      DO 200 I=1,IJAS
  200 IFR(I)=JF2(IFR(I))
*
* COMPUTE THE VOLUMES AND SET-UP THE NEW GENERATING BLOCKS.
      DO 210 IKK=1,NMERGE
  210 IGEN(IKK)=0
      NGEN=0
      IOF=0
      IMIN=0
      DO 290 IKK=1,NMERG2
      IKG=IGEN2(IKK)
      A=XX(IKG)
      B=YY(IKG)
      I1=NMC(IKG)
      I2=NMC(IKG+1)-I1
      RIN=0.0
      DO 260 I=1,I2-1
      MAT=ISS(IMIN+I)
      ROU=RAYRE(I1+I+1)
      VOL(IMIN+I)=PI*(ROU*ROU-RIN*RIN)*POURCE(IKK)
      IF(IGEN(IMIN+I).EQ.0) THEN
         NGEN=NGEN+1
         IGEN(IMIN+I)=NGEN
         IF(I.EQ.1) THEN
            GAR(1)=2.0*VOL(IMIN+I)/(PI*ROU*POURCE(IKK))
            ISURF(NGEN)=IAPP
         ELSE
            GAR(1)=2.0*VOL(IMIN+I)/(PI*RIN*POURCE(IKK))
            GAR(2)=2.0*VOL(IMIN+I)/(PI*ROU*POURCE(IKK))
            ISURF(NGEN)=2*IAPP
         ENDIF
         IF(IAPP.EQ.1) THEN
            DO 220 J=1,ISURF(NGEN)
  220       CHORD(IOF+J)=GAR(J)
            ITGEN(NGEN)=1
         ELSE IF(IAPP.EQ.2) THEN
            DO 230 J=1,ISURF(NGEN)
            CHORD(IOF+2*J-1)=GAR(J)
  230       CHORD(IOF+2*J)=GAR(J)
            ITGEN(NGEN)=2
         ENDIF
         IOF=IOF+ISURF(NGEN)
         COORD(1,NGEN)=RIN
         COORD(2,NGEN)=ROU
         COORD(3,NGEN)=0.0
         JMIN=IMIN+I2
         DO 250 JKK=IKK+1,NMERG2
         JKG=IGEN2(JKK)
         J1=NMC(JKG)
         J2=NMC(JKG+1)-J1
         RINJ=0.0
         DO 240 J=1,J2-1
         ROUJ=RAYRE(J1+J+1)
         LOG=(MAT.EQ.ISS(JMIN+J)).AND.(RIN.EQ.RINJ).AND.(ROU.EQ.ROUJ)
         IF(LOG) IGEN(JMIN+J)=NGEN
  240    RINJ=ROUJ
  250    JMIN=JMIN+J2
      ENDIF
  260 RIN=ROU
      IF(IHEX.EQ.0) THEN
         VOL(IMIN+I2)=(A*B-PI*RIN*RIN)*POURCE(IKK)
      ELSE
         VOL(IMIN+I2)=(1.5*SQRT(3.0)*A*A-PI*RIN*RIN)*POURCE(IKK)
      ENDIF
      MAT=ISS(IMIN+I2)
      IF(IGEN(IMIN+I2).EQ.0) THEN
         NGEN=NGEN+1
         IGEN(IMIN+I2)=NGEN
         IF(I2.EQ.1) THEN
            ISURF(NGEN)=NCOUR
         ELSE
            ISURF(NGEN)=NCOUR+IAPP
         ENDIF
         IF((IHEX.GT.0).AND.(MULTC.LE.2).AND.(IWIGN.EQ.2)) THEN
            SCALE=SQRT(1.5*SQRT(3.0)*PI)/3.0
         ELSE IF((MULTC.LE.2).AND.(IWIGN.EQ.2)) THEN
            SCALE=SQRT(A*B*PI)/(A+B)
         ELSE
            SCALE=1.0
         ENDIF
         IF((I2.GT.1).AND.(IAPP.EQ.1)) THEN
            CHORD(IOF+1)=2.0*VOL(IMIN+I2)/(PI*RIN*POURCE(IKK))
            IOF=IOF+1
         ELSE IF((I2.GT.1).AND.(IAPP.EQ.2)) THEN
            CHORD(IOF+1)=2.0*VOL(IMIN+I2)/(PI*RIN*POURCE(IKK))
            CHORD(IOF+2)=CHORD(IOF+1)
            IOF=IOF+2
         ENDIF
         IF(IHEX.EQ.0) THEN
            GAR(1)=4.0*VOL(IMIN+I2)/(SCALE*B*POURCE(IKK))
            GAR(2)=4.0*VOL(IMIN+I2)/(SCALE*A*POURCE(IKK))
            DO 270 J=1,NCOUR/2
            CHORD(IOF+J)=GAR(1)
  270       CHORD(IOF+J+NCOUR/2)=GAR(2)
            IF(MULTC.EQ.1) THEN
               IF(IAPP.EQ.1) ITGEN(NGEN)=5
               IF(IAPP.EQ.2) ITGEN(NGEN)=7
            ELSE IF(MULTC.EQ.2) THEN
               IF(IAPP.EQ.1) ITGEN(NGEN)=6
               IF(IAPP.EQ.2) ITGEN(NGEN)=8
            ELSE IF(MULTC.EQ.3) THEN
               IF(I2.EQ.1) ITGEN(NGEN)=3
               IF((IAPP.EQ.1).AND.(I2.GT.1)) ITGEN(NGEN)=9
               IF((IAPP.EQ.2).AND.(I2.GT.1)) ITGEN(NGEN)=10
            ELSE IF(MULTC.EQ.4) THEN
               IF(I2.EQ.1) ITGEN(NGEN)=4
               IF((IAPP.EQ.1).AND.(I2.GT.1)) ITGEN(NGEN)=11
               IF((IAPP.EQ.2).AND.(I2.GT.1)) ITGEN(NGEN)=12
            ENDIF
         ELSE
            GAR(1)=4.0*VOL(IMIN+I2)/(SCALE*A*POURCE(IKK))
            DO 275 J=1,NCOUR
  275       CHORD(IOF+J)=GAR(1)
            IF(MULTC.EQ.1) THEN
               IF(IAPP.EQ.1) ITGEN(NGEN)=15
               IF(IAPP.EQ.2) ITGEN(NGEN)=17
            ELSE IF(MULTC.EQ.2) THEN
               IF(IAPP.EQ.1) ITGEN(NGEN)=16
               IF(IAPP.EQ.2) ITGEN(NGEN)=18
            ELSE IF(MULTC.EQ.3) THEN
               IF(I2.EQ.1) ITGEN(NGEN)=13
               IF((IAPP.EQ.1).AND.(I2.GT.1)) ITGEN(NGEN)=19
               IF((IAPP.EQ.2).AND.(I2.GT.1)) ITGEN(NGEN)=20
            ELSE IF(MULTC.EQ.4) THEN
               IF(I2.EQ.1) ITGEN(NGEN)=14
               IF((IAPP.EQ.1).AND.(I2.GT.1)) ITGEN(NGEN)=21
               IF((IAPP.EQ.2).AND.(I2.GT.1)) ITGEN(NGEN)=22
            ENDIF
         ENDIF
         IOF=IOF+NCOUR
         COORD(1,NGEN)=RIN
         COORD(2,NGEN)=A
         COORD(3,NGEN)=B
         JMIN=IMIN+I2
         DO 280 JKK=IKK+1,NMERG2
         JKG=IGEN2(JKK)
         J1=NMC(JKG)
         J2=NMC(JKG+1)-J1
         RINJ=RAYRE(J1+J2)
         LOG=(MAT.EQ.ISS(JMIN+J2)).AND.(RIN.EQ.RINJ).AND.(A.EQ.XX(JKG))
     1   .AND.(B.EQ.YY(JKG))
         IF(LOG) IGEN(JMIN+J2)=NGEN
  280    JMIN=JMIN+J2
      ENDIF
  290 IMIN=IMIN+I2
      IF(IMPX.GE.1) WRITE (6,380) NMBLK,NMERGE,NGEN,MAXPTS
      IF(NMBLK.GT.MAXPTS) THEN
         WRITE(HSMG,'(37HNUMER4: MAXPTS SHOULD BE INCREASED TO,I8)')
     1   NMBLK
         CALL XABORT(HSMG)
      ENDIF
*
* PRINT THE SURFACE NUMBERS OF EACH CELL.
      IF(IMPX.GT.2) THEN
         MIN8=MIN(8,NCOUR)
         WRITE (6,620)
         WRITE (6,650) ('---------------',I=1,8)
         IBLKN=0
         IT0=0
         DO 340 IBLK=1,NMBLK2
         IKK=INUM2(IBLK)
         IKG=IGEN2(IKK)
         I2=NMC(IKG+1)-NMC(IKG)
         IT1=0
         DO 294 I=1,INUM(IBLKN+1)-1
  294    IT1=IT1+ISURF(IGEN(I))
         NSURF=0
         DO 295 I=1,I2
         IBLKN=IBLKN+1
         IKGN=IGEN(INUM(IBLKN))
  295    NSURF=NSURF+ISURF(IKGN)
         WRITE (6,630) IBLK,IKK,IKG,NSURF
         NSURF=NSURF-NCOUR
         IF(NSURF.GT.0) THEN
            I1=1
            DO 300 I=1,1+(NSURF-1)/8
            I2=I1+7
            IF(I2.GT.NSURF) I2=NSURF
            WRITE (6,634) (J,J=I1,I2)
            WRITE (6,635) (MIX(IT1+J),IFR(IT0+J),J=I1,I2)
            WRITE (6,640) (ALB(IT0+J),J=I1,I2)
            WRITE (6,645) (DVX(IT1+J),J=I1,I2)
  300       I1=I1+8
            IT0=IT0+NSURF
            IT1=IT1+NSURF
         ENDIF
         I1=IORI(IBLK)
         IF((MULTC.EQ.4).AND.(NCOUR.EQ.18)) THEN
            DO 305 I=1,NCOUR
            ISMZ(I)=3*ISM(1+(I-1)/3,I1)+MOD(I-1,3)-2
  305       DIRZ(I)=DIRH(1+(I-1)/3)
         ELSE IF((MULTC.EQ.4).AND.(NCOUR.EQ.12)) THEN
            DO 310 I=1,NCOUR
            ISMZ(I)=3*ISM(1+(I-1)/3,I1)+MOD(I-1,3)-2
  310       DIRZ(I)=DIRR(1+(I-1)/3)
         ELSE IF((MULTC.EQ.4).AND.(NCOUR.EQ.4)) THEN
            DO 320 I=1,NCOUR
            ISMZ(I)=2*ISM(1+(I-1)/2,I1)+MOD(I-1,2)-1
  320       DIRZ(I)=DIRR(1+(I-1)/2)
         ELSE IF(IHEX.EQ.0) THEN
            DO 330 I=1,NCOUR
            ISMZ(I)=ISM(I,I1)
  330       DIRZ(I)=DIRR(I)
         ELSE
            DO 335 I=1,NCOUR
            ISMZ(I)=ISM(I,I1)
  335       DIRZ(I)=DIRH(I)
         ENDIF
         WRITE (6,660) (DIRZ(I),I=1,MIN8)
         WRITE (6,635) (MIX(IT1+ISMZ(I)),IFR(IT0+ISMZ(I)),I=1,MIN8)
         WRITE (6,640) (ALB(IT0+ISMZ(I)),I=1,MIN8)
         WRITE (6,645) (DVX(IT1+ISMZ(I)),I=1,MIN8)
         IF(NCOUR.GT.8) THEN
            MIN16=MIN(16,NCOUR)
            WRITE (6,660) (DIRZ(I),I=9,MIN16)
            WRITE (6,635) (MIX(IT1+ISMZ(I)),IFR(IT0+ISMZ(I)),I=9,MIN16)
            WRITE (6,640) (ALB(IT0+ISMZ(I)),I=9,MIN16)
            WRITE (6,645) (DVX(IT1+ISMZ(I)),I=9,MIN16)
         ENDIF
         IF(NCOUR.GT.16) THEN
            WRITE (6,660) (DIRZ(I),I=17,18)
            WRITE (6,635) (MIX(IT1+ISMZ(I)),IFR(IT0+ISMZ(I)),I=17,18)
            WRITE (6,640) (ALB(IT0+ISMZ(I)),I=17,18)
            WRITE (6,645) (DVX(IT1+ISMZ(I)),I=17,18)
         ENDIF
         IT0=IT0+NCOUR
         WRITE (6,650) ('---------------',I=1,8)
  340    CONTINUE
         WRITE (6,'(/)')
      ENDIF
      RETURN
*
  360 FORMAT (14HNUMER4: MAXJ (,I5,31H) SHOULD BE GREATER OR EQUAL TO,
     1 I5)
  380 FORMAT (/39H TOTAL NUMBER OF BLOCKS IN THE DOMAIN =,I5/
     1         32H TOTAL NUMBER OF MERGED BLOCKS =,I5/
     2         36H TOTAL NUMBER OF GENERATING BLOCKS =,I5,5X,
     3         26HAVAILABLE STORAGE MAXPTS =,I5/)
  400 FORMAT (/40H TOTAL NUMBER OF DISTINCT OUT-CURRENTS =,I5/
     1 61H TOTAL NUMBER OF OUT-CURRENTS LEAVING ALL THE MERGED BLOCKS =,
     2 I5/57H TOTAL NUMBER OF IN-CURRENTS SURROUNDING ALL THE BLOCKS =,
     3 I5,5X,24HAVAILABLE STORAGE MAXJ =,I5/)
  620 FORMAT (///30H SURFACE NUMBERS OF EACH CELL:/)
  630 FORMAT (6H CELL=,I5,5X,12HMERGED CELL=,I5,5X,16HGENERATING CELL=,
     1 I5,5X,19HNUMBER OF SURFACES=,I3)
  634 FORMAT (/8H  SURF.:,8(I7,7X,1HI))
  635 FORMAT (8H IN/OUT:,8(I5,2H /,I5,3H  I))
  640 FORMAT (8H ALBEDO:,1P,8(E12.5,3H  I))
  645 FORMAT (8H    DVX:,1P,8(E12.5,3H  I))
  650 FORMAT (8H -------,8(A15))
  660 FORMAT (/8H   SIDE:,8(A9,5X,1HI))
      END
