*DECK NUMERC
      SUBROUTINE NUMERC (ITYPE,IAPP,NCODE,ZCODE,LX,LZ,POURCE,IMPX,
     1 NMBLK,IFR,ALB,NMERGE,INUM,NGEN,IGEN,XX,YY,ZZ,ISURF,JF1,JF2)
*
*-----------------------------------------------------------------------
*
* SURFACE RENUMBERING ALGORITHM FOR CYLINDRICAL GEOMETRY.
*
* GENERAL PARAMETERS:
*   ITYPE  : TYPE OF GEOMETRY.
*   IAPP   : TYPE OF INTERFACE CURRENTS.
*            =1 : DP-0   =2 : DP-1   =3 : DP-1 INSIDE / DP-0 OUTSIDE
*   NCODE  : TYPE OF BOUNDARY CONDITION ON EACH SIDE OF THE DOMAIN:
*            =0 : NOT USED   =1 : VIDE     =2 : REFL
*            =3 : NOT USED   =4 : TRAN     =5 : SYME
*   ZCODE  : VALUE OF THE ALBEDO ON EACH SIDE OF THE DOMAIN.
*            DIMENSION NCODE(6),ZCODE(6)
*   LX,LZ  : TOTAL NUMBER OF BLOCKS ALONG EACH DIRECTION.
*   POURCE : WEIGHT ASSOCIATED WITH EACH MERGED BLOCK.
*            DIMENSION POURCE(NMERGE)
*   IMPX   : PRINT FLAG. EQUAL TO 0 FOR NO PRINT.
*
*   PARAMETERS RELATED TO BLOCKS DEFINITION:
*   NMBLK  : TOTAL NUMBER OF BLOCKS IN THE DOMAIN INCLUDING THE VIRTUAL
*            BLOCKS (AT INPUT). NMBLK=NMERGE AT OUTPUT
*   NMERGE : TOTAL NUMBER OF BLOCKS IN THE DOMAIN.
*   INUM   : VOLUME INDEX ASSOCIATED WITH EACH BLOCK. INUM(I)=0 MEANS
*            THAT BLOCK I IS NOT PART OF THE DOMAIN.
*            DIMENSION INUM(NMBLK)
*   NGEN   : NUMBER OF GENERATING BLOCKS.
*   IFR    : INDEX-NUMBER OF IN-CURRENTS. DIMENSION IFR(IJAR) WHERE
*            IJAR IS THE TOTAL NUMBER OF IN-CURRENTS SURROUNDING ALL
*            THE BLOCKS.
*   ALB    : TRANSMISSION/ALBEDO ASSOCIATED WITH EACH IN-CURRENT.
*            DIMENSION ALB(IJAR)
*
*            REMARK:
*            IFR AND ALB CONTAINS INFORMATION TO REBUILD THE
*            GEOMETRICAL "A" MATRIX.
*
*   PARAMETERS RELATED TO THE GENERATING BLOCKS DEFINITION:
*   IGEN   : INDEX-NUMBER OF THE GENERATING BLOCK ASSOCIATED WITH EACH
*            BLOCK. DIMENSION IGEN(NMERGE)
*   ISURF  : NUMBER OF SURFACES ASSOCIATED WITH EACH GENERATING BLOCK.
*            DIMENSION ISURF(NGEN)
*   XX,YY  : INTERNAL AND EXTERNAL RADIUS OF THE GENERATING CELLS.
*            DIMENSION XX(NGEN),YY(NGEN)
*   ZZ     : HEIGHT OF THE 2-D CYLINDERS. DIMENSION ZZ(NGEN)
*
* SCRATCH STORAGE:
*   JF1    : DIMENSION JF1(4*NMERGE)
*   JF2    : DIMENSION JF2(4*NMERGE)
*
*------------------------------------------------ A. HEBERT 19/07/88 ---
*
      PARAMETER (EPS=1.0E-5)
      LOGICAL LOG1,LOG2,LOG3,FILLED
      CHARACTER DIRR(4)*2,DIR2(4)*2,HSMG*131
      DIMENSION IGEN(*),IFR(*),ALB(*),POURCE(*),NCODE(*),ZCODE(*),
     1 INUM(*),XX(*),YY(*),ZZ(*),ISURF(*),JF1(*),JF2(*),IDDD(4)
      SAVE DIRR,DIR2
      DATA DIRR/'R-','R+','Z-','Z+'/
      DATA DIR2/'R-','R-','R+','R+'/
*
      NCOUR=2
      IF (ITYPE.EQ.6) NCOUR=4
      IBLK=0
      DO 30 K1=1,LZ
      DO 30 K2=1,LX
      IBLK=IBLK+1
      IKK=INUM(IBLK)
      FRY=1.0
      IF (IKK.EQ.0) GO TO 30
      IS=NCOUR*(IBLK-1)
      IT=NCOUR*(IKK-1)
      DO 10 IC=1,4
10    IDDD(IC)=-1
      IF (K2.GT.1)  IDDD(1)=IBLK-1
      IF (K2.LT.LX) IDDD(2)=IBLK+1
      IF (K1.GT.1)  IDDD(3)=IBLK-LX
      IF (K1.LT.LZ) IDDD(4)=IBLK+LX
      DO 20 IC=1,NCOUR
      ALB(IS+IC)=1.0
      JBLK=IDDD(IC)
      IF (JBLK.GT.0) THEN
         JKK=INUM(JBLK)
         JT=NCOUR*(JKK-1)
         IF ((MOD(IC,2).EQ.1).AND.(JKK.GT.0)) THEN
            IFR(IS+IC)=JT+IC+1
         ELSE IF ((MOD(IC,2).EQ.0).AND.(JKK.GT.0)) THEN
            IFR(IS+IC)=JT+IC-1
         ELSE
            IFR(IS+IC)=0
         ENDIF
         IDDD(IC)=JKK
      ELSE
         IFR(IS+IC)=0
      ENDIF
20    JF1(IT+IC)=IT+IC
*
* BOUNDARY LOCATED ON THE CYLINDRICAL AXIS.
      IF (K2.EQ.1) THEN
         IFR(IS+1)=-99
         JF1(IT+1)=-99
      ENDIF
*
* VIDE OR REFL BOUNDARY CONDITIONS.
      LOG1=(K2.EQ.LX).OR.(IDDD(2).EQ.0)
      LOG2=(NCODE(2).EQ.1)
      LOG3=(NCODE(2).EQ.2)
      IF (LOG1.AND.LOG2) THEN
         ALB(IS+2)=ZCODE(2)
         IFR(IS+2)=IT+2
      ELSE IF (LOG1.AND.LOG3) THEN
         ALB(IS+2)=1.0
         IFR(IS+2)=IT+2
      ENDIF
      LOG1=(K1.EQ.1).OR.(IDDD(3).EQ.0)
      LOG2=(NCODE(5).EQ.1)
      LOG3=(NCODE(5).EQ.2)
      IF (LOG1.AND.LOG2) THEN
         ALB(IS+3)=ZCODE(5)
         IFR(IS+3)=IT+3
      ELSE IF (LOG1.AND.LOG3) THEN
         ALB(IS+3)=1.0
         IFR(IS+3)=IT+3
      ENDIF
      LOG1=(K1.EQ.LZ).OR.(IDDD(4).EQ.0)
      LOG2=(NCODE(6).EQ.1)
      LOG3=(NCODE(6).EQ.2)
      IF (LOG1.AND.LOG2) THEN
         ALB(IS+4)=ZCODE(6)
         IFR(IS+4)=IT+4
      ELSE IF (LOG1.AND.LOG3) THEN
         ALB(IS+4)=1.0
         IFR(IS+4)=IT+4
      ENDIF
*
* TRAN BOUNDARY CONDITION.
      IF ((K1.EQ.1).AND.(NCODE(5).EQ.4)) THEN
         JBLK=IBLK+(LZ-1)*LX
         IFR(IS+3)=NCOUR*(INUM(JBLK)-1)+4
      ENDIF
      IF ((K1.EQ.LZ).AND.(NCODE(6).EQ.4)) THEN
         JBLK=IBLK-(LZ-1)*LX
         IFR(IS+4)=NCOUR*(INUM(JBLK)-1)+3
      ENDIF
*
* SYME BOUNDARY CONDITION.
      IF ((K1.EQ.1).AND.(NCODE(5).EQ.5)) THEN
         FRY=0.5
         ALB(IS+3)=ALB(IS+4)
         IFR(IS+3)=IFR(IS+4)
         JF1(IT+3)=JF1(IT+4)
      ELSE IF ((K1.EQ.LZ).AND.(NCODE(6).EQ.5)) THEN
         FRY=0.5
         ALB(IS+4)=ALB(IS+3)
         IFR(IS+4)=IFR(IS+3)
         JF1(IT+4)=JF1(IT+3)
      ENDIF
30    POURCE(IKK)=FRY
*
* ELIMINATION OF THE BLOCKS OUTSIDE THE DOMAIN.
      JBLK=0
      DO 70 IBLK=1,NMBLK
      IKK=INUM(IBLK)
      IF (IKK.GT.0) THEN
         JBLK=JBLK+1
         INUM(JBLK)=IKK
         IS=NCOUR*(IBLK-1)
         JS=NCOUR*(JBLK-1)
         DO 60 IC=1,NCOUR
         IFR(JS+IC)=IFR(IS+IC)
60       ALB(JS+IC)=ALB(IS+IC)
      ENDIF
70    CONTINUE
      NMBLK=JBLK
      IF (NMBLK.NE.NMERGE) GO TO 570
*
* VALIDATION OF VECTOR IFR.
      DO 80 IBLK=1,NMERGE
      IT=NCOUR*(IBLK-1)
      DO 80 IC=1,NCOUR
      IS=IFR(IT+IC)
      IF (IS.EQ.-99) GO TO 80
      IF (IS.EQ.0) GO TO 570
      JC=1+MOD(IS-1,NCOUR)
      JT=NCOUR*((IS-1)/NCOUR)
      IF (JF1(JT+JC).NE.IS) GO TO 570
80    CONTINUE
*
* VALIDATION OF THE GEOMETRICAL RECIPROCITY.
      DO 90 IKK=1,NMERGE
      IKG=IGEN(IKK)
      IT=NCOUR*(IKK-1)
      DO 90 IC=1,NCOUR
      IP=1+MOD(JF1(IT+IC)-1,NCOUR)
      IF ((IP.EQ.1).AND.(XX(IKG).EQ.0.0)) GO TO 90
      IF (IP.EQ.1) FR1=XX(IKG)*ZZ(IKG)
      IF (IP.EQ.2) FR1=YY(IKG)*ZZ(IKG)
      IF ((IP.EQ.3).OR.(IP.EQ.4)) FR1=YY(IKG)**2-XX(IKG)**2
      JP=1+MOD(IFR(IT+IC)-1,NCOUR)
      JKG=IGEN(1+(IFR(IT+IC)-1)/NCOUR)
      IF (JP.EQ.1) FR2=XX(JKG)*ZZ(JKG)
      IF (JP.EQ.2) FR2=YY(JKG)*ZZ(JKG)
      IF ((JP.EQ.3).OR.(JP.EQ.4)) FR2=YY(JKG)**2-XX(JKG)**2
      DELTA=ABS(FR1-FR2)
      IF (ABS(FR1-FR2).GT.EPS) THEN
         WRITE (HSMG,680) DIRR(IP),IKG,DIRR(JP),JKG
         CALL XABORT(HSMG)
      ENDIF
90    CONTINUE
*
* INCLUDE THE DP-1 APPROXIMATION.
      IF ((IAPP.GE.2).AND.(NCOUR.EQ.2)) THEN
         DO 120 I1=2*NMERGE,1,-1
         IF (JF1(I1).EQ.-99) THEN
            DO 100 JCOUR=1,2
            JSURF=(I1-1)*2+JCOUR
            JF1(JSURF)=-99
100         IFR(JSURF)=-99
         ELSE
            FR1=ALB(I1)
            JND1=(JF1(I1)-1)*2
            JND2=(IFR(I1)-1)*2
            DO 110 JCOUR=1,2
            JSURF=(I1-1)*2+JCOUR
            ALB(JSURF)=FR1
            JF1(JSURF)=JND1+JCOUR
110         IFR(JSURF)=JND2+JCOUR
         ENDIF
120      CONTINUE
         IF (IAPP.EQ.3) JF1(4*NMERGE)=-99
         NCOUR=4
      ENDIF
*
* REMOVE THE SURFACES LOCATED ON THE CYLINDRICAL AXIS.
      IJAR=NCOUR*NMERGE
      IT=0
      DO 230 IKK=1,NMERGE
      IS=0
      DO 220 IC=1,NCOUR
      IF (JF1(IT+IS+1).GT.0) THEN
         IS=IS+1
      ELSE IF (JF1(IT+IS+1).EQ.-99) THEN
         IJAR=IJAR-1
         DO 210 I=IT+IS+1,IJAR
         JF1(I)=JF1(I+1)
         IFR(I)=IFR(I+1)
210      ALB(I)=ALB(I+1)
      ELSE
         GO TO 590
      ENDIF
220   CONTINUE
      IF (IS.NE.ISURF(IGEN(IKK))) GO TO 590
230   IT=IT+IS
*
* REMOVE THE UNUSED SURFACE NUMBERS.
      DO 240 I=1,NCOUR*NMERGE
240   JF2(I)=0
      IJAT=0
      DO 250 I=1,IJAR
      J=JF1(I)
      IF (J.GT.NCOUR*NMERGE) GO TO 600
      IF (JF2(J).EQ.0) THEN
         IJAT=IJAT+1
         JF2(J)=IJAT
      ENDIF
250   CONTINUE
      DO 270 I=1,IJAR
      JF1(I)=JF2(JF1(I))
      IF (JF1(I).NE.I) GO TO 600
270   IFR(I)=JF2(IFR(I))
*
* PRINT THE SURFACE NUMBERS AFTER MERGING.
      IF (IMPX.GT.2) THEN
         WRITE (6,610)
         WRITE (6,650) ('----------------',I=1,NCOUR)
         IT0=0
         DO 390 IKK=1,NMERGE
         NC=ISURF(IGEN(IKK))
         FILLED=(NC.EQ.1).OR.((ITYPE.EQ.6).AND.(NC.EQ.3)).OR.
     1   ((ITYPE.LE.4).AND.(IAPP.GE.2).AND.(NC.EQ.2))
         IF (FILLED.AND.(IAPP.EQ.1)) THEN
            WRITE (6,620) IKK,IGEN(IKK),(DIRR(I+1),I=1,NC)
         ELSE IF (FILLED) THEN
            WRITE (6,620) IKK,IGEN(IKK),(DIR2(I+2),I=1,NC)
         ELSE IF (.NOT.FILLED.AND.(IAPP.EQ.1)) THEN
            WRITE (6,630) IKK,IGEN(IKK),(DIRR(I),I=1,NC)
         ELSE IF (.NOT.FILLED) THEN
            WRITE (6,630) IKK,IGEN(IKK),(DIR2(I),I=1,NC)
         ENDIF
         WRITE (6,635) (JF1(IT0+I),IFR(IT0+I),I=1,NC)
         WRITE (6,640) (ALB(IT0+I),I=1,NC)
         WRITE (6,650) ('----------------',I=1,NCOUR)
390      IT0=IT0+NC
      ENDIF
      RETURN
*
570   WRITE (6,610)
      WRITE (6,650) ('----------------',I=1,NCOUR)
      IT0=0
      DO 580 IKK=1,NMERGE
      NC=ISURF(IGEN(IKK))
      FILLED=(NC.EQ.1).OR.((ITYPE.EQ.6).AND.(NC.EQ.3)).OR.
     1 ((ITYPE.LE.4).AND.(IAPP.GE.2).AND.(NC.EQ.2))
      IF (FILLED.AND.(IAPP.EQ.1)) THEN
         WRITE (6,620) IKK,IGEN(IKK),(DIRR(I),I=1,NCOUR)
      ELSE IF (FILLED) THEN
         WRITE (6,620) IKK,IGEN(IKK),(DIR2(I),I=1,NCOUR)
      ELSE IF (.NOT.FILLED.AND.(IAPP.EQ.1)) THEN
         WRITE (6,630) IKK,IGEN(IKK),(DIRR(I),I=1,NCOUR)
      ELSE IF (.NOT.FILLED) THEN
         WRITE (6,630) IKK,IGEN(IKK),(DIR2(I),I=1,NCOUR)
      ENDIF
      WRITE (6,635) (JF1(IT0+I),IFR(IT0+I),I=1,NCOUR)
      WRITE (6,640) (ALB(IT0+I),I=1,NCOUR)
      WRITE (6,650) ('----------------',I=1,NCOUR)
580   IT0=IT0+NCOUR
      CALL XABORT('NUMERC: FAILURE OF THE SURFACE RENUMBERING ALGORITH'
     1 //'M.')
590   CALL XABORT('NUMERC: FAILURE OF THE ALGORITHM (1).')
600   CALL XABORT('NUMERC: FAILURE OF THE ALGORITHM (2).')
*
610   FORMAT (///17H SURFACE NUMBERS./)
620   FORMAT (7H BLOCK=,I5,5X,17HGENERATING BLOCK=,I5,5X,10H(FILLED CY,
     1 14HLINDER/SPHERE)/8H   SIDE:,6(A9,6X,1HI))
630   FORMAT (7H BLOCK=,I5,5X,17HGENERATING BLOCK=,I5,5X,10H(TUBE/SHEL,
     1 2HL)/8H   SIDE:,6(A9,6X,1HI))
635   FORMAT (8H IN/OUT:,6(I6,2H /,I5,3H  I))
640   FORMAT (8H ALBEDO:,1P,6(E13.5,3H  I))
650   FORMAT (8H -------,6(A16))
680   FORMAT (51HNUMERC: A GEOMETRICAL RECIPROCITY CONDITION IS VIOL,
     1 10HATED (SIDE,A3,20H OF GENERATING BLOCK,I5,8H VS SIDE,A3,
     2 20H OF GENERATING BLOCK,I5,3H ).)
      END
