*DECK PERMAC
      SUBROUTINE PERMAC(IPMACR,IPMACP,NBMIX ,NGROUP,
     >                  MXNL  ,NIFISS,MAXFIS,LOLXSP,
     >                  XSTOTR,XSTRAR,XSFISR,
     >                  XSFIXR,XSSCAR,XEFISR,
     >                  XSTOTP,XSTRAP,XSFISP,
     >                  XSFIXP,XSSCAP,XEFISP,
     >                  XSABSR,XSABSP)
C
C---------------------------  PERMAC  ---------------------------------
C
C   1- PROGRAMME STATISTICS:
C       NAME     : PERMAC
C       USE      : CROSS SECTIONS VARIATION COMPUTATION
C       AUTHOR   : T. COURAU (2000-08-01)
C
C   2- ROUTINE PARAMETERS:
C    INPUT
C       IPMACR   : POINTER TO THE REFERENCE  MACROLIB I
C       IPMACP   : POINTER TO THE PERTURBED         I
C                  MACROLIB
C       NBMIX    : MAXIMUM NUMBER OF MIXTURES         I
C       NGROUP   : NUMBER OF ENERGY GROUPS            I
C       MXNL     : MAXIMUM LEGENDRE ORDER             I
C                  = 1 ISOTROPIC COLLISION
C                  = 2 LINEARLY ANISOTROPIC COLLISION
C       NIFISS   : NUMBER OF FISSILE ISOTOPES         I
C       MAXFIS   : MAX(1,NIFISS)                      I
C   INPUT/OUTPUT
C       LOLXSP   : CHECK IF CROSS SECTION TYPE IS     L(NBXS)
C                  ALREADY PRESENT ON THE MACROLIB.
C                1 TOTAL, 2 NUSIGF, 3 FIXE, 4 CHI, 5  SCAD
C                6 TRANC, 7 FIXA, 8 FIXG, 9 NFTOT, 10 EFISS
C
C       XSTOT    : TOTAL CROSS SECTION OF MIXTURE     R(NBMIX,NGROUP)
C       XSABS    : ABS CROSS SECTION OF MIXTURE       R(NBMIX,NGROUP)
C       XSTRA    : TRANSPORT CORRECTION XS OF MIXTURE R(NBMIX,NGROUP)
C       XSFIS    : FISSION CROSS SECTION OF MIXTURE   R(NBMIX,
C                                                    MAXFIS,NGROUP,3)
C       XSFIX    : FIXE SOURCES                       R(NBMIX,
C                                                       NGROUP,3)
C       XSSCA    : SCATT X-SECTION OF MIXTURE/GROUP   R(NBMIX,MXNL,
C                                                      NGROUP,NGROUP)
C       XEFISS   : AVERAGE ENERGY PER FISSION         R(NBMIX,MAXFIS)
C   LOCAL PARAMETERS
C       INGSCT   : NB OF SCATTERING GROUP FOR  XS     I
C       IFGSCT   : FIRST SCATTERING GROUP FOR  XS     I
C       ISCATA   : CHECK FOR SCATTERING ANISOTROPY    I
C       IWORK    : WORK CROSS-SECTION VECTOR          I
C
C---------------------------   PERMAC  --------------------------------
C
      IMPLICIT    NONE
      INTEGER     IOUT,ILCMUP,ILCMDN,NBXS
      CHARACTER   NAMSBR*6
      PARAMETER  (IOUT=6,ILCMUP=1,ILCMDN=2,NBXS=10,NAMSBR='PERMAC')
C----
C  ROUTINE PARAMETERS
C----
      INTEGER     IPMACR,IPMACP,NBMIX,NGROUP,MXNL,NIFISS,MAXFIS
      REAL        XSTOTR(NBMIX,NGROUP),XSTRAR(NBMIX,NGROUP),
     >            XSFISR(NBMIX,MAXFIS,NGROUP,3),
     >            XSFIXR(NBMIX,NGROUP,3),
     >            XSSCAR(NGROUP,NBMIX,MXNL,NGROUP),
     >            XEFISR(NBMIX,MAXFIS),
     >            XSTOTP(NBMIX,NGROUP),XSTRAP(NBMIX,NGROUP),
     >            XSFISP(NBMIX,MAXFIS,NGROUP,3),
     >            XSFIXP(NBMIX,NGROUP,3),
     >            XSSCAP(NGROUP,NBMIX,MXNL,NGROUP),
     >            XEFISP(NBMIX,MAXFIS)
      REAL        XSABSR(NBMIX,NGROUP),XSABSP(NBMIX,NGROUP)
      LOGICAL     LOLXSP(NBXS)
C----
C  MEMORY ALLOCATION
C----
      INTEGER     IBASE(1)
      REAL        RBASE
      COMMON      RBASE(1)
      EQUIVALENCE(RBASE(1),IBASE(1))
      INTEGER     INGSCT,IFGSCT,ISCATA,IWORK
C----
C LOCAL PARAMETERS
C---
      INTEGER     IL,IMIX,IGR,JGR,IFISS
      LOGICAL     LOLXSR(NBXS)
      DO 100 IL=1,NBXS
        LOLXSR(IL)=.FALSE.
        LOLXSP(IL)=.FALSE.
 100  CONTINUE
C----
C REFERENCE MACROLIB READING
C----
      CALL SETARA(IBASE,MXNL,ISCATA)
      CALL XDISET(IBASE(ISCATA),MXNL,0)
      CALL SETARA(IBASE,NBMIX,INGSCT)
      CALL SETARA(IBASE,NBMIX,IFGSCT)
      CALL SETARA(RBASE,NBMIX*NGROUP,IWORK)
      CALL MACIXS(IPMACR,NBMIX ,NGROUP,MXNL  ,NIFISS,MAXFIS,LOLXSR,
     >            XSTOTR,XSTRAR,XSFISR,XSFIXR,XSSCAR,RBASE(IWORK),
     >            IBASE(INGSCT),IBASE(IFGSCT),IBASE(ISCATA),XEFISR)
      CALL PERABS(NBMIX ,NGROUP,MXNL,XSTOTR,XSSCAR,XSABSR)
C----
C PERTURBED MACROLIB READING
C----
      CALL XDISET(IBASE(ISCATA),MXNL,0)
      CALL MACIXS(IPMACP,NBMIX ,NGROUP,MXNL  ,NIFISS,MAXFIS,LOLXSP,
     >            XSTOTP,XSTRAP,XSFISP,XSFIXP,XSSCAP,RBASE(IWORK),
     >            IBASE(INGSCT),IBASE(IFGSCT),IBASE(ISCATA),XEFISP)
*----
*  ABSORPTION
*----
      CALL PERABS(NBMIX ,NGROUP,MXNL,XSTOTP,XSSCAP,XSABSP)
      CALL RLSARA(RBASE(IWORK))
      CALL RLSARA(IBASE(IFGSCT))
      CALL RLSARA(IBASE(INGSCT))
      CALL RLSARA(IBASE(ISCATA))
C---
C CHECK WHICH XS WILL BE TREATED
C--
      DO 200 IL=1,NBXS
        IF (IL.EQ.3 .OR. IL.EQ.7 .OR. IL.EQ. 8 .OR. IL.EQ.10) THEN
         LOLXSP(IL)=.FALSE.
         LOLXSR(IL)=.FALSE.
        ELSE
         IF (LOLXSP(IL) .NEQV. LOLXSR(IL)) CALL XABORT(NAMSBR//
     >  ': REFERENCE AND PERTURBED DATA INCOHERENCE')
         LOLXSR(IL)=.FALSE.
        ENDIF
 200  CONTINUE
      IF (.NOT.(LOLXSP(1).AND.LOLXSP(2).AND.LOLXSP(4).AND.LOLXSP(5)))
     >  CALL XABORT(NAMSBR//': REQUIRED XS ARE NOT PRESENT ON MACROLIB')
C---
C STORE DELTA MACR ON PERTUBATED XS DATA
C--
      DO 300 IMIX=1,NBMIX
        DO 400 IGR=1,NGROUP
         IF (LOLXSP(1)) THEN
           XSTOTP(IMIX,IGR)=XSTOTP(IMIX,IGR)-XSTOTR(IMIX,IGR)
           XSABSP(IMIX,IGR)=XSABSP(IMIX,IGR)-XSABSR(IMIX,IGR)
           LOLXSR(1)=(LOLXSR(1).OR.(XSTOTP(IMIX,IGR).NE.0.0))
         ENDIF
         IF (LOLXSP(6)) THEN
           XSTRAP(IMIX,IGR)=XSTRAP(IMIX,IGR)-XSTRAR(IMIX,IGR)
           LOLXSR(6)=(LOLXSR(6).OR.(XSTRAP(IMIX,IGR).NE.0.0))
         ENDIF
         DO 500 IFISS=1,NIFISS
          IF (LOLXSP(2)) THEN
             XSFISP(IMIX,IFISS,IGR,1)=XSFISP(IMIX,IFISS,IGR,1)
     >                               -XSFISR(IMIX,IFISS,IGR,1)
             LOLXSR(2)=(LOLXSR(2).OR.(XSFISP(IMIX,IFISS,IGR,1).NE.0.0))
          ENDIF
          IF (LOLXSP(9))THEN
             XSFISP(IMIX,IFISS,IGR,2)=XSFISP(IMIX,IFISS,IGR,2)
     >                               -XSFISR(IMIX,IFISS,IGR,2)
             LOLXSR(9)=(LOLXSR(9).OR.(XSFISP(IMIX,IFISS,IGR,2).NE.0.0))
          ENDIF
          IF (LOLXSP(4))THEN
             XSFISP(IMIX,IFISS,IGR,3)=XSFISP(IMIX,IFISS,IGR,3)
     >                               -XSFISR(IMIX,IFISS,IGR,3)
             LOLXSR(4)=(LOLXSR(4).OR.(XSFISP(IMIX,IFISS,IGR,3).NE.0.0))
          ENDIF
 500     CONTINUE
         IF (LOLXSP(5)) THEN
          IL=1
c$$$          DO 600 IL=1,MXNL
           DO 700 JGR=1,NGROUP
              XSSCAP(IGR,IMIX,IL,JGR)=XSSCAP(IGR,IMIX,IL,JGR)-
     >                                XSSCAR(IGR,IMIX,IL,JGR)
              LOLXSR(5)=(LOLXSR(5).OR.(XSSCAP(IGR,IMIX,IL,JGR).NE.0.0))
 700       CONTINUE
c$$$ 600      CONTINUE
         ENDIF
 400    CONTINUE
 300  CONTINUE
C---
C IF  THE XS IS PRESENT ON  BOTH MACROLIB
C AND DIFFERENT -> LOLXSP(IL)=.TRUE.
C---
      DO 800 IL=1,NBXS
         LOLXSP(IL)=(LOLXSP(IL).AND.LOLXSR(IL))
 800  CONTINUE
      RETURN
      END
