*DECK QIJRGL
      SUBROUTINE QIJRGL(NREG,NSOUT,NGRP,SIGTAL,PROB,
     >                  RI,RBARRE,GBARRE)
C
C-------------------------    QIJRGL    -------------------------------
C
C 1- SUBROUTINE STATISTICS:
C     NAME     : QIJRGL
C     LEVEL    : 4 (CALLED BY 'XL3TRK')
C     USE      : NORMALIZATION OF COLLISION PROBS USING GELBARD SCHEME
C     AUTHOR   : R. ROY (96-04-16)
C     REFERENCE: 'NORMALIZATION TECHNIQUES FOR CP MATRICES',
C                 R.ROY AND G.MARLEAU,
C                 CONF/PHYSOR-90, MARSEILLE/FRANCE, V 2, P IX-40 (1990).
C
C 2- PARAMETERS:
C  INPUT
C     NREG    : # OF ZONES FOR GEOMETRY.             I
C     NSOUT   : # OF SURFACES FOR GEOMETRY.          I
C     NGRP    : # OF ENERGY GROUPS.                  I
C     SIGTAL  : ALBEDO-SIGT VECTOR                   R(NGRP,NSOUT:NREG)
C     PROB    : -CP- MATRIX FOR ALL TYPES.           R(NGRP,NPRB)
C               NPRB=(-NSOUT+NREG+1)*(-NSOUT+NREG+2)/2
C  OUTPUT
C     PROB    : -CP- MATRIX FOR ALL TYPES.           R(NGRP,NPRB)
C  WORK
C     RI      : GENERALISED NORMALIZATION FACTOR     R(NGRP,NSOUT:NREG)
C
C-------------------------    QIJRGL    -------------------------------
C
      IMPLICIT   NONE
      INTEGER    NREG,NSOUT,NGRP,
     >           IPRB,IPRF,IUNK,JUNK,IVOL,JVOL,IR,JR,IG
      DOUBLE PRECISION PROB(NGRP,*),
     >           RI(NGRP,NSOUT:NREG),RBARRE(NGRP),GBARRE(NGRP)
      REAL       SIGTAL(NGRP,NSOUT:NREG)
      DO 5 IG= 1, NGRP
         RBARRE(IG)=0.0D0
         GBARRE(IG)=0.0D0
    5 CONTINUE
      IPRB= 0
      IUNK= 0
      IVOL= -NSOUT*(1-NSOUT)/2
C
C     COMPUTE R-SUB(I) FACTORS AND: GBARRE, RBARRE
      DO 100 IR=NSOUT, NREG
         IUNK= IUNK+1
         DO 10 IG= 1, NGRP
            RI(IG,IR)=0.0D0
   10    CONTINUE
         DO 30 JR=NSOUT, IR
            IPRB= IPRB+1
            IF( JR .LT. 0) THEN
               DO 20 IG= 1, NGRP
                  RI(IG,IR)=RI(IG,IR)+PROB(IG,IPRB)
   20          CONTINUE
            ELSE IF( JR .GT. 0) THEN
               DO 21 IG= 1, NGRP
                 IF(SIGTAL(IG,JR).GT.0.0 )THEN
                  RI(IG,IR)=RI(IG,IR)+PROB(IG,IPRB)
                 ENDIF
   21          CONTINUE
            ENDIF
   30    CONTINUE
         IPRF= IPRB
         JUNK= IUNK
         DO 50 JR= IR+1 , NREG
            IPRF= IPRF+JUNK
            JUNK= JUNK+1
            IF( JR .LT. 0)THEN
               DO 40 IG= 1, NGRP
                  RI(IG,IR)=RI(IG,IR)+PROB(IG,IPRF)
   40          CONTINUE
            ELSE IF( JR .GT. 0) THEN
               DO 41 IG= 1, NGRP
                 IF(SIGTAL(IG,JR).GT.0.0 )THEN
                  RI(IG,IR)=RI(IG,IR)+PROB(IG,IPRF)
                 ENDIF
   41          CONTINUE
            ENDIF
   50    CONTINUE
         IF( IR.LT.0 )THEN
            IVOL= IVOL+1
            DO 60 IG= 1, NGRP
               RI(IG,IR)= PROB(IG,IVOL)-RI(IG,IR)
               GBARRE(IG)= GBARRE(IG)+PROB(IG,IVOL)
               RBARRE(IG)= RBARRE(IG)+RI(IG,IR)
   60       CONTINUE
         ELSEIF( IR.GT.0 )THEN
            IVOL= IVOL+IUNK-1
            DO 70 IG= 1, NGRP
               RI(IG,IR)= PROB(IG,IVOL)-RI(IG,IR)
               IF( SIGTAL(IG,IR).GT.0.0 )THEN
                  GBARRE(IG)= GBARRE(IG)+PROB(IG,IVOL)
                  RBARRE(IG)= RBARRE(IG)+RI(IG,IR)
               ENDIF
   70       CONTINUE
         ELSE
            IVOL= IVOL+1
            DO 80 IG= 1, NGRP
               RI(IG,IR)=0.0D0
   80       CONTINUE
         ENDIF
  100 CONTINUE
      DO 110 IG=1, NGRP
         GBARRE(IG)=1.0D0/GBARRE(IG)
         RBARRE(IG)=RBARRE(IG)*GBARRE(IG)
  110 CONTINUE
C
C     RENORMALIZE PROB MATRIX
      IVOL= -NSOUT*(1-NSOUT)/2
      IPRB= 0
      IUNK= 0
      DO 210 IR   = NSOUT, NREG
         IF( IR.LE.0 )THEN
            IVOL= IVOL+1
         ELSE
            IVOL= IVOL+IUNK
         ENDIF
         IUNK= IUNK+1
         JVOL= -NSOUT*(1-NSOUT)/2
         JUNK= 0
         DO 200 JR= NSOUT, IR
            IF( JR.LE.0 )THEN
               JVOL= JVOL+1
            ELSE
               JVOL= JVOL+JUNK
            ENDIF
            JUNK= JUNK+1
            IPRB= IPRB+1
            IF( IR.NE.0.AND.JR.NE.0 )THEN
               DO 190 IG= 1, NGRP
                  PROB(IG,IPRB)= PROB(IG,IPRB)+
     1              (PROB(IG,JVOL)*RI(IG,IR)+PROB(IG,IVOL)*RI(IG,JR)
     2              -PROB(IG,IVOL)*PROB(IG,JVOL)*RBARRE(IG))*GBARRE(IG)
 190           CONTINUE
           ENDIF
 200     CONTINUE
 210  CONTINUE
      RETURN
      END
