*DECK QIJSBC
      SUBROUTINE QIJSBC(IPSYS,NREG,NSOUT,NPRB,NGRP,ITYPE,SIGTAL,PROB,
     >                  PSVT ,PSVF,LGRP ,NGRT)
C
C-------------------------    QIJSBC    -------------------------------
C
C 1- SUBROUTINE STATISTICS:
C     NAME     : QIJSBC
C     LEVEL    : 2 (CALLED BY 'XL3TRK')
C     USE      : THIS ROUTINE RECONSTRUCT EXCAPE PROBABILITIES (Psv)
C                FOR  ALL ZONES ELIMINATING INCLUDING BOUNDARY
c                CONDITIONS
C     CREATED  : 2003/03/19
C     AUTHOR   : P. Dufour
C
C 2- PARAMETERS:
C  INPUT
C     IPSYS   : POINTER TO THE PIJ data structure    I
C     NREG    : # OF ZONES FOR GEOMETRY.             I
C     NSOUT   : # OF SURFACES FOR GEOMETRY.          I
C     NPRB    : NUMBER OF PROBABILITIES IN PROB      I
C     NGRP    : NUMBER OF ENERGY GROUPS              I
C     ITYPE   : BC type. If ITYPE=1, use BC
C               if ITYPE=0 do not use BC             I
C     SIGTAL  : ALBEDO-SIGT VECTOR                   R(NGRP,
C                                                      -NSOUT:NREG)
C     PROB    : -CP- MATRIX FOR ALL TYPES.           R(NGRP,NPRB)
C     PSVT    : WORKING AREA FOR PSVT                D(NGRP,
C               PSVT=PSST*PSV                          NSOUT,NREG)
C     LGRP    : exact group number
C     NGRT    : total number of group
C  OUTPUT    
C     PSVF    : Complete PSV matrix for surface flux D(NSOUT,NREG,
C               calculation                            NGRP)
C
C  3-EXTERNAL ROUTINE CALLED
C     XABORT  : DRAGON ABORT ROUTINE
C
C----------------------------------------------------------------------
      IMPLICIT         NONE
      INTEGER          ILCMUP,ILCMDN
      PARAMETER       (ILCMUP=1,ILCMDN=2)
C----
C  ROUTINE PARAMETERS
C-----
      INTEGER          IPSYS,NREG,NSOUT,NPRB,NGRP,ITYPE
      REAL             SIGTAL(NGRP,-NSOUT:NREG)
      DOUBLE PRECISION PROB(NGRP,NPRB)
      DOUBLE PRECISION PSVT(NGRP,NSOUT,NREG),PSVF(NSOUT,NREG,NGRP)
      INTEGER          LGRP(NGRP),NGRT
C---- 
C  VARIABLES LOCALES
C----
      INTEGER          NSP1,IPSS,ISUR,ISX,IS,JSX,JS,IER,
     >                 IV,JV,IG,IVSI,IVVI,ISV,IVS,IVV,ISS,ISSI
      CHARACTER        CGRPNM*12
C----
C    [PSVF]= [PSS][PSVT]+[PSV]
C----
      NSP1=NSOUT+1
      DO 200 IG=1,NGRP
         IVSI=(NSP1*(NSP1+1))/2
         IVVI=IVSI+NSP1
         IF (ITYPE .EQ. 1) THEN
            DO 110 IV=1,NREG
               ISSI=1
               DO 112 IS=1,NSOUT
                  PSVF(NSP1-IS,IV,IG)=0.0D0
                  ISSI=ISSI+IS-1
                  ISS=ISSI
                  DO 120 JS=1,NSOUT
                     PSVF(NSP1-IS,IV,IG)=PSVF(NSP1-IS,IV,IG)+
     >                                   (PROB(IG,ISS)*PSVT(IG,JS,IV))
                     IF(JS.LT.IS) THEN
                        ISS=ISS+1
                     ELSE
                        ISS=ISS+JS
                     ENDIF
 120              CONTINUE
                  ISV=IVSI+IS
                  PSVF(NSP1-IS,IV,IG)=PSVF(NSP1-IS,IV,IG)+PROB(IG,ISV)
 112           CONTINUE
               IVSI=IVSI+NSP1+IV
 110        CONTINUE
         ELSE
            DO 100 IV=1,NREG
               ISSI=1
               DO 102 IS=1,NSOUT
                  ISV=IVSI+IS
                  PSVF(NSP1-IS,IV,IG)=PROB(IG,ISV)
 102           CONTINUE
               IVSI=IVSI+NSP1+IV
 100        CONTINUE
         ENDIF
C----
C    [PSVF]=[S^(-1)][PSVF][SIGTOT^(-1)]
C----
         ISSI=(NSOUT*NSP1)/2
         DO 130 IS=1,NSOUT
            ISS=ISSI+IS
            DO 131 IV=1,NREG
               IF(SIGTAL(IG,IV).EQ.0.0) THEN
                  PSVF(NSP1-IS,IV,IG)=(1/PROB(IG,ISS))*
     >                                PSVF(NSP1-IS,IV,IG)
               ELSE
                  PSVF(NSP1-IS,IV,IG)=(1/PROB(IG,ISS))*
     >                                (1/SIGTAL(IG,IV))*
     >                                PSVF(NSP1-IS,IV,IG)
               ENDIF
 131        CONTINUE
 130     CONTINUE
C----
C    TRANSCRIPTION DE PSVF
C----
         WRITE(CGRPNM,1000) LGRP(IG),NGRT
         CALL LCMSIX(IPSYS,CGRPNM,ILCMUP)
         CALL LCMPUT(IPSYS,'DRAGON-PCFUI',2*NREG*NSOUT,4,PSVF(1,1,IG))
         CALL LCMSIX(IPSYS,CGRPNM,ILCMDN)
 200  CONTINUE

      RETURN
C----
C    FORMAT
C----
 1000 FORMAT('GROUP',I3,'/',I3)
      
      END
