*DECK READBH
      SUBROUTINE READBH (MAXPTS,IPGEOM,IR,IR2,IPAS,IPAS2,MAT,VOL,NG,
     1 NSMAX,MICRO,NS,IBI,RS,FRACT,VOLK,IMPX,MILIEU,IDIL,ITEMP,FTEMP)
*
*-----------------------------------------------------------------------
*
* RECOVER THE INPUT DATA FOR THE DOUBLE HETEROGENEITY OPTION (BIHET).
*
* INPUT PARAMETERS:
*   MAXPTS : ALLOCATED STORAGE FOR ARRAYS OF DIMENSION IPAS.
*   IPGEOM : POINTER TO THE GEOMETRY (L_GEOM SIGNATURE).
*   IR2    : NUMBER OF ORDINARY AND COMPOSITE MIXTURES (A MIXTURE THAT
*            INCLUDE A MICRO STRUCTURE).
*   IPAS2  : NUMBER OF VOLUMES IN THE MACRO GEOMETRY;
*   VOL    : VOLUMES OF THE MACRO GEOMETRY. DIMENSION VOL(IPAS2)
*   NG     : NUMBER OF DIFFERENT KIND OF MICRO STRUCTURES. A KIND OF
*            MICRO STRUCTURE IS CHARACTERIZED BY THE RADIUS OF ITS
*            TUBES OR SHELLS. ALL THE MICRO VOLUMES OF THE SAME KIND
*            SHOULD OWN THE SAME NUCLEAR PROPERTIES IN A GIVEN VOLUME
*            OF THE MACRO GEOMETRY.
*   NSMAX  : MAXIMUM NUMBER OF VOLUMES (TUBES OR SHELLS) IN EACH KIND
*            OF MICRO STRUCTURE).
*   IBI    : TYPE OF COMPOSITE MIXTURE IN EACH VOLUME OF THE MACRO
*            GEOMETRY. IF IBI(IKK) IS GREATER THAN IR, THE VOLUME IKK
*            CONTAIN A MICRO STRUCTURE. DIMENSION IBI(IPAS2)
*   IMPX   : PRINT FLAG. EQUAL TO ZERO FOR NO PRINT.
*
* OUTPUT PARAMETERS:
*   IR     : NUMBER OF ORDINARY MIXTURES.
*   IPAS   : NUMBER OF VOLUMES IN THE COMPOSITE GEOMETRY.
*   MAT    : INDEX-NUMBER OF THE MIXTURE TYPE ASSIGNED TO EACH VOLUME
*            OF THE COMPOSITE GEOMETRY. DIMENSION MAT(IPAS)
*   VOL    : VOLUMES OF THE COMPOSITE GEOMETRY. DIMENSION VOL(IPAS)
*   MICRO  : TYPE OF MICRO VOLUMES: =3: CYLINDER ; =4: SPHERE.
*   NS     : NUMBER OF TUBES OR SHELLS IN EACH KIND OF MICRO STRUCTURE.
*            DIMENSION NS(NG)
*   RS       RADIUS OF THE MICRO VOLUMES. DIMENSION RS((1+NSMAX)*NG)
*   FRACT  : VOLUMIC FRACTIONS OF EACH TYPE OF MICRO VOLUMES IN EACH
*            ORDINARY OR COMPOSITE MIXTURE. DIMENSION FRACT(NG*IR2)
*   VOLK   : VOLUMIC FRACTIONS OF THE TUBES OR SHELLS IN THE MICRO
*            VOLUMES. DIMENSION VOLK(NG*NSMAX)
*
* SCRATCH STORAGE.
*  MILIEU  : DIMENSION MILIEU(IR2)
*  IDIL    : DIMENSION IDIL(IR2)
*  ITEMP   : DIMENSION ITEMP(NSMAX*NG*IR2)
*  FTEMP   : DIMENSION FTEMP(IR2)
*
*---------------------------------------- AUTHOR: A. HEBERT 20/02/87 ---
*
      CHARACTER GEONAM*12,HSMG*131,NAMT*12,TEXT12*12
      LOGICAL LCM
      DIMENSION MAT(*),VOL(*),NS(*),IBI(*),RS(NSMAX+1,*),FRACT(NG,*),
     1 MILIEU(*),IDIL(*),VOLK(NG,*),ITEMP(NSMAX,NG,*),FTEMP(*)
*
      NAMT=' '
      CALL LCMNXT(IPGEOM,NAMT,GEONAM,TEXT12,LCM)
      CALL LCMGET(IPGEOM,'MICRO',MICRO)
      CALL LCMGET(IPGEOM,'NS',NS)
      CALL LCMGET(IPGEOM,'RS',RS)
      CALL LCMGET(IPGEOM,'FRACT',FRACT)
      CALL LCMLEN(IPGEOM,'MILIE',NMILG,ITYLCM)
      IF (NMILG.GT.IR2) CALL XABORT('READBH: INVALID VALUE FOR IR2.')
      CALL LCMGET(IPGEOM,'MILIE',MILIEU)
      CALL LCMGET(IPGEOM,'MIXDIL',IDIL)
      CALL LCMGET(IPGEOM,'MIXGR',ITEMP)
*
      DO 20 J=1,NG
      FACT=RS(NS(J)+1,J)**(MICRO-1)
      W=0.0
      DO 20 K=1,NS(J)
      ZZZ=RS(K+1,J)**(MICRO-1)/FACT
      VOLK(J,K)=ZZZ-W
   20 W=ZZZ
*
      IND1=IR2
      DO 40 I=1,NMILG
      IF (MILIEU(I).GT.IR2) THEN
         WRITE (HSMG,390) MILIEU(I),IR2
         CALL XABORT(HSMG)
      ENDIF
   40 IND1=MIN(IND1,MILIEU(I))
*
* SET-UP THE NEW VOLUMES.
      IPAS=IPAS2
      DO 90 IKK=1,IPAS2
      MAT(IKK)=IBI(IKK)
      IF (IBI(IKK).GE.IND1) THEN
         IND=0
         DO 50 I=1,NMILG
         IF (MILIEU(I).EQ.IBI(IKK)) IND=I
   50    CONTINUE
         IF (IND.EQ.0) CALL XABORT('READBH: A COMPOSITE MIXTURE IS NOT'
     1   //' DEFINED.')
         DILF=1.0
         DO 60 J=1,NG
   60    DILF=DILF-FRACT(J,IND)
         VHET=VOL(IKK)
         MAT(IKK)=IDIL(IND)
         VOL(IKK)=VHET*DILF
         DO 80 J=1,NG
         FRT=FRACT(J,IND)
         IF (FRT.GT.0.00001) THEN
            FACT=RS(NS(J)+1,J)**(MICRO-1)
            W=0.0
            DO 70 K=1,NS(J)
            ZZZ=RS(K+1,J)**(MICRO-1)/FACT
            IPAS=IPAS+1
            MAT(IPAS)=ITEMP(K,J,IND)
            VOL(IPAS)=VHET*FRT*(ZZZ-W)
   70       W=ZZZ
          ENDIF
   80     CONTINUE
      ENDIF
   90 CONTINUE
      IF (IPAS.GT.MAXPTS) CALL XABORT('READBH: MAXPTS IS TOO SMALL.')
      IR=0
      DO 100 I=1,IPAS
  100 IR=MAX(IR,MAT(I))
      IF (IR+1.GT.IR2) CALL XABORT('READBH: INVALID MIX NUMBERS.')
      DO 130 J=1,NG
      DO 110 IND=1,NMILG
  110 FTEMP(IND)=FRACT(J,IND)
      DO 120 IND=1,IR2
  120 FRACT(J,IND)=0.0
      DO 130 IND=1,NMILG
  130 FRACT(J,MILIEU(IND))=FTEMP(IND)
*
      IF (IMPX.GE.1) THEN
         WRITE (6,300) GEONAM
         IF (MICRO.EQ.3) THEN
            WRITE (6,'(44H THE MICRO STRUCTURE IS MADE OF TUBES OR CYL,
     1      7HINDERS./)')
         ELSE IF (MICRO.EQ.4) THEN
            WRITE (6,'(44H THE MICRO STRUCTURE IS MADE OF SPHERES OR S,
     1      16HPHERICAL SHELLS./)')
         ENDIF
         WRITE (6,360) IPAS2,NG,NSMAX
         WRITE (6,370) IR+1,IR2
         DO 140 J=1,NG
         WRITE (6,310) J
         WRITE (6,320) (RS(K,J),K=1,NS(J)+1)
         WRITE (6,330)
         WRITE (6,320) (FRACT(J,IBI(IKK)),IKK=1,IPAS2)
  140    CONTINUE
         WRITE (6,'(///)')
         WRITE (6,400) IPAS,MAXPTS,IR
      ENDIF
      RETURN
*
  300 FORMAT (///50H BIHET: INTRODUCTION OF A MICRO STRUCTURE IN THE M,
     1 26HACRO GEOMETRY LOCATED IN ',A12,2H'./)
  310 FORMAT (//23H MICRO STRUCTURE NUMBER,I4//20H RADIUS OF THE MICRO,
     1 17H TUBES OR SHELLS:)
  320 FORMAT (1X,1P,10E12.5)
  330 FORMAT (/53H VOLUMIC CONCENTRATIONS OF THE MICRO STRUCTURE IN EAC,
     1 31HH VOLUME OF THE MACRO GEOMETRY:)
  360 FORMAT (/42H NUMBER OF VOLUMES IN THE MACRO GEOMETRY =,I5/
     1         38H NUMBER OF KINDS OF MICRO STRUCTURES =,I4/
     2         49H MAXIMUM NUMBER OF VOLUMES IN A MICRO STRUCTURE =,I4/)
  370 FORMAT (/51H THE INDEX-NUMBERS OF THE MIXTURES WITH A MICRO STR,
     1 21HUCTURE VARIES BETWEEN,I4,4H AND,I4,1H.)
  390 FORMAT (34HREADBH: THE INPUT MIXTURE NUMBER (,I4,12H) IS GREATER,
     1 10H THAN IR (,I4,2H ))
  400 FORMAT (/20H NUMBER OF VOLUMES =,I4,5X,22HAVAILABLE STORAGE: MAX,
     1 5HBLK =,I4/21H NUMBER OF MIXTURES =,I4/)
      END
