*DECK SADMCL
      SUBROUTINE SADMCL(IPFLUX,IPMACR,IPTRK ,IPBTF,NUNKNO,IPRINT,NFUNL ,
     >                  NGROUP,NIFISS,NBMIX ,NREG ,MXNL  ,LEAKSW,ITRANC,
     >                  TITLE ,ITPIJ ,MATCOD,VOLUME,
     >                  KEYFLX,FUNKNO,SUNKNO,SIGMA ,CORMAT,
     >                  ITYPEC,ILEAK ,ICOPAR,EPSPAR,
     >                  LMOC  ,NGSS  ,ITRB  ,NL    ,NBN   ,
     >                  NANIS ,LCACA ,LEXIN ,ILK   ,NALB  ,FLUXO ,
     >                  ADJFLO,DIFFB1,VOLINV,XSOLD ,XSSIGT,XTINV ,
     >                  XSSIGW,XSCHI ,XSNUF ,XSTK  ,XSCAT ,VSCAT ,
     >                  ALBEDO,NJJ   ,IJJ   ,IPOS  , 
     >                  LMODE ,FLUX)
C
C---------------------------  SADMCL  ---------------------------------
C
C 1- PROGRAMME STATISTICS:
C
C     NAME     :  SADMCL
C     USE      :  COMPUTE LINEARIZED CORRECTION VECTORS BY MOCC METHOD
C     AUTHOR   : M. Assawaroongruengchot 2007/01/01
C
C 2- ROUTINE PARAMETERS:
C   INPUT
C     IPFLUX   : POINTER TO THE FLUX SOLUTION            I
C     IPTRK    : POINTER TO THE TRACKING                 I
C     IPMACR   : POINTER TO THE MACROLIB                 I
C     IPBTF    : BINARY TRACKING FILE                    I
C     NUNKNO   : NUMBER OF UNKNOWNS                      I
C     IPRINT   : PRINT SELECTION FOR SAD MODULE:         I
C     NGROUP   : NUMBER OF GROUPS                        I
C     NIFISS   : NUMBER OF FISSILE ISOTOPES              I
C     NBMIX    : NUMBER OF MIXTURES                      I
C     NREG     : NUMBER OF REGIONS                       I
C     NL       : NUMBER OF LEGENDRE ORDERS               I
C     LEAKSW   : LEAKAGE SWITCH                          L
C     ITRANC   : TYPE OF TRANSPORT CORRECTION            I
C     TITLE    : TITLE OF THE DRAGON CASE.               C*72
C     LNORM    : SWITCH FOR REMOVING LEAKAGE FROM CP     L
C     LALBS    : SWITCH FOR KEEPING THE PIS              L
C     LR2      : =.TRUE. TO STORE INFORMATION REQUIRED   L
C     CNAME    : NAME USED TO SAVE ADDITIONAL PIJ DATA   C*12
C     ITPIJ    : TYPE OF CP AVAILABLE                    I
C     MATCOD   : MATERIAL PER REGION                     I(NREG)
C     VOLUME   : VOLUME OF REGION                        R(NREG)
C     KEYFLX   : AVERAGE FLUX POSITION PER REGION        I(NREG)
C   OUTUT
C     CORMAT   : CORRECTION MATRIX                       R(NREG,NBMIX,
C                                                          NGROUP)
C   SCRATCH STORAGE
C     FUNKNO   : UNKNOWN VECTOR SOLVED FOR.              R(NUNKNO,
C                                                          NGROUP)
C     SUNKNO   : SOURCE FOR SYSTEM OF UNKNOWN.           R(NUNKNO,
C                                                          NGROUP)
C     SIGMA    : STORE XS TOT AND TRANC                  R(NBMIX,2)
C
C---------------------------   SADMCL  --------------------------------
C
      IMPLICIT NONE
C----
C PARAMETERS
C----
      INTEGER     IOUT,ILCMUP,ILCMDN,NBEPS
      CHARACTER   NAMSBR*6
      PARAMETER  (IOUT=6,NBEPS=5,ILCMUP=1,ILCMDN=2,NAMSBR='SADMCL')
C----
C MEMORY ALLOCATION
C----
      INTEGER      IBASE(1)
      REAL         RBASE(1)
      COMMON       RBASE
      EQUIVALENCE (RBASE(1),IBASE(1))
      INTEGER      IQTOTL,IXSCHI,IXSSCAT,ISIGS0,IJJ1,NJJ1,IPOS1
C---
C INPUT/OUTPUT VARIABLES
C---
      INTEGER  IPFLUX,IPSYS,IPMACR,IPTRK,IPBTF,NREG,NUNKNO,IPRINT,
     >         IREG,NIFISS,ITPIJ,NFUNL,
     >         NBMIX,MXNL,ITRANC,NGROUP,
     >         MATCOD(NREG),KEYFLX(NREG)
      LOGICAL  LEAKSW
      CHARACTER TITLE*72
      REAL     VOLUME(NREG),SIGMA(NBMIX,2),
     >         FUNKNO(NUNKNO,NGROUP), SUNKNO(NUNKNO,NGROUP),
     >         CORMAT(NREG,NBMIX,NGROUP),
     >         SFUNKN(NUNKNO,NGROUP)
C----
C LOCAL VARIABLES
C----
      INTEGER IXSSIT,IXSSI0,IPSYSM,IPSYSR,IPMAD,ILCMLN,ILCMTY,IGR,JGR,
     >        JMIX,IBM,ISCATP,IMAT,IS,IANI,IFIS,IXSS,IA
      REAL    ALFACOR,KEFF
      CHARACTER CGRPNM*12,NAMFLX*8
      DOUBLE PRECISION FACTNO,FISOUR
      INTEGER     ICOPAR(6)
      DOUBLE PRECISION REFKEF,FLUX(NFUNL,NREG,NGROUP,8)
      REAL        EPSPAR(NBEPS)
      INTEGER     NJJ(NBMIX),IJJ(NBMIX),IPOS(NBMIX)
      REAL        FLUXO(NREG,NFUNL,NGROUP),ADJFLO(NREG,NFUNL,NGROUP)
      REAL        RKEFF
      DOUBLE PRECISION DZERO, DONE
      PARAMETER (DZERO=0.00000000000000D+00,DONE=1.00000000000000D+0)
      REAL        ZERO, ONE
      PARAMETER (ZERO=0., ONE=1. )
      REAL      AIEPS,AEEPS  
      LOGICAL   LMOC,LDRFLX,LDAJCB,LEXIN,ILK,ILKO,LMODE(20)
      CHARACTER CDIRFL*12,CADJFL(2)*12,CFIXSR*12
      INTEGER   NGSS,ITRB,NL,NBN,NANIS,LCACA,NALB,IMADJ
      INTEGER   ITYLCM,ILNLCM,IFRITR,IACITR,NINTMX,NEXTMX,
     >          IPTRK2,IFTRK2,NGSS2,ITYPEC,ILEAK
      REAL        VOLINV(NREG),DIFFB1(NGROUP),XSOLD(0:NBMIX,NGROUP),
     >            XSSIGT(0:NBMIX,NGROUP),XTINV(0:NBMIX,NGROUP),
     >            XSSIGW(0:NBMIX,NGROUP),XSCHI(0:NBMIX,NIFISS,NGROUP),
     >            XSNUF(0:NBMIX,NIFISS,NGROUP),XSTK(NBMIX,NIFISS),
     >            XSCAT(NGROUP,0:NBMIX,0:NANIS,NGROUP),
     >            VSCAT(NBMIX*NGROUP*NGROUP),
     >            ALBEDO(NALB)
      CHARACTER*2 CAN(0:9)
      DATA CAN /' 0',' 1',' 2',' 3',' 4',' 5',' 6',' 7',' 8',' 9'/
          IF(ITYPEC.NE.1 ) CALL XABORT('SADMCL:INVALID ITYPEC FOR '//
     >             'TOTAL DIRECT SOURCE CALCULATION')

      IF ( .NOT. LMODE(4)  .OR.  .NOT. LMOC  )  RETURN
C---  DIRECTORY ---
      CFIXSR   ='FIXA        '    
      CDIRFL   ='FLUXDIRECT  '
      CADJFL(1)='FLUXADJOINT '
      CADJFL(2)='FLUXADJ     '
C---  MOC VARIABLE INITIALIZATION ---
      IFRITR = ICOPAR(1)
      IACITR = ICOPAR(2)
      NINTMX = ICOPAR(4)
      NEXTMX = ICOPAR(5)
      AIEPS  = EPSPAR(1)
      AEEPS  = EPSPAR(2)
C---  THE DOUBLE TRACKING FILE IS DISABLE ---
      IPTRK2 = 0
      IFTRK2 = 0
      NGSS2 = 0
C---  ENABLE REBALANCE ACC. FOR DIRECT AND ADJOINT CALCULATION  ---
C     THE ORIGINAL REBAL STATUS IS RESTORED AT THE END OF SADMCR---
C     *** ILK == LREBAL *** 
      ILKO = ILK
      ILK  = .TRUE.
      CALL LCMLEN(IPFLUX,'FLUX1-CORR  ',ILCMLN,ILCMTY)
      IF(IPRINT .GE. 1000) THEN
        WRITE(IOUT,6000) NAMSBR
        WRITE(IOUT,6001)
        WRITE(IOUT,6002) ILCMLN,ILCMTY
      ENDIF
C---  VERIFY THE CORRECTION MATRIX DIRECTORY IN IPFLUX ---------------------
C     --- IF DIRECTORY EXIST, ILCMLN = -1 AND ILCMTY = 0  
C         , THEN CHK SUBDIRECTORY ---
      IF (ILCMLN .EQ. -1) THEN
C     --- CHANGE DIRECTORY TO CP DIRECTORY IN IPFLUX ---
      CALL LCMSIX(IPFLUX,'FLUX1-CORR  ',ILCMUP)
       DO 101 IGR=1,NGROUP
         WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGROUP
         IF( IPRINT.GE.1000) WRITE(IOUT,6003) CGRPNM
C        --- VERIFY THE CORRECTION MATRIX DIRECTORY IN IPFLUX ---
         CALL LCMLEN(IPFLUX,CGRPNM,ILCMLN,ILCMTY)
         IF (ILCMLN.NE.NREG*NBMIX .OR. ILCMTY.NE.2) 
     >       GO TO 1000
101    CONTINUE
C     --- GO TO MAIN DIRECTORY IN IPFLUX ---
      CALL LCMSIX(IPFLUX,'FLUX1-CORR  ',ILCMDN)
      IF( IPRINT.GE.1000) WRITE(IOUT,6004) 
C     --- RETURN AS THE CP DIRECTORY EXIST --- 
         RETURN
      ENDIF
1000  CONTINUE
C--- END VERIFY THE CORRECTION MATRIX DIRECTORY IN IPFLUX --------------------- 
      IF( IPRINT.GE.1000) WRITE(IOUT,6005) 
C----
C UPLOAD CROSSSECTIONS FOR DIRECT SOURCE EVALUATION
C----
      DO 150 IGR=1,NGROUP
        WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGROUP
        CALL LCMSIX(IPMACR,CGRPNM,ILCMUP)
        CALL LCMLEN(IPMACR,'TOTAL',ILNLCM,ITYLCM)
        IF(ILNLCM.GT.0) THEN
          CALL LCMGET(IPMACR,'TOTAL',XSSIGT(1,IGR))
          XSSIGT(0,IGR)=ZERO
          DO 111 IMAT= 0, NBMIX
 111      XSOLD(IMAT,IGR)=XSSIGT(IMAT,IGR)
        ELSE
          CALL LCMLIB(IPMACR)
          CALL XABORT('MOCDRV: READ ERROR ON LCM RECORD= TOTAL')
        ENDIF
        CALL LCMLEN(IPMACR,'SIGW 0',ILNLCM,ITYLCM)
        IF(ILNLCM.GT.0) THEN
          CALL LCMGET(IPMACR,'SIGW 0',XSSIGW(1,IGR))
          XSSIGW(0,IGR)=ZERO
        ELSE
          DO 110 IMAT=0,NBMIX
 110      XSSIGW(IMAT,IGR)=ZERO
        ENDIF
        IF(ITRANC.NE.0) THEN
*         TRANSPORT CORRECTION.
          CALL LCMLEN(IPMACR,'TRANC',ILNLCM,ITYLCM)
          IF(ILNLCM.GT.0) THEN
            CALL LCMGET(IPMACR,'TRANC',VSCAT)
            DO 120 IMAT=1,NBMIX
              XSSIGT(IMAT,IGR)=XSSIGT(IMAT,IGR)-VSCAT(IMAT)
              XSSIGW(IMAT,IGR)=XSSIGW(IMAT,IGR)-VSCAT(IMAT)
 120        CONTINUE
          ENDIF
        ENDIF
        XTINV(0,IGR)= ONE
        DO 121 IMAT=1,NBMIX
           IF( XSSIGT(IMAT,IGR).NE.ZERO )THEN
              XTINV(IMAT,IGR)= ONE/XSSIGT(IMAT,IGR)
           ELSE
              XTINV(IMAT,IGR)= ONE
           ENDIF
 121    CONTINUE
        IF( ITYPEC .EQ. 6 ) THEN      
           XSNUF(0,1,IGR)= ZERO
           CALL LCMGET(IPMACR,'FIXE',XSNUF(1,1,IGR))
        ELSE
           CALL LCMGET(IPMACR,'CHI',XSTK)
           DO 132 IFIS= 1, NIFISS
              XSCHI(0,IFIS,IGR)= ZERO
              DO 131 IMAT= 1, NBMIX
                 XSCHI(IMAT,IFIS,IGR)= XSTK(IMAT,IFIS)
  131         CONTINUE
  132      CONTINUE
           CALL LCMGET(IPMACR,'NUSIGF',XSTK)
           DO 134 IFIS= 1, NIFISS
              XSNUF(0,IFIS,IGR)= ZERO
              DO 133 IMAT= 1, NBMIX
                 XSNUF(IMAT,IFIS,IGR)= XSTK(IMAT,IFIS)
  133         CONTINUE
  134      CONTINUE
        ENDIF
        DO 50 IANI=0,NANIS
          CALL LCMLEN(IPMACR,'NJJD'//CAN(IANI),ILNLCM,ITYLCM)
          DO 39 IMAT= 0, NBMIX
            DO 38 JGR= 1, NGROUP
              XSCAT(JGR,IMAT,IANI,IGR)= ZERO
   38       CONTINUE
   39     CONTINUE
          IF(ILNLCM.GT.0) THEN
            CALL LCMGET(IPMACR,'NJJD'//CAN(IANI),NJJ)
            CALL LCMGET(IPMACR,'IJJD'//CAN(IANI),IJJ)
            CALL LCMGET(IPMACR,'IPOD'//CAN(IANI),IPOS)
            CALL LCMGET(IPMACR,'SCAD'//CAN(IANI),VSCAT)
            DO 45 IMAT= 1, NBMIX
              DO 40 JGR= IJJ(IMAT)-NJJ(IMAT)+1, IJJ(IMAT)
                IXSS= IPOS(IMAT)+IJJ(IMAT)-JGR
                XSCAT(JGR,IMAT,IANI,IGR)= VSCAT(IXSS)
   40         CONTINUE
   45       CONTINUE
          ENDIF
   50   CONTINUE
        CALL LCMSIX(IPMACR,CGRPNM,ILCMDN)
 150  CONTINUE
C----
C UPLOAD DIRECT FLUX & KEFF
C----
       CALL LCMGET(IPFLUX,'K-EFFECTIVE',KEFF)
       RKEFF = KEFF
       CALL LCMSIX(IPFLUX,CDIRFL,ILCMUP)
       DO 22 IGR= 1, NGROUP
         WRITE(NAMFLX,'(4HFLUX,I3,1X)') IGR
         CALL LCMGET(IPFLUX,NAMFLX,FLUXO(1,1,IGR))
         DO 12 IREG= 1, NREG
            FLUX(1,IREG,IGR,3) = FLUXO(IREG,1,IGR)
            IF( ITYPEC .EQ. 6 )THEN
                FLUX(1,IREG,IGR,4) = XSNUF(MATCOD(IREG),1,IGR)
            ELSE
                FLUX(1,IREG,IGR,4) = ZERO
            ENDIF
            IF( NL.GT.0 )THEN
               DO 2 IA= 2, NFUNL
                  FLUX(IA,IREG,IGR,2) = ZERO
                  FLUX(IA,IREG,IGR,4) = ZERO
    2          CONTINUE
            ENDIF
   12    CONTINUE
   22  CONTINUE
       CALL LCMSIX(IPFLUX,CDIRFL,ILCMDN) 
C---
C   EVAL TOTAL DIRECT SOURCES 
C---
            DO 821 IGR= 1, NGROUP
               DO 811 IREG= 1,NREG
                  FLUX(1,IREG,IGR,4)= ZERO
  811          CONTINUE
  821       CONTINUE
            DO 861 IREG= 1,NREG
               DO 851 IS= 1,NIFISS
                  FISOUR= DZERO
                  DO 831 IGR= 1, NGROUP
                     FISOUR= FISOUR+
     >                    XSNUF(MATCOD(IREG),IS,IGR)*FLUX(1,IREG,IGR,3)
  831             CONTINUE
                  FISOUR= FISOUR/RKEFF
                  DO 841 IGR= 1,NGROUP
                     FLUX(1,IREG,IGR,4)= FLUX(1,IREG,IGR,4)+
     >                                XSCHI(MATCOD(IREG),IS,IGR)*FISOUR
  841             CONTINUE
  851          CONTINUE
  861       CONTINUE
          DO 245 IGR= 1,NGROUP 
             DO 265 IREG= 1,NREG
C               ADDING ISOTROPIC SCATTERING FROM PRESENT INTERNAL
                FLUX(1,IREG,IGR,8)= FLUX(1,IREG,IGR,4)
                DO 235 JGR= 1,NGROUP
                   IF( IGR.EQ.JGR )THEN
                      FLUX(1,IREG,IGR,8)= FLUX(1,IREG,IGR,8)+
     >                XSSIGW(MATCOD(IREG),IGR)*FLUX(1,IREG,JGR,3)
                   ELSE
                      FLUX(1,IREG,IGR,8)= FLUX(1,IREG,IGR,8)+
     >                XSCAT(JGR,MATCOD(IREG),0,IGR)*FLUX(1,IREG,JGR,3)
                   ENDIF
  235           CONTINUE
  265        CONTINUE
  245     CONTINUE
      ALFACOR=-0.0001 
      IF(IPRINT .GE. 1000) WRITE(IOUT,6006)  ALFACOR
      CALL XDRSET(CORMAT,NREG*NBMIX*NGROUP,0.0)
      DO 5 JMIX=1,NBMIX
C---
C MODIFY MACROLIB
C---
       CALL LCMOP(IPMAD,'MACR_MODIFTR',0,1,0)
       CALL LCMEQU(IPMACR,IPMAD)
       DO 10 IGR=1,NGROUP
        CALL XDRSET(SIGMA,2*NBMIX,0.0)
        WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGROUP
        CALL LCMSIX(IPMAD,CGRPNM,ILCMUP)
        CALL LCMLEN(IPMAD,'TOTAL       ',ILCMLN,ILCMTY)
        IF(ILCMLN .EQ. 0) CALL XABORT(NAMSBR//
     >    ': READ ERROR FOR RECORD-> TOTAL')
        CALL LCMGET(IPMAD,'TOTAL       ',SIGMA(1,1))
        IF (ITRANC.GT.0)  CALL LCMGET(IPMAD,'TRANC       ',SIGMA(1,2))
        SIGMA(JMIX,2)=SIGMA(JMIX,2)+
     >       (SIGMA(JMIX,1)-SIGMA(JMIX,2))*ALFACOR
        CALL LCMPUT(IPMAD,'TRANC       ',NBMIX,2,SIGMA(1,2))
        CALL LCMSIX(IPMAD,CGRPNM,ILCMDN)
 10    CONTINUE
C-----
C THE COORECTION VECTOR COMPUTATION
C----
C---  DISABLE DIRECT FLUX CALCULATION ---
        LDRFLX = .FALSE. 
        LDAJCB = .FALSE.
C---  DISABLE ADJOINT CALCULATION  ---
        IF (ITYPEC .EQ. 1) THEN
           IMADJ = 0
        ELSE IF (ITYPEC .EQ. 6) THEN
           IMADJ = 0 
        ELSE
           CALL XABORT('SADMCR:INVALID TYPEC FOR ADJOINT CALCULATION')
        ENDIF
C      --- CAL  FLUX(,,,7)= [P+DELTA P]*[Q] : USING IPMAD ---
       CALL MOCDRV(IPFLUX,IPTRK,IPMAD,IPBTF ,NGROUP,NBMIX,NIFISS,
     1 NANIS,NL,NBN,NUNKNO,FLUXO,DIFFB1,
     2 NINTMX,AIEPS,NEXTMX,AEEPS,IFRITR,IACITR,ITYPEC,ILEAK,
     3 NREG  ,MATCOD,VOLUME,VOLINV,
     4 XSOLD,XSSIGT,XTINV,XSSIGW,
     5 XSCHI,XSNUF,XSTK,
     6 NJJ,IJJ,IPOS,XSCAT,VSCAT,
     7 NALB,ALBEDO,LEAKSW,ILK,ITRANC,ITRB,TITLE,LCACA,NGSS,LEXIN,
     8 REFKEF,IPRINT,CDIRFL,
     9 ADJFLO,CADJFL,IMADJ,LDRFLX,LDAJCB,RKEFF,LMODE,
     9 IPTRK2,IFTRK2,NGSS2,FLUX)
C         --- STORE FLUX(,,,7) IN FLUX(,,,1) ---
          DO 295 IGR= 1, NGROUP
             DO 285 IREG= 1, NREG
                 FLUX(1,IREG,IGR,1) = FLUX(1,IREG,IGR,7)
  285        CONTINUE
  295     CONTINUE
C      --- CAL  FLUX(,,,7)= [P]*[Q] : USING IPMACR ---
       CALL MOCDRV(IPFLUX,IPTRK,IPMACR,IPBTF ,NGROUP,NBMIX,NIFISS,
     1 NANIS,NL,NBN,NUNKNO,FLUXO,DIFFB1,
     2 NINTMX,AIEPS,NEXTMX,AEEPS,IFRITR,IACITR,ITYPEC,ILEAK,
     3 NREG  ,MATCOD,VOLUME,VOLINV,
     4 XSOLD,XSSIGT,XTINV,XSSIGW,
     5 XSCHI,XSNUF,XSTK,
     6 NJJ,IJJ,IPOS,XSCAT,VSCAT,
     7 NALB,ALBEDO,LEAKSW,ILK,ITRANC,ITRB,TITLE,LCACA,NGSS,LEXIN,
     8 REFKEF,IPRINT,CDIRFL,
     9 ADJFLO,CADJFL,IMADJ,LDRFLX,LDAJCB,RKEFF,LMODE,
     9 IPTRK2,IFTRK2,NGSS2,FLUX)
       DO 296 IGR= 1, NGROUP
         DO 286 IREG= 1, NREG
         CORMAT(IREG,JMIX,IGR)= FLUX(1,IREG,IGR,7)-FLUX(1,IREG,IGR,1)
  286    CONTINUE
  296  CONTINUE
C---
C COMPUTE THE ROW OF THE MATRIX ASSOCIATED TO JMIX
C---
       CALL XDRSET(SIGMA,2*NBMIX,0.0)
       DO 300 IGR=1,NGROUP
        WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGROUP
        CALL LCMSIX(IPMACR,CGRPNM,ILCMUP)
        CALL LCMGET(IPMACR,'TOTAL       ',SIGMA(1,1))
        IF (ITRANC.GT.0)  CALL LCMGET(IPMACR,'TRANC       ',SIGMA(1,2))
        DO 310 IREG=1,NREG
           IF ( (SIGMA(JMIX,1)-SIGMA(JMIX,2)) .NE.0.0) THEN
             CORMAT(IREG,JMIX,IGR) = - CORMAT(IREG,JMIX,IGR)/ALFACOR/
     >           (SIGMA(JMIX,1)-SIGMA(JMIX,2))
           ELSEIF (CORMAT(IREG,JMIX,IGR).NE.0.0) THEN
             CALL XABORT( NAMSBR//'TOTAL CROSS SECTION VANISHES')
           ENDIF
 310    CONTINUE
        CALL LCMSIX(IPMACR,CGRPNM,ILCMDN)
 300   CONTINUE
C---
C DELETE TEMPORARY LINKED LISTS
C---
       CALL LCMCL(IPMAD,2)
 5    CONTINUE
C-----
C PRINT CORRECTION MATRIX
C-----
      CALL LCMSIX(IPFLUX,'FLUX1-CORR  ',ILCMUP)
      DO 400 IGR=1,NGROUP
       WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGROUP
       CALL LCMPUT(IPFLUX,CGRPNM,NREG*NBMIX,2,CORMAT(1,1,IGR))
 400  CONTINUE
      CALL LCMSIX(IPFLUX,'FLUX1-CORR  ',ILCMDN)
C---  THE ORIGINAL REBAL STATUS IS RESTORED---
      ILK = ILKO
      RETURN
C---
C FORMAT
C---
 6000 FORMAT(1X,'Output from ',A6)
 6001 FORMAT(10X,'IMPOSED REBALANCE FLAG FOR DIRECT',
     >           ' & ADJOINT CALCULATION')
 6002 FORMAT(10X,' ILCMLN = ',I10/10X,' ILCMTY = ',I10)
 6003 FORMAT(10X,' CGRPNM = ',I10)
 6004 FORMAT(10X,'CORRECTION MATRIX DIR EXIST RETURN TO OUTER LOOP')
 6005 FORMAT(10X,'RUNNING')
 6006 FORMAT(10X,' ALFACOR = ',E20.10)
      END
