*DECK SHIDAN
      SUBROUTINE SHIDAN(IMPX,DIAG,NMBLK,IFR,ALB,NMERGE,IJAT,INUM,VOL,
     > MIX,DVX,MU1,IMA,NGEN,IGEN,ISURF,CHORD,SIG0,SIG1,SIG2,PII,PIS,
     > PSS,NRES,IRES,PSTAR,PSSB,IGAT,IGAU,IGAU2,OPTIC,PST,VST,WP,GAR,
     > SSSS)
C
C-----------------------------------------------------------------------
C
C FAST CALCULATION OF THE DANCOFF MATRIX AND THE RELATED ESCAPE MATRIX
C USING THE JPM METHOD.
C
C   IMPX   : PRINT FLAG. EQUAL TO 0 FOR NO PRINT.
C   DIAG   : IF DIAG=.TRUE., COMPUTE A DIAGONAL APPROXIMATION OF PSTAR.
C   NMBLK  : TOTAL NUMBER OF BLOCKS IN THE DOMAIN.
C   IFR    : INDEX-NUMBER OF IN-CURRENTS. DIMENSION IFR(IJAS) WHERE
C            IJAS IS THE TOTAL NUMBER OF IN-CURRENTS SURROUNDING ALL
C            THE BLOCKS.
C   ALB    : TRANSMISSION/ALBEDO ASSOCIATED WITH EACH IN-CURRENT.
C            DIMENSION ALB(IJAS)
C   NMERGE : TOTAL NUMBER OF MERGED BLOCKS FOR WHICH SPECIFIC VALUES
C            OF THE NEUTRON FLUX AND REACTIONS RATES ARE REQUIRED.
C            MANY BLOCKS WITH DIFFERENT POSITION IN THE DOMAIN CAN
C            BE MERGED BEFORE THE NEUTRON FLUX CALCULATION IF THEY
C            OWN THE SAME GENERATING BLOCK. THIS ALLOWS SOME REDUCTION
C            IN CPU TIME AND MEMORY. NMERGE.LE.NMBLK
C   IJAT   : TOTAL NUMBER OF DISTINCT OUT-CURRENTS.
C   INUM   : INDEX-NUMBER OF THE MERGED BLOCK ASSOCIATED TO EACH
C            BLOCK. DIMENSION INUM(NMBLK)
C   VOL    : VOLUMES. DIMENSION VOL(NMERGE)
C   MIX    : INDEX-NUMBER OF OUT-CURRENTS. DIMENSION MIX(IJAR) WHERE
C            IJAR IS THE TOTAL NUMBER OF OUT-CURRENTS BELONGING TO THE
C            MERGED BLOCKS.
C   DVX    : WEIGHT ASSOCIATED WITH EACH OUT-CURRENT.
C            DIMENSION DVX(IJAR)
C   MU1    : POSITION OF EACH DIAGONAL ELEMENT IN MATRIX (I-PSS*A).
C   IMA    : POSITION OF THE FIRST NON-ZERO COLUMN ELEMENT IN MATRIX
C            (I-PSS*A). DIMENSION MU1(IJAT),IMA(IJAT)
C
C            REMARK:
C            IFR, ALB, MIX AND DVX CONTAINS INFORMATION TO REBUILD THE
C            GEOMETRICAL "A" MATRIX.
C
C INPUT PARAMETERS RELATED TO THE GENERATING BLOCKS DEFINITION:
C   NGEN   : TOTAL NUMBER OF GENERATING BLOCKS. A GENERATING BLOCK IS
C            DEFINED BY ITS MIXTURE AND DIMENSIONS IRRESPECTIVE OF
C            ITS POSITION IN THE DOMAIN. NGEN.LE.NMERGE
C   IGEN   : INDEX-NUMBER OF THE GENERATING BLOCK ASSOCIATED WITH EACH
C            MERGED BLOCK. DIMENSION IGEN(NMERGE)
C   ISURF  : NUMBER OF SURFACES ASSOCIATED WITH EACH GENERATING BLOCK.
C            DIMENSION ISURF(NGEN)
C   CHORD  : MEAN CHORD LENGTH ASSOCIATED WITH EACH SURFACE.
C            DIMENSION CHORD(NPIS) WHERE NPIS=SUM(ISURF(I)) IS THE
C            TOTAL NUMBER OF SURFACES SURROUNDING ALL THE GENERATING
C            BLOCKS.
C   SIG0   : TOTAL MACROSCOPIC CROSS SECTIONS OF THE HEAVY ISOTOPES.
C   SIG1   : TOTAL MACROSCOPIC CROSS SECTIONS OF THE LIGHT ISOTOPES.
C   SIG2   : TRANSPORT CORRECTION OF THE LIGHT ISOTOPES.
C            DIMENSION SIG0(NGEN),SIG1(NGEN),SIG2(NGEN)
C   PII    : VOLUME TO VOLUME REDUCED COLLISION PROBABILITIES.
C            DIMENSION PII(NGEN)
C   PIS    : VOLUME TO SURFACE COLLISION PROBABILITIES.
C            DIMENSION PIS(NPIS) WHERE NPIS=SUM(ISURF(I))
C   PSS    : SURFACE TO SURFACE COLLISION PROBABILITIES.
C            DIMENSION PSS(NPSS) WHERE NPSS=SUM(ISURF(I)**2)
C
C INPUT PARAMETERS RELATED TO THE RESONANT REGIONS:
C   NRES   : NUMBER OF RESONANT REGIONS.
C   IRES   : INDEX-NUMBER OF RESONANT REGION ASSOCIATED WITH EACH
C            GENERATING BLOCK. IRES(I) IS EQUAL TO ZERO IN NON RESONANT
C            BLOCKS. DIMENSION IRES(NGEN)
C
C OUTPUT PARAMETERS:
C   PSTAR  : ESCAPE MATRIX. DIMENSION PSTAR(NRES,NRES)
C
C SCRATCH STORAGE:
C   PSSB   : DIMENSION PSSB(IMA(IJAT))
C   IGAT   : DIMENSION IGAT(NMERGE)
C   IGAU   : DIMENSION IGAU(NGEN)
C   IGAU2  : DIMENSION IGAU2(NGEN)
C   OPTIC  : DIMENSION OPTIC(NGEN)
C   PST    : DIMENSION PST(2*NRES*NRES)
C   VST    : DIMENSION VST(NRES)
C   WP     : DIMENSION WP(IJAT,NRES)
C   GAR    : DIMENSION GAR(IJAT)
C   SSSS   : DIMENSION SSSS(NRES)
C
C------------------------------------------- AUTHOR: A. HEBERT 20/05/86
C
      IMPLICIT    NONE
      INTEGER     IOUT
      CHARACTER   NAMSBR*6
      REAL        BLACK
      PARAMETER  (IOUT=6,BLACK=1000.0,NAMSBR='SHIDAN')
C----
C  ROUTINE PARAMETERS
C----
      INTEGER     IMPX,NMBLK,NMERGE,IJAT,NGEN,NRES
      LOGICAL     DIAG
      INTEGER     IFR(*),INUM(NMBLK),MIX(*),MU1(*),IMA(*),IGEN(NMERGE),
     >            ISURF(NGEN),IRES(NGEN),IGAT(NMERGE),IGAU(NGEN),
     >            IGAU2(NGEN)
      REAL        ALB(*),VOL(NMERGE),DVX(*),CHORD(*),SIG0(NGEN),
     >            SIG1(NGEN),SIG2(NGEN),PII(NGEN),PIS(*),PSS(*),
     >            PSTAR(NRES,NRES),PSSB(*),OPTIC(NGEN),
     >            PST(NRES,2*NRES),VST(NRES),WP(IJAT,NRES),GAR(IJAT),
     >            SSSS(NRES)
C----
C  LOCAL PARAMETERS
C----
      INTEGER     IJAR,I,J,NPIS,NPSS,IKG,IKK,NCOUR,IP,JP,IOFSET,
     >            IS,IT,J2,IDLJPJ,J1,IDLIMI,IND,JND,ICC,ICEL,IER
      REAL        ZZZ,FACT,ALBEDO,WSS,GAS2,VND,GAS1,PBJ
*
      IF(NRES .EQ. 0) CALL XABORT(NAMSBR//
     >': NO RESONANT REGION.')
      IJAR=0
      DO 10 I=1,NMERGE
        IGAT(I)=IJAR
        IJAR=IJAR+ISURF(IGEN(I))
 10   CONTINUE
      NPIS=0
      NPSS=0
      DO 40 IKG=1,NGEN
        NCOUR=ISURF(IKG)
        IF(SIG0(IKG) .NE. 0.0 .AND.
     >     IRES(IKG) .EQ. 0) THEN
          CALL XABORT(NAMSBR//
     >    ': HEAVY ISOTOPES CAN ONLY APPEAR INTO RESONANT BLOCKS.')
        ELSE IF(SIG0(IKG) .EQ. 0.0 .AND.
     >          IRES(IKG) .NE. 0 ) THEN
          CALL XABORT(NAMSBR//
     >    ': RESONANT CROSS SECTION CANNOT BE ZERO.')
        ENDIF
        ZZZ=0.0
        DO 20 IP=1,NCOUR
          ZZZ=ZZZ+1.0/CHORD(NPIS+IP)
 20     CONTINUE
        OPTIC(IKG)=1.0/ZZZ
        IF(IRES(IKG) .NE. 0) THEN
          IF(OPTIC(IKG)*(SIG0(IKG)+SIG1(IKG)) .LE. BLACK) THEN
            SSSS(IRES(IKG))=1.0/SIG0(IKG)
          ELSE
            SSSS(IRES(IKG))=0.0
          ENDIF
        ENDIF
        IGAU(IKG)=NPIS
        IGAU2(IKG)=NPSS
*
* PERFORM TRANSPORT CORRECTION ON BLOCKWISE COLLISION PROBABILITIES.
        IF(SIG2(IKG) .NE. 0.0) THEN
          FACT=1.0/(1.0+SIG2(IKG)*PII(IKG))
          PII(IKG)=PII(IKG)*FACT
          DO 25 IP=1,NCOUR
            PBJ=PIS(NPIS+IP)*CHORD(NPIS+IP)
            DO 26 JP=1,NCOUR
              IOFSET=NPSS+(JP-1)*NCOUR+IP
              PSS(IOFSET)=PSS(IOFSET)-SIG2(IKG)*PBJ*PIS(NPIS+JP)*FACT
 26         CONTINUE
 25       CONTINUE
          DO 30 IP=1,NCOUR
            PIS(NPIS+IP)=PIS(NPIS+IP)*FACT
 30       CONTINUE
        ENDIF
        NPIS=NPIS+NCOUR
        NPSS=NPSS+NCOUR**2
 40   CONTINUE
*
* PRINT THE BLOCKWISE COLLISION PROBABILILTIES.
      IF(IMPX .GE. 10) THEN
        NPIS=0
        NPSS=0
        DO 50 IKG=1,NGEN
          NCOUR=ISURF(IKG)
          IF(OPTIC(IKG)*(SIG0(IKG)+SIG1(IKG)) .LE. BLACK) THEN
            WRITE(IOUT,300) NAMSBR,IKG,SIG0(IKG),SIG1(IKG),PII(IKG)
            WRITE(IOUT,301)
            WRITE(IOUT,310) (PIS(NPIS+I),I=1,NCOUR)
            WRITE(IOUT,311)
            WRITE(IOUT,320) (J,J=1,NCOUR)
            DO 45 I=1,NCOUR
              WRITE(IOUT,330) I,(PSS(NPSS+(J-1)*NCOUR+I),J=1,NCOUR)
   45       CONTINUE
            WRITE(IOUT,'(//)')
          ENDIF
          NPIS=NPIS+NCOUR
          NPSS=NPSS+NCOUR*NCOUR
   50   CONTINUE
      ENDIF
*
* COMPUTATION OF L*U=(I-PSS*A)
      DO 60 I=1,IMA(IJAT)
        PSSB(I)=0.0
 60   CONTINUE
      DO 70 I=1,IJAT
        PSSB(MU1(I))=1.0
 70   CONTINUE
      IS=0
      DO 100 ICEL=1,NMBLK
        IKK=INUM(ICEL)
        IKG=IGEN(IKK)
        NCOUR=ISURF(IKG)
        IF(OPTIC(IKG)*(SIG0(IKG)+SIG1(IKG)) .LE. BLACK) THEN
          IT=IGAT(IKK)
          DO 80 JP=1,NCOUR
           J2=IFR(IS+JP)
           IDLJPJ=MU1(J2)+J2
           ALBEDO=ALB(IS+JP)
           IOFSET=IGAU2(IKG)+(JP-1)*NCOUR
           DO 81 IP=1,NCOUR
             J1=MIX(IT+IP)
             WSS=PSS(IOFSET+IP)*ALBEDO*DVX(IT+IP)
             IF(J1 .GE. J2) THEN
               IDLIMI=MU1(J1)-J1
               PSSB(IDLIMI+J2)=PSSB(IDLIMI+J2)-WSS
             ELSE
               PSSB(IDLJPJ-J1)=PSSB(IDLJPJ-J1)-WSS
             ENDIF
 81        CONTINUE
 80      CONTINUE
      ENDIF
        IS=IS+NCOUR
 100  CONTINUE
      CALL ALLUF(IJAT,PSSB,MU1,IMA)
*
* COMPUTATION OF CONDENSED PSJ AND PSJ*SIG0 MATRICES.
      DO 110 IND=1,NRES
        DO 111 ICC=1,IJAT
          WP(ICC,IND)=0.0
 111    CONTINUE
 110  CONTINUE
      DO 130 IKK=1,NMERGE
        IKG=IGEN(IKK)
        IND=IRES(IKG)
        IF(IND .GT. 0) THEN
          IT=IGAT(IKK)
          IF(OPTIC(IKG)*(SIG0(IKG)+SIG1(IKG)) .LE. BLACK) THEN
            DO 120 IP=1,ISURF(IKG)
              ICC=MIX(IT+IP)
              GAS2=PIS(IGAU(IKG)+IP)*CHORD(IGAU(IKG)+IP)*SIG0(IKG)
              WP(ICC,IND)=WP(ICC,IND)+DVX(IT+IP)*GAS2
 120        CONTINUE
          ELSE
            DO 125 IP=1,ISURF(IKG)
              ICC=MIX(IT+IP)
              WP(ICC,IND)=WP(ICC,IND)+DVX(IT+IP)
 125        CONTINUE
          ENDIF
        ENDIF
130   CONTINUE
*
* COMPUTATION OF CONDENSED A*((L*U)**-1)*PSJ AND A*((L*U)**-1)*PSJ*SIG0
* MATRICES.
      DO 170 IND=1,NRES
        CALL ALLUS(IJAT,MU1,IMA,PSSB,WP(1,IND))
        DO 140 I=1,IJAT
          GAR(I)=0.0
 140    CONTINUE
        IS=0
        DO 160 ICEL=1,NMBLK
          IKK=INUM(ICEL)
          NCOUR=ISURF(IGEN(IKK))
          IT=IGAT(IKK)
          DO 150 IP=1,NCOUR
            J1=MIX(IT+IP)
            GAR(J1)=GAR(J1)+ALB(IS+IP)*DVX(IT+IP)*WP(IFR(IS+IP),IND)
 150      CONTINUE
          IS=IS+NCOUR
 160    CONTINUE
        DO 165 I=1,IJAT
          WP(I,IND)=GAR(I)
 165    CONTINUE
        VST(IND)=0.0
        DO 171 J=1,2*NRES
           PST(IND,J)=0.0
 171    CONTINUE
 170  CONTINUE
*
* COMPUTATION OF CONDENSED PIJ AND (I-PIJ*SIG0) MATRICES.
      DO 180 IKK=1,NMERGE
        IND=IRES(IGEN(IKK))
        IF(IND .GT. 0) VST(IND)=VST(IND)+VOL(IKK)
  180 CONTINUE
      DO 200 IKK=1,NMERGE
        IKG=IGEN(IKK)
        IND=IRES(IKG)
        IF(IND .NE. 0) THEN
          VND=VOL(IKK)/VST(IND)
          IT=IGAT(IKK)
          IF(OPTIC(IKG)*(SIG0(IKG)+SIG1(IKG)) .LE. BLACK) THEN
            DO 190 IP=1,ISURF(IKG)
              ZZZ=VND*PIS(IGAU(IKG)+IP)
              DO 191 JND=1,NRES
                IF(.NOT.DIAG .OR. JND .EQ. IND) THEN
                  GAS1=ZZZ*WP(MIX(IT+IP),JND)
                  PST(IND,JND)=PST(IND,JND)+GAS1*SSSS(JND)
                  PST(IND,NRES+JND)=PST(IND,NRES+JND)-GAS1
                ENDIF
 191          CONTINUE
 190        CONTINUE
            GAS1=PII(IKG)
            GAS2=1.0-GAS1*SIG0(IKG)
          ELSE
            DO 195 IP=1,ISURF(IKG)
              ZZZ=VND/CHORD(IGAU(IKG)+IP)
              DO 196 JND=1,NRES
                IF(.NOT. DIAG .OR. JND .EQ. IND) THEN
                  GAS1=ZZZ*WP(MIX(IT+IP),JND)
                  PST(IND,JND)=PST(IND,JND)+GAS1*SSSS(JND)
                  PST(IND,NRES+JND)=PST(IND,NRES+JND)-GAS1
                ENDIF
 196          CONTINUE
 195        CONTINUE
            GAS1=1.0
            GAS2=SIG1(IKG)+1.0/OPTIC(IKG)
          ENDIF
          PST(IND,IND)=PST(IND,IND)+VND*GAS1
          PST(IND,NRES+IND)=PST(IND,NRES+IND)+VND*GAS2
        ENDIF
 200  CONTINUE
*
* COMPUTATION OF ESCAPE MATRIX.
      IF(NRES .EQ. 1) THEN
        IF(PST(1,1) .EQ. 0.0) CALL XABORT(NAMSBR//
     >  ': SINGULAR MATRIX.')
        PSTAR(1,1)=PST(1,2)/PST(1,1)
      ELSE
        CALL ALSB(NRES,NRES,PST,IER,NRES)
        IF(IER .NE. 0) CALL XABORT(NAMSBR//
     >  ': SINGULAR MATRIX.')
        DO 210 I=1,NRES
          DO 211 J=1,NRES
            PSTAR(I,J)=PST(I,NRES+J)
 211      CONTINUE
 210    CONTINUE
      ENDIF
      RETURN
*
  300 FORMAT(////A6,': GENERATING BLOCK NUMBER',I5//
     > ' TOTAL MACROSCOPIC CROSS SECTION: HEAVY ISOTOPES =',1P,E11.3,
     > 5X,'LIGHT ISOTOPES =',E11.3,5X,'PII =',E11.3)
  301 FORMAT(//' TRANSPORT CORRECTED MATRIX PIS(I)'/)
  310 FORMAT(1X,1P,11E11.3)
  311 FORMAT(//' TRANSPORT CORRECTED MATRIX PSS(I,J) (I TOWARD J)'/)
  320 FORMAT(6X,11(5X,'J=',I4,:))
  330 FORMAT(' I=',I4,': ',1P,11E11.3/(9X,11E11.3))
      END
