*DECK SHISN2
      SUBROUTINE SHISN2(IPLIB ,IPTRK ,IFTRAK,NGRO  ,NBISO ,NBMIX ,
     >                  NREG  ,CDOOR ,NRES  ,IPRINT,ISONAM,MIX   ,
     >                  DEN   ,TN    ,SN    ,SB    ,LSHI  ,MAT   ,
     >                  VOL   ,LEAKSW,TITR  ,START ,SIGT  ,SIGT3 ,
     >                  NOCONV,BIEFF ,LGC   ,SIGE  ,IHLIB ,NTFG  ,
     >                  ISONRF,AWR   ,NEDMAC,HVECT ,NL    ,ITIME ,
     >                  ITRANC,NPROC ,NBRMIX,
     >                  IRES  ,SIG0  ,SIG3  ,TOTAL ,SIGOLD,GAR   ,
     >                  MIX2  ,IRNBM ,SIGRES,MASKI ,ITISOM,DELTA ,
     >                  ITYPRO)
C
C-----------------------------------------------------------------------
C
C PERFORM ONE MULTIDIMENSIONAL SELF-SHIELDING ITERATION USING THE
C GENERALIZED STAMM'LER ALGORITHM WITH PIC APPROXIMATION.
C
C  IPLIB  : POINTER TO THE INTERNAL MICROSCOPIC CROSS SECTION LIBRARY
C           (L_LIBRARY SIGNATURE).
C  IPTRK  : POINTER TO THE TRACKING. (L_JPM, L_SYBIL OR L_EXCELL
C           SIGNATURE).
C  IFTRAK : UNIT NUMBER OF THE SEQUENTIAL BINARY TRACKING FILE.
C  NGRO   : NUMBER OF ENERGY GROUPS.
C  NBISO  : NUMBER OF ISOTOPES PRESENT IN THE CALCULATION DOMAIN.
C  NBMIX  : NUMBER OF MIXTURES IN THE MACROLIB.
C  NREG   : NUMBER OF VOLUMES.
C  CDOOR  : CHARACTER*12 NAME OF THE GEOMETRY/SOLUTION MODULE.
C           IF CDOOR='JPM' SHIBA WILL PERFORM A 'FAST' SELF-SHIELDING
C           CALCULATION.
C  NRES   : NUMBER OF RESONANT MIXTURES.
C  IPRINT : PRINT FLAG.
C  ISONAM : NAME OF ISOTOPES. ISONAM(3,NBISO)
C  MIX    : MIX NUMBER OF EACH ISOTOPE. CAN BE ZERO.
C           DIMENSION MIX(NBISO)
C  DEN    : DENSITY OF EACH ISOTOPE. DIMENSION DEN(NBISO)
C  TN     : TEMPERATURE OF EACH ISOTOPE. DIMENSION TN(NBISO)
C  SN     : ESTIMATE (INPUT) OR COMPUTED (OUTPUT) DILUTION CROSS
C           SECTION IN EACH ENERGY GROUP FOR EACH ISOTOPE
C           DIMENSION SN(NGRO,NBISO)
C  SB     : COMPUTED (OUTPUT) DILUTION CROSS SECTION AS USED
C           IN LIVOLANT AND JEANPIERRE NORMALIZATION
C           DIMENSION SB(NGRO,NBISO)
C  LSHI   : RESONANT REGION NUMBER ASSOCIATED WITH ISOTOPE I.
C           INFINITE DILUTION WILL BE ASSUMED IF LSHI(I)=0.
C           DIMENSION LSHI(NBISO)
C  MAT    : INDEX-NUMBER OF THE MIXTURE TYPE ASSIGNED TO EACH VOLUME.
C           DIMENSION MAT(NREG)
C  VOL    : VOLUMES. DIMENSION VOL(NREG)
C  LEAKSW : LEAKSW=.TRUE. IF NEUTRON LEAKAGE THROUGH EXTERNAL BOUNDARY
C           IS PRESENT.
C  TITR   : CHARACTER*72 TITLE.
C  START  : START=.TRUE. IF SHISN2 IS CALLED FOR THE FIRST TIME.
C  SIGT   : TOTAL MACROSCOPIC CROSS SECTIONS. DIMENSION SIGT(NBMIX,NGRO)
C  SIGT3  : TRANSPORT CORRECTION. DIMENSION SIGT3(NBMIX,NGRO)
C  NOCONV : NOCONV(IBM,L)=.TRUE. IF MIXTURE IBM IS NOT CONVERGED IN
C           GROUP L. DIMENSION NOCONV(NBMIX,NGRO)
C  BIEFF  : BIEFF=.TRUE. WILL ACTIVATE A LIVOLANT AND JEANPIERRE
C           EQUIVALENCE SIMILAR TO THE TECHNIQUE USED UN THE BIEFF
C           SUBROUTINE OF THE APOLLO-1 SOFTWARE.
C  LGC    : LGC=.TRUE. WILL ACTIVATE THE GOLDSTEIN-COHEN APPROXIMATION
C           IF GOLDSTEIN-COHEN ARE AVAILABLE.
C  SIGE   : COMPUTED MACROSCOPIC DILUTION X-S IN EACH RESONANT MIXTURE
C           AND EACH ENERGY GROUP. DIMENSION SIGE(NRES,NGRO)
C  IHLIB  : DIMENSION IHLIB(2,NBISO,5)
C  NTFG   : DIMENSION NTFG(NBISO)
C  ISONRF : DIMENSION ISONRF(3,NBISO)
C  AWR    : DIMENSION AWR(NBISO)
C  NEDMAC : NUMBER OF ADDITIONAL XS
C  HVECT  : NAME OF ADDITIONAL XS HVECT(NEDMAC)*6
C  NL     : LEGENDRE ORDER OF SCATTERING
C  ITIME  : TIME STEP
C  ITRANC : TRANSPORT CORRECTION
C  NPROC  : NUMBER OF VECTOR CROSS SECTION
C  NBRMIX : NUMBER OF RESONANCE MIXTURES
C  SIGT   : TOTAL MACROSCOPIC CROSS SECTIONS AS MODIFIED BY SHIBA.
C           DIMENSION SIGT(NBMIX,NGRO)
C  IRES   : DIMENSION IRES(NBMIX)
C  SIG0   : DIMENSION SIG0(NBMIX)
C  SIG3   : DIMENSION SIG3(NBMIX)
C  TOTAL  : DIMENSION TOTAL(NGRO,NBRMIX)
C  SIGOLD : DIMENSION SIGOLD(NGRO,NBRMIX)
C  GAR    : DIMENSION GAR(NGRO)
C  MIX2   : DIMENSION MIX2(NBISO)
C  IRNBM  : DIMENSION IRNBM(NBM)
C  SIGRES : DIMENSION SIGRES(NBM)
C  MASKI  : DIMENSION MASKI(NBISO)
C  ITISOM : DIMENSION ITISOM(NBISO)
C  DELTA  : DIMENSION DELTA(NGRO)
C  ITYPRO : DIMENSION ITYPRO(NPROC)
C
C REFERENCE:
C  A. HEBERT AND G. MARLEAU, 'GENERALIZATION OF THE STAMM'LER METHOD
C  FOR THE SELF-SHIELDING OF RESONANT ISOTOPES IN ARBITRARY GEOMETRIES',
C  NUCL. SCI. ENG. 108, 230 (1991).
C
C--------------------------------------- AUTHOR: A. HEBERT 03/02/89 ----
C
      IMPLICIT   NONE
      INTEGER    IOUT,NSTATE,NDPROC,NALPHA,ILCMUP,ILCMDN
      CHARACTER  NAMSBR*6
      LOGICAL    LDIAG
      PARAMETER (IOUT=6,NSTATE=40,NDPROC=20,NALPHA=5,
     >           ILCMUP=1,ILCMDN=2,LDIAG=.TRUE.,NAMSBR='SHISN2')
C----
C  ROUTINE PARAMETERS
C----
      INTEGER    IPLIB,IPTRK,IFTRAK,NGRO,NBISO,NBMIX,
     >           NREG,NRES,IPRINT,NEDMAC,NL,ITIME,
     >           ITRANC,NPROC,NBRMIX
      INTEGER    ISONAM(3,NBISO),MIX(NBISO),LSHI(NBISO),MAT(NREG),
     >           IHLIB(2,NBISO,5),NTFG(NBISO),ISONRF(3,NBISO),
     >           IRES(NBMIX),MIX2(NBISO),IRNBM(NBMIX),ITISOM(NBISO),
     >           ITYPRO(NPROC)
      REAL       DEN(NBISO),TN(NBISO),SN(NGRO,NBISO),SB(NGRO,NBISO),
     >           VOL(NREG),SIGT(NBMIX,NGRO),SIGT3(NBMIX,NGRO),
     >           SIGE(NRES,NGRO),AWR(NBISO),SIG0(NBMIX),SIG3(NBMIX),
     >           TOTAL(NGRO,NBRMIX),SIGOLD(NGRO,NBRMIX),GAR(NGRO),
     >           SIGRES(NBMIX),DELTA(NGRO)
      CHARACTER  CDOOR*12,TITR*72,HVECT(NEDMAC)*6
      LOGICAL    LEAKSW,START ,NOCONV(NBMIX,NGRO),BIEFF,LGC,
     >           MASKI(NBISO)
C----
C  LOCAL PARAMETERS
C----
      CHARACTER  CGRPNM*12,HSMG*131,HNAMIS*12
      INTEGER    ISTATE(NSTATE),IPRNT2,I1,KI,I,I2,J,IBM,JBM,
     >           ISO,JSO,IGR,NBNRS,INRS,ITC,IALP,NGFR,NRAT
      LOGICAL    LOGDO
      COMPLEX    COEF(3),DENOM(3),EAV
      REAL       DILUT(NALPHA),SIGX(NALPHA),PXX,DILAV,SUM,SUB,
     >           SA,PP,QQ,BEL
*----
      INTEGER    ISOADD
*
      ISOADD=0
      IF(IPRINT .GE. 10) THEN
        WRITE(IOUT,'(//15H VALUES OF MAT:)')
        I1=1
        KI=(NREG-1)/11+1
        DO 10 I=1,KI
          I2=I1+10
          IF(I2 .GT. NREG) I2=NREG
          WRITE(IOUT,440) (J,J=I1,I2)
          WRITE(IOUT,450) (MAT(J),J=I1,I2)
          I1=I1+11
 10     CONTINUE
        WRITE(IOUT,'(//)')
      ENDIF
*
* UNLOAD MICROSCOPIC X-S FROM LCM TO SCRATCH STORAGE.
      IBM=0
      DO 25 ISO=1,NBISO
        MIX2(ISO)=0
        IF(LSHI(ISO) .GT. 0) THEN
          IBM=IBM+1
          MIX2(ISO)=IBM
          WRITE(HNAMIS,'(3A4)') (ISONAM(ITC,ISO),ITC=1,3)
          CALL LCMSIX(IPLIB,HNAMIS,ILCMUP)
          CALL LCMGET(IPLIB,'XS-SAVED',ITYPRO)
          IF(MOD(ITYPRO(1),2) .EQ. 1) THEN
            CALL LCMGET(IPLIB,'TOTAL',TOTAL(1,IBM))
          ELSE
            CALL XABORT(NAMSBR//
     >      ': TOTAL CROSS SECTION ABSENT FROM IPLIB')
          ENDIF
          CALL XDRSET(SIGOLD(1,IBM),NGRO,0.0)
          IF(LGC .AND. (.NOT. START)) THEN
            IF(MOD(ITYPRO(14),2) .EQ. 1) THEN
              IF(IPRINT .GE. 10) WRITE(IOUT,390) NAMSBR,HNAMIS
              IF(MOD(ITYPRO(NDPROC+1),2) .EQ. 1) THEN
                CALL LCMGET(IPLIB,'SIGS 0',SIGOLD(1,IBM))
              ENDIF
              CALL LCMGET(IPLIB,'GOLD',GAR)
              DO 15 IGR=1,NGRO
                SIGOLD(IGR,IBM)=(1.0-GAR(IGR))*SIGOLD(IGR,IBM)
 15           CONTINUE
            ENDIF
          ENDIF
          CALL LCMSIX(IPLIB,HNAMIS,ILCMDN)
        ENDIF
 25   CONTINUE
*
* LOOP OVER RESONANT REGIONS. THE CP ARE STORED ON DIRECTORY SHIBA.
      CALL LCMSIX(IPLIB,'SHIBA',ILCMUP)
      DO 210 INRS=1,NRES
*
* FIND THE RESONANT MIXTURE NUMBERS (IRNBM) ASSOCIATED WITH REGION INRS.
        NBNRS=0
        DO 40 IBM=1,NBMIX
          IRES(IBM)=0
          DO 30 ISO=1,NBISO
            IF(MIX(ISO) .EQ. IBM .AND. LSHI(ISO) .EQ. INRS) THEN
              NBNRS=NBNRS+1
              IRNBM(NBNRS)=IBM
              IRES(IBM)=1
              GO TO 40
            ENDIF
   30     CONTINUE
   35     CONTINUE
   40   CONTINUE
        IF(NBNRS .EQ. 0) THEN
          WRITE(HSMG,9000) NAMSBR,INRS
          CALL XABORT(HSMG)
        ELSE IF(START .AND. NBNRS .GT. 1 .AND. IPRINT .GE. 1) THEN
          WRITE(IOUT,380) NAMSBR,NBNRS,INRS
        ENDIF
*
        DO 211 IGR=1,NGRO
          LOGDO=.FALSE.
          DO 50 I=1,NBNRS
            LOGDO=LOGDO.OR.NOCONV(IRNBM(I),IGR)
 50       CONTINUE
          IF(LOGDO) THEN
*
*        COMPUTE THE LIGHT AND RESONANT COMPONENTS OF THE MACROSCOPIC
*        CROSS SECTIONS IN EACH RESONANT MIXTURE.
            DO 65 I=1,NBNRS
              SIGRES(I)=0.0
              DO 60 ISO=1,NBISO
                IF(MIX(ISO) .EQ. IRNBM(I) .AND.
     >             LSHI(ISO) .EQ. INRS) THEN
                  SIGRES(I)=SIGRES(I)+TOTAL(IGR,MIX2(ISO))*DEN(ISO)
                ENDIF
   60         CONTINUE
              SIGT(IRNBM(I),IGR)=SIGT(IRNBM(I),IGR)-SIGRES(I)
   65       CONTINUE
*
            WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGRO
            CALL LCMSIX(IPLIB,CGRPNM,ILCMUP)
*
*        SET UP VECTORS DILUT AND SIGX.
            IF(IPRINT .GE. 10)
     >      WRITE(IOUT,400) IGR,(IRNBM(I),I=1,NBNRS)
            IF(START) THEN
*           USE A VERY CHEAP APPROXIMATION TO START ITERATIONS.
              DO 70 IBM=1,NBMIX
                SIG0(IBM)=0.0
   70         CONTINUE
              DO 80 I=1,NBNRS
                SIG0(IRNBM(I))=SIGRES(I)
   80         CONTINUE
              CALL SHIPST(IPLIB ,IPTRK ,IFTRAK,CDOOR ,IPRINT,NBMIX ,
     >                    NREG  ,MAT   ,VOL   ,LEAKSW,1     ,IRES  ,
     >                    SIG0  ,SIGT(1,IGR)  ,SIGT3(1,IGR) ,LDIAG ,
     >                    TITR  ,DILAV         )
              DO 90 IALP=1,NALPHA
                SIGX(IALP)=0.0
                DILUT(IALP)=DILAV
   90         CONTINUE
            ELSE
              SIGX(3)=SIGE(INRS,IGR)
              SIGX(1)=0.01*SIGX(3)
              SIGX(2)=0.1*SIGX(3)
              SIGX(4)=10.0*SIGX(3)
              SIGX(5)=100.0*SIGX(3)
              DO 120 IALP=1,NALPHA
                DO 100 IBM=1,NBMIX
                  SIG0(IBM)=0.0
                  SIG3(IBM)=SIGT3(IBM,IGR)
  100           CONTINUE
                DO 110 I=1,NBNRS
                  SIG0(IRNBM(I))=SIGX(IALP)
                  SIG3(IRNBM(I))=0.0
  110           CONTINUE
                CALL SHIPST(IPLIB ,IPTRK ,IFTRAK,CDOOR ,IPRINT,NBMIX ,
     >                      NREG  ,MAT   ,VOL   ,LEAKSW,1     ,IRES  ,
     >                      SIG0  ,SIGT(1,IGR)  ,SIG3         ,LDIAG ,
     >                      TITR  ,DILUT(IALP)  )
  120         CONTINUE
            ENDIF
            CALL LCMSIX(IPLIB,CGRPNM,ILCMDN)
*
*        COMPUTE AVERAGE MACROSCOPIC DILUTION X-S (SIGE) USING A THREE
*        TERMS RATIONAL APPROXIMATION.
*        ******************************************************
            NRAT=(NALPHA+1)/2
            CALL SHIRAT(IPRINT,NRAT  ,SIGX  ,DILUT ,IGR   ,SA    ,
     >                  COEF  ,DENOM )
*        ******************************************************
            EAV=(COEF(1)*SQRT(DENOM(1))+COEF(2)*SQRT(DENOM(2))+
     >           COEF(3)*SQRT(DENOM(3)))**2
            SIGE(INRS,IGR)=REAL(EAV)
*
*        COMPUTE THE ISOTOPE DILUTION MICROSCOPIC CROSS SECTIONS (SN)
*        AND A SIMILAR VALUE (SB) USED IN L-J NORMALIZATION.
            IF(.NOT. START .AND. BIEFF) THEN
*           COMPUTE DILAV FOR THE L-J NORMALIZATION.
              IF(NBNRS .EQ. 1) THEN
                PXX=REAL(COEF(1)/(SIGRES(1)+DENOM(1))+COEF(2)/
     1          (SIGRES(1)+DENOM(2))+COEF(3)/(SIGRES(1)+DENOM(3)))
                DILAV=1.0/PXX-SIGRES(1)
              ELSE
                DO 140 IBM=1,NBMIX
                  SIG0(IBM)=0.0
  140           CONTINUE
                DO 150 I=1,NBNRS
                  SIG0(IRNBM(I))=SIGRES(I)
  150           CONTINUE
                CALL SHIPST(IPLIB ,IPTRK ,IFTRAK,CDOOR ,IPRINT,NBMIX ,
     >                      NREG  ,MAT   ,VOL   ,LEAKSW,1     ,IRES  ,
     >                      SIG0  ,SIGT(1,IGR)  ,SIGT3(1,IGR) ,LDIAG ,
     >                      TITR  ,DILAV        )
              ENDIF
            ENDIF
            DO 180 ISO=1,NBISO
              IF(LSHI(ISO) .EQ. INRS .AND.
     >           IRES(MIX(ISO)) .EQ. 1 .AND.
     >           DEN(ISO) .NE. 0. ) THEN
                SUM=0.0
                SUB=0.0
                DO 160 JSO=1,NBISO
                  IBM=MIX(JSO)
                  IF(LSHI(JSO) .EQ. INRS .AND.
     >               IBM .EQ. MIX(ISO)   .AND.
     >               ISO .NE. JSO) THEN
                    JBM=MIX2(JSO)
                    SUM=SUM+
     >              (TOTAL(IGR,JBM)-SIGOLD(IGR,JBM))*DEN(JSO)
                    SUB=SUB+TOTAL(IGR,JBM)*DEN(JSO)
                  ENDIF
  160           CONTINUE
                SN(IGR,ISO)=REAL(COEF(1)*SQRT(DENOM(1)+SUM)+COEF(2)*
     >             SQRT(DENOM(2)+SUM)+COEF(3)*SQRT(DENOM(3)+SUM))**2
     >             /DEN(ISO)
                IF(SN(IGR,ISO) .LE. 0.0) THEN
                  WRITE(HSMG,510) NAMSBR,(ISONAM(ITC,ISO),ITC=1,3),
     >            SN(IGR,ISO),IGR
                  CALL XABORT(HSMG)
                ENDIF
                IF(START .OR. .NOT.BIEFF) THEN
                  SB(IGR,ISO)=SN(IGR,ISO)
                ELSE
                  SB(IGR,ISO)=(DILAV+SUB)/DEN(ISO)
                  IF(SB(IGR,ISO) .LT. 0.0) THEN
                    WRITE(HSMG,515) NAMSBR,(ISONAM(ITC,ISO),ITC=1,3),
     >               SB(IGR,ISO),IGR
                    CALL XABORT(HSMG)
                  ELSE IF(SB(IGR,ISO) .LT. SN(IGR,ISO)) THEN
                    WRITE(IOUT,520) NAMSBR,(ISONAM(ITC,ISO),ITC=1,3),
     >              SB(IGR,ISO)/SN(IGR,ISO),IGR
                    SB(IGR,ISO)=SN(IGR,ISO)
                  ENDIF
                ENDIF
              ELSE IF(LSHI(ISO) .EQ. INRS .AND.
     >                IRES(MIX(ISO)) .EQ. 1 .AND.
     >                DEN(ISO) .EQ. 0.) THEN
                SN(IGR,ISO)=1.0E10
                SB(IGR,ISO)=1.0E10
              ENDIF
  180       CONTINUE
*
*        RESTORE SIGT ARRAY.
            IF(IPRINT .GE. 10) THEN
              DO 190 I=1,NBNRS
                SIG0(IRNBM(I))=SIGRES(I)
  190         CONTINUE
              WRITE(IOUT,405) (SIG0(I),I=1,NBMIX)
              WRITE(IOUT,410) (SIGT(I,IGR),I=1,NBMIX)
            ENDIF
            DO 200 I=1,NBNRS
              IF(IPRINT .GE. 10) THEN
                PP=SA-SIGT(IRNBM(I),IGR)
                QQ=SIGE(INRS,IGR)-SIGT(IRNBM(I),IGR)
                IF(ABS(PP).GT.1.0E-4*SIGT(IRNBM(I),IGR)) THEN
                  BEL=QQ/PP
                ELSE
                  BEL=0.0
                ENDIF
                WRITE(IOUT,610) I,SIGE(INRS,IGR),BEL
              ENDIF
              SIGT(IRNBM(I),IGR)=SIGT(IRNBM(I),IGR)+SIGRES(I)
  200       CONTINUE
          ENDIF
  211   CONTINUE
  210 CONTINUE
      CALL LCMSIX(IPLIB,'SHIBA',ILCMDN)
*
* COMPUTE THE SELF-SHIELDED MICROSCOPIC CROSS SECTIONS AND UPDATE
* VECTOR SIGT.
      DO 290 ISO=1,NBISO
        LOGDO=START.OR.(DEN(ISO).NE.0.)
        MASKI(ISO)=(LSHI(ISO).GT.0).AND.LOGDO
  290 CONTINUE
      IPRNT2=MAX(0,IPRINT-10)
      CALL XDRSET(ITISOM,NBISO,0)
      CALL LCMGET(IPLIB,'STATE-VECTOR',ISTATE)
      NGFR=ISTATE(10)
      CALL LIBLIB(IPLIB,IPRNT2,NBISO,NBISO,NGRO,ISONAM,ISONRF,
     >            IHLIB,NTFG,TN,AWR,SN,SB,MASKI,NL,ITIME,
     >            NEDMAC,HVECT,ITRANC,ITISOM,DELTA,NGFR,ISOADD)
      DO 320 ISO=1,NBISO
        IBM=MIX(ISO)
        IF(LSHI(ISO) .GT. 0 .AND.
     >     IBM .GT. 0 .AND.
     >     DEN(ISO) .NE. 0.) THEN
          WRITE(HNAMIS,'(3A4)') (ISONAM(ITC,ISO),ITC=1,3)
          CALL LCMSIX(IPLIB,HNAMIS,ILCMUP)
          CALL LCMGET(IPLIB,'TOTAL',GAR)
          DO 300 IGR=1,NGRO
            TOTAL(IGR,MIX2(ISO))=TOTAL(IGR,MIX2(ISO))-GAR(IGR)
  300     CONTINUE
          CALL LCMSIX(IPLIB,HNAMIS,ILCMDN)
          DO 310 IGR=1,NGRO
            IF(NOCONV(IBM,IGR))
     >      SIGT(IBM,IGR)=SIGT(IBM,IGR)-DEN(ISO)*TOTAL(IGR,MIX2(ISO))
  310     CONTINUE
        ENDIF
  320 CONTINUE
      RETURN
*
  380 FORMAT(/1X,A6,': USE PIC APPROXIMATION TO MERGE',I3,
     >' RESONANT MIXTURES IN RESONANT REGION NUMBER ',I3)
  390 FORMAT(/1X,A6,
     >': GOLDSTEIN AND COHEN APPROXIMATION USED FOR ISOTOPE ',A12)
  400 FORMAT(//' GROUP NUMBER',I4,10X,'RESONANT MIXTURE NUMBERS',16I5/
     1 (1X,26I5))
  405 FORMAT(//' TOTAL MACROSCOPIC CROSS SECTIONS OF THE RESONANT ',
     1 'MATERIALS IN EACH MIXTURE :'/(1X,1P,11E11.3))
  410 FORMAT(//' TOTAL MACROSCOPIC CROSS SECTIONS OF THE OTHER ',
     1 'MATERIALS IN EACH MIXTURE :'/(1X,1P,11E11.3))
  440 FORMAT(//' VOLUME NB.              :',11(I5,3X,1HI))
  450 FORMAT(  ' MIXTURE (MAT)           :',11(I5,3X,1HI))
  460 FORMAT(//'  SIGX:',1P,5E11.4/' DILUT:',5E11.4/)
  470 FORMAT(/' RATIONAL APPROXIMATION COEFFICIENTS FOR ',
     >'FUEL-TO-FUEL REDUCED COLLISION PROBABILITIES OF ',
     >'THE CLOSED CELL:'/1P,
     > 7X,'  NUMERATOR ',3(' (',E11.4,1H,,E11.4,')')/
     > 7X,' DENOMINATOR',3(' (',E11.4,1H,,E11.4,')')/)
  510 FORMAT(A6,': THE RESONANT ISOTOPE- ',3A4,
     > ' -HAS A NEGATIVE DILUTION CROSS-SECTION (',1P,E14.4,0P,
     > ') IN GROUP ',I4)
  515 FORMAT(A6,': THE RESONANT ISOTOPE - ',3A4,
     > ' -HAS A NEGATIVE L-J CROSS-SECTION (',1P,E14.4,0P,
     > ') IN GROUP ',I4)
  520 FORMAT(A6,': THE L-J EQUIVALENCE FACTOR OF RESONANT ISOTOPE ',
     > 3A4,' WAS CHANGED FROM',F6.3,' TO 1.0 IN GROUP ',I4)
  590 FORMAT(/' MACROSCOPIC DILUTION CROSS SECTION =',1P,E12.4//)
  610 FORMAT(8X,'AVERAGE(',I2,')',1P,E13.5/
     >       8X,'BELL FACTOR ',E13.5)
 9000 FORMAT(A6,'INVALID RESONANT REGION =',I10)
      END
