*DECK SPHDIF
      SUBROUTINE SPHDIF(IPTRK2,KSPH,IPRINT,NREG,NUN,NMERGE,MAT,VOL,KEY,
     1 MERG,ILK,CTITRE,COURIN,SIGMA,DIFF0,NGCOND,FLXMER,SUNMER,FUNKNO,
     2 FLXTOT,IGR,IEX,SOURCE)
C
C----------------------------------------------------------------------
C
C DIFFUSION CALCULATION OVER THE MACRO-GEOMETRY IN A SINGLE CONDENSED
C GROUP.
C
C INPUT PARAMETERS:
C  IPTRK2 : POINTER TO THE BIVAC TRACKING OF THE MACRO-GEOMETRY
C           (L_TRACK SIGNATURE).
C  KSPH   : =1 : AVERAGE FLUX NORMALIZATION;
C           =2 OR 3 : SELENGUT NORMALIZATION. THE 'ALBS' OPTION SHOULD
C           BE USED INSIDE MODULE ASM:.
C  IPRINT : PRINT FLAG. EQUAL TO 0 FOR NO PRINT.
C  NREG   : NUMBER OF MACRO-REGIONS (IN THE MACRO CALCULATION).
C  NUN    : NUMBER OF UNKNOWNS IN THE MACRO-CALCULATION.
C  NMERGE : NUMBER OF MERGED REGIONS.
C  MAT    : MIXTURE INDEX PER MACRO-REGION. DIMENSION MAT(NREG)
C  VOL    : VOLUME OF MACRO-REGIONS. DIMENSION VOL(NREG)
C  KEY    : POSITION OF THE FLUX COMPONENTS ASSOCIATED WITH EACH
C           VOLUME. DIMENSION KEY(NREG)
C  MERG   : INDEX OF MERGED MACRO-REGIONS. DIMENSION MERG(NREG)
C  ILK    : LEAKAGE SWITCH.
C  CTITRE : TITLE.
C  COURIN : AVERAGED FLUX IF KSPH=1. 4 TIMES THE INCOMING CURRENT PER
C           UNIT SURFACE IF KSPH=2 OR 3.
C  SIGMA  : REMOVAL MACROSCOPIC X-SECTIONS. DIMENSION SIGMA(NBMIX)
C           WHERE NBMIX=MAX(MAT(I))
C  DIFF0  : DIFFUSION COEFFICIENTS. DIMENSION DIFF(NBMIX)
C  NGCOND : NUMBER OF GROUPS CONDENSED.
C  FLXMER : FLUX ESTIMATE PER MIXTURE. DIMENSION FLXMER(NMERGE,NGCOND)
C  SUNMER : INCOMING SOURCE (SCATTERING+FISSION) X-SECTIONS.
C           DIMENSION SUNMER(NMERGE,NGCOND)
C  IGR    : ENERGY GROUP INDEX.
C  IEX    : ITERATION INDEX.
C
C OUTPUT PARAMETERS:
C  FUNKNO : NEUTRON FLUX. DIMENSION FUNKNO(NUN,NGCOND)
C  FLXTOT : AVERAGED FLUX.
C
C SCRATCH STORAGE
C  SOURCE : DIMENSION SOURCE(NUN)
C
C------------------------------------- AUTHOR: A. HEBERT ; 15/04/91 ----
C
      IMPLICIT NONE
C----
C PARAMETERS
C----
      INTEGER   NSTATE
      PARAMETER(NSTATE=40)
C----
C MEMORY ALLOCATION
C----
      INTEGER      IBASE(1)
      REAL         RBASE(1),BASE(1)
      COMMON       RBASE
      EQUIVALENCE (RBASE(1),IBASE(1),BASE(1))
      INTEGER      IXX,IYY,IDD,IKN,IQFR,IBFR,IMU,IGAR1,IGAR2,
     1             IR,IRS,IT,ITS,IRH,IRT,IA11,IQ,IQS,IV,IQH,IQT
C----
C INTERFACE VARIABLES
C----
      INTEGER      IPTRK2,KSPH,IPRINT,NREG,NUN,NMERGE
      INTEGER      MAT(*),KEY(*),MERG(*),IGP(NSTATE)
      REAL         VOL(*),SIGMA(*),DIFF0(*),SOURCE(*),
     1             FLXMER(NMERGE,*),SUNMER(NMERGE,*),FUNKNO(NUN,*)
      CHARACTER    CTITRE*72
C-----
C LOCAL VARIABLES
C-----
      INTEGER      ITYPE,IELEM,ICOL,ISPLH,LL4,LX,LY,MAXKN,ITYLCM,
     1             MAXQF,NBMIX,LC,I,IEX,NGCOND,IGR,NELEM,IUNK,IREG,
     2             IUN
      REAL         SIDE,COUR,COURIN,VOLTOT,FLXTOT
      LOGICAL      ILK,CYLIND
C
C RECOVER BIVAC SPECIFIC TRACKING INFORMATION.
C
      CALL XDRSET(IGP,NSTATE,0)
      CALL LCMGET(IPTRK2,'STATE-VECTOR',IGP)
      ITYPE=IGP(6)
      CYLIND=(ITYPE.EQ.3).OR.(ITYPE.EQ.6)
      IELEM=IGP(8)
      ICOL=IGP(9)
      ISPLH=IGP(10)
      LL4=IGP(11)
      LX=IGP(12)
      LY=IGP(13)
      CALL LCMLEN(IPTRK2,'KN',MAXKN,ITYLCM)
      CALL LCMLEN(IPTRK2,'QFR',MAXQF,ITYLCM)
      CALL SETARA(BASE,LX*LY,IXX)
      CALL SETARA(BASE,LX*LY,IYY)
      CALL SETARA(BASE,LX*LY,IDD)
      CALL SETARA(IBASE,MAXKN,IKN)
      CALL SETARA(BASE,MAXQF,IQFR)
      CALL SETARA(BASE,MAXQF,IBFR)
      CALL SETARA(IBASE,LL4,IMU)
      IF(ITYPE.EQ.8) THEN
         CALL LCMGET(IPTRK2,'SIDE',SIDE)
      ELSE
         CALL LCMGET(IPTRK2,'XX',BASE(IXX))
         CALL LCMGET(IPTRK2,'YY',BASE(IYY))
         CALL LCMGET(IPTRK2,'DD',BASE(IDD))
      ENDIF
      CALL LCMGET(IPTRK2,'KN',IBASE(IKN))
      CALL LCMGET(IPTRK2,'QFR',BASE(IQFR))
      CALL LCMGET(IPTRK2,'BFR',BASE(IBFR))
      CALL LCMGET(IPTRK2,'MU',IBASE(IMU))
*
* COMPUTE THE NEUTRON SOURCES IN GROUP IGR.
      CYLIND=(ITYPE.EQ.3).OR.(ITYPE.EQ.6)
      COUR=0.0
      IF(KSPH.GT.1) COUR=COURIN
      NBMIX=0
      DO 10 I=1,NREG
   10 NBMIX=MAX(NBMIX,MAT(I))
      CALL SETARA(BASE,NBMIX,IGAR1)
      CALL SETARA(BASE,NBMIX,IGAR2)
      IF((IELEM.LT.0).AND.(ITYPE.NE.8)) THEN
         CALL LCMSIX(IPTRK2,'BIVCOL',1)
         CALL LCMLEN(IPTRK2,'T',LC,ITYLCM)
         CALL SETARA(BASE,LC*LC,IR)
         CALL SETARA(BASE,LC*LC,IRS)
         CALL SETARA(BASE,LC,IT)
         CALL SETARA(BASE,LC,ITS)
         CALL LCMGET(IPTRK2,'R',BASE(IR))
         CALL LCMGET(IPTRK2,'RS',BASE(IRS))
         CALL LCMGET(IPTRK2,'T',BASE(IT))
         CALL LCMGET(IPTRK2,'TS',BASE(ITS))
         CALL LCMSIX(IPTRK2,' ',2)
         CALL BIVFSO(IEX,NGCOND,IGR,NMERGE,FLXMER,SUNMER,CYLIND,NREG,
     1   NUN,BASE(IXX),BASE(IDD),MAT,IBASE(IKN),BASE(IQFR),VOL,MERG,
     2   COUR,FUNKNO,LC,BASE(IT),BASE(ITS),BASE(IR),BASE(IRS),SOURCE,
     3   BASE(IGAR1),BASE(IGAR2))
         CALL RLSARA(BASE(ITS))
         CALL RLSARA(BASE(IT))
         CALL RLSARA(BASE(IRS))
         CALL RLSARA(BASE(IR))
      ELSE IF((IELEM.GT.0).AND.(ITYPE.NE.8)) THEN
         CALL BIVGSO(IEX,NGCOND,IGR,NMERGE,FLXMER,SUNMER,CYLIND,IELEM,
     1   NREG,NUN,MAT,BASE(IXX),BASE(IDD),IBASE(IKN),BASE(IQFR),VOL,
     2   MERG,COUR,FUNKNO,SOURCE,BASE(IGAR1),BASE(IGAR2))
      ELSE IF((IELEM.LT.0).AND.(ITYPE.EQ.8)) THEN
         CALL LCMSIX(IPTRK2,'BIVCOL',1)
         CALL SETARA(BASE,2,IT)
         CALL SETARA(BASE,36,IRH)
         CALL SETARA(BASE,9,IRT)
         CALL LCMGET(IPTRK2,'T',BASE(IT))
         CALL LCMGET(IPTRK2,'RH',BASE(IRH))
         CALL LCMGET(IPTRK2,'RT',BASE(IRT))
         CALL LCMSIX(IPTRK2,' ',2)
         IF(ISPLH.EQ.1) THEN
            NELEM=MAXKN/7
         ELSE
            NELEM=MAXKN/4
         ENDIF
         CALL BIVFSH (IEX,NGCOND,IGR,NMERGE,FLXMER,SUNMER,NREG,NUN,
     1   ISPLH,NELEM,SIDE,MAT,IBASE(IKN),BASE(IQFR),VOL,MERG,COUR,
     2   FUNKNO,BASE(IT),BASE(IRH),BASE(IRT),SOURCE,BASE(IGAR1),
     3   BASE(IGAR2))
         CALL RLSARA(BASE(IT))
         CALL RLSARA(BASE(IRT))
         CALL RLSARA(BASE(IRH))
      ELSE IF((IELEM.GT.0).AND.(ITYPE.EQ.8)) THEN
         CALL BIVGSH(IEX,NGCOND,IGR,NMERGE,FLXMER,SUNMER,NREG,NUN,LL4,
     1   ISPLH,SIDE,MAT,IBASE(IKN),BASE(IQFR),VOL,DIFF0,MERG,COUR,
     2   FUNKNO,SOURCE,BASE(IGAR1),BASE(IGAR2))
      ENDIF
      CALL RLSARA(BASE(IGAR2))
      CALL RLSARA(BASE(IGAR1))
      IF(IPRINT.GT.0) THEN
         WRITE(6,210) 'SOURCE',(SOURCE(IUNK),IUNK=1,NUN)
      ENDIF
*
* COMPUTE THE NEUTRON FLUX IN GROUP IGR.
      CALL SETARA(BASE,IBASE(IMU+LL4-1),IA11)
      IF((IELEM.LT.0).AND.(ITYPE.NE.8)) THEN
         CALL LCMSIX(IPTRK2,'BIVCOL',1)
         CALL LCMLEN(IPTRK2,'T',LC,ITYLCM)
         CALL SETARA(BASE,LC,IT)
         CALL SETARA(BASE,LC,ITS)
         CALL SETARA(BASE,LC*LC,IR)
         CALL SETARA(BASE,LC*LC,IRS)
         CALL SETARA(BASE,LC*LC,IQ)
         CALL SETARA(BASE,LC*LC,IQS)
         CALL LCMGET(IPTRK2,'T',BASE(IT))
         CALL LCMGET(IPTRK2,'TS',BASE(ITS))
         CALL LCMGET(IPTRK2,'R',BASE(IR))
         CALL LCMGET(IPTRK2,'RS',BASE(IRS))
         CALL LCMGET(IPTRK2,'Q',BASE(IQ))
         CALL LCMGET(IPTRK2,'QS',BASE(IQS))
         CALL LCMSIX(IPTRK2,' ',2)
         CALL BIVF2D(DIFF0,SIGMA,CYLIND,NREG,NUN,LL4,BASE(IXX),
     1   BASE(IYY),BASE(IDD),MAT,IBASE(IKN),BASE(IQFR),BASE(IBFR),
     2   VOL,KEY,IBASE(IMU),SOURCE,LC,BASE(IT),BASE(ITS),BASE(IR),
     3   BASE(IRS),BASE(IQ),BASE(IQS),FUNKNO(1,IGR),BASE(IA11))
         CALL RLSARA(BASE(IQS))
         CALL RLSARA(BASE(IQ))
         CALL RLSARA(BASE(IRS))
         CALL RLSARA(BASE(IR))
         CALL RLSARA(BASE(ITS))
         CALL RLSARA(BASE(IT))
      ELSE IF((IELEM.GT.0).AND.(ITYPE.NE.8)) THEN
         CALL LCMSIX(IPTRK2,'BIVCOL',1)
         CALL LCMLEN(IPTRK2,'T',LC,ITYLCM)
         CALL SETARA(BASE,LC*LC,IR)
         CALL SETARA(BASE,LC*(LC-1),IV)
         CALL LCMGET(IPTRK2,'R',BASE(IR))
         CALL LCMGET(IPTRK2,'V',BASE(IV))
         CALL LCMSIX(IPTRK2,' ',2)
         CALL BIVG2D(DIFF0,SIGMA,CYLIND,IELEM,ICOL,NREG,NUN,LL4,
     1   BASE(IXX),BASE(IYY),BASE(IDD),MAT,IBASE(IKN),BASE(IQFR),
     2   BASE(IBFR),VOL,IBASE(IMU),SOURCE,LC,BASE(IR),BASE(IV),
     3   FUNKNO(1,IGR),BASE(IA11))
         CALL RLSARA(BASE(IV))
         CALL RLSARA(BASE(IR))
      ELSE IF((IELEM.LT.0).AND.(ITYPE.EQ.8)) THEN
         CALL LCMSIX(IPTRK2,'BIVCOL',1)
         CALL SETARA(BASE,4,IR)
         CALL SETARA(BASE,36,IRH)
         CALL SETARA(BASE,36,IQH)
         CALL SETARA(BASE,9,IRT)
         CALL SETARA(BASE,9,IQT)
         CALL LCMGET(IPTRK2,'R',BASE(IR))
         CALL LCMGET(IPTRK2,'RH',BASE(IRH))
         CALL LCMGET(IPTRK2,'QH',BASE(IQH))
         CALL LCMGET(IPTRK2,'RT',BASE(IRT))
         CALL LCMGET(IPTRK2,'QT',BASE(IQT))
         CALL LCMSIX(IPTRK2,' ',2)
         IF(ISPLH.EQ.1) THEN
            NELEM=MAXKN/7
         ELSE
            NELEM=MAXKN/4
         ENDIF
         CALL BIVF2H(DIFF0,SIGMA,NREG,NUN,LL4,ISPLH,NELEM,SIDE,MAT,
     1   IBASE(IKN),BASE(IQFR),BASE(IBFR),VOL,KEY,IBASE(IMU),SOURCE,
     2   BASE(IR),BASE(IRH),BASE(IQH),BASE(IRT),BASE(IQT),FUNKNO(1,IGR),
     3   BASE(IA11))
         CALL RLSARA(BASE(IQT))
         CALL RLSARA(BASE(IRT))
         CALL RLSARA(BASE(IQH))
         CALL RLSARA(BASE(IRH))
         CALL RLSARA(BASE(IR))
      ELSE IF((IELEM.GT.0).AND.(ITYPE.EQ.8)) THEN
         CALL BIVG2H(DIFF0,SIGMA,NREG,NUN,LL4,ISPLH,SIDE,MAT,IBASE(IKN),
     1   BASE(IQFR),BASE(IBFR),VOL,KEY,IBASE(IMU),SOURCE,FUNKNO(1,IGR),
     2   BASE(IA11))
      ENDIF
      CALL RLSARA(BASE(IA11))
      IF(IPRINT.GT.0) THEN
         WRITE(6,210) 'FUNKNO',(FUNKNO(IUNK,IGR),IUNK=1,NUN)
      ENDIF
*
      VOLTOT=0.0
      FLXTOT=0.0
      DO 120 IREG=1,NREG
      VOLTOT=VOLTOT+VOL(IREG)
  120 FLXTOT=FLXTOT+FUNKNO(KEY(IREG),IGR)*VOL(IREG)
      FLXTOT=FLXTOT/VOLTOT
*
* FLUX NORMALIZATION.
      IF(KSPH.EQ.1) THEN
         IF(IPRINT.GT.0) WRITE(6,200) COURIN/FLXTOT
         DO 130 IUN=1,NUN
  130    FUNKNO(IUN,IGR)=FUNKNO(IUN,IGR)*COURIN/FLXTOT
         FLXTOT=COURIN
      ENDIF
*
* RELEASE BIVAC SPECIFIC TRACKING INFORMATION.
      CALL RLSARA(IBASE(IMU))
      CALL RLSARA(BASE(IBFR))
      CALL RLSARA(BASE(IQFR))
      CALL RLSARA(IBASE(IKN))
      CALL RLSARA(BASE(IDD))
      CALL RLSARA(BASE(IYY))
      CALL RLSARA(BASE(IXX))
      RETURN
  200 FORMAT(/35H SPHDIF: FLUX NORMALIZATION FACTOR=,1P,E13.5)
  210 FORMAT(/26H SPHDIF: VALUES OF VECTOR ,A6,4H ARE/(1X,1P,10E13.5))
      END
