*DECK SPHREF
      SUBROUTINE SPHREF(IPFLUX,IPMACR,IPSYS,IPRINT,ITRANC,ILEAKS,B2,
     1 NGROUP,NGCOND,NBMIX,NREGIO,NMERGE,NIFISS,MATCOD,VOLUME,KEYFLX,
     2 IGCOND,IMERGE,FLUXES,VOLMER,FLXMER,SUNMER,SIGT,SIGW,COURIN,CURH,
     3 PRODUC,SIGMA,NJJ,IJJ,IPOS,XSCAT,WORK,DIFF)
C
C-----------------------------------------------------------------------
C
C  CALCULATION OF THE REFERENCE MERGED/CONDENSED SET OF CROSS SECTIONS
C  TO BE USED BY AN SPH HOMOGENIZATION MODULE.
C
C INPUT PARAMETERS:
C   IPFLUX : POINTER TO THE REFERENCE SOLUTION (L_FLUX SIGNATURE).
C   IPMACR : POINTER TO THE REFERENCE MACROLIB (L_MACROLIB SIGNATURE).
C   IPSYS  : POINTER TO THE REFERENCE PIJ (L_PIJ SIGNATURE).
C   IPRINT : PRINT FLAG. EQUAL TO 0 FOR NO PRINT.
C   ITRANC : TYPE OF TRANSPORT CORRECTION.
C   ILEAKS : LEAK CALCULATION
C            = 0 NO LEAKAGE
C            = 1 LEAKAGE BY SURFACE (ALBEDO>0.0)
C            = 2 HOMOGENEOUS LEAKAGE COEFFICIENTS
C            = 3 DIRECTIONAL LEAKAGE COEFFICIENTS
C   B2     : SQUARE BUCKLING     DIMENSION B2(4)
C            FOR ILEAKS = 2 B2(4) IS HOMOGENEOUS
C            FOR ILEAKS = 3 B2(1),B2(2),B2(3)
C            ARE DIRECTIONAL HETEROGENEOUS
C            AND B2(4) IS HOMOGENEOUS
C   NGROUP : NUMBER OF GROUPS.
C   NGCOND : NUMBER OF GROUPS CONDENSED.
C   NBMIX  : NUMBER OF MIXTURES.
C   NREGIO : NUMBER OF REGIONS.
C   NMERGE : NUMBER OF REGIONS MERGED.
C   NIFISS : NUMBER OF FISSILE ISOTOPES.
C   MATCOD : MIXTURE INDEX PER REGION. DIMENSION MATCOD(NREGIO)
C   VOLUME : VOLUME OF REGIONS. DIMENSION VOLUME(NREGIO)
C   KEYFLX : POSITION OF AVERAGE FLUXES. DIMENSION KEYFLX(NREGIO)
C   IGCOND : LIMIT CONDENSED GROUPS. DIMENSION IGCOND(NGCOND)
C   IMERGE : INDEX OF MERGED REGIONS. DIMENSION IMERGE(NREGIO)
C   FLUXES : FLUXES. DIMENSION FLUXES(NREGIO,NGROUP)
C
C OUTPUT PARAMETERS:
C   VOLMER : MERGED VOLUMES. DIMENSION VOLMER(NMERGE)
C   FLXMER : MERGED/COND. FLUXES. DIMENSION FLXMER(NMERGE,NGCOND)
C   SUNMER : MERGED/COND. PRODUCTION (FISSION + SCATTERING) CROSS
C            SECTIONS. THE LAST DIMENSION IS FOR SECONDARY NEUTRONS.
C            DIMENSION SUNMER(NMERGE,NGCOND,NGCOND)
C   SIGT   : MERGED/COND. TOTAL CROSS SECTIONS.
C   SIGW   : MERGED/COND. WITHIN-GROUP SCATTERING CROSS SECTIONS.
C            DIMENSION SIGT(NMERGE,NGCOND),SIGW(NMERGE,NGCOND)
C   COURIN : INGOING CURRENTS (4*J-/S). THEY ARE COMPUTED IF PIS INFOR-
C            MATION IS AVAILABLE ON LCM, OTHERWISE THEY ARE SET TO 0.0.
C            DIMENSION COURIN(NGCOND)
C   CURH   : AVERAGED HETEROGENEOUS B-N CURRENTS. COMPUTED IF ILEAKS=3.
C            DIMENSION CURH(NREGIO,NGROUP,3)
C
C SCRATCH STORAGE:
C   PRODUC : DIMENSION PRODUC(NREGIO,NGCOND,NIFISS)
C   SIGMA  : DIMENSION SIGMA(NBMIX*NIFISS+1)
C   NJJ    : DIMENSION NJJ(NBMIX)
C   IJJ    : DIMENSION IJJ(NBMIX)
C   IPOS   : DIMENSION IPOS(NBMIX)
C   XSCAT  : DIMENSION XSCAT(NBMIX*NGROUP)
C   WORK   : DIMENSION WORK(NREGIO,2)
C   DIFF   : DIMENSION DIFF(NGROUP)
C
C------------------------------------- AUTHOR: A. HEBERT ; 25/11/88 ----
C
      IMPLICIT NONE
C----
C MEMORY ALLOCATION
C----
      INTEGER      IBASE(1)
      REAL          RBASE
      COMMON        RBASE(1)
      EQUIVALENCE  (RBASE(1),IBASE(1))
      INTEGER      IWORK,JSIG1
C----
C INTERFACE VARIABLES
C----
      INTEGER     IPFLUX,IPMACR,IPSYS,IPRINT,ITRANC,ILEAKS,
     >            NGROUP,NGCOND,NBMIX,NREGIO,NMERGE,NIFISS
      INTEGER     MATCOD(*),KEYFLX(*),IGCOND(*),IMERGE(*),NJJ(*),IJJ(*),
     >            IPOS(*)
      REAL        VOLUME(*),FLUXES(NREGIO,*),VOLMER(*),FLXMER(NMERGE,*),
     >            SUNMER(NMERGE,NGCOND,*),SIGT(NMERGE,*),SIGW(NMERGE,*),
     >            COURIN(*),PRODUC(NREGIO,NGCOND,*),SIGMA(0:*),XSCAT(*),
     >            WORK(NREGIO,*),DIFF(*),CURH(NREGIO,NGROUP,*),B2(4)
C----
C LOCAL VARIABLES
C----
      INTEGER          ILCMLN,IGR,IREG,IFIS,IREGIO,IGRFIN,IGRCND,ITYLCM,
     1                 IGRDEB,MATNUM,NEWREG,JGRCND,IMAT,I,J2,J1,
     2                 JGRFIN,JGRDEB,IPO,JGR
      REAL             ZLEAK,FACTOR,EIGENK
      CHARACTER        CGRPNM*12,NAMREC*12
      DOUBLE PRECISION SS,SUM,SUD
C
C RECOVER THE AVERAGE FLUX AND CURRENT REFERENCE INFORMATION.
C
      CALL LCMLEN(IPFLUX,'FLUX  1',ILCMLN,ITYLCM)
      IF(ILCMLN.EQ.0) CALL XABORT('SPHREF: MISSING FLUX INFO.')
      CALL SETARA(RBASE,ILCMLN,IWORK)
      IF(ILEAKS.EQ.3) THEN
         DO 10 IGR=1,NGROUP
         WRITE(NAMREC,'(4HFLUX,I3)') IGR
         CALL LCMGET(IPFLUX,NAMREC,RBASE(IWORK))
         DO 10 IREG=1,NREGIO
         CURH(IREG,IGR,1)=RBASE(IWORK+KEYFLX(IREG)+NREGIO-1)
         CURH(IREG,IGR,2)=RBASE(IWORK+KEYFLX(IREG)+2*NREGIO-1)
   10    CURH(IREG,IGR,3)=RBASE(IWORK+KEYFLX(IREG)+3*NREGIO-1)
      ENDIF
      DO 15 IGR=1,NGROUP
      WRITE(NAMREC,'(4HFLUX,I3)') IGR
      CALL LCMGET(IPFLUX,NAMREC,RBASE(IWORK))
      DO 15 IREG=1,NREGIO
   15 FLUXES(IREG,IGR)=RBASE(IWORK+KEYFLX(IREG)-1)
      CALL RLSARA(RBASE(IWORK))
*
* COMPUTE THE FISSION RATE INFORMATION.
      SIGMA(0)=0.0
      DO 20 IFIS=1,NIFISS
      DO 20 IGRCND=1,NGCOND
      DO 20 IREGIO=1,NREGIO
   20 PRODUC(IREGIO,IGRCND,IFIS)=0.0
      IGRFIN=0
      DO 40 IGRCND=1,NGCOND
      IGRDEB=IGRFIN+1
      IGRFIN=IGCOND(IGRCND)
      DO 40 IGR=IGRDEB,IGRFIN
      WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGROUP
      CALL LCMSIX(IPMACR,CGRPNM,1)
      CALL LCMLEN(IPMACR,'NUSIGF',ILCMLN,ITYLCM)
      IF(ILCMLN.GT.0) THEN
         CALL LCMGET(IPMACR,'NUSIGF',SIGMA(1))
         DO 30 IFIS=1,NIFISS
         DO 30 IREGIO=1,NREGIO
         MATNUM=MATCOD(IREGIO)
         IF(MATNUM.GT.0) THEN
            SS=FLUXES(IREGIO,IGR)*SIGMA((IFIS-1)*NBMIX+MATNUM)
            PRODUC(IREGIO,IGRCND,IFIS)=PRODUC(IREGIO,IGRCND,IFIS)+SS
         ENDIF
  30     CONTINUE
      ENDIF
      CALL LCMSIX(IPMACR,' ',2)
  40  CONTINUE
      CALL LCMLEN(IPFLUX,'1/EIGENVALUE',ILCMLN,ITYLCM)
      IF(ILCMLN.EQ.1) THEN
         CALL LCMGET(IPFLUX,'1/EIGENVALUE',EIGENK)
      ELSE
         EIGENK=1.0
      ENDIF
      IF(IPRINT.GT.5) WRITE(6,'(/16H SPHREF: EIGENK=,1P,E12.4)') EIGENK
*
* COMPUTE MERGED/CONDENSED CROSS SECTIONS.
      IF(ILEAKS.EQ.2) THEN
         CALL LCMGET(IPFLUX,'DIFFB1HOM',DIFF)
      ENDIF
      IGRFIN=0
      DO 170 IGRCND=1,NGCOND
      DO 45 NEWREG=1,NMERGE
      FLXMER(NEWREG,IGRCND)=0.0
      SIGT(NEWREG,IGRCND)=0.0
      SIGW(NEWREG,IGRCND)=0.0
      DO 45 JGRCND=1,NGCOND
   45 SUNMER(NEWREG,JGRCND,IGRCND)=0.0
      COURIN(IGRCND)=0.0
      IGRDEB=IGRFIN+1
      IGRFIN=IGCOND(IGRCND)
      DO 170 IGR=IGRDEB,IGRFIN
      WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGROUP
      CALL LCMSIX(IPMACR,CGRPNM,1)
      CALL LCMLEN(IPMACR,'TOTAL',ILCMLN,ITYLCM)
      IF(ILCMLN.GT.0) THEN
         CALL LCMGET(IPMACR,'TOTAL',SIGMA(1))
      ELSE
         CALL XABORT('SPHREF: READ ERROR ON LCM RECORD= TOTAL.')
      ENDIF
      IF(ITRANC.NE.0) THEN
*        TRANSPORT CORRECTION.
         CALL LCMLEN(IPMACR,'TRANC',ILCMLN,ITYLCM)
         IF(ILCMLN.GT.0) THEN
         CALL SETARA(RBASE,NBMIX,JSIG1)
         CALL LCMGET(IPMACR,'TRANC',RBASE(JSIG1))
         DO 50 IMAT=1,NBMIX
   50    SIGMA(IMAT)=SIGMA(IMAT)-RBASE(JSIG1+IMAT-1)
         CALL RLSARA(RBASE(JSIG1))
         ENDIF
      ENDIF
      DO 60 IREGIO=1,NREGIO
      MATNUM=MATCOD(IREGIO)
      NEWREG=IMERGE(IREGIO)
      IF(ILEAKS.EQ.2) THEN
        ZLEAK=DIFF(IGR)*B2(4)*FLUXES(IREGIO,IGR)
      ELSE IF(ILEAKS.EQ.3) THEN
        ZLEAK=CURH(IREGIO,IGR,1)*B2(1)+CURH(IREGIO,IGR,2)*B2(2)
     >       +CURH(IREGIO,IGR,3)*B2(3)
      ELSE
        ZLEAK=0.0
      ENDIF
      IF(NEWREG.GT.0) THEN
        FLXMER(NEWREG,IGRCND)=FLXMER(NEWREG,IGRCND)
     >                       +FLUXES(IREGIO,IGR)*VOLUME(IREGIO)
        SIGT(NEWREG,IGRCND)=SIGT(NEWREG,IGRCND)+SIGMA(MATNUM)
     >                     *FLUXES(IREGIO,IGR)*VOLUME(IREGIO)
        SIGW(NEWREG,IGRCND)=SIGW(NEWREG,IGRCND)-ZLEAK*VOLUME(IREGIO)
        SUNMER(NEWREG,IGRCND,IGRCND)=SUNMER(NEWREG,IGRCND,IGRCND)
     >                              -ZLEAK*VOLUME(IREGIO)
      ENDIF
   60 WORK(IREGIO,1)=-ZLEAK
*
      CALL LCMLEN(IPMACR,'CHI',ILCMLN,ITYLCM)
      IF(ILCMLN.GT.0) THEN
         CALL LCMGET(IPMACR,'CHI',SIGMA(1))
         DO 80 IFIS=1,NIFISS
         DO 80 IREGIO=1,NREGIO
         MATNUM=MATCOD(IREGIO)
         NEWREG=IMERGE(IREGIO)
         IF(MATNUM.GT.0) THEN
            DO 70 JGRCND=1,NGCOND
            SS=SIGMA((IFIS-1)*NBMIX+MATNUM)*PRODUC(IREGIO,JGRCND,IFIS)/
     >        EIGENK
            IF(NEWREG.GT.0) SUNMER(NEWREG,JGRCND,IGRCND)=
     >      SUNMER(NEWREG,JGRCND,IGRCND)+SS*VOLUME(IREGIO)
   70       WORK(IREGIO,1)=WORK(IREGIO,1)+SS
         ENDIF
   80    CONTINUE
      ENDIF
*
      CALL LCMLEN(IPMACR,'SIGW 0',ILCMLN,ITYLCM)
      IF(ILCMLN.GT.0) THEN
         CALL LCMGET(IPMACR,'SIGW 0',SIGMA(1))
      ELSE
         DO 100 I=1,NBMIX
  100    SIGMA(I)=0.0
      ENDIF
      IF(ITRANC.NE.0) THEN
*        TRANSPORT CORRECTION.
         CALL LCMLEN(IPMACR,'TRANC',ILCMLN,ITYLCM)
         IF(ILCMLN.GT.0) THEN
         CALL SETARA(RBASE,NBMIX,JSIG1)
         CALL LCMGET(IPMACR,'TRANC',RBASE(JSIG1))
         DO 120 IMAT=1,NBMIX
  120    SIGMA(IMAT)=SIGMA(IMAT)-RBASE(JSIG1+IMAT-1)
         CALL RLSARA(RBASE(JSIG1))
         ENDIF
      ENDIF
      CALL LCMLEN(IPMACR,'NJJD 0',ILCMLN,ITYLCM)
      IF(ILCMLN.GT.0) THEN
         CALL LCMGET(IPMACR,'NJJD 0',NJJ)
         CALL LCMGET(IPMACR,'IJJD 0',IJJ)
         CALL LCMGET(IPMACR,'IPOD 0',IPOS)
         CALL LCMGET(IPMACR,'SCAD 0',XSCAT)
         DO 150 IREGIO=1,NREGIO
         MATNUM=MATCOD(IREGIO)
         NEWREG=IMERGE(IREGIO)
         IF(MATNUM.GT.0) THEN
            JGRFIN=0
            DO 140 JGRCND=1,NGCOND
            SS=0.0D0
            JGRDEB=JGRFIN+1
            JGRFIN=IGCOND(JGRCND)
            J2=MIN(JGRFIN,IJJ(MATNUM))
            J1=MAX(JGRDEB,IJJ(MATNUM)-NJJ(MATNUM)+1)
            IPO=IPOS(MATNUM)+IJJ(MATNUM)-J2
            DO 130 JGR=J2,J1,-1
            IF(IGR.EQ.JGR) THEN
               SS=SS+SIGMA(MATNUM)*FLUXES(IREGIO,JGR)
            ELSE
               SS=SS+XSCAT(IPO)*FLUXES(IREGIO,JGR)
            ENDIF
  130       IPO=IPO+1
            IF(NEWREG.GT.0) THEN
               IF(IGRCND.EQ.JGRCND)
     >         SIGW(NEWREG,IGRCND)=SIGW(NEWREG,IGRCND)+SS*VOLUME(IREGIO)
               SUNMER(NEWREG,JGRCND,IGRCND)=SUNMER(NEWREG,JGRCND,IGRCND)
     >                                     +SS*VOLUME(IREGIO)
            ENDIF
  140       WORK(IREGIO,1)=WORK(IREGIO,1)+SS
         ENDIF
  150    CONTINUE
      ENDIF
*
      IF(IPSYS.NE.0) THEN
         CALL LCMSIX(IPSYS,CGRPNM,1)
         CALL LCMLEN(IPSYS,'DRAGON-PIS',ILCMLN,ITYLCM)
         IF(ILCMLN.EQ.NREGIO) THEN
            CALL LCMGET(IPSYS,'DRAGON-PIS',WORK(1,2))
            CALL LCMGET(IPSYS,'DRAGON-TXSC',SIGMA(0))
            SUM=0.0D0
            SUD=0.0D0
            DO 160 IREGIO=1,NREGIO
            FACTOR=VOLUME(IREGIO)
            MATNUM=MATCOD(IREGIO)
            SUM=SUM+SIGMA(MATNUM)*FACTOR*FLUXES(IREGIO,IGR)
     >             -WORK(IREGIO,1)*FACTOR*(1.0-WORK(IREGIO,2))
  160       SUD=SUD+SIGMA(MATNUM)*FACTOR*WORK(IREGIO,2)
            COURIN(IGRCND)=COURIN(IGRCND)+SUM/SUD
         ENDIF
         CALL LCMSIX(IPSYS,' ',2)
      ENDIF
      CALL LCMSIX(IPMACR,' ',2)
  170 CONTINUE
*
      DO 180 NEWREG=1,NMERGE
  180 VOLMER(NEWREG)=0.0
      DO 190 IREGIO=1,NREGIO
      NEWREG=IMERGE(IREGIO)
      IF(NEWREG.GT.0) THEN
        VOLMER(NEWREG)=VOLMER(NEWREG)+VOLUME(IREGIO)
      ENDIF
  190 CONTINUE
      DO 200 IGRCND=1,NGCOND
      DO 200 NEWREG=1,NMERGE
      SIGT(NEWREG,IGRCND)=SIGT(NEWREG,IGRCND)/FLXMER(NEWREG,IGRCND)
      SIGW(NEWREG,IGRCND)=SIGW(NEWREG,IGRCND)/FLXMER(NEWREG,IGRCND)
      DO 200 JGRCND=1,NGCOND
      SUNMER(NEWREG,JGRCND,IGRCND)=SUNMER(NEWREG,JGRCND,IGRCND)
     >                            /FLXMER(NEWREG,JGRCND)
  200 CONTINUE
      DO 210 IGRCND=1,NGCOND
      DO 210 NEWREG=1,NMERGE
  210 FLXMER(NEWREG,IGRCND)=FLXMER(NEWREG,IGRCND)/VOLMER(NEWREG)
      RETURN
      END
