*DECK SYBHT2
      SUBROUTINE SYBHT2 (IR,IPAS,IPAS2,NG,NSMAX,NS,IBI,FRACT,VOLK,P,
     1 RRRR,SIGT0,QKDEL,PIL)
*
*-----------------------------------------------------------------------
*
* CALCULATION OF THE REDUCED COLLISION PROBABILITIES FOR THE DOUBLE
* HETEROGENEITY OPTION OF SYBIL. PART 2.
*
* GENERAL PARAMETERS:
*   IR     : NUMBER OF MIXTURES IN THE DOMAIN.
*   IPAS   : NUMBER OF VOLUMES IN THE COMPOSITE GEOMETRY.
*   IPAS2  : NUMBER OF VOLUMES IN THE MACRO GEOMETRY.
*   NG     : NUMBER OF DIFFERENT KIND OF MICRO STRUCTURES. A KIND OF
*            MICRO STRUCTURE IS CHARACTERIZED BY THE RADIUS OF ITS
*            MICRO VOLUMES. ALL THE MICRO VOLUMES OF THE SAME KIND
*            SHOULD OWN THE SAME NUCLEAR PROPERTIES IN A GIVEN
*            MACRO VOLUME.
*   NSMAX  : MAXIMUM NUMBER OF VOLUMES (TUBES OR SHELLS) IN EACH KIND
*            OF MICRO STRUCTURE.
*   NS     : NUMBER OF VOLUMES IN EACH KIND OF MICRO STRUCTURE.
*            DIMENSION NS(NG)
*   IBI    : TYPE OF MIXTURE IN EACH VOLUME OF THE MACRO GEOMETRY.
*            IF IBI(IKK) IS GREATER THAN IR, THE VOLUME IKK CONTAIN A
*            MICRO STRUCTURE. DIMENSION IBI(IPAS2)
*   FRACT  : VOLUMIC FRACTIONS OF THE MICRO VOLUMES.
*            DIMENSION FRACT(NG,IR2) WHERE IR2 IS THE NUMBER OF
*            ORDINARY AND COMPOSITE MIXTURES.
*   VOLK   : VOLUMIC FRACTIONS OF THE TUBES OR SHELLS IN THE MICRO
*            VOLUMES. DIMENSION VOLK(NG,NSMAX)
*   P      : REDUCED COLLISION PROBABILITIES FOR THE COMPOSITE GEOMETRY.
*            DIMENSION P(IPAS,IPAS)
*   RRRR   : INFORMATION COMPUTED BY SYBHT1.
*   SIGT0  : EQUIVALENT MACROSCOPIC CROSS SECTION IN EACH MACRO VOLUME.
*            DIMENSION RRRR(IPAS2),SIGT0(IPAS2)
*   QKDEL  : INFORMATION COMPUTED BY SYBHT1.
*            DIMENSION QKDEL(NG,NSMAX,IPAS2)
*   PIL    : REDUCED COLLISION PROBABILITIES BETWEEN THE MACRO VOLUMES.
*            DIMENSION PIL(IPAS2,IPAS2)
*
*---------------------------------------- AUTHOR: A. HEBERT 15/06/87 ---
*
      DIMENSION NS(*),IBI(*),FRACT(NG,*),VOLK(NG,*),P(IPAS,*),
     1 RRRR(*),SIGT0(*),QKDEL(NG,NSMAX,*),PIL(IPAS2,*)
*
      IND1=IPAS2
      DO 200 I=1,IPAS2
      IMIL=IBI(I)
      I1PL=0
      IF (IMIL.GT.IR) THEN
         RSLN=(1.0-RRRR(I))*RRRR(I)/SIGT0(I)
         DILF=1.0
         DO 50 J=1,NG
50       DILF=DILF-FRACT(J,IMIL)
         P(I,I)=P(I,I)+DILF*RSLN
         IND1P=IND1
         DO 90 J=1,NG
         IF (FRACT(J,IMIL).LE.0.00001) GO TO 90
         I1PL=I1PL+NS(J)
         DO 80 K=1,NS(J)
         IND1P=IND1P+1
         RIKS=QKDEL(J,K,I)*RSLN
         P(IND1P,I)=P(IND1P,I)+DILF*RIKS
         P(I,IND1P)=P(I,IND1P)+FRACT(J,IMIL)*VOLK(J,K)*RIKS
         IND2P=IND1
         DO 80 M=1,NG
         FRT=FRACT(M,IMIL)
         IF (FRT.LE.0.00001) GO TO 80
         DO 70 N=1,NS(M)
         IND2P=IND2P+1
70       P(IND1P,IND2P)=P(IND1P,IND2P)+FRT*VOLK(M,N)*QKDEL(M,N,I)*RIKS
80       CONTINUE
90       CONTINUE
      ENDIF
*
      IND2=IPAS2
      DO 190 L=1,IPAS2
      LMIL=IBI(L)
      I2PL=0
      IF (LMIL.GT.IR) THEN
         DILF=1.0
         DO 100 M=1,NG
100      DILF=DILF-FRACT(M,LMIL)
         RSL0=DILF*RRRR(L)
         P(I,L)=P(I,L)+RRRR(I)*PIL(I,L)*RSL0
         IND2P=IND2
         DO 130 M=1,NG
         FRT=FRACT(M,LMIL)
         IF (FRT.LE.0.00001) GO TO 130
         I2PL=I2PL+NS(M)
         DO 120 N=1,NS(M)
         IND2P=IND2P+1
         RSLN=QKDEL(M,N,L)*FRT*VOLK(M,N)*RRRR(L)
120      P(I,IND2P)=P(I,IND2P)+RRRR(I)*PIL(I,L)*RSLN
130      CONTINUE
         IND1P=IND1
         DO 160 J=1,NG
         IF (FRACT(J,IMIL).LE.0.00001) GO TO 160
         DO 150 K=1,NS(J)
         IND1P=IND1P+1
         RIKS=QKDEL(J,K,I)*RRRR(I)
         P(IND1P,L)=P(IND1P,L)+RIKS*PIL(I,L)*RSL0
         IND2P=IND2
         DO 150 M=1,NG
         FRT=FRACT(M,LMIL)
         IF (FRT.LE.0.00001) GO TO 150
         DO 140 N=1,NS(M)
         IND2P=IND2P+1
         RSLN=QKDEL(M,N,L)*FRT*VOLK(M,N)*RRRR(L)
140      P(IND1P,IND2P)=P(IND1P,IND2P)+RIKS*PIL(I,L)*RSLN
150      CONTINUE
160      CONTINUE
      ELSE
         P(I,L)=P(I,L)+RRRR(I)*PIL(I,L)
         IND1P=IND1
         DO 180 J=1,NG
         IF (FRACT(J,IMIL).LE.0.00001) GO TO 180
         DO 170 K=1,NS(J)
         IND1P=IND1P+1
         RIKS=QKDEL(J,K,I)*RRRR(I)
170      P(IND1P,L)=P(IND1P,L)+RIKS*PIL(I,L)
180      CONTINUE
      ENDIF
      IND2=IND2+I2PL
190   CONTINUE
      IND1=IND1+I1PL
200   CONTINUE
      RETURN
      END
