*DECK SYBRN1
      SUBROUTINE SYBRN1 (NZ,A,B,Z,IZ,RAYRE,SIGT,TRONC,PVS,PSS)
*
*-----------------------------------------------------------------------
*
* INTEGRATION OF THE ESCAPE AND TRANSMISSION PROBABILITIES OF A 2-D
* SQUARE OR RECTANGULAR CROWN WITH DP-0 SURFACE ANGULAR FLUX EXPANSION
* ON THE EXTERNAL BOUNDARY. THE TRACKS ARE COMPUTED BY SYBRTK.
*
* INPUT PARAMETERS:
*   NZ      : NUMBER OF REGIONS IN THE CELL.
*   A,B     : DIMENSIONS OF THE EXTERNAL X AND Y SIDES.
*   Z,IZ    : TRACKING INFORMATION.
*   RAYRE   : RADIUS OF THE TUBES. DIMENSION RAYRE(NZ)
*   SIGT    : TOTAL MACROSCOPIC CROSS SECTION.
*   TRONC   : VOIDED BLOCK CRITERION.
*
* OUTPUT PARAMETERS:
*   PVS     : VOLUME TO SURFACE PROBABILITIES. DIMENSION PVS(4,NZ)
*             XINF: SURFACE 1   XSUP: SURFACE 2
*             YINF: SURFACE 3   YSUP: SURFACE 4
*   PSS     : SURFACE TO SURFACE PROBABILITIES IN THE FOLLOWING ORDER:
*             PSS(I,J) IS THE PROBABILITY FROM SURFACE I TO SURFACE J.
*             DIMENSION PSS(4,4)
*
*------------------------------------------ AUTHOR: A. HEBERT ; 1981 ---
*
      PARAMETER (MKI3=600)
      PARAMETER (PI=3.141592654,ZI30=0.785398164)
      LOGICAL ICARE
      REAL KI3
      DIMENSION Z(*),IZ(*),RAYRE(*),SIGT(*),PVS(4,*),PSS(4,*),PBB(5),
     1 ISL(4,4)
      COMMON /BICKL3/BI3(0:MKI3,3),PAS3,XLIM3,L3
      SAVE ISL
      DATA ISL/5,4,1,1,
     1         4,5,1,1,
     2         3,3,5,2,
     3         3,3,2,5/
*
      ICARE=IZ(1).EQ.2
      ZERO=TRONC*(A+B)/(2.0*A*B)
      AOB=A/B
      DO 5 I=1,5
    5 PBB(I)=0.0
      DO 10 I=1,4
      DO 10 J=1,NZ
   10 PVS(I,J)=0.0
      L=6
      NA=IZ(6)
      DO 170 IA=1,NA
      L=L+1
      NLX=IZ(L)
      IF (NLX.EQ.0) GO TO 102
      L=L+2
      DO 101 IX=1,NLX
      Z1=Z(L+1)
      KMAX=IZ(L+2)
      L=L+2
      MMAX=KMAX+KMAX-1
      KI3=ZI30
      POP=0.0
      DO 70 I=1,MMAX
      III=IABS(KMAX-I)+1+NZ-KMAX
      SIGTI=SIGT(III)
      POP=POP+SIGTI*Z(L+I)
      IF (POP-XLIM3) 50,45,45
   45 IF (SIGTI.LE.ZERO) GO TO 80
      PVS(3,III)=PVS(3,III)+2.0*KI3*Z1
      GO TO 80
   50 K=NINT(POP*PAS3)
      WI3=BI3(K,1)+POP*(BI3(K,2)+POP*BI3(K,3))
      IF (SIGTI.LE.ZERO) THEN
         WI2=TABKI(2,POP-SIGTI*Z(L+I))
         PVS(3,III)=PVS(3,III)+2.0*WI2*Z(L+I)*Z1
      ELSE
         PVS(3,III)=PVS(3,III)+2.0*(KI3-WI3)*Z1
      ENDIF
   70 KI3=WI3
   80 KI3=ZI30
      POP=0.0
      DO 95 I=1,MMAX
      III=IABS(KMAX-I)+1+NZ-KMAX
      SIGTI=SIGT(III)
      POP=POP+SIGTI*Z(L+MMAX+1-I)
      IF (POP-XLIM3) 90,85,85
   85 IF (SIGTI.LE.ZERO) GO TO 100
      PVS(1,III)=PVS(1,III)+2.0*KI3*Z1*AOB
      GO TO 100
   90 K=NINT(POP*PAS3)
      WI3=BI3(K,1)+POP*(BI3(K,2)+POP*BI3(K,3))
      IF (SIGTI.LE.ZERO) THEN
         WI2=TABKI(2,POP-SIGTI*Z(L+MMAX+1-I))
         PVS(1,III)=PVS(1,III)+2.0*WI2*Z(L+MMAX+1-I)*Z1*AOB
      ELSE
         PVS(1,III)=PVS(1,III)+2.0*(KI3-WI3)*Z1*AOB
      ENDIF
   95 KI3=WI3
      PBB(1)=PBB(1)+KI3*Z1
  100 L=L+MMAX
  101 CONTINUE
*
  102 L=L+1
      NLX=IZ(L)
      IF (NLX.EQ.0) GO TO 104
      L=L+1
      DO 103 IX=1,NLX
      Z1=Z(L+1)
      KMAX=IZ(L+2)
      L=L+2
      MMAX=KMAX+KMAX-1
      KI3=ZI30
      POP=0.0
      DO 30 I=1,MMAX
      III=IABS(KMAX-I)+1+NZ-KMAX
      SIGTI=SIGT(III)
      POP=POP+SIGTI*Z(L+I)
      IF (POP-XLIM3) 20,15,15
   15 IF (SIGTI.LE.ZERO) GO TO 40
      PVS(3,III)=PVS(3,III)+KI3*Z1
      GO TO 40
   20 K=NINT(POP*PAS3)
      WI3=BI3(K,1)+POP*(BI3(K,2)+POP*BI3(K,3))
      IF (SIGTI.LE.ZERO) THEN
         WI2=TABKI(2,POP-SIGTI*Z(L+I))
         PVS(3,III)=PVS(3,III)+WI2*Z(L+I)*Z1
      ELSE
         PVS(3,III)=PVS(3,III)+(KI3-WI3)*Z1
      ENDIF
   30 KI3=WI3
   40 KI3=ZI30
      POP=0.0
      DO 350 I=1,MMAX
      III=IABS(KMAX-I)+1+NZ-KMAX
      SIGTI=SIGT(III)
      POP=POP+SIGTI*Z(L+MMAX+1-I)
      IF (POP-XLIM3) 320,310,310
  310 IF (SIGTI.LE.ZERO) GO TO 370
      PVS(3,III)=PVS(3,III)+KI3*Z1
      GO TO 370
  320 K=NINT(POP*PAS3)
      WI3=BI3(K,1)+POP*(BI3(K,2)+POP*BI3(K,3))
      IF (SIGTI.LE.ZERO) THEN
         WI2=TABKI(2,POP-SIGTI*Z(L+MMAX+1-I))
         PVS(3,III)=PVS(3,III)+WI2*Z(L+MMAX+1-I)*Z1
      ELSE
         PVS(3,III)=PVS(3,III)+(KI3-WI3)*Z1
      ENDIF
  350 KI3=WI3
      PBB(2)=PBB(2)+2.0*KI3*Z1
  370 L=L+MMAX
  103 CONTINUE
*
  104 IF (ICARE) GO TO 170
      L=L+1
      NLX=IZ(L)
      IF (NLX.EQ.0) GO TO 170
      L=L+1
      DO 165 IX=1,NLX
      Z1=Z(L+1)
      KMAX=IZ(L+2)
      L=L+2
      MMAX=KMAX+KMAX-1
      KI3=ZI30
      POP=0.0
      DO 120 I=1,MMAX
      III=IABS(KMAX-I)+1+NZ-KMAX
      SIGTI=SIGT(III)
      POP=POP+SIGTI*Z(L+I)
      IF (POP-XLIM3) 110,105,105
  105 IF (SIGTI.LE.ZERO) GO TO 160
      PVS(1,III)=PVS(1,III)+KI3*Z1
      GO TO 160
  110 K=NINT(POP*PAS3)
      WI3=BI3(K,1)+POP*(BI3(K,2)+POP*BI3(K,3))
      IF (SIGTI.LE.ZERO) THEN
         WI2=TABKI(2,POP-SIGTI*Z(L+I))
         PVS(1,III)=PVS(1,III)+WI2*Z(L+I)*Z1
      ELSE
         PVS(1,III)=PVS(1,III)+(KI3-WI3)*Z1
      ENDIF
  120 KI3=WI3
  160 KI3=ZI30
      POP=0.0
      DO 450 I=1,MMAX
      III=IABS(KMAX-I)+1+NZ-KMAX
      SIGTI=SIGT(III)
      POP=POP+SIGTI*Z(L+MMAX+1-I)
      IF (POP-XLIM3) 420,410,410
  410 IF (SIGTI.LE.ZERO) GO TO 470
      PVS(1,III)=PVS(1,III)+KI3*Z1
      GO TO 470
  420 K=NINT(POP*PAS3)
      WI3=BI3(K,1)+POP*(BI3(K,2)+POP*BI3(K,3))
      IF (SIGTI.LE.ZERO) THEN
         WI2=TABKI(2,POP-SIGTI*Z(L+MMAX+1-I))
         PVS(1,III)=PVS(1,III)+WI2*Z(L+MMAX+1-I)*Z1
      ELSE
         PVS(1,III)=PVS(1,III)+(KI3-WI3)*Z1
      ENDIF
  450 KI3=WI3
      PBB(4)=PBB(4)+2.0*KI3*Z1
  470 L=L+MMAX
  165 CONTINUE
  170 CONTINUE
      IF (ICARE) THEN
         PBB(1)=PBB(1)+PBB(1)
         PBB(2)=PBB(2)+PBB(4)
         PBB(4)=PBB(2)
         DO 180 I=1,NZ
         PVS(3,I)=PVS(3,I)+PVS(1,I)
  180    PVS(1,I)=PVS(3,I)
      ENDIF
      PBB(3)=PBB(1)*AOB
*
      Z1=Z(2)
      DO 191 I=1,4
  191 PBB(I)=Z1*Z1*PBB(I)
      DO 192 J=1,NZ
      PVS(3,J)=Z1*Z1*PVS(3,J)
  192 PVS(1,J)=Z1*Z1*PVS(1,J)
      VOL=A*B
      Z1=0.0
      Z2=0.0
      DO 210 J=1,NZ
      JJJ=NZ+1-J
      R2=PI*RAYRE(JJJ)**2
      X=4.0*(VOL-R2)
      SIGTI=SIGT(JJJ)
      IF (SIGTI.LE.ZERO) GO TO 195
      Z1=Z1+PVS(3,JJJ)
      Z2=Z2+PVS(1,JJJ)
      X=X*SIGTI
  195 PVS(3,JJJ)=PVS(3,JJJ)*A/X
      PVS(1,JJJ)=PVS(1,JJJ)*B/X
  210 VOL=R2
      IF ((Z1.GT.TRONC).AND.(Z2.GT.TRONC)) THEN
         Z1=(1.0-2.0*PBB(1)-PBB(2))/Z1
         Z2=(1.0-2.0*PBB(3)-PBB(4))/Z2
         DO 220 J=1,NZ
         PVS(3,J)=PVS(3,J)*Z1
  220    PVS(1,J)=PVS(1,J)*Z2
      ENDIF
      DO 240 I=1,NZ
      PVS(2,I)=PVS(1,I)
  240 PVS(4,I)=PVS(3,I)
      DO 270 JC=1,4
      DO 270 IC=1,4
  270 PSS(IC,JC)=PBB(ISL(IC,JC))
      RETURN
      END
