*DECK SYBRX2
      SUBROUTINE SYBRX2 (IPAS,SIGT,P,IMPX,NCOUR,IWIGN,NMCEL,NMERGE,NGEN,
     1 IQUAD,XX,YY,NMC,RAYRE,IFR,ALB,INUM,MIX,DVX,IGEN,PIS,PSS,PSSB,
     2 SIGT2,SCALE1,SCALE2,ROUT)
*
*-----------------------------------------------------------------------
*
* COMPUTE THE REDUCED COLLISION PROBABILITIES IN A 2-D CARTESIAN OR
* HEXAGONAL ASSEMBLY USING THE INTERFACE CURRENT METHOD WITH ROTH
* APPROXIMATION
*
* GENERAL PARAMETERS:
*   IPAS   : TOTAL NUMBER OF VOLUMES.
*   SIGT   : TOTAL MACROSCOPIC CROSS SECTIONS. DIMENSION SIGT(IPAS)
*   P      : REDUCED COLLISION PROBABILITIES. DIMENSION P(IPAS,IPAS)
*   IMPX   : PRINT FLAG. EQUAL TO 0 FOR NO PRINT.
*   NCOUR  : =4 : CARTESIAN LATTICE ; =6 : HEXAGONAL LATTICE.
*   IWIGN  : TYPE OF CYLINDERIZATION.
*   IQUAD  : QUADRATURE PARAMETERS. DIMENSION IQUAD(4)
*
*   PARAMETERS RELATED TO CELLS DEFINITION:
*   NMCEL  : TOTAL NUMBER OF CELLS IN THE DOMAIN.
*   IFR    : INDEX-NUMBER OF IN-CURRENTS. DIMENSION IFR(IJAS) WHERE
*            IJAS IS THE TOTAL NUMBER OF SURFACES SURROUNDING ALL THE
*            CELLS.
*   ALB    : TRANSMISSION/ALBEDO ASSOCIATED WITH EACH IN-CURRENT.
*            DIMENSION ALB(IJAS)
*
*   PARAMETERS RELATED TO THE MERGED CELLS DEFINITION:
*   NMERGE : TOTAL NUMBER OF MERGED CELLS FOR WHICH SPECIFIC VALUES
*            OF THE NEUTRON FLUX AND REACTIONS RATES ARE REQUIRED.
*            MANY CELLS WITH DIFFERENT POSITION IN THE DOMAIN CAN
*            BE MERGED BEFORE THE NEUTRON FLUX CALCULATION IF THEY
*            OWN THE SAME GENERATING CELL. THIS ALLOWS SOME REDUCTION
*            IN CPU TIME AND MEMORY. NMERGE.LE.NMCEL
*   INUM   : INDEX-NUMBER OF THE MERGED CELL ASSOCIATED TO EACH CELL.
*            DIMENSION INUM(NMCEL)
*   MIX    : INDEX-NUMBER OF OUT-CURRENTS. DIMENSION MIX(IJAZ) WHERE
*            IJAZ IS THE TOTAL NUMBER OF SURFACES SURROUNDING ALL THE
*            MERGED CELLS.
*   DVX    : WEIGHT ASSOCIATED WITH EACH OUT-CURRENT.
*            DIMENSION DVX(IJAZ)
*
*            REMARK:
*            IFR, ALB, MIX AND DVX CONTAINS INFORMATION TO REBUILD THE
*            GEOMETRICAL 'A' MATRIX.
*
*   PARAMETERS RELATED TO THE GENERATING CELLS DEFINITION:
*   NGEN   : TOTAL NUMBER OF GENERATING CELLS. A GENERATING CELL IS
*            DEFINED BY ITS MATERIAL AND DIMENSIONS IRRESPECTIVE OF
*            ITS POSITION IN THE DOMAIN. NGEN.LE.NMERGE
*   XX,YY  : X AND Y-THICKNESS OF THE GENERATING CELLS.
*            DIMENSION XX(NGEN),YY(NGEN)
*   NMC    : OFFSET OF THE FIRST VOLUME IN EACH GENERATING CELL.
*            DIMENSION NMC(NGEN+1)
*   RAYRE  : RADIUS OF THE TUBES IN EACH GENERATING CELL.
*            DIMENSION RAYRE(NMC(NGEN+1))
*   IGEN   : INDEX-NUMBER OF THE GENERATING CELL ASSOCIATED WITH EACH
*            MERGED CELL. DIMENSION IGEN(NMERGE)
*
* SCRATCH STORAGE:
*   PIS    : DIMENSION PIS(IPAS)
*   PSS    : DIMENSION PSS(NGEN)
*   PSSB   : DIMENSION PSSB(2*NMERGE**2)
*   SIGT2  : DIMENSION SIGT2(IPAS)
*   SCALE1 : DIMENSION SCALE1(NGEN)
*   SCALE2 : DIMENSION SCALE2(NGEN)
*   ROUT   : DIMENSION ROUT(NGEN)
*
*---------------------------------------- AUTHOR: A. HEBERT 22/05/87 ---
*
      PARAMETER (PI=3.141592654)
      DIMENSION SIGT(*),P(IPAS,*),IQUAD(*),XX(*),YY(*),NMC(*),RAYRE(*),
     1 IFR(*),ALB(*),INUM(*),MIX(*),DVX(*),IGEN(*),PIS(*),PSS(*),
     2 PSSB(NMERGE,*),SIGT2(*),SCALE1(*),SCALE2(*),ROUT(*),PIBB(6)
      COMMON BASE(1)
      MR=IQUAD(4)
*
      DO 10 I=1,IPAS
      DO 10 J=1,IPAS
10    P(I,J)=0.0
*
      DO 20 IKG=1,NGEN
      A=XX(IKG)
      B=YY(IKG)
      RJ1=RAYRE(NMC(IKG+1))
      IF ((NCOUR.EQ.4).AND.(IWIGN.EQ.1)) THEN
*        ASKEW CYLINDERIZATION CARTESIAN.
         RJ2=(A+B)/PI
         SCALE1(IKG)=(A*B-PI*RJ1**2)/(PI*RJ2**2-PI*RJ1**2)
         SCALE2(IKG)=1.0
         ROUT(IKG)=RJ2
      ELSE IF ((NCOUR.EQ.4).AND.(IWIGN.EQ.2)) THEN
*        WIGNER CYLINDERIZATION CARTESIAN.
         SCALE1(IKG)=1.0
         SCALE2(IKG)=1.0
         ROUT(IKG)=SQRT(A*B/PI)
      ELSE IF ((NCOUR.EQ.4).AND.(IWIGN.EQ.3)) THEN
*        SANCHEZ CYLINDERIZATION CARTESIAN.
         SCALE1(IKG)=1.0
         SCALE2(IKG)=SQRT(PI*A*B)/(A+B)
         ROUT(IKG)=SQRT(A*B/PI)
      ELSE IF (IWIGN.EQ.1) THEN
*        ASKEW CYLINDERIZATION HEXAGONAL.
         RJ2=3.0*A/PI
         SCALE1(IKG)=(1.5*SQRT(3.0)*A*A-PI*RJ1**2)/(PI*RJ2**2-PI*RJ1**2)
         SCALE2(IKG)=1.0
         ROUT(IKG)=RJ2
      ELSE IF (IWIGN.EQ.2) THEN
*        WIGNER CYLINDERIZATION HEXAGONAL.
         SCALE1(IKG)=1.0
         SCALE2(IKG)=1.0
         ROUT(IKG)=SQRT(1.5*SQRT(3.0)/PI)*A
      ELSE IF (IWIGN.EQ.3) THEN
*        SANCHEZ CYLINDERIZATION HEXAGONAL.
         SCALE1(IKG)=1.0
         SCALE2(IKG)=SQRT(PI*SQRT(3.0)/6.0)
         ROUT(IKG)=SQRT(1.5*SQRT(3.0)/PI)*A
      ENDIF
      IF(ROUT(IKG).LE.RJ1) CALL XABORT('SYBRX2: CYLINDERIZATION ERROR.')
20    CONTINUE
*
      I1=0
      DO 40 IKK=1,NMERGE
      IKG=IGEN(IKK)
      J1=NMC(IKG)
      I2=NMC(IKG+1)-J1
      DO 30 I=1,I2-1
30    SIGT2(J1+I)=SIGT(I1+I)
      SIGT2(J1+I2)=SIGT(I1+I2)*SCALE1(IKG)
40    I1=I1+I2
*
      DO 95 JKG=1,NGEN
      J1=NMC(JKG)
      J2=NMC(JKG+1)-J1
      SURFA=0.5*PI*ROUT(JKG)
      CALL SETARA(BASE,J2*J2,IP)
      CALL SETARA(BASE,J2+1,IRAYR)
      DO 45 I=1,J2
45    BASE(IRAYR+I-1)=RAYRE(J1+I)
      BASE(IRAYR+J2)=ROUT(JKG)
      CALL SETARA(BASE,4*J2,IX)
      CALL SYBALC(J2,J2,BASE(IRAYR),SIGT2(J1+1),BASE(IP),MR,0.0,
     1 BASE(IX))
      CALL RLSARA(BASE(IX))
      PSS(JKG)=0.0
      RJ=0.0
      DO 60 I=1,J2
      PIS(J1+I)=1.0
      RJ1=BASE(IRAYR+I)**2
      VV=PI*(RJ1-RJ)
      DO 50 J=1,J2
50    PIS(J1+I)=PIS(J1+I)-BASE(IP+(J-1)*J2+I-1)*SIGT2(J1+J)
      PSS(JKG)=PSS(JKG)+PIS(J1+I)*SIGT2(J1+I)*VV/SURFA
60    RJ=RJ1
      CALL RLSARA(BASE(IRAYR))
      PSS(JKG)=1.0-SCALE2(JKG)*PSS(JKG)
      IF (IMPX.GE.8) CALL SYBPRX (1,J2,JKG,SIGT2(J1+1),BASE(IP),
     1 PIS(J1+1),PSS(JKG))
      I1=0
      DO 90 IKK=1,NMERGE
      IKG=IGEN(IKK)
      I2=NMC(IKG+1)-NMC(IKG)
      IF (IKG.EQ.JKG) THEN
         DO 70 I=1,J2-1
         DO 70 J=1,J2
70       P(I1+J,I1+I)=BASE(IP+(I-1)*J2+J-1)
         DO 80 J=1,J2
80       P(I1+J,I1+J2)=BASE(IP+(J2-1)*J2+J-1)*SCALE1(JKG)
      ENDIF
90    I1=I1+I2
      CALL RLSARA(BASE(IP))
95    CONTINUE
*
* COMPUTATION OF PSSB=A*(I-PSS*A)**-1
      DO 110 I=1,NMERGE
      DO 100 J=1,NMERGE
      PSSB(I,J)=0.0
100   PSSB(I,NMERGE+J)=0.0
110   PSSB(I,I)=1.0
      DO 120 ICEL=1,NMCEL
      IKK=INUM(ICEL)
      IKG=IGEN(IKK)
      IS=NCOUR*(ICEL-1)
      IF (NCOUR.EQ.4) THEN
         A=XX(IKG)
         B=YY(IKG)
         DEN1=2.0*(A+B)
         PIBB(1)=B/DEN1
         PIBB(2)=B/DEN1
         PIBB(3)=A/DEN1
         PIBB(4)=A/DEN1
      ELSE
         DO 115 JC=1,NCOUR
115      PIBB(JC)=1.0/6.0
      ENDIF
      ZZZ=PSS(IKG)
      DO 120 JC=1,NCOUR
      J1=IFR(IS+JC)
      ALBEDO=PIBB(JC)*ALB(IS+JC)
      PSSB(J1,NMERGE+IKK)=PSSB(J1,NMERGE+IKK)+ALBEDO
120   PSSB(J1,IKK)=PSSB(J1,IKK)-ZZZ*ALBEDO
      CALL ALSB (NMERGE,NMERGE,PSSB,IER,NMERGE)
      IF (IER.NE.0) CALL XABORT('SYBRX2: SINGULAR MATRIX.')
*
* COMPUTATION OF PIS*PSSB*PSJ
      I1=0
      DO 150 IKK=1,NMERGE
      IKG=IGEN(IKK)
      I1P=NMC(IKG)
      I2=NMC(IKG+1)-I1P
      DO 140 I=1,I2
      ZZZ=PIS(I1P+I)
      J1=0
      DO 140 JKK=1,NMERGE
      JKG=IGEN(JKK)
      J1P=NMC(JKG)
      J2=NMC(JKG+1)-J1P
      SURFA=(0.5*PI*ROUT(JKG))/SCALE2(JKG)
      RJ1=0.0
      DO 130 J=1,J2-1
      RJ2=PI*RAYRE(J1P+J+1)**2
      PBJ=PIS(J1P+J)*(RJ2-RJ1)/SURFA
      P(I1+I,J1+J)=P(I1+I,J1+J)+ZZZ*PSSB(JKK,NMERGE+IKK)*PBJ
130   RJ1=RJ2
      RJ2=PI*ROUT(JKG)**2
      PBJ=PIS(J1P+J2)*(RJ2-RJ1)*SCALE1(JKG)/SURFA
      P(I1+I,J1+J2)=P(I1+I,J1+J2)+ZZZ*PSSB(JKK,NMERGE+IKK)*PBJ
140   J1=J1+J2
150   I1=I1+I2
      RETURN
      END
