*DECK SYBRX3
      SUBROUTINE SYBRX3 (IPAS,SIGT,P,IMPX,NCOUR,IWIGN,NMCEL,NMERGE,NGEN,
     1 IJAT,IQUAD,XX,YY,NMC,RAYRE,IFR,ALB,INUM,MIX,DVX,IGEN,PIS,PSS,
     2 PSSB,SIGT2,SCALE1,SCALE2,ROUT)
*
*-----------------------------------------------------------------------
*
* COMPUTE THE REDUCED COLLISION PROBABILITIES IN A 2-D CARTESIAN OR
* HEXAGONAL ASSEMBLY USING THE INTERFACE CURRENT METHOD WITH ROTH X 4
* OR ROTH X 6 APPROXIMATION.
*
* GENERAL PARAMETERS:
*   IPAS   : TOTAL NUMBER OF VOLUMES.
*   SIGT   : TOTAL MACROSCOPIC CROSS SECTIONS. DIMENSION SIGT(IPAS)
*   P      : REDUCED COLLISION PROBABILITIES. DIMENSION P(IPAS,IPAS)
*   IMPX   : PRINT FLAG. EQUAL TO 0 FOR NO PRINT.
*   NCOUR  : =4 : CARTESIAN LATTICE ; =6 : HEXAGONAL LATTICE.
*   IWIGN  : TYPE OF CYLINDERIZATION.
*   IQUAD  : QUADRATURE PARAMETERS. DIMENSION IQUAD(4)
*
*   PARAMETERS RELATED TO CELLS DEFINITION:
*   NMCEL  : TOTAL NUMBER OF CELLS IN THE DOMAIN.
*   IFR    : INDEX-NUMBER OF IN-CURRENTS. DIMENSION IFR(IJAS) WHERE
*            IJAS IS THE TOTAL NUMBER OF SURFACES SURROUNDING ALL THE
*            CELLS.
*   ALB    : TRANSMISSION/ALBEDO ASSOCIATED WITH EACH IN-CURRENT.
*            DIMENSION ALB(IJAS)
*
*   PARAMETERS RELATED TO THE MERGED CELLS DEFINITION:
*   NMERGE : TOTAL NUMBER OF MERGED CELLS FOR WHICH SPECIFIC VALUES
*            OF THE NEUTRON FLUX AND REACTIONS RATES ARE REQUIRED.
*            MANY CELLS WITH DIFFERENT POSITION IN THE DOMAIN CAN
*            BE MERGED BEFORE THE NEUTRON FLUX CALCULATION IF THEY
*            OWN THE SAME GENERATING CELL. THIS ALLOWS SOME REDUCTION
*            IN CPU TIME AND MEMORY. NMERGE.LE.NMCEL
*   IJAT   : TOTAL NUMBER OF DISTINCT OUT-CURRENTS.
*   INUM   : INDEX-NUMBER OF THE MERGED CELL ASSOCIATED TO EACH CELL.
*            DIMENSION INUM(NMCEL)
*   MIX    : INDEX-NUMBER OF OUT-CURRENTS. DIMENSION MIX(IJAZ) WHERE
*            IJAZ IS THE TOTAL NUMBER OF SURFACES SURROUNDING ALL THE
*            MERGED CELLS.
*   DVX    : WEIGHT ASSOCIATED WITH EACH OUT-CURRENT.
*            DIMENSION DVX(IJAZ)
*
*            REMARK:
*            IFR, ALB, MIX AND DVX CONTAINS INFORMATION TO REBUILD THE
*            GEOMETRICAL 'A' MATRIX.
*
*   PARAMETERS RELATED TO THE GENERATING CELLS DEFINITION:
*   NGEN   : TOTAL NUMBER OF GENERATING CELLS. A GENERATING CELL IS
*            DEFINED BY ITS MATERIAL AND DIMENSIONS IRRESPECTIVE OF
*            ITS POSITION IN THE DOMAIN. NGEN.LE.NMERGE
*   XX,YY  : X AND Y-THICKNESS OF THE GENERATING CELLS.
*            DIMENSION XX(NGEN),YY(NGEN)
*   NMC    : OFFSET OF THE FIRST VOLUME IN EACH GENERATING CELL.
*            DIMENSION NMC(NGEN+1)
*   RAYRE  : RADIUS OF THE TUBES IN EACH GENERATING CELL.
*            DIMENSION RAYRE(NMC(NGEN+1))
*   IGEN   : INDEX-NUMBER OF THE GENERATING CELL ASSOCIATED WITH EACH
*            MERGED CELL. DIMENSION IGEN(NMERGE)
*
* SCRATCH STORAGE:
*   PIS    : DIMENSION PIS(IPAS)
*   PSS    : DIMENSION PSS(NGEN)
*   PSSB   : DIMENSION PSSB(2*IJAT**2)
*   SIGT2  : DIMENSION SIGT2(IPAS)
*   SCALE1 : DIMENSION SCALE1(NGEN)
*   SCALE2 : DIMENSION SCALE2(NGEN)
*   ROUT   : DIMENSION ROUT(NGEN)
*
*---------------------------------------- AUTHOR: A. HEBERT 22/05/87 ---
*
      PARAMETER (PI=3.141592654)
      DIMENSION SIGT(*),P(IPAS,*),IQUAD(*),XX(*),YY(*),NMC(*),RAYRE(*),
     1 IFR(*),ALB(*),INUM(*),MIX(*),DVX(*),IGEN(*),PIS(*),PSS(*),
     2 PSSB(IJAT,*),SIGT2(*),SCALE1(*),SCALE2(*),ROUT(*),ISLR(4,4),
     3 ISLH(6,6),PBB(6)
      COMMON BASE(1)
      SAVE ISLR,ISLH
      DATA ISLR/0,4,1,1,4,0,1,1,3,3,0,2,3,3,2,0/
      DATA ISLH/0,1,2,3,2,1,1,0,1,2,3,2,2,1,0,1,2,3,
     1          3,2,1,0,1,2,2,3,2,1,0,1,1,2,3,2,1,0/
      MR=IQUAD(4)
*
      DO 10 I=1,IPAS
      DO 10 J=1,IPAS
10    P(I,J)=0.0
*
      DO 20 IKG=1,NGEN
      A=XX(IKG)
      B=YY(IKG)
      RJ1=RAYRE(NMC(IKG+1))
      IF ((NCOUR.EQ.4).AND.(IWIGN.EQ.1)) THEN
*        ASKEW CYLINDERIZATION CARTESIAN.
         RJ2=(A+B)/PI
         SCALE1(IKG)=(A*B-PI*RJ1**2)/(PI*RJ2**2-PI*RJ1**2)
         SCALE2(IKG)=1.0
         ROUT(IKG)=RJ2
      ELSE IF ((NCOUR.EQ.4).AND.(IWIGN.EQ.2)) THEN
*        WIGNER CYLINDERIZATION CARTESIAN.
         SCALE1(IKG)=1.0
         SCALE2(IKG)=1.0
         ROUT(IKG)=SQRT(A*B/PI)
      ELSE IF ((NCOUR.EQ.4).AND.(IWIGN.EQ.3)) THEN
*        SANCHEZ CYLINDERIZATION CARTESIAN.
         SCALE1(IKG)=1.0
         SCALE2(IKG)=SQRT(PI*A*B)/(A+B)
         ROUT(IKG)=SQRT(A*B/PI)
      ELSE IF (IWIGN.EQ.1) THEN
*        ASKEW CYLINDERIZATION HEXAGONAL.
         RJ2=3.0*A/PI
         SCALE1(IKG)=(1.5*SQRT(3.0)*A*A-PI*RJ1**2)/(PI*RJ2**2-PI*RJ1**2)
         SCALE2(IKG)=1.0
         ROUT(IKG)=RJ2
      ELSE IF (IWIGN.EQ.2) THEN
*        WIGNER CYLINDERIZATION HEXAGONAL.
         SCALE1(IKG)=1.0
         SCALE2(IKG)=1.0
         ROUT(IKG)=SQRT(1.5*SQRT(3.0)/PI)*A
      ELSE IF (IWIGN.EQ.3) THEN
*        SANCHEZ CYLINDERIZATION HEXAGONAL.
         SCALE1(IKG)=1.0
         SCALE2(IKG)=SQRT(PI*SQRT(3.0)/6.0)
         ROUT(IKG)=SQRT(1.5*SQRT(3.0)/PI)*A
      ENDIF
      IF(ROUT(IKG).LE.RJ1) CALL XABORT('SYBRX3: CYLINDERIZATION ERROR.')
20    CONTINUE
*
      I1=0
      DO 40 IKK=1,NMERGE
      IKG=IGEN(IKK)
      J1=NMC(IKG)
      I2=NMC(IKG+1)-J1
      DO 30 I=1,I2-1
30    SIGT2(J1+I)=SIGT(I1+I)
      SIGT2(J1+I2)=SIGT(I1+I2)*SCALE1(IKG)
40    I1=I1+I2
*
      DO 95 JKG=1,NGEN
      J1=NMC(JKG)
      J2=NMC(JKG+1)-J1
      SURFA=0.5*PI*ROUT(JKG)
      CALL SETARA(BASE,J2*J2,IP)
      CALL SETARA(BASE,J2+1,IRAYR)
      DO 45 I=1,J2
45    BASE(IRAYR+I-1)=RAYRE(J1+I)
      BASE(IRAYR+J2)=ROUT(JKG)
      CALL SETARA(BASE,4*J2,IX)
      CALL SYBALC(J2,J2,BASE(IRAYR),SIGT2(J1+1),BASE(IP),MR,0.0,
     1 BASE(IX))
      CALL RLSARA(BASE(IX))
      PSS(JKG)=0.0
      RJ=0.0
      DO 60 I=1,J2
      PIS(J1+I)=1.0
      RJ1=BASE(IRAYR+I)**2
      VV=PI*(RJ1-RJ)
      DO 50 J=1,J2
50    PIS(J1+I)=PIS(J1+I)-BASE(IP+(J-1)*J2+I-1)*SIGT2(J1+J)
      PSS(JKG)=PSS(JKG)+PIS(J1+I)*SIGT2(J1+I)*VV/SURFA
60    RJ=RJ1
      CALL RLSARA(BASE(IRAYR))
      PSS(JKG)=1.0-SCALE2(JKG)*PSS(JKG)
      IF (IMPX.GE.8) CALL SYBPRX (1,J2,JKG,SIGT2(J1+1),BASE(IP),
     1 PIS(J1+1),PSS(JKG))
      I1=0
      DO 90 IKK=1,NMERGE
      IKG=IGEN(IKK)
      I2=NMC(IKG+1)-NMC(IKG)
      IF (IKG.EQ.JKG) THEN
         DO 70 I=1,J2-1
         DO 70 J=1,J2
70       P(I1+J,I1+I)=BASE(IP+(I-1)*J2+J-1)
         DO 80 J=1,J2
80       P(I1+J,I1+J2)=BASE(IP+(J2-1)*J2+J-1)*SCALE1(JKG)
      ENDIF
90    I1=I1+I2
      CALL RLSARA(BASE(IP))
95    CONTINUE
*
* COMPUTATION OF PSSB=A*(I-PSS*A)**-1
      DO 110 I=1,IJAT
      DO 100 J=1,IJAT
      PSSB(I,J)=0.0
100   PSSB(I,IJAT+J)=0.0
110   PSSB(I,I)=1.0
      DO 140 ICEL=1,NMCEL
      IKK=INUM(ICEL)
      IKG=IGEN(IKK)
      IT=NCOUR*(IKK-1)
      IS=NCOUR*(ICEL-1)
      ZZZ=PSS(IKG)
      IF (NCOUR.EQ.4) THEN
         A=XX(IKG)
         B=YY(IKG)
         DEN1=MAX(2.0*B+A,2.0*A+B)
         PBB(1)=B/DEN1
         PBB(2)=MAX(A,2.0*A-B)/DEN1
         PBB(3)=A/DEN1
         PBB(4)=MAX(B,2.0*B-A)/DEN1
      ELSE
         PBB(1)=1.0/5.0
         PBB(2)=1.0/5.0
         PBB(3)=1.0/5.0
      ENDIF
      DO 140 JC=1,NCOUR
      J1=IFR(IS+JC)
      J2=MIX(IT+JC)
      ALBEDO=ALB(IS+JC)
      PSSB(J1,IJAT+J2)=PSSB(J1,IJAT+J2)+ALBEDO*DVX(IT+JC)
      DO 140 IC=1,NCOUR
      J2=MIX(IT+IC)
      IF (NCOUR.EQ.4) THEN
         IB=ISLR(IC,JC)
      ELSE
         IB=ISLH(IC,JC)
      ENDIF
      IF (IB.EQ.0) GO TO 140
      PSSB(J1,J2)=PSSB(J1,J2)-ZZZ*PBB(IB)*ALBEDO*DVX(IT+IC)
140   CONTINUE
      CALL ALSB (IJAT,IJAT,PSSB,IER,IJAT)
      IF (IER.NE.0) CALL XABORT('SYBRX3: SINGULAR MATRIX.')
*
* COMPUTATION OF PIS*PSSB*PSJ
      I1=0
      DO 240 IKK=1,NMERGE
      IKG=IGEN(IKK)
      I1P=NMC(IKG)
      I2=NMC(IKG+1)-I1P
      IT=NCOUR*(IKK-1)
      A=XX(IKG)
      B=YY(IKG)
      DO 230 I=1,I2
      ZZZ=PIS(I1P+I)
      IF (NCOUR.EQ.4) THEN
         DEN1=2.0*(A+B)
         PBB(1)=ZZZ*B/DEN1
         PBB(2)=ZZZ*B/DEN1
         PBB(3)=ZZZ*A/DEN1
         PBB(4)=ZZZ*A/DEN1
      ELSE
         DO 150 IC=1,NCOUR
150      PBB(IC)=ZZZ/6.0
      ENDIF
      DO 230 IC=1,NCOUR
      ICC=MIX(IT+IC)
      ZZZ=PBB(IC)
      J1=0
      DO 230 JKK=1,NMERGE
      JKG=IGEN(JKK)
      J1P=NMC(JKG)
      J2=NMC(JKG+1)-J1P
      JT=NCOUR*(JKK-1)
      SURFA=(0.5*PI*ROUT(JKG))/SCALE2(JKG)
      RJ1=0.0
      DO 220 J=1,J2-1
      RJ2=PI*RAYRE(J1P+J+1)**2
      PBJ=PIS(J1P+J)*(RJ2-RJ1)/SURFA
      DO 210 JC=1,NCOUR
      JCC=MIX(JT+JC)
210   P(I1+I,J1+J)=P(I1+I,J1+J)+ZZZ*DVX(JT+JC)*PSSB(JCC,IJAT+ICC)*PBJ
220   RJ1=RJ2
      RJ2=PI*ROUT(JKG)**2
      PBJ=PIS(J1P+J2)*(RJ2-RJ1)*SCALE1(JKG)/SURFA
      DO 225 JC=1,NCOUR
      JCC=MIX(JT+JC)
225   P(I1+I,J1+J2)=P(I1+I,J1+J2)+ZZZ*DVX(JT+JC)*PSSB(JCC,IJAT+ICC)*PBJ
230   J1=J1+J2
240   I1=I1+I2
      RETURN
      END
