*DECK XDRLGS
      SUBROUTINE XDRLGS(IPLIB ,IGS   ,NPROC ,MINLEG,MAXLEG,IORD  ,
     >                  NGROUP,INDPRO,ITYPRO,XSREC ,SCAT  ,XSSCMP,
     >                  NJJ   ,IJJ   )
C
C------------------------------  XDRLGS  ------------------------------
C
C  PROGRAMME STATISTICS:
C     NAME     : XDRLGS
C     ENTRY    : XDRLGS
C     USE      : GET/SAVE XS FROM/ON IPLIB
C     MODIFIED : 97-08-17
C     AUTHOR   : G. MARLEAU
C
C  ROUTINE PARAMETERS:
C   INPUT
C     IPLIB  : POINTER TO THE INTERNAL LIBRARY      I
C     IGS    : GET OR SAVE FLAG                     I
C              =  2 SAVE SCAT ONLY
C              =  1 SAVE ALL
C              = -1 GET ALL
C              = -2 GET SCAT ONLY
C     NPROC  : NUMBER OF TERMS TO PROCESS           I
C     MINLEG : MIMIMUM LEGENDRE ORDER TO PROCESS    I
C              FOR SCATTERING
C     MAXLEG : MAXIMUM LEGENDRE ORDER TO PROCESS    I
C              FOR SCATTERING
C     IORD   : ORDER                                I
C              = 1 CONSTANT
C              = 2 LINEAR
C              = 3 QUADRATIC
C     NGROUP : NUMBER OF GROUPS                     I
C     INDPRO : VECTOR FOR CROSS SECTION TO PROCESS  I(NPROC)
C              = 0 -> DO NOT PROCESS
C              > 0 -> PROCESS
C  INPUT/OUTPUT
C     ITYPRO : VECTOR FOR CROSS SECTION PROCESSED   I(NPROC)
C              PROCESSED
C              = 0 -> ABSENT  (NOT PROCESSED)
C              > 0 -> PRESENT (PROCESSED)
C     XSREC  : CROSS SECTION RECORDS                R(NGROUP,NPROC)
C              DEFAULT CROSS SECTIONS FOR
C              IRPROC=1,NDPROC
C              SCATTERING CROSS SECTION OF
C              ORDER MINLEG TO MAXLEG FOR
C              IRPROC=NDPROC+MINLEG+1,NDPROC+MAXLEG+1
C     SCAT   : COMPLETE SCATTERING MATRIX           R(NGROUP,
C              SCAT(JG,IG) IS FROM IG TO JG           NGROUP,
C              FOR ORDER MINLEG TO MAXLEG     MAXLEG-MINLEG+1)
C  WORK VECTOR
C     XSSCMP : COMPRESS SCATTERING FOR TRANSFER     R(NGROUP*
C                                                     NGROUP)
C     NJJ    : NB SCATTERING GROUP                  I(NGROUP)
C     IJJ    : MAXIMUM SCATTERING GROUP             I(NGROUP)
C
C------------------------------  XDRLGS  ------------------------------
C
      INTEGER          IPLIB ,IGS   ,NPROC ,MINLEG,MAXLEG,IORD  ,
     >                 NGROUP,INDPRO(NPROC),ITYPRO(NPROC),
     >                 NJJ(NGROUP),IJJ(NGROUP)
      REAL             XSREC(NGROUP,NPROC),
     >                 SCAT(NGROUP,NGROUP,MAXLEG-MINLEG+1),
     >                 XSSCMP(NGROUP*NGROUP)
C----
C  LOCAL PARAMETERS
C  NDPROC = NUMBER OF DEFAULT CROSS SECTIONS = 20
C  NAMDXS = NAME OF NDPROC DEFAULT XS
C  SCATTERING CROSS SECTIONS START AT NDPROC+1 WITH
C  NAME NAMSCT='SIGS'//NAMLEG AND NAMSCT='SCAT'//NAMLEG
C  WITH NAMLEG DEFINED BY
C  WRITE(NAMLEG ,'(I2)') ILEG
C  FOR ILEG=0 TO NDPROC-NPROC-1
C----
      PARAMETER       (NDPROC=20,IOUT=6)
      CHARACTER        NAMDXS(NDPROC)*6,NAMLEG*2,NORD*6
      SAVE             NAMDXS
      DATA    NAMDXS  /'TOTAL ','TRANC ','NUSIGF','NFTOT ','CHI   ',
     >                 'NU    ','NG    ','NHEAT ','N2N   ','N3N   ',
     >                 'N4N   ','NP    ','NA    ','GOLD  ','ABS   ',
     >                 'NWT0  ','STRD  ','STRD X','STRD Y','STRD Z'/
      IF(IORD.EQ.1) THEN
        NORD='      '
        IODIV=1
      ELSE IF(IORD.EQ.2) THEN
        NORD='   LIN'
        IODIV=2
      ELSE IF(IORD.EQ.3) THEN
        NORD='   QUA'
        IODIV=4
      ENDIF
C----
C  READ/INITIALIZE STATE VECTOR
C----
      CALL LCMLEN(IPLIB,'XS-SAVED',LONG,ITYP)
      IF(LONG.EQ.NPROC) THEN
        CALL LCMGET(IPLIB,'XS-SAVED',ITYPRO)
      ELSE IF(LONG.EQ.0) THEN
        CALL XDRSET(ITYPRO,NPROC,0)
      ELSE
        WRITE(IOUT,9000) NPROC,LONG
        CALL XABORT('XDRLGS: INVALID VALUE FOR NPROC')
      ENDIF
      IF(IGS.GT.0) THEN
C----
C  SAVE LOCAL DEFAULT XS IF REQUIRED
C----
        IF(IGS.EQ.1) THEN
          DO 100 IXSR=1,NDPROC
            IF(INDPRO(IXSR).EQ.1) THEN
              IXSTN=MOD(ITYPRO(IXSR)/IODIV,2)
C----
C  FIND IF XS NOT ALL 0.0
C----
              DO 110 IG=1,NGROUP
                IF(XSREC(IG,IXSR).NE.0.0) THEN
                  IF(IXSTN.EQ.0) THEN
                    ITYPRO(IXSR)=ITYPRO(IXSR)+IODIV
                    IXSTN=1
                  ENDIF
                  GO TO 115
                ENDIF
 110          CONTINUE
 115          CONTINUE
              IF(IXSTN.NE.0) THEN
C----
C  SAVE IF XS NOT ALL 0.0
C----
                CALL LCMPUT(IPLIB,NAMDXS(IXSR)//NORD,NGROUP,2,
     >                      XSREC(1,IXSR))
              ENDIF
            ENDIF
 100      CONTINUE
        ENDIF
C----
C  SCATTERING MATRIX
C----
        IL=0
        DO 120 ILEG=MINLEG+1,MAXLEG+1
          IXSR=NDPROC+ILEG
          IL=IL+1
          IF(INDPRO(IXSR).EQ.1) THEN
            IXSTN=MOD(ITYPRO(IXSR)/IODIV,2)
C----
C  FIND IF SCATTERING XS NOT ALL 0.0
C----
            DO 130 IG=1,NGROUP
              DO 131 JG=1,NGROUP
                IF(SCAT(IG,JG,IL).NE.0.0) THEN
                  IF(IXSTN.EQ.0) THEN
                    ITYPRO(IXSR)=ITYPRO(IXSR)+IODIV
                    IXSTN=1
                  ENDIF
                  GO TO 135
                ENDIF
 131          CONTINUE
 130        CONTINUE
 135        CONTINUE
            IF(IXSTN.NE.0) THEN
              WRITE(NAMLEG,'(I2)') ILEG-1
              CALL LCMPUT(IPLIB,'SIGS'//NAMLEG//NORD,NGROUP,2,
     >                    XSREC(1,IXSR))
C----
C  COMPRESS SCATTERING MATRIX
C  SCAT(IGTO,IGFROM) REPRESENT SCATTERING CROSS SECTION
C    FROM GROUP "IGFROM" TO GROUP "IGTO"
C  IJJ(IGTO) IS MAXIMUM GROUP NUMBER
C    WITH SCATTERING TO "IGTO" GROUP
C  NJJ(IGTO) IS NUMBER OF GROUPS
C    WITH SCATTERING TO "IGTO" GROUP
C  XSSCMP(IX) IS COMPRESSED SCATTERING MATRIX
C  IX CAN BE LOCALIZED IN SCAT(IGTO,IGFROM) USING
C    IF(IGTO=1) THEN
C      IPOSD=1
C    ELSE
C      IPOSD=1+SUM( NJJ(IGF) , IGF=1,IGTO-1)
C    ENDIF
C    IF(IGFROM.GT.IJJ(IGTO)) THEN
C      XSSCMP NOT STORED
C    ELSE IF(IGFROM.LT.IJJ(IGTO)-NJJ(IGTO)+1) THEN
C      XSSCMP NOT STORED
C    ELSE
C      IX=IPOSD+IJJ(IGTO)-IGFROM
C      XSSCMP(IX)=SCAT(IGTO,IGFROM)
C  ENDIF
C----
              NXSCMP=0
              DO 140 IGTO=1,NGROUP
                IGMIN=IGTO
                IGMAX=IGTO
                DO 150 IGFROM=1,NGROUP
                  IF(SCAT(IGTO,IGFROM,IL).NE.0.0) THEN
                    IGMIN=MIN(IGMIN,IGFROM)
                    IGMAX=MAX(IGMAX,IGFROM)
                  ENDIF
 150            CONTINUE
                IJJ(IGTO)=IGMAX
                NJJ(IGTO)=IGMAX-IGMIN+1
                DO 160 IGFROM=IGMAX,IGMIN,-1
                  NXSCMP=NXSCMP+1
                  XSSCMP(NXSCMP)=SCAT(IGTO,IGFROM,IL)
 160            CONTINUE
 140          CONTINUE
              CALL LCMPUT(IPLIB,'NJJ '//NAMLEG//NORD,NGROUP,1,NJJ)
              CALL LCMPUT(IPLIB,'IJJ '//NAMLEG//NORD,NGROUP,1,IJJ)
              CALL LCMPUT(IPLIB,'SCAT'//NAMLEG//NORD,NXSCMP,2,XSSCMP)
            ENDIF
          ENDIF
 120    CONTINUE
        CALL LCMPUT(IPLIB,'XS-SAVED',NPROC,1,ITYPRO)
      ELSE
C----
C  GET LOCAL DEFAULT XS IF REQUIRED
C----
        IF(IGS.EQ.-1) THEN
          DO 200 IXSR=1,NDPROC
            IF(INDPRO(IXSR).EQ.1) THEN
              IXSTN=MOD(ITYPRO(IXSR)/IODIV,2)
C----
C  READ IF IXSTN = 1
C  INITIALIZE TO 0.0 IF IXSTN = 0
C----
              IF(IXSTN.EQ.1) THEN
                CALL LCMLEN(IPLIB,NAMDXS(IXSR)//NORD,LONG,ITYP)
                IF(LONG .EQ. 0) THEN 
                  CALL XDRSET(XSREC(1,IXSR),NGROUP,0.0)
                ELSE
                  CALL LCMGET(IPLIB,NAMDXS(IXSR)//NORD,XSREC(1,IXSR))
                ENDIF
              ELSE
                CALL XDRSET(XSREC(1,IXSR),NGROUP,0.0)
              ENDIF
            ENDIF
 200      CONTINUE
        ENDIF
C----
C  SCATTERING MATRIX
C----
        IL=0
        DO 220 ILEG=MINLEG+1,MAXLEG+1
          IXSR=NDPROC+ILEG
          IL=IL+1
          IF(INDPRO(IXSR).EQ.1) THEN
C----
C  READ IF IXSTN = 1
C  INITIALIZE TO 0.0 IF IXSTN = 0
C----
            CALL XDRSET(SCAT(1,1,IL),NGROUP*NGROUP,0.0)
            IXSTN=MOD(ITYPRO(IXSR)/IODIV,2)
            IF(IXSTN.EQ.1) THEN
              WRITE(NAMLEG,'(I2)') ILEG-1
              CALL LCMLEN(IPLIB,'SIGS'//NAMLEG//NORD,LONG,ITYP)
              IF(LONG .EQ. 0) THEN
                CALL XDRSET(XSREC(1,IXSR),NGROUP,0.0)
                CALL XDISET(NJJ,NGROUP,1)
                DO IGTO=1,NGROUP
                  IJJ(IGTO)=IGTO
                ENDDO  
                CALL XDRSET(XSSCMP,NGROUP,0.0)
              ELSE
                CALL LCMGET(IPLIB,'SIGS'//NAMLEG//NORD,XSREC(1,IXSR))
                CALL LCMGET(IPLIB,'NJJ '//NAMLEG//NORD,NJJ)
                CALL LCMGET(IPLIB,'IJJ '//NAMLEG//NORD,IJJ)
                CALL LCMGET(IPLIB,'SCAT'//NAMLEG//NORD,XSSCMP)
                NXSCMP=0
C----
C  DECOMPRESS SCATTERING MATRIX
C  SCAT(IGTO,IGFROM) REPRESENT SCATTERING CROSS SECTION
C    FROM GROUP "IGFROM" TO GROUP "IGTO"
C  IJJ(IGTO) IS MAXIMUM GROUP NUMBER
C    WITH SCATTERING TO "IGTO" GROUP
C  NJJ(IGTO) IS NUMBER OF GROUPS
C    WITH SCATTERING TO "IGTO" GROUP
C  XSSCMP(IX) IS COMPRESSED SCATTERING MATRIX
C  SCAT(IGTO,IGFROM) CAN BE LOCALIZED IN XSSCMP(IX) USING
C    IF(IGTO=1) THEN
C      IPOSD=1
C    ELSE
C      IPOSD=1+SUM( NJJ(IGF) , IGF=1,IGTO-1)
C    ENDIF
C    IF(IGFROM.GT.IJJ(IGTO)) THEN
C      SCAT(IGTO,IGFROM)=0.0
C    ELSE IF(IGFROM.LT.IJJ(IGTO)-NJJ(IGTO)+1) THEN
C      SCAT(IGTO,IGFROM)=0.0
C    ELSE
C      SCAT(IGTO,IGFROM)=XSSCMP(IPOSD+IJJ(IGTO)-IGFROM)
C    ENDIF
C----
                DO 240 IGTO=1,NGROUP
                  IGMAX=IJJ(IGTO)
                  IGMIN=IGMAX-NJJ(IGTO)+1
                  DO 250 IGFROM=IGMAX,IGMIN,-1
                    NXSCMP=NXSCMP+1
                    SCAT(IGTO,IGFROM,IL)=XSSCMP(NXSCMP)
 250              CONTINUE
 240            CONTINUE
              ENDIF
            ELSE
              CALL XDRSET(XSREC(1,IXSR),NGROUP,0.0)
            ENDIF
          ENDIF
 220    CONTINUE
      ENDIF
      RETURN
C----
C  ABORT FORMAT
C----
 9000 FORMAT(' XDRLGS: ****** ABORT ******'/
     >       ' INVALID LENGTH OF RECORD XS-SAVED '/
     >       ' STORAGE SPACE NPROC    = ',I10/
     >       ' LENGTH OF RECORD LONG  = ',I10/
     >       ' ***************************')
      END
