*DECK XELPA3
      SUBROUTINE XELPA3( NGAUSS, XGAUSS, WGAUSS, SIGMAT,  DELTA,   PART)
************************************************************************
*                                                                      *
*           NAME: XELPA3                                               *
*      COMPONENT: EXCELL                                               *
*        VERSION: 1.0                                                  *
*       CREATION: 89/01                                                *
*       MODIFIED: 00/03 (R.R.) BRING GAUSS PTS/WGTS AS 1ST ARGUMENTS   *
*                              IN *XELPA0* CALLS                       *
*                              DECLARE ALL VARIABLE TYPES              *
*         AUTHOR: ROBERT ROY                                           *
*                                                                      *
*     SUBROUTINE: THIS ROUTINE WILL COMPUTE CP FOR A HOMOGENEOUS       *
*                 3D RECTANGULAR BLOCK.                                *
*                                                                      *
*           NOTE: PART(IN6(I,J)) FOR I,J < 0                           *
*                 CONTAINS THE SURFACE TO SURFACE PROBS (PI*SURF-BEG)  *
*                 WHERE XINF: SURFACE 1   XSUP: SURFACE 2              *
*                       YINF: SURFACE 3   YSUP: SURFACE 4              *
*                       ZINF: SURFACE 5   ZSUP: SURFACE 6              *
*                                                                      *
*                 PART(IN6(0,J)) FOR J < 0 CONTAINS PI*SURFACE         *
*                                                                      *
*      REFERENCE: 'ANALYTIC REDUCTIONS FOR TRANSMISSION AND LEAKAGE    *
*                  PROBABILITIES IN FINITE TUBES AND HEXAHEDRA',       *
*                  G.MARLEAU, R.ROY AND A.HEBERT,                      *
*                  NUCLEAR SCIENCE & ENGINEERING, V 104, P 204 (1990). *
*                                                                      *
*--------+------------- P A R A M E T E R S --------------+--+---------*
*  NAME  /                  DESCRIPTION                   /IO/  MODE   *
*--------+------------------------------------------------+--+---------*
* NGAUSS / # OF GAUSS INTEGRATION POINTS & WEIGHTS.       /I./INT      *
* XGAUSS / "ARRAY" WITH GAUSS POINTS.                     /I./REL      *
*        /                                                /  / (NGAUSS)*
* WGAUSS / "ARRAY" WITH GAUSS WEIGHTS.                    /I./REL      *
*        /                                                /  / (NGAUSS)*
* SIGMAT / TOTAL XS                                       /I./REL      *
* DELTA  / DIMENSIONS OF A 3D RECTANGLE ( DX, DY, DZ )    /I./REL(3)   *
* PART   / CP FOR A 3D RECTANGLE                          /.O/REL(36)  *
*--------+-------------- R O U T I N E S -----------------+--+---------*
*  NAME  /                  DESCRIPTION                                *
*--------+-------------------------------------------------------------*
* XELPA0 / PSS  FOR A VOID 3D BLOCK                                    *
* XELPA1 / PSS  FOR A NON-VOID 3D BLOCK                                *
*  TABEN / TABULAR EXPONENTIAL-INTEGRAL FUNCTIONS                      *
************************************************************************
C
      IMPLICIT   NONE
C
C     DECLARE    DUMMY ARGUMENTS
      INTEGER    NGAUSS
      REAL       XGAUSS(NGAUSS),WGAUSS(NGAUSS),SIGMAT,DELTA(3),PART(36)
C
C     DECLARE    LOCAL VARIABLES
      REAL       DMFP(3), PSS(15)
      LOGICAL    AEQB, AEQC, BEQC, POUT, CUBE
      REAL       PI, EPS, TRONC
      PARAMETER( PI=3.1415926536, EPS=1.E-6, TRONC= 5.E-3 )
      REAL       CHORD, A, B, C, V, SURFA, SURFB, SURFC,
     >           PARAA, PARAB, PARAC, PVA, PVB, PVC, PVVA, PVVB, PVVC,
     >           PERAB, PERAC, PERBA, PERBC, PERCA, PERCB,
     >           PARA1, PARA2, PARB1, PARB2, PARC1, PARC2,
     >           PERAB1, PERAB2, PERAC1, PERAC2, PERBA1, PERBA2,
     >           PERBC1, PERBC2, PERCA1, PERCA2, PERCB1, PERCB2,
     >           FACT1, FACT2
C
C     DECLARE    FUNCTIONS (AND ARGUMENTS)
      INTEGER    IND, IN6, I, J
      LOGICAL    EQREAL
      REAL       XA, XB
      REAL       TABEN
C
      IND(I,J) = MAX(I     +4,J     +4)*(MAX(I     +4,J     +4)-1)/2
     >         + MIN(I     +4,J     +4)
      EQREAL(XA,XB)= ( ABS(XA - XB) / MAX(XA , XB) ) .LT. EPS
      IN6(I,J)= MAX(I+7,J+7)*(MAX(I+7,J+7)-1)/2 + MIN(I+7,J+7)
C
      CHORD = 2.*DELTA(1)*DELTA(2)*DELTA(3)
     >      / (DELTA(1)*DELTA(2)+DELTA(1)*DELTA(3)+DELTA(2)*DELTA(3))
      IF( SIGMAT*CHORD.LT.TRONC )THEN
C
C        COMPUTE PSS FOR     VOID CASE
         A=DELTA(1)
         B=DELTA(2)
         C=DELTA(3)
         AEQB=EQREAL(A,B)
         AEQC=EQREAL(A,C)
         BEQC=EQREAL(B,C)
         POUT=AEQB.OR.AEQC.OR.BEQC
         CUBE=AEQB.AND.BEQC
         SURFA = PI * B * C
         SURFB = PI * A * C
         SURFC = PI * A * B
         PSS(IND( 0,-1))= SURFA
         PSS(IND( 0,-2))= SURFB
         PSS(IND( 0,-3))= SURFC
         IF(CUBE)THEN
            CALL XELPA0( NGAUSS, XGAUSS, WGAUSS,
     >                   A, B, C, SURFC, PARAC, PERCB, PERCA,
     >                                   PVC,   PVVC )
            PSS(IND(-1,-1))= PARAC
            PSS(IND(-1,-2))= PERCB
            PSS(IND(-1,-3))= PERCA
            PSS(IND(-2,-2))= PSS(IND(-1,-1))
            PSS(IND(-2,-3))= PSS(IND(-1,-2))
            PSS(IND(-3,-3))= PSS(IND(-1,-1))
            PVA   = PVC
            PVB   = PVC
            PVVA  = PVVC
            PVVB  = PVVC
         ELSEIF(POUT)THEN
            IF(AEQB)THEN
               CALL XELPA0( NGAUSS, XGAUSS, WGAUSS,
     >                      A, B, C, SURFC, PARAC, PERCB, PERCA,
     >                                      PVC,   PVVC )
               CALL XELPA0( NGAUSS, XGAUSS, WGAUSS,
     >                      A, C, B, SURFB, PARAB, PERBC, PERBA,
     >                                      PVB,   PVVB )
               PSS(IND(-1,-1))= PARAB
               PSS(IND(-1,-2))= PERBA
               PSS(IND(-1,-3))= PERBC
               PSS(IND(-2,-2))= PSS(IND(-1,-1))
               PSS(IND(-2,-3))= PSS(IND(-1,-3))
               PSS(IND(-3,-3))= PARAC
               PVA   = PVB
               PVVA  = PVVB
            ELSEIF(AEQC)THEN
               CALL XELPA0( NGAUSS, XGAUSS, WGAUSS,
     >                      A, C, B, SURFB, PARAB, PERBC, PERBA,
     >                                      PVB,   PVVB )
               CALL XELPA0( NGAUSS, XGAUSS, WGAUSS,
     >                      A, B, C, SURFC, PARAC, PERCB, PERCA,
     >                                      PVC,   PVVC )
               PSS(IND(-1,-1))= PARAC
               PSS(IND(-1,-2))= PERCB
               PSS(IND(-1,-3))= PERCA
               PSS(IND(-2,-2))= PARAB
               PSS(IND(-2,-3))= PSS(IND(-1,-2))
               PSS(IND(-3,-3))= PSS(IND(-1,-1))
               PVA   = PVC
               PVVA  = PVVC
            ELSEIF(BEQC)THEN
               CALL XELPA0( NGAUSS, XGAUSS, WGAUSS,
     >                      B, C, A, SURFA, PARAA, PERAC, PERAB,
     >                                      PVA,   PVVA )
               CALL XELPA0( NGAUSS, XGAUSS, WGAUSS,
     >                      B, A, C, SURFC, PARAC, PERCA, PERCB,
     >                                      PVC,   PVVC )
               PSS(IND(-1,-1))= PARAA
               PSS(IND(-1,-2))= PERAC
               PSS(IND(-1,-3))= PSS(IND(-1,-2))
               PSS(IND(-2,-2))= PARAC
               PSS(IND(-2,-3))= PERCB
               PSS(IND(-3,-3))= PSS(IND(-2,-2))
               PVB   = PVC
               PVVB  = PVVC
            ENDIF
         ELSE
            CALL XELPA0( NGAUSS, XGAUSS, WGAUSS,
     >                   A, B, C, SURFC, PARAC, PERCB, PERCA,
     >                                   PVC,   PVVC )
            CALL XELPA0( NGAUSS, XGAUSS, WGAUSS,
     >                   B, C, A, SURFA, PARAA, PERAC, PERAB,
     >                                   PVA,   PVVA )
            CALL XELPA0( NGAUSS, XGAUSS, WGAUSS,
     >                   A, C, B, SURFB, PARAB, PERBC, PERBA,
     >                                   PVB,   PVVB )
            PSS(IND(-1,-1))= PARAA
            PSS(IND(-1,-2))= PERAB
            PSS(IND(-1,-3))= PERAC
            PSS(IND(-2,-2))= PARAB
            PSS(IND(-2,-3))= PERBC
            PSS(IND(-3,-3))= PARAC
         ENDIF
         V=4.0*PI*A*B*C
         PSS( IND(+1, 0) )=  V
C
         PSS( IND(+1,-1) )= PVA
         PSS( IND(+1,-2) )= PVB
         PSS( IND(+1,-3) )= PVC
         PSS( IND(+1,+1) )= PVVA + PVVB + PVVC
      ELSE
         DO 10 I   = 1, 3
             DMFP(I)= DELTA(I)*SIGMAT
   10    CONTINUE
C
C        COMPUTE PSS FOR NON-VOID CASE
         A=DMFP(1)
         B=DMFP(2)
         C=DMFP(3)
         AEQB=EQREAL(A,B)
         AEQC=EQREAL(A,C)
         BEQC=EQREAL(B,C)
         POUT=AEQB.OR.AEQC.OR.BEQC
         CUBE=AEQB.AND.BEQC
         SURFA = PI * B * C
         SURFB = PI * A * C
         SURFC = PI * A * B
         PSS(IND( 0,-1))= SURFA
         PSS(IND( 0,-2))= SURFB
         PSS(IND( 0,-3))= SURFC
         IF(CUBE)THEN
            CALL XELPA1( NGAUSS, XGAUSS, WGAUSS,
     >                        A,      B,      C,
     >                    PARC1, PERCB1, PERCA2 )
            PSS(IND(-1,-1))= 2.*((PI*A*B)*TABEN(3,C)+PARC1+PARC1)
            PSS(IND(-1,-2))= PERCB1+PERCA2
            PSS(IND(-1,-3))= PSS(IND(-1,-2))
            PSS(IND(-2,-2))= PSS(IND(-1,-1))
            PSS(IND(-2,-3))= PSS(IND(-1,-2))
            PSS(IND(-3,-3))= PSS(IND(-1,-1))
         ELSEIF(POUT)THEN
            IF(AEQB)THEN
               CALL XELPA1( NGAUSS, XGAUSS, WGAUSS,
     >                           A,      B,      C,
     >                       PARC1, PERCB1, PERCA2 )
               CALL XELPA1( NGAUSS, XGAUSS, WGAUSS,
     >                           A,      C,      B,
     >                       PARB1, PERBC1, PERBA2 )
               CALL XELPA1( NGAUSS, XGAUSS, WGAUSS,
     >                           C,      A,      B,
     >                       PARB2, PERBA1, PERBC2 )
               PSS(IND(-1,-1))= 2.*((PI*A*C)*TABEN(3,B)+PARB1+PARB2)
               PSS(IND(-1,-2))= PERBA1+PERBA2
               PSS(IND(-1,-3))= PERBC1+PERBC2
               PSS(IND(-2,-2))= PSS(IND(-1,-1))
               PSS(IND(-2,-3))= PSS(IND(-1,-3))
               PSS(IND(-3,-3))= 2.*((PI*A*B)*TABEN(3,C)+PARC1+PARC1)
            ELSEIF(AEQC)THEN
               CALL XELPA1( NGAUSS, XGAUSS, WGAUSS,
     >                           A,      C,      B,
     >                       PARB1, PERBC1, PERBA2 )
               CALL XELPA1( NGAUSS, XGAUSS, WGAUSS,
     >                           A,      B,      C,
     >                       PARC1, PERCB1, PERCA2 )
               CALL XELPA1( NGAUSS, XGAUSS, WGAUSS,
     >                           B,      A,      C,
     >                       PARC2, PERCA1, PERCB2 )
               PSS(IND(-1,-1))= 2.*((PI*A*B)*TABEN(3,C)+PARC1+PARC2)
               PSS(IND(-1,-2))= PERCB1+PERCB2
               PSS(IND(-1,-3))= PERCA1+PERCA2
               PSS(IND(-2,-2))= 2.*((PI*A*C)*TABEN(3,B)+PARB1+PARB1)
               PSS(IND(-2,-3))= PSS(IND(-1,-2))
               PSS(IND(-3,-3))= PSS(IND(-1,-1))
            ELSEIF(BEQC)THEN
               CALL XELPA1( NGAUSS, XGAUSS, WGAUSS,
     >                           B,      C,      A,
     >                       PARA1, PERAC1, PERAB2 )
               CALL XELPA1( NGAUSS, XGAUSS, WGAUSS,
     >                           B,      A,      C,
     >                       PARC2, PERCA1, PERCB2 )
               CALL XELPA1( NGAUSS, XGAUSS, WGAUSS,
     >                           A,      B,      C,
     >                       PARC1, PERCB1, PERCA2 )
               PSS(IND(-1,-1))= 2.*((PI*B*C)*TABEN(3,A)+PARA1+PARA1)
               PSS(IND(-1,-2))= PERAC1+PERAB2
               PSS(IND(-1,-3))= PSS(IND(-1,-2))
               PSS(IND(-2,-2))= 2.*((PI*A*B)*TABEN(3,C)+PARC1+PARC2)
               PSS(IND(-2,-3))= PERCB1+PERCB2
               PSS(IND(-3,-3))= PSS(IND(-2,-2))
            ENDIF
         ELSE
            CALL XELPA1( NGAUSS, XGAUSS, WGAUSS,
     >                        A,      B,      C,
     >                    PARC1, PERCB1, PERCA2 )
            CALL XELPA1( NGAUSS, XGAUSS, WGAUSS,
     >                        B,      A,      C,
     >                    PARC2, PERCA1, PERCB2 )
            CALL XELPA1( NGAUSS, XGAUSS, WGAUSS,
     >                        B,      C,      A,
     >                    PARA1, PERAC1, PERAB2 )
            CALL XELPA1( NGAUSS, XGAUSS, WGAUSS,
     >                        C,      B,      A,
     >                    PARA2, PERAB1, PERAC2 )
            CALL XELPA1( NGAUSS, XGAUSS, WGAUSS,
     >                        A,      C,      B,
     >                    PARB1, PERBC1, PERBA2 )
            CALL XELPA1( NGAUSS, XGAUSS, WGAUSS,
     >                        C,      A,      B,
     >                    PARB2, PERBA1, PERBC2 )
            PSS(IND(-1,-1))= 2.*((PI*B*C)*TABEN(3,A)+PARA1+PARA2)
            PSS(IND(-1,-2))= PERAB1+PERAB2
            PSS(IND(-1,-3))= PERAC1+PERAC2
            PSS(IND(-2,-2))= 2.*((PI*A*C)*TABEN(3,B)+PARB1+PARB2)
            PSS(IND(-2,-3))= PERBC1+PERBC2
            PSS(IND(-3,-3))= 2.*((PI*A*B)*TABEN(3,C)+PARC1+PARC2)
         ENDIF
         V=4.0*PI*A*B*C
C
C        APPLY CONSERVATION LAWS FOR "PVS" AND "PVV"
         PSS(IND(+1, 0))= V
         PSS(IND(+1,+1))= V
         DO 35 I=-3, -1
            PSS(IND(+1, I))= PSS(IND( 0, I)) +    PSS(IND( I, I))
            DO 45 J=-3, -1
               PSS(IND(+1, I))= PSS(IND(+1, I)) - 2. * PSS(IND( I, J))
   45       CONTINUE
            PSS(IND(+1,+1))= PSS(IND(+1,+1)) - 2. * PSS(IND(+1, I))
   35    CONTINUE
      ENDIF
C
C     FILLING THE FULL MATRIX OF COLL. PROB.
      PART( 1)=  0.0
      PART( 2)= PSS( 1)
      PART( 3)=  0.0
      PART( 4)= PSS( 2)
      PART( 5)= PSS( 2)
      PART( 6)=  0.0
      PART( 7)= PSS( 2)
      PART( 8)= PSS( 2)
      PART( 9)= PSS( 3)
      PART(10)=  0.0
      PART(11)= PSS( 4)
      PART(12)= PSS( 4)
      PART(13)= PSS( 5)
      PART(14)= PSS( 5)
      PART(15)=  0.0
      PART(16)= PSS( 4)
      PART(17)= PSS( 4)
      PART(18)= PSS( 5)
      PART(19)= PSS( 5)
      PART(20)= PSS( 6)
      PART(21)=  0.0
      PART(22)= PSS( 7)
      PART(23)= PSS( 7)
      PART(24)= PSS( 8)
      PART(25)= PSS( 8)
      PART(26)= PSS( 9)
      PART(27)= PSS( 9)
      PART(28)=  0.0
      PART(29)= PSS(11)
      PART(30)= PSS(11)
      PART(31)= PSS(12)
      PART(32)= PSS(12)
      PART(33)= PSS(13)
      PART(34)= PSS(13)
      PART(35)= PSS(14)
      PART(36)= PSS(15)
C
      IF( SIGMAT*CHORD.GE.TRONC )THEN
         FACT1 = 1./SIGMAT
         FACT2 = FACT1*FACT1
         DO 30 I   =-6, 1
            DO 20 J   =-6, I
               PART(IN6(I,J))= FACT2 * PART(IN6(I,J))
               IF(I.EQ.1) PART(IN6(I,J))= FACT1 * PART(IN6(I,J))
               IF(J.EQ.1) PART(IN6(I,J))= FACT1 * PART(IN6(I,J))
   20       CONTINUE
   30    CONTINUE
      ENDIF
C
      RETURN
      END
