*DECK XHX2D1
      SUBROUTINE XHX2D1 (NGPT,ZGAUS,WGAUS,COTE,SIGT,TRONC,PII,PIS,PSS)
C
C---------------------------  XHX2D1  ---------------------------------
C
C   1-  PROGRAMME STATISTICS:
C
C          NAME       :  XHX2D1
C          FUNCTION   :  COMPUTE DP1 COLLISION, LEAKAGE AND TRANSMISSION
C                        PROBABILITIES FOR HEXAGONAL 2D GEOMETRIES
C          REVISED    :  14-03-1990
C          AUTHOR     :  M.OUISLOUMEN
C
C   2-  PROGRAMME PARAMETERS:
C
C INPUT PARAMETERS.
C   NGPT    : NUMBER OF GAUSS INTEGRATION POINTS.        I
C   COTE    : LENGTH OF ONE OF SIDES OF THE HEXAGONE     R
C   SIGT    : TOTAL CROSS SECTION.                       R
C   TRONC   : VOIDED BLOCK CUTOFF CRITERION.             R
C   ZGAUS   : GAUSS-LEGENDRE INTEGRATION POINTS          R
C   WGAUS   : GAUSS-LEGENDRE  INTEGRATION WEIGHTS        R
C
C OUTPUT PARAMETERS.
C   PII     : VOLUME TO VOLUME REDUCED PROBABILITY.      R
C   PIS     : LEAKAGE PROBABILITY                        R(18)
C             PIS(I)  -> VOLUME TO SIDE I
C   P       : TRANSMISSION PROBABILITY                   R(18,18)
C             PSS(I,J) -> SIDE I TO SIDE J
C
C                                            SIDE A,B,C
C                      SIDE 4,5,6           DIR A -> ISOTROPIC
C                     XXXXXXXX              DIR C -> TANGENT TO SURFACE
C                    X        X             DIR B -> NORMAL TO SURFACE
C      SIDE 7,8,9   X          X SIDE 1,2,3
C                  X            X
C                 X              X
C                  X            X
C   SIDE 10,11,12   X          X SIDE 16,17,18
C                    X        X
C                     XXXXXXXX
C                      SIDE 13,14,15
C
C---------------------------   XHX2D1  --------------------------------
C
      PARAMETER (MKI3=600,MKI4=600,MKI5=600)
      PARAMETER (PI=3.141592653589793,SQRT3=1.732050807568877,
     +           SQRT2=1.414213562373095,ALOG2=.693147180559945,
     +           ALOG3=1.0986122886681097,ALOGX=.7676517525907618)
C
      DIMENSION  PIS(18),PSS(18,*),ZGAUS(NGPT),WGAUS(NGPT),P(16)
      DIMENSION  TAU(3),FKI3(3),FKI4(3),FKI5(3),FKI6(3),IROT(18,18)
C
C ASSUME THAT BICKLEY KI TABLES HAVE THE SAME TABULATION POINTS AND
C THE SAME TRUNCATION LIMIT.
C
      COMMON /BICKL3/BI3(0:MKI3,3),PAS3,XLIM3,L3
      COMMON /BICKL4/BI4(0:MKI4,3),PAS4,XLIM4,L4
      COMMON /BICKL5/BI5(0:MKI5,3),PAS5,XLIM5,L5
C
      SAVE IROT
      DATA IROT /
     +          0, 0, 0, 1, 2,-3, 7, 8,-9,13,14, 0, 7, 8, 9, 1, 2, 3,
     +          0, 0, 0, 2, 4, 5, 8,10,11,14,15, 0, 8,10,-11, 2, 4,-5,
     +          0, 0, 0, 3,-5, 6, 9,-11,12,0, 0,16,-9,11,12,-3, 5,6,
     +          1, 2, 3, 0, 0, 0, 1, 2,-3, 7, 8,-9,13,14, 0, 7, 8, 9,
     +          2, 4,-5, 0, 0, 0, 2, 4, 5, 8,10,11,14,15, 0, 8,10,-11,
     +         -3, 5, 6, 0, 0, 0, 3,-5, 6, 9,-11,12,0, 0,16,-9,11,12,
     +          7, 8, 9, 1, 2, 3, 0, 0, 0, 1, 2,-3, 7, 8,-9,13,14, 0,
     +          8,10,-11, 2, 4,-5, 0, 0, 0, 2, 4, 5, 8,10,11,14,15, 0,
     +         -9,11,12,-3, 5, 6, 0, 0, 0, 3,-5, 6, 9,-11,12,0, 0,16,
     +          13,14, 0, 7, 8, 9, 1, 2, 3, 0, 0, 0, 1, 2,-3, 7, 8,-9,
     +          14,15, 0, 8,10,-11, 2, 4,-5, 0, 0, 0, 2, 4, 5, 8,10,11,
     +          0, 0,16,-9,11,12,-3, 5, 6, 0, 0, 0, 3,-5, 6, 9,-11,12,
     +          7, 8,-9, 13,14, 0, 7, 8, 9, 1, 2, 3, 0, 0, 0, 1, 2,-3,
     +          8,10,11,14,15, 0, 8,10,-11, 2, 4,-5, 0, 0, 0, 2, 4, 5,
     +          9,-11,12,  0, 0,16,-9,11,12,-3, 5, 6, 0, 0, 0, 3,-5, 6,
     +          1, 2,-3, 7, 8,-9, 13,14, 0, 7, 8, 9, 1, 2, 3, 0, 0, 0,
     +          2, 4, 5, 8,10,11,14,15, 0, 8,10,-11, 2, 4,-5, 0, 0, 0,
     +          3,-5, 6, 9,-11,12,  0, 0,16,-9,11,12,-3, 5, 6, 0, 0, 0/
C
      FUNC3(X,K)=BI3(K,1)+X*(BI3(K,2)+X*BI3(K,3))
      FUNC4(X,K)=BI4(K,1)+X*(BI4(K,2)+X*BI4(K,3))
      FUNC5(X,K)=BI5(K,2)+X*(BI5(K,2)+X*BI5(K,3))
C
C    INITIALISATION DES PROBABILITES DE COLLISION
C
      DO 5 I=1,16
      P(I)=0.
    5 CONTINUE
C----
C  COMPUTE CORDE =4*V*SIGMA/S=SQRT(3)*COTE*SIGMA  (CORDE MOYENNE)
C----
      S2S3=SQRT2*SQRT3
      S3DS2=SQRT3/SQRT2
      CAUX=SQRT3*COTE
      CORDE=CAUX*SIGT
      IF (CORDE.LE.TRONC) GO TO 300
C----
C  CONSIDER EXPLICIT INTEGRATION OF F PSS
C----
      PI12=PI/12.
C
C  ON CONSIDERE 2 CAS  1) SI ZGAUS(I)<0 => 1/COSFI>1/SINA>1/COSA
C                      2) SI ZGAUS(I)>0 => 1/COSFI>1/COSA>1/SINA
C
      NGPT2=IFIX(FLOAT(NGPT)/2.)
      DO 50 I=1,NGPT
      FI=PI12*(1.+ZGAUS(I))
      COSFI=COS(FI)
      SINFI=SIN(FI)
      AUX=SQRT3*COSFI
      COSA=AUX-SINFI
      COSB=AUX+SINFI
      AUX=SQRT3*SINFI
      SINA=AUX+COSFI
      SINB=COSFI-AUX
C
C    POIDS MULTIPLIONS LES FONCTIONS DE BICKLEY NAYLOR
C
      W006=SINFI*COSA
      W106=W006*COSB
      W005=COSFI*SINA
      W105=W005*COSB
      W204=COSFI*SINB
      W114=SINFI*SINFI*SINB
      W224=COSFI*W204
C
C   FONCTIONS DE BICKLEY NAYLOR
C
      TAU(1)=CORDE/COSFI
      IAUX1=2
      IAUX2=3
      IF(I.GT.NGPT2) THEN
      IAUX1=3
      IAUX2=2
      ENDIF
      TAU(IAUX1)=CORDE/SINA
      TAU(IAUX2)=CORDE/COSA
C
      LB=4
      IF(TAU(1).LT.XLIM3) THEN
      LB=1
      ELSEIF(TAU(2).LT.XLIM3) THEN
      LB=2
      ELSEIF(TAU(3).LT.XLIM3) THEN
      LB=3
      ENDIF
C
      DO 10 J=LB,3
      K1=NINT(PAS3*TAU(J))
      FKI3(J)=FUNC3(TAU(J),K1)
      FKI4(J)=FUNC4(TAU(J),K1)
      FKI5(J)=FUNC5(TAU(J),K1)
   10 FKI6(J)=.8*FKI4(J)+.2*TAU(J)*(FKI3(J)-FKI5(J))
C
      GO TO (20,25,30,50),LB
C
C   CALCUL DES PSS
C
   20 WEIGHT=WGAUS(I)
      P(13)=P(13)+WEIGHT*SINB*FKI3(1)
      P(14)=P(14)+WEIGHT*W204*FKI4(1)
      P(16)=P(16)+WEIGHT*W114*FKI5(1)
      P(15)=P(15)+WEIGHT*W224*FKI5(1)
      AUX5=WEIGHT*W005
      P(7)=P(7)+AUX5*(FKI4(IAUX1)-FKI4(1))
      P(9)=P(9)+WEIGHT*W105*(FKI5(IAUX1)-FKI5(1))
      AUX=AUX5*(FKI6(IAUX1)-FKI6(1))
      P(11)=P(11)+AUX
      P(10)=P(10)+W005*AUX
      GO TO 30
C
   25 WEIGHT=WGAUS(I)
      AUX5=WEIGHT*W005
      P(7)=P(7)+AUX5*FKI4(IAUX1)
      P(9)=P(9)+WEIGHT*W105*FKI5(IAUX1)
      AUX=AUX5*FKI6(IAUX1)
      P(11)=P(11)+AUX
      P(10)=P(10)+W005*AUX
   30 P(1)=P(1)+WEIGHT*W006*(BI4(0,1)-FKI4(IAUX2))
      P(3)=P(3)+WEIGHT*W106*(BI5(0,1)-FKI5(IAUX2))
      AUX=W006*WEIGHT*(.533333333333333-FKI6(IAUX2))
      P(5)=P(5)+AUX
      P(4)=P(4)+AUX*W006
   50 CONTINUE
C
C    NORMALISATION
C
      CORDI=1./CORDE
      X1=CORDI/3.
      X2=CORDI*SQRT3
      X3=3.*CORDI
      X4=X2/SQRT2
      P(1)=X1*P(1)
      P(3)=X2*P(3)/6.
      P(5)=-X4*P(5)
      P(4)=X3*P(4)
      P(7)=X1*P(7)
      P(9)=X1*P(9)*.5
      P(11)=-X4*P(11)
      P(10)=X3*P(10)
      P(13)=P(13)/3.
      P(14)=SQRT2*P(14)
      P(16)=4.*P(16)/3.
      P(15)=6.*P(15)
      AUX=2.*SQRT2
      P(2)=S3DS2*P(3)-AUX*P(1)
      P(8)=3.*S3DS2*P(9)-AUX*P(7)
      P(14)=P(14)-AUX*P(13)
      COEF=1./(6.*CORDE)
      PIS10=(1.-2.*(P(1)+P(7))-P(13))*COEF
      PIS(1)=PIS10
      PIS(2)=-(2.*(P(2)+P(8))+P(14))*COEF
      PII=(1.-6.*PIS10)/SIGT
C
      GO TO 350
C----
C  USE SERIES EXPANSION FOR CORDE->0: DEVELOPPEMENT EN SERIE DE MCLAURIN
C  DES FONCTIONS KI
C----
  300 TAU0=CORDE*.5
      TAU02=TAU0*TAU0
      AUX=SQRT3/PI
      AUX0=1.-.5*SQRT3
      AUX1=AUX*ALOG3-.33333333333333333
      AUX2=2./SQRT3-(2.+ALOGX)/3.
      P(1)=AUX0-.5*(TAU0*AUX1-TAU02*AUX2)
      P(5)=-S2S3*(1.125*AUX0-.5*TAU0*AUX1+.375*TAU02*AUX2)
      P(3)=(-(1.5*ALOGX+1.-SQRT3)*TAU0*.25+(1.-TAU02)/9+
     +             AUX*TAU02/3.)*SQRT3
      P(4)=2.25*(1.25*SQRT3-2.)-TAU0*(2.-3.*AUX)+TAU02*
     +           (2.25*ALOGX-1.5)
      XAUX=5.*SQRT3-9.
      XAUX0=SQRT3-1.5
      AUX1=AUX*(2.5*ALOG3-4.*ALOG2)-.5
      AUX2=XAUX-2.*ALOG3+.5*ALOGX
      P(7)=XAUX0+TAU0*AUX1-TAU02*AUX2/3.
      P(11)=-S2S3*(1.125*XAUX0+TAU0*AUX1-.25*AUX2*TAU02)
      P(9)=SQRT3/9.+.25*(XAUX-.5*SQRT3*(2.*ALOG3-ALOGX))*TAU0-
     +           (9.*ALOG3-16.*ALOG2-3.)*TAU02/(3.*PI)
      P(10)=2.25*(SQRT3-.75)+TAU0*(3.*AUX-6.)-1.5*TAU02*
     +           (9.*(2.-SQRT3)+1.5*ALOGX-6.*ALOG3)
      AUX0=2.*AUX0
      AUX1=AUX*(.5*ALOG3-ALOG2)
      P(13)=AUX0-8.*(1./6.+AUX1)*TAU0-4.*TAU02*(AUX0-.5*ALOG3)
      P(15)=4.5*(2.5-SQRT3)-8.*TAU0+36.*TAU02*AUX0
      AUX2=XAUX-2.*ALOG3+.5*ALOGX
      P(16)=3.5-2.*SQRT3-64.*TAU0*(AUX1+1./12.)/3.+8.*TAU02*
     +           (.5*ALOG3-2.*AUX0)
      PSQ3=SQRT3/PI
      P(14)=SQRT2*(2./3.-6.*TAU0*AUX0+TAU02*(4.+24.*PSQ3*
     + (.5*ALOG3-ALOG2)))
      PII=COTE*SQRT3*(4.*SQRT3-8.-2.*ALOGX+10.*ALOG3)/12.
      PIS(1)=(1.-SIGT*PII)/6.
      PIS(2)=-SQRT2*(2.5-2.25*ALOG3+TAU0*(9.+PSQ3*(4.-8.*ALOG2+
     +  3.*ALOG3)-8./SQRT3-(20.*ALOG3-4.*ALOGX)/3.))/12.
      AUX=2.*SQRT2
      P(2)=S3DS2*P(3)-AUX*P(1)
      P(8)=3.*S3DS2*P(9)-AUX*P(7)
      P(14)=P(14)-AUX*P(13)
C
  350 CONTINUE
C
C    CALCUL DES AUTRES PROBABILITES
C
      P(5)=P(5)+AUX*P(3)
      P(4)=P(4)-4.*SQRT3*P(3)+8.*P(1)
      P(6)=(S2S3*P(5)+8.*(-SQRT3*P(3)+P(1))-P(4))*2./9.
      P(11)=P(11)+AUX*P(9)
      P(10)=P(10)-4.*SQRT2*P(8)-8.*P(7)
      P(12)=(8.*(P(7)-SQRT3*P(9))-S2S3*P(11)-P(10))*2./9.
      P(15)=P(15)-4.*SQRT2*P(14)-8.*P(13)
      P(3)=-P(3)
      P(5)=-P(5)
      P(9)=-P(9)
      P(11)=-P(11)
C
C    TRANSMISSION MATRIX
C
      DO  58 I=1,18
      DO  58 J=1,18
      IB=IROT(J,I)
      IF(IB.LT.0) THEN
         PSS(I,J)=-P(-IB)
      ELSEIF(IB.GT.0) THEN
         PSS(I,J)=P(IB)
      ELSE
         PSS(I,J)=0.
      ENDIF
   58 CONTINUE
C
C      LEAKAGE PRABABILITIES MATRIX
C
      PIS(3)=0.
      K=3
      DO 56 I=1,5
       K=K+3
       PIS(K-2)=PIS(1)
       PIS(K-1)=PIS(2)
       PIS(K)=0.
   56 CONTINUE
C
      RETURN
      END
