*DECK XL3TRK
      SUBROUTINE XL3TRK( IPSYS, IPGEOM,  IPTRK, GEONAM, MREGIO,
     >                    IPRT,   NDIM,  ISPEC,
     >                     NVO,    NSO,     NV,     NS,
     >                   NANGL,  ISYMM, DENUSR, RCUTOF,  MXSEG,
     >                  ALBEDO,  ICODE,   NORE,
     >                    NGRT, MAXMIX, XSSIGT, XSSIGW,
     >                  TITREC, MAXPTS, MATCOD, VOLUME, KEYFLX,
     >                  LEAKSW,  IUSED )
************************************************************************
*                                                                      *
*           NAME: XL3TRK                                               *
*      COMPONENT: EXCELL                                               *
*          LEVEL: 3 (CALLED BY 'XL3DRV')                               *
*        VERSION: 1.0                                                  *
*       CREATION: 96/09 (R.R.)                                         *
*       MODIFIED: 00/03 (R.R.) DECLARE ALL VARIABLE TYPES              *
*         AUTHOR: ROBERT ROY                                           *
*                                                                      *
*     SUBROUTINE: THIS ROUTINE IS USED TO TREAT CARTESIAN ASSEMBLIES   *
*                 OF CELLS; THIS IS A TWO-STEP PROCESS:                *
*                 1) STUDY THE GEOMETRY TO GET VOLUMES AND MATERIALS   *
*                 2) PRODUCE TEMPORARY TRACKING FILE IF NECESSARY      *
*                                                                      *
*--------+-------------- V A R I A B L E S -------------+--+-----------*
*  NAME  /                  DESCRIPTION                 /IO/MOD(DIMENS)*
*--------+----------------------------------------------+--+-----------*
* IPSYS  / POINTER TO THE SYSTEM (L_PIJ)                /IO/INT        *
* IPGEOM / POINTER TO THE GEOMETRY (L_GEOM)             /I./INT        *
* IPTRK  / POINTER TO THE EXCELL TRACKING (L_EXCELL)    /IO/INT        *
* GEONAM / GEOMETRY NAME                                /I./CAR*12     *
* MREGIO / # OF ZONES ACCORDING TO USER                 /I./INT        *
* IPRT   / GEOMETRY PRINT LEVEL                         /I./INT        *
* NDIM   / # OF DIMENSIONS (2.OR.3)                     /I./INT        *
* ISPEC  / KIND OF TRACKING (0:ISOTROPIC;1:SPECULAR)    /IO/INT        *
* NRMV   / VOLUME FACTORS (0:NO;1:YES)                  /I./INT        *
* NV     / # OF ZONES IN THE ASSEMBLY.                  /.O/INT        *
* NS     / # OF SURFACES IN THE ASSEMBLY.               /.O/INT        *
* NANGL  / # OF ANGLES USED IN TEMPORARY TRACKING FILE. /IO/INT        *
* ISYMM  / SYMMETRY FACTOR.                             /IO/INT        *
* DENUSR / DENSITY OF TRACKS IN THE PLANE PERPENDICULAR /IO/REL        *
*        /      TO THE TRACKING ANGLES.                 /  /           *
* RCUTOF / CUTOF FOR CORNER TRACKING( 0.25 SUGGESTED )  /I./REL        *
* MXSEG  / MAXIMUM # OF SEGMENTS IN A SINGLE TRACK.     /.O/INT        *
* ICODE  / INDEX FOR BOUNDARY CONDITIONS.               /.O/INT(6)     *
* ALBEDO / GEOMETRIC ALBEDOS.                           /.O/REL(6)     *
* NORE   / TRACK NORMALIZATION (0:YES 1:NO)             /I./INT        *
* NGRT   / NUMBER OF GROUPS                             /I./INT        *
* MAXMIX / # OF MIXTURES IN THE MACROLIB                /I./INT        *
* XSSIGT / TOTAL XS FOR MIXTURES IN THE MACROLIB        /I./REL(MAXMIX *
*        /                                              /I./  +1,NGRT) *
* XSSIGW / SCATT XS FOR MIXTURES IN THE MACROLIB        /I./REL(MAXMIX *
*        /                                              /I./  +1,NGRT) *
* TITREC / TITLE FOR THIS CASE                          /I./C*72       *
* MAXPTS / # OF ZONES ACCORDING TO USER                 /I./INT        *
* MATCOD / MATERIAL #S FOR ZONES IN THE SUPERCELL       /.O/INT(MAXPTS)*
* VOLUME / VOLUMES OF ZONES IN THE SUPERCELL            /.O/REL(MAXPTS)*
* KEYFLX / ZONE KEY FOR THE UNKNOWN VECTORS (FLUXES...) /.O/REL(MAXPTS)*
* LEAKSW / LEAKAGE SWITCH                               /.O/LOG        *
* IUSED  / INDEX FOR BOUNDARY CONDITIONS USED (0/1)     /.O/INT(6)     *
************************************************************************
C
      IMPLICIT        NONE
      INTEGER         IOUT,NALB,NSTATE,ICPALL,ICPEND
      PARAMETER      (IOUT=6,NALB=6,NSTATE=40,ICPALL=4,ICPEND=3)
C----
C  ROUTINE PARAMETERS
C----
      INTEGER         IPSYS ,IPGEOM,IPTRK ,MREGIO,IPRT  ,NDIM  ,
     >                ISPEC ,NVO   ,NSO   ,NV    ,NS    ,NANGL ,
     >                ISYMM ,MXSEG ,NORE  ,NGRT  ,MAXMIX,MAXPTS
      CHARACTER       GEONAM*12,TITREC*72
      REAL            DENUSR,RCUTOF
      INTEGER         ICODE(NALB),MATCOD(MAXPTS),KEYFLX(MAXPTS),
     >                IUSED(NALB)
      REAL            ALBEDO(NALB),XSSIGT(0:MAXMIX,NGRT),
     >                XSSIGW(0:MAXMIX,NGRT),VOLUME(MAXPTS)
      LOGICAL         LEAKSW
      INTEGER         MXANGL
      INTEGER         IFUITE
C----
C  PARAMETERS FOR MEMORY ALLOCATION
C----
      REAL            RD
      INTEGER         ID(1)
      COMMON          RD(1)
      EQUIVALENCE    (RD(1),ID(1)) 
C----
C  INTEGER ARRAYS
C----
      INTEGER         LGRP  ,MINDIM,MAXDIM,ICORD ,INDEL ,KEYMRG,
     >                MATALB,INAM  ,MATMRG,ICUR  ,INCR  ,NUMERO,
     >                NRSEG ,IDL   ,MATRT 
C----
C  REAL AND DOUBLE PRECISION ARRAYS
C----
      INTEGER         REMESH,VOLSUR,VOLMRG,SIGTAL,SIGINV,SIGSS ,
     >                CONV  ,TRKBEG,TRKDIR,LENGTH,ANGLES,DENSTY,
     >                SEGLEN,STAYIN,GOSOUT,PRODUC,DSCBEG,DSCEND,
     >                IWEIG ,ICHI  ,IWFS  ,IWPAD ,IRI   ,IRBAR ,
     >                IGBAR ,IBILAN,ICIJ  ,IWSP  , IWORK ,JPSST ,
     >                JPSVT ,IDD   ,IPIS  , IARRAY,VOLTRK,DBLPIJ
      INTEGER         JPSVF
C----
C  LOCAL PARAMETERS
C----
      INTEGER         ISTATE(NSTATE)
      INTEGER         NCODE(NALB),MAXGRI(3),LCLSYM(3)
      INTEGER         NUNKMR,NREG  ,NPIJ  ,NALBG ,NC    ,NTR   ,
     >                NTZ   ,ICL   ,NSOUT ,ITGEO ,NGRP  ,IENDGR,
     >                NSPOS ,IR    ,ICMAX ,IBEGGR,LENGHT,IPRTT ,
     >                NRMV  ,IRMV  ,NTY   ,INDATA,NSUBG ,IG    ,
     >                NRENOR,JG    ,NANGLE,LTRK  ,LINMAX,NTOTG ,
     >                INDLEC,NUNK  ,MAXR  ,NEXTGE,NTX   ,NCOR  ,
     >                ISCATR,IPHASE,NSUR  ,NTOTCL,NVOL  
      LOGICAL         SWZERO,SWVOID,SWNZBC,LHALT ,LALBS ,
     >                LNORM ,LTAKE
      REAL            ALBOLD(NALB)
      REAL            CUTOFX, REDATA
      DOUBLE PRECISION DBLINP
      CHARACTER       TEDATA*8 
      CHARACTER*8     CTISO, CTSPC, CCORN, CHALT, CSYMM, CMEDI, CPNOR,
     >                CSKIP, CSUBG, CTAKE, CLIST, CALBS, CNORM, CBLAN 
      SAVE            CTISO, CTSPC, CCORN, CHALT, CSYMM, CMEDI, CPNOR,
     >                CSKIP, CSUBG, CTAKE, CLIST, CALBS, CNORM, CBLAN
      DATA            CTISO, CTSPC, CCORN, CHALT, CSYMM, CMEDI, CPNOR,
     >                CSKIP, CSUBG, CTAKE, CLIST, CALBS, CNORM, CBLAN
     >              / 'TISO','TSPC','CORN','HALT','SYMM','MEDI','PNOR',
     >                'SKIP','SUBG','TAKE','LIST','ALBS','NORM','    '/
C
      LTAKE= .FALSE.
      LHALT= .FALSE.
      LNORM= .FALSE.
      LALBS= .FALSE.
      SWZERO=.TRUE.
      LTRK  = 0
      NANGLE= 0
      DENUSR= 0.0
      RCUTOF= 0.0
      LINMAX= 0
      CUTOFX= 0.0
      TEDATA= CBLAN
      ISYMM=1
C
C. 1) READ ALL USER INPUT.
C
C     READ TRACKING PARAMETERS  LTRK= 0 : NO  TRACKING
C                               LTRK= 1 : ISOTROPIC TRACKING   (TISO)
C                               LTRK= 2 : SPECULAR TRACKING    (TSPC)
         CALL REDGET( INDLEC, NANGLE, DENUSR, TEDATA,DBLINP)
         NTOTG= 1
         NSUBG= NGRT
         IF( TEDATA(1:4).EQ.CSUBG(1:4) )THEN
            NSUBG= 0
            CALL REDGET( INDLEC, NSUBG, DENUSR, TEDATA,DBLINP)
            IF( NSUBG.LE.0.OR.NSUBG.GT.NGRT )THEN
               CALL XABORT('XL3TRK: INVALID *SUBG* PARAMETER')
            ENDIF
            NTOTG= NGRT/NSUBG
            IF( MOD(NGRT,NSUBG).NE.0 ) NTOTG= NTOTG+1
            CALL REDGET( INDLEC, NANGLE, DENUSR, TEDATA,DBLINP)
         ELSEIF( TEDATA(1:4).EQ.CTAKE(1:4) )THEN
            NSUBG= 0
            CALL REDGET( INDLEC, NSUBG, DENUSR, TEDATA,DBLINP)
            IF( NSUBG.LE.0.OR.NSUBG.GT.NGRT )THEN
               CALL XABORT('XL3TRK: INVALID *TAKE* PARAMETER')
            ENDIF
            LTAKE=.TRUE.
         ENDIF
         CALL SETARA(ID, NSUBG, LGRP)
         IF( LTAKE )THEN
            CALL REDGET( INDLEC, INDATA, DENUSR, TEDATA,DBLINP)
            IF( INDLEC.NE.3.OR.TEDATA(1:4).NE.CLIST(1:4) )THEN
               CALL XABORT('XL3TRK: *LIST* KEYWORD EXPECTED')
            ENDIF
            DO 110 IG= 1, NSUBG
               CALL REDGET( INDLEC, INDATA, DENUSR, TEDATA,DBLINP)
               IF( INDLEC.NE.1 )THEN
                  CALL XABORT('XL3TRK: INTEGER GROUP NUMBER EXPECTED')
               ELSEIF( INDATA.LE.0.OR.INDATA.GT.NGRT )THEN
                  CALL XABORT('XL3TRK: BAD GROUP NUMBER')
               ENDIF
               DO 111 JG= 1, IG-1
                 IF( INDATA.EQ.ID(LGRP+JG-1) )THEN
                   CALL XABORT('XL3TRK:  2 TIMES SAME GROUP')
                 ENDIF
 111           CONTINUE
               ID(LGRP+IG-1)= INDATA
 110        CONTINUE
            CALL REDGET( INDLEC, NANGLE, DENUSR, TEDATA,DBLINP)
         ENDIF
         NRENOR=0
         IF( TEDATA(1:4).EQ.CPNOR(1:4) )THEN
            CALL REDGET( INDLEC, NANGLE, DENUSR, TEDATA,DBLINP)
            IF( INDLEC.NE.3 )THEN
            CALL XABORT('XL3TRK: VALID PNOR OPTION REQUIRED' )
            ENDIF
            IF(     TEDATA(1:4).EQ.'NONE' )THEN
               NRENOR=-1
            ELSEIF( TEDATA(1:4).EQ.'HELI' )THEN
               NRENOR=0
            ELSEIF( TEDATA(1:4).EQ.'GELB' )THEN
               NRENOR=1
            ELSEIF( TEDATA(1:4).EQ.'DIAG' )THEN
               NRENOR=2
            ELSEIF( TEDATA(1:4).EQ.'NONL' )THEN
               NRENOR=3
            ELSE
            CALL XABORT('XL3TRK: INVALID PNOR OPTION PROVIDED' )
            ENDIF
            CALL REDGET( INDLEC, NANGLE, DENUSR, TEDATA,DBLINP)
         ENDIF
         IF( TEDATA(1:4).EQ.CNORM(1:4) )THEN
            LNORM= .TRUE.
            CALL REDGET( INDLEC, NANGLE, DENUSR, TEDATA,DBLINP)
         ELSEIF( TEDATA(1:4).EQ.CALBS(1:4) )THEN
            LALBS= .TRUE.
            CALL REDGET( INDLEC, NANGLE, DENUSR, TEDATA,DBLINP)
         ENDIF
         IPHASE=0
         ISCATR=0
         IF( TEDATA(1:4).EQ.CSKIP(1:4) )THEN
            ISCATR=1
            CALL REDGET( INDLEC, NANGLE, DENUSR, TEDATA,DBLINP)
         ENDIF
         IF( INDLEC.NE.3 )THEN
            LTRK  = 1
            CALL REDGET( INDLEC, NANGLE, DENUSR, TEDATA,DBLINP)
            IF( INDLEC.EQ.3 )THEN
            CALL XABORT('XL3TRK: *TISO* ASSUMED, PUT NANGLE & DENSTY' )
            ENDIF
            CALL REDGET( INDLEC, INDATA, REDATA, TEDATA,DBLINP)
         ELSE
            IF( TEDATA(1:4).EQ.CTISO(1:4) )THEN
               LTRK  = 1
            ELSEIF( TEDATA(1:4).EQ.CTSPC(1:4) )THEN
               LTRK  = 2
            ENDIF
            IF( LTRK.GT.0 )THEN
               CALL REDGET( INDLEC, NANGLE, DENUSR, TEDATA,DBLINP)
               IF( LTRK.EQ.2.AND.TEDATA(1:4).EQ.CMEDI(1:4) )THEN
                  SWZERO= .FALSE.
                  CALL REDGET( INDLEC, NANGLE, DENUSR, TEDATA,DBLINP)
               ENDIF
               CALL REDGET( INDLEC, NANGLE, DENUSR, TEDATA,DBLINP)
               CALL REDGET( INDLEC, INDATA, REDATA, TEDATA,DBLINP)
            ELSE
               CALL XABORT( 'XL3TRK: *TISO* OR *TSPC* HERE')
            ENDIF
         ENDIF
         IF( TEDATA(1:4).EQ.CHALT(1:4) )THEN
            LHALT= .TRUE.
            CALL REDGET( INDLEC, INDATA, REDATA, TEDATA,DBLINP)
         ENDIF
         IF( INDLEC.NE.3 )THEN
            CALL XABORT( 'XL3TRK: ; OR *CORN* IS SUPPOSED TO BE HERE' )
         ELSEIF( TEDATA(1:4).EQ.CCORN(1:4) )THEN
            CALL REDGET( INDLEC, INDATA, RCUTOF, TEDATA,DBLINP)
            IF(INDLEC.NE.2) CALL XABORT('XL3TRK: REAL DATA EXPECTED')
            CALL REDGET( INDLEC, INDATA, REDATA, TEDATA,DBLINP)
         ENDIF
         IF( INDLEC.NE.3 )THEN
            CALL XABORT( 'XL3TRK: ; OR *SYMM* IS SUPPOSED TO BE HERE' )
         ELSEIF( TEDATA(1:4) .EQ. 'NOSY'   )THEN
            ISYMM=0
            CALL REDGET( INDLEC, INDATA, REDATA, TEDATA,DBLINP)
         ELSEIF( TEDATA(1:4).EQ.CSYMM(1:4) )THEN
            CALL REDGET( INDLEC, INDATA, REDATA, TEDATA,DBLINP)
            IF(INDLEC.NE.1) CALL XABORT('XL3TRK: INTEGER DATA EXPECTED')
            ISYMM=INDATA
            CALL REDGET( INDLEC, INDATA, REDATA, TEDATA,DBLINP)
         ENDIF
         IF( INDLEC.NE.3.OR.TEDATA(1:4).NE.';' )THEN
            CALL XABORT( 'XL3TRK: ; IS SUPPOSED TO BE HERE' )
         ENDIF
         IF( LTRK.GT.0.AND.NANGLE.LE.0 )THEN
         CALL XABORT( 'XL3TRK: INVALID NUMBER OF ANGLES (NANGLE < 0)' )
         ENDIF
         IF( LTRK.GT.0.AND.DENUSR.LE.0.0 )THEN
         CALL XABORT( 'XL3TRK: INVALID DENSITY (DENSTY < 0.0) ' )
         ENDIF
   10 CONTINUE
C----
C  PROCESS THE GEOMETRY
C----
      CALL AXGXEL(IPGEOM,IPTRK ,IPRT  ,GEONAM)
C----
C  READ THE GEOMETRY INFORMATION STORED ON IPTRK
C----      
      CALL LCMSIX(IPTRK,'EXCELL      ',1)
      CALL XDISET(ISTATE,NSTATE,0)
      CALL LCMGET(IPTRK,'STATE-VECTOR',ISTATE)
      NDIM     =ISTATE(1)
      NSUR    =-ISTATE(2)
      NVOL     =ISTATE(3)
      NTOTCL   =ISTATE(4)
      MAXR     =ISTATE(5)
      NUNK     =ISTATE(6)
      NEXTGE   =ISTATE(7)
C----
C  Intrinsic symmetries used in geometry
C  Use these to simplify tracking unless 
C  NOSYMM tracking option activated
C----
      LCLSYM(1) =ISTATE(8)
      LCLSYM(2) =ISTATE(9)
      LCLSYM(3) =ISTATE(10)
      IF(ISYMM .NE. 0) THEN
        ISYMM=0
        IF(NDIM .EQ. 2) THEN
          IF(LCLSYM(1) .NE. 0) THEN
C----
C  X SYMMETRY
C----
            ISYMM=2
          ENDIF
          IF(LCLSYM(2) .NE. 0) THEN
            IF(ISYMM .EQ. 0) THEN
C----
C  Y SYMMETRY
C----
              ISYMM=4
            ELSE
C----
C  X AND Y SYMMETRY
C----
              ISYMM=8 
            ENDIF
          ENDIF
CC          IF(ISTATE(11) .NE. 0) THEN
CCC----
CCC  X-Y DIAGONAL SYMMETRY
CCC---- 
CC            IF(ISYMM .EQ. 0) THEN
CC              ISYMM=10
CC            ELSE
CC              ISYMM=12
CC            ENDIF
CC          ENDIF 
        ELSE
          IF(LCLSYM(1) .NE. 0) THEN
C----
C  X SYMMETRY
C----
            ISYMM=2
          ENDIF
          IF(LCLSYM(2) .NE. 0) THEN
            IF(ISYMM .EQ. 0) THEN
C----
C  Y SYMMETRY
C----
              ISYMM=4
            ELSE
C----
C  X AND Y SYMMETRY
C----
              ISYMM=8 
            ENDIF
          ENDIF 
          IF(LCLSYM(3) .NE. 0) THEN
C----
C  Z SYMMETRY
C----
            ISYMM=ISYMM+16
          ENDIF
        ENDIF
        IF(ISYMM .EQ. 0) ISYMM=1
      ENDIF
      CALL SETARA(ID,NTOTCL,MINDIM)
      CALL SETARA(ID,NTOTCL,MAXDIM)
      CALL SETARA(ID,NTOTCL,ICORD )
      CALL SETARA(ID,4*NUNK,INDEL )
      CALL SETARA(RD,MAXR  ,REMESH)
      CALL SETARA(ID,NUNK  ,KEYMRG)
      CALL SETARA(ID,NUNK  ,MATALB)
      CALL SETARA(RD,NUNK  ,VOLSUR)
      CALL LCMGET(IPTRK,'MINDIM      ',ID(MINDIM))
      CALL LCMGET(IPTRK,'MAXDIM      ',ID(MAXDIM))
      CALL LCMGET(IPTRK,'ICORD       ',ID(ICORD) )
      CALL LCMGET(IPTRK,'INDEX       ',ID(INDEL) )
      CALL LCMGET(IPTRK,'REMESH      ',RD(REMESH))
      CALL LCMGET(IPTRK,'KEYMRG      ',ID(KEYMRG))
      CALL LCMGET(IPTRK,'MATALB      ',ID(MATALB))
      CALL LCMGET(IPTRK,'VOLSUR      ',RD(VOLSUR))
      CALL LCMSIX(IPTRK,'EXCELL      ',2)
      CALL LCMGET(IPTRK,'ICODE       ',ICODE )
      CALL LCMGET(IPTRK,'NCODE       ',NCODE )
      CALL LCMGET(IPTRK,'ALBEDO      ',ALBOLD)
C----
C  VERIFY SYMMETRY AND
C  STUDY TRACKING PARAMETERS. ARE THEY BASICALLY POSSIBLE ?
C----
      IF(LTRK .EQ. 1)THEN
        NCOR= 1
        IF(NDIM .EQ. 2) THEN
          MXANGL=NANGLE
CC          IF(ISYMM .EQ. 12) THEN
CC            NANGL = NANGLE/4
          IF(ISYMM .GE. 2) THEN
            NANGL = (NANGLE+1)/2
          ELSE 
            NANGL = NANGLE
          ENDIF
          IF( RCUTOF.GT.0.0 ) NCOR= 2
        ELSE IF(NDIM .EQ. 3) THEN
          IF(MOD(NANGLE,2) .EQ. 1)THEN
            NANGLE=NANGLE+1
            WRITE(IOUT,'(/31H MESS = ONLY EVEN # EQN ANGLES )')
          ENDIF
          IF(NANGLE .GT. 16)THEN
            NANGLE=16
            WRITE(IOUT,'(/31H MESS = 16 IS MAX # EQN ANGLES )')
          ENDIF
          MXANGL=(NANGLE * (NANGLE+2)) / 2 
          IF(NEXTGE .EQ. 1) THEN
            NANGL = (NANGLE * (NANGLE+2)) / 8
          ELSE
            IF(ISYMM .EQ. 8 .OR. ISYMM .EQ. 24) THEN
              NANGL = (NANGLE * (NANGLE+2)) / 8 
            ELSE IF(ISYMM .EQ. 2  .OR. ISYMM .EQ. 4 .OR. 
     >              ISYMM .EQ. 18 .OR. ISYMM .EQ. 20 ) THEN
              NANGL = (NANGLE * (NANGLE+2)) / 4 
            ELSE
              NANGL = (NANGLE * (NANGLE+2)) / 2 
            ENDIF
          ENDIF
          IF(RCUTOF .GT. 0.0) NCOR= 4
        ENDIF
      ELSEIF( LTRK.EQ.2 )THEN
        NCOR  = 1
        MXANGL=NANGLE
        IF( NDIM.EQ.2 )THEN
           NANGL = NANGLE
        ELSEIF( NDIM.EQ.3 )THEN
           CALL XABORT('XL3TRK: *TSPC* NOT AVAILABLE FOR 3-D GEOMETRY')
        ENDIF
        CUTOFX= RCUTOF
      ENDIF
      IF( IPRT.GE.1 )THEN
        WRITE(IOUT,'(1H )')
        WRITE(IOUT,'(26H >>> GEOMETRY TREATED:    ,A12)') GEONAM
        WRITE(IOUT,'(26H >>> EXCELL TREATMENT <<< )')
        WRITE(IOUT,'(1H )')
      ENDIF
      IF( IPRT.GT.1 .AND. NEXTGE.EQ.0 )THEN
C
C        IF PRINT REQUIRED AND OVERALL CARTESIAN GEOMETRY
C        PRINT CARTESIAN REGION MAP
         NTX= ID(MAXDIM  )-ID(MINDIM  )
         NTY= ID(MAXDIM+1)-ID(MINDIM+1)
         NTZ= ID(MAXDIM+2)-ID(MINDIM+2)
         NTR=0
         DO 103 ICL=3,NTOTCL-1
            NTR= MAX(NTR,ID(MAXDIM+ICL)-ID(MINDIM+ICL)+1)
  103    CONTINUE
         CALL SETARA(ID,4*(NTR+1)*(NTX+2)*(NTY+2)*(NTZ+2),INAM)
         CALL XELGPR(NDIM,NTX,NTY,NTZ,NTR,ISYMM,
     >               NSUR,NVOL,NTOTCL,ID(MINDIM),ID(MAXDIM),
     >               ID(KEYMRG),ID(INDEL),ID(MATALB),ID(INAM))
         CALL RLSARA(ID(INAM))
      ENDIF
      CALL SETARA( RD, 2*(NANGL+1)*NUNK, VOLTRK)
C
      NV=  NVOL
      NS= -NSUR
      CALL SETARA( RD,     NUNK, VOLMRG)
      CALL SETARA( ID,     NUNK, MATMRG)
      ITGEO=3
      CALL XELCMP(   NS,    NV, RD(VOLSUR), ID(MATALB), ID(KEYMRG),
     >            NSOUT,  NREG, RD(VOLMRG), ID(MATMRG), ITGEO,ICODE)
      NUNKMR= NREG+NSOUT+1
      NPIJ= MAX(NUNKMR*NUNKMR,(NREG*(3*NREG+1))/2)
      IF( IPRT .GT. 1 )THEN
        WRITE(IOUT,6000) (NSUBG*NPIJ/128) 
      ENDIF
      CALL SETARA( RD, 2*NSUBG*NPIJ, DBLPIJ)
      IF( IPRT .GT. 1 )THEN
        WRITE(IOUT,6001) 
      ENDIF
      NSOUT= -NSOUT
C
C     ALLOCATE AND CHARGE TOTAL XS PER REGION
      CALL SETARA( RD, NSUBG*NUNKMR, SIGTAL)
      CALL SETARA( RD, NSUBG*NREG, SIGINV)
      CALL SETARA( RD, NSUBG*NREG, SIGSS)
C
C. 3) DO THE TRACKING OF THE EXACT GEOMETRY FOR *NEWT* OPTION.
      IF( LTRK.NE.0 )THEN
         NC= NTOTCL - 3
         IF( IPRT.GE.1 )THEN
            WRITE(IOUT,'(1H )')
            IF    ( NC.EQ.0 )THEN
               WRITE(IOUT,'(/21H NOW, TRACKING   >>> ,A12,
     >                   13H GEOMETRY <<<,
     >                   13H    (WITH  NO,11H CYLINDER ) /)')
     >                                            GEONAM
            ELSEIF( NC.EQ.1 )THEN
               WRITE(IOUT,'(/21H NOW, TRACKING   >>> ,A12,
     >                   13H GEOMETRY <<<,
     >                   13H    (WITH ONE,11H CYLINDER ) /)')
     >                                            GEONAM
            ELSE
               WRITE(IOUT,'(/21H NOW, TRACKING   >>> ,A12,
     >                   13H GEOMETRY <<<,
     >                   10H    (WITH ,I3,11H CYLINDERS) /)')
     >                                            GEONAM, NC
            ENDIF
         ENDIF
         CALL SETARA( ID,  NTOTCL,   ICUR )
         CALL SETARA( ID,  NTOTCL,   INCR )
         CALL SETARA( RD,  NTOTCL,   CONV )
         CALL SETARA( RD,  NTOTCL, TRKBEG )
         CALL SETARA( RD,  NTOTCL, TRKDIR )
C
C  3.0)  WRITE FIRST RECORDS OF THE UNNORMALIZED TRACKING FILE
         IF( LTRK.EQ.1 )THEN
            LINMAX= 2*NVOL + 10
         ELSE
            LINMAX= 8*NANGL*(2*NVOL + 8)
         ENDIF
         ISPEC = LTRK-1
         NALBG = 6
         CALL SETARA( ID,  LINMAX, NUMERO )
         CALL SETARA( RD,  LINMAX, LENGHT )
         CALL SETARA( RD,3*MXANGL, ANGLES )
         CALL SETARA( RD,  MXANGL, DENSTY )
         MXSEG= LINMAX
         CALL SETARA( RD,  MXSEG, SEGLEN )
         CALL SETARA( ID,  MXSEG, NRSEG  )
         CALL SETARA( RD,  NSUBG*MXSEG, STAYIN )
         CALL SETARA( RD,  NSUBG*MXSEG, GOSOUT )
         CALL SETARA( RD,  NSUBG, PRODUC )
         CALL SETARA( RD,  NSUBG, DSCBEG )
         CALL SETARA( RD,  NSUBG, DSCEND )
CRR
         IBEGGR= 1
         IPRTT= IPRT
         DO 303 IRMV= 1, 1-NTOTG, -1
               NRMV= IRMV
               IF( NRMV.NE.1 )THEN
                  IF( .NOT.LTAKE )THEN
                     IENDGR= MIN(NGRT,IBEGGR+NSUBG-1)
                     NGRP= IENDGR-IBEGGR+1
                     JG= IBEGGR-1
                     DO 112 IG= 1, NGRP
                        JG= JG+1
                        ID(LGRP+IG-1)= JG
  112                CONTINUE
                  ELSE
                     NGRP= NSUBG
                  ENDIF
                  CALL XL3SIG( NGRT, MAXMIX, XSSIGT, XSSIGW, ALBOLD,
     >                        ID(LGRP), NGRP,
     >                        NSOUT, NREG, ID(MATMRG), RD(SIGTAL),
     >                        RD(SIGINV), RD(SIGSS), SWVOID, SWNZBC)
               ENDIF
               CALL XL3TI3(  IPRTT, NANGLE, DENUSR,ISYMM,
     >                   RD(ANGLES), RD(DENSTY), NTOTCL,
     >                   NEXTGE, MAXR, RD(REMESH), LINMAX, RCUTOF,
     >                     NSUR,   NVOL, ID(INDEL),
     >                   ID(MINDIM), ID(MAXDIM), ID(ICORD),
     >                   ID(INCR), ID(ICUR),
     >                   RD(TRKBEG), RD(CONV), RD(TRKDIR),
     >                   RD(LENGHT), ID(NUMERO),
     >                   NGRP, RD(SIGTAL), SWVOID,
     >                   ID(NRSEG), RD(SEGLEN),
     >                   RD(STAYIN),RD(GOSOUT),
     >                   RD(PRODUC),RD(DSCBEG),RD(DSCEND),
     >                   NORE, NRMV, RD(VOLTRK),
     >                   ID(KEYMRG),NSOUT,NREG,RD(DBLPIJ) )
               IF( NRMV.EQ.1 )THEN
                  CALL XL3NTR( IPRT, NDIM, ISPEC, NS, NV, NORE,
     >                        RD(VOLSUR), ID(KEYMRG), ID(MATALB),
     >                        NANGL, RD(VOLTRK), RD(DENSTY) )
                  IPRTT= 0
                  IF( LHALT )THEN
                     CALL XABORT('XL3TRK: *HALT* AT USER DEMAND')
                  ENDIF
               ELSE
                  CALL QIJCMP(NREG,NSOUT,NGRP,NCOR,
     >                        RD(VOLMRG),RD(SIGTAL),RD(DBLPIJ))
C
C     RENORMALIZE ALL ISOTROPIC PROBS WITH VARIOUS OPTIONS.
C
      IF( IPRT.GE.1 )THEN
         IF(     NRENOR .EQ. -1 )THEN
            WRITE(IOUT,*) ' ECHO = >>> CP NOT NORMALIZED'
         ELSEIF( NRENOR .EQ.  0 )THEN
           WRITE(IOUT,*) ' ECHO = >>> CP NORMALIZATION METHOD: HELIOS.'
         ELSEIF( NRENOR .EQ.  1 )THEN
           WRITE(IOUT,*) ' ECHO = >>> CP NORMALIZATION METHOD: GELB.'
         ELSEIF( NRENOR .EQ.  2 )THEN
           WRITE(IOUT,*) ' ECHO = >>> CP NORMALIZATION METHOD: DIAG.'
         ELSEIF( NRENOR .EQ.  3 )THEN
           WRITE(IOUT,*) ' ECHO = >>> CP NORMALIZATION METHOD: NONL.'
         ENDIF
      ENDIF
      IF     ( NRENOR .EQ. 0 )THEN
C
C        NORMALIZATION WORKING ON WEIGHT FACTORS ADDITIVE (HELIOS)
         CALL SETARA( RD, 2*NGRP*NUNKMR*3, IWEIG)
         CALL SETARA( RD, 2*NGRP*NUNKMR, ICHI)
         CALL SETARA( RD, 2*NGRP, IWFS)
         CALL SETARA( RD, 2*NGRP, IWPAD)
         CALL QIJRHL( NREG, NSOUT, ID(LGRP), NGRP, RD(SIGTAL),
     >                RD(DBLPIJ),
     >                RD(ICHI), RD(IWFS), RD(IWEIG), RD(IWPAD) )
         CALL RLSARA( RD(IWPAD) )
         CALL RLSARA( RD(IWEIG) )
         CALL RLSARA( RD(IWFS) )
         CALL RLSARA( RD(ICHI) )
      ELSE IF( NRENOR.EQ.1 )THEN
C
C        NORMALIZATION USING GELBARD SCHEME
         CALL SETARA( RD, 2*NGRP*NUNKMR, IRI)
         CALL SETARA( RD, 2*NGRP, IRBAR)
         CALL SETARA( RD, 2*NGRP, IGBAR)
         CALL QIJRGL( NREG, NSOUT, NGRP, RD(SIGTAL), RD(DBLPIJ),
     >                RD(IRI), RD(IRBAR), RD(IGBAR) )
         CALL RLSARA( RD(IGBAR ))
         CALL RLSARA( RD(IRBAR ))
         CALL RLSARA( RD(IRI   ))
      ELSE IF( NRENOR .EQ. 2 )THEN
C
C        NORMALIZATION WORKING ON DIAGONAL COEFFICIENTS
         CALL SETARA( RD, 2*NGRP, IBILAN)
         CALL QIJRDG( NREG, NSOUT, NGRP, RD(SIGTAL), RD(DBLPIJ),
     >                RD(IBILAN) )
         CALL RLSARA( RD(IBILAN ))
      ELSE IF( NRENOR .EQ. 3 )THEN
C
C        NORMALIZATION WORKING ON WEIGHT FACTORS TO KEEP DIAG = 0.0
         CALL SETARA( RD, 2*NGRP*NPIJ, ICIJ)
         CALL SETARA( RD, 2*NGRP*NPIJ, IWSP)
         CALL SETARA( RD, 2*NGRP*NUNKMR, IWFS)
         CALL SETARA( RD, 2*NGRP*NUNKMR, IWEIG)
         CALL SETARA( RD, 2*NGRP, IWPAD)
         CALL SETARA( RD, 4*NGRP, IWORK)
         CALL SETARA( ID,   NUNKMR, IDL)
         CALL QIJRNL( NREG, NSOUT, ID(LGRP), NGRP, RD(SIGTAL),
     >                RD(DBLPIJ),
     >                RD(ICIJ), RD(IWSP), RD(IWFS), ID(IDL), RD(IWEIG),
     >                RD(IWPAD), RD(IWORK) )
         CALL RLSARA( ID(IDL) )
         CALL RLSARA( RD(IWORK) )
         CALL RLSARA( RD(IWPAD) )
         CALL RLSARA( RD(IWEIG) )
         CALL RLSARA( RD(IWFS) )
         CALL RLSARA( RD(IWSP) )
         CALL RLSARA( RD(ICIJ) )
      ENDIF
      IF( IPRT.GE.ICPALL )THEN
         CALL QIJWPR( 0, NREG, NSOUT, ID(LGRP), NGRP,
     >                RD(SIGTAL), RD(DBLPIJ) )
      ENDIF
      NSPOS=-NSOUT
      IFUITE=0
      CALL SETARA( RD, 2*NSPOS*NREG*NGRP , JPSVT )
      CALL SETARA( RD, 2*NSPOS*NREG*NGRP , JPSVF )
      IF( SWNZBC )THEN
         CALL SETARA( RD, 2*NSPOS*NSPOS , JPSST )
         CALL SETARA( ID,         NSPOS ,   IDL )
         CALL SETARA( ID,         NSPOS , MATRT )
         CALL LCMGET(IPTRK,'BC-REFL+TRAN',ID(MATRT))
         CALL QIJABC( NREG, NSPOS, NPIJ, NGRP, RD(SIGTAL),ID(MATRT),
     >                RD(DBLPIJ),ID(IDL), RD(JPSST), RD(JPSVT)  )
C----
C  COMPUTE COMPLETE [Psv] MATRIX FOR SURFACE FLUX COMPUTATION
C----
         CALL QIJSBC( IPSYS,  NREG, NSPOS,  NPIJ,  NGRP,1,
     >                RD(SIGTAL),RD(DBLPIJ),
     >                RD(JPSVT) ,RD(JPSVF),
     >                ID(LGRP)  ,NGRT)
         IFUITE=1
         CALL RLSARA( ID( MATRT ))
         CALL RLSARA( ID(   IDL ))
         CALL RLSARA( RD( JPSST ))
      ELSE
C----
C  COMPUTE COMPLETE [Psv] MATRIX FOR SURFACE FLUX COMPUTATION
C----
         CALL QIJSBC( IPSYS,  NREG, NSPOS,  NPIJ,  NGRP,0,
     >                RD(SIGTAL),RD(DBLPIJ),
     >                RD(JPSVT) ,RD(JPSVF) ,
     >                ID(LGRP)  ,NGRT)
         IFUITE=1
      ENDIF
      CALL RLSARA( RD( JPSVF ))
      CALL RLSARA( RD( JPSVT ))
      IF( IPRT.GE.ICPEND )THEN
         CALL QIJWPR( 1, NREG, NSOUT, ID(LGRP), NGRP,
     >                RD(SIGTAL), RD(DBLPIJ) )
      ENDIF
      CALL SETARA( RD, NREG*NREG, IARRAY)
      CALL SETARA( RD,  2*NGRP*NREG, IPIS)
      CALL SETARA( RD,  2*NGRP, IDD)
      IF( LNORM.OR.LALBS )THEN
         CALL QIJNRM(NREG  ,NSOUT ,NGRP, ID(MATMRG-NSOUT+1),
     >               RD(VOLMRG-NSOUT+1),RD(SIGINV),
     >               RD(DBLPIJ),RD(IPIS),RD(IDD) )
        IF(LALBS) THEN
          CALL QIJPRL(IPSYS ,NREG  ,NGRP  ,NGRT  ,
     >                ID(LGRP),RD(VOLMRG-NSOUT+1),RD(SIGINV),
     >                RD(IPIS),RD(IARRAY))
        ENDIF
      ELSE
         CALL QIJCPL(NREG  ,NSOUT ,NPIJ  ,NGRP  ,
     >               ID(MATMRG-NSOUT+1),RD(VOLMRG-NSOUT+1),
     >               RD(SIGINV),RD(DBLPIJ),RD(IPIS),RD(IDD))
         CALL QIJPRL(IPSYS ,NREG  ,NGRP  ,NGRT  ,
     >               ID(LGRP),RD(VOLMRG-NSOUT+1),RD(SIGINV),
     >               RD(IPIS),RD(IARRAY))
      ENDIF
      CALL RLSARA( RD(IDD) )
      CALL RLSARA( RD(IPIS) )
      IF( ISCATR .EQ. 0 )THEN
         CALL SETARA( RD,4*NREG*NREG,IWORK)
         CALL QIJSMD( NREG, NSOUT, NPIJ, NGRP, RD(SIGINV), RD(SIGSS),
     >             RD(DBLPIJ),RD(VOLMRG-NSOUT+1),RD(IWORK) )
         CALL RLSARA( RD(IWORK) )
      ELSEIF( ISCATR .EQ. 1 )THEN
         CALL SETARA( RD,   NREG*2, IWORK)
         CALL QIJNOS( NREG, NSOUT, NPIJ, NGRP, RD(SIGINV),
     >             RD(DBLPIJ), RD(VOLMRG-NSOUT+1), RD(IWORK) )
         CALL RLSARA( RD(IWORK) )
      ENDIF
      CALL QIJPRT( IPSYS, ID(LGRP), NGRP, NGRT, NREG,
     >             RD(DBLPIJ), RD(IARRAY), IPRT)
      CALL RLSARA( RD(IARRAY) )
                 IBEGGR= MIN(NGRT,IENDGR+1)
               ENDIF
  303 CONTINUE
         CALL RLSARA( RD(DSCEND) )
         CALL RLSARA( RD(DSCBEG) )
         CALL RLSARA( RD(PRODUC) )
         CALL RLSARA( RD(GOSOUT) )
         CALL RLSARA( RD(STAYIN) )
         CALL RLSARA( ID(NRSEG ) )
         CALL RLSARA( RD(SEGLEN) )
         CALL RLSARA( RD(DENSTY) )
         CALL RLSARA( RD(ANGLES) )
         CALL RLSARA( RD(LENGHT) )
         CALL RLSARA( ID(NUMERO) )
         CALL RLSARA( RD(TRKDIR) )
         CALL RLSARA( RD(TRKBEG) )
         CALL RLSARA( RD(  CONV) )
         CALL RLSARA( ID(  INCR) )
         CALL RLSARA( ID(  ICUR) )
      ENDIF
      NVO= NV
      NSO= NS
      NV=  NREG
      NS= -NSOUT
      CALL RLSARA( ID( INDEL) )
      CALL RLSARA( ID( ICORD) )
      CALL RLSARA( ID(MAXDIM) )
      CALL RLSARA( ID(MINDIM) )
      CALL RLSARA( RD(REMESH) )
      CALL RLSARA( RD(DBLPIJ) )
      CALL RLSARA( RD(SIGTAL) )
      CALL RLSARA( RD(SIGINV) )
      CALL RLSARA( RD(SIGSS ) )
      CALL RLSARA( RD(VOLSUR) )
      CALL RLSARA( RD(VOLTRK) )
      CALL RLSARA( ID(KEYMRG) )
      CALL RLSARA( ID(MATALB) )
      ICMAX= 0
      DO 60 IR= 1, 6
        ALBEDO(IR)=1.0
        IUSED(IR)= 0
        ICMAX= MAX(ICMAX,ICODE(IR))
        IF( ICODE(IR).LT.0 ) ALBEDO(IR)= ALBOLD(-ICODE(IR))
   60 CONTINUE
      IF( ICMAX.GT.0 )THEN
         CALL XABORT('XL3DRV: PHYSICAL ALBEDOS NOT IMPLEMENTED')
      ENDIF
      LEAKSW=.TRUE.
      DO 80 IR= NSOUT, -1
         IUSED(-ID(MATMRG+IR-NSOUT))= 1
         LEAKSW= LEAKSW .AND. ALBEDO(-ID(MATMRG+IR-NSOUT)).EQ.1.0
   80 CONTINUE
      LEAKSW=.NOT.LEAKSW
      IF( LALBS.AND.(.NOT.LEAKSW) )
     > CALL XABORT('XL3TRK: INVALID BOUNDARY CONDITIONS. THE ALSB '
     >      //'OPTION REQUIRES SOME BOUNDARY LEAKAGE.')
      IF( NREG.GT.MAXPTS ) THEN
        WRITE(IOUT,'(28H ****** XL3TRK ERROR  ******,/
     >   28H NUMBER OF REGION COMPUTED =,I10/
     >   28H MAXIMUM NUMBER OF REGION  =,I10)') NREG,MAXPTS
        CALL XABORT('XL3TRK: MAXR TOO SMALL')
      ENDIF
      CALL XDISET(ISTATE,NSTATE,0)
      ISTATE(1)=IPHASE
      ISTATE(2)=ISCATR
      IF(LNORM) THEN
        ISTATE(4)=1
      ELSE IF (LALBS) THEN
        ISTATE(4)=2
      ELSE IF (LEAKSW) THEN
        ISTATE(4)=-1
      ENDIF
      ISTATE(5)=NRENOR
      ISTATE(6)=NSUBG
      ISTATE(7)=NGRT
      ISTATE(8)=NREG
      ISTATE(9)=MAXMIX
      ISTATE(10)=1
      ISTATE(11)=1
      ISTATE(12)=NV
      ISTATE(13)=-NSOUT
      ISTATE(16)=IFUITE
      CALL LCMPUT(IPSYS,'STATE-VECTOR',NSTATE,1,ISTATE)
      DO 70 IR= 1, NREG
         KEYFLX(IR)= IR
         VOLUME(IR)= RD( VOLMRG+IR-NSOUT )
         MATCOD(IR)= ID( MATMRG+IR-NSOUT )
   70 CONTINUE
      CALL RLSARA( RD(VOLMRG) )
      CALL RLSARA( ID(MATMRG) )
      CALL RLSARA( ID(  LGRP) )
      RETURN 
*----
*  FORMAT
*----
 6000 FORMAT(' *** SPACE REQUIRED FOR CP MATRICES = ',I10,' K ***')
 6001 FORMAT(' *** CP MATRICES ALLOCATED            ',10X,'   ***')
      END
