*DECK ALPADE
      SUBROUTINE ALPADE(NORIN,X,Y,EPSRID,NOR,A,B,PREC,IERR)
*
*-----------------------------------------------------------------------
*
* COMPUTE THE POLYNOMIAL COEFFICIENTS OF A PADE COLLOCATION.
*
* INPUT PARAMETERS:
*  NORIN  : 2*NORIN+1 IS THE NUMBER OF COLLOCATION POINTS.
*  X      : ABSCISSA OF THE COLLOCATION POINTS. DIMENSION X(0:2*NORIN)
*  Y      : ORDINATES OF THE COLLOCATION POINTS. DIMENSION Y(0:2*NORIN)
*  EPSRID : EPSILON USED IN POLYNOMIAL SIMPLIFICATION.
*
* OUTPUT PARAMETERS:
*  NOR    : ORDER OF THE POLYNOMIALS.
*  A      : POLYNOMIAL COEFFICIENTS OF THE NUMERATOR OF THE PADE
*           APPROXIMATION. A(0) IS THE LINEAR TERM.
*  B      : POLYNOMIAL COEFFICIENTS OF THE DENOMINATOR OF THE PADE
*           APPROXIMATION. B(0) IS THE LINEAR TERM.
*           DOUBLE PRECISION A(0:NOR),B(0:NOR)
*  PREC   : ACCURACY OF THE FIT.
*  IERR   : WARNING FLAG
*           = 0  NO WARNING
*           = 1  WARNING FOR NEGATIVE POLE REMOVING.
*
*------------------------------------- AUTHOR: A. HEBERT ; 10/01/96 ----
*
      IMPLICIT         NONE
      INTEGER          NORIN,NOR,IERR
      REAL             X(0:2*NORIN),Y(0:2*NORIN),PREC
      DOUBLE PRECISION EPSRID,A(0:NORIN),B(0:NORIN)
*----
*  LOCAL VARIABLES
*----
      INTEGER          MAXNOR,IREAL,IMAGI,J,I,N,MM,IJNOR
      PARAMETER       (MAXNOR=10,IREAL=1,IMAGI=2)
      LOGICAL          LINF
      REAL             PREC1
      DOUBLE PRECISION DC(0:MAXNOR),DENOM,DY(0:2*MAXNOR,0:2*MAXNOR),
     >                 DDA(2,0:MAXNOR-2),DDB(2,0:MAXNOR-2),
     >                 SIGX0(2,MAXNOR),SIGXW(2,MAXNOR),DABSX,DABSW,
     >                 DCTMP(2),SCC,SDD
*
      IERR=0
      IF(NORIN.GT.MAXNOR) CALL XABORT
     >  ('ALPADE: INSUFFICIENT MAXNOR.')
      LINF=X(2*NORIN).GE.1.0E10
      DO 100 J=0,2*NORIN
        IF(X(J).LE.0.0) CALL XABORT
     >    ('ALPADE: ZERO OR NEGATIVE ABSCISSA.')
        IF(Y(J).LE.0.0) CALL XABORT
     >    ('ALPADE: ZERO OR NEGATIVE ORDINATE.')
        DO 110 I=J,2*NORIN
          IF(J.EQ.0) THEN
            DY(I,J)=DBLE(Y(I))
          ELSE IF(LINF.AND.(MOD(J,2).EQ.1).AND.(I.EQ.2*NORIN)) THEN
            DENOM=DY(I,J-1)-DY(J-1,J-1)
            IF(DENOM.EQ.0.0) CALL XABORT
     >        ('ALPADE: ALGORITHM FAILURE(1).')
            DY(I,J)=1.0D0/DENOM
          ELSE IF(LINF.AND.(I.EQ.2*NORIN)) THEN
            DENOM=DY(I,J-1)
            IF(DENOM.EQ.0.0) CALL XABORT
     >        ('ALPADE: ALGORITHM FAILURE(2).')
            DY(I,J)=1.0D0/DENOM
          ELSE
            DENOM=DY(I,J-1)-DY(J-1,J-1)
            IF(DENOM.EQ.0.0) CALL XABORT
     >        ('ALPADE: ALGORITHM FAILURE(3).')
            DY(I,J)=(DBLE(X(I))-DBLE(X(J-1)))/DENOM
          ENDIF
 110    CONTINUE
 100  CONTINUE
      N=0
      MM=2*NORIN-1
      A(0)=DY(2*NORIN,2*NORIN)
      B(0)=1.0D0
      DO 120 J=1,NORIN
        DC(0)=0.0D0
        DO 130 I=0,N
          DC(I+1)=B(I)
          DC(I)=DC(I)-B(I)*X(MM)+A(I)*DY(MM,MM)
 130    CONTINUE
        MM=MM-1
        B(0)=0.0D0
        DO 140 I=0,N
          B(I+1)=A(I)
          B(I)=B(I)-A(I)*X(MM)+DC(I)*DY(MM,MM)
 140    CONTINUE
        B(N+1)=B(I)+DC(N+1)*DY(MM,MM)
        DO 150 I=0,N+1
          A(I)=B(I)
          B(I)=DC(I)
 150    CONTINUE
        MM=MM-1
        N=N+1
 120  CONTINUE
*
* POLYNOMIAL SIMPLIFICATION.
      NOR=NORIN
      DDA(IREAL,0)=A(NOR)
      DDA(IMAGI,0)=0.0D0
      DDB(IREAL,0)=B(NOR)
      DDB(IMAGI,0)=0.0D0
      CALL ALROOT(A,NOR,SIGX0)
      CALL ALROOT(B,NOR,SIGXW)
      IJNOR=1
 160  CONTINUE
      DCTMP(IREAL)=SIGXW(IREAL,IJNOR)-SIGX0(IREAL,IJNOR)
      DCTMP(IMAGI)=SIGXW(IMAGI,IJNOR)-SIGX0(IMAGI,IJNOR)
      CALL DCAABS(DABSX,DCTMP)
      CALL DCAABS(DABSW,SIGXW(1,IJNOR))
      IF(DABSX.LT.DBLE(EPSRID)*DABSW) THEN
        NOR=NOR-1
        DO 170 I=IJNOR,NOR
          SIGX0(IREAL,I)=SIGX0(IREAL,I+1)
          SIGX0(IMAGI,I)=SIGX0(IMAGI,I+1)
          SIGXW(IREAL,I)=SIGXW(IREAL,I+1)
          SIGXW(IMAGI,I)=SIGXW(IMAGI,I+1)
 170    CONTINUE
      ELSE IF((SIGXW(IREAL,IJNOR).GT.0.0D0).AND.
     >        (SIGXW(IMAGI,IJNOR).EQ.0.0D0) ) THEN
        IERR=1
        NOR=NOR-1
        DO 180 I=IJNOR,NOR
          SIGX0(IREAL,I)=SIGX0(IREAL,I+1)
          SIGX0(IMAGI,I)=SIGX0(IMAGI,I+1)
          SIGXW(IREAL,I)=SIGXW(IREAL,I+1)
          SIGXW(IMAGI,I)=SIGXW(IMAGI,I+1)
 180    CONTINUE
      ELSE
        IJNOR=IJNOR+1
      ENDIF
      IF(IJNOR.LE.NOR) GO TO 160
      IF(NOR.LT.0) CALL XABORT
     >  ('ALPADE: ALGORITHM FAILURE.')
      DO 200 I=1,NOR
        DDA(IREAL,I)=DDA(IREAL,I-1)
        DDA(IMAGI,I)=DDA(IMAGI,I-1)
        DDB(IREAL,I)=DDB(IREAL,I-1)
        DDB(IMAGI,I)=DDB(IMAGI,I-1)
        DO 210 J=I-1,1,-1
          DCTMP(IREAL)=DDA(IREAL,J)*SIGX0(IREAL,I)-
     >                 DDA(IMAGI,J)*SIGX0(IMAGI,I)
          DCTMP(IMAGI)=DDA(IREAL,J)*SIGX0(IMAGI,I)+
     >                 DDA(IMAGI,J)*SIGX0(IREAL,I)
          DDA(IREAL,J)=DDA(IREAL,J-1)-DCTMP(IREAL)
          DDA(IMAGI,J)=DDA(IMAGI,J-1)-DCTMP(IMAGI)
          DCTMP(IREAL)=DDB(IREAL,J)*SIGXW(IREAL,I)-
     >                 DDB(IMAGI,J)*SIGXW(IMAGI,I)
          DCTMP(IMAGI)=DDB(IREAL,J)*SIGXW(IMAGI,I)+
     >                 DDB(IMAGI,J)*SIGXW(IREAL,I)
          DDB(IREAL,J)=DDB(IREAL,J-1)-DCTMP(IREAL)
          DDB(IMAGI,J)=DDB(IMAGI,J-1)-DCTMP(IMAGI)
 210    CONTINUE
        DCTMP(IREAL)=DDA(IREAL,0)*SIGX0(IREAL,I)-
     >               DDA(IMAGI,0)*SIGX0(IMAGI,I)
        DCTMP(IMAGI)=DDA(IREAL,0)*SIGX0(IMAGI,I)+
     >               DDA(IMAGI,0)*SIGX0(IREAL,I)
        DDA(IREAL,0)=-DCTMP(IREAL)
        DDA(IMAGI,0)=-DCTMP(IMAGI)
        DCTMP(IREAL)=DDB(IREAL,0)*SIGXW(IREAL,I)-
     >               DDB(IMAGI,0)*SIGXW(IMAGI,I)
        DCTMP(IMAGI)=DDB(IREAL,0)*SIGXW(IMAGI,I)+
     >               DDB(IMAGI,0)*SIGXW(IREAL,I)
        DDB(IREAL,0)=-DCTMP(IREAL)
        DDB(IMAGI,0)=-DCTMP(IMAGI)
 200  CONTINUE
      DENOM=DDB(IREAL,NOR)
      DO 220 I=0,NOR
        A(I)=DDA(IREAL,I)/DENOM
        B(I)=DDB(IREAL,I)/DENOM
 220  CONTINUE
*
* TEST THE ACCURACY OF THE PADE APPROXIMATION.
      PREC=0.0
      PREC1=0.0
      DO 230 I=0,2*NORIN
        SCC=A(NOR)
        SDD=B(NOR)
        IF(X(I).LT.1.0E10) THEN
          DO 240 IJNOR=NOR-1,0,-1
            SCC=A(IJNOR)+SCC*X(I)
            SDD=B(IJNOR)+SDD*X(I)
 240      CONTINUE
        ENDIF
        PREC=MAX(PREC,ABS(REAL(SCC/SDD)/Y(I)-1.0))
        PREC1=MAX(PREC1,ABS(Y(2*NORIN)/Y(I)-1.0))
 230  CONTINUE
      IF((IERR.NE.0).AND.(PREC.GT.0.99*PREC1)) THEN
*       USE A UNIFORM REPRESENTATION.
        NOR=0
        A(0)=DBLE(Y(2*NORIN))
        B(0)=1.0D0
        PREC=PREC1
      ENDIF
      RETURN
      END
